@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -29,6 +29,15 @@
 #ifndef Q_OS_WASM
     #include <QNetworkProxyFactory>
 #endif
+#ifdef Q_OS_MACOS
+    #include <CoreFoundation/CFBundle.h> // Needed for implementation of ovitoAppFileName()
+#endif
+#ifdef Q_OS_WIN
+    #ifndef NOMINMAX
+        #define NOMINMAX // Prevent <Windows.h> from defining min() and max() macros
+    #endif
+    #include <Windows.h> // Needed for implementation of ovitoAppFileName()
+#endif
 
 // Called from Application::initialize() to register the embedded Qt resource files
 // when running a statically linked executable. The Qt documentation says this
@@ -42,9 +51,62 @@ static void registerQtResources()
 
 namespace Ovito {
 
-/// The one and only instance of this class.
+/// Determines the path of the OVITO main executable.
+static QString ovitoAppFileName()
+{
+#if defined(Q_OS_WIN)
+    QVarLengthArray<wchar_t, MAX_PATH + 1> space;
+    DWORD v;
+    size_t size = 1;
+    do {
+        size += MAX_PATH;
+        space.resize(int(size));
+        v = ::GetModuleFileName(NULL, space.data(), DWORD(space.size()));
+    }
+    while(v >= size);
+    return QString::fromWCharArray(space.data(), v);
+#elif defined(Q_OS_MACOS)
+    static QString appFileName;
+    if (appFileName.isEmpty()) {
+        CFBundleRef mainBundle = CFBundleGetMainBundle();
+        CFURLRef executableURL = CFBundleCopyExecutableURL(mainBundle);
+        if(executableURL) {
+            CFStringRef cfStr = CFURLCopyFileSystemPath(executableURL, kCFURLPOSIXPathStyle);
+            appFileName = QString::fromCFString(cfStr);
+            CFRelease(cfStr);
+            CFRelease(executableURL);
+        }
+    }
+    return appFileName;
+#elif defined(Q_OS_LINUX)
+    const char *path = "/proc/self/exe";
+    QByteArray buf(256, Qt::Uninitialized);
+    ssize_t len = ::readlink(path, buf.data(), buf.size());
+    while(len == buf.size()) {
+        // readlink(2) will fill our buffer and not necessarily terminate with NUL;
+        if(buf.size() >= 4096)
+            return {};
+
+        // double the size and try again
+        buf.resize(buf.size() * 2);
+        len = ::readlink(path, buf.data(), buf.size());
+    }
+    if(len == -1)
+        return {};
+
+    buf.resize(len);
+    return QFile::decodeName(buf);
+#else
+    return {};
+#endif
+}
+
+/// The one and only instance of the class.
 Application* Application::_instance = nullptr;
 
+/// Indicates what kind of application is running (GUI, console, Python module, etc.)
+Application::RunMode Application::_runMode = Application::AppMode;
+
 /// Stores a pointer to the original Qt message handler function, which has been replaced with our own handler.
 QtMessageHandler Application::defaultQtMessageHandler = nullptr;
 
@@ -93,26 +155,11 @@ static void qtMessageLogFile(QtMsgType type, const QMessageLogContext& context,
 /******************************************************************************
 * Constructor.
 ******************************************************************************/
-Application::Application(FileManager& fileManager) :
-    UserInterface(_globalDatasetContainer),
-    _fileManager(fileManager),
-    _globalDatasetContainer(UserInterface::taskManager(), *this)
+Application::Application()
 {
     // Set global application pointer.
     OVITO_ASSERT(_instance == nullptr); // Only allowed to create one Application class instance.
     _instance = this;
-
-    // Use all available processor cores by default, or the user-specified
-    // number given by the OVITO_THREAD_COUNT environment variable.
-    _idealThreadCount = std::max(1, QThread::idealThreadCount());
-    if(qEnvironmentVariableIsSet("OVITO_THREAD_COUNT"))
-        _idealThreadCount = std::max(1, qgetenv("OVITO_THREAD_COUNT").toInt());
-
-#ifndef OVITO_DISABLE_THREADING
-    // Request at least two parallel threads in the global thread pool, even when running on a single-core machine (e.g. a virtual machine).
-    // Concurrent multi-threading is required to avoid deadlocks due to task interdependencies.
-    QThreadPool::globalInstance()->setMaxThreadCount(std::max(2, QThreadPool::globalInstance()->maxThreadCount()));
-#endif
 }
 
 /******************************************************************************
@@ -120,12 +167,8 @@ Application::Application(FileManager& fileManager) :
 ******************************************************************************/
 Application::~Application()
 {
-    OVITO_ASSERT(isShuttingDown()); // Make sure this UserInterface was properly shutdown before being deleted.
-
+    OVITO_ASSERT(taskManager().isShuttingDown()); // Make sure this UserInterface was properly shut down before being deleted.
     _instance = nullptr;
-#ifdef OVITO_DEBUG
-    UserInterface::_isBeingDestructed = true;
-#endif
 }
 
 /******************************************************************************
@@ -178,6 +221,10 @@ QString Application::applicationName()
 ******************************************************************************/
 bool Application::initialize(int& argc, char** argv)
 {
+    // Store command line arguments for later use.
+    _argc = &argc;
+    _argv = argv;
+
     // Install custom Qt error message handler to catch fatal errors in debug mode
     // or redirect log output to file instead of the console if requested by the user.
     if(qEnvironmentVariableIsSet("OVITO_LOG_FILE")) {
@@ -191,13 +238,31 @@ bool Application::initialize(int& argc, char** argv)
         defaultQtMessageHandler = qInstallMessageHandler(qtMessageOutput);
     }
 
-#ifdef OVITO_DEBUG
-    // Activate Qt logging messages related to Vulkan.
-    QLoggingCategory::setFilterRules(QStringLiteral("qt.vulkan=true"));
+    // Force "C" locale, which will be used to parse numbers, e.g. from simulation data files.
+    // QCoreApplication requires a UTF-8 locale and will print a warning to the terminal during initialization otherwise.
+    // Note: On macOS, C.UTF-8 is not available, so we use en_US.UTF-8 instead.
+#ifndef Q_OS_WIN
+#if defined(Q_OS_MACOS)
+    qputenv("LC_ALL", "en_US.UTF-8");
+#else
+    qputenv("LC_ALL", "C.UTF-8");
 #endif
-
-    // Activate default "C" locale, which will be used to parse numbers in strings.
+    // Activate the locale selected by the LC_ALL environment variable.
+    std::setlocale(LC_ALL, "");
+#else
+    // Reactivate the "C" locale (just in case someone else has changed it).
     std::setlocale(LC_ALL, "C");
+#endif
+    QLocale::setDefault(QLocale::c());
+
+    // Double-check if a "C"-like locale is active. Otherwise, parsing of floating-point numbers from text files
+    // using the sscanf() function may fail.
+    FloatType d;
+    if(sscanf("1234.56", FLOATTYPE_SCANF_STRING, &d) != 1 || d != FloatType(1234.56)) {
+        qWarning() << "Failed to set the C locale for parsing floating-point numbers with decimal points. "
+                "Please check your system's locale settings or set the LC_ALL environment variable to C.UTF-8. "
+                "Otherwise, OVITO is not able to read & write simulation data files correctly.";
+    }
 
     // Suppress console messages "qt.network.ssl: QSslSocket: cannot resolve ..."
     QLoggingCategory::setFilterRules(QStringLiteral("qt.network.ssl.warning=false"));
@@ -218,25 +283,22 @@ bool Application::initialize(int& argc, char** argv)
     QMetaType::registerConverter<Vector3, QVector3D>(&Vector3::operator QVector3D);
     QMetaType::registerConverter<QVector3D, Vector3>();
     QMetaType::registerConverter<Color, Vector3>();
+    QMetaType::registerConverter<Color, QString>(&Color::toString);
     QMetaType::registerConverter<Vector3, Color>();
     QMetaType::registerConverter<QVector3D, Color>();
+    QMetaType::registerConverter<Vector3, QString>(&Vector3::toString);
     QMetaType::registerConverter<Color, QVector3D>(&Color::operator QVector3D);
     QMetaType::registerConverter<AffineTransformation, QMatrix4x4>();
     QMetaType::registerConverter<QMatrix4x4, AffineTransformation>();
 
     // Note: Setting the AA_ShareOpenGLContexts and AA_UseDesktopOpenGL flags is only valid BEFORE
-    // the global Qt application object is created. When running in a Python interpreter environment, a Qt
-    // application object may have already been created externally before the ovito module is loaded.
+    // the global Qt application object is created. When running in a Python interpreter environment, THE Qt
+    // application object may have already been created externally before the ovito module is imported.
     if(QCoreApplication::startingUp()) {
         // Enable OpenGL context sharing globally.
         QCoreApplication::setAttribute(Qt::AA_ShareOpenGLContexts);
-#if 1
         // Always use desktop OpenGL (avoid ANGLE on Windows):
         QCoreApplication::setAttribute(Qt::AA_UseDesktopOpenGL);
-#else
-        // Use ANGLE OpenGL-to-DirectX translation layer on Windows:
-        QCoreApplication::setAttribute(Qt::AA_UseOpenGLES);
-#endif
     }
 
     // Specify default OpenGL surface format.
@@ -256,10 +318,9 @@ bool Application::initialize(int& argc, char** argv)
     format.setMinorVersion(0);
 #endif
 
-    // Enable this code to display debug log messages from the OpenGL driver.
-//#ifdef OVITO_DEBUG
-//  format.setOption(QSurfaceFormat::DebugContext);
-//#endif
+    // Enable this to display debug log messages from the OpenGL driver.
+    if(qEnvironmentVariableIntValue("OVITO_OPENGL_DEBUG_CONTEXT") )
+        format.setOption(QSurfaceFormat::DebugContext);
 
     QSurfaceFormat::setDefaultFormat(format);
 
@@ -270,54 +331,84 @@ bool Application::initialize(int& argc, char** argv)
 }
 
 /******************************************************************************
-* Create the global instance of the right QCoreApplication derived class.
+* Create the global Qt application object.
 ******************************************************************************/
-void Application::createQtApplication(int& argc, char** argv)
+void Application::createQtApplication(bool supportGui)
 {
-    // OVITO prefers the "C" locale over the system's default locale.
-    QLocale::setDefault(QLocale::c());
+    // Let the user override the application type with the OVITO_GUI_MODE environment variable.
+    if(qEnvironmentVariableIsSet("OVITO_GUI_MODE")) {
+        if(qEnvironmentVariableIntValue("OVITO_GUI_MODE")) {
+            supportGui = true;
+        }
+        else if(supportGui) {
+            throw Exception(tr("Cannot use this program function, which requires a GUI-enabled Qt application (a connection to the platform's windowing system): "
+                    "The OVITO_GUI_MODE environment variable is set to 0, which forces OVITO to run in non-GUI mode."));
+        }
+    }
 
-    if(headlessMode()) {
-#if defined(Q_OS_LINUX)
-        // Determine font directory path.
-        std::string applicationPath = argv[0];
-        auto sepIndex = applicationPath.rfind('/');
-        if(sepIndex != std::string::npos)
-            applicationPath.resize(sepIndex + 1);
-        std::string fontPath = applicationPath + "../share/ovito/fonts";
-        if(!QDir(QString::fromStdString(fontPath)).exists())
-            fontPath = "/usr/share/fonts";
+    // Do nothing if a global Qt application object already exists. Just check that it is of the right type.
+    if(QCoreApplication::instance()) {
+        if(supportGui) {
+            QGuiApplication* guiApp = qobject_cast<QGuiApplication*>(QCoreApplication::instance());
+            if(!guiApp || guiApp->platformName() == QStringLiteral("minimal")) {
+                throw Exception(tr("This function requires a GUI-enabled Qt application object, i.e. a connection to a windowing system, but it cannot be initialized at this point: A process-wide, non-GUI Qt application object already exists. "
+                        "To solve this issue, you can explicitly create a PySide6 QApplication object before importing the ovito Python module, invoke this function sooner in the Python script to initialize a "
+                        "connection to the windowing system (e.g. X Windows or Wayland), or set the environment variable OVITO_GUI_MODE=1."));
+            }
+        }
+        return;
+    }
 
-        // On Linux, use the 'minimal' QPA platform plugin instead of the standard XCB plugin when no X server is available.
-        // Still create a Qt GUI application object, because otherwise we cannot use (offscreen) font rendering functions.
-        if(!qEnvironmentVariableIsSet("QT_QPA_PLATFORM")) qputenv("QT_QPA_PLATFORM", "minimal");
-        // Enable rudimentary font rendering support, which is implemented by the 'minimal' platform plugin:
-        if(!qEnvironmentVariableIsSet("QT_DEBUG_BACKINGSTORE")) qputenv("QT_DEBUG_BACKINGSTORE", "1");
-        if(!qEnvironmentVariableIsSet("QT_QPA_FONTDIR")) qputenv("QT_QPA_FONTDIR", fontPath.c_str());
+    // The Qt application must be created in the main thread.
+    if(this_task::isMainThread()) {
+        // Let the derived class create the right QCoreApplication object.
+        QCoreApplication* qtApp = createQtApplicationImpl(supportGui, *_argc, _argv);
 
-        // Disable OpenGL context sharing, because we cannot create GL contexts when using the 'minimal' QPA plugin.
-        // If AA_ShareOpenGLContexts is set, the QGuiApplication constructor tries to create an OpenGL context, which fails with a warning message:
-        // "This plugin does not support createPlatformOpenGLContext!".
-        QCoreApplication::setAttribute(Qt::AA_ShareOpenGLContexts, false);
+        // Make the Qt application a child of OVITO's Application object to destroy it on shutdown.
+        if(!qtApp->parent())
+            qtApp->setParent(this);
 
-        new QGuiApplication(argc, argv);
-#elif defined(Q_OS_MACOS)
-        new QGuiApplication(argc, argv);
-#else
-        new QCoreApplication(argc, argv);
-#endif
+        // Keep Qt event loop running indefinitely in Jupyter kernel mode, such that
+        // new main windows can be opened and closed more than once.
+        if(runMode() == KernelMode) {
+            qtApp->setQuitLockEnabled(false);
+            if(QGuiApplication* guiApp = qobject_cast<QGuiApplication*>(qtApp)) {
+                guiApp->setQuitOnLastWindowClosed(false);
+            }
+        }
+
+        // OVITO prefers the "C" locale over the system's default locale.
+        QLocale::setDefault(QLocale::c());
+
+        // Double-check if a "C"-like locale is active. Otherwise, parsing of floating-point numbers from text files
+        // using the sscanf() function may fail.
+        FloatType d;
+        if(sscanf("1234.56", FLOATTYPE_SCANF_STRING, &d) != 1 || d != FloatType(1234.56)) {
+            throw Exception(tr("Failed to set the C locale for parsing floating-point numbers with decimal points. "
+                    "Please check your system's locale settings or set the LC_ALL environment variable to C.UTF-8. "
+                    "Otherwise, OVITO is not able to read & write simulation data files correctly."));
+        }
     }
     else {
-        new QGuiApplication(argc, argv);
+        // If called from a worker thread, we need to perform the creation of the Qt app in the main thread
+        // and block the worker thread until the Qt app has been created.
+        launchAsync(DeferredObjectExecutor(Application::instance()), [supportGui]() noexcept {
+            Application::instance()->createQtApplication(supportGui);
+        }).waitForFinished();
     }
 }
 
 /******************************************************************************
-* Is called by UserInterface::shutdown() when application is shutting down.
+* Cancels all running tasks and closes the user interface as soon as possible
+* (without asking user to save changes).
 ******************************************************************************/
-void Application::signalAboutToQuit()
+void Application::shutdown()
 {
-    Q_EMIT aboutToQuit();
+    // Close the user interface.
+    UserInterface::shutdown();
+
+    // Wait for all running tasks to stop, empty the deferred work queue, and leave all nested event loops.
+    taskManager().requestShutdown();
 }
 
 #ifndef Q_OS_WASM
@@ -327,7 +418,7 @@ void Application::signalAboutToQuit()
 QNetworkAccessManager* Application::networkAccessManager()
 {
     if(!_networkAccessManager) {
-        if(qEnvironmentVariableIsSet("OVITO_ENABLE_SYSTEM_PROXY")) {
+        if(qEnvironmentVariableIntValue("OVITO_ENABLE_SYSTEM_PROXY")) {
             QNetworkProxyFactory::setUseSystemConfiguration(true);
         }
         _networkAccessManager = new QNetworkAccessManager(this);
@@ -336,4 +427,30 @@ QNetworkAccessManager* Application::networkAccessManager()
 }
 #endif
 
+/******************************************************************************
+* Reports task progress on the console (from any thread).
+******************************************************************************/
+void Application::logTaskActivity(const QString& text)
+{
+    // Print task messages to the console if task logging is enabled (via Python method ovito.enable_logging()).
+    if(taskConsoleLoggingEnabled() && !text.isEmpty())
+        qInfo().noquote() << "OVITO:" << text;
+}
+
+/******************************************************************************
+* Similar to QCoreApplication::applicationDirPath() but doesn't require a Qt application.
+******************************************************************************/
+QString Application::applicationDirPath() const
+{
+    return qApp ? QCoreApplication::applicationDirPath() : QFileInfo(ovitoAppFileName()).path();
+}
+
+/******************************************************************************
+* Similar to QCoreApplication::applicationFilePath() but doesn't require a Qt application.
+******************************************************************************/
+QString Application::applicationFilePath() const
+{
+    return qApp ? QCoreApplication::applicationFilePath() : ovitoAppFileName();
+}
+
 }   // End of namespace
