@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -25,6 +25,6 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(ApplicationService);
+IMPLEMENT_ABSTRACT_OVITO_CLASS(ApplicationService);
 
 }   // End of namespace
