@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -48,25 +48,25 @@ protected:
 
 #ifdef OVITO_USE_SYCL
     /// The type of SYCL buffer used by the DataBuffer class.
-    using sycl_buffer_type = SYCL_NS::buffer<std::byte>;
+    using sycl_buffer_type = sycl::buffer<DataBuffer::Byte>;
 
     /// The type of SYCL accessor used to access the buffer's contents on the host.
-    using sycl_accessor_type = SYCL_NS::host_accessor<std::byte, 1,
-        (accessmode == access_mode::read) ? SYCL_NS::access_mode::read : (
-            (accessmode == access_mode::write || accessmode == access_mode::discard_write) ? SYCL_NS::access_mode::write : SYCL_NS::access_mode::read_write)>;
+    using sycl_accessor_type = sycl::host_accessor<DataBuffer::Byte, 1,
+        (accessmode == access_mode::read) ? sycl::access_mode::read : (
+            (accessmode == access_mode::write || accessmode == access_mode::discard_write) ? sycl::access_mode::write : sycl::access_mode::read_write)>;
 
     /// SYCL accessor providing direct memory access to the underlying SYCL buffer.
     sycl_accessor_type _syclAccessor;
 #endif
 
     /// Raw pointer to the buffer's underlying memory. Needs to be updated whenever a reallocation occurs.
-    std::conditional_t<accessmode == access_mode::read, const std::byte*, std::byte*> _data = nullptr;
+    std::conditional_t<accessmode == access_mode::read, const DataBuffer::Byte*, DataBuffer::Byte*> _data = nullptr;
 
     /// Helper function that obtains the buffer's internal storage address for data elements.
     auto dataStorageAddress() const {
 #ifdef OVITO_USE_SYCL
         if constexpr(accessmode == access_mode::read)
-            return !_syclAccessor.empty() ? static_cast<const std::byte*>(_syclAccessor.get_pointer()) : nullptr;
+            return !_syclAccessor.empty() ? static_cast<const DataBuffer::Byte*>(_syclAccessor.get_pointer()) : nullptr;
         else
             return !_syclAccessor.empty() ? _syclAccessor.get_pointer() : nullptr;
 #else
@@ -82,8 +82,8 @@ protected:
         return (buffer && buffer->_data)
             ? sycl_accessor_type{const_cast<sycl_buffer_type&>(*buffer->_data),
                 no_init
-                    ? SYCL_NS::property_list{SYCL_NS::no_init}
-                    : SYCL_NS::property_list{}}
+                    ? sycl::property_list{sycl::no_init}
+                    : sycl::property_list{}}
             : sycl_accessor_type{};
     }
 #endif
@@ -94,9 +94,16 @@ protected:
     BufferAccessBase(decltype(BufferAccessBase::_buffer) buffer, bool no_init, PrivateConstructorTag) :
         _buffer(std::move(buffer)),
 #ifdef OVITO_USE_SYCL
-        _syclAccessor(create_accessor(_buffer, no_init || accessmode == access_mode::discard_write || accessmode == access_mode::discard_read_write)),
+        _syclAccessor(create_accessor(_buffer, accessmode == access_mode::discard_write || accessmode == access_mode::discard_read_write || no_init)),
 #endif
         _data(_buffer ? dataStorageAddress() : nullptr) {
+
+        // When modifying the contents of the buffer, invalidate the cached number of non-zero elements.
+        if constexpr(accessmode != access_mode::read) {
+            if(this->_buffer)
+                this->_buffer->invalidateCachedInfo();
+        }
+
 #ifdef OVITO_DEBUG
         if(this->_buffer) {
             if constexpr(accessmode != access_mode::read)
@@ -122,13 +129,14 @@ public:
     /// Constructor that initializes the accessor in a null state, i.e. not associated with any underlying buffer.
     BufferAccessBase(std::nullptr_t) noexcept {}
 
-    /// Constructor that associates the access object with a buffer object (reference may be null).
+    /// Constructor that associates the access object with a read-only buffer object (reference may be null).
     template<typename V = BufferType>
     BufferAccessBase(std::enable_if_t<std::is_const_v<V>, const BufferType*> buffer) : BufferAccessBase(buffer, false, PrivateConstructorTag{}) {}
 
     /// Constructor that associates the access object with a buffer object (reference may be null).
     template<bool U = StrongReference>
-    BufferAccessBase(std::enable_if_t<!U, std::remove_const_t<BufferType>*> buffer, bool no_init = false) : BufferAccessBase(buffer, no_init, PrivateConstructorTag{}) {}
+    BufferAccessBase(std::enable_if_t<!U, std::remove_const_t<BufferType>*> buffer, DataBuffer::BufferInitialization initMode = DataBuffer::BufferInitialization::Initialized)
+        : BufferAccessBase(buffer, (initMode == DataBuffer::BufferInitialization::Uninitialized) ? true : false, PrivateConstructorTag{}) {}
 
     /// Constructor that associates the access object with a buffer object (reference may be null).
     template<typename Derived, typename U = std::enable_if_t<!StrongReference && std::is_base_of_v<BufferType, Derived>>>
@@ -140,7 +148,8 @@ public:
 
     /// Constructor that associates the access object with a buffer object (reference may be null).
     template<typename Derived, typename U = std::enable_if_t<StrongReference && std::is_base_of_v<BufferType, Derived>>>
-    BufferAccessBase(DataOORef<Derived> buffer, bool no_init = false) : BufferAccessBase(std::move(buffer), no_init, PrivateConstructorTag{}) {}
+    BufferAccessBase(DataOORef<Derived> buffer, DataBuffer::BufferInitialization initMode = DataBuffer::BufferInitialization::Initialized)
+        : BufferAccessBase(std::move(buffer), (initMode == DataBuffer::BufferInitialization::Uninitialized) ? true : false, PrivateConstructorTag{}) {}
 
 #ifdef OVITO_DEBUG
     /// Copy construction (only enabled for read-only accessors).
@@ -273,7 +282,7 @@ public:
     inline std::enable_if_t<CanUpdateStorage, void> updateDataStorageAddress(bool discardMode) {
         OVITO_ASSERT(_buffer);
 #ifdef OVITO_USE_SYCL
-        _syclAccessor = create_accessor(this->_buffer, discardMode || accessmode == access_mode::discard_write || accessmode == access_mode::discard_read_write);
+        _syclAccessor = create_accessor(this->_buffer, accessmode == access_mode::discard_write || accessmode == access_mode::discard_read_write || discardMode);
 #endif
         _data = dataStorageAddress();
 #ifdef OVITO_DEBUG
@@ -423,8 +432,8 @@ public:
             auto end() const noexcept { return _accessor.begin() + _endIndex; }
         private:
             const BufferAccessTyped<T, BufferType, StrongReference, accessmode> _accessor;
-            const size_type _beginIndex;
-            const size_type _endIndex;
+            const size_type _beginIndex = 0;
+            const size_type _endIndex = 0;
         };
         OVITO_ASSERT(beginIndex <= endIndex);
         OVITO_ASSERT(endIndex <= this->size());
@@ -514,7 +523,7 @@ public:
 
     /// Returns a pointer to the raw data of the data array.
     template<bool CanRead = (accessmode != access_mode::write && accessmode != access_mode::discard_write)>
-    inline std::enable_if_t<CanRead, const std::byte*> cdata(size_type component = 0) const {
+    inline std::enable_if_t<CanRead, const DataBuffer::Byte*> cdata(size_type component = 0) const {
         OVITO_ASSERT(this->_buffer);
         OVITO_ASSERT(this->dataStorageAddress() == this->_data);
         OVITO_ASSERT(component < this->componentCount());
@@ -523,7 +532,7 @@ public:
 
     /// Returns a pointer to the raw data of the data array.
     template<bool CanRead = (accessmode != access_mode::write && accessmode != access_mode::discard_write)>
-    inline std::enable_if_t<CanRead, const std::byte*> cdata(size_type index, size_type component) const {
+    inline std::enable_if_t<CanRead, const DataBuffer::Byte*> cdata(size_type index, size_type component) const {
         OVITO_ASSERT(this->_buffer);
         OVITO_ASSERT(this->dataStorageAddress() == this->_data);
         OVITO_ASSERT(index < this->size());
@@ -560,7 +569,7 @@ public:
 
     /// Returns a pointer to the raw data of the data array.
     template<bool CanWrite = (accessmode != access_mode::read)>
-    inline std::enable_if_t<CanWrite, std::byte*> data(size_type component = 0) const {
+    inline std::enable_if_t<CanWrite, DataBuffer::Byte*> data(size_type component = 0) const {
         OVITO_ASSERT(this->_buffer);
         OVITO_ASSERT(this->dataStorageAddress() == this->_data);
         OVITO_ASSERT(component < this->componentCount());
@@ -569,7 +578,7 @@ public:
 
     /// Returns a pointer to the raw data of the data array.
     template<bool CanWrite = (accessmode != access_mode::read)>
-    inline std::enable_if_t<CanWrite, std::byte*> data(size_type index, size_type component) const {
+    inline std::enable_if_t<CanWrite, DataBuffer::Byte*> data(size_type index, size_type component) const {
         OVITO_ASSERT(this->_buffer);
         OVITO_ASSERT(this->dataStorageAddress() == this->_data);
         OVITO_ASSERT(index < this->size());
@@ -595,7 +604,7 @@ using RawBufferReadAccess = RawBufferAccess<access_mode::read>;
 
 /**
  * Same as RawBufferReadAccess but additionally keeps a strong reference to the DataBuffer object
- * to keep it alive while the acessor object exists.
+ * to keep it alive while the accessor object exists.
 */
 using RawBufferReadAccessAndRef = detail::BufferAccessUntyped<const DataBuffer, true, access_mode::read>;;
 
@@ -616,14 +625,14 @@ using BufferWriteAccess = detail::BufferAccessTyped<T, DataBuffer, false, access
 
 /**
  * Same as BufferWriteAccess but additionally keeps a strong reference to the DataBuffer object
- * to keep it alive while the acessor object exists.
+ * to keep it alive while the accessor object exists.
 */
 template<typename T, access_mode accessmode>
 using BufferWriteAccessAndRef = detail::BufferAccessTyped<T, DataBuffer, true, accessmode>;
 
 /**
  * Same as BufferReadAccess but additionally keeps a strong reference to the DataBuffer object
- * to keep it alive while the acessor object exists.
+ * to keep it alive while the accessor object exists.
 */
 template<typename T>
 using BufferReadAccessAndRef = detail::BufferAccessTyped<std::conditional_t<std::is_pointer_v<T>,
@@ -655,21 +664,24 @@ public:
     BufferAccessConvertedTo() = default;
 
     /// Constructor that associates the access object with a buffer object (reference may be null).
-    BufferAccessConvertedTo(DataOORef<const DataBuffer> buffer) : base_type(performDataTypeConversion(std::move(buffer))) {}
+    BufferAccessConvertedTo(ConstDataBufferPtr buffer) : base_type(performDataTypeConversion(std::move(buffer))) {}
 
-    /// Constructor that takes a raw pointer to a DataBuffer.
+    /// Constructor that takes a raw pointer to a DataBuffer (may be null).
     BufferAccessConvertedTo(const DataBuffer* buffer) : BufferAccessConvertedTo(ConstDataBufferPtr(buffer)) {}
 
-    /// Constructor that takes a raw pointer to a DataBuffer.
-    BufferAccessConvertedTo(DataBuffer* buffer) : BufferAccessConvertedTo(DataBufferPtr(buffer)) {}
+    /// Constructor that takes a raw pointer to a DataBuffer (may be null).
+    BufferAccessConvertedTo(DataBuffer* buffer) : BufferAccessConvertedTo(static_cast<const DataBuffer*>(buffer)) {}
 
 private:
 
     /// Helper function that checks the data type of the incoming data buffer and performs a copy-and-conversion
     /// operation only if necessary.
-    static DataOORef<const DataBuffer> performDataTypeConversion(DataOORef<const DataBuffer> buffer) {
-        if(buffer && buffer->dataType() != DataBufferPrimitiveType<std::remove_cv_t<element_type>>::value) {
-            buffer.makeMutableInplace()->convertToDataType(DataBufferPrimitiveType<std::remove_cv_t<element_type>>::value);
+    static ConstDataBufferPtr performDataTypeConversion(ConstDataBufferPtr buffer) {
+        constexpr int TargetDataType = DataBufferPrimitiveType<std::remove_cv_t<element_type>>::value;
+        if(buffer && buffer->dataType() != TargetDataType) {
+            DataBufferPtr convertedBuffer = DataBufferPtr::create(DataBuffer::Uninitialized, buffer->size(), TargetDataType, buffer->componentCount(), buffer->componentNames());
+            convertedBuffer->copyFromAndConvert(*buffer);
+            return convertedBuffer;
         }
         return buffer;
     }
