@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -39,18 +39,44 @@ enum class access_mode
 };
 
 namespace detail {
-    // Forward declarations
+    // Forward declarations:
     template<typename BufferType, bool StrongReference, Ovito::access_mode accessmode> class BufferAccessBase;
+    template<typename T, typename BufferType, bool StrongReference, Ovito::access_mode accessmode> class BufferAccessTyped;
+#ifdef OVITO_USE_SYCL
     template<typename T, Ovito::access_mode AccessMode> class SyclBufferAccessTyped;
+#endif
+
+#ifdef OVITO_USE_SYCL
+    /// Allocates a new SYCL buffer object of the given type and dimensions.
+    /// If possible, creates a buffer that won't block upon destruction.
+    template<typename T, int Dimensions = 1>
+    inline sycl::buffer<T, Dimensions> allocateSyclBuffer(const sycl::range<Dimensions>& r) {
+#ifdef OVITO_USE_SYCL_ACPP
+        // Using AdaptiveCpp's make_async_buffer() to create a buffer with a destructor that won't block.
+        return sycl::make_async_buffer<T, Dimensions>(r);
+#else
+        return sycl::buffer<T, Dimensions>(r);
+#endif
+    }
+#endif
 }
 
+#ifdef OVITO_USE_SYCL
+    // The OVITO_SYCL_PARALLEL_FOR macro is used to make definitions of SYCL kernels compatible with different SYCL implements.
+    // For AdaptiveCpp, the kernels must be named. DPC++, on the other hand, doesn't accept named kernels.
+    #ifdef OVITO_USE_SYCL_ACPP
+        #define OVITO_SYCL_PARALLEL_FOR(cgh, kernel_name) (cgh).parallel_for<struct kernel_name>
+    #else
+        #define OVITO_SYCL_PARALLEL_FOR(cgh, kernel_name) (cgh).parallel_for
+    #endif
+#endif
+
 /**
  * \brief A one- or two-dimensional array of data elements.
  */
 class OVITO_CORE_EXPORT DataBuffer : public DataObject
 {
     OVITO_CLASS(DataBuffer);
-    Q_CLASSINFO("DisplayName", "Data buffer");
 
 public:
 
@@ -63,8 +89,8 @@ public:
 
     /// \brief The most commonly used data types. Note that, at least in principle,
     ///        the class supports any data type registered with the Qt meta type system.
-    enum DataTypes {
-
+    enum DataTypes
+    {
         Int8 = QMetaType::SChar,
         Int32 = QMetaType::Int,
         Int64 = QMetaType::LongLong,
@@ -93,11 +119,22 @@ public:
     template<typename dummy> struct TypeFromDataTypeId<Float32, dummy> { using type = float; };
     template<typename dummy> struct TypeFromDataTypeId<Float64, dummy> { using type = double; };
 
-    enum BufferInitialization {
+    enum BufferInitialization
+    {
         Uninitialized = 0,
         Initialized = 1
     };
 
+    /// Raw data type used for internal memory buffer.
+#ifndef OVITO_USE_SYCL
+    using Byte = std::byte;
+#else
+    using Byte = unsigned char;
+#endif
+
+    /// Type used for MD5 checksum of the buffer's data.
+    using Checksum = std::array<std::uint64_t, 2>;
+
     /// RAII utility class that guards read access to a DataBuffer.
     class ReadAccess
     {
@@ -120,17 +157,18 @@ public:
 
 public:
 
-    /// \brief Creates an empty buffer.
-    Q_INVOKABLE explicit DataBuffer(ObjectInitializationFlags flags) : DataObject(flags) {}
+    /// Null constructor.
+    void initializeObject(ObjectInitializationFlags flags) { DataObject::initializeObject(flags); }
 
-    /// \brief Constructor that creates and initializes a new buffer array.
-    DataBuffer(ObjectInitializationFlags flags, BufferInitialization init, size_t elementCount, int dataType, size_t componentCount = 1, QStringList componentNames = QStringList());
+    /// Constructor that creates and initializes a new buffer array.
+    void initializeObject(ObjectInitializationFlags flags, BufferInitialization init, size_t elementCount, int dataType, size_t componentCount = 1, QStringList componentNames = QStringList());
 
-    /// \brief Constructor that creates a new buffer array.
-    DataBuffer(ObjectInitializationFlags flags, size_t elementCount, int dataType, size_t componentCount = 1, QStringList componentNames = QStringList()) :
-        DataBuffer(flags, BufferInitialization::Uninitialized, elementCount, dataType, componentCount, std::move(componentNames)) {}
+    /// Constructor that creates a new buffer array.
+    void initializeObject(ObjectInitializationFlags flags, size_t elementCount, int dataType, size_t componentCount = 1, QStringList componentNames = QStringList()) {
+        initializeObject(flags, BufferInitialization::Uninitialized, elementCount, dataType, componentCount, std::move(componentNames));
+    }
 
-    /// \brief Returns the number of elements stored in the buffer array.
+    /// Returns the number of elements stored in the buffer array.
     size_t size() const { return _numElements; }
 
     /// \brief Resizes the buffer.
@@ -139,7 +177,7 @@ public:
     ///                     This also determines whether newly allocated memory is initialized to zero.
     void resize(size_t newSize, bool preserveData);
 
-    /// \brief Resizes the buffer and copies the data element from an existing buffer.
+    /// Resizes the buffer and copies the data element from an existing buffer.
     void resizeCopyFrom(size_t newSize, const DataBuffer& original);
 
     /// \brief Grows the number of data elements while preserving the exiting data.
@@ -148,7 +186,7 @@ public:
     /// to accommodate the new elements.
     bool grow(size_t numAdditionalElements, bool callerAlreadyHasWriteAccess = false);
 
-    /// \brief Reduces the number of data elements while preserving the exiting data.
+    /// Reduces the number of data elements while preserving the exiting data.
     /// Note: This method never reallocates the memory buffer. Thus, the capacity of the array remains unchanged and the
     /// memory of the truncated elements is not released by the method.
     void truncate(size_t numElementsToRemove, bool callerAlreadyHasWriteAccess = false);
@@ -158,7 +196,7 @@ public:
     ///         this property storage according to the Qt meta type system.
     int dataType() const { return _dataType; }
 
-    /// \brief Returns the data type as a human-readable string.
+    /// Returns the data type as a human-readable string.
     const char* dataTypeName() const { return QMetaType(dataType()).name(); }
 
     /// \brief Returns the number of bytes per value.
@@ -166,7 +204,7 @@ public:
     ///         specified by type().
     size_t dataTypeSize() const { return _dataTypeSize; }
 
-    /// \brief Returns the number of bytes used per element.
+    /// Returns the number of bytes used per element.
     size_t stride() const { return _stride; }
 
     /// \brief Returns the number of vector components per element.
@@ -184,9 +222,6 @@ public:
         _componentNames = std::move(names);
     }
 
-    /// Changes the data type of the buffer in place (only if necessary) and converts the stored values.
-    void convertToDataType(int dataType);
-
     /// \brief Sets all array elements to the given uniform value.
     template<typename T>
     void fill(const T value);
@@ -194,14 +229,14 @@ public:
     /// \brief Sets all array elements for which the corresponding entries in the
     ///        selection array are non-zero to the given uniform value.
     template<typename T>
-    void fillSelected(const T value, const DataBuffer& selectionProperty);
+    void fillSelected(const T value, const DataBuffer& selection);
 
     /// \brief Sets all array elements for which the corresponding entries in the
     ///        selection array are non-zero to the given uniform value.
     template<typename T>
-    void fillSelected(const T& value, const DataBuffer* selectionProperty) {
-        if(selectionProperty)
-            fillSelected(value, *selectionProperty);
+    void fillSelected(const T& value, const DataBuffer* selection) {
+        if(selection)
+            fillSelected(value, *selection);
         else
             fill(value);
     }
@@ -212,26 +247,58 @@ public:
     /// Replicates existing data N times.
     void replicateFrom(size_t n, const DataBuffer& original);
 
-    /// Reduces the size of the storage array, deleting elements for are marked in the selection array.
+    /// Reduces the size of the storage array, deleting elements that are marked in the boolean selection array.
     void filterResizeCopyFrom(size_t newSize, const DataBuffer& selection, const DataBuffer& original);
 
-    /// Creates a copy of the array, not containing those elements for which
-    /// the corresponding bits in the given bit array were set.
-    OORef<DataBuffer> filterCopy(const boost::dynamic_bitset<>& mask) const;
+    /// Copies the contents from the given source buffer into this buffer using an index mapping.
+    /// This method overload accepts a std::span with an integral value type as index mapping.
+    template<std::integral MappingT>
+    OVITO_CORE_EXPORT void mappedCopyFrom(const DataBuffer& source, std::span<const MappingT> mapping, bool discardOldContents);
 
-    /// Copies the contents from the given source into this storage using a element mapping.
-    void mappedCopyFrom(const DataBuffer& source, const std::vector<size_t>& mapping, bool discardOldContents);
+    /// Copies the contents from the given source buffer into this buffer using an index mapping.
+    /// This method overload accepts a std::vector with an integral value type as index mapping.
+    template<std::integral MappingT>
+    void mappedCopyFrom(const DataBuffer& source, const std::vector<MappingT>& mapping, bool discardOldContents) {
+        mappedCopyFrom(source, std::span(mapping), discardOldContents);
+    }
 
-    /// Copies the elements from this storage array into the given destination array using an index mapping.
-    void mappedCopyTo(DataBuffer& destination, const std::vector<size_t>& mapping) const;
+    /// Copies the contents from the given source buffer into this buffer using an index mapping.
+    /// This method overload accepts a buffer accessor with an integral value type as index mapping.
+    template<std::integral MappingT, typename BufferType, bool StrongReference, Ovito::access_mode accessmode>
+    void mappedCopyFrom(const DataBuffer& source, const detail::BufferAccessTyped<MappingT, BufferType, StrongReference, accessmode>& mapping, bool discardOldContents) {
+        mappedCopyFrom(source, std::span(mapping), discardOldContents);
+    }
 
-    /// Reorders the existing elements in this storage array using an index map.
+    /// Copies the elements from this buffer into the given destination buffer using an index mapping.
+    /// This method overload accepts a std::span with an integral value type as index mapping.
+    template<std::integral MappingT>
+    OVITO_CORE_EXPORT void mappedCopyTo(DataBuffer& destination, std::span<const MappingT> mapping, bool allowOutOfBoundsIndices = false) const;
+
+    /// Copies the elements from this buffer into the given destination buffer using an index mapping.
+    /// This method overload accepts a std::vector with an integral value type as index mapping.
+    template<std::integral MappingT>
+    void mappedCopyTo(DataBuffer& destination, const std::vector<MappingT>& mapping, bool allowOutOfBoundsIndices = false) const {
+        mappedCopyTo(destination, std::span(mapping), allowOutOfBoundsIndices);
+    }
+
+    /// Copies the elements from this buffer into the given destination buffer using an index mapping.
+    /// This method overload accepts a buffer accessor with an integral value type as index mapping.
+    template<std::integral MappingT, typename BufferType, bool StrongReference, Ovito::access_mode accessmode>
+    void mappedCopyTo(DataBuffer& destination, const detail::BufferAccessTyped<MappingT, BufferType, StrongReference, accessmode>& mapping, bool allowOutOfBoundsIndices = false) const {
+        mappedCopyTo(destination, std::span(mapping), allowOutOfBoundsIndices);
+    }
+
+    /// Reorders the existing elements in this storage array according to an index map.
     void reorderElements(const std::vector<size_t>& mapping);
 
-    /// Copies the data elements from the given source array into this array.
-    /// Array size, component count and data type of source and destination must match exactly.
+    /// Copies the data elements from the given source buffer into this buffer.
+    /// Size, component count, and data type of source and destination buffers must match exactly.
     void copyFrom(const DataBuffer& source);
 
+    /// Copies the data elements from the given source buffer into this buffer while performing a numeric data type conversion.
+    /// Array size and component count of source and destination must match but data type can be different.
+    void copyFromAndConvert(const DataBuffer& source);
+
     /// Copies a range of data elements from the given source array into this array.
     /// Component count and data type of source and destination must be compatible.
     void copyRangeFrom(const DataBuffer& source, size_t sourceIndex, size_t destIndex, size_t count);
@@ -254,7 +321,49 @@ public:
     /// Checks if this buffer|s metadata and the contents exactly match those of another buffer.
     bool equals(const DataBuffer& other) const;
 
-    /// Invokes a generic lampda function with the current data type of the buffer.
+    /// Determines the value range (minimum & maximum value) of a particular vector component in the buffer.
+    /// The results are returned as a pair of floating-point values - even if the buffer stores a different data type.
+    /// Optionally, a selection flags array can be specified, which restricts the considered data elements to a subset.
+    std::pair<FloatType, FloatType> minMax(size_t component = 0, const DataBuffer* selection = nullptr) const;
+
+    /// Computes the axis-aligned bounding box of the 3d coordinates stored in the buffer.
+    Box3 boundingBox3() const;
+
+    /// Computes the axis-aligned bounding box of a sub-set of the 3d coordinates stored in the buffer.
+    Box3 boundingBox3Indexed(const DataBuffer& indices) const;
+
+    /// Based on a selection flag array as input, computes the mapping of original indices to a packed array.
+    ConstDataBufferPtr computePackedMapping() const;
+
+    /// Counts how often the given value occurs in the buffer.
+    /// The data type T must be compatible with the buffer's data type.
+    template<typename T>
+    size_t count(const T value) const;
+
+    /// Returns the number of non-zero entries in the array.
+    size_t nonzeroCount() const;
+
+    /// Caches the number of non-zero entries in the array (without actually counting them).
+    /// This method can be called by user code if the number of non-zero elements is known
+    /// from a previous computation.
+    void setNonzeroCount(size_t count) {
+        OVITO_ASSERT(count <= size());
+        _nonzeroCount.store(count, std::memory_order_relaxed);
+    }
+
+    /// Returns the MD5 checksum of the buffer's data.
+    Checksum checksum() const;
+
+    /// Invalidates the cached count of non-zero array elements and the checksum.
+    /// Normally, there is no need for user code to call this function.
+    /// Any write access to the buffer implicitly invalidates the cached information.
+    void invalidateCachedInfo() {
+        _nonzeroCount.store(std::numeric_limits<size_t>::max(), std::memory_order_relaxed);
+        for(size_t i = 0; i < _checksum.size(); i++)
+            _checksum[i].store(0, std::memory_order_relaxed);
+    }
+
+    /// Invokes a generic lambda function with the current data type of the buffer.
     /// The lambda function must accept exactly one generic parameter ("auto _"), whose type
     /// will be the type of the DataBuffer. The value of the parameter is not used.
     template<typename F>
@@ -262,7 +371,7 @@ public:
         forTypes<Float64, Float32, Int32, Int64, Int8>(std::forward<F>(f));
     }
 
-    /// Invokes a generic lampda function with the current data type of the buffer.
+    /// Invokes a generic lambda function with the current data type of the buffer.
     /// The lambda function must accept exactly one generic parameter ("auto _"), whose type
     /// will be the type of the DataBuffer. The value of the parameter is not used.
     template<DataTypes... TypeIds, typename F>
@@ -278,6 +387,10 @@ public:
 
     ////////////////////////////// Data access management //////////////////////////////
 
+    /// Indicates that there currently exists an external memory access to this buffer's data.
+    /// This flag is set by the Python binding layer when a NumPy view is created that references this buffer's memory directly.
+    bool isBeingAccessedExternally() const { return _isBeingAccessedExternally.load(std::memory_order_acquire); }
+
     /// Informs the buffer object that a read accessor is becoming active.
     inline void prepareReadAccess() const {
 #ifdef OVITO_DEBUG
@@ -296,12 +409,13 @@ public:
     }
 
     /// Informs the buffer object that a read/write accessor is becoming active.
-    inline void prepareWriteAccess() const {
+    inline void prepareWriteAccess() {
 #ifdef OVITO_DEBUG
         if(_activeAccessors.fetchAndStoreAcquire(-1) != 0) {
-            OVITO_ASSERT_MSG(false, "DataBuffer::prepareWriteAccess()", "Property cannot be locked for write acccess while it is already locked.");
+            OVITO_ASSERT_MSG(false, "DataBuffer::prepareWriteAccess()", "Property cannot be locked for write access while it is already locked.");
         }
 #endif
+        invalidateCachedInfo();
     }
 
     /// Informs the buffer object that a write accessor is done.
@@ -314,7 +428,7 @@ public:
 
 #ifdef OVITO_USE_SYCL
     /// Provides direct access to the internal SYCL buffer managed by this class.
-    SYCL_NS::buffer<std::byte>& syclBuffer() {
+    sycl::buffer<Byte>& syclBuffer() {
         OVITO_ASSERT(_data);
         return *_data;
     }
@@ -340,12 +454,12 @@ private:
 
 #ifndef OVITO_USE_SYCL
     /// \brief Returns a read-only pointer to the raw element data stored in this buffer.
-    const std::byte* cdata() const {
+    const Byte* cdata() const {
         return _data.get();
     }
 
     /// \brief Returns a read-write pointer to the raw element data stored in this buffer.
-    std::byte* data() {
+    Byte* data() {
         return _data.get();
     }
 #endif
@@ -355,6 +469,10 @@ private:
     /// The data type of the array (a Qt metadata type identifier).
     int _dataType = QMetaType::Void;
 
+    /// Signals that a direct memory access to this buffer's memory is currently active.
+    /// This is set by the Python binding layer when a NumPy view is created that references this buffer's memory directly.
+    std::atomic<bool> _isBeingAccessedExternally{false};
+
     /// The number of bytes per single value.
     size_t _dataTypeSize = 0;
 
@@ -378,12 +496,21 @@ private:
 #ifdef OVITO_USE_SYCL
     /// The internal memory buffer holding the data elements.
     /// Note: We are using std::optional<> here, because SYCL won't allow us to allocate 0-size buffers.
-    mutable std::optional<SYCL_NS::buffer<std::byte>> _data;
+    mutable std::optional<sycl::buffer<Byte>> _data;
 #else
     /// The internal memory buffer holding the data elements.
-    std::unique_ptr<std::byte[]> _data;
+    std::unique_ptr<Byte[]> _data;
 #endif
 
+    /// The number of non-zero entries in the array (if known).
+    /// This field can provide a performance optimization if the number of
+    /// selected elements is queried via nonzeroCount().
+    std::atomic<size_t> _nonzeroCount{std::numeric_limits<size_t>::max()};
+
+    /// The MD5 checksum computed from the data elements.
+    /// This field can provide a performance optimization if the checksum is queried via checksum().
+    std::array<std::atomic<std::uint64_t>, 2> _checksum{{std::uint64_t{0}, std::uint64_t{0}}};
+
 #ifdef OVITO_USE_SYCL
     /// Flag indicating that new kernels have been scheduled for execution that read from the buffer.
     /// This signals blockUntilSyclKernelsFinished() to wait for these operation(s) to finish.
@@ -402,8 +529,11 @@ private:
     bool _isDataInitialized = false;
 #endif
 
+    friend class RegisteredBufferAccess;
     template<typename BufferType, bool StrongReference, Ovito::access_mode accessmode> friend class detail::BufferAccessBase;
+#ifdef OVITO_USE_SYCL
     template<typename T, Ovito::access_mode AccessMode> friend class detail::SyclBufferAccessTyped;
+#endif
 };
 
 /// Class template returning the data type identifier for the components in the given C++ array structure.
@@ -415,7 +545,7 @@ template<> struct DataBufferPrimitiveType<float> { static constexpr DataBuffer::
 template<> struct DataBufferPrimitiveType<double> { static constexpr DataBuffer::DataTypes value = DataBuffer::DataTypes::Float64; };
 template<typename T, std::size_t N> struct DataBufferPrimitiveType<std::array<T,N>> : public DataBufferPrimitiveType<T> {};
 #ifdef OVITO_USE_SYCL
-template<typename T, std::size_t N> struct DataBufferPrimitiveType<SYCL_NS::marray<T,N>> : public DataBufferPrimitiveType<T> {};
+template<typename T, std::size_t N> struct DataBufferPrimitiveType<sycl::marray<T,N>> : public DataBufferPrimitiveType<T> {};
 #endif
 template<typename T> struct DataBufferPrimitiveType<Point_2<T>> : public DataBufferPrimitiveType<T> {};
 template<typename T> struct DataBufferPrimitiveType<Point_3<T>> : public DataBufferPrimitiveType<T> {};
@@ -438,7 +568,7 @@ OVITO_STATIC_ASSERT(DataBufferPrimitiveType<GraphicsFloatType>::value == DataBuf
 template<typename T, typename = void> struct DataBufferPrimitiveComponentCount { static constexpr size_t value = 1; };
 template<typename T, std::size_t N> struct DataBufferPrimitiveComponentCount<std::array<T,N>> { static constexpr size_t value = N; };
 #ifdef OVITO_USE_SYCL
-template<typename T, std::size_t N> struct DataBufferPrimitiveComponentCount<SYCL_NS::marray<T,N>> { static constexpr size_t value = N; };
+template<typename T, std::size_t N> struct DataBufferPrimitiveComponentCount<sycl::marray<T,N>> { static constexpr size_t value = N; };
 #endif
 template<typename T> struct DataBufferPrimitiveComponentCount<Point_2<T>> { static constexpr size_t value = 2; };
 template<typename T> struct DataBufferPrimitiveComponentCount<Point_3<T>> { static constexpr size_t value = 3; };
@@ -471,10 +601,10 @@ inline void DataBuffer::fill(const T value)
     _isDataInitialized = true;
 #endif
 #ifdef OVITO_USE_SYCL
-    ExecutionContext::current().ui().taskManager().syclQueue().submit([&](SYCL_NS::handler& cgh) {
+    this_task::ui()->taskManager().syclQueue().submit([&](sycl::handler& cgh) {
         SyclBufferAccess<T, access_mode::discard_write> accessor(this, cgh);
         // Note: Tried handler.fill() method, but it led to segfaults. Using a custom fill kernel instead:
-        cgh.parallel_for<class databuffer_fill>(SYCL_NS::range(size()), [=](size_t i) {
+        OVITO_SYCL_PARALLEL_FOR(cgh, databuffer_fill)(sycl::range(size()), [=](size_t i) {
             accessor[i] = value;
         });
     });
@@ -483,34 +613,37 @@ inline void DataBuffer::fill(const T value)
     T* end = begin + this->size();
     std::fill(begin, end, value);
 #endif
+    if constexpr(std::is_same_v<T, SelectionIntType>) {
+        setNonzeroCount(!value ? 0 : size());
+    }
 }
 
 /// \brief Sets all array elements for which the corresponding entries in the
 ///        selection array are non-zero to the given uniform value.
 template<typename T>
-inline void DataBuffer::fillSelected(const T value, const DataBuffer& selectionProperty)
+inline void DataBuffer::fillSelected(const T value, const DataBuffer& selection)
 {
-    OVITO_ASSERT(&selectionProperty != this); // Do not allow aliasing.
-    OVITO_ASSERT(selectionProperty.size() == this->size());
-    OVITO_ASSERT(selectionProperty.dataType() == IntSelection);
-    OVITO_ASSERT(selectionProperty.componentCount() == 1);
+    OVITO_ASSERT(&selection != this); // Do not allow aliasing.
+    OVITO_ASSERT(selection.size() == this->size());
+    OVITO_ASSERT(selection.dataType() == IntSelection);
+    OVITO_ASSERT(selection.componentCount() == 1);
     OVITO_ASSERT(stride() == sizeof(T));
     if(size() == 0)
         return;
     OVITO_ASSERT(_isDataInitialized);
     WriteAccess writeAccess(*this);
-    ReadAccess readAccess(selectionProperty);
+    ReadAccess readAccess(selection);
 #ifdef OVITO_USE_SYCL
-    ExecutionContext::current().ui().taskManager().syclQueue().submit([&](SYCL_NS::handler& cgh) {
+    this_task::ui()->taskManager().syclQueue().submit([&](sycl::handler& cgh) {
         SyclBufferAccess<T, access_mode::write> outputAccessor(this, cgh);
-        SyclBufferAccess<SelectionIntType, access_mode::read> selectionAccessor(&selectionProperty, cgh);
-        cgh.parallel_for<class databuffer_fillSelected>(SYCL_NS::range(size()), [=](size_t i) {
+        SyclBufferAccess<SelectionIntType, access_mode::read> selectionAccessor(&selection, cgh);
+        OVITO_SYCL_PARALLEL_FOR(cgh, databuffer_fillSelected)(sycl::range(size()), [=](size_t i) {
             if(selectionAccessor[i])
                 outputAccessor[i] = value;
         });
     });
 #else
-    const SelectionIntType* __restrict selectionIter = reinterpret_cast<const SelectionIntType*>(selectionProperty.cdata());
+    const SelectionIntType* __restrict selectionIter = reinterpret_cast<const SelectionIntType*>(selection.cdata());
     for(T* __restrict v = reinterpret_cast<T*>(data()), *end = v + this->size(); v != end; ++v) {
         if(*selectionIter++)
             *v = value;
@@ -533,8 +666,8 @@ inline void DataBuffer::copyTo(Iter iter) const
     forAnyType([&](auto _) {
         using T = decltype(_);
         OVITO_ASSERT(sizeof(T) == dataTypeSize());
-        SYCL_NS::buffer<T> valueBuffer = _data->reinterpret<T, 1>();
-        SYCL_NS::host_accessor valueAccess(valueBuffer, SYCL_NS::read_only);
+        auto valueBuffer = _data->reinterpret<T, 1>();
+        sycl::host_accessor valueAccess(valueBuffer, sycl::read_only);
         for(const auto* __restrict v = valueAccess.get_pointer(), *v_end = v + size() * cmpntCount; v != v_end;)
             *iter++ = *v++;
     });
@@ -566,8 +699,8 @@ inline void DataBuffer::copyComponentTo(Iter iter, size_t component) const
     forAnyType([&](auto _) {
         using T = decltype(_);
         OVITO_ASSERT(sizeof(T) == dataTypeSize());
-        SYCL_NS::buffer<T> valueBuffer = _data->reinterpret<T, 1>();
-        SYCL_NS::host_accessor valueAccess(valueBuffer, SYCL_NS::read_only);
+        auto valueBuffer = _data->reinterpret<T, 1>();
+        sycl::host_accessor valueAccess(valueBuffer, sycl::read_only);
         for(const auto* __restrict v = valueAccess.get_pointer() + component, *v_end = v + size() * cmpntCount; v != v_end; v += cmpntCount)
             *iter++ = *v;
     });
@@ -598,8 +731,8 @@ inline bool DataBuffer::forEach(size_t component, F&& func) const
     forAnyType([&](auto _) {
         using T = decltype(_);
         OVITO_ASSERT(sizeof(T) == dataTypeSize());
-        SYCL_NS::buffer<T> valueBuffer = _data->reinterpret<T, 1>();
-        SYCL_NS::host_accessor valueAccess(valueBuffer, SYCL_NS::read_only);
+        auto valueBuffer = _data->reinterpret<T, 1>();
+        sycl::host_accessor valueAccess(valueBuffer, sycl::read_only);
         auto v = valueAccess.get_pointer() + component;
         for(size_t i = 0; i < s; i++, v += cmpntCount)
             std::invoke(std::forward<F>(func), i, *v);
@@ -628,11 +761,54 @@ inline void DataBuffer::moveElement(size_t fromIndex, size_t toIndex, bool calle
 #endif
 #ifdef OVITO_USE_SYCL
     OVITO_ASSERT(_data);
-    SYCL_NS::host_accessor valueAccess(*_data, SYCL_NS::read_write);
+    sycl::host_accessor valueAccess(*_data, sycl::read_write);
     std::memmove(valueAccess.get_pointer() + toIndex * stride(), valueAccess.get_pointer() + fromIndex * stride(), stride());
 #else
     std::memmove(data() + toIndex * stride(), cdata() + fromIndex * stride(), stride());
 #endif
+    invalidateCachedInfo();
 }
 
+/// Counts how often the given value occurs in the buffer.
+/// The data type T must be compatible with the buffer's data type.
+template<typename T>
+inline size_t DataBuffer::count(const T value) const
+{
+    size_t count = 0;
+
+#ifdef OVITO_USE_SYCL
+    if(size() != 0) {
+        sycl::buffer<size_t> countBuf(&count, 1);
+        this_task::ui()->taskManager().syclQueue().submit([&](sycl::handler& cgh) {
+            SyclBufferAccess<T, access_mode::read> acc(this, cgh);
+#ifdef OVITO_USE_SYCL_ACPP
+            auto reduction = sycl::reduction(sycl::accessor{countBuf, cgh, sycl::no_init}, size_t{0}, sycl::plus<size_t>());
+#else
+            auto reduction = sycl::reduction(countBuf, cgh, size_t{0}, sycl::plus<size_t>(), sycl::property::reduction::initialize_to_identity{});
+#endif
+            OVITO_SYCL_PARALLEL_FOR(cgh, DataBuffer_count)(sycl::range(acc.size()), reduction, [=](size_t i, auto& red) {
+                if(acc[i] == value)
+                    red += (size_t)1;
+            });
+        });
+    }
+#else
+    for(const auto& s : BufferReadAccess<T>(this)) {
+        if(s == value)
+            count++;
+    }
+#endif
+    return count;
+}
+
+// Instantiate function templates for different integral types.
+#ifndef OVITO_BUILD_MONOLITHIC
+    #if !defined(Core_EXPORTS)
+        extern template OVITO_CORE_EXPORT void DataBuffer::mappedCopyFrom(const DataBuffer& source, std::span<const size_t> mapping, bool discardOldContents);
+        extern template OVITO_CORE_EXPORT void DataBuffer::mappedCopyFrom(const DataBuffer& source, std::span<const int> mapping, bool discardOldContents);
+        extern template OVITO_CORE_EXPORT void DataBuffer::mappedCopyTo(DataBuffer& destination, std::span<const size_t> mapping, bool allowOutOfBoundsIndices) const;
+        extern template OVITO_CORE_EXPORT void DataBuffer::mappedCopyTo(DataBuffer& destination, std::span<const int> mapping, bool allowOutOfBoundsIndices) const;
+    #endif
+#endif
+
 }   // End of namespace
\ No newline at end of file
