@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -30,13 +30,12 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(DataCollection);
+IMPLEMENT_CREATABLE_OVITO_CLASS(DataCollection);
 DEFINE_VECTOR_REFERENCE_FIELD(DataCollection, objects);
 SET_PROPERTY_FIELD_LABEL(DataCollection, objects, "Data objects");
 
 /******************************************************************************
-* Returns true if the given object is part of this pipeline flow state.
-* The method ignores the revision number of the object.
+* Tests if the given object is part of this collection (only at the root-level).
 ******************************************************************************/
 bool DataCollection::contains(const DataObject* obj) const
 {
@@ -44,7 +43,8 @@ bool DataCollection::contains(const DataObject* obj) const
 }
 
 /******************************************************************************
-* Adds an additional data object to this state.
+* Adds an additional root-level data object to this collection.
+* The object must not already be part of the collection.
 ******************************************************************************/
 void DataCollection::addObject(const DataObject* obj)
 {
@@ -54,7 +54,8 @@ void DataCollection::addObject(const DataObject* obj)
 }
 
 /******************************************************************************
-* Inserts an additional data object into this state.
+* Inserts an additional root-level data object into this collection.
+* The object must not already be part of the collection.
 ******************************************************************************/
 void DataCollection::insertObject(qsizetype index, DataOORef<const DataObject> obj)
 {
@@ -69,7 +70,7 @@ void DataCollection::insertObject(qsizetype index, DataOORef<const DataObject> o
 }
 
 /******************************************************************************
-* Replaces a data object with a new one.
+* Removes a root-level data object from this collection.
 ******************************************************************************/
 void DataCollection::removeObjectByIndex(qsizetype index)
 {
@@ -78,7 +79,7 @@ void DataCollection::removeObjectByIndex(qsizetype index)
 }
 
 /******************************************************************************
-* Replaces a data object with a new one.
+* Replaces a root-level data object in this collection with a different one.
 ******************************************************************************/
 bool DataCollection::replaceObject(const DataObject* oldObj, const DataObject* newObj)
 {
@@ -87,7 +88,7 @@ bool DataCollection::replaceObject(const DataObject* oldObj, const DataObject* n
         OVITO_ASSERT_MSG(false, "DataCollection::replaceObject", "Old data object not found.");
         return false;
     }
-    if(newObj)
+    if(newObj && !contains(newObj))
         replaceReferencesTo(oldObj, newObj);
     else
         clearReferencesTo(oldObj);
@@ -95,8 +96,8 @@ bool DataCollection::replaceObject(const DataObject* oldObj, const DataObject* n
 }
 
 /******************************************************************************
-* Finds an object of the given type in the list of data objects stored in this
-* flow state.
+* Finds the first root-level object of the given class type in this collection.
+* Return nullptr if not found.
 ******************************************************************************/
 const DataObject* DataCollection::getObject(const DataObject::OOMetaClass& objectClass) const
 {
@@ -107,20 +108,6 @@ const DataObject* DataCollection::getObject(const DataObject::OOMetaClass& objec
     return nullptr;
 }
 
-/******************************************************************************
-* Finds all objects of the given type in the list of data objects stored in this
-* flow state.
-******************************************************************************/
-std::vector<const DataObject*> DataCollection::getObjects(const DataObject::OOMetaClass& objectClass) const
-{
-    std::vector<const DataObject*> list;
-    for(const DataObject* obj : objects()) {
-        if(objectClass.isMember(obj))
-            list.push_back(obj);
-    }
-    return list;
-}
-
 /******************************************************************************
 * Throws an exception if the input does not contain a data object of the given type.
 ******************************************************************************/
@@ -129,7 +116,7 @@ const DataObject* DataCollection::expectObject(const DataObject::OOMetaClass& ob
     if(const DataObject* obj = getObject(objectClass))
         return obj;
     else {
-        if(ExecutionContext::isInteractive()) {
+        if(this_task::isInteractive()) {
             throw Exception(tr("The dataset does not contain an object of type: %1").arg(objectClass.displayName()));
         }
         else {
@@ -146,7 +133,7 @@ const DataObject* DataCollection::expectLeafObject(const DataObject::OOMetaClass
 {
     const DataObject* obj = getLeafObject(objectClass, pathString);
     if(!obj) {
-        if(ExecutionContext::isInteractive()) {
+        if(this_task::isInteractive()) {
             if(pathString.isEmpty())
                 throw Exception(tr("The dataset does not contain an object of type: %1").arg(objectClass.displayName()));
             else
@@ -154,16 +141,16 @@ const DataObject* DataCollection::expectLeafObject(const DataObject::OOMetaClass
         }
         else {
             if(pathString.isEmpty())
-                throw Exception(tr("No '%1' data object in data collection.").arg(objectClass.displayName()));
+                throw Exception(tr("No '%1' object in data collection.").arg(objectClass.displayName()));
             else
-                throw Exception(tr("No '%1' data object named '%2' in data collection.").arg(objectClass.displayName()).arg(pathString));
+                throw Exception(tr("No '%1' object named '%2' in data collection.").arg(objectClass.displayName()).arg(pathString));
         }
     }
     return obj;
 }
 
 /******************************************************************************
-* Ensures that a DataObject from this flow state is not shared with others and is safe to modify.
+* Ensures that a DataObject from this collection is not shared with others and is safe to modify.
 ******************************************************************************/
 DataObject* DataCollection::makeMutable(const DataObject* obj)
 {
@@ -180,7 +167,7 @@ DataObject* DataCollection::makeMutable(const DataObject* obj)
 }
 
 /******************************************************************************
-* Ensures that a DataObject from this flow state is not shared with others and is safe to modify.
+* Ensures that a DataObject from this collection is not shared with others and is safe to modify.
 ******************************************************************************/
 DataObjectPath DataCollection::makeMutable(const ConstDataObjectPath& path)
 {
@@ -200,7 +187,7 @@ DataObjectPath DataCollection::makeMutable(const ConstDataObjectPath& path)
 }
 
 /******************************************************************************
-* Ensures that a DataObject from this flow state is not shared with others and is safe to modify.
+* Ensures that a DataObject from this collection is not shared with others and is safe to modify.
 ******************************************************************************/
 DataObjectPath DataCollection::makeMutable(const ConstDataObjectPath& path, CloneHelper& cloneHelper)
 {
@@ -221,19 +208,18 @@ DataObjectPath DataCollection::makeMutable(const ConstDataObjectPath& path, Clon
 
 /******************************************************************************
 * Finds an object of the given type and with the given identifier in the list
-* of data objects stored in this flow state.
+* of data objects stored in this collection.
 ******************************************************************************/
 const DataObject* DataCollection::getObjectBy(const DataObject::OOMetaClass& objectClass, const PipelineNode* createdByNode, const QString& identifier) const
 {
-    OVITO_ASSERT(!identifier.isEmpty());
     if(!createdByNode)
         return nullptr;
 
     // Look for the data object with the given ID, or with the given ID followed
     // an enumeration index that was appended by generateUniqueIdentifier().
     for(const DataObject* obj : objects()) {
-        if(objectClass.isMember(obj) && obj->createdByNode() == createdByNode) {
-            if(obj->identifier() == identifier || obj->identifier().startsWith(identifier + QChar('.')))
+        if(objectClass.isMember(obj) && obj->createdByNode().lock().get() == createdByNode) {
+            if(obj->identifier() == identifier || (!identifier.isEmpty() && obj->identifier().startsWith(identifier + QChar('.'))))
                 return obj;
         }
     }
@@ -272,7 +258,7 @@ void DataCollection::getObjectsRecursiveImpl(ConstDataObjectPath& path, const Da
 }
 
 /******************************************************************************
-* Finds an object of the given type and under the hierarchy path in this flow state.
+* Finds an object of the given type and under the hierarchy path in this collection.
 ******************************************************************************/
 ConstDataObjectPath DataCollection::getObject(const DataObject::OOMetaClass& objectClass, const QString& pathString) const
 {
@@ -303,7 +289,7 @@ ConstDataObjectPath DataCollection::expectObject(const DataObject::OOMetaClass&
 {
     ConstDataObjectPath path = getObject(objectClass, pathString);
     if(path.empty()) {
-        if(ExecutionContext::isInteractive()) {
+        if(this_task::isInteractive()) {
             if(pathString.isEmpty())
                 throw Exception(tr("The dataset does not contain an object of type: %1").arg(objectClass.displayName()));
             else
@@ -327,7 +313,7 @@ DataObjectPath DataCollection::expectMutableObject(const DataObject::OOMetaClass
 {
     DataObjectPath path = getMutableObject(objectClass, pathString);
     if(path.empty()) {
-        if(ExecutionContext::isInteractive()) {
+        if(this_task::isInteractive()) {
             if(pathString.isEmpty())
                 throw Exception(tr("The dataset does not contain an object of type: %1").arg(objectClass.displayName()));
             else
@@ -399,7 +385,7 @@ bool DataCollection::getObjectImpl(const DataObject::OOMetaClass& objectClass, Q
 }
 
 /******************************************************************************
-* Finds an object of the given type and under the hierarchy path in this flow state.
+* Finds an object of the given type and under the hierarchy path in this collection.
 ******************************************************************************/
 const DataObject* DataCollection::getLeafObject(const DataObject::OOMetaClass& objectClass, const QString& pathString) const
 {
@@ -455,7 +441,7 @@ const DataObject* DataCollection::getLeafObjectImpl(const DataObject::OOMetaClas
 }
 
 /******************************************************************************
-* Finds an object of the given type and under the hierarchy path in this flow state.
+* Finds an object of the given type and under the hierarchy path in this collection.
 * Duplicates it, and all its parent objects, if needed so that it can be safely
 * modified without unwanted side effects.
 ******************************************************************************/
@@ -542,28 +528,43 @@ QVariant DataCollection::getAttributeValue(const PipelineNode* createdByNode, co
 /******************************************************************************
 * Inserts a new global attribute into the pipeline state.
 ******************************************************************************/
-AttributeDataObject* DataCollection::addAttribute(const QString& key, QVariant value, const PipelineNode* createdByNode)
+AttributeDataObject* DataCollection::addAttribute(const QString& key, QVariant value, OOWeakRef<const PipelineNode> createdByNode)
 {
-    return createObject<AttributeDataObject>(key, createdByNode, std::move(value));
+    return createObject<AttributeDataObject>(key, std::move(createdByNode), std::move(value));
 }
 
 /******************************************************************************
 * Inserts a new global attribute into the pipeline state overwritting any
 * existing attribute with the same name.
 ******************************************************************************/
-AttributeDataObject* DataCollection::setAttribute(const QString& key, QVariant value, const PipelineNode* createdByNode)
+AttributeDataObject* DataCollection::setAttribute(const QString& key, QVariant value, OOWeakRef<const PipelineNode> createdByNode)
 {
     for(const DataObject* obj : objects()) {
         if(const AttributeDataObject* attribute = dynamic_object_cast<AttributeDataObject>(obj)) {
             if(attribute->identifier() == key) {
                 AttributeDataObject* newAttribute = makeMutable(attribute);
                 newAttribute->setValue(std::move(value));
-                newAttribute->setCreatedByNode(const_cast<PipelineNode*>(createdByNode));
+                newAttribute->setCreatedByNode(createdByNode);
                 return newAttribute;
             }
         }
     }
-    return createObject<AttributeDataObject>(key, createdByNode, std::move(value));
+    return createObject<AttributeDataObject>(key, std::move(createdByNode), std::move(value));
+}
+
+/******************************************************************************
+* Copies all global attribute created by the given pipeline node over to this
+* data collection.
+******************************************************************************/
+void DataCollection::adoptAttributesFrom(const DataCollection& other, const OOWeakRef<const PipelineNode>& createdByNode)
+{
+    for(const DataObject* obj : other.objects()) {
+        if(const AttributeDataObject* attribute = dynamic_object_cast<AttributeDataObject>(obj)) {
+            if(attribute->createdByNode() == createdByNode) {
+                addObject(attribute);
+            }
+        }
+    }
 }
 
 /******************************************************************************
