@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -42,44 +42,43 @@ class OVITO_CORE_EXPORT DataCollection : public DataObject
 
 public:
 
-    /// \brief Constructor.
-    Q_INVOKABLE DataCollection(ObjectInitializationFlags flags) : DataObject(flags) {}
+    /// Discards all content of this data collection.
+    void clear() { _objects.clear(this, PROPERTY_FIELD(objects)); }
 
-    /// \brief Discards all contents of this data collection.
-    void clear() {
-        _objects.clear(this, PROPERTY_FIELD(objects));
-    }
-
-    /// \brief Returns true if the given object is part of this pipeline flow state.
-    /// \note The method ignores the revision number of the object.
+    /// Tests if the given object is part of this collection (only at the root-level).
     bool contains(const DataObject* obj) const;
 
-    /// \brief Adds an additional data object to this state.
+    /// Adds an additional root-level data object to this collection.
+    /// The object must not already be part of the collection.
     void addObject(const DataObject* obj);
 
-    /// \brief Inserts an additional data object into this state.
+    /// Inserts an additional root-level data object into this collection.
+    /// The object must not already be part of the collection.
     void insertObject(qsizetype index, DataOORef<const DataObject> obj);
 
-    /// \brief Replaces a data object with a new one.
-    bool replaceObject(const DataObject* oldObj, const DataObject* newObj);
-
-    /// \brief Removes a data object from this state.
+    /// Removes a root-level data object from this collection. The object must exist in the collection.
     void removeObject(const DataObject* obj) { replaceObject(obj, nullptr); }
 
-    /// \brief Removes a data object from this state.
+    /// Removes a root-level data object from this collection.
     void removeObjectByIndex(qsizetype index);
 
-    /// \brief Finds an object of the given type in the list of data objects stored in this flow state.
+    /// Replaces a root-level data object in this collection with a different one.
+    /// If the new object is a nullptr, the old object is simply removed from the collection.
+    bool replaceObject(const DataObject* oldObj, const DataObject* newObj);
+
+    /// Finds the first root-level object of the given class type in this collection.
+    /// Return nullptr if not found.
     const DataObject* getObject(const DataObject::OOMetaClass& objectClass) const;
 
-    /// \brief Finds an object of the given type in the list of data objects stored in this flow state.
+    /// Finds the first root-level object of the given class type in this collection.
+    /// Return nullptr if not found.
     template<class DataObjectClass>
     const DataObjectClass* getObject() const {
         return static_object_cast<DataObjectClass>(getObject(DataObjectClass::OOClass()));
     }
 
-    /// Finds an object of the given type in the list of data objects stored in this flow state.
-    /// If it exists, makes the data object mutable.
+    /// Finds the first root-level object of the given type in this collection.
+    /// If it exists, makes the data object mutable; otherwise, returns nullptr.
     template<class DataObjectClass>
     DataObjectClass* getMutableObject() {
         if(const DataObjectClass* obj = getObject<DataObjectClass>())
@@ -87,10 +86,7 @@ public:
         return nullptr;
     }
 
-    /// \brief Finds all objects of the given type in the list of data objects stored in this flow state.
-    std::vector<const DataObject*> getObjects(const DataObject::OOMetaClass& objectClass) const;
-
-    /// \brief Determines if an object of the given type is in this flow state.
+    /// Determines whether at least one root-level object of the given class type exists in this collection.
     template<class DataObjectClass>
     bool containsObject() const {
         return getObject(DataObjectClass::OOClass()) != nullptr;
@@ -116,7 +112,7 @@ public:
         return static_object_cast<DataObjectClass>(expectMutableObject(DataObjectClass::OOClass()));
     }
 
-    /// Finds an object of the given type in the list of data objects stored in this flow state
+    /// Finds an object of the given type in the list of data objects stored in this collection
     /// or among any of their sub-objects.
     bool containsObjectRecursive(const DataObject::OOMetaClass& objectClass) const {
         for(const DataObject* obj : objects()) {
@@ -126,7 +122,7 @@ public:
         return false;
     }
 
-    /// Finds all objects of the given type in this flow state (also searching among sub-objects).
+    /// Finds all objects of the given type in this collection (also searching among sub-objects).
     /// Returns them as a list of object paths.
     std::vector<ConstDataObjectPath> getObjectsRecursive(const DataObject::OOMetaClass& objectClass) const {
         std::vector<ConstDataObjectPath> result;
@@ -139,20 +135,20 @@ public:
         return result;
     }
 
-    /// Finds an object of the given type and under the hierarchy path in this flow state.
+    /// Finds an object of the given type and under the hierarchy path in this collection.
     ConstDataObjectPath getObject(const DataObject::OOMetaClass& objectClass, const QString& pathString) const;
 
-    /// Finds an object of the given type and under the hierarchy path in this flow state.
+    /// Finds an object of the given type and under the hierarchy path in this collection.
     ConstDataObjectPath getObject(const DataObjectReference& dataRef) const {
         OVITO_ASSERT(dataRef);
         return getObject(*dataRef.dataClass(), dataRef.dataPath());
     }
 
-    /// Finds an object of the given type and under the hierarchy path in this flow state.
+    /// Finds an object of the given type and under the hierarchy path in this collection.
     template<class DataObjectClass>
     ConstDataObjectPath getObject(const QString& pathString) const { return getObject(DataObjectClass::OOClass(), pathString); }
 
-    /// Finds an object of the given type and under the hierarchy path in this flow state.
+    /// Finds an object of the given type and under the hierarchy path in this collection.
     template<class DataObjectClass>
     ConstDataObjectPath getObject(const TypedDataObjectReference<DataObjectClass>& dataRef) const {
         OVITO_ASSERT(dataRef);
@@ -173,7 +169,7 @@ public:
     template<class DataObjectClass>
     ConstDataObjectPath expectObject(const QString& pathString) const { return expectObject(DataObjectClass::OOClass(), pathString); }
 
-    /// Finds an object of the given type and under the hierarchy path in this flow state.
+    /// Finds an object of the given type and under the hierarchy path in this collection.
     template<class DataObjectClass>
     ConstDataObjectPath expectObject(const TypedDataObjectReference<DataObjectClass>& dataRef) const {
         OVITO_ASSERT(dataRef);
@@ -181,16 +177,16 @@ public:
         return expectObject(*dataRef.dataClass(), dataRef.dataPath());
     }
 
-    /// Finds an object of the given type and under the hierarchy path in this flow state.
+    /// Finds an object of the given type and under the hierarchy path in this collection.
     const DataObject* getLeafObject(const DataObject::OOMetaClass& objectClass, const QString& pathString) const;
 
-    /// Finds an object of the given type and under the hierarchy path in this flow state.
+    /// Finds an object of the given type and under the hierarchy path in this collection.
     const DataObject* getLeafObject(const DataObjectReference& dataRef) const {
         OVITO_ASSERT(dataRef);
         return getLeafObject(*dataRef.dataClass(), dataRef.dataPath());
     }
 
-    /// Finds an object of the given type and under the hierarchy path in this flow state.
+    /// Finds an object of the given type and under the hierarchy path in this collection.
     template<class DataObjectClass>
     const DataObjectClass* getLeafObject(const TypedDataObjectReference<DataObjectClass>& dataRef) const {
         OVITO_ASSERT(dataRef);
@@ -215,20 +211,27 @@ public:
         return static_object_cast<DataObjectClass>(expectLeafObject(*dataRef.dataClass(), dataRef.dataPath()));
     }
 
-    /// Finds an object of the given type and with the given identifier in the list of data objects stored in this flow state.
+    /// Finds an object of the given type and with the given identifier in the list of data objects stored in this collection.
     const DataObject* getObjectBy(const DataObject::OOMetaClass& objectClass, const PipelineNode* dataSource, const QString& identifier) const;
 
-    /// Finds an object of the given type and with the given identifier in the list of data objects stored in this flow state.
+    /// Finds an object of the given type and with the given identifier in the list of data objects stored in this collection.
     template<class DataObjectClass>
     const DataObjectClass* getObjectBy(const PipelineNode* dataSource, const QString& identifier) const {
         return static_object_cast<DataObjectClass>(getObjectBy(DataObjectClass::OOClass(), dataSource, identifier));
     }
 
-    /// Finds an object of the given type and under the hierarchy path in this flow state.
+    /// Finds an object of the given type and under the hierarchy path in this collection.
     /// Duplicates it, and all its parent objects, if needed so that it can be safely modified without unwanted side effects.
     DataObjectPath getMutableObject(const DataObject::OOMetaClass& objectClass, const QString& pathString);
 
-    /// Finds an object of the given type and under the hierarchy path in this flow state.
+    /// Finds an object of the given type and under the hierarchy path in this collection.
+    /// Duplicates it, and all its parent objects, if needed so that it can be safely modified without unwanted side effects.
+    DataObjectPath getMutableObject(const DataObjectReference& dataRef) {
+        OVITO_ASSERT(dataRef);
+        return getMutableObject(*dataRef.dataClass(), dataRef.dataPath());
+    }
+
+    /// Finds an object of the given type and under the hierarchy path in this collection.
     /// Duplicates it, and all its parent objects, if needed so that it can be safely modified without unwanted side effects.
     template<class DataObjectClass>
     DataObjectPath getMutableObject(const TypedDataObjectReference<DataObjectClass>& dataRef) {
@@ -237,23 +240,23 @@ public:
         return getMutableObject(*dataRef.dataClass(), dataRef.dataPath());
     }
 
-    /// Finds an object of the given type and under the hierarchy path in this flow state.
+    /// Finds an object of the given type and under the hierarchy path in this collection.
     DataObject* getMutableLeafObject(const DataObject::OOMetaClass& objectClass, const QString& pathString);
 
-    /// Finds an object of the given type and under the hierarchy path in this flow state.
+    /// Finds an object of the given type and under the hierarchy path in this collection.
     DataObject* getMutableLeafObject(const DataObjectReference& dataRef) {
         OVITO_ASSERT(dataRef);
         return getMutableLeafObject(*dataRef.dataClass(), dataRef.dataPath());
     }
 
-    /// Finds an object of the given type and under the hierarchy path in this flow state.
+    /// Finds an object of the given type and under the hierarchy path in this collection.
     template<class DataObjectClass>
     DataObjectClass* getMutableLeafObject(const DataObject::OOMetaClass& objectClass, const QString& pathString) {
         OVITO_ASSERT(objectClass.isDerivedFrom(DataObjectClass::OOClass()));
         return static_object_cast<DataObjectClass>(getMutableLeafObject(objectClass, pathString));
     }
 
-    /// Finds an object of the given type and under the hierarchy path in this flow state.
+    /// Finds an object of the given type and under the hierarchy path in this collection.
     template<class DataObjectClass>
     DataObjectClass* getMutableLeafObject(const TypedDataObjectReference<DataObjectClass>& dataRef) {
         OVITO_ASSERT(dataRef);
@@ -292,22 +295,22 @@ public:
         return expectMutableLeafObject<DataObjectClass>(*dataRef.dataClass(), dataRef.dataPath());
     }
 
-    /// Ensures that a DataObject from this flow state is not shared with others and is safe to modify.
+    /// Ensures that a DataObject from this collection is not shared with others and is safe to modify.
     DataObject* makeMutable(const DataObject* obj);
 
-    /// Ensures that a DataObject from this flow state is not shared with others and is safe to modify.
+    /// Ensures that a DataObject from this collection is not shared with others and is safe to modify.
     template<class DataObjectClass>
     DataObjectClass* makeMutable(const DataObjectClass* obj) {
         return static_object_cast<DataObjectClass>(makeMutable(static_cast<const DataObject*>(obj)));
     }
 
-    /// Ensures that a DataObject from this flow state is not shared with others and is safe to modify.
+    /// Ensures that a DataObject from this collection is not shared with others and is safe to modify.
     DataObjectPath makeMutable(const ConstDataObjectPath& path);
 
-    /// Ensures that a DataObject from this flow state is not shared with others and is safe to modify.
+    /// Ensures that a DataObject from this collection is not shared with others and is safe to modify.
     DataObjectPath makeMutable(const ConstDataObjectPath& path, CloneHelper& cloneHelper);
 
-    /// \brief Returns true if this state object has no valid contents.
+    /// Returns true if this state object has no valid contents.
     bool isEmpty() const { return objects().empty(); }
 
     /// Returns the source frame number associated with this pipeline state.
@@ -315,21 +318,21 @@ public:
     int sourceFrame() const;
 
     /// Instantiates a new data object, passes the given parameters to its class constructor,
-    /// assigns the given data source object, and finally inserts the data object into this pipeline flow state.
+    /// assigns the given data source object, and finally inserts the data object into this collection.
     template<class DataObjectType, typename... Args>
-    DataObjectType* createObject(const PipelineNode* createdByNode, Args&&... args) {
+    DataObjectType* createObject(OOWeakRef<const PipelineNode> createdByNode, Args&&... args) {
         OORef<DataObjectType> obj = OORef<DataObjectType>::create(std::forward<Args>(args)...);
-        obj->setCreatedByNode(const_cast<PipelineNode*>(createdByNode));
+        obj->setCreatedByNode(std::move(createdByNode));
         addObject(obj);
         return obj;
     }
 
     /// Instantiates a new data object, passes the given parameters to its class constructor,
     /// assign a unique identifier to the object, assigns the given data source object, and
-    /// finally inserts the data object into this pipeline flow state.
+    /// finally inserts the data object into this collection.
     template<class DataObjectType, typename... Args>
-    DataObjectType* createObject(const QString& baseName, const PipelineNode* createdByNode, Args&&... args) {
-        DataObjectType* obj = createObject<DataObjectType, Args...>(createdByNode, std::forward<Args>(args)...);
+    DataObjectType* createObject(const QString& baseName, OOWeakRef<const PipelineNode> createdByNode, Args&&... args) {
+        DataObjectType* obj = createObject<DataObjectType, Args...>(std::move(createdByNode), std::forward<Args>(args)...);
         OVITO_ASSERT(!baseName.isEmpty());
         obj->setIdentifier(generateUniqueIdentifier<DataObjectType>(baseName));
         return obj;
@@ -337,10 +340,10 @@ public:
 
     /// Instantiates a new data object, passes the given parameters to its class constructor,
     /// assign a unique identifier to the object, assigns the given data source object and visual element, and
-    /// finally inserts the data object into this pipeline flow state.
+    /// finally inserts the data object into this collection.
     template<class DataObjectType, typename... Args>
-    DataObjectType* createObjectWithVis(const QString& baseName, const PipelineNode* createdByNode, DataVis* visElement, Args&&... args) {
-        DataObjectType* obj = createObject<DataObjectType>(baseName, createdByNode, ObjectInitializationFlag::DontCreateVisElement, std::forward<Args>(args)...);
+    DataObjectType* createObjectWithVis(const QString& baseName, OOWeakRef<const PipelineNode> createdByNode, DataVis* visElement, Args&&... args) {
+        DataObjectType* obj = createObject<DataObjectType>(baseName, std::move(createdByNode), ObjectInitializationFlag::DontCreateVisElement, std::forward<Args>(args)...);
         obj->setVisElement(visElement);
         return obj;
     }
@@ -349,8 +352,8 @@ public:
     /// assigns the pipeline creator node and visual element, and
     /// finally inserts the data object into this data collection.
     template<class DataObjectType, typename... Args>
-    DataObjectType* createObjectWithVis(const PipelineNode* createdByNode, DataVis* visElement, Args&&... args) {
-        DataObjectType* obj = createObject<DataObjectType>(createdByNode, ObjectInitializationFlag::DontCreateVisElement, std::forward<Args>(args)...);
+    DataObjectType* createObjectWithVis(OOWeakRef<const PipelineNode> createdByNode, DataVis* visElement, Args&&... args) {
+        DataObjectType* obj = createObject<DataObjectType>(std::move(createdByNode), ObjectInitializationFlag::DontCreateVisElement, std::forward<Args>(args)...);
         obj->setVisElement(visElement);
         return obj;
     }
@@ -382,10 +385,13 @@ public:
     QVariant getAttributeValue(const PipelineNode* createdByNode, const QString& attrBaseName, const QVariant& defaultValue = QVariant()) const;
 
     /// Inserts a new global attribute into the pipeline state.
-    AttributeDataObject* addAttribute(const QString& key, QVariant value, const PipelineNode* createdByNode);
+    AttributeDataObject* addAttribute(const QString& key, QVariant value, OOWeakRef<const PipelineNode> createdByNode);
 
-    /// Inserts a new global attribute into the pipeline state overwritting any existing attribute with the same name.
-    AttributeDataObject* setAttribute(const QString& key, QVariant value, const PipelineNode* createdByNode);
+    /// Inserts a new global attribute into the pipeline state overwriting any existing attribute with the same name.
+    AttributeDataObject* setAttribute(const QString& key, QVariant value, OOWeakRef<const PipelineNode> createdByNode);
+
+    /// Copies all global attribute created by the given pipeline node over to this data collection.
+    void adoptAttributesFrom(const DataCollection& other, const OOWeakRef<const PipelineNode>& createdByNode);
 
     /// Returns a new unique data object identifier that does not collide with the
     /// identifiers of any existing data object of the given type in the same data
@@ -416,7 +422,7 @@ private:
 
 private:
 
-    /// Stores the list of data objects.
+    /// The list of root-level data objects in the collection.
     DECLARE_MODIFIABLE_VECTOR_REFERENCE_FIELD(DataOORef<const DataObject>, objects, setObjects);
 };
 
