@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -162,17 +162,16 @@ private:
 Q_DECLARE_OPERATORS_FOR_FLAGS(TriMeshFace::MeshFaceFlags);
 
 /**
- * \brief A data object represeting a mesh made of vertices and triangles.
+ * \brief A data object representing a mesh made of vertices and triangles.
  */
 class OVITO_CORE_EXPORT TriangleMesh : public DataObject
 {
     OVITO_CLASS(TriangleMesh)
-    Q_CLASSINFO("ClassNameAlias", "TriMeshObject");  // For backward compatibility with OVITO 3.9.2
 
 public:
 
-    /// Constructor that creates an object with an empty triangle mesh.
-    Q_INVOKABLE TriangleMesh(ObjectInitializationFlags flags);
+    /// Constructor.
+    void initializeObject(ObjectInitializationFlags flags);
 
     /// \brief Returns the title of this object.
     virtual QString objectTitle() const override { return tr("Triangle mesh"); }
@@ -328,7 +327,7 @@ public:
         _vertexPseudoColors.resize(enablePseudoColors ? _vertices.size() : 0);
     }
 
-    /// \brief Allows direct access to the per-vertex psudo-color array of the mesh.
+    /// \brief Allows direct access to the per-vertex pseudo-color array of the mesh.
     /// \return A reference to the vector that stores all vertex pseudo-color values.
     QVector<FloatType>& vertexPseudoColors() {
         OVITO_ASSERT(_hasVertexPseudoColors);
@@ -412,6 +411,14 @@ public:
     /// \brief Flip the orientation of all faces.
     void flipFaces();
 
+    /// Determines whether any of the mesh's face has the selection flag set.
+    bool hasSelectedFaces() const {
+        for(const TriMeshFace& face : faces())
+            if(face.isSelected())
+                return true;
+        return false;
+    }
+
     /// \brief Returns whether this mesh has RGBA colors associated with the individual faces.
     bool hasFaceColors() const {
         return _hasFaceColors;
@@ -573,7 +580,7 @@ public:
     /// \param t If an intersection has been found, the method stores the distance of the intersection in this output parameter.
     /// \param normal If an intersection has been found, the method stores the surface normal at the point of intersection in this output parameter.
     /// \param faceIndex If an intersection has been found, the method stores the index of the face intersected by the ray in this output parameter.
-    /// \param backfaceCull Controls whether backfacing faces are tested too.
+    /// \param backfaceCull Controls whether back-facing faces are tested too.
     /// \return True if the closest intersection has been found. False if no intersection has been found.
     bool intersectRay(const Ray3& ray, FloatType& t, Vector3& normal, int& faceIndex, bool backfaceCull) const;
 
