@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -37,7 +37,8 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(FileSource);
+IMPLEMENT_CREATABLE_OVITO_CLASS(FileSource);
+OVITO_CLASSINFO(FileSource, "DisplayName", "External file source");
 DEFINE_REFERENCE_FIELD(FileSource, importer);
 DEFINE_PROPERTY_FIELD(FileSource, sourceUrls);
 DEFINE_PROPERTY_FIELD(FileSource, playbackSpeedNumerator);
@@ -73,24 +74,13 @@ static int countNumberOfFiles(const QVector<FileSourceImporter::Frame>& frames)
     return numberOfFiles;
 }
 
-/******************************************************************************
-* Constructs the object.
-******************************************************************************/
-FileSource::FileSource(ObjectInitializationFlags flags) : BasePipelineSource(flags),
-    _playbackSpeedNumerator(1),
-    _playbackSpeedDenominator(1),
-    _playbackStartTime(0),
-    _autoGenerateFilePattern(true),
-    _restrictToFrame(-1)
-{
-}
-
 /******************************************************************************
 * Sets the source location for importing data.
 ******************************************************************************/
-bool FileSource::setSource(std::vector<QUrl> sourceUrls, FileSourceImporter* importer, bool autodetectFileSequences, bool keepExistingDataCollection)
+void FileSource::setSource(std::vector<QUrl> sourceUrls, FileSourceImporter* importer, bool autodetectFileSequences, bool keepExistingDataCollection)
 {
-    OVITO_ASSERT(ExecutionContext::current().isValid());
+    OVITO_ASSERT(this_task::get());
+    OVITO_ASSERT(this_task::isMainThread());
 
     // Make relative file paths absolute.
     for(QUrl& url : sourceUrls) {
@@ -102,7 +92,7 @@ bool FileSource::setSource(std::vector<QUrl> sourceUrls, FileSourceImporter* imp
     }
 
     if(this->sourceUrls() == sourceUrls && this->importer() == importer)
-        return true;
+        return;
 
     if(!sourceUrls.empty()) {
         QFileInfo fileInfo(sourceUrls.front().path());
@@ -135,7 +125,7 @@ bool FileSource::setSource(std::vector<QUrl> sourceUrls, FileSourceImporter* imp
         }
 
         if(this->sourceUrls() == sourceUrls && this->importer() == importer)
-            return true;
+            return;
     }
 
     // Make the call to setSource() undoable.
@@ -177,8 +167,6 @@ bool FileSource::setSource(std::vector<QUrl> sourceUrls, FileSourceImporter* imp
 
     if(turnOffPatternGeneration)
         setAutoGenerateFilePattern(false);
-
-    return true;
 }
 
 /******************************************************************************
@@ -195,15 +183,12 @@ SharedFuture<QVector<FileSourceImporter::Frame>> FileSource::updateListOfFrames(
     SharedFuture<QVector<FileSourceImporter::Frame>> framesFuture = requestFrameList(true);
 
     // Display any errors that occurred during scan operation to the user.
-    framesFuture.finally(*this, [](Task& task) {
-        try { task.throwPossibleException(); }
-        catch(const Exception& ex) { ExecutionContext::current().ui().reportError(ex); }
+    framesFuture.finally(ObjectExecutor(this), [ui=this_task::ui()](Task& task) noexcept {
+        try { if(!task.isCanceled()) task.throwPossibleException(); }
+        catch(const Exception& ex) { ui->reportError(ex); }
         catch(...) {}
     });
 
-    // Show progress of the scan operation in the status bar.
-    ExecutionContext::current().ui().taskManager().registerFuture(framesFuture);
-
     return framesFuture;
 }
 
@@ -213,8 +198,6 @@ SharedFuture<QVector<FileSourceImporter::Frame>> FileSource::updateListOfFrames(
 ******************************************************************************/
 void FileSource::setListOfFrames(QVector<FileSourceImporter::Frame> frames)
 {
-    _framesListFuture.reset();
-
     // Determine the new validity of the existing pipeline state in the cache.
     TimeInterval remainingCacheValidity = TimeInterval::infinite();
 
@@ -259,7 +242,7 @@ void FileSource::setListOfFrames(QVector<FileSourceImporter::Frame> frames)
     // Adjust the global animation length to match the new number of source frames.
     notifyDependents(ReferenceEvent::AnimationFramesChanged);
 
-    if(ExecutionContext::isInteractive()) {
+    if(this_task::isInteractive()) {
         if(dataCollectionFrame() < 0 && !_originallySelectedFilename.contains(QChar('*'))) {
             // Position time slider to the frame corresponding to the file that was initially picked by the user
             // in the file selection dialog.
@@ -274,7 +257,7 @@ void FileSource::setListOfFrames(QVector<FileSourceImporter::Frame> frames)
         else {
             // If trajectory frames have been inserted, reposition time slider to remain at the previously selected frame.
             if(!previouslySelectedFrame.sourceFile.isEmpty()) {
-                int currentFrameIndex = animationTimeToSourceFrame(ExecutionContext::current().ui().datasetContainer().currentAnimationTime());
+                int currentFrameIndex = animationTimeToSourceFrame(this_task::ui()->datasetContainer().currentAnimationTime());
                 if(currentFrameIndex >= 0 && currentFrameIndex < _frames.size()) {
                     if(_frames[currentFrameIndex].sourceFile != previouslySelectedFrame.sourceFile) {
                         for(int frameIndex = 0; frameIndex < _frames.size(); frameIndex++) {
@@ -289,9 +272,6 @@ void FileSource::setListOfFrames(QVector<FileSourceImporter::Frame> frames)
             }
         }
     }
-
-    // Notify UI that the list of source frames has changed.
-    Q_EMIT framesListChanged();
 }
 
 /******************************************************************************
@@ -352,103 +332,124 @@ QMap<int, QString> FileSource::animationFrameLabels() const
 }
 
 /******************************************************************************
-* Determines the time interval over which a computed pipeline state will remain valid.
-******************************************************************************/
-TimeInterval FileSource::validityInterval(const PipelineEvaluationRequest& request) const
+ * This function is called by the pipeline system before a new modifier evaluation
+ * begins to query the validity interval and evaluation result type for this pipeline stage.
+ ******************************************************************************/
+void FileSource::preevaluateInternal(const PipelineEvaluationRequest& request, PipelineEvaluationResult::EvaluationTypes& evaluationTypes, TimeInterval& validityInterval)
 {
-    TimeInterval iv = BasePipelineSource::validityInterval(request);
-
-    // Restrict the validity interval to the duration of the requested source frame.
-    if(restrictToFrame() < 0 && frames().size() > 1) {
-        int frame = animationTimeToSourceFrame(request.time());
-        if(frame > 0)
-            iv.intersect(TimeInterval(sourceFrameToAnimationTime(frame), AnimationTime::positiveInfinity()));
-        if(frame < frames().size() - 1)
-            iv.intersect(TimeInterval(AnimationTime::negativeInfinity(), std::max(sourceFrameToAnimationTime(frame + 1) - 1, sourceFrameToAnimationTime(frame))));
+    if(request.interactiveMode()) {
+        // In interactive mode, there is not enough time to wait for file I/O. Therefore, we'll
+        // return the current internal state to serve the interactive evaluation request immediately.
+        evaluationTypes = PipelineEvaluationResult::EvaluationType::Interactive;
     }
+    else {
+        // Marking the pipeline evaluation results as "non-interactive", because the loading process takes a longer time.
+        // We don't want interactive requests to wait on it.
+        evaluationTypes = PipelineEvaluationResult::EvaluationType::Noninteractive;
 
-    return iv;
+        // Compute validity interval of the requested trajectory frame.
+        int frame = animationTimeToSourceFrame(request.time());
+        int frameCount = frames().size();
+        if(restrictToFrame() < 0 && frameCount > 1) {
+            if(frame > 0)
+                validityInterval.intersect(TimeInterval(sourceFrameToAnimationTime(frame), AnimationTime::positiveInfinity()));
+            if(frame < frameCount - 1)
+                validityInterval.intersect(TimeInterval(AnimationTime::negativeInfinity(), std::max(sourceFrameToAnimationTime(frame + 1) - 1, sourceFrameToAnimationTime(frame))));
+        }
+        else if(frameCount == 0) {
+            // We haven't loaded the frames list yet. In this case, we need to be conservative, because the validity interval of the frame is likely going to change.
+            validityInterval.intersect(sourceFrameToAnimationTime(frame));
+        }
+    }
+    OVITO_ASSERT(validityInterval.contains(request.time()));
 }
 
 /******************************************************************************
 * Asks the object for the result of the data pipeline.
 ******************************************************************************/
-Future<PipelineFlowState> FileSource::evaluateInternal(const PipelineEvaluationRequest& request)
+SharedFuture<PipelineFlowState> FileSource::evaluateInternal(const PipelineEvaluationRequest& request)
 {
+    if(request.interactiveMode()) {
+        // In interactive mode, there is not enough time to wait for file I/O. Therefore, directly
+        // return the current internal state to serve the evaluation request immediately.
+        return PipelineFlowState(getSourceDataCollection(), status());
+    }
+
     // Convert the animation time to a frame number.
     int frame = animationTimeToSourceFrame(request.time());
     int frameCount = frames().size();
 
     // Clamp to frame range.
-    if(frame < 0) frame = 0;
-    else if(frame >= frameCount && frameCount > 0) frame = frameCount - 1;
+    if(frame < 0)
+        frame = 0;
+    else if(frame >= frameCount && frameCount > 0)
+        frame = frameCount - 1;
 
-    OVITO_ASSERT(frameTimeInterval(frame).contains(request.time()));
+    // First, request the list of trajectory frames.
+    SharedFuture<QVector<FileSourceImporter::Frame>> framesListFuture = requestFrameList(false);
 
-    // First request the list of source frames and wait until it becomes available.
-    Future<PipelineFlowState> stateFuture = requestFrameList(false)
-        .then(*this, [this, frame](const QVector<FileSourceImporter::Frame>& sourceFrames) -> Future<PipelineFlowState> {
+    // Then continue by loading the file that contains the requested frame.
+    Future<PipelineFlowState> stateFuture = framesListFuture.then(ObjectExecutor(this), [this, frame](const QVector<FileSourceImporter::Frame>& sourceFrames) -> Future<PipelineFlowState> {
 
-            // Is the requested frame out of range?
-            if(frame >= sourceFrames.size()) {
-                TimeInterval interval = TimeInterval::infinite();
-                if(frame < 0)
-                    interval.setEnd(sourceFrameToAnimationTime(0) - 1);
-                else if(frame >= sourceFrames.size() && !sourceFrames.empty())
-                    interval.setStart(sourceFrameToAnimationTime(sourceFrames.size()));
-                else if(sourceFrames.empty() && _framesValid)
-                    return PipelineFlowState(dataCollection(), PipelineStatus(PipelineStatus::Error, tr("No frames found.")), interval);
-                return PipelineFlowState(dataCollection(), PipelineStatus(PipelineStatus::Error, tr("The file source path is empty or has not been set (no files found).")), interval);
-            }
-            else if(frame < 0) {
-                return PipelineFlowState(dataCollection(), PipelineStatus(PipelineStatus::Error, tr("The requested source frame is out of range.")));
+        // Is the requested frame number out of range?
+        if(frame >= sourceFrames.size()) {
+            TimeInterval interval = TimeInterval::infinite();
+            if(frame < 0)
+                interval.setEnd(sourceFrameToAnimationTime(0) - 1);
+            else if(frame >= sourceFrames.size() && !sourceFrames.empty())
+                interval.setStart(sourceFrameToAnimationTime(sourceFrames.size()));
+            else if(sourceFrames.empty() && _framesValid)
+                return PipelineFlowState(dataCollection(), PipelineStatus(PipelineStatus::Error, tr("No frames found.")), interval);
+            return PipelineFlowState(dataCollection(), PipelineStatus(PipelineStatus::Error, tr("The file source path is empty or has not been set (no files found).")), interval);
+        }
+        else if(frame < 0) {
+            return PipelineFlowState(dataCollection(), PipelineStatus(PipelineStatus::Error, tr("The requested source frame is out of range.")));
+        }
+
+        // Retrieve the source file.
+        SharedFuture<FileHandle> fileFuture = Application::instance()->fileManager().fetchUrl(sourceFrames[frame].sourceFile);
+
+        // Pass the file to the file importer.
+        return fileFuture.then(ObjectExecutor(this), [this, frame](const FileHandle& fileHandle) -> Future<PipelineFlowState> {
+
+            // Without an importer object we have to give up immediately.
+            if(!importer()) {
+                // In case of an error, just return the stale data that we have cached.
+                return PipelineFlowState(dataCollection(), PipelineStatus(PipelineStatus::Error, tr("The file source path has not been set.")));
             }
+            if(frame >= frames().size())
+                throw Exception(tr("Requested source frame index is out of range."));
 
-            // Retrieve the file.
-            Future<PipelineFlowState> loadFrameFuture = Application::instance()->fileManager().fetchUrl(sourceFrames[frame].sourceFile)
-                .then(*this, [this, frame](const FileHandle& fileHandle) -> Future<PipelineFlowState> {
+            // Compute the validity interval of the returned pipeline s tate.
+            TimeInterval interval = frameTimeInterval(frame);
+            const FileSourceImporter::Frame& frameInfo = frames()[frame];
 
-                    // Without an importer object we have to give up immediately.
-                    if(!importer()) {
-                        // In case of an error, just return the stale data that we have cached.
-                        return PipelineFlowState(dataCollection(), PipelineStatus(PipelineStatus::Error, tr("The file source path has not been set.")));
-                    }
-                    if(frame >= frames().size())
-                        throw Exception(tr("Requested source frame index is out of range."));
+            // Set up the load request to be submitted to the FileSourceImporter.
+            FileSourceImporter::LoadOperationRequest loadRequest;
+            loadRequest.pipelineNode = this;
+            loadRequest.fileHandle = fileHandle;
+            loadRequest.frame = frameInfo;
+            loadRequest.isNewlyImportedFile = (dataCollection() == nullptr);
+            loadRequest.state.setData(dataCollection()
+                ? DataOORef<const DataCollection>(dataCollection())
+                : DataOORef<const DataCollection>::create());
 
-                    // Compute the validity interval of the returned pipeline state.
-                    TimeInterval interval = frameTimeInterval(frame);
-                    const FileSourceImporter::Frame& frameInfo = frames()[frame];
+            // Add some standard global attributes to the pipeline state to indicate where it is coming from.
+            loadRequest.state.setAttribute(QStringLiteral("SourceFrame"), frame, this);
+            loadRequest.state.setAttribute(QStringLiteral("SourceFile"), frameInfo.sourceFile.toString(QUrl::RemovePassword | QUrl::PreferLocalFile | QUrl::PrettyDecoded), this);
 
-                    // Set up the load request to be submitted to the FileSourceImporter.
-                    FileSourceImporter::LoadOperationRequest loadRequest;
-                    loadRequest.pipelineNode = this;
-                    loadRequest.fileHandle = fileHandle;
-                    loadRequest.frame = frameInfo;
-                    loadRequest.isNewlyImportedFile = (dataCollection() == nullptr);
-                    loadRequest.state.setData(dataCollection()
-                        ? DataOORef<const DataCollection>(dataCollection())
-                        : DataOORef<const DataCollection>::create());
+            // Also give the state the precomputed validity interval.
+            loadRequest.state.setStateValidity(interval);
 
-                    // Add some standard global attributes to the pipeline state to indicate where it is coming from.
-                    loadRequest.state.setAttribute(QStringLiteral("SourceFrame"), frame, this);
-                    loadRequest.state.setAttribute(QStringLiteral("SourceFile"), frameInfo.sourceFile.toString(QUrl::RemovePassword | QUrl::PreferLocalFile | QUrl::PrettyDecoded), this);
-
-                    // Also give the state the precomputed validity interval.
-                    loadRequest.state.setStateValidity(interval);
-
-                    // Load the frame data and return results to the caller.
-                    return importer()->loadFrame(loadRequest);
-                });
-
-            // Change activity status during long-running load operations.
-            registerActiveFuture(loadFrameFuture);
-
-            return loadFrameFuture;
+            // Load the frame data and return results to the caller.
+            return importer()->loadFrame(std::move(loadRequest));
         });
+    });
 
     // Post-process the results of the loading operation before returning them to the caller.
-    return postprocessDataCollection(frame, frameTimeInterval(frame), std::move(stateFuture), request.throwOnError());
+    postprocessDataCollection(stateFuture, request);
+
+    return std::move(stateFuture);
 }
 
 /******************************************************************************
@@ -456,14 +457,14 @@ Future<PipelineFlowState> FileSource::evaluateInternal(const PipelineEvaluationR
 ******************************************************************************/
 SharedFuture<QVector<FileSourceImporter::Frame>> FileSource::requestFrameList(bool forceRescan)
 {
-    OVITO_ASSERT(ExecutionContext::current().isValid());
+    OVITO_ASSERT(this_task::get());
 
     // Without an importer object the list of frames is empty.
     if(!importer())
         return Future<QVector<FileSourceImporter::Frame>>::createImmediateEmplace();
 
     // Return the active future when the frame loading process is currently in progress.
-    if(_framesListFuture.isValid()) {
+    if(_framesListFuture) {
         if(!forceRescan || !_framesListFuture.isFinished())
             return _framesListFuture;
         _framesListFuture.reset();
@@ -477,10 +478,10 @@ SharedFuture<QVector<FileSourceImporter::Frame>> FileSource::requestFrameList(bo
     // Forward request to the importer object.
     // Intercept future results when they become available and cache them.
     _framesListFuture = importer()->discoverFrames(sourceUrls())
-        // Note that execution of the following continuation function is explicitly deferred to a later time,
+        // Note that storing the frames list in this FileSource is explicitly deferred to some later time,
         // because setListOfFrames() generates a TargetChanged event, which is not allowed during
-        // a synchronous call to the pipeline evaulation function.
-        .then(ObjectExecutor(this, true), [this](QVector<FileSourceImporter::Frame>&& frameList) {
+        // a synchronous call to the pipeline evaluation function.
+        .then(DeferredObjectExecutor(this), [this](QVector<FileSourceImporter::Frame>&& frameList) {
             // Store the new list of frames in the FileSource.
             setListOfFrames(frameList);
             // Pass the frame list on to the caller.
@@ -492,6 +493,12 @@ SharedFuture<QVector<FileSourceImporter::Frame>> FileSource::requestFrameList(bo
     if(_framesListFuture.isFinished())
         return std::move(_framesListFuture);
 
+    // Reset _framesListFuture when finished.
+    _framesListFuture.finally(ObjectExecutor(this), [this](Task& task) noexcept {
+        if(_framesListFuture.task().get() == &task)
+            _framesListFuture.reset();
+    });
+
     // The status of this pipeline object changes while loading is in progress.
     registerActiveFuture(_framesListFuture);
 
@@ -521,7 +528,7 @@ TimeInterval FileSource::frameTimeInterval(int frame) const
 ******************************************************************************/
 void FileSource::reloadFrame(bool refetchFiles, int frameIndex)
 {
-    OVITO_ASSERT(ExecutionContext::current().isValid());
+    OVITO_ASSERT(this_task::get());
 
     if(!importer())
         return;
@@ -602,12 +609,10 @@ void FileSource::propertyChanged(const PropertyFieldDescriptor* field)
             field == PROPERTY_FIELD(playbackSpeedDenominator) ||
             field == PROPERTY_FIELD(playbackStartTime)) {
 
-        // Clear frame label list. It will be regnerated upon request in animationFrameLabels().
+        // Clear frame label list. It will be regenerated upon request in animationFrameLabels().
         _frameLabels.clear();
 
         // Invalidate cached frames, because their validity intervals have changed.
-        int unchangedFromFrame = std::min(playbackStartTime(), 0);
-        int unchangedToFrame = std::max(playbackStartTime(), 0);
         TimeInterval unchangedInterval = (field == PROPERTY_FIELD(playbackStartTime)) ?
             TimeInterval::empty() :
             TimeInterval(sourceFrameToAnimationTime(playbackStartTime()));
@@ -631,14 +636,10 @@ void FileSource::propertyChanged(const PropertyFieldDescriptor* field)
         // Inform animation system that global time line length probably changed.
         notifyDependents(ReferenceEvent::AnimationFramesChanged);
     }
-    else if(field == PROPERTY_FIELD(sourceUrls)) {
-        Q_EMIT currentFileChanged();
-    }
     else if(field == PROPERTY_FIELD(BasePipelineSource::dataCollectionFrame)) {
         // The active frame is part of the source's UI title.
         if(numberOfFiles() > 1)
             notifyDependents(ReferenceEvent::TitleChanged);
-        Q_EMIT currentFileChanged();
     }
     BasePipelineSource::propertyChanged(field);
 }
