@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -30,6 +30,8 @@
 #include <ovito/core/dataset/animation/AnimationSettings.h>
 #include <ovito/core/viewport/ViewportConfiguration.h>
 #include <ovito/core/utilities/io/FileManager.h>
+#include <ovito/core/utilities/concurrent/Launch.h>
+#include <ovito/core/utilities/concurrent/ForEach.h>
 #include <ovito/core/app/UserInterface.h>
 #include <ovito/core/app/Application.h>
 #include "FileSourceImporter.h"
@@ -37,7 +39,7 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(FileSourceImporter);
+IMPLEMENT_ABSTRACT_OVITO_CLASS(FileSourceImporter);
 DEFINE_PROPERTY_FIELD(FileSourceImporter, isMultiTimestepFile);
 SET_PROPERTY_FIELD_LABEL(FileSourceImporter, isMultiTimestepFile, "File contains multiple timesteps");
 
@@ -49,11 +51,11 @@ void FileSourceImporter::propertyChanged(const PropertyFieldDescriptor* field)
     FileImporter::propertyChanged(field);
 
     if(field == PROPERTY_FIELD(isMultiTimestepFile)) {
-        // Automatically rescan input file for animation frames when this option is changed.
+        // Automatically re-scan input file for animation frames when this option is changed.
         requestFramesUpdate();
 
-        // Also notify the UI explicitly, because target-changed messages are supressed for this property field.
-        Q_EMIT isMultiTimestepFileChanged();
+        // Also notify the UI explicitly, because target-changed messages are suppressed for this property field.
+        notifyDependents(FileSourceImporter::MultiTimestepFileChanged);
     }
 }
 
@@ -63,13 +65,13 @@ void FileSourceImporter::propertyChanged(const PropertyFieldDescriptor* field)
 ******************************************************************************/
 void FileSourceImporter::requestReload(bool refetchFiles, int frame)
 {
-    OVITO_ASSERT_MSG(!QCoreApplication::instance() || QThread::currentThread() == QCoreApplication::instance()->thread(), "FileSourceImporter::requestReload", "This function may only be called from the main thread.");
-    OVITO_ASSERT(ExecutionContext::current().isValid());
+    OVITO_ASSERT_MSG(this_task::isMainThread(), "FileSourceImporter::requestReload", "This function may only be called from the main thread.");
+    OVITO_ASSERT(this_task::get());
 
     // Retrieve the FileSource that owns this importer by looking it up in the list of dependents.
     visitDependents([&](RefMaker* dependent) {
         if(FileSource* fileSource = dynamic_object_cast<FileSource>(dependent)) {
-            ExecutionContext::current().ui().handleExceptions([&] {
+            this_task::ui()->handleExceptions([&] {
                 fileSource->reloadFrame(refetchFiles, frame);
             });
         }
@@ -86,14 +88,14 @@ void FileSourceImporter::requestReload(bool refetchFiles, int frame)
 ******************************************************************************/
 void FileSourceImporter::requestFramesUpdate(bool refetchCurrentFile)
 {
-    OVITO_ASSERT_MSG(!QCoreApplication::instance() || QThread::currentThread() == QCoreApplication::instance()->thread(), "FileSourceImporter::requestReload", "This function may only be called from the main thread.");
-    OVITO_ASSERT(ExecutionContext::current().isValid());
+    OVITO_ASSERT_MSG(this_task::isMainThread(), "FileSourceImporter::requestReload", "This function may only be called from the main thread.");
+    OVITO_ASSERT(this_task::get());
 
     // Retrieve the FileSource that owns this importer by looking it up in the list of dependents.
     visitDependents([&](RefMaker* dependent) {
         if(FileSource* fileSource = dynamic_object_cast<FileSource>(dependent)) {
             // Scan input source for animation frames.
-            ExecutionContext::current().ui().handleExceptions([&] {
+            this_task::ui()->handleExceptions([&] {
                 fileSource->updateListOfFrames(refetchCurrentFile);
             });
         }
@@ -127,7 +129,7 @@ bool FileSourceImporter::isReplaceExistingPossible(Scene* scene, const std::vect
         // Look for an existing FileSource in the scene whose
         // data source we can replace with the new file.
         for(SceneNode* node : scene->selection()->nodes()) {
-            if(Pipeline* pipeline = dynamic_object_cast<Pipeline>(node)) {
+            if(Pipeline* pipeline = node->pipeline()) {
                 if(dynamic_object_cast<FileSource>(pipeline->source()))
                     return true;
             }
@@ -137,23 +139,23 @@ bool FileSourceImporter::isReplaceExistingPossible(Scene* scene, const std::vect
 }
 
 /******************************************************************************
-* Imports the given file into the scene.
+* Imports the data from a file into the scene.
 * Return true if the file has been imported.
 * Return false if the import has been aborted by the user.
 * Throws an exception when the import has failed.
 ******************************************************************************/
-OORef<Pipeline> FileSourceImporter::importFileSet(Scene* scene, std::vector<std::pair<QUrl, OORef<FileImporter>>> sourceUrlsAndImporters, ImportMode importMode, bool autodetectFileSequences, MultiFileImportMode multiFileImportMode)
+Future<OORef<Pipeline>> FileSourceImporter::importFileSet(OORef<Scene> scene, std::vector<std::pair<QUrl, OORef<FileImporter>>> sourceUrlsAndImporters, ImportMode importMode, bool autodetectFileSequences, MultiFileImportMode multiFileImportMode)
 {
     OVITO_ASSERT(!sourceUrlsAndImporters.empty());
     OORef<FileSource> existingFileSource;
-    Pipeline* existingPipeline = nullptr;
+    OORef<Pipeline> existingPipeline;
 
     if(importMode == ReplaceSelected) {
         // Look for an existing FileSource in the scene whose
         // data source can be replaced with the newly imported file.
         if(scene) {
             for(SceneNode* node : scene->selection()->nodes()) {
-                if(Pipeline* pipeline = dynamic_object_cast<Pipeline>(node)) {
+                if(Pipeline* pipeline = node->pipeline()) {
                     existingFileSource = dynamic_object_cast<FileSource>(pipeline->source());
                     if(existingFileSource) {
                         existingPipeline = pipeline;
@@ -165,7 +167,7 @@ OORef<Pipeline> FileSourceImporter::importFileSet(Scene* scene, std::vector<std:
     }
     else if(importMode == ResetScene) {
         if(scene)
-            scene->clear();
+            scene->deleteChildren();
     }
     else if(importMode == AddToScene) {
         OVITO_ASSERT(scene != nullptr);
@@ -190,30 +192,32 @@ OORef<Pipeline> FileSourceImporter::importFileSet(Scene* scene, std::vector<std:
     if(existingFileSource && existingFileSource->importer() && existingFileSource->importer()->getOOClass() == this->getOOClass() && existingFileSource->importer()->isMultiTimestepFile())
         setMultiTimestepFile(true);
 
-    // Create a new pipeline in the scene for the linked data.
+    // Create a new pipeline and a new node in the scene.
     OORef<Pipeline> pipeline;
-    if(existingPipeline == nullptr) {
+    if(!existingPipeline) {
+        OORef<SceneNode> sceneNode;
         {
-            UndoSuspender unsoSuspender;    // Do not create undo records for this part.
+            UndoSuspender undoSuspender;    // Do not create undo records for this part.
+            AnimationSuspender animSuspender(*this_task::ui());
 
             // Add object to scene.
             pipeline = OORef<Pipeline>::create();
             pipeline->setHead(fileSource);
-
-            // Let the importer subclass customize the pipeline scene node.
-            setupPipeline(pipeline, fileSource);
+            sceneNode = OORef<SceneNode>::create();
+            sceneNode->setPipeline(pipeline);
         }
 
-        // Insert pipeline into scene.
-        if(importMode != DontAddToScene && scene != nullptr)
-            scene->addChildNode(pipeline);
+        // Insert node into scene.
+        if(importMode != DontAddToScene && scene != nullptr) {
+            scene->addChildNode(sceneNode);
+
+            // Select new object in the scene.
+            if(this_task::isInteractive())
+                scene->selection()->setNode(sceneNode);
+        }
     }
     else pipeline = existingPipeline;
 
-    // Select new object in the scene.
-    if(importMode != DontAddToScene && scene != nullptr)
-        scene->selection()->setNode(pipeline);
-
     // Concatenate all files from the input list having the same file format into one sequence,
     // which gets handled by this importer.
     std::vector<QUrl> sourceUrls;
@@ -231,32 +235,38 @@ OORef<Pipeline> FileSourceImporter::importFileSet(Scene* scene, std::vector<std:
 
     // Set the input file location(s) and importer.
     bool keepExistingDataCollection = true;
-    if(!fileSource->setSource(std::move(sourceUrls), this, autodetectFileSequences && (sourceUrls.size() == 1 && sourceUrlsAndImporters.empty()), keepExistingDataCollection))
-        return {};
+    auto urlCount = sourceUrls.size();
+    fileSource->setSource(std::move(sourceUrls), this, autodetectFileSequences && (urlCount == 1 && sourceUrlsAndImporters.empty()), keepExistingDataCollection);
+
+    // Let the importer subclass customize the pipeline.
+    // Placing this after setSource allows a call to evaluatePipeline() in setupPipeline() implementation.
+    Future<void> setupFuture = setupPipeline(pipeline, fileSource);
+    if(setupFuture) {
+        co_await FutureAwaiter(ObjectExecutor(this), std::move(setupFuture));
+    }
 
     if(importMode != ReplaceSelected && importMode != DontAddToScene) {
         // Adjust viewports to completely show the newly imported object.
         // This needs to be deferred until after the data has been completely loaded and its extents are known.
-        ExecutionContext::current().ui().zoomToSceneExtentsWhenReady();
+        this_task::ui()->zoomToSceneExtentsWhenReady();
     }
 
     // If this importer did not handle all supplied input files,
     // continue importing the remaining files.
     if(!sourceUrlsAndImporters.empty()) {
-        if(!importFurtherFiles(scene, std::move(sourceUrlsAndImporters), importMode, autodetectFileSequences, multiFileImportMode, pipeline))
-            return {};
+        co_await FutureAwaiter(ObjectExecutor(this), importFurtherFiles(std::move(scene), std::move(sourceUrlsAndImporters), importMode, autodetectFileSequences, multiFileImportMode, pipeline));
     }
 
-    return pipeline;
+    co_return pipeline;
 }
 
 /******************************************************************************
 * Is called when importing multiple files of different formats.
 ******************************************************************************/
-bool FileSourceImporter::importFurtherFiles(Scene* scene, std::vector<std::pair<QUrl, OORef<FileImporter>>> sourceUrlsAndImporters, ImportMode importMode, bool autodetectFileSequences, MultiFileImportMode multiFileImportMode, Pipeline* pipeline)
+Future<OORef<Pipeline>> FileSourceImporter::importFurtherFiles(OORef<Scene> scene, std::vector<std::pair<QUrl, OORef<FileImporter>>> sourceUrlsAndImporters, ImportMode importMode, bool autodetectFileSequences, MultiFileImportMode multiFileImportMode, Pipeline* pipeline)
 {
     if(importMode == DontAddToScene)
-        return true;    // It doesn't make sense to import additional datasets if they are not being added to the scene. They would get lost.
+        return Future<OORef<Pipeline>>::createImmediateEmplace();    // It doesn't make sense to import additional datasets if they are not being added to the scene. They would get lost.
 
     OVITO_ASSERT(!sourceUrlsAndImporters.empty());
     OORef<FileImporter> importer = sourceUrlsAndImporters.front().second;
@@ -315,8 +325,8 @@ QString FileSourceImporter::deriveWildcardPatternFromFilename(const QString& fil
 ******************************************************************************/
 Future<QVector<FileSourceImporter::Frame>> FileSourceImporter::discoverFrames(const std::vector<QUrl>& sourceUrls)
 {
-    OVITO_ASSERT_MSG(!QCoreApplication::instance() || QThread::currentThread() == QCoreApplication::instance()->thread(), "FileSourceImporter::discoverFrames", "This function may only be called from the main thread.");
-    OVITO_ASSERT(ExecutionContext::current().isValid());
+    OVITO_ASSERT_MSG(this_task::isMainThread(), "FileSourceImporter::discoverFrames", "This function may only be called from the main thread.");
+    OVITO_ASSERT(this_task::get());
 
     // No output if there is no input.
     if(sourceUrls.empty())
@@ -326,25 +336,18 @@ Future<QVector<FileSourceImporter::Frame>> FileSourceImporter::discoverFrames(co
     if(sourceUrls.size() == 1)
         return discoverFrames(sourceUrls.front());
 
-    // Sequentually invoke single-path routine for each input path and compile results
+    // Sequentially invoke single-path routine for each input path and compile results
     // into one big list that is returned to the caller.
-    auto combinedList = std::make_shared<QVector<Frame>>();
-    Future<QVector<Frame>> future;
-    for(const QUrl& url : sourceUrls) {
-        if(!future.isValid()) {
-            future = discoverFrames(url);
-        }
-        else {
-            future = future.then(*this, [this, combinedList, url](const QVector<Frame>& frames) {
-                *combinedList += frames;
-                return discoverFrames(url);
-            });
-        }
-    }
-    return future.then([combinedList](const QVector<Frame>& frames) {
-        *combinedList += frames;
-        return std::move(*combinedList);
-    });
+    return for_each_sequential(
+        sourceUrls,
+        DeferredObjectExecutor(this),
+        [this](const QUrl& url) {
+            return discoverFrames(url);
+        },
+        [](const QUrl& url, QVector<Frame>&& frames, QVector<FileSourceImporter::Frame>& combinedList) {
+            combinedList += std::move(frames);
+        },
+        QVector<FileSourceImporter::Frame>{});
 }
 
 
@@ -354,22 +357,22 @@ Future<QVector<FileSourceImporter::Frame>> FileSourceImporter::discoverFrames(co
 ******************************************************************************/
 Future<QVector<FileSourceImporter::Frame>> FileSourceImporter::discoverFrames(const QUrl& sourceUrl)
 {
-    OVITO_ASSERT_MSG(!QCoreApplication::instance() || QThread::currentThread() == QCoreApplication::instance()->thread(), "FileSourceImporter::discoverFrames", "This function may only be called from the main thread.");
-    OVITO_ASSERT(ExecutionContext::current().isValid());
+    OVITO_ASSERT_MSG(this_task::isMainThread(), "FileSourceImporter::discoverFrames", "This function may only be called from the main thread.");
+    OVITO_ASSERT(this_task::get());
 
     if(shouldScanFileForFrames(sourceUrl)) {
         // Check if filename is a wildcard pattern.
         // If yes, find all matching files and scan each one of them.
         if(isWildcardPattern(sourceUrl)) {
             return findWildcardMatches(sourceUrl)
-                .then(*this, [this](const std::vector<QUrl>& fileList) {
+                .then(ObjectExecutor(this), [this](const std::vector<QUrl>& fileList) {
                     return discoverFrames(fileList);
                 });
         }
 
         // Fetch file, then scan it.
         return Application::instance()->fileManager().fetchUrl(sourceUrl)
-            .then(*this, [this](const FileHandle& fileHandle) {
+            .then(ObjectExecutor(this), [this](const FileHandle& fileHandle) {
                 return discoverFrames(fileHandle);
             });
     }
@@ -377,7 +380,7 @@ Future<QVector<FileSourceImporter::Frame>> FileSourceImporter::discoverFrames(co
         if(isWildcardPattern(sourceUrl)) {
             // Find all files matching the file pattern.
             return findWildcardMatches(sourceUrl)
-                .then(*this, [](const std::vector<QUrl>& fileList) {
+                .then(ObjectExecutor(this), [](const std::vector<QUrl>& fileList) {
                     // Turn the file list into a frame list.
                     QVector<Frame> frames;
                     frames.reserve(fileList.size());
@@ -404,14 +407,28 @@ Future<QVector<FileSourceImporter::Frame>> FileSourceImporter::discoverFrames(co
 ******************************************************************************/
 Future<QVector<FileSourceImporter::Frame>> FileSourceImporter::discoverFrames(const FileHandle& fileHandle)
 {
-    OVITO_ASSERT_MSG(!QCoreApplication::instance() || QThread::currentThread() == QCoreApplication::instance()->thread(), "FileSourceImporter::discoverFrames", "This function may only be called from the main thread.");
-    OVITO_ASSERT(ExecutionContext::current().isValid());
+    return asyncLaunch([fileHandle, self=OORef<const FileSourceImporter>(this)]() {
+        QVector<Frame> framesList;
+        try {
+            self->discoverFramesInFile(fileHandle, framesList);
+        }
+        catch(const Exception& ex) {
+            // Silently ignore parsing and I/O errors if at least two frames have been read.
+            // Keep all frames read up to where the error occurred.
+            if(framesList.size() <= 1)
+                throw;
+            else {
+                qWarning() << "WARNING: Scanning of the trajectory file" << fileHandle.localFilePath() << "aborted prematurely after" << framesList.size() << "frames due to an error:" << ex.messages().join(' ');
+                framesList.pop_back();       // Remove last discovered frame because it may be corrupted or only partially written.
+            }
+        }
 
-    // Scan file.
-    if(FrameFinderPtr frameFinder = createFrameFinder(fileHandle))
-        return frameFinder->runAsync(true);
-    else
-        return QVector<Frame>{{ Frame(fileHandle) }};
+        // If it's not a trajectory file, report a single frame.
+        if(framesList.empty())
+            framesList.emplace_back(fileHandle);
+
+        return framesList;
+    });
 }
 
 /******************************************************************************
@@ -419,66 +436,35 @@ Future<QVector<FileSourceImporter::Frame>> FileSourceImporter::discoverFrames(co
 ******************************************************************************/
 Future<PipelineFlowState> FileSourceImporter::loadFrame(const LoadOperationRequest& request)
 {
-    OVITO_ASSERT_MSG(!QCoreApplication::instance() || QThread::currentThread() == QCoreApplication::instance()->thread(), "FileSourceImporter::loadFrame", "This function may only be called from the main thread.");
-    OVITO_ASSERT(ExecutionContext::current().isValid());
-
-    // Note: FileSourceImporter::loadFrame() may not be called while undo recording is active.
-    OVITO_ASSERT(!isUndoRecording());
+    OVITO_ASSERT_MSG(this_task::isMainThread(), "FileSourceImporter::loadFrame", "This function may only be called from the main thread.");
 
     // Create the frame loader for the requested frame.
     FrameLoaderPtr frameLoader = createFrameLoader(request);
-    OVITO_ASSERT(frameLoader);
+    if(!frameLoader)
+        throw Exception(tr("Failed to create frame loader for file format '%1'.").arg(objectTitle()));
 
-    // Execute the loader in a background thread.
-    Future<PipelineFlowState> future = frameLoader->runAsync(true);
+    // Execute the loader in a worker thread.
+    return asyncLaunch([frameLoader=std::move(frameLoader), self=OORef<FileSourceImporter>(this)]() {
 
-    // If the parser has detects additional frames following the first frame in the
-    // input file being loaded, automatically turn on scanning of the input file.
-    // Only automatically turn scanning on if the file is being newly imported, i.e. if the file source has not loaded a data collection yet.
-    if(request.isNewlyImportedFile) {
-        // Note: Changing a parameter of the file importer must be done in the correct thread.
-        future.finally(*this, [this](Task& task) noexcept {
-            if(!task.isCanceled()) {
-                FrameLoader& frameLoader = static_cast<FrameLoader&>(task);
-                if(frameLoader.additionalFramesDetected() && !isMultiTimestepFile()) {
-                    ExecutionContext::current().ui().handleExceptions([&] {
-                        setMultiTimestepFile(true);
-                    });
-                }
-            }
-        });
-    }
+        // Let the subclass implementation parse the file.
+        frameLoader->loadFile();
 
-    return future;
-}
+        // Stop the task if it has been canceled.
+        this_task::throwIfCanceled();
 
-/******************************************************************************
-* Scans the source URL for input frames.
-******************************************************************************/
-void FileSourceImporter::FrameFinder::perform()
-{
-    QVector<Frame> frameList;
-    try {
-        discoverFramesInFile(frameList);
-    }
-    catch(const Exception&) {
-        // Silently ignore parsing and I/O errors if at least two frames have been read.
-        // Keep all frames read up to where the error occurred.
-        if(frameList.size() <= 1)
-            throw;
-        else
-            frameList.pop_back();       // Remove last discovered frame because it may be corrupted or only partially written.
-    }
-    setResult(std::move(frameList));
-}
+        // If the parser has detected additional frames following the first frame in the
+        // input file, automatically turn on scanning of the input file.
+        // Only do this if the file is being newly imported by the user.
+        if(frameLoader->additionalFramesDetected() && frameLoader->loadRequest().isNewlyImportedFile && !self->isMultiTimestepFile()) {
+            // Note: Changing a parameter of the file importer must be done in the main thread.
+            launchDetached(ObjectExecutor(self), [self]() {
+                self->setMultiTimestepFile(true);
+            });
+        }
 
-/******************************************************************************
-* Scans the given file for source frames
-******************************************************************************/
-void FileSourceImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::Frame>& frames)
-{
-    // By default, register a single frame.
-    frames.push_back(Frame(fileHandle()));
+        // Pass the constructed pipeline state back to the caller.
+        return std::move(frameLoader->state());
+    });
 }
 
 /******************************************************************************
@@ -486,7 +472,7 @@ void FileSourceImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImp
 ******************************************************************************/
 Future<std::vector<QUrl>> FileSourceImporter::findWildcardMatches(const QUrl& sourceUrl)
 {
-    OVITO_ASSERT(ExecutionContext::current().isValid());
+    OVITO_ASSERT(this_task::get());
 
     // Determine whether the filename contains a wildcard character.
     if(!isWildcardPattern(sourceUrl)) {
@@ -637,17 +623,4 @@ LoadStream& operator>>(LoadStream& stream, FileSourceImporter::Frame& frame)
     return stream;
 }
 
-/******************************************************************************
-* Calls loadFile() and sets the returned frame data as result of the
-* asynchronous task.
-******************************************************************************/
-void FileSourceImporter::FrameLoader::perform()
-{
-    // Let the subclass implementation parse the file.
-    loadFile();
-
-    // Pass the constructed pipeline state back to the caller.
-    setResult(std::move(_loadRequest.state));
-}
-
 }   // End of namespace
