@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -23,24 +23,23 @@
 #include <ovito/core/Core.h>
 #include <ovito/core/dataset/scene/Pipeline.h>
 #include <ovito/core/dataset/data/DataObject.h>
-#include <ovito/core/dataset/data/TransformingDataVis.h>
 #include <ovito/core/dataset/pipeline/PipelineNode.h>
-#include <ovito/core/dataset/pipeline/PipelineEvaluation.h>
+#include <ovito/core/dataset/pipeline/PipelineEvaluationRequest.h>
 #include <ovito/core/dataset/pipeline/ModificationNode.h>
 #include <ovito/core/dataset/DataSetContainer.h>
 #include <ovito/core/dataset/animation/AnimationSettings.h>
 #include <ovito/core/viewport/Viewport.h>
-#include <ovito/core/app/Application.h>
 #include <ovito/core/app/UserInterface.h>
 #include <ovito/core/app/undo/RefTargetOperations.h>
 #include <ovito/core/oo/CloneHelper.h>
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(Pipeline);
+IMPLEMENT_CREATABLE_OVITO_CLASS(Pipeline);
+OVITO_CLASSINFO(Pipeline, "ClassNameAlias", "PipelineSceneNode");  // For backward compatibility with OVITO 3.9.2
 DEFINE_REFERENCE_FIELD(Pipeline, head);
 DEFINE_VECTOR_REFERENCE_FIELD(Pipeline, visElements);
-DEFINE_VECTOR_REFERENCE_FIELD(Pipeline, replacedVisElements);
+DEFINE_RUNTIME_PROPERTY_FIELD(Pipeline, replacedVisElements);
 DEFINE_VECTOR_REFERENCE_FIELD(Pipeline, replacementVisElements);
 DEFINE_REFERENCE_FIELD(Pipeline, source);
 DEFINE_PROPERTY_FIELD(Pipeline, pipelineTrajectoryCachingEnabled);
@@ -52,64 +51,35 @@ SET_PROPERTY_FIELD_CHANGE_EVENT(Pipeline, head, ReferenceEvent::PipelineChanged)
 SET_PROPERTY_FIELD_ALIAS_IDENTIFIER(Pipeline, head, "dataProvider"); // For backward compatibility with OVITO 3.9.2
 SET_PROPERTY_FIELD_ALIAS_IDENTIFIER(Pipeline, source, "pipelineSource"); // For backward compatibility with OVITO 3.9.2
 
-/******************************************************************************
-* Constructor.
-******************************************************************************/
-Pipeline::Pipeline(ObjectInitializationFlags flags) : SceneNode(flags),
-    _pipelineCache(this, false),
-    _pipelineRenderingCache(this, true),
-    _pipelineTrajectoryCachingEnabled(false),
-    _preliminaryUpdatesEnabled(true)
-{
-}
-
+#ifdef OVITO_DEBUG
 /******************************************************************************
 * Destructor.
 ******************************************************************************/
 Pipeline::~Pipeline() // NOLINT
 {
 }
+#endif
 
 /******************************************************************************
-* Performs a synchronous evaluation of the pipeline yielding only preliminary results.
+* Throws an exception if the pipeline stage cannot be evaluated at this time.
+* This is called by the system to catch user mistakes that would lead to infinite recursion.
 ******************************************************************************/
-const PipelineFlowState& Pipeline::evaluatePipelineSynchronous(const PipelineEvaluationRequest& request, bool includeVisElements)
+void Pipeline::preEvaluationCheck(const PipelineEvaluationRequest& request) const
 {
-    OVITO_ASSERT(ExecutionContext::current().isValid());
-    return includeVisElements ?
-        _pipelineRenderingCache.evaluatePipelineSynchronous(request) :
-        _pipelineCache.evaluatePipelineSynchronous(request);
-}
-
-/******************************************************************************
-* Performs an asynchronous evaluation of the data pipeline.
-******************************************************************************/
-PipelineEvaluationFuture Pipeline::evaluatePipeline(const PipelineEvaluationRequest& request)
-{
-    OVITO_ASSERT(ExecutionContext::current().isValid());
-    return PipelineEvaluationFuture(request, _pipelineCache.evaluatePipeline(request), this);
-}
-
-/******************************************************************************
-* Performs an asynchronous evaluation of the data pipeline.
-******************************************************************************/
-PipelineEvaluationFuture Pipeline::evaluateRenderingPipeline(const PipelineEvaluationRequest& request)
-{
-    OVITO_ASSERT(ExecutionContext::current().isValid());
-    return PipelineEvaluationFuture(request, _pipelineRenderingCache.evaluatePipeline(request), this);
+    if(head())
+        head()->preEvaluationCheck(request);
 }
 
 /******************************************************************************
 * Invalidates the data pipeline cache of the object node.
 ******************************************************************************/
-void Pipeline::invalidatePipelineCache(TimeInterval keepInterval, bool resetSynchronousCache)
+void Pipeline::invalidatePipelineCache(TimeInterval keepInterval)
 {
-    // Invalidate data caches.
-    _pipelineCache.invalidate(keepInterval, resetSynchronousCache);
-    _pipelineRenderingCache.invalidate(keepInterval, resetSynchronousCache);
+    // Invalidate data cache.
+    _pipelineCache.invalidate(keepInterval);
 
-    // Also mark the cached bounding box of this scene node as invalid.
-    invalidateBoundingBox();
+    // Also mark the cached bounding box of the scene node as invalid.
+    notifyDependents(Pipeline::BoundingBoxChanged);
 }
 
 /******************************************************************************
@@ -183,11 +153,11 @@ bool Pipeline::referenceEvent(RefTarget* source, const ReferenceEvent& event)
         }
         else if(event.type() == ReferenceEvent::TargetDeleted) {
             // Reduce memory footprint when the pipeline's data provider gets deleted.
-            invalidatePipelineCache(TimeInterval::empty(), true);
+            _pipelineCache.reset();
 
-            // Data provider has been deleted -> delete scene node as well.
+            // Data provider has been deleted -> delete pipeline as well.
             if(!isUndoingOrRedoing())
-                deleteSceneNode();
+                requestObjectDeletion();
         }
         else if(event.type() == ReferenceEvent::PipelineChanged) {
             // Determine the new source node of the pipeline.
@@ -199,25 +169,29 @@ bool Pipeline::referenceEvent(RefTarget* source, const ReferenceEvent& event)
             // Forward animation interval events from the pipeline.
             return true;
         }
-        else if(event.type() == ReferenceEvent::PreliminaryStateAvailable) {
+        else if(event.type() == ReferenceEvent::InteractiveStateAvailable) {
             if(preliminaryUpdatesEnabled()) {
-                // Invalidate the cache whenever the pipeline can provide a new preliminary state.
-                _pipelineCache.invalidateSynchronousState();
-                _pipelineRenderingCache.invalidateSynchronousState();
-                // Also recompute the cached bounding box of this scene node.
-                invalidateBoundingBox();
+                // Invalidate the cache whenever the last pipeline stage can provide a new interactive state.
+                _pipelineCache.invalidateInteractiveState();
+                // Recompute the cached bounding box of the scene node.
+                notifyDependents(Pipeline::BoundingBoxChanged);
                 // Inform all vis elements that their input state has changed when the pipeline reports that a new preliminary output state is available.
                 for(DataVis* vis : visElements())
                     vis->notifyDependents(ReferenceEvent::PipelineInputChanged);
+                // Do not forward this signal to ScenePreparation objects if it comes from the final pipeline stage.
+                // That's to avoid refreshing the viewports twice - once for the preliminary update and once for the final update
+                // when the pipeline evaluation is completed.
+                if(event.sender() == head() && head()->shouldRefreshViewportsAfterEvaluation())
+                    return false;
             }
             else {
-                // Do not forward signal to scene in order to suppress preliminary viewport updates.
+                // Do not forward signal to scene in order to suppress interactive viewport updates.
                 return false;
             }
         }
         else if(event.type() == ReferenceEvent::TargetEnabledOrDisabled) {
             // Inform vis elements that their input state has changed if the last pipeline stage was disabled.
-            // This is necessary, because we don't receive a PreliminaryStateAvailable signal from the pipeline stage in this case.
+            // This is necessary, because we don't receive a InteractiveStateAvailable signal from the pipeline stage in this case.
             for(DataVis* vis : visElements())
                 vis->notifyDependents(ReferenceEvent::PipelineInputChanged);
         }
@@ -225,31 +199,18 @@ bool Pipeline::referenceEvent(RefTarget* source, const ReferenceEvent& event)
     else if(_visElements.contains(source)) {
         if(event.type() == ReferenceEvent::TargetChanged) {
 
-            // Recompute bounding box when a visual element changes.
-            invalidateBoundingBox();
+            // Recompute bounding box of scene node when a visual element changes.
+            notifyDependents(Pipeline::BoundingBoxChanged);
 
-            // Invalidate the rendering pipeline cache whenever an asynchronous visual element changes.
-            if(dynamic_object_cast<TransformingDataVis>(source)) {
-                // Do not completely discard these cached objects, because we might be able to re-use the transformed data objects.
-                _pipelineRenderingCache.invalidate();
-
-                // Trigger a pipeline re-evaluation.
-                // Note: We have to call notifyTargetChanged() here, because the visElements field is flagged as PROPERTY_FIELD_NO_CHANGE_MESSAGE.
-                notifyTargetChanged(PROPERTY_FIELD(visElements));
-            }
-            else {
-                // Trigger an immediate viewport repaint without pipeline re-evaluation.
-                notifyDependents(ReferenceEvent::PreliminaryStateAvailable);
-            }
+            // Trigger an interactive viewport repaint without pipeline re-evaluation.
+            notifyDependents(ReferenceEvent::InteractiveStateAvailable);
         }
     }
-    if(source == this->source()) {
-        if(event.type() == ReferenceEvent::TitleChanged && sceneNodeName().isEmpty()) {
-            // Forward this event to dependents of the pipeline.
-            return true;
-        }
+    if(source == this->source() && event.type() == ReferenceEvent::TitleChanged) {
+        // Forward this event to scene nodes referencing this the pipeline.
+        return true;
     }
-    return SceneNode::referenceEvent(source, event);
+    return RefTarget::referenceEvent(source, event);
 }
 
 /******************************************************************************
@@ -258,11 +219,11 @@ bool Pipeline::referenceEvent(RefTarget* source, const ReferenceEvent& event)
 void Pipeline::referenceReplaced(const PropertyFieldDescriptor* field, RefTarget* oldTarget, RefTarget* newTarget, int listIndex)
 {
     if(field == PROPERTY_FIELD(head)) {
-        // Reset caches when the pipeline's data provider is replaced.
-        invalidatePipelineCache(TimeInterval::empty(), false);
+        // Invalidate caches when the pipeline's data provider is replaced.
+        invalidatePipelineCache();
 
         // The animation length and the title of the pipeline might have changed.
-        if(!isBeingLoaded() && !isAboutToBeDeleted())
+        if(!isBeingLoaded() && !isBeingDeleted())
             notifyDependents(ReferenceEvent::AnimationFramesChanged);
 
         // Determine the new source node of the pipeline.
@@ -277,35 +238,51 @@ void Pipeline::referenceReplaced(const PropertyFieldDescriptor* field, RefTarget
     }
     else if(field == PROPERTY_FIELD(source)) {
         // When the source node of the pipeline is being replaced, the pipeline's title changes.
-        if(sceneNodeName().isEmpty())
-            notifyDependents(ReferenceEvent::TitleChanged);
+        notifyDependents(ReferenceEvent::TitleChanged);
     }
-    SceneNode::referenceReplaced(field, oldTarget, newTarget, listIndex);
+    RefTarget::referenceReplaced(field, oldTarget, newTarget, listIndex);
 }
 
 /******************************************************************************
-* Saves the class' contents to the given stream.
+* Returns the list of scene nodes that share this pipeline.
 ******************************************************************************/
-void Pipeline::saveToStream(ObjectSaveStream& stream, bool excludeRecomputableData) const
+QVector<SceneNode*> Pipeline::sceneNodes() const
 {
-    SceneNode::saveToStream(stream, excludeRecomputableData);
-    stream.beginChunk(0x01);
-    // For future use...
-    stream.endChunk();
+    QVector<SceneNode*> list;
+    visitDependents([&](RefMaker* dependent) {
+        SceneNode* node = dynamic_object_cast<SceneNode>(dependent);
+        if(node != nullptr && node->pipeline() == this)
+            list.push_back(node);
+    });
+    return list;
 }
 
 /******************************************************************************
-* Loads the class' contents from the given stream.
+* Returns one of the scene nodes referencing this pipeline (if any).
 ******************************************************************************/
-void Pipeline::loadFromStream(ObjectLoadStream& stream)
+SceneNode* Pipeline::someSceneNode() const
 {
-    SceneNode::loadFromStream(stream);
-    stream.expectChunk(0x01);
-    // For future use...
-    stream.closeChunk();
+    SceneNode* result = nullptr;
+    visitDependents([&](RefMaker* dependent) {
+        SceneNode* node = dynamic_object_cast<SceneNode>(dependent);
+        if(node != nullptr && node->pipeline() == this)
+            result = node;
+    });
+    return result;
+}
 
-    // Transfer the caching flag loaded from the state file to the internal cache instance.
-    _pipelineRenderingCache.setPrecomputeAllFrames(pipelineTrajectoryCachingEnabled());
+/******************************************************************************
+* Determines  whether this pipeline is currently part of a visualization scene.
+******************************************************************************/
+bool Pipeline::isInScene() const
+{
+    bool result = false;
+    visitDependents([&](RefMaker* dependent) {
+        SceneNode* node = dynamic_object_cast<SceneNode>(dependent);
+        if(node != nullptr && node->pipeline() == this && node->isInScene())
+            result = true;
+    });
+    return result;
 }
 
 /******************************************************************************
@@ -313,9 +290,8 @@ void Pipeline::loadFromStream(ObjectLoadStream& stream)
 ******************************************************************************/
 void Pipeline::rescaleTime(const TimeInterval& oldAnimationInterval, const TimeInterval& newAnimationInterval)
 {
-    SceneNode::rescaleTime(oldAnimationInterval, newAnimationInterval);
+    RefTarget::rescaleTime(oldAnimationInterval, newAnimationInterval);
     _pipelineCache.invalidate();
-    _pipelineRenderingCache.invalidate();
 }
 
 /******************************************************************************
@@ -323,30 +299,26 @@ void Pipeline::rescaleTime(const TimeInterval& oldAnimationInterval, const TimeI
 ******************************************************************************/
 QString Pipeline::objectTitle() const
 {
-    // If a user-defined name has been assigned to this pipeline, return it as the pipeline's display title.
-    if(!sceneNodeName().isEmpty())
-        return sceneNodeName();
-
-    // Otherwise, use the title of the pipeline's data source.
+    // Use the title of the pipeline's data source by default.
     if(source())
         return source()->objectTitle();
 
     // Fall back to default behavior.
-    return SceneNode::objectTitle();
+    return RefTarget::objectTitle();
 }
 
 /******************************************************************************
 * Applies a modifier by appending a node for it to the pipeline.
 ******************************************************************************/
-ModificationNode* Pipeline::applyModifier(AnimationTime time, Modifier* modifier)
+ModificationNode* Pipeline::applyModifier(AnimationTime time, bool interactiveMode, Modifier* modifier)
 {
     OVITO_ASSERT(modifier);
-    OVITO_ASSERT(ExecutionContext::current().isValid());
+    OVITO_ASSERT(this_task::get());
 
     OORef<ModificationNode> node = modifier->createModificationNode();
     node->setModifier(modifier);
     node->setInput(head());
-    modifier->initializeModifier(ModifierInitializationRequest(time, node));
+    modifier->initializeModifier(ModifierInitializationRequest(time, false, interactiveMode, node));
     setHead(node);
     return node;
 }
@@ -385,11 +357,11 @@ void Pipeline::setSource(PipelineNode* sourceObject)
 }
 
 /******************************************************************************
-* Computes the bounding box of the scene node in local coordinates.
+* Computes the axis-aligned bounding box of the pipeline's visual output in local coordinates.
 ******************************************************************************/
 Box3 Pipeline::localBoundingBox(AnimationTime time, TimeInterval& validity) const
 {
-    const PipelineFlowState& state = const_cast<Pipeline*>(this)->evaluatePipelineSynchronous(time, true);
+    const PipelineFlowState& state = getCachedPipelineOutput(time);
 
     // Let visual elements compute the bounding boxes of the data objects.
     Box3 bb;
@@ -410,21 +382,20 @@ void Pipeline::getDataObjectBoundingBox(AnimationTime time, const DataObject* da
 
     // Call all vis elements of the data object.
     for(DataVis* vis : dataObj->visElements()) {
-        // Let the pipeline substitude the vis element with another one.
+        // Let the pipeline substitute the vis element with another one.
         vis = getReplacementVisElement(vis);
         if(vis->isEnabled()) {
-            MixedKeyCache& visCache = Application::instance()->visCache();
-
             // Push the data object onto the stack.
             if(!isOnStack) {
                 dataObjectPath.push_back(dataObj);
                 isOnStack = true;
             }
             try {
-                // Let the vis element do the rendering.
-                bb.addBox(vis->boundingBox(time, dataObjectPath, this, state, visCache, validity));
+                // Let the vis element compute the bounding box in local coordinates.
+                bb.addBox(vis->boundingBoxImmediate(time, dataObjectPath, this, state, validity));
             }
             catch(const Exception& ex) {
+                // Swallow any errors that might occur during the computation of the bounding box after printing them to the console.
                 ex.logError();
             }
         }
@@ -448,11 +419,11 @@ void Pipeline::getDataObjectBoundingBox(AnimationTime time, const DataObject* da
 }
 
 /******************************************************************************
-* Deletes this node from the scene.
+* Asks this object to delete itself.
 ******************************************************************************/
-void Pipeline::deleteSceneNode()
+void Pipeline::requestObjectDeletion()
 {
-    OVITO_ASSERT(ExecutionContext::current().isValid());
+    OVITO_ASSERT(this_task::get());
 
     // Temporary reference to the pipeline's stages.
     OORef<PipelineNode> oldHead = head();
@@ -470,14 +441,14 @@ void Pipeline::deleteSceneNode()
             next = modNode->input();
         // Delete the pipeline stage if it is not part of any other pipeline in the scene.
         if(oldHead->pipelines(false).isEmpty())
-            oldHead->deleteReferenceObject();
+            oldHead->requestObjectDeletion();
         oldHead = std::move(next);
     }
 
     // Discard transient references to visual elements.
     _visElements.clear(this, PROPERTY_FIELD(visElements));
 
-    SceneNode::deleteSceneNode();
+    RefTarget::requestObjectDeletion();
 }
 
 /******************************************************************************
@@ -489,25 +460,7 @@ void Pipeline::referenceInserted(const PropertyFieldDescriptor* field, RefTarget
         // Reset pipeline cache if a new replacement for a visual element is assigned.
         invalidatePipelineCache();
     }
-    SceneNode::referenceInserted(field, newTarget, listIndex);
-}
-
-/******************************************************************************
-* Is called when a RefTarget has been added to a VectorReferenceField of this RefMaker.
-******************************************************************************/
-void Pipeline::referenceRemoved(const PropertyFieldDescriptor* field, RefTarget* oldTarget, int listIndex)
-{
-    if(field == PROPERTY_FIELD(replacedVisElements) && !isAboutToBeDeleted()) {
-        // If an upstream vis element is being removed from the list, because the weakly referenced vis element is being deleted,
-        // then also discard our corresponding replacement element managed by the pipeline.
-        if(!isUndoingOrRedoing()) {
-            OVITO_ASSERT(replacedVisElements().size() + 1 == replacementVisElements().size());
-            _replacementVisElements.remove(this, PROPERTY_FIELD(replacementVisElements), listIndex);
-        }
-        // Reset pipeline cache if a replacement for a visual element is removed.
-        invalidatePipelineCache();
-    }
-    SceneNode::referenceRemoved(field, oldTarget, listIndex);
+    RefTarget::referenceInserted(field, newTarget, listIndex);
 }
 
 /******************************************************************************
@@ -516,7 +469,7 @@ void Pipeline::referenceRemoved(const PropertyFieldDescriptor* field, RefTarget*
 void Pipeline::propertyChanged(const PropertyFieldDescriptor* field)
 {
     if(field == PROPERTY_FIELD(pipelineTrajectoryCachingEnabled)) {
-        _pipelineRenderingCache.setPrecomputeAllFrames(pipelineTrajectoryCachingEnabled());
+        _pipelineCache.setPrecomputeAllFrames(pipelineTrajectoryCachingEnabled());
 
         // Send target changed event to trigger a new pipeline evaluation, which is
         // needed to start the precomputation process.
@@ -524,7 +477,52 @@ void Pipeline::propertyChanged(const PropertyFieldDescriptor* field)
             notifyTargetChanged(PROPERTY_FIELD(pipelineTrajectoryCachingEnabled));
     }
 
-    SceneNode::propertyChanged(field);
+    RefTarget::propertyChanged(field);
+}
+
+/******************************************************************************
+* Saves the class' contents to the given stream.
+******************************************************************************/
+void Pipeline::saveToStream(ObjectSaveStream& stream, bool excludeRecomputableData) const
+{
+    RefTarget::saveToStream(stream, excludeRecomputableData);
+    stream.beginChunk(0x02);
+    // Save list of weak references to vis elements that have been replaced with local copies.
+    stream.writeSizeT(replacedVisElements().size());
+    for(const auto& weakRef : replacedVisElements()) {
+        stream.saveObject(weakRef.lock(), excludeRecomputableData);
+    }
+    stream.endChunk();
+}
+
+/******************************************************************************
+* Loads the class' contents from the given stream.
+******************************************************************************/
+void Pipeline::loadFromStream(ObjectLoadStream& stream)
+{
+    RefTarget::loadFromStream(stream);
+
+    // For backward compatibility with OVITO 3.11:
+    // The Pipeline class used to be derived from the SceneNode class.
+    // Parse the placeholder chunk which used to be written by the SceneNode::saveToStream() method in older OVITO versions.
+    if(stream.formatVersion() < 30013) {
+        stream.expectChunk(0x02);
+        stream.closeChunk();
+    }
+
+    int version = stream.expectChunkRange(0x01, 1);
+    if(version >= 1) {
+        // Load list of weak references to replaced vis elements.
+        std::vector<OOWeakRef<DataVis>> visElements(stream.readSizeT());
+        for(auto& weakRef : visElements) {
+            weakRef = stream.loadObject<DataVis>();
+        }
+        setReplacedVisElements(std::move(visElements));
+    }
+    stream.closeChunk();
+
+    // Transfer the caching flag loaded from the state file to the internal cache instance.
+    _pipelineCache.setPrecomputeAllFrames(pipelineTrajectoryCachingEnabled());
 }
 
 /******************************************************************************
@@ -533,30 +531,141 @@ void Pipeline::propertyChanged(const PropertyFieldDescriptor* field)
 ******************************************************************************/
 void Pipeline::loadFromStreamComplete(ObjectLoadStream& stream)
 {
-    SceneNode::loadFromStreamComplete(stream);
+    RefTarget::loadFromStreamComplete(stream);
 
-    // Remove null entries from the replacedVisElements list due to expired weak references.
-    for(int i = replacedVisElements().size() - 1; i >= 0; i--) {
-        if(replacedVisElements()[i] == nullptr) {
-            _replacedVisElements.remove(this, PROPERTY_FIELD(replacedVisElements), i);
-        }
-    }
     OVITO_ASSERT(replacedVisElements().size() == replacementVisElements().size());
     OVITO_ASSERT(!isUndoRecording());
+
+    // Remove null entries from the replacedVisElements and replacementVisElements lists due to expired weak references.
+    if(boost::algorithm::any_of(replacedVisElements(), std::mem_fn(&OOWeakRef<DataVis>::expired))) {
+        auto newReplacedVisElements = replacedVisElements();
+        for(int i = (int)newReplacedVisElements.size() - 1; i >= 0; i--) {
+            if(newReplacedVisElements[i].expired()) {
+                newReplacedVisElements.erase(newReplacedVisElements.begin() + i);
+                _replacementVisElements.remove(this, PROPERTY_FIELD(replacementVisElements), i);
+            }
+        }
+        setReplacedVisElements(std::move(newReplacedVisElements));
+        OVITO_ASSERT(replacedVisElements().size() == replacementVisElements().size());
+    }
+
+    // Clear the ad-hoc reference to the SceneNode to avoid the circular reference.
+    _deserializationSceneNode.reset();
 }
 
 /******************************************************************************
-* Returns the internal replacement for the given data vis element.
+* For backward compatibility with OVITO 3.11:
+* The Pipeline class has been separated from the SceneNode base class in OVITO 3.12.
+* A separate SceneNode must now be created when loading an old Pipeline object from a state file.
+* This function creates and returns the SceneNode for this pipeline object during deserialization of legacy state files.
+******************************************************************************/
+OORef<SceneNode>& Pipeline::deserializationSceneNode()
+{
+    if(!_deserializationSceneNode) {
+        _deserializationSceneNode = OORef<SceneNode>::create();
+        _deserializationSceneNode->setPipeline(this);
+    }
+    return _deserializationSceneNode;
+}
+
+/******************************************************************************
+* Provides a custom function that takes are of the deserialization of a
+* serialized property field that has been removed or changed in a newer version of OVITO.
+* This is needed for file backward compatibility with OVITO 3.11.
+******************************************************************************/
+RefMakerClass::SerializedClassInfo::PropertyFieldInfo::CustomDeserializationFunctionPtr Pipeline::OOMetaClass::overrideFieldDeserialization(LoadStream& stream, const SerializedClassInfo::PropertyFieldInfo& field) const
+{
+    // For backward compatibility with OVITO 3.11:
+    // The 'replacedVisElements' list used to be a vector reference field in previous program versions.
+    // Now it is a simple property field holding a vector of weak references to vis elements.
+    if(field.definingClass == &Pipeline::OOClass() && stream.formatVersion() < 30013) {
+        if(field.identifier == "replacedVisElements") {
+            return [](const SerializedClassInfo::PropertyFieldInfo& field, ObjectLoadStream& stream, RefMaker& owner) {
+                stream.expectChunk(0x02);
+                qint32 numElements;
+                stream >> numElements;
+                std::vector<OOWeakRef<DataVis>> elements;
+                for(qint32 i = 0; i < numElements; i++) {
+                    elements.push_back(stream.loadObject<DataVis>());
+                }
+                static_object_cast<Pipeline>(&owner)->setReplacedVisElements(std::move(elements));
+                stream.closeChunk();
+            };
+        }
+    }
+
+    // For backward compatibility with OVITO 3.11:
+    // The Pipeline class has been split from the SceneNode base class. This means we have to handle
+    // the deserialization of the SceneNode fields here and then copy them over to the separate SceneNode instance.
+    if(field.definingClass == &SceneNode::OOClass() && stream.formatVersion() < 30013) {
+        if(field.identifier == "displayColor") {
+            return [](const SerializedClassInfo::PropertyFieldInfo& field, ObjectLoadStream& stream, RefMaker& owner) {
+                stream.expectChunk(0x04);
+                static_object_cast<Pipeline>(&owner)->deserializationSceneNode()->_displayColor.loadFromStream(stream);
+                stream.closeChunk();
+            };
+        }
+        else if(field.identifier == "sceneNodeName" || field.identifier == "nodeName") {
+            return [](const SerializedClassInfo::PropertyFieldInfo& field, ObjectLoadStream& stream, RefMaker& owner) {
+                stream.expectChunk(0x04);
+                static_object_cast<Pipeline>(&owner)->deserializationSceneNode()->_sceneNodeName.loadFromStream(stream);
+                stream.closeChunk();
+            };
+        }
+        else if(field.identifier == "transformationController") {
+            return [](const SerializedClassInfo::PropertyFieldInfo& field, ObjectLoadStream& stream, RefMaker& owner) {
+                stream.expectChunk(0x02);
+                static_object_cast<Pipeline>(&owner)->deserializationSceneNode()->setTransformationController(stream.loadObject<Controller>());
+                stream.closeChunk();
+            };
+        }
+        else if(field.identifier == "lookatTargetNode") {
+            return [](const SerializedClassInfo::PropertyFieldInfo& field, ObjectLoadStream& stream, RefMaker& owner) {
+                stream.expectChunk(0x02);
+                SceneNode* node = static_object_cast<Pipeline>(&owner)->deserializationSceneNode();
+                if(OORef<Pipeline> targetPipeline = stream.loadObject<Pipeline>())
+                    node->_lookatTargetNode.set(node, PROPERTY_FIELD(SceneNode::lookatTargetNode), targetPipeline->deserializationSceneNode());
+                stream.closeChunk();
+            };
+        }
+        else if(field.identifier == "hiddenInViewports") {
+            return [](const SerializedClassInfo::PropertyFieldInfo& field, ObjectLoadStream& stream, RefMaker& owner) {
+                stream.expectChunk(0x02);
+                qint32 numHiddenInViewports;
+                stream >> numHiddenInViewports;
+                std::vector<OOWeakRef<Viewport>> viewports;
+                for(qint32 i = 0; i < numHiddenInViewports; i++) {
+                    viewports.push_back(stream.loadObject<Viewport>());
+                }
+                static_object_cast<Pipeline>(&owner)->deserializationSceneNode()->setHiddenInViewports(std::move(viewports));
+                stream.closeChunk();
+            };
+        }
+        else if(field.identifier == "children") {
+            return [](const SerializedClassInfo::PropertyFieldInfo& field, ObjectLoadStream& stream, RefMaker& owner) {
+                stream.expectChunk(0x02);
+                qint32 numChildren;
+                stream >> numChildren;
+                OVITO_ASSERT(numChildren == 0);
+                stream.closeChunk();
+            };
+        }
+    }
+
+    return RefTarget::OOMetaClass::overrideFieldDeserialization(stream, field);
+}
+
+/******************************************************************************
+* Returns the internal replacement for the given vis element.
 * If there is no replacement, the original vis element is returned.
 ******************************************************************************/
 DataVis* Pipeline::getReplacementVisElement(DataVis* vis) const
 {
-    OVITO_ASSERT(replacementVisElements().size() == replacedVisElements().size());
-    OVITO_ASSERT(std::find(replacedVisElements().begin(), replacedVisElements().end(), nullptr) == replacedVisElements().end());
     OVITO_ASSERT(vis);
-    int index = replacedVisElements().indexOf(vis);
-    if(index >= 0)
-        return replacementVisElements()[index];
+    OVITO_ASSERT(replacementVisElements().size() == replacedVisElements().size());
+    auto it = boost::find(replacedVisElements(), vis);
+    if(it != replacedVisElements().end())
+        return replacementVisElements()[std::distance(replacedVisElements().begin(), it)];
     else
         return vis;
 }
@@ -568,9 +677,12 @@ DataVis* Pipeline::getReplacementVisElement(DataVis* vis) const
 DataVis* Pipeline::makeVisElementIndependent(DataVis* visElement)
 {
     OVITO_ASSERT(visElement != nullptr);
-    OVITO_ASSERT(!replacedVisElements().contains(visElement));
     OVITO_ASSERT(replacedVisElements().size() == replacementVisElements().size());
 
+    // Check if the visual element is already replaced.
+    if(DataVis* replacement = getReplacementVisElement(visElement); replacement != visElement)
+        return replacement;
+
     // Clone the visual element.
     OORef<DataVis> clonedVisElement = CloneHelper::cloneSingleObject(visElement, true);
     DataVis* newVis = clonedVisElement.get();
@@ -580,10 +692,12 @@ DataVis* Pipeline::makeVisElementIndependent(DataVis* visElement)
 
     // Put the copy into our mapping table, which will subsequently be applied
     // after every pipeline evaluation to replace the upstream visual element
-    // with our local copy.
-    int index = replacementVisElements().indexOf(visElement);
+    // with our private copy.
+    auto index = replacementVisElements().indexOf(visElement);
     if(index == -1) {
-        _replacedVisElements.push_back(this, PROPERTY_FIELD(replacedVisElements), visElement);
+        auto newList = replacedVisElements();
+        newList.push_back(visElement);
+        setReplacedVisElements(std::move(newList));
         _replacementVisElements.push_back(this, PROPERTY_FIELD(replacementVisElements), std::move(clonedVisElement));
     }
     else {
@@ -626,8 +740,7 @@ void Pipeline::collectDataObjectsForVisElement(ConstDataObjectPath& path, DataVi
 /******************************************************************************
 * Gathers a list of data objects from the given pipeline flow state (which
 * should have been produced by this pipeline) that are associated with the
-* given vis element. This method takes into account replacement vis elements
-* of this pipeline node.
+* given vis element. This method takes into account replacement vis elements.
 ******************************************************************************/
 std::vector<ConstDataObjectPath> Pipeline::getDataObjectsForVisElement(const PipelineFlowState& state, DataVis* vis) const
 {
