@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -32,16 +32,16 @@
 #include <ovito/core/dataset/DataSetContainer.h>
 #include <ovito/core/dataset/animation/TimeInterval.h>
 #include <ovito/core/oo/CloneHelper.h>
-#include <ovito/core/app/Application.h>
 #include <ovito/core/app/UserInterface.h>
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(SceneNode);
+IMPLEMENT_CREATABLE_OVITO_CLASS(SceneNode);
 DEFINE_REFERENCE_FIELD(SceneNode, transformationController);
 DEFINE_REFERENCE_FIELD(SceneNode, lookatTargetNode);
+DEFINE_REFERENCE_FIELD(SceneNode, pipeline);
 DEFINE_VECTOR_REFERENCE_FIELD(SceneNode, children);
-DEFINE_VECTOR_REFERENCE_FIELD(SceneNode, hiddenInViewports);
+DEFINE_RUNTIME_PROPERTY_FIELD(SceneNode, hiddenInViewports);
 DEFINE_PROPERTY_FIELD(SceneNode, sceneNodeName);
 DEFINE_PROPERTY_FIELD(SceneNode, displayColor);
 SET_PROPERTY_FIELD_LABEL(SceneNode, transformationController, "Transformation");
@@ -49,26 +49,27 @@ SET_PROPERTY_FIELD_LABEL(SceneNode, lookatTargetNode, "Target");
 SET_PROPERTY_FIELD_LABEL(SceneNode, children, "Children");
 SET_PROPERTY_FIELD_LABEL(SceneNode, sceneNodeName, "Name");
 SET_PROPERTY_FIELD_LABEL(SceneNode, displayColor, "Display color");
+SET_PROPERTY_FIELD_LABEL(SceneNode, pipeline, "Pipeline");
 SET_PROPERTY_FIELD_CHANGE_EVENT(SceneNode, sceneNodeName, ReferenceEvent::TitleChanged);
 SET_PROPERTY_FIELD_ALIAS_IDENTIFIER(SceneNode, sceneNodeName, "nodeName"); // For backward compatibility with OVITO 3.9.2
 
 /******************************************************************************
 * Constructor.
 ******************************************************************************/
-SceneNode::SceneNode(ObjectInitializationFlags flags) : RefTarget(flags),
-    _worldTransform(AffineTransformation::Identity()),
-    _worldTransformValidity(TimeInterval::empty()),
-    _boundingBoxValidity(TimeInterval::empty()),
-    _displayColor(0,0,0)
+void SceneNode::initializeObject(ObjectInitializationFlags flags)
 {
-    if(!flags.testFlag(DontInitializeObject)) {
-        // Create a transformation controller for the node.
-        setTransformationController(ControllerManager::createTransformationController());
+    RefTarget::initializeObject(flags);
 
+    if(!flags.testFlag(DontInitializeObject)) {
         // Assign random color to scene node.
-        if(ExecutionContext::isInteractive()) {
+        if(this_task::isInteractive()) {
             static std::default_random_engine rng;
-            setDisplayColor(Color::fromHSV(std::uniform_real_distribution<FloatType>()(rng), 1, 1));
+            setDisplayColor(Color::fromHSV(boost::random::uniform_real_distribution<FloatType>()(rng), 1, 1));
+        }
+
+        if(!isRootNode()) {
+            // Create a transformation controller for the node (the root doesn't need one).
+            setTransformationController(ControllerManager::createTransformationController());
         }
     }
 }
@@ -79,6 +80,8 @@ SceneNode::SceneNode(ObjectInitializationFlags flags) : RefTarget(flags),
 ******************************************************************************/
 const AffineTransformation& SceneNode::getWorldTransform(AnimationTime time, TimeInterval& validityInterval) const
 {
+    OVITO_ASSERT(this_task::isMainThread());
+
     if(!_worldTransformValidity.contains(time)) {
         _worldTransformValidity.setInfinite();
         _worldTransform.setIdentity();
@@ -101,6 +104,8 @@ const AffineTransformation& SceneNode::getWorldTransform(AnimationTime time, Tim
 ******************************************************************************/
 AffineTransformation SceneNode::getLocalTransform(AnimationTime time, TimeInterval& validityInterval) const
 {
+    OVITO_ASSERT(this_task::isMainThread());
+
     AffineTransformation result = AffineTransformation::Identity();
     if(transformationController())
         transformationController()->applyTransformation(time, result, validityInterval);
@@ -113,34 +118,57 @@ AffineTransformation SceneNode::getLocalTransform(AnimationTime time, TimeInterv
 ******************************************************************************/
 void SceneNode::invalidateWorldTransformation()
 {
+    OVITO_ASSERT(this_task::isMainThread());
+
     _worldTransformValidity.setEmpty();
     invalidateBoundingBox();
     for(SceneNode* child : children())
         child->invalidateWorldTransformation();
-    notifyDependents(ReferenceEvent::TransformationChanged);
+    notifyDependents(SceneNode::TransformationChanged);
 }
 
 /******************************************************************************
-* Deletes this node from the scene. This will also delete all child nodes.
+* Asks this object to delete itself.
 ******************************************************************************/
-void SceneNode::deleteSceneNode()
+void SceneNode::requestObjectDeletion()
 {
-    // Delete target too.
-    OORef<SceneNode> tn = lookatTargetNode();
-    if(tn) {
+    // Delete target scene node too.
+    if(OORef<SceneNode> tn = lookatTargetNode()) {
         // Clear reference first to prevent infinite recursion.
         _lookatTargetNode.set(this, PROPERTY_FIELD(lookatTargetNode), nullptr);
-        tn->deleteSceneNode();
+        tn->requestObjectDeletion();
     }
 
     // Delete all child nodes recursively.
-    for(SceneNode* child : children())
-        child->deleteSceneNode();
-
+    deleteChildren();
     OVITO_ASSERT(children().empty());
 
-    // Delete node itself.
-    deleteReferenceObject();
+    // Delete pipeline if there are no more scene nodes referencing the same pipeline.
+    if(OORef<Pipeline> pl = pipeline()) {
+        setPipeline(nullptr);
+        if(!pl->someSceneNode())
+            pl->requestObjectDeletion();
+    }
+
+    // Delete scene node itself.
+    RefTarget::requestObjectDeletion();
+}
+
+/******************************************************************************
+* Returns the title of this object.
+******************************************************************************/
+QString SceneNode::objectTitle() const
+{
+    // If a user-defined name has been assigned to this scene node, return it as the its display title.
+    if(!sceneNodeName().isEmpty())
+        return sceneNodeName();
+
+    // Otherwise, use the title of the pipeline.
+    if(pipeline())
+        return pipeline()->objectTitle();
+
+    // Fall back to default behavior.
+    return RefTarget::objectTitle();
 }
 
 /******************************************************************************
@@ -168,7 +196,7 @@ LookAtController* SceneNode::setLookatTargetNode(AnimationTime time, SceneNode*
         if(targetNode) {
             OVITO_CHECK_OBJECT_POINTER(targetNode);
 
-            // Create a look at controller.
+            // Create a look-at controller.
             OORef<LookAtController> lookAtCtrl = dynamic_object_cast<LookAtController>(prs->rotationController());
             if(!lookAtCtrl)
                 lookAtCtrl = OORef<LookAtController>::create();
@@ -210,13 +238,24 @@ bool SceneNode::referenceEvent(RefTarget* source, const ReferenceEvent& event)
             invalidateBoundingBox();
         }
     }
-    else if(event.type() == ReferenceEvent::TargetDeleted && source == lookatTargetNode()) {
-        // Lookat target node has been deleted -> delete this node too.
-        if(!isUndoingOrRedoing())
-            deleteSceneNode();
+    else if(event.type() == ReferenceEvent::TargetDeleted) {
+        if(source == pipeline() || source == lookatTargetNode()) {
+            // Pipeline or look-at target have been deleted -> delete this node too.
+            if(!isUndoingOrRedoing())
+                requestObjectDeletion();
+        }
     }
-    else if(event.type() == ReferenceEvent::AnimationFramesChanged && children().contains(static_cast<SceneNode*>(source))) {
-        onAnimationFramesChanged();
+    else if(event.type() == ReferenceEvent::AnimationFramesChanged) {
+        if(source == pipeline() || children().contains(static_cast<SceneNode*>(source)))
+            onAnimationFramesChanged();
+    }
+    else if(source == pipeline() && event.type() == Pipeline::BoundingBoxChanged) {
+        // Mark the cached bounding box of this scene node as invalid.
+        invalidateBoundingBox();
+    }
+    if(source == this->pipeline() && event.type() == ReferenceEvent::TitleChanged && sceneNodeName().isEmpty()) {
+        // Forward this event to dependents.
+        return true;
     }
     return RefTarget::referenceEvent(source, event);
 }
@@ -247,6 +286,11 @@ void SceneNode::referenceReplaced(const PropertyFieldDescriptor* field, RefTarge
         // The animation length might have changed when an object has been removed from the scene.
         onAnimationFramesChanged();
     }
+    else if(field == PROPERTY_FIELD(pipeline)) {
+        // When the pipeline of the scene node is being replaced, the scene node's title changes.
+        if(sceneNodeName().isEmpty())
+            notifyDependents(ReferenceEvent::TitleChanged);
+    }
     RefTarget::referenceReplaced(field, oldTarget, newTarget, listIndex);
 }
 
@@ -283,7 +327,7 @@ void SceneNode::referenceRemoved(const PropertyFieldDescriptor* field, RefTarget
         OVITO_ASSERT(child->parentNode() == this);
         child->_parentNode = nullptr;
 
-        if(!isAboutToBeDeleted()) {
+        if(!isBeingDeleted()) {
             // Invalidate cached world bounding box of this parent node.
             invalidateBoundingBox();
 
@@ -311,6 +355,7 @@ void SceneNode::invalidateBoundingBox()
 void SceneNode::insertChildNode(qsizetype index, OORef<SceneNode> newChild)
 {
     OVITO_CHECK_OBJECT_POINTER(newChild);
+    OVITO_ASSERT(this_task::isMainThread());
 
     // Check whether it is already a child of this parent.
     if(newChild->parentNode() == this) {
@@ -332,7 +377,7 @@ void SceneNode::insertChildNode(qsizetype index, OORef<SceneNode> newChild)
 
     // Adjust transformation to preserve world position.
     TimeInterval iv;
-    AnimationTime time = ExecutionContext::current().ui().datasetContainer().currentAnimationTime();
+    AnimationTime time = this_task::ui()->datasetContainer().currentAnimationTime();
     const AffineTransformation& newParentTM = getWorldTransform(time, iv);
     if(newParentTM != AffineTransformation::Identity())
         newChild->transformationController()->changeParent(time, AffineTransformation::Identity(), newParentTM, newChild);
@@ -345,6 +390,7 @@ void SceneNode::insertChildNode(qsizetype index, OORef<SceneNode> newChild)
 void SceneNode::removeChildNode(qsizetype index)
 {
     OVITO_ASSERT(index >= 0 && index < children().size());
+    OVITO_ASSERT(this_task::isMainThread());
 
     OORef<SceneNode> child = children()[index];
     OVITO_ASSERT_MSG(child->parentNode() == this, "SceneNode::removeChildNode()", "The node to be removed is not a child of this parent node.");
@@ -356,7 +402,7 @@ void SceneNode::removeChildNode(qsizetype index)
 
     // Update child node.
     TimeInterval iv;
-    AnimationTime time = ExecutionContext::current().ui().datasetContainer().currentAnimationTime();
+    AnimationTime time = this_task::ui()->datasetContainer().currentAnimationTime();
     AffineTransformation oldParentTM = getWorldTransform(time, iv);
     if(oldParentTM != AffineTransformation::Identity())
         child->transformationController()->changeParent(time, oldParentTM, AffineTransformation::Identity(), child);
@@ -397,8 +443,12 @@ void SceneNode::saveToStream(ObjectSaveStream& stream, bool excludeRecomputableD
 {
     RefTarget::saveToStream(stream, excludeRecomputableData);
 
-    stream.beginChunk(0x02);
-    // This is for future use...
+    stream.beginChunk(0x03);
+    // Save list of weak references to viewports in which the node is hidden.
+    stream.writeSizeT(hiddenInViewports().size());
+    for(const auto& vpWeakRef : hiddenInViewports()) {
+        stream.saveObject(vpWeakRef.lock(), excludeRecomputableData);
+    }
     stream.endChunk();
 }
 
@@ -409,8 +459,17 @@ void SceneNode::loadFromStream(ObjectLoadStream& stream)
 {
     RefTarget::loadFromStream(stream);
 
-    stream.expectChunkRange(0x01, 0x02);
-    // This is for future use...
+    int version = stream.expectChunkRange(0x01, 2);
+    if(version >= 2) {
+        // Load list of weak references to viewports in which the node is hidden.
+        size_t numHiddenInViewports = stream.readSizeT();
+        std::vector<OOWeakRef<Viewport>> viewports;
+        for(size_t i = 0; i < numHiddenInViewports; i++) {
+            if(OORef<Viewport> vp = stream.loadObject<Viewport>())
+                viewports.push_back(std::move(vp));
+        }
+        setHiddenInViewports(std::move(viewports));
+    }
     stream.closeChunk();
 
     // Restore parent/child hierarchy.
@@ -418,6 +477,47 @@ void SceneNode::loadFromStream(ObjectLoadStream& stream)
         child->_parentNode = this;
 }
 
+/******************************************************************************
+* Provides a custom function that takes are of the deserialization of a
+* serialized property field that has been removed or changed in a newer version of OVITO.
+* This is needed for file backward compatibility with OVITO 3.11.
+******************************************************************************/
+RefMakerClass::SerializedClassInfo::PropertyFieldInfo::CustomDeserializationFunctionPtr SceneNode::OOMetaClass::overrideFieldDeserialization(LoadStream& stream, const SerializedClassInfo::PropertyFieldInfo& field) const
+{
+    // For backward compatibility with OVITO 3.11:
+    if(field.definingClass == &SceneNode::OOClass() && stream.formatVersion() < 30013) {
+        // The 'hiddenInViewports' list used to be a vector reference field in previous OVITO versions.
+        // Now it is a simple property field holding a vector of weak references to viewports.
+        if(field.identifier == "hiddenInViewports") {
+            return [](const SerializedClassInfo::PropertyFieldInfo& field, ObjectLoadStream& stream, RefMaker& owner) {
+                stream.expectChunk(0x02);
+                qint32 numHiddenInViewports;
+                stream >> numHiddenInViewports;
+                std::vector<OOWeakRef<Viewport>> viewports;
+                for(qint32 i = 0; i < numHiddenInViewports; i++) {
+                    viewports.push_back(stream.loadObject<Viewport>());
+                }
+                static_object_cast<SceneNode>(&owner)->setHiddenInViewports(std::move(viewports));
+                stream.closeChunk();
+            };
+        }
+        // The Pipeline class has been split from the SceneNode base class in OVITO 3.12. This means we have to handle
+        // the deserialization of the children field here, which used to be a list of Pipeline objects (now a list of SceneNode instances).
+        else if(field.identifier == "children") {
+            return [](const SerializedClassInfo::PropertyFieldInfo& field, ObjectLoadStream& stream, RefMaker& owner) {
+                stream.expectChunk(0x02);
+                qint32 numChildren;
+                stream >> numChildren;
+                for(qint32 i = 0; i < numChildren; i++) {
+                    static_object_cast<SceneNode>(&owner)->_children.insert(&owner, PROPERTY_FIELD(SceneNode::children), i, stream.loadObject<Pipeline>()->deserializationSceneNode());
+                }
+                stream.closeChunk();
+            };
+        }
+    }
+    return RefTarget::OOMetaClass::overrideFieldDeserialization(stream, field);
+}
+
 /******************************************************************************
 * Creates a copy of this object.
 ******************************************************************************/
@@ -430,7 +530,7 @@ OORef<RefTarget> SceneNode::clone(bool deepCopy, CloneHelper& cloneHelper) const
     if(clone->lookatTargetNode()) {
         OVITO_ASSERT(lookatTargetNode());
 
-        // Insert the cloned target into the same scene as out target.
+        // Insert the cloned target into the same scene as our target.
         if(lookatTargetNode()->parentNode() && !clone->lookatTargetNode()->parentNode()) {
             lookatTargetNode()->parentNode()->addChildNode(clone->lookatTargetNode());
         }
@@ -442,21 +542,31 @@ OORef<RefTarget> SceneNode::clone(bool deepCopy, CloneHelper& cloneHelper) const
     return clone;
 }
 
+/******************************************************************************
+* Computes the bounding box of the scene node in local coordinates.
+******************************************************************************/
+Box3 SceneNode::localBoundingBox(AnimationTime time) const
+{
+    if(!_boundingBoxValidity.contains(time)) {
+        _boundingBoxValidity.setInfinite();
+        _localBoundingBox = localBoundingBoxInternal(time, _boundingBoxValidity);
+    }
+    return _localBoundingBox;
+}
+
 /******************************************************************************
 * Returns the bounding box of the scene node in world coordinates.
 *    time - The time at which the bounding box should be returned.
 ******************************************************************************/
 Box3 SceneNode::worldBoundingBox(AnimationTime time, Viewport* vp) const
 {
+    OVITO_ASSERT(this_task::isMainThread());
+
     if(vp && isHiddenInViewport(vp, true))
         return Box3();
-    if(!_boundingBoxValidity.contains(time)) {
-        _boundingBoxValidity.setInfinite();
-        _localBoundingBox = localBoundingBox(time, _boundingBoxValidity);
-    }
     TimeInterval iv;
     const AffineTransformation& tm = getWorldTransform(time, iv);
-    Box3 worldBoundingBox = _localBoundingBox.transformed(tm);
+    Box3 worldBoundingBox = localBoundingBox(time).transformed(tm);
     for(SceneNode* child : children()) {
         worldBoundingBox.addBox(child->worldBoundingBox(time, vp));
     }
@@ -469,15 +579,28 @@ Box3 SceneNode::worldBoundingBox(AnimationTime time, Viewport* vp) const
 void SceneNode::setPerViewportVisibility(Viewport* vp, bool visible)
 {
     OVITO_ASSERT(vp);
+    OVITO_ASSERT(this_task::isMainThread());
 
     if(visible) {
-        int index = _hiddenInViewports.indexOf(vp);
-        if(index >= 0)
-            _hiddenInViewports.remove(this, PROPERTY_FIELD(hiddenInViewports), index);
+        for(size_t i = 0; i < hiddenInViewports().size(); i++) {
+            if(hiddenInViewports()[i] == vp) {
+                auto newHiddenInViewports = hiddenInViewports();
+                newHiddenInViewports.erase(newHiddenInViewports.begin() + i);
+                // Remove expired weak references from the list.
+                std::erase_if(newHiddenInViewports, std::mem_fn(&OOWeakRef<Viewport>::expired));
+                setHiddenInViewports(std::move(newHiddenInViewports));
+                break;
+            }
+        }
     }
     else {
-        if(!_hiddenInViewports.contains(vp))
-            _hiddenInViewports.push_back(this, PROPERTY_FIELD(hiddenInViewports), vp);
+        if(boost::find(hiddenInViewports(), vp) == hiddenInViewports().end()) {
+            auto newHiddenInViewports = hiddenInViewports();
+            newHiddenInViewports.push_back(vp);
+            // Remove expired weak references from the list.
+            std::erase_if(newHiddenInViewports, std::mem_fn(&OOWeakRef<Viewport>::expired));
+            setHiddenInViewports(std::move(newHiddenInViewports));
+        }
     }
 }
 
@@ -485,10 +608,12 @@ void SceneNode::setPerViewportVisibility(Viewport* vp, bool visible)
 * Returns whether this scene node (or one of its parents in the node hierarchy) has been hidden
 * specifically in the given viewport.
 ******************************************************************************/
-bool SceneNode::isHiddenInViewport(Viewport* vp, bool includeHierarchyParent) const
+bool SceneNode::isHiddenInViewport(const Viewport* vp, bool includeHierarchyParent) const
 {
     OVITO_ASSERT(vp);
-    if(_hiddenInViewports.contains(vp))
+    OVITO_ASSERT(this_task::isMainThread());
+
+    if(boost::find(hiddenInViewports(), vp) != hiddenInViewports().end())
         return true;
     if(includeHierarchyParent && parentNode())
         return parentNode()->isHiddenInViewport(vp, true);
