@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -26,7 +26,6 @@
 #include <ovito/core/Core.h>
 #include <ovito/core/oo/OORef.h>
 #include <ovito/core/oo/OvitoClass.h>
-#include <ovito/core/oo/ObjectExecutor.h>
 
 namespace Ovito {
 
@@ -40,62 +39,66 @@ namespace Ovito {
 
 /**
  * \brief Universal base class for most objects in OVITO.
- *
- * The OvitoObject class offers a reference counting mechanism to manage the
- * the lifetime of object instances. User code should make use of the OORef
- * smart-pointer class, which automatically increments and decrements the reference counter
- * of an OvitoObject when it holds a pointer to it.
- *
- * When the reference counter of an OvitoObject reaches zero, the virtual aboutToBeDeleted()
- * function is called to notify the object that is about to be deleted from memory.
- * After this function returns, the object instance destroys itself.
- *
- * The OvitoObject class provides serialization and deserialization functions, which allow
- * the object to be saved to disk and restored at a later time. Subclasses that want
- * to support serialization of their data should override the virtual functions
- * saveToStream() and loadFromStream().
  */
-class OVITO_CORE_EXPORT OvitoObject : public QObject
+class OVITO_CORE_EXPORT OvitoObject : public std::enable_shared_from_this<OvitoObject>
 {
-    Q_OBJECT
-
-    Q_PROPERTY(QString className READ className CONSTANT)
-    Q_PROPERTY(QString pluginId READ pluginId CONSTANT)
+    Q_GADGET
 
 private:
 
-    // The class descriptor instance.
+    /// The meta-class descriptor for the OvitoObject C++ class.
     static const OvitoClass __OOClass_instance;
+    inline static OvitoClass::MetadataItem* __OOClass_metadata_head = nullptr;
 
 public:
 
+    /// Flags which may be associated with an OvitoObject.
+    enum ObjectFlag
+    {
+        NoFlags = 0,                  //< No flags set.
+        BeingConstructed = (1 << 0),  //< Indicates that this object's constructor is executing.
+        BeingInitialized = (1 << 1),  //< Indicates that this object is being initialized (initializeObject() hasn't finished yet).
+        BeingDeleted = (1 << 2),      //< Indicates that this object is in the process of being deleted.
+        BeingLoaded = (1 << 3),       //< Indicates that this object is in the process of being restored from an ObjectLoadStream.
+        BeingCopied = (1 << 4),       //< Indicates that this object is in the process of being constructed by copying another object.
+    };
+    Q_DECLARE_FLAGS(ObjectFlags, ObjectFlag);
+
     using ovito_class = OvitoObject;
     using OOMetaClass = OvitoClass;
 
-    /// Returns the type descriptor that every OvitoObject-derived class has.
+    /// Returns the class' meta-class descriptor.
     static const OvitoClass& OOClass() { return __OOClass_instance; }
 
-    /// \brief Default constructor, which initializes the reference count to zero.
-    OvitoObject() = default;
+    /// Mimic Qt's string localization function tr() for string literals.
+    static inline QString tr(const char* sourceText) { return QString::fromUtf8(sourceText); }
 
 #ifdef OVITO_DEBUG
-    /// \brief Destructor.
-    virtual ~OvitoObject();
+    /// Note: No need to make the base class destructor virtual, because we use std::shared_ptr to manage an object's lifetime.
+    /// std::shared_ptr will call the right destructor of a derived class automatically.
+    ~OvitoObject();
 #endif
 
-    /// Returns true if this object is currently being loaded from an ObjectLoadStream.
-    bool isBeingLoaded() const;
+    /// Indicates whether this object is currently being constructed, i.e., the constructor is still executing.
+    inline bool isBeingConstructed() const { return _flags.testFlag(BeingConstructed); }
 
-    /// Returns true if this object is about to be deleted, i.e. if the reference count has reached zero
+    /// Indicates whether this object is currently being constructed and initialized,
+    /// which means the object is not yet in a fully initialized state (initializeObject() has not finished yet).
+    inline bool isBeingInitialized() const { return _flags.testFlag(BeingInitialized); }
+
+    /// Indicates whether this object is currently being loaded from an ObjectLoadStream,
+    /// which means it is not yet in a fully initialized state.
+    inline bool isBeingLoaded() const { return _flags.testFlag(BeingLoaded); }
+
+    /// Returns true if this object is about to be deleted, i.e., if the reference count has reached zero
     /// and aboutToBeDeleted() is being invoked.
-    bool isAboutToBeDeleted() const {
-        return objectReferenceCount() >= INVALID_REFERENCE_COUNT;
-    }
+    inline bool isBeingDeleted() const { return _flags.testFlag(BeingDeleted); }
 
-    /// \brief Returns the current value of the object's reference counter.
-    /// \return The reference count for this object, i.e. the number of references
-    ///         pointing to this object.
-    const std::atomic<int>& objectReferenceCount() const noexcept { return _referenceCount; }
+    /// Indicates whether this object is currently being constructed by copying parameter values from another object.
+    inline bool isBeingCopied() const { return _flags.testFlag(BeingCopied); }
+
+    /// Indicates whether this object is currently being initialized or destroyed.
+    inline bool isBeingInitializedOrDeleted() const { return _flags.testAnyFlags(ObjectFlags(BeingInitialized | BeingDeleted)); }
 
 #ifdef OVITO_DEBUG
     /// \brief Returns whether this object has not been deleted yet.
@@ -111,42 +114,30 @@ public:
     /// Returns the class descriptor for this object.
     const OvitoClass& getOOMetaClass() const { return OOClass(); }
 
-    /// This method implements the Executor interface.
-    /// It creates some work that can be submitted for execution later and which will be executed in the context of this object.
-    /// If the object gets deleted in the meantime, the work will be discarded.
-    template<typename Function>
-    auto schedule(Function&& f) const {
-        OVITO_ASSERT(this);
-        OVITO_ASSERT(ExecutionContext::current().isValid());
-        return [obj = QPointer<const QObject>(this), context = ExecutionContext::current(), f = std::forward<Function>(f)]() mutable noexcept {
-            if(const OvitoObject* self = static_cast<const OvitoObject*>(obj.data())) {
-                ExecutionContext::Scope execScope(std::move(context));
-                self->execute(std::move(f));
-            }
-        };
-    }
-
-    /// This method implements the Executor interface.
-    /// It executes work in the context of this object.
-    template<typename Function>
-    void execute(Function&& f) const {
-        OVITO_ASSERT(this);
-        OVITO_ASSERT(ExecutionContext::current().isValid());
-        if(QThread::currentThread() != this->thread()) {
-            // When not in the main thread, or if deferred execution was requested, schedule work for later execution in the main thread.
-            auto event = new detail::ObjectExecutorWorkEvent<Function>(ObjectExecutor::workEventType(), this, ExecutionContext::current(), std::forward<Function>(f));
-            QCoreApplication::postEvent(const_cast<QObject*>(event->object()), event);
-        }
-        else {
-            // Temporarily suspend undo recording, because deferred operations never get recorded by convention.
-            UndoSuspender noUndo;
-            // Execute the work function.
-            std::invoke(std::forward<Function>(f));
-        }
-    }
-
 protected:
 
+    /// This method gets called by OORef<T>::create() right after the object has been constructed by std::make_shared<>.
+    /// Subclasses can override this method to perform initialization work that may raise an exception or which needs to create
+    /// OORef references to the object itself (which isn't possible in the class constructor).
+    inline void initializeObject() {
+        OVITO_ASSERT(isBeingConstructed());
+        _flags.setFlag(BeingConstructed, false);
+    }
+
+    /// Clears the BeingInitialized flag of the object. This method gets called by OORef<T>::create() after the object has been fully initialized.
+    inline void completeObjectInitialization() {
+        OVITO_ASSERT(!isBeingConstructed());
+        OVITO_ASSERT(isBeingInitialized());
+        _flags.setFlag(BeingInitialized, false);
+    }
+
+    /// Sets the BeingCopied flag of the object indicating that the object's parameters are in the
+    /// process of being copied from another object. Set by RefTarget::clone() when cloning begins and cleared
+    /// by CloneHelper when cloning is complete.
+    void setIsBeingCopied(bool isBeingCopied) {
+        _flags.setFlag(BeingCopied, isBeingCopied);
+    }
+
     /// \brief Saves the internal data of this object to an output stream.
     /// \param stream The destination data stream.
     /// \param excludeRecomputableData Controls whether the object should not store data that can be recomputed at runtime.
@@ -194,68 +185,42 @@ protected:
 
 private:
 
-    /// The current number of references to this object that keep it alive.
-    mutable std::atomic<int> _referenceCount{0};
-
-    /// This is the special value the reference count of the object is set to while it is being deleted:
-    constexpr static auto INVALID_REFERENCE_COUNT = std::numeric_limits<int>::max() / 2;
-
-    /// \brief Increments the reference count by one.
-    void incrementReferenceCount() const noexcept {
-        OVITO_CHECK_OBJECT_POINTER(this);
-        OVITO_ASSERT_MSG(_isAllocatedOnTheHeap, "OvitoObject::incrementReferenceCount()", "Cannot use OORef<> to hold an object that has not been created with OORef<>::create().");
-        OVITO_ASSERT_MSG(!_isBeingDestructed, "OvitoObject::incrementReferenceCount()", "Cannot create new OORef<> for an object that is currently being destructed.");
-        _referenceCount.fetch_add(1);
-    }
-
-    /// \brief Decrements the reference count by one.
-    ///
-    /// When the reference count becomes zero, then the object deletes itself automatically.
-    void decrementReferenceCount() const noexcept {
-        OVITO_CHECK_OBJECT_POINTER(this);
-        OVITO_ASSERT_MSG(_isAllocatedOnTheHeap, "OvitoObject::decrementReferenceCount()", "Cannot use OORef<> to hold an object that was allocated on the stack.");
-        OVITO_ASSERT(_referenceCount.load() >= 1);
-        if(_referenceCount.fetch_sub(1) == 1) {
-            const_cast<OvitoObject*>(this)->deleteObjectInternal();
-        }
-    }
-
     /// Internal method that calls this object's aboutToBeDeleted() routine and the deletes the object.
     /// It is automatically called when the object's reference counter reaches zero.
-    Q_INVOKABLE void deleteObjectInternal() noexcept;
+    void deleteObjectInternal() noexcept;
 
     /// Returns the name of the plugin class this object is an instance of.
     /// This method is an implementation detail required by the Q_PROPERTY macro above.
     const QString& className() const { return getOOClass().name(); }
 
-    /// Returns the idenitifier of the plugin module this object belongs to.
+    /// Returns the identifier of the plugin module this object belongs to.
     /// This method is an implementation detail required by the Q_PROPERTY macro above.
     QString pluginId() const { return QString::fromLatin1(getOOClass().pluginId()); }
 
+    /// Bit-wise flags.
+    ObjectFlags _flags = ObjectFlags(BeingConstructed | BeingInitialized);
+
 #ifdef OVITO_DEBUG
     /// This field is initialized with a special value by the class constructor to indicate that
     /// the object is still alive and has not been deleted. When the object is deleted, the
     /// destructor sets the field to a different value to indicate that the object is no longer alive.
     quint32 _magicAliveCode = 0x87ABCDEF;
 
-    /// Indicates that this object lives on the heap, because it has been created by the OORef<>::create() method.
-    /// Otherwise it was allocated on the stack, which means the object's reference counting mechanism may not be used.
-    bool _isAllocatedOnTheHeap = false;
-
-    /// Indicates that this object's destructor is being run.
-    bool _isBeingDestructed = false;
-
     friend class OvitoClass;
 #endif
 
     // Give OORef smart-pointer class access to the internal reference counter.
     template<class T> friend class OORef;
+    template<typename T> friend struct OOAllocator;
 
     // These classes need to access the protected serialization functions.
     friend class ObjectSaveStream;
     friend class ObjectLoadStream;
 };
 
+/// Prints an object to Qt debug stream.
+OVITO_CORE_EXPORT QDebug operator<<(QDebug dbg, const OvitoObject* o);
+
 /// \brief Dynamic cast operator for subclasses of OvitoObject.
 ///
 /// Returns a pointer to the input object, cast to type \c T if the object is of type \c T
@@ -264,7 +229,7 @@ private:
 /// \relates OvitoObject
 template<class T, class U>
 inline T* dynamic_object_cast(U* obj) noexcept {
-    return qobject_cast<T*>(obj);
+    return dynamic_cast<T*>(obj);
 }
 
 /// \brief Dynamic cast operator for subclasses of OvitoObject derived.
@@ -275,7 +240,7 @@ inline T* dynamic_object_cast(U* obj) noexcept {
 /// \relates OvitoObject
 template<class T, class U>
 inline const T* dynamic_object_cast(const U* obj) noexcept {
-    return qobject_cast<const T*>(obj);
+    return dynamic_cast<const T*>(obj);
 }
 
 /// \brief Static cast operator for OvitoObject derived classes.
@@ -323,6 +288,17 @@ inline Pointer<T> dynamic_object_cast(const Pointer<U>& obj) noexcept {
     return dynamic_pointer_cast<T, U>(obj);
 }
 
+/// \brief Dynamic cast operator for fancy pointers to OVITO objects.
+///
+/// Returns a fancy pointer to the input object, cast to type \c T if the object is of type \c T
+/// (or a subclass); otherwise returns \c nullptr.
+///
+/// \relates OORef, DataOORef
+template<class T, class U, template<typename> class Pointer>
+inline Pointer<const T> dynamic_object_cast(const Pointer<const U>& obj) noexcept {
+    return dynamic_pointer_cast<const T, const U>(obj);
+}
+
 /// \brief Dynamic cast operator for fancy pointers to OVITO objects.
 ///
 /// Returns a fancy pointer to the input object, cast to type \c T if the object is of type \c T
@@ -334,6 +310,17 @@ inline Pointer<T> dynamic_object_cast(Pointer<U>&& obj) noexcept {
     return dynamic_pointer_cast<T, U>(std::move(obj));
 }
 
+/// \brief Dynamic cast operator for fancy pointers to OVITO objects.
+///
+/// Returns a fancy pointer to the input object, cast to type \c T if the object is of type \c T
+/// (or a subclass); otherwise returns \c nullptr.
+///
+/// \relates OORef, DataOORef
+template<class T, class U, template<typename> class Pointer>
+inline Pointer<const T> dynamic_object_cast(Pointer<const U>&& obj) noexcept {
+    return dynamic_pointer_cast<const T, const U>(std::move(obj));
+}
+
 /// \brief Static cast operator for fancy pointers to OVITO objects.
 ///
 /// Returns the given object cast to type \c T.
@@ -347,6 +334,19 @@ inline Pointer<T> static_object_cast(const Pointer<U>& obj) noexcept {
     return static_pointer_cast<T, U>(obj);
 }
 
+/// \brief Static cast operator for fancy pointers to OVITO objects.
+///
+/// Returns the given object cast to type \c T.
+/// Performs a runtime check of the object type in debug build.
+///
+/// \relates OORef, DataOORef
+template<class T, class U, template<typename> class Pointer>
+inline Pointer<const T> static_object_cast(const Pointer<const U>& obj) noexcept {
+    OVITO_ASSERT_MSG(!obj || obj->getOOClass().isDerivedFrom(T::OOClass()), "static_object_cast",
+        qPrintable(QStringLiteral("Runtime type check failed. The source object %1 is not an instance of the target class %2.").arg(obj->getOOClass().name()).arg(T::OOClass().name())));
+    return static_pointer_cast<const T, const U>(obj);
+}
+
 /// \brief Static cast operator for fancy pointers to OVITO objects.
 ///
 /// Returns the given object cast to type \c T.
@@ -360,10 +360,24 @@ inline Pointer<T> static_object_cast(Pointer<U>&& obj) noexcept {
     return static_pointer_cast<T, U>(std::move(obj));
 }
 
+/// \brief Static cast operator for fancy pointers to OVITO objects.
+///
+/// Returns the given object cast to type \c T.
+/// Performs a runtime check of the object type in debug build.
+///
+/// \relates OORef, DataOORef
+template<class T, class U, template<typename> class Pointer>
+inline Pointer<const T> static_object_cast(Pointer<const U>&& obj) noexcept {
+    OVITO_ASSERT_MSG(!obj || obj->getOOClass().isDerivedFrom(T::OOClass()), "static_object_cast",
+        qPrintable(QStringLiteral("Runtime type check failed. The source object %1 is not an instance of the target class %2.").arg(obj->getOOClass().name()).arg(T::OOClass().name())));
+    return static_pointer_cast<const T, const U>(std::move(obj));
+}
+
 }   // End of namespace
 
 Q_DECLARE_SMART_POINTER_METATYPE(Ovito::OORef);
 
 #include <ovito/core/utilities/io/ObjectSaveStream.h>
 #include <ovito/core/utilities/io/ObjectLoadStream.h>
-#include <ovito/core/oo/ObjectExecutor.h>
+#include <ovito/core/utilities/concurrent/ObjectExecutor.h>
+#include <ovito/core/utilities/concurrent/DeferredObjectExecutor.h>
