@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -32,11 +32,22 @@
 
 namespace Ovito {
 
+#ifdef OVITO_DEBUG
+/******************************************************************************
+* Verifies that owner is of the right type, i.e., defines the given property field.
+******************************************************************************/
+bool PropertyFieldBase::ownerTypeCheck(RefMaker* owner, const PropertyFieldDescriptor* descriptor)
+{
+    OVITO_CHECK_OBJECT_POINTER(owner);
+    return descriptor->definingClass()->isMember(owner);
+}
+#endif
+
 /******************************************************************************
 * Generates a notification event to inform the dependents of the field's owner
 * that it has changed.
 ******************************************************************************/
-void PropertyFieldBase::generateTargetChangedEvent(RefMaker* owner, const PropertyFieldDescriptor* descriptor, ReferenceEvent::Type eventType)
+void PropertyFieldBase::generateTargetChangedEvent(RefMaker* owner, const PropertyFieldDescriptor* descriptor, int eventType)
 {
     // Make sure we are not trying to generate a change message for objects that are not RefTargets.
     OVITO_ASSERT_MSG(!descriptor->shouldGenerateChangeEvent() || descriptor->definingClass()->isDerivedFrom(RefTarget::OOClass()),
@@ -44,22 +55,28 @@ void PropertyFieldBase::generateTargetChangedEvent(RefMaker* owner, const Proper
             qPrintable(QString("Flag PROPERTY_FIELD_NO_CHANGE_MESSAGE has not been set for property field '%1' of class '%2' even though '%2' is not derived from RefTarget.")
                     .arg(descriptor->identifier()).arg(descriptor->definingClass()->name())));
 
+    // Suppress all change messages while the owner object is being initialized or destroyed.
+    if(owner->isBeingInitializedOrDeleted())
+        return;
+
     if(descriptor->definingClass()->isDerivedFrom(DataObject::OOClass())) {
-        // Change events are only sent by a DataObject if the object
+        // Change events only need to be sent by a DataObject if the object
         // is not shared by multiple owners and if we are in the main thread.
-        if(QThread::currentThread() != owner->thread())
+        // This is a performance optimization to avoid sending change events unnecessarily
+        // in situations where they certainly don't matter.
+        if(this_task::isMainThread() == false)
             return;
-        if(!static_object_cast<DataObject>(owner)->isSafeToModify())
+        if(static_object_cast<DataObject>(owner)->dataReferenceCount() > 1) // Note: Using dataReferenceCount() instead of isSafeToModify() here is a performance optimization.
             return;
     }
 
     // Send notification message to dependents of owner object.
     if(eventType != ReferenceEvent::TargetChanged) {
-        OVITO_ASSERT(owner->isRefTarget());
+        OVITO_ASSERT(RefTarget::OOClass().isMember(owner));
         static_object_cast<RefTarget>(owner)->notifyDependents(eventType);
     }
-    else if(descriptor->shouldGenerateChangeEvent() && !owner->isAboutToBeDeleted()) {
-        OVITO_ASSERT(owner->isRefTarget());
+    else if(descriptor->shouldGenerateChangeEvent() && !owner->isBeingDeleted()) {
+        OVITO_ASSERT(RefTarget::OOClass().isMember(owner));
         static_object_cast<RefTarget>(owner)->notifyTargetChanged(descriptor);
     }
 }
@@ -95,9 +112,6 @@ RefMaker* PropertyFieldBase::PropertyFieldOperation::owner() const
 ******************************************************************************/
 template<typename T> SingleReferenceFieldBase<T>::~SingleReferenceFieldBase()
 {
-    if(_target)
-        qDebug() << "Reference field value:" << get();
-    OVITO_ASSERT_MSG(!_target, "~ReferenceField()", "Owner object of reference field has not been deleted correctly. The reference field was not empty when the class destructor was called.");
 }
 #endif
 
@@ -106,18 +120,20 @@ template<typename T> SingleReferenceFieldBase<T>::~SingleReferenceFieldBase()
 ******************************************************************************/
 template<typename T> void SingleReferenceFieldBase<T>::set(RefMaker* owner, const PropertyFieldDescriptor* descriptor, pointer newTarget)
 {
+    OVITO_ASSERT(ownerTypeCheck(owner, descriptor));
+
     if(_target == newTarget)
         return; // Nothing to change.
 
     // Check object type
     if(newTarget && !newTarget->getOOClass().isDerivedFrom(*descriptor->targetClass())) {
         OVITO_ASSERT_MSG(false, "SingleReferenceFieldBase::set()", "Tried to create a reference to an incompatible object for this reference field.");
-        throw Exception(QString("Cannot set a reference field of type %1 to an incompatible object of type %2.").arg(descriptor->targetClass()->name(), newTarget->getOOClass().name()));
+        throw Exception(QStringLiteral("Cannot set a reference field of type %1 to an incompatible object of type %2.").arg(descriptor->targetClass()->name(), newTarget->getOOClass().name()));
     }
 
     // Make sure automatic undo is disabled for a reference field of a class that is not derived from RefTarget.
-    OVITO_ASSERT_MSG(descriptor->automaticUndo() == false || owner->isRefTarget(), "SingleReferenceFieldBase::set()",
-            qPrintable(QString("PROPERTY_FIELD_NO_UNDO flag has not been set for reference field '%1' of non-RefTarget derived class '%2'.")
+    OVITO_ASSERT_MSG(descriptor->automaticUndo() == false || RefTarget::OOClass().isMember(owner), "SingleReferenceFieldBase::set()",
+            qPrintable(QStringLiteral("PROPERTY_FIELD_NO_UNDO flag has not been set for reference field '%1' of non-RefTarget derived class '%2'.")
                 .arg(descriptor->identifier()).arg(descriptor->definingClass()->name())));
 
     class SetReferenceOperation final : public PropertyFieldOperation
@@ -148,7 +164,7 @@ template<typename T> void SingleReferenceFieldBase<T>::set(RefMaker* owner, cons
         }
     };
 
-    if(descriptor->automaticUndo() && CompoundOperation::isUndoRecording()) {
+    if(descriptor->automaticUndo() && !owner->isBeingInitializedOrDeleted() && CompoundOperation::isUndoRecording()) {
         auto op = std::make_unique<SetReferenceOperation>(owner, std::move(newTarget), *this, descriptor);
         op->redo();
         CompoundOperation::current()->addOperation(std::move(op));
@@ -163,37 +179,35 @@ template<typename T> void SingleReferenceFieldBase<T>::set(RefMaker* owner, cons
 ******************************************************************************/
 template<typename T> void SingleReferenceFieldBase<T>::swapReference(RefMaker* owner, const PropertyFieldDescriptor* descriptor, pointer& inactiveTarget)
 {
-    OVITO_CHECK_POINTER(this);
-    OVITO_CHECK_OBJECT_POINTER(owner);
+    OVITO_ASSERT(ownerTypeCheck(owner, descriptor));
     OVITO_ASSERT(!descriptor->isVector());
-    OVITO_ASSERT((descriptor->isWeakReference() == std::is_same<pointer, RefTarget*>::value));
+    OVITO_ASSERT(inactiveTarget != _target);
 
-    // Check for cyclic strong references.
-    if(inactiveTarget && (!descriptor->flags().testFlag(PROPERTY_FIELD_DONT_PROPAGATE_MESSAGES) || !descriptor->isWeakReference()) && owner->isReferencedBy(inactiveTarget, true))
+    // Check for cyclic references.
+    if(inactiveTarget && owner->isReferencedBy(inactiveTarget, true))
         throw CyclicReferenceError();
 
     // Move the old pointer value into a local temporary.
     pointer oldTarget = std::exchange(_target, nullptr);
     OVITO_ASSERT(!_target);
 
-    // Disconnect the Qt signal/slot connection, but only if the dependent has no other references to the old target.
+    // Disconnect from the old target, but only if the dependent has no other references to the old target.
     if(oldTarget && !owner->hasReferenceTo(oldTarget)) {
-        bool success = QObject::disconnect(to_address(oldTarget), &RefTarget::objectEvent, owner, &RefMaker::receiveObjectEvent);
-        OVITO_ASSERT(success);
+        oldTarget->unregisterDependent(owner);
     }
 
     // Exchange pointer values.
     _target = std::move(inactiveTarget);
     inactiveTarget = std::move(oldTarget);
 
-    // Create a Qt signal/slot connection to the newly referenced object.
+    // Register the dependent with the newly referenced object (only if it isn't already registered).
     if(_target)
-        QObject::connect(to_address(_target), &RefTarget::objectEvent, owner, &RefMaker::receiveObjectEvent, static_cast<Qt::ConnectionType>(Qt::DirectConnection | Qt::UniqueConnection));
+        _target->registerDependent(owner);
 
     // Inform owner object about the changed reference value.
     owner->referenceReplaced(descriptor,
-        const_cast<RefTarget*>(static_cast<const RefTarget*>(to_address(inactiveTarget))),
-        const_cast<RefTarget*>(static_cast<const RefTarget*>(to_address(_target))),
+        const_cast<RefTarget*>(static_cast<const RefTarget*>(inactiveTarget.get())),
+        const_cast<RefTarget*>(static_cast<const RefTarget*>(_target.get())),
         -1);
 
     // Emit object-changed signal.
@@ -201,12 +215,11 @@ template<typename T> void SingleReferenceFieldBase<T>::swapReference(RefMaker* o
 
     // Emit additional signal if SET_PROPERTY_FIELD_CHANGE_EVENT macro was used for this property field.
     if(descriptor->extraChangeEventType() != 0)
-        generateTargetChangedEvent(owner, descriptor, static_cast<ReferenceEvent::Type>(descriptor->extraChangeEventType()));
+        generateTargetChangedEvent(owner, descriptor, descriptor->extraChangeEventType());
 }
 
 // Instantiate base class template for the fancy pointer base types needed.
 #if defined(Q_CC_MSVC) || defined(Q_CC_CLANG) || defined(OVITO_BUILD_MONOLITHIC)
-    template class OVITO_CORE_EXPORT SingleReferenceFieldBase<RefTarget*>;
     template class OVITO_CORE_EXPORT SingleReferenceFieldBase<OORef<RefTarget>>;
     template class OVITO_CORE_EXPORT SingleReferenceFieldBase<DataOORef<const DataObject>>;
 #endif
@@ -217,7 +230,6 @@ template<typename T> void SingleReferenceFieldBase<T>::swapReference(RefMaker* o
 ******************************************************************************/
 template<typename T> VectorReferenceFieldBase<T>::~VectorReferenceFieldBase()
 {
-    OVITO_ASSERT_MSG(_targets.empty(), "~VectorReferenceField()", "Owner object of vector reference field has not been deleted correctly. The vector reference field was not empty when the class destructor was called.");
 }
 #endif
 
@@ -226,6 +238,8 @@ template<typename T> VectorReferenceFieldBase<T>::~VectorReferenceFieldBase()
 ******************************************************************************/
 template<typename T> void VectorReferenceFieldBase<T>::set(RefMaker* owner, const PropertyFieldDescriptor* descriptor, size_type i, pointer newTarget)
 {
+    OVITO_ASSERT(ownerTypeCheck(owner, descriptor));
+
     OVITO_ASSERT(i >= 0 && i < size());
     if(_targets[i] == newTarget)
         return; // Nothing to change.
@@ -237,7 +251,7 @@ template<typename T> void VectorReferenceFieldBase<T>::set(RefMaker* owner, cons
     }
 
     // Make sure automatic undo is disabled for a reference field of a class that is not derived from RefTarget.
-    OVITO_ASSERT_MSG(descriptor->automaticUndo() == false || owner->isRefTarget(), "VectorReferenceFieldBase::set()",
+    OVITO_ASSERT_MSG(descriptor->automaticUndo() == false || RefTarget::OOClass().isMember(owner), "VectorReferenceFieldBase::set()",
             qPrintable(QString("PROPERTY_FIELD_NO_UNDO flag has not been set for reference field '%1' of non-RefTarget derived class '%2'.")
                 .arg(descriptor->identifier()).arg(descriptor->definingClass()->name())));
 
@@ -273,7 +287,7 @@ template<typename T> void VectorReferenceFieldBase<T>::set(RefMaker* owner, cons
         }
     };
 
-    if(descriptor->automaticUndo() && CompoundOperation::isUndoRecording()) {
+    if(descriptor->automaticUndo() && !owner->isBeingInitializedOrDeleted() && CompoundOperation::isUndoRecording()) {
         auto op = std::make_unique<SetReferenceOperation>(owner, std::move(newTarget), i, *this, descriptor);
         op->redo();
         CompoundOperation::current()->addOperation(std::move(op));
@@ -288,6 +302,8 @@ template<typename T> void VectorReferenceFieldBase<T>::set(RefMaker* owner, cons
 ******************************************************************************/
 template<typename T> auto VectorReferenceFieldBase<T>::insert(RefMaker* owner, const PropertyFieldDescriptor* descriptor, size_type i, pointer newTarget) -> size_type
 {
+    OVITO_ASSERT(ownerTypeCheck(owner, descriptor));
+
     // Check object type
     if(newTarget && !newTarget->getOOClass().isDerivedFrom(*descriptor->targetClass())) {
         OVITO_ASSERT_MSG(false, "VectorReferenceFieldBase::insert()", "Cannot add incompatible object to this vector reference field.");
@@ -295,7 +311,7 @@ template<typename T> auto VectorReferenceFieldBase<T>::insert(RefMaker* owner, c
     }
 
     // Make sure automatic undo is disabled for a reference field of a class that is not derived from RefTarget.
-    OVITO_ASSERT_MSG(descriptor->automaticUndo() == false || owner->isRefTarget(), "VectorReferenceFieldBase::insert()",
+    OVITO_ASSERT_MSG(descriptor->automaticUndo() == false || RefTarget::OOClass().isMember(owner), "VectorReferenceFieldBase::insert()",
             qPrintable(QString("PROPERTY_FIELD_NO_UNDO flag has not been set for reference field '%1' of non-RefTarget derived class '%2'.")
                     .arg(descriptor->identifier()).arg(descriptor->definingClass()->name())));
 
@@ -337,7 +353,7 @@ template<typename T> auto VectorReferenceFieldBase<T>::insert(RefMaker* owner, c
         }
     };
 
-    if(descriptor->automaticUndo() && CompoundOperation::isUndoRecording()) {
+    if(descriptor->automaticUndo() && !owner->isBeingInitializedOrDeleted() && CompoundOperation::isUndoRecording()) {
         auto op = std::make_unique<InsertReferenceOperation>(owner, std::move(newTarget), i, *this, descriptor);
         op->redo();
         int index = op->insertionIndex();
@@ -355,10 +371,11 @@ template<typename T> auto VectorReferenceFieldBase<T>::insert(RefMaker* owner, c
 ******************************************************************************/
 template<typename T> T VectorReferenceFieldBase<T>::remove(RefMaker* owner, const PropertyFieldDescriptor* descriptor, size_type i)
 {
+    OVITO_ASSERT(ownerTypeCheck(owner, descriptor));
     OVITO_ASSERT(i >= 0 && i < size());
 
     // Make sure automatic undo is disabled for a reference field of a class that is not derived from RefTarget.
-    OVITO_ASSERT_MSG(descriptor->automaticUndo() == false || owner->isRefTarget(), "VectorReferenceFieldBase::remove()",
+    OVITO_ASSERT_MSG(descriptor->automaticUndo() == false || RefTarget::OOClass().isMember(owner), "VectorReferenceFieldBase::remove()",
             qPrintable(QString("PROPERTY_FIELD_NO_UNDO flag has not been set for reference field '%1' of non-RefTarget derived class '%2'.")
                     .arg(descriptor->identifier()).arg(descriptor->definingClass()->name())));
 
@@ -400,7 +417,7 @@ template<typename T> T VectorReferenceFieldBase<T>::remove(RefMaker* owner, cons
         }
     };
 
-    if(descriptor->automaticUndo() && CompoundOperation::isUndoRecording()) {
+    if(descriptor->automaticUndo() && !owner->isBeingInitializedOrDeleted() && CompoundOperation::isUndoRecording()) {
         auto op = std::make_unique<RemoveReferenceOperation>(owner, i, *this, descriptor);
         op->redo();
         pointer removedReference = op->storedTarget();
@@ -431,34 +448,31 @@ template<typename T> void VectorReferenceFieldBase<T>::swapReference(RefMaker* o
     OVITO_CHECK_POINTER(this);
     OVITO_CHECK_OBJECT_POINTER(owner);
     OVITO_ASSERT(descriptor->isVector());
-    OVITO_ASSERT((descriptor->isWeakReference() == std::is_same<pointer, RefTarget*>::value));
 
     // Check for cyclic strong references.
-    if(inactiveTarget && (!descriptor->flags().testFlag(PROPERTY_FIELD_DONT_PROPAGATE_MESSAGES) || !descriptor->isWeakReference()) && owner->isReferencedBy(inactiveTarget, true))
+    if(inactiveTarget && owner->isReferencedBy(inactiveTarget, true))
         throw CyclicReferenceError();
 
     // Move the old pointer value into a local temporary.
     pointer oldTarget = std::exchange(_targets[index], nullptr);
     OVITO_ASSERT(!_targets[index]);
 
-    // Disconnect the Qt signal/slot connection, but only if the dependent has no other references to the old target.
-    if(oldTarget && !owner->hasReferenceTo(oldTarget)) {
-        bool success = QObject::disconnect(to_address(oldTarget), &RefTarget::objectEvent, owner, &RefMaker::receiveObjectEvent);
-        OVITO_ASSERT(success);
-    }
+    // Disconnect from the old target, but only if the dependent has no other references to the old target.
+    if(oldTarget && !owner->hasReferenceTo(oldTarget))
+        oldTarget->unregisterDependent(owner);
 
     // Exchange pointer values.
     _targets[index] = std::move(inactiveTarget);
     inactiveTarget = std::move(oldTarget);
 
-    // Create a Qt signal/slot connection to the newly referenced object.
+    // Register the dependent with the newly referenced object (only if it isn't already registered).
     if(_targets[index])
-        QObject::connect(to_address(_targets[index]), &RefTarget::objectEvent, owner, &RefMaker::receiveObjectEvent, static_cast<Qt::ConnectionType>(Qt::DirectConnection | Qt::UniqueConnection));
+        _targets[index]->registerDependent(owner);
 
     // Inform owner object about the changed reference value.
     owner->referenceReplaced(descriptor,
-        const_cast<RefTarget*>(static_cast<const RefTarget*>(to_address(inactiveTarget))),
-        const_cast<RefTarget*>(static_cast<const RefTarget*>(to_address(_targets[index]))),
+        const_cast<RefTarget*>(static_cast<const RefTarget*>(inactiveTarget.get())),
+        const_cast<RefTarget*>(static_cast<const RefTarget*>(_targets[index].get())),
         index);
 
     // Emit object-changed signal.
@@ -466,7 +480,7 @@ template<typename T> void VectorReferenceFieldBase<T>::swapReference(RefMaker* o
 
     // Emit additional signal if SET_PROPERTY_FIELD_CHANGE_EVENT macro was used for this property field.
     if(descriptor->extraChangeEventType() != 0)
-        generateTargetChangedEvent(owner, descriptor, static_cast<ReferenceEvent::Type>(descriptor->extraChangeEventType()));
+        generateTargetChangedEvent(owner, descriptor, descriptor->extraChangeEventType());
 }
 
 /******************************************************************************
@@ -481,15 +495,13 @@ template<typename T> void VectorReferenceFieldBase<T>::removeReference(RefMaker*
     inactiveTarget = std::move(_targets[index]);
     _targets.remove(index);
 
-    // Disconnect the Qt signal/slot connection, but only if the dependent has no other references to the old target.
-    if(inactiveTarget && !owner->hasReferenceTo(inactiveTarget)) {
-        bool success = QObject::disconnect(to_address(inactiveTarget), &RefTarget::objectEvent, owner, &RefMaker::receiveObjectEvent);
-        OVITO_ASSERT(success);
-    }
+    // Disconnect from the old target, but only if the dependent has no other references to the old target.
+    if(inactiveTarget && !owner->hasReferenceTo(inactiveTarget))
+        inactiveTarget->unregisterDependent(owner);
 
     // Inform owner object about the removed reference value.
     owner->referenceRemoved(descriptor,
-        const_cast<RefTarget*>(static_cast<const RefTarget*>(to_address(inactiveTarget))),
+        const_cast<RefTarget*>(static_cast<const RefTarget*>(inactiveTarget.get())),
         index);
 
     // Emit object-changed signal.
@@ -497,7 +509,7 @@ template<typename T> void VectorReferenceFieldBase<T>::removeReference(RefMaker*
 
     // Emit additional signal if SET_PROPERTY_FIELD_CHANGE_EVENT macro was used for this property field.
     if(descriptor->extraChangeEventType() != 0)
-        generateTargetChangedEvent(owner, descriptor, static_cast<ReferenceEvent::Type>(descriptor->extraChangeEventType()));
+        generateTargetChangedEvent(owner, descriptor, descriptor->extraChangeEventType());
 }
 
 /******************************************************************************
@@ -505,12 +517,11 @@ template<typename T> void VectorReferenceFieldBase<T>::removeReference(RefMaker*
 ******************************************************************************/
 template<typename T> auto VectorReferenceFieldBase<T>::addReference(RefMaker* owner, const PropertyFieldDescriptor* descriptor, size_type index, pointer& target) -> size_type
 {
-    OVITO_CHECK_POINTER(this);
-    OVITO_CHECK_OBJECT_POINTER(owner);
+    OVITO_ASSERT(ownerTypeCheck(owner, descriptor));
     OVITO_ASSERT(descriptor->isVector());
 
     // Check for cyclic strong references.
-    if(target && (!descriptor->flags().testFlag(PROPERTY_FIELD_DONT_PROPAGATE_MESSAGES) || !descriptor->isWeakReference()) && owner->isReferencedBy(target, true))
+    if(target && owner->isReferencedBy(target, true))
         throw CyclicReferenceError();
 
     // Add new reference to list.
@@ -524,26 +535,25 @@ template<typename T> auto VectorReferenceFieldBase<T>::addReference(RefMaker* ow
     }
     OVITO_ASSERT(!target);
 
-    // Create a Qt signal/slot connection to the newly referenced object.
+    // Register the dependent with the newly referenced object (only if it isn't already registered).
     if(_targets[index])
-        QObject::connect(to_address(_targets[index]), &RefTarget::objectEvent, owner, &RefMaker::receiveObjectEvent, static_cast<Qt::ConnectionType>(Qt::DirectConnection | Qt::UniqueConnection));
+         _targets[index]->registerDependent(owner);
 
     // Inform derived classes.
-    owner->referenceInserted(descriptor, const_cast<RefTarget*>(static_cast<const RefTarget*>(to_address(_targets[index]))), index);
+    owner->referenceInserted(descriptor, const_cast<RefTarget*>(static_cast<const RefTarget*>(_targets[index].get())), index);
 
     // Send auto change message.
     generateTargetChangedEvent(owner, descriptor);
 
     // An additional message can be requested by the user using the SET_PROPERTY_FIELD_CHANGE_EVENT macro.
     if(descriptor->extraChangeEventType() != 0)
-        generateTargetChangedEvent(owner, descriptor, static_cast<ReferenceEvent::Type>(descriptor->extraChangeEventType()));
+        generateTargetChangedEvent(owner, descriptor, descriptor->extraChangeEventType());
 
     return index;
 }
 
 // Instantiate base class template for the fancy pointer base types needed.
 #if defined(Q_CC_MSVC) || defined(Q_CC_CLANG) || defined(OVITO_BUILD_MONOLITHIC)
-    template class OVITO_CORE_EXPORT VectorReferenceFieldBase<RefTarget*>;
     template class OVITO_CORE_EXPORT VectorReferenceFieldBase<OORef<RefTarget>>;
     template class OVITO_CORE_EXPORT VectorReferenceFieldBase<DataOORef<const DataObject>>;
 #endif
