@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -28,6 +28,7 @@
 #include <ovito/core/app/undo/UndoableOperation.h>
 #include <ovito/core/oo/PropertyFieldDescriptor.h>
 #include <ovito/core/oo/ReferenceEvent.h>
+#include <ovito/core/oo/RefMaker.h>
 
 #include <boost/type_traits/has_equal_to.hpp>
 
@@ -41,11 +42,16 @@ class OVITO_CORE_EXPORT PropertyFieldBase
 protected:
 
     /// Generates a notification event to inform the dependents of the field's owner that it has changed.
-    static void generateTargetChangedEvent(RefMaker* owner, const PropertyFieldDescriptor* descriptor, ReferenceEvent::Type eventType = ReferenceEvent::TargetChanged);
+    static void generateTargetChangedEvent(RefMaker* owner, const PropertyFieldDescriptor* descriptor, int eventType = ReferenceEvent::TargetChanged);
 
     /// Generates a notification event to inform the dependents of the field's owner that it has changed.
     static void generatePropertyChangedEvent(RefMaker* owner, const PropertyFieldDescriptor* descriptor);
 
+#ifdef OVITO_DEBUG
+    /// Verifies that owner is of the right type, i.e., defines the given property field.
+    static bool ownerTypeCheck(RefMaker* owner, const PropertyFieldDescriptor* descriptor);
+#endif
+
 protected:
 
     /// This abstract undo record class keeps a strong reference object whose property has been changed.
@@ -100,15 +106,25 @@ public:
     /// Changes the value of the property. Handles undo and sends a notification message.
     template<typename T>
     void set(RefMaker* owner, const PropertyFieldDescriptor* descriptor, T&& newValue) {
-        OVITO_ASSERT(owner != nullptr);
+        OVITO_ASSERT(ownerTypeCheck(owner, descriptor));
+
+        // The value type supports comparison, do nothing if the new value is equal to the old one.
         if constexpr(boost::has_equal_to<property_type, std::decay_t<T>>::value) {
-            if(get() == newValue) return;
+            if(get() == newValue)
+                return;
         }
+
+        // Handle automatic undo recording of the value change (if enabled for this property field).
         if constexpr(!(flags & PROPERTY_FIELD_NO_UNDO)) {
-            if(descriptor->automaticUndo() && CompoundOperation::isUndoRecording())
+            OVITO_ASSERT(descriptor->automaticUndo());
+            if(!owner->isBeingInitializedOrDeleted() && CompoundOperation::isUndoRecording())
                 CompoundOperation::current()->addOperation(std::make_unique<PropertyChangeOperation>(owner, *this, descriptor));
         }
+
+        // Assign the new value to the storage.
         mutableValue() = std::forward<T>(newValue);
+
+        // Send notification signals.
         valueChangedInternal(owner, descriptor);
     }
 
@@ -139,14 +155,14 @@ public:
     }
 
     /// Returns the internal value stored in this property field.
-    inline const property_type& get() const { return _value; }
+    inline std::add_const_t<property_type>& get() const { return _value; }
 
     /// Returns a reference to the internal value stored in this property field.
     /// Warning: Do not use this function unless you know what you are doing!
     inline property_type& mutableValue() { return _value; }
 
     /// Cast the property field to the property value.
-    inline operator const property_type&() const { return get(); }
+    inline operator std::add_const_t<property_type>&() const { return get(); }
 
 private:
 
@@ -155,7 +171,7 @@ private:
         generatePropertyChangedEvent(owner, descriptor);
         generateTargetChangedEvent(owner, descriptor);
         if(descriptor->extraChangeEventType() != 0)
-            generateTargetChangedEvent(owner, descriptor, static_cast<ReferenceEvent::Type>(descriptor->extraChangeEventType()));
+            generateTargetChangedEvent(owner, descriptor, descriptor->extraChangeEventType());
     }
 
     /// This undo class records a change to the property value.
@@ -268,11 +284,9 @@ private:
 };
 
 /// This utility class template maps a specific fancy pointer type to the right general fancy pointer type.
-///    T*                 ->  RefTarget*
 ///    OORef<T>           ->  OORef<RefTarget>
 ///    DataOORef<const T> ->  DataOORef<const DataObject>
 template<typename T> struct SelectGenericReferenceType {};
-template<typename T> struct SelectGenericReferenceType<T*> { using type = RefTarget*; };
 template<typename T> struct SelectGenericReferenceType<OORef<T>> { using type = OORef<RefTarget>; };
 template<typename T> struct SelectGenericReferenceType<DataOORef<const T>> { using type = DataOORef<const DataObject>; };
 
@@ -293,7 +307,7 @@ public:
 #endif
 
     /// Returns a raw, untyped pointer to the currently referenced object.
-    inline typename std::pointer_traits<pointer>::element_type* get() const noexcept { return to_address(_target); }
+    inline typename std::pointer_traits<pointer>::element_type* get() const noexcept { return _target.get(); }
 
 protected:
 
@@ -303,10 +317,6 @@ protected:
     /// Replaces the target stored in the reference field.
     void swapReference(RefMaker* owner, const PropertyFieldDescriptor* descriptor, pointer& inactiveTarget);
 
-    /// Obtains the object address represented by a fancy pointer.
-    template<class U> static constexpr U* to_address(U* p) noexcept { return p; }
-    template<class U> static constexpr auto to_address(const U& p) noexcept { return p.get(); }
-
     /// The actual fancy pointer value.
     pointer _target{};
 };
@@ -314,10 +324,8 @@ protected:
 // Instantiate base class template for the fancy pointer base types needed.
 #ifndef OVITO_BUILD_MONOLITHIC
     #if !defined(Core_EXPORTS)
-        extern template class OVITO_CORE_EXPORT SingleReferenceFieldBase<RefTarget*>;
         extern template class OVITO_CORE_EXPORT SingleReferenceFieldBase<OORef<RefTarget>>;
     #elif !defined(Q_CC_MSVC) && !defined(Q_CC_CLANG)
-        template class OVITO_CORE_EXPORT SingleReferenceFieldBase<RefTarget*>;
         template class OVITO_CORE_EXPORT SingleReferenceFieldBase<OORef<RefTarget>>;
     #endif
 #endif
@@ -344,15 +352,15 @@ public:
 
     /// Write access to the RefTarget pointer. Changes the value of the reference field.
     inline void set(RefMaker* owner, const PropertyFieldDescriptor* descriptor, fancy_pointer newPointer) {
-        if constexpr(!std::is_pointer_v<fancy_pointer> || !std::is_const_v<std::remove_pointer_t<fancy_pointer>>)
+        if constexpr(!std::is_const_v<typename std::pointer_traits<fancy_pointer>::element_type>)
             base_class::set(owner, descriptor, std::move(newPointer));
         else
-            base_class::set(owner, descriptor, const_cast<RefTarget*>(static_cast<const RefTarget*>(newPointer)));
+            base_class::set(owner, descriptor, const_pointer_cast<target_object_type>(std::move(newPointer)));
     }
 
     /// Returns the target object currently being referenced by the reference field.
     inline raw_pointer get() const noexcept {
-        return static_object_cast<target_object_type>(base_class::to_address(base_class::get()));
+        return static_object_cast<target_object_type>(base_class::get());
     }
 };
 
@@ -377,7 +385,7 @@ public:
 #endif
 
     /// Returns a raw, untyped pointer to the i-th object in the vector.
-    inline typename std::pointer_traits<pointer>::element_type* get(size_type i) const noexcept { return to_address(_targets[i]); }
+    inline typename std::pointer_traits<pointer>::element_type* get(size_type i) const noexcept { return _targets[i].get(); }
 
     /// Clears all references and sets the vector size to zero.
     void clear(RefMaker* owner, const PropertyFieldDescriptor* descriptor);
@@ -397,8 +405,9 @@ public:
     /// Returns the index position of the first occurrence of value in the vector,
     /// searching forward from index position from. Returns -1 if no item matched.
     size_type indexOf(const RefTarget* value, size_type from = 0) const noexcept {
-        for(size_type i = from; i < _targets.size(); i++)
-            if(_targets[i] == value) return i;
+        for(size_type i = from, count = _targets.size(); i < count; i++)
+            if(_targets[i] == value)
+                return i;
         return -1;
     }
 
@@ -419,21 +428,15 @@ protected:
     /// Adds the target to the vector reference field.
     size_type addReference(RefMaker* owner, const PropertyFieldDescriptor* descriptor, size_type index, pointer& target);
 
-    /// Obtains the object address represented by a fancy pointer.
-    template<class U> static constexpr U* to_address(U* p) noexcept { return p; }
-    template<class U> static constexpr auto to_address(const U& p) noexcept { return p.get(); }
-
-    /// The actual list of fancy pointers.
+    /// The actual list of fancy pointers to the RefTargets.
     container _targets;
 };
 
 // Instantiate base class template for the fancy pointer base types needed.
 #ifndef OVITO_BUILD_MONOLITHIC
     #if !defined(Core_EXPORTS)
-        extern template class OVITO_CORE_EXPORT VectorReferenceFieldBase<RefTarget*>;
         extern template class OVITO_CORE_EXPORT VectorReferenceFieldBase<OORef<RefTarget>>;
     #elif !defined(Q_CC_MSVC) && !defined(Q_CC_CLANG)
-        template class OVITO_CORE_EXPORT VectorReferenceFieldBase<RefTarget*>;
         template class OVITO_CORE_EXPORT VectorReferenceFieldBase<OORef<RefTarget>>;
     #endif
 #endif
@@ -504,9 +507,6 @@ public:
     }
 };
 
-/// Vector container type used by vector reference fields with T* regular pointers.
-template<typename T> using WeakRefVector = typename VectorReferenceField<T*>::container;
-
 /// Vector container type used by vector reference fields with OORef<T> fancy pointers.
 template<typename T> using OORefVector = typename VectorReferenceField<OORef<T>>::container;
 
