@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -35,24 +35,7 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(RefMaker);
-
-/******************************************************************************
-* This method is called when the reference counter of this OvitoObject
-* has reached zero.
-******************************************************************************/
-void RefMaker::aboutToBeDeleted()
-{
-    OVITO_CHECK_OBJECT_POINTER(this);
-
-    // Make sure undo recording is not active while deleting a RefTarget.
-    OVITO_ASSERT_MSG(!isRefTarget() || isUndoRecording() == false, "RefMaker::aboutToBeDeleted()", "Cannot delete object from memory while undo recording is active.");
-
-    // Clear all references this object has to other objects.
-    clearAllReferences();
-
-    OvitoObject::aboutToBeDeleted();
-}
+IMPLEMENT_ABSTRACT_OVITO_CLASS(RefMaker);
 
 /******************************************************************************
 * Returns the value stored in a non-animatable property field of this RefMaker object.
@@ -62,7 +45,7 @@ QVariant RefMaker::getPropertyFieldValue(const PropertyFieldDescriptor* field) c
     OVITO_ASSERT_MSG(!field->isReferenceField(), "RefMaker::getPropertyFieldValue", "This function may be used only to access property fields and not reference fields.");
     OVITO_ASSERT_MSG(getOOClass().isDerivedFrom(*field->definingClass()), "RefMaker::getPropertyFieldValue", "The property field has not been defined in this class or its base classes.");
     OVITO_ASSERT_MSG(field->_propertyStorageReadFunc != nullptr, "RefMaker::getPropertyFieldValue", "The property field is a runtime property field, which doesn't allow conversion to a QVariant value.");
-    return field->_propertyStorageReadFunc(const_cast<RefMaker*>(this));
+    return field->_propertyStorageReadFunc(const_cast<RefMaker*>(this), field);
 }
 
 /******************************************************************************
@@ -73,7 +56,7 @@ void RefMaker::setPropertyFieldValue(const PropertyFieldDescriptor* field, const
     OVITO_ASSERT_MSG(!field->isReferenceField(), "RefMaker::setPropertyFieldValue", "This function may be used only to access property fields and not reference fields.");
     OVITO_ASSERT_MSG(getOOClass().isDerivedFrom(*field->definingClass()), "RefMaker::setPropertyFieldValue", "The property field has not been defined in this class or its base classes.");
     OVITO_ASSERT_MSG(field->_propertyStorageWriteFunc != nullptr, "RefMaker::getPropertyFieldValue", "The property field is a runtime property field, which doesn't allow assignment of a QVariant value.");
-    field->_propertyStorageWriteFunc(this, newValue);
+    field->_propertyStorageWriteFunc(this, field, newValue);
 }
 
 /******************************************************************************
@@ -86,7 +69,7 @@ void RefMaker::copyPropertyFieldValue(const PropertyFieldDescriptor* field, cons
     OVITO_ASSERT_MSG(getOOClass().isDerivedFrom(*field->definingClass()), "RefMaker::copyPropertyFieldValue", "The property field has not been defined in this class or its base classes.");
     OVITO_ASSERT_MSG(other.getOOClass().isDerivedFrom(*field->definingClass()), "RefMaker::copyPropertyFieldValue", "The property field has not been defined in the source's class or its base classes.");
     OVITO_ASSERT(field->_propertyStorageCopyFunc != nullptr);
-    field->_propertyStorageCopyFunc(this, &other);
+    field->_propertyStorageCopyFunc(this, field, &other);
 }
 
 /******************************************************************************
@@ -98,7 +81,19 @@ RefTarget* RefMaker::getReferenceFieldTarget(const PropertyFieldDescriptor* fiel
     OVITO_ASSERT_MSG(field->isVector() == false, "RefMaker::getReferenceFieldTarget()", "This function may not be used to retrieve vector reference fields.");
     OVITO_ASSERT_MSG(getOOClass().isDerivedFrom(*field->definingClass()), "RefMaker::getReferenceFieldTarget()", "The reference field has not been defined in this class or its base classes.");
     OVITO_ASSERT(field->_singleReferenceReadFunc != nullptr);
-    return field->_singleReferenceReadFunc(this);
+    return field->_singleReferenceReadFunc(this, field);
+}
+
+/******************************************************************************
+* Sets a reference field of this RefMaker to reference a different target.
+******************************************************************************/
+void RefMaker::setReferenceFieldTarget(const PropertyFieldDescriptor* field, OORef<const RefTarget> target)
+{
+    OVITO_ASSERT_MSG(field->isReferenceField(), "RefMaker::setReferenceFieldTarget()", "This function may not be used to set property fields.");
+    OVITO_ASSERT_MSG(field->isVector() == false, "RefMaker::setReferenceFieldTarget()", "This function may not be used to set vector reference fields.");
+    OVITO_ASSERT_MSG(getOOClass().isDerivedFrom(*field->definingClass()), "RefMaker::setReferenceFieldTarget()", "The reference field has not been defined in this class or its base classes.");
+    OVITO_ASSERT(field->_singleReferenceWriteFuncRef != nullptr);
+    field->_singleReferenceWriteFuncRef(this, field, std::move(target));
 }
 
 /******************************************************************************
@@ -110,7 +105,7 @@ int RefMaker::getVectorReferenceFieldSize(const PropertyFieldDescriptor* field)
     OVITO_ASSERT_MSG(field->isVector() == true, "RefMaker::getVectorReferenceFieldSize", "This function may not be used to retrieve single reference fields.");
     OVITO_ASSERT_MSG(getOOClass().isDerivedFrom(*field->definingClass()), "RefMaker::getVectorReferenceFieldSize", "The reference field has not been defined in this class or its base classes.");
     OVITO_ASSERT(field->_vectorReferenceCountFunc != nullptr);
-    return field->_vectorReferenceCountFunc(this);
+    return field->_vectorReferenceCountFunc(this, field);
 }
 
 /******************************************************************************
@@ -122,7 +117,7 @@ RefTarget* RefMaker::getVectorReferenceFieldTarget(const PropertyFieldDescriptor
     OVITO_ASSERT_MSG(field->isVector() == true, "RefMaker::getVectorReferenceFieldTarget", "This function may not be used to retrieve single reference fields.");
     OVITO_ASSERT_MSG(getOOClass().isDerivedFrom(*field->definingClass()), "RefMaker::getVectorReferenceFieldTarget", "The reference field has not been defined in this class or its base classes.");
     OVITO_ASSERT(field->_vectorReferenceGetFunc != nullptr);
-    return field->_vectorReferenceGetFunc(this, index);
+    return field->_vectorReferenceGetFunc(this, field, index);
 }
 
 /******************************************************************************
@@ -134,7 +129,7 @@ void RefMaker::setVectorReferenceFieldTarget(const PropertyFieldDescriptor* fiel
     OVITO_ASSERT_MSG(field->isVector() == true, "RefMaker::setVectorReferenceFieldTarget", "This function may not be used to retrieve single reference fields.");
     OVITO_ASSERT_MSG(getOOClass().isDerivedFrom(*field->definingClass()), "RefMaker::setVectorReferenceFieldTarget", "The reference field has not been defined in this class or its base classes.");
     OVITO_ASSERT(field->_vectorReferenceSetFunc != nullptr);
-    field->_vectorReferenceSetFunc(this, index, target);
+    field->_vectorReferenceSetFunc(this, field, index, target);
 }
 
 /******************************************************************************
@@ -146,7 +141,7 @@ void RefMaker::removeVectorReferenceFieldTarget(const PropertyFieldDescriptor* f
     OVITO_ASSERT_MSG(field->isVector() == true, "RefMaker::removeVectorReferenceFieldTarget", "This function may not be used to retrieve single reference fields.");
     OVITO_ASSERT_MSG(getOOClass().isDerivedFrom(*field->definingClass()), "RefMaker::removeVectorReferenceFieldTarget", "The reference field has not been defined in this class or its base classes.");
     OVITO_ASSERT(field->_vectorReferenceRemoveFunc != nullptr);
-    field->_vectorReferenceRemoveFunc(this, index);
+    field->_vectorReferenceRemoveFunc(this, field, index);
 }
 
 /******************************************************************************
@@ -161,14 +156,6 @@ bool RefMaker::vectorReferenceFieldContains(const PropertyFieldDescriptor* field
     return false;
 }
 
-/******************************************************************************
-* This Qt slot receives signals from the target objects referenced by this object.
-******************************************************************************/
-void RefMaker::receiveObjectEvent(RefTarget* sender, const ReferenceEvent& event)
-{
-    handleReferenceEvent(sender, event);
-}
-
 /******************************************************************************
 * Handles a notification event from a RefTarget referenced by this object.
 ******************************************************************************/
@@ -200,51 +187,47 @@ bool RefMaker::handleReferenceEvent(RefTarget* source, const ReferenceEvent& eve
         return true;
     }
 
-    // Handle VisitDependents signals.
-    if(event.type() == ReferenceEvent::VisitDependents) {
-        const VisitDependentsEvent& visitEvent = static_cast<const VisitDependentsEvent&>(event);
-        visitEvent.visitDependent(this);
-        return false;
-    }
-
     // Let the RefMaker-derived class process the message.
     return referenceEvent(source, event);
 }
 
 /******************************************************************************
-* Is called when a RefTarget referenced by this object has generated an event.
+* Is called when a RefTarget referenced by this object generated an event.
 ******************************************************************************/
 bool RefMaker::referenceEvent(RefTarget* source, const ReferenceEvent& event)
 {
     if(event.shouldPropagate()) {
-        // Check if message is comming from a reference field for which message propagation is explicitly disabled.
+        // Check if message is coming from a reference field for which message propagation is explicitly disabled.
         // Note that a target object may be referenced from multiple reference fields, some of which having
         // message propagation enabled and some not.
-        bool isSupressedField = false;
+        bool isSuppressedField = false;
         for(const PropertyFieldDescriptor* field : getOOMetaClass().propertyFields()) {
-            if(!field->isReferenceField()) continue;
-            if(!field->flags().testFlag(PROPERTY_FIELD_DONT_PROPAGATE_MESSAGES)) continue;
+            if(!field->isReferenceField())
+                continue;
+            if(!field->flags().testFlag(PROPERTY_FIELD_DONT_PROPAGATE_MESSAGES))
+                continue;
             if(!field->isVector()) {
-                if(field->_singleReferenceReadFunc(this) == source) {
-                    isSupressedField = true;
+                if(getReferenceFieldTarget(field) == source) {
+                    isSuppressedField = true;
                     break;
                 }
             }
             else {
                 if(vectorReferenceFieldContains(field, source)) {
-                    isSupressedField = true;
+                    isSuppressedField = true;
                     break;
                 }
             }
         }
-        if(!isSupressedField)
+        if(!isSuppressedField)
             return true;
-        // Perform counter check and determine if message is comming from a reference field for which message propagation
+        // Perform counter check and determine if message is coming from a reference field for which message propagation
         // is NOT explicitly disabled.
         for(const PropertyFieldDescriptor* field : getOOMetaClass().propertyFields()) {
-            if(!field->isReferenceField()) continue;
+            if(!field->isReferenceField())
+                continue;
             if(!field->isVector()) {
-                if(field->_singleReferenceReadFunc(this) == source) {
+                if(getReferenceFieldTarget(field) == source) {
                     if(!field->flags().testFlag(PROPERTY_FIELD_DONT_PROPAGATE_MESSAGES))
                         return true;
                 }
@@ -267,13 +250,13 @@ bool RefMaker::referenceEvent(RefTarget* source, const ReferenceEvent& event)
 ******************************************************************************/
 bool RefMaker::hasReferenceTo(const RefTarget* target) const
 {
-    if(!target) return false;
-    OVITO_CHECK_OBJECT_POINTER(target);
+    OVITO_ASSERT(target != nullptr);
 
     for(const PropertyFieldDescriptor* field : getOOMetaClass().propertyFields()) {
-        if(!field->isReferenceField()) continue;
+        if(!field->isReferenceField())
+            continue;
         if(!field->isVector()) {
-            if(field->_singleReferenceReadFunc(this) == target)
+            if(getReferenceFieldTarget(field) == target)
                 return true;
         }
         else {
@@ -281,6 +264,7 @@ bool RefMaker::hasReferenceTo(const RefTarget* target) const
                 return true;
         }
     }
+
     return false;
 }
 
@@ -289,15 +273,13 @@ bool RefMaker::hasReferenceTo(const RefTarget* target) const
 ******************************************************************************/
 bool RefMaker::hasStrongReferenceTo(const RefTarget* target) const
 {
-    if(!target) return false;
-    OVITO_CHECK_OBJECT_POINTER(target);
+    OVITO_ASSERT(target != nullptr);
 
     for(const PropertyFieldDescriptor* field : getOOMetaClass().propertyFields()) {
-        if(!field->isReferenceField()) continue;
-        // Skip weak references for which event propagation is disabled.
-        if(field->isWeakReference() && field->flags().testFlag(PROPERTY_FIELD_DONT_PROPAGATE_MESSAGES)) continue;
+        if(!field->isReferenceField())
+            continue;
         if(!field->isVector()) {
-            if(field->_singleReferenceReadFunc(this) == target)
+            if(getReferenceFieldTarget(field) == target)
                 return true;
         }
         else {
@@ -314,7 +296,8 @@ bool RefMaker::hasStrongReferenceTo(const RefTarget* target) const
 ******************************************************************************/
 void RefMaker::replaceReferencesTo(const RefTarget* oldTarget, const RefTarget* newTarget)
 {
-    if(!oldTarget) return;
+    if(!oldTarget)
+        return;
     OVITO_CHECK_OBJECT_POINTER(oldTarget);
 
     // Iterate over all reference fields in the class hierarchy.
@@ -323,14 +306,13 @@ void RefMaker::replaceReferencesTo(const RefTarget* oldTarget, const RefTarget*
 #endif
     const OvitoClass& oldTargetClass = oldTarget->getOOClass();
     for(const PropertyFieldDescriptor* field : getOOMetaClass().propertyFields()) {
-        if(!field->isReferenceField()) continue;
-        if(!oldTargetClass.isDerivedFrom(*field->targetClass())) continue;
+        if(!field->isReferenceField())
+            continue;
+        if(!oldTargetClass.isDerivedFrom(*field->targetClass()))
+            continue;
         if(!field->isVector()) {
-            if(field->_singleReferenceReadFunc(this) == oldTarget) {
-                // Check for cyclic strong references.
-                if(newTarget && (!field->flags().testFlag(PROPERTY_FIELD_DONT_PROPAGATE_MESSAGES) || !field->isWeakReference()) && isReferencedBy(newTarget, true))
-                    throw CyclicReferenceError();
-                field->_singleReferenceWriteFunc(this, newTarget);
+            if(getReferenceFieldTarget(field) == oldTarget) {
+                setReferenceFieldTarget(field, newTarget);
 #ifdef OVITO_DEBUG
                 hasBeenReplaced = true;
 #endif
@@ -340,9 +322,6 @@ void RefMaker::replaceReferencesTo(const RefTarget* oldTarget, const RefTarget*
             int count = getVectorReferenceFieldSize(field);
             for(int i = count; i--;) {
                 if(getVectorReferenceFieldTarget(field, i) == oldTarget) {
-                    // Check for cyclic references.
-                    if(newTarget && (!field->flags().testFlag(PROPERTY_FIELD_DONT_PROPAGATE_MESSAGES) || !field->isWeakReference()) && isReferencedBy(newTarget, true))
-                        throw CyclicReferenceError();
                     setVectorReferenceFieldTarget(field, i, newTarget);
 #ifdef OVITO_DEBUG
                     hasBeenReplaced = true;
@@ -367,10 +346,11 @@ void RefMaker::clearReferencesTo(const RefTarget* target)
 
     // Iterate over all reference fields in the class hierarchy.
     for(const PropertyFieldDescriptor* field : getOOMetaClass().propertyFields()) {
-        if(!field->isReferenceField()) continue;
+        if(!field->isReferenceField())
+            continue;
         if(!field->isVector()) {
-            if(field->_singleReferenceReadFunc(this) == target)
-                field->_singleReferenceWriteFunc(this, nullptr);
+            if(getReferenceFieldTarget(field) == target)
+                setReferenceFieldTarget(field, nullptr);
         }
         else {
             for(int i = getVectorReferenceFieldSize(field); i--; ) {
@@ -407,7 +387,7 @@ void RefMaker::clearReferenceField(const PropertyFieldDescriptor* field)
     OVITO_ASSERT_MSG(getOOClass().isDerivedFrom(*field->definingClass()), "RefMaker::clearReferenceField()", "The reference field has not been defined in this class or its base classes.");
 
     if(!field->isVector()) {
-        field->_singleReferenceWriteFunc(this, nullptr);
+        setReferenceFieldTarget(field, nullptr);
     }
     else {
         while(int count = getVectorReferenceFieldSize(field))
@@ -428,39 +408,43 @@ void RefMaker::saveToStream(ObjectSaveStream& stream, bool excludeRecomputableDa
 
     // Iterate over all property fields in the class hierarchy.
     for(const PropertyFieldDescriptor* field : getOOMetaClass().propertyFields()) {
-
         if(field->isReferenceField()) {
             // Write the object pointed to by the reference field to the stream.
-
-            // Write reference target object to stream.
-            stream.beginChunk(0x02);
-            try {
-                if(!field->isVector()) {
-                    stream.saveObject(field->_singleReferenceReadFunc(this), excludeRecomputableData || field->dontSaveRecomputableData());
+            if(!field->dontSaveTarget()) {
+                stream.beginChunk(0x02);
+                try {
+                    if(!field->isVector()) {
+                        stream.saveObject(getReferenceFieldTarget(field), excludeRecomputableData || field->dontSaveRecomputableData());
+                    }
+                    else {
+                        qint32 count = getVectorReferenceFieldSize(field);
+                        stream << count;
+                        for(int i = 0; i < count; i++)
+                            stream.saveObject(getVectorReferenceFieldTarget(field, i), excludeRecomputableData || field->dontSaveRecomputableData());
+                    }
                 }
-                else {
-                    qint32 count = getVectorReferenceFieldSize(field);
-                    stream << count;
-                    for(int i = 0; i < count; i++)
-                        stream.saveObject(getVectorReferenceFieldTarget(field, i), excludeRecomputableData || field->dontSaveRecomputableData());
+                catch(Exception& ex) {
+                    throw ex.prependGeneralMessage(tr("Failed to serialize contents of reference field %1 of class %2.").arg(field->identifier()).arg(field->definingClass()->name()));
                 }
+                stream.endChunk();
             }
-            catch(Exception& ex) {
-                throw ex.prependGeneralMessage(tr("Failed to serialize contents of reference field %1 of class %2.").arg(field->identifier()).arg(field->definingClass()->name()));
+            else {
+                // Indicate that this target is not serialized.
+                stream.beginChunk(0x05);
+                stream.endChunk();
             }
-            stream.endChunk();
         }
         else {
             // Write the primitive value stored in the property field to the stream.
             if(field->_propertyStorageSaveFunc != nullptr) {
                 stream.beginChunk(0x04);
-                field->_propertyStorageSaveFunc(this, stream);
+                field->_propertyStorageSaveFunc(this, field, stream);
 #if 0
-                qDebug() << "  Property field" << field->identifier() << " contains" << field->propertyStorageReadFunc(this);
+                qDebug() << "  Property field" << field->identifier() << " contains" << field->propertyStorageReadFunc(this, field);
 #endif
             }
             else {
-                // Indicate that this property field is not serizaliable.
+                // Indicate that this property field is not serializable.
                 stream.beginChunk(0x05);
             }
             stream.endChunk();
@@ -506,18 +490,15 @@ void RefMaker::loadFromStream(ObjectLoadStream& stream)
                         OORef<RefTarget> target = stream.loadObject<RefTarget>();
                         if(target && !target->getOOClass().isDerivedFrom(*fieldEntry.targetClass)) {
                             throw Exception(tr("Incompatible object stored in reference field %1 of class %2. Expected class %3 but found class %4 in file.")
-                                .arg(QString(fieldEntry.identifier)).arg(fieldEntry.definingClass->name()).arg(fieldEntry.targetClass->name()).arg(target->getOOClass().name()));
+                                .arg(QString::fromUtf8(fieldEntry.identifier)).arg(fieldEntry.definingClass->name()).arg(fieldEntry.targetClass->name()).arg(target->getOOClass().name()));
                         }
 #if 0
                         qDebug() << "  Reference field" << fieldEntry.identifier << " contains" << target;
 #endif
-                        if(!fieldEntry.field->isWeakReference())
-                            fieldEntry.field->_singleReferenceWriteFuncRef(this, std::move(target));
-                        else
-                            fieldEntry.field->_singleReferenceWriteFunc(this, target.get());
+                        fieldEntry.field->_singleReferenceWriteFuncRef(this, fieldEntry.field, std::move(target));
                     }
                     else {
-                        // Remove any prexisting targets from the reference field.
+                        // Remove any pre-existing targets from the reference field.
                         clearReferenceField(fieldEntry.field);
 
                         // Load each target object and store it in the list reference field.
@@ -528,12 +509,12 @@ void RefMaker::loadFromStream(ObjectLoadStream& stream)
                             OORef<RefTarget> target = stream.loadObject<RefTarget>();
                             if(target && !target->getOOClass().isDerivedFrom(*fieldEntry.targetClass)) {
                                 throw Exception(tr("Incompatible object stored in reference field %1 of class %2. Expected class %3 but found class %4 in file.")
-                                    .arg(QString(fieldEntry.identifier)).arg(fieldEntry.definingClass->name(), fieldEntry.targetClass->name(), target->getOOClass().name()));
+                                    .arg(QString::fromUtf8(fieldEntry.identifier)).arg(fieldEntry.definingClass->name(), fieldEntry.targetClass->name(), target->getOOClass().name()));
                             }
 #if 0
                             qDebug() << "  Vector reference field" << fieldEntry.identifier << " contains" << target;
 #endif
-                            fieldEntry.field->_vectorReferenceInsertFunc(this, i, std::move(target));
+                            fieldEntry.field->_vectorReferenceInsertFunc(this, fieldEntry.field, i, std::move(target));
                         }
                     }
                 }
@@ -554,6 +535,9 @@ void RefMaker::loadFromStream(ObjectLoadStream& stream)
                     }
                 }
             }
+            else if(chunkId == 0x05) {
+                // The target object has not been serialized (typically due to flag PROPERTY_FIELD_DONT_SAVE_TARGET).
+            }
             else {
                 throw Exception(tr("Expected reference field '%1' in object %2").arg(QString(fieldEntry.identifier)).arg(fieldEntry.definingClass->name()));
             }
@@ -566,7 +550,7 @@ void RefMaker::loadFromStream(ObjectLoadStream& stream)
             if(chunkId == 0x04) {
                 if(!loadPropertyFieldFromStream(stream, fieldEntry)) {
                     if(fieldEntry.field && fieldEntry.field->_propertyStorageLoadFunc != nullptr) {
-                        fieldEntry.field->_propertyStorageLoadFunc(this, stream);
+                        fieldEntry.field->_propertyStorageLoadFunc(this, fieldEntry.field, stream);
                     }
                     else {
                         // The property field no longer exists.
@@ -608,7 +592,7 @@ void RefMaker::walkNode(QSet<RefTarget*>& nodes, const RefMaker* node)
     for(const PropertyFieldDescriptor* field : node->getOOMetaClass().propertyFields()) {
         if(!field->isReferenceField()) continue;
         if(!field->isVector()) {
-            RefTarget* target = field->_singleReferenceReadFunc(node);
+            RefTarget* target = node->getReferenceFieldTarget(field);
             if(target != nullptr && !nodes.contains(target)) {
                 nodes.insert(target);
                 walkNode(nodes, target);
@@ -633,7 +617,7 @@ void RefMaker::walkNode(QSet<RefTarget*>& nodes, const RefMaker* node)
 * by the OORef<>::create() function. It loads the initial values for property fields
 * with user-defined default settings (those having the PROPERTY_FIELD_MEMORIZE flag set).
 ******************************************************************************/
-void RefMaker::initializeParametersToUserDefaults()
+void RefMaker::initializeParametersToUserDefaultsNonrecursive()
 {
     // Iterate over all property fields in the class hierarchy.
     for(const PropertyFieldDescriptor* field : getOOMetaClass().propertyFields()) {
@@ -645,7 +629,7 @@ void RefMaker::initializeParametersToUserDefaults()
             else if(!field->isVector()) {
 #ifndef OVITO_DISABLE_QSETTINGS
                 // If it's a controller type, load default controller value.
-                if(Controller* ctrl = dynamic_object_cast<Controller>(field->_singleReferenceReadFunc(this))) {
+                if(Controller* ctrl = dynamic_object_cast<Controller>(getReferenceFieldTarget(field))) {
                     QSettings settings;
                     settings.beginGroup(getOOClass().plugin()->pluginId());
                     settings.beginGroup(getOOClass().name());
@@ -673,13 +657,13 @@ void RefMaker::initializeParametersToUserDefaults()
 ******************************************************************************/
 void RefMaker::initializeParametersToUserDefaultsRecursive()
 {
-    initializeParametersToUserDefaults();
+    initializeParametersToUserDefaultsNonrecursive();
 
     // Iterate over all reference fields in the class hierarchy.
     for(const PropertyFieldDescriptor* field : getOOMetaClass().propertyFields()) {
         if(field->isReferenceField()) {
             if(!field->isVector()) {
-                if(RefTarget* target = field->_singleReferenceReadFunc(this))
+                if(RefTarget* target = getReferenceFieldTarget(field))
                     target->initializeParametersToUserDefaultsRecursive();
             }
             else {
@@ -705,7 +689,7 @@ void RefMaker::freezeInitialParameterValues(std::initializer_list<const Property
         OVITO_ASSERT_MSG(getOOClass().isDerivedFrom(*field->definingClass()), "RefMaker::freezeInitialParameterValues", "The shadow property field has not been defined in this class or its base classes.");
         OVITO_ASSERT_MSG(field->_propertyStorageTakeSnapshotFunc != nullptr, "RefMaker::freezeInitialParameterValues", "The property field is not a shadow property field.");
 
-        field->_propertyStorageTakeSnapshotFunc(this);
+        field->_propertyStorageTakeSnapshotFunc(this, field);
     }
 }
 
@@ -720,7 +704,7 @@ void RefMaker::copyInitialParametersToObject(RefMaker* obj) const
 
     for(const PropertyFieldDescriptor* field : getOOMetaClass().propertyFields()) {
         if(field->_propertyStorageRestoreSnapshotFunc)
-            field->_propertyStorageRestoreSnapshotFunc(this, obj);
+            field->_propertyStorageRestoreSnapshotFunc(this, field, obj);
     }
 }
 
