@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -26,6 +26,7 @@
 #include <ovito/core/Core.h>
 #include <ovito/core/oo/RefMakerClass.h>
 #include <ovito/core/oo/ReferenceEvent.h>
+#include <ovito/core/oo/PropertyFieldFlags.h>
 #include <ovito/core/app/undo/UndoableOperation.h>
 
 namespace Ovito {
@@ -40,8 +41,7 @@ namespace Ovito {
 class OVITO_CORE_EXPORT CyclicReferenceError : public Exception
 {
 public:
-
-    /// \brief Default constructor.
+    /// Constructor.
     CyclicReferenceError() : Exception(QStringLiteral("Cyclic reference error")) {}
 };
 
@@ -54,12 +54,17 @@ class OVITO_CORE_EXPORT RefMaker : public OvitoObject
 
 protected:
 
-    /// \brief Constructor.
-    RefMaker() = default;
+    // Specifies the default flags for all reference fields of RefMaker-derived classes that are not a RefTarget.
+    static constexpr auto PROPERTY_FIELD_STANDARD_FLAGS =
+                PROPERTY_FIELD_NEVER_CLONE_TARGET |
+                PROPERTY_FIELD_NO_UNDO |
+                PROPERTY_FIELD_DONT_PROPAGATE_MESSAGES |
+                PROPERTY_FIELD_NO_CHANGE_MESSAGE |
+                PROPERTY_FIELD_NO_SUB_ANIM;
 
     /////////////////////////////// Reference field events ///////////////////////////////////
 
-    /// \brief Is called when a RefTarget referenced by this object has generated an event.
+    /// \brief Is called when a RefTarget referenced by this object generated an event.
     /// \param source A direct reference target of this RefMaker specifying the source of the message.
     ///               Note that this may not be the sender or generator of the notification
     ///               event. The sender is returned by ReferenceEvent::sender().
@@ -148,8 +153,6 @@ protected:
     /// All reference fields containing a pointer to \a target will be reset to \c NULL.
     /// If \a target is referenced in a VectorReferenceField then the item is
     /// removed from the vector field.
-    ///
-    /// \undoable
     void clearReferencesTo(const RefTarget* target);
 
     /// \brief Clears a reference field.
@@ -157,8 +160,6 @@ protected:
     ///
     /// If the reference field specified by \a field is a single reference field then it is set to the value \c NULL.
     /// If it is a VectorReferenceField then all references are removed.
-    ///
-    /// \undoable
     void clearReferenceField(const PropertyFieldDescriptor* field);
 
     /// \brief Saves the class' contents to an output stream.
@@ -200,30 +201,14 @@ protected:
     /// The default implementation returns \c false.
     virtual bool loadPropertyFieldFromStream(ObjectLoadStream& stream, const RefMakerClass::SerializedClassInfo::PropertyFieldInfo& serializedField) { return false; }
 
-    /// This method is called after the reference counter of this object has reached zero
-    /// and before the object is being deleted.
-    virtual void aboutToBeDeleted() override;
-
-public Q_SLOTS:
+public:
 
     /// \brief Clears all references held by this RefMarker.
     ///
     /// All single reference fields are set to \c NULL and all vector reference
     /// fields are cleared.
-    ///
-    /// \undoable
     void clearAllReferences();
 
-private Q_SLOTS:
-
-    /// This Qt slot receives signals from the target objects referenced by this object.
-    void receiveObjectEvent(RefTarget* sender, const ReferenceEvent& event);
-
-public:
-
-    /// \brief Returns true if this object is an instance of a RefTarget derived class.
-    virtual bool isRefTarget() const { return false; }
-
     /////////////////////////// Property field access ///////////////////////////////
 
     /// \brief Returns the value stored in a non-animatable property field of this RefMaker object.
@@ -250,6 +235,11 @@ public:
     /// \return The current value of the reference field.
     RefTarget* getReferenceFieldTarget(const PropertyFieldDescriptor* field) const;
 
+    /// \brief Sets a reference field of this RefMaker to reference a different target.
+    /// \param field The descriptor of a reference field defined in this RefMaker derived class.
+    /// \param target The new target to replace the old one with.
+    void setReferenceFieldTarget(const PropertyFieldDescriptor* field, OORef<const RefTarget> target);
+
     /// \brief Returns the number of target objects in a vector reference field of this RefMaker.
     /// \param field The descriptor of a vector reference field defined in this RefMaker derived class.
     /// \return The length of the vector.
@@ -282,7 +272,7 @@ public:
     ///
     /// This method is automatically called right after creation of a new object instance by the OORef<>::create() function.
     /// It loads the initial values for property fields with user-defined default settings (those having the PROPERTY_FIELD_MEMORIZE flag set).
-    void initializeParametersToUserDefaults();
+    void initializeParametersToUserDefaultsNonrecursive();
 
     /// \brief Initializes a new instance and all its children as part of two-phase object initialization.
     void initializeParametersToUserDefaultsRecursive();
@@ -319,22 +309,21 @@ public:
     /// \brief Replaces all references of this RefMaker to some RefTarget with new ones.
     /// \param oldTarget Specifies which references should be replaced.
     /// \param newTarget Specifies the new target that should replace the old one.
-    /// \undoable
     void replaceReferencesTo(const RefTarget* oldTarget, const RefTarget* newTarget);
 
     ///////////////////////////// User interface context ///////////////////////////////
 
     /// Indicates whether the current action being performed should be recorded on the undo stack.
-    static bool isUndoRecording() { return CompoundOperation::isUndoRecording(); }
+    inline static bool isUndoRecording() { return CompoundOperation::isUndoRecording(); }
 
     /// Indicates whether a previously recorded action on the undo stack is currently being undone or redone.
-    static bool isUndoingOrRedoing() { return CompoundOperation::isUndoingOrRedoing(); }
+    inline static bool isUndoingOrRedoing() { return CompoundOperation::isUndoingOrRedoing(); }
 
     /// Pushes an operation onto the undo stack if the undo stack is currently recording.
     /// The undo record class specified as a template parameter is instantiated only if the undo stack is recording.
     template<class UndoableOperationClass, class... Args>
     void pushIfUndoRecording(Args&&... args) {
-        if(isUndoRecording())
+        if(!isBeingInitializedOrDeleted() && isUndoRecording())
             CompoundOperation::current()->addOperation(std::make_unique<UndoableOperationClass>(std::forward<Args>(args)...));
     }
 
