@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -23,6 +23,7 @@
 #include <ovito/core/Core.h>
 #include <ovito/core/utilities/concurrent/Future.h>
 #include <ovito/core/utilities/concurrent/TaskManager.h>
+#include <ovito/core/utilities/concurrent/LaunchTask.h>
 #include <ovito/core/utilities/io/ssh/SshConnection.h>
 #include <ovito/core/utilities/io/ssh/openssh/OpensshConnection.h>
 #ifdef OVITO_SSH_CLIENT
@@ -55,7 +56,7 @@ std::unique_ptr<QIODevice> FileHandle::createIODevice() const
 /******************************************************************************
 * Constructor.
 ******************************************************************************/
-FileManager::FileManager(TaskManager& taskManager) : _taskManager(taskManager)
+FileManager::FileManager()
 {
 }
 
@@ -76,7 +77,7 @@ FileManager::~FileManager()
 ******************************************************************************/
 SharedFuture<FileHandle> FileManager::fetchUrl(const QUrl& url)
 {
-    OVITO_ASSERT(ExecutionContext::current().isValid());
+    OVITO_ASSERT(this_task::get());
 
     if(url.isLocalFile()) {
         // Nothing to do to fetch local files. Simply return a finished Future object.
@@ -99,18 +100,14 @@ SharedFuture<FileHandle> FileManager::fetchUrl(const QUrl& url)
 
         // Check if requested URL is already being loaded.
         if(auto inProgressEntry = _pendingFiles.find(normalizedUrl); inProgressEntry != _pendingFiles.end()) {
-            SharedFuture<FileHandle> future = inProgressEntry->second.lock();
-            if(future.isValid())
+            if(SharedFuture<FileHandle> future = inProgressEntry->second.lock())
                 return future;
             else
                 _pendingFiles.erase(inProgressEntry);
         }
 
         // Start the background download job.
-        DownloadRemoteFileJob* job = new DownloadRemoteFileJob(url);
-        auto future = job->sharedFuture();
-        // Show task progress in the GUI.
-        _taskManager.registerFuture(future);
+        auto future = launchTask(std::make_shared<DownloadRemoteFileJob>(url));
         _pendingFiles.emplace(normalizedUrl, future);
         return future;
     }
@@ -124,12 +121,10 @@ SharedFuture<FileHandle> FileManager::fetchUrl(const QUrl& url)
 ******************************************************************************/
 Future<QStringList> FileManager::listDirectoryContents(const QUrl& url)
 {
-    OVITO_ASSERT(ExecutionContext::current().isValid());
+    OVITO_ASSERT(this_task::get());
 
     if(url.scheme() == QStringLiteral("sftp")) {
-        ListRemoteDirectoryJob* job = new ListRemoteDirectoryJob(url);
-        ExecutionContext::current().ui().taskManager().registerPromise(job->promise());
-        return job->future();
+        return launchTask(std::make_shared<ListRemoteDirectoryJob>(url));
     }
     else if(url.scheme() == QStringLiteral("http") || url.scheme() == QStringLiteral("https")) {
 #ifndef Q_OS_WASM
@@ -175,9 +170,9 @@ void FileManager::removeFromCache(const QUrl& url)
 /******************************************************************************
 * Is called when a remote file has been fetched.
 ******************************************************************************/
-void FileManager::fileFetched(QUrl url, QTemporaryFile* localFile)
+void FileManager::fileFetched(const QUrl& url, QTemporaryFile* localFile)
 {
-    QUrl normalizedUrl = normalizeUrl(std::move(url));
+    QUrl normalizedUrl = normalizeUrl(url);
     QMutexLocker lock(&mutex());
 
     if(auto itemInProgress = _pendingFiles.find(normalizedUrl); itemInProgress != _pendingFiles.end())
@@ -212,7 +207,7 @@ QUrl FileManager::urlFromUserInput(const QString& path)
 ******************************************************************************/
 SshConnection* FileManager::acquireSshConnection(const SshConnectionParameters& sshParams)
 {
-    OVITO_ASSERT(!QCoreApplication::instance() || QThread::currentThread() == QCoreApplication::instance()->thread());
+    OVITO_ASSERT(this_task::isMainThread());
 
     // Determine the kind of ssh connection method to use.
     SshConnection::SshImplementation sshImpl = SshConnection::getSshImplementation();
@@ -264,10 +259,10 @@ SshConnection* FileManager::acquireSshConnection(const SshConnectionParameters&
 ******************************************************************************/
 void FileManager::releaseSshConnection(SshConnection* connection)
 {
-    OVITO_ASSERT(!QCoreApplication::instance() || QThread::currentThread() == QCoreApplication::instance()->thread());
+    OVITO_ASSERT(this_task::isMainThread());
 
-    bool wasAquired = _acquiredConnections.removeOne(connection);
-    OVITO_ASSERT(wasAquired);
+    Q_DECL_UNUSED bool wasAcquired = _acquiredConnections.removeOne(connection);
+    OVITO_ASSERT(wasAcquired);
     if(_acquiredConnections.contains(connection))
         return;
 
@@ -318,13 +313,18 @@ void FileManager::unknownSshServer()
 ******************************************************************************/
 bool FileManager::detectedUnknownSshServer(const QString& hostname, const QString& unknownHostMessage, const QString& hostPublicKeyHash)
 {
-    std::cout << "OVITO is connecting to remote host '" << qPrintable(hostname) << "' via SSH." << std::endl;
-    std::cout << qPrintable(unknownHostMessage) << std::endl;
-    std::cout << "Host key fingerprint is " << qPrintable(hostPublicKeyHash) << std::endl;
-    std::cout << "Are you sure you want to continue connecting (yes/no)? " << std::flush;
-    std::string reply;
-    std::cin >> reply;
-    return reply == "yes";
+    if(_detectedUnknownSshServerImpl) {
+        return _detectedUnknownSshServerImpl(hostname, unknownHostMessage, hostPublicKeyHash);
+    }
+    else {
+        std::cout << "OVITO is connecting to remote host '" << qPrintable(hostname) << "' via SSH." << std::endl;
+        std::cout << qPrintable(unknownHostMessage) << std::endl;
+        std::cout << "Host key fingerprint is " << qPrintable(hostPublicKeyHash) << std::endl;
+        std::cout << "Are you sure you want to continue connecting (yes/no)? " << std::flush;
+        std::string reply;
+        std::cin >> reply;
+        return reply == "yes";
+    }
 }
 
 /******************************************************************************
@@ -391,12 +391,17 @@ void FileManager::needKbiAnswers()
 ******************************************************************************/
 bool FileManager::askUserForPassword(const QString& hostname, const QString& username, QString& password)
 {
-    std::string pw;
-    std::cout << "Please enter the password for user '" << qPrintable(username) << "' ";
-    std::cout << "on SSH remote host '" << qPrintable(hostname) << "' (set echo off beforehand!): " << std::flush;
-    std::cin >> pw;
-    password = QString::fromStdString(pw);
-    return true;
+    if(_askUserForPasswordImpl) {
+        return _askUserForPasswordImpl(hostname, username, password);
+    }
+    else {
+        std::string pw;
+        std::cout << "Please enter the password for user '" << qPrintable(username) << "' ";
+        std::cout << "on SSH remote host '" << qPrintable(hostname) << "' (set echo off beforehand!): " << std::flush;
+        std::cin >> pw;
+        password = QString::fromStdString(pw);
+        return true;
+    }
 }
 
 /******************************************************************************
@@ -404,14 +409,19 @@ bool FileManager::askUserForPassword(const QString& hostname, const QString& use
 ******************************************************************************/
 bool FileManager::askUserForKbiResponse(const QString& hostname, const QString& username, const QString& instruction, const QString& question, bool showAnswer, QString& answer)
 {
-    std::cout << "SSH keyboard interactive authentication";
-    if(!showAnswer)
-        std::cout << " (set echo off beforehand!)";
-    std::cout << " - " << qPrintable(question) << std::flush;
-    std::string pw;
-    std::cin >> pw;
-    answer = QString::fromStdString(pw);
-    return true;
+    if(_askUserForKbiResponseImpl) {
+        return _askUserForKbiResponseImpl(hostname, username, instruction, question, showAnswer, answer);
+    }
+    else {
+        std::cout << "SSH keyboard interactive authentication";
+        if(!showAnswer)
+            std::cout << " (set echo off beforehand!)";
+        std::cout << " - " << qPrintable(question) << std::flush;
+        std::string pw;
+        std::cin >> pw;
+        answer = QString::fromStdString(pw);
+        return true;
+    }
 }
 
 /******************************************************************************
@@ -434,11 +444,16 @@ void FileManager::needSshPassphrase(const QString& prompt)
 ******************************************************************************/
 bool FileManager::askUserForKeyPassphrase(const QString& hostname, const QString& prompt, QString& passphrase)
 {
-    std::string pp;
-    std::cout << qPrintable(prompt) << std::flush;
-    std::cin >> pp;
-    passphrase = QString::fromStdString(pp);
-    return true;
+    if(_askUserForKeyPassphraseImpl) {
+        return _askUserForKeyPassphraseImpl(hostname, prompt, passphrase);
+    }
+    else {
+        std::string pp;
+        std::cout << qPrintable(prompt) << std::flush;
+        std::cin >> pp;
+        passphrase = QString::fromStdString(pp);
+        return true;
+    }
 }
 #endif
 
