@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -22,7 +22,6 @@
 
 #include <ovito/core/Core.h>
 #include <ovito/core/utilities/concurrent/Future.h>
-#include <ovito/core/utilities/concurrent/TaskWatcher.h>
 #include <ovito/core/utilities/io/FileManager.h>
 #include <ovito/core/app/Application.h>
 #ifdef OVITO_SSH_CLIENT
@@ -42,7 +41,7 @@
 namespace Ovito {
 
 /// List SFTP jobs that are waiting to be executed.
-QQueue<RemoteFileJob*> RemoteFileJob::_queuedJobs;
+std::queue<PromiseBase> RemoteFileJob::_queuedJobs;
 
 /// Tracks of how many jobs are currently active.
 int RemoteFileJob::_numActiveJobs = 0;
@@ -53,51 +52,70 @@ constexpr int MaximumNumberOfSimultaneousJobs = 2;
 /******************************************************************************
 * Constructor.
 ******************************************************************************/
-RemoteFileJob::RemoteFileJob(QUrl url, PromiseBase& promise) :
-        _url(std::move(url)), _promise(promise)
+RemoteFileJob::RemoteFileJob(const QUrl& url, void* resultsStorage) :
+    Task(Task::NoState, resultsStorage),
+    _url(url)
 {
-    OVITO_ASSERT(QCoreApplication::instance() != nullptr);
+    // Run all signal handlers of this class in the main thread.
+    moveToThread(Application::instance()->thread());
+}
 
-    // Run all event handlers of this class in the main thread.
-    moveToThread(QCoreApplication::instance()->thread());
-
-    // Start download process in the main thread.
-    QMetaObject::invokeMethod(this, "start", Qt::QueuedConnection);
+/******************************************************************************
+* Begins execution of the task. This function gets invoked by the launchTask() helper.
+******************************************************************************/
+void RemoteFileJob::operator()()
+{
+    // We may not be in the main thread at this point.
+    // Make sure to start the download process in the main thread.
+    Application::instance()->taskManager().submitWork([promise = PromiseBase(shared_from_this())]() mutable noexcept {
+        static_cast<RemoteFileJob*>(promise.task().get())->start(std::move(promise));
+    });
 }
 
 /******************************************************************************
 * Opens the network connection.
 ******************************************************************************/
-void RemoteFileJob::start()
+void RemoteFileJob::start(PromiseBase promise) noexcept
 {
-    if(!_isActive) {
-        // Keep a counter of active jobs.
-        // If there are too many jobs active simultaneously, queue them to be executed later.
-        if(_numActiveJobs >= MaximumNumberOfSimultaneousJobs) {
-            _queuedJobs.enqueue(this);
-            return;
-        }
-        else {
-            _numActiveJobs++;
-            _isActive = true;
-        }
-    }
+    OVITO_ASSERT(this_task::isMainThread());
+    OVITO_ASSERT(!_promise);
 
-    // This background task started to run.
-    _promise.setStarted();
-
-    // Check if process has already been canceled.
-    if(_promise.isCanceled()) {
+    // Check if job has been canceled in the meantime.
+    if(isCanceled()) {
         shutdown(false);
         return;
     }
 
-    // When the user cancels the task, cancel the remote connection.
-    _promise.finally([this](Task& task) noexcept {
+    // We'll need a Qt application object to access the network and perform signal/slot handling.
+    try {
+        Application::instance()->createQtApplication(false);
+    }
+    catch(Exception& ex) {
+        ex.prependGeneralMessage(tr("Network file access is not possible in this environment. A Qt application object could not be created."));
+        captureExceptionAndFinish();
+        return;
+    }
+
+    // Keep a counter of active jobs.
+    // If there are too many jobs active simultaneously, queue them to be executed later.
+    if(_numActiveJobs >= MaximumNumberOfSimultaneousJobs) {
+        _queuedJobs.push(std::move(promise));
+        return;
+    }
+    else {
+        _numActiveJobs++;
+        _promise = std::move(promise);
+    }
+
+    // Handle cancellation of this task.
+    finally(ObjectExecutor(Application::instance()), [this](Task& task) noexcept {
         if(task.isCanceled())
-            QMetaObject::invokeMethod(this, "connectionCanceled");
+            shutdown(false);
     });
 
+    // Show progress in the GUI.
+    _progress.emplace(this->userInterface());
+
     if(_url.scheme() == QStringLiteral("sftp")) {
         // Handle sftp URLs.
 
@@ -106,12 +124,12 @@ void RemoteFileJob::start()
         connectionParams.userName = _url.userName();
         connectionParams.password = _url.password();
         connectionParams.port = _url.port(0);
-        _promise.setProgressText(tr("Connecting to remote host %1").arg(connectionParams.host));
+        _progress->setText(tr("Connecting to remote host %1").arg(connectionParams.host));
 
         // Open connection.
         _connection = Application::instance()->fileManager().acquireSshConnection(connectionParams);
         if(!_connection) {
-            _promise.setException(std::make_exception_ptr(Exception(tr("This particular build of OVITO has no SSH connection support. Please use a different distribution of OVITO to access remote files via SSH."))));
+            setException(std::make_exception_ptr(Exception(tr("This particular build of OVITO has no SSH connection support. Please use a different distribution of OVITO to access remote files via SSH."))));
             shutdown(false);
             return;
         }
@@ -121,7 +139,7 @@ void RemoteFileJob::start()
         connect(_connection, &SshConnection::canceled, this, &RemoteFileJob::connectionCanceled);
         connect(_connection, &SshConnection::allAuthsFailed, this, &RemoteFileJob::authenticationFailed);
         if(_connection->isConnected()) {
-            // The connection may already be established at this point if it was chached by the FileManager.
+            // The connection may already be established at this point if it was cached by the FileManager.
             QTimer::singleShot(0, this, &RemoteFileJob::connectionEstablished);
             return;
         }
@@ -133,7 +151,7 @@ void RemoteFileJob::start()
     else {
         // Handle http(s) URLs.
 #ifndef Q_OS_WASM
-        _promise.setProgressText(tr("Downloading file %1 from %2").arg(_url.fileName()).arg(_url.host()));
+        _progress->setText(tr("Downloading file %1 from %2").arg(_url.fileName()).arg(_url.host()));
         QNetworkAccessManager* networkAccessManager = Application::instance()->networkAccessManager();
         _networkReply = networkAccessManager->get(QNetworkRequest(_url));
 
@@ -148,6 +166,8 @@ void RemoteFileJob::start()
 ******************************************************************************/
 void RemoteFileJob::shutdown(bool success)
 {
+    OVITO_ASSERT(this_task::isMainThread());
+
     if(_connection) {
         disconnect(_connection, nullptr, this, nullptr);
         Application::instance()->fileManager().releaseSshConnection(_connection);
@@ -161,27 +181,20 @@ void RemoteFileJob::shutdown(bool success)
         _networkReply = nullptr;
     }
 #endif
-    _promise.setFinished();
+    _progress.reset();
+    PromiseBase promise = std::move(_promise); // Extend lifetime to the end of this function. Otherwise, task may be destroyed by the setFinished() call.
+    setFinished();
 
     // Update the counter of active jobs.
-    if(_isActive) {
+    if(promise) {
         _numActiveJobs--;
-        _isActive = false;
-    }
 
-    // Schedule this object for deletion.
-    deleteLater();
-
-    // If there jobs waiting in the queue, execute next job.
-    if(!_queuedJobs.isEmpty() && _numActiveJobs < MaximumNumberOfSimultaneousJobs) {
-        RemoteFileJob* waitingJob = _queuedJobs.dequeue();
-        if(!waitingJob->_promise.isCanceled()) {
-            waitingJob->start();
-        }
-        else {
-            // Skip canceled jobs.
-            waitingJob->_promise.setStarted();
-            waitingJob->shutdown(false);
+        // If there jobs waiting in the queue, execute next job.
+        while(!_queuedJobs.empty() && _numActiveJobs < MaximumNumberOfSimultaneousJobs) {
+            PromiseBase promise = std::move(_queuedJobs.front());
+            _queuedJobs.pop();
+            RemoteFileJob* job = static_cast<RemoteFileJob*>(promise.task().get());
+            job->start(std::move(promise));
         }
     }
 }
@@ -193,7 +206,7 @@ void RemoteFileJob::connectionError()
 {
     QStringList errorMessages = _connection->errorMessages();
     if(errorMessages.size() != 0) {
-        if(Application::instance()->guiMode()) {
+        if(isInteractive()) {
             errorMessages[0] = tr("<p>Cannot access URL:</p><p><i>%1</i></p><p>SSH connection error: %2</p><p>See <a href=\"https://docs.ovito.org/advanced_topics/remote_file_access.html#troubleshooting-information\">troubleshooting information</a>.</p>")
                 .arg(_url.toString(QUrl::RemovePassword | QUrl::PreferLocalFile | QUrl::PrettyDecoded).toHtmlEscaped())
                 .arg(errorMessages[0].toHtmlEscaped());
@@ -206,7 +219,7 @@ void RemoteFileJob::connectionError()
         }
     }
 
-    _promise.setException(std::make_exception_ptr(Exception(std::move(errorMessages))));
+    setException(std::make_exception_ptr(Exception(std::move(errorMessages))));
 
     shutdown(false);
 }
@@ -216,19 +229,19 @@ void RemoteFileJob::connectionError()
 ******************************************************************************/
 void RemoteFileJob::authenticationFailed()
 {
-    _promise.setException(std::make_exception_ptr(
+    setException(std::make_exception_ptr(
         Exception(tr("Cannot access URL\n\n%1\n\nSSH authentication failed").arg(_url.toString(QUrl::RemovePassword | QUrl::PreferLocalFile | QUrl::PrettyDecoded)))));
 
     shutdown(false);
 }
 
 /******************************************************************************
-* Handles cancelation by the user.
+* Handles cancellation by the user.
 ******************************************************************************/
 void RemoteFileJob::connectionCanceled()
 {
     // If user has canceled the connection, abort the file retrieval operation as well.
-    _promise.setException(std::make_exception_ptr(Exception(tr("SSH connection was canceled by the user"))));
+    setException(std::make_exception_ptr(Exception(tr("SSH connection was canceled by the user"))));
     shutdown(false);
 }
 
@@ -242,7 +255,7 @@ void RemoteFileJob::networkReplyFinished()
         shutdown(true);
     }
     else {
-        _promise.setException(std::make_exception_ptr(
+        setException(std::make_exception_ptr(
             Exception(tr("Cannot access URL\n\n%1\n\n%2").arg(_url.toString(QUrl::RemovePassword | QUrl::PreferLocalFile | QUrl::PrettyDecoded)).
                 arg(_networkReply->errorString()))));
 
@@ -256,8 +269,8 @@ void RemoteFileJob::networkReplyFinished()
 ******************************************************************************/
 void DownloadRemoteFileJob::channelClosed()
 {
-    if(!_promise.isFinished()) {
-        _promise.setException(std::make_exception_ptr(
+    if(!isFinished()) {
+        setException(std::make_exception_ptr(
             Exception(tr("Failed to download URL\n\n%1\n\nSSH channel was closed unexpectedly.")
                 .arg(_url.toString(QUrl::RemovePassword | QUrl::PreferLocalFile | QUrl::PrettyDecoded)))));
     }
@@ -270,14 +283,16 @@ void DownloadRemoteFileJob::channelClosed()
 ******************************************************************************/
 void DownloadRemoteFileJob::connectionEstablished()
 {
-    if(_promise.isCanceled()) {
+    if(isCanceled()) {
         shutdown(false);
         return;
     }
 
+    Task::Scope taskScope(this);
+
 #ifdef OVITO_SSH_CLIENT
     if(LibsshConnection* libsshConnection = qobject_cast<LibsshConnection*>(_connection)) {
-        _promise.setProgressText(tr("Opening SCP channel to remote host %1").arg(libsshConnection->hostname()));
+        _progress->setText(tr("Opening SCP channel to remote host %1").arg(libsshConnection->hostname()));
         ScpChannel* scpChannel = new ScpChannel(libsshConnection, _url.path());
         connect(scpChannel, &ScpChannel::receivingFile, this, &DownloadRemoteFileJob::receivingFile);
         connect(scpChannel, &ScpChannel::receivedData, this, &DownloadRemoteFileJob::receivedData);
@@ -290,7 +305,7 @@ void DownloadRemoteFileJob::connectionEstablished()
     }
 #endif
     if(OpensshConnection* opensshConnection = qobject_cast<OpensshConnection*>(_connection)) {
-        _promise.setProgressText(tr("Opening download channel to remote host %1").arg(opensshConnection->hostname()));
+        _progress->setText(tr("Opening download channel to remote host %1").arg(opensshConnection->hostname()));
         DownloadRequest* downloadRequest = new DownloadRequest(opensshConnection, _url.path());
         connect(downloadRequest, &DownloadRequest::receivingFile, this, &DownloadRemoteFileJob::receivingFile);
         connect(downloadRequest, &DownloadRequest::receivedData, this, &DownloadRemoteFileJob::receivedData);
@@ -301,7 +316,7 @@ void DownloadRemoteFileJob::connectionEstablished()
         downloadRequest->submit();
         return;
     }
-    _promise.setException(std::make_exception_ptr(Exception(tr("No SSH client implementation available."))));
+    setException(std::make_exception_ptr(Exception(tr("No SSH client implementation available."))));
     shutdown(false);
 }
 
@@ -310,7 +325,7 @@ void DownloadRemoteFileJob::connectionEstablished()
 ******************************************************************************/
 void DownloadRemoteFileJob::channelError(const QString& errorMessage)
 {
-    _promise.setException(std::make_exception_ptr(
+    setException(std::make_exception_ptr(
         Exception(tr("Cannot access remote URL\n\n%1\n\n%2")
             .arg(_url.toString(QUrl::RemovePassword | QUrl::PreferLocalFile | QUrl::PrettyDecoded))
             .arg(errorMessage))));
@@ -329,11 +344,12 @@ void DownloadRemoteFileJob::shutdown(bool success)
 
     if(_localFile && success) {
         _localFile->flush();
-        _promise.setResults(FileHandle(url(), _localFile->fileName()));
+        setResult<FileHandle>(FileHandle(url(), _localFile->fileName()));
     }
     else {
         _localFile.reset();
     }
+    TaskPtr self = shared_from_this(); // Keep alive until the end of this function. Otherwise, task may be destroyed by the shutdown() call.
 
     // Close network connection.
     RemoteFileJob::shutdown(success);
@@ -347,12 +363,12 @@ void DownloadRemoteFileJob::shutdown(bool success)
 ******************************************************************************/
 void DownloadRemoteFileJob::receivingFile(qint64 fileSize)
 {
-    if(_promise.isCanceled()) {
+    if(isCanceled()) {
         shutdown(false);
         return;
     }
-    _promise.setProgressMaximum(fileSize);
-    _promise.setProgressText(tr("Fetching remote file %1").arg(_url.toString(QUrl::RemovePassword | QUrl::PreferLocalFile | QUrl::PrettyDecoded)));
+    _progress->setMaximum(fileSize);
+    _progress->setText(tr("Fetching remote file %1").arg(_url.toString(QUrl::RemovePassword | QUrl::PreferLocalFile | QUrl::PrettyDecoded)));
 }
 
 /******************************************************************************
@@ -360,7 +376,7 @@ void DownloadRemoteFileJob::receivingFile(qint64 fileSize)
 ******************************************************************************/
 void DownloadRemoteFileJob::receivedFileComplete(std::unique_ptr<QTemporaryFile>* localFile)
 {
-    if(_promise.isCanceled()) {
+    if(isCanceled()) {
         shutdown(false);
         return;
     }
@@ -373,11 +389,11 @@ void DownloadRemoteFileJob::receivedFileComplete(std::unique_ptr<QTemporaryFile>
 ******************************************************************************/
 void DownloadRemoteFileJob::receivedData(qint64 totalReceivedBytes)
 {
-    if(_promise.isCanceled()) {
+    if(isCanceled()) {
         shutdown(false);
         return;
     }
-    _promise.setProgressValue(totalReceivedBytes);
+    _progress->setValueNoCancel(totalReceivedBytes);
 }
 
 /******************************************************************************
@@ -385,13 +401,13 @@ void DownloadRemoteFileJob::receivedData(qint64 totalReceivedBytes)
 ******************************************************************************/
 void DownloadRemoteFileJob::networkReplyDownloadProgress(qint64 bytesReceived, qint64 bytesTotal)
 {
-    if(_promise.isCanceled()) {
+    if(isCanceled()) {
         shutdown(false);
         return;
     }
     if(bytesTotal > 0) {
-        _promise.setProgressMaximum(bytesTotal);
-        _promise.setProgressValue(bytesReceived);
+        _progress->setMaximum(bytesTotal);
+        _progress->setValueNoCancel(bytesReceived);
     }
     storeReceivedData();
 }
@@ -425,7 +441,7 @@ void DownloadRemoteFileJob::storeReceivedData()
             throw Exception(tr("Failed to write received data to temporary local file '%1': %2").arg(_localFile->fileName()).arg(_localFile->errorString()));
     }
     catch(Exception&) {
-        _promise.captureException();
+        captureException();
         shutdown(false);
     }
 #endif
@@ -436,15 +452,17 @@ void DownloadRemoteFileJob::storeReceivedData()
 ******************************************************************************/
 void ListRemoteDirectoryJob::connectionEstablished()
 {
-    if(_promise.isCanceled()) {
+    if(isCanceled()) {
         shutdown(false);
         return;
     }
 
+    Task::Scope taskScope(this);
+
 #ifdef OVITO_SSH_CLIENT
     if(LibsshConnection* libsshConnection = qobject_cast<LibsshConnection*>(_connection)) {
         // Open the LS channel.
-        _promise.setProgressText(tr("Opening channel to remote host %1").arg(libsshConnection->hostname()));
+        _progress->setText(tr("Opening channel to remote host %1").arg(libsshConnection->hostname()));
         LsChannel* lsChannel = new LsChannel(libsshConnection, _url.path());
         connect(lsChannel, &LsChannel::error, this, &ListRemoteDirectoryJob::channelError);
         connect(lsChannel, &LsChannel::receivingDirectory, this, &ListRemoteDirectoryJob::receivingDirectory);
@@ -456,7 +474,7 @@ void ListRemoteDirectoryJob::connectionEstablished()
     }
 #endif
     if(OpensshConnection* opensshConnection = qobject_cast<OpensshConnection*>(_connection)) {
-        _promise.setProgressText(tr("Opening channel to remote host %1").arg(opensshConnection->hostname()));
+        _progress->setText(tr("Opening channel to remote host %1").arg(opensshConnection->hostname()));
         FileListingRequest* listingRequest = new FileListingRequest(opensshConnection, _url.path());
         connect(listingRequest, &FileListingRequest::error, this, &ListRemoteDirectoryJob::channelError);
         connect(listingRequest, &FileListingRequest::receivingDirectory, this, &ListRemoteDirectoryJob::receivingDirectory);
@@ -466,7 +484,7 @@ void ListRemoteDirectoryJob::connectionEstablished()
         listingRequest->submit();
         return;
     }
-    _promise.setException(std::make_exception_ptr(Exception(tr("No SSH client implementation available."))));
+    setException(std::make_exception_ptr(Exception(tr("No SSH client implementation available."))));
     shutdown(false);
 }
 
@@ -475,13 +493,12 @@ void ListRemoteDirectoryJob::connectionEstablished()
 ******************************************************************************/
 void ListRemoteDirectoryJob::receivingDirectory()
 {
-    if(_promise.isCanceled()) {
+    if(isCanceled()) {
         shutdown(false);
         return;
     }
 
-    // Set progress text.
-    _promise.setProgressText(tr("Listing remote directory %1").arg(_url.toString(QUrl::RemovePassword | QUrl::PreferLocalFile | QUrl::PrettyDecoded)));
+    _progress->setText(tr("Listing remote directory %1").arg(_url.toString(QUrl::RemovePassword | QUrl::PreferLocalFile | QUrl::PrettyDecoded)));
 }
 
 /******************************************************************************
@@ -489,7 +506,7 @@ void ListRemoteDirectoryJob::receivingDirectory()
 ******************************************************************************/
 void ListRemoteDirectoryJob::channelError(const QString& errorMessage)
 {
-    _promise.setException(std::make_exception_ptr(
+    setException(std::make_exception_ptr(
         Exception(tr("Cannot access remote location:\n\n%1\n\n%2")
             .arg(_url.toString(QUrl::RemovePassword | QUrl::PreferLocalFile | QUrl::PrettyDecoded))
             .arg(errorMessage))));
@@ -502,12 +519,12 @@ void ListRemoteDirectoryJob::channelError(const QString& errorMessage)
 ******************************************************************************/
 void ListRemoteDirectoryJob::receivedDirectoryComplete(const QStringList& listing)
 {
-    if(_promise.isCanceled()) {
+    if(isCanceled()) {
         shutdown(false);
         return;
     }
 
-    _promise.setResults(listing);
+    setResult<QStringList>(listing);
     shutdown(true);
 }
 
@@ -516,8 +533,8 @@ void ListRemoteDirectoryJob::receivedDirectoryComplete(const QStringList& listin
 ******************************************************************************/
 void ListRemoteDirectoryJob::channelClosed()
 {
-    if(!_promise.isFinished()) {
-        _promise.setException(std::make_exception_ptr(
+    if(!isFinished()) {
+        setException(std::make_exception_ptr(
             Exception(tr("Failed to list contents of:\n\n%1\n\nSSH channel was closed unexpectedly.")
                 .arg(_url.toString(QUrl::RemovePassword | QUrl::PreferLocalFile | QUrl::PrettyDecoded)))));
     }
