@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -33,8 +33,8 @@ OVITO_STATIC_ASSERT((std::is_same_v<ZlibByte, unsigned char>));
 /// Constructor
 GzipIODevice::GzipIODevice(QIODevice* device, int bufferSize, int compressionLevel) :
     _device(device),
-    _bufferSize(bufferSize),
-    _compressionLevel(compressionLevel)
+    _compressionLevel(compressionLevel),
+    _bufferSize(bufferSize)
 {
     std::memset(&_zlibStream, 0, sizeof(_zlibStream));
 }
@@ -49,7 +49,8 @@ GzipIODevice::~GzipIODevice()
 void GzipIODevice::lookupGzipIndex(bool createIfNeeded)
 {
     OVITO_ASSERT(!_index);
-    _index = Application::instance()->fileManager().lookupGzipIndex(_device, createIfNeeded);
+    if(_enableSeekIndex)
+        _index = Application::instance()->fileManager().lookupGzipIndex(_device, createIfNeeded);
 }
 
 /// Makes the device generate an index, which will enable random access to the
@@ -80,7 +81,7 @@ bool GzipIODevice::setUnderlyingDevice(QIODevice* device)
 
 /*!
     Opens the GzipIODevice in \a mode. Only ReadOnly and WriteOnly is supported.
-    This functon will return false if you try to open in other modes.
+    This function will return false if you try to open in other modes.
 
     If the underlying device is not opened, this function will open it in a suitable mode. If this happens
     the device will also be closed when close() is called.
@@ -187,7 +188,7 @@ void GzipIODevice::close()
     // Flush and close the zlib stream.
     if(openMode() & ReadOnly) {
         _state = NotReadFirstByte;
-        int status = ::inflateEnd(&_zlibStream);
+        Q_DECL_UNUSED int status = ::inflateEnd(&_zlibStream);
         OVITO_ASSERT(status == Z_OK);
     }
     else {
@@ -195,7 +196,7 @@ void GzipIODevice::close()
             _state = NoBytesWritten;
             flushZlib(Z_FINISH);
         }
-        int status = ::deflateEnd(&_zlibStream);
+        Q_DECL_UNUSED int status = ::deflateEnd(&_zlibStream);
         OVITO_ASSERT(status == Z_OK);
     }
 
@@ -248,7 +249,7 @@ void GzipIODevice::flushZlib(int flushMode)
         OVITO_ASSERT(status == Z_OK);
 }
 
-// Writes outputSize bytes from buffer to the inderlying device.
+// Writes outputSize bytes from buffer to the underlying device.
 bool GzipIODevice::writeBytes(qint64 outputSize)
 {
     ZlibSize totalBytesWritten = 0;
@@ -352,6 +353,7 @@ qint64 GzipIODevice::readData(char* data, qint64 maxSize)
     _zlibStream.avail_out = maxSize;
 
     int status;
+    bool reachedEnd = false;
     do {
         // Read data if the input buffer is empty. There could be data in the buffer
         // from a previous readData call.
@@ -366,6 +368,13 @@ qint64 GzipIODevice::readData(char* data, qint64 maxSize)
                 return -1;
             }
 
+            // If the underlying file stream is still opened for writing by the simulation code, QIODevice::read() does not return -1 when the end of the file is reached.
+            // To not wait for more data indefinitely, we should check if the underlying device is at the end of the file.
+            // This explicit check seems to be necessary only for zstd streams.
+            if(bytesAvailable == 0 && _device->atEnd()) {
+                reachedEnd = true;
+            }
+
             if(_state != InStream) {
                 // If we are not in a stream and get 0 bytes, we are probably trying to read from an empty device.
                 if(bytesAvailable == 0)
@@ -387,16 +396,19 @@ qint64 GzipIODevice::readData(char* data, qint64 maxSize)
             case Z_BUF_ERROR: // No more input and zlib can not provide more output - Not an error, we can try to read again when we have more input.
                 return 0;
         }
-    // Loop until data buffer is full or we reach the end of the input stream.
+        // Loop until data buffer is full or we reach the end of the input stream.
     }
-    while(_zlibStream.avail_out != 0 && status != Z_STREAM_END);
+    while(_zlibStream.avail_out != 0 && status != Z_STREAM_END && !reachedEnd);
 
     if(status == Z_STREAM_END) {
         _state = EndOfStream;
-
         // Unget any data left in the read buffer.
-        for(int i = _zlibStream.avail_in;  i >= 0; --i)
-            _device->ungetChar(*reinterpret_cast<char*>(_zlibStream.next_in + i));
+        for(int i = _zlibStream.avail_in; i >= 0; --i)
+            _device->ungetChar(*reinterpret_cast<const char*>(_zlibStream.next_in + i));
+    }
+    else if(reachedEnd) {
+        // Reached premature end of compressed data stream. The file may be truncated.
+        _state = EndOfStream;
     }
 
     return maxSize - _zlibStream.avail_out;
