@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -84,15 +84,15 @@ public:
 
     /// \brief Empty default constructor that does not initialize the matrix elements (for performance reasons).
     ///        The matrix elements will have an undefined value and need to be initialized later.
-    AffineTransformationT() = default;
+    AffineTransformationT() noexcept = default;
 
     /// \brief Constructor that initializes 9 elements of the left 3x3 submatrix to the given values.
     ///        The translation (4th column) is set to zero.
     /// \note Matrix elements are specified in row-major order, i.e. row by row.
-    Q_DECL_CONSTEXPR AffineTransformationT(
+    constexpr AffineTransformationT(
                         T m11, T m12, T m13,
                         T m21, T m22, T m23,
-                        T m31, T m32, T m33)
+                        T m31, T m32, T m33) noexcept
         : std::array<Vector_3<T>,4>{{
             Vector_3<T>(m11,m21,m31),
             Vector_3<T>(m12,m22,m32),
@@ -101,10 +101,10 @@ public:
 
     /// \brief Constructor that initializes the elements of the matrix to the given values.
     /// \note Elements are specified in row-major order, i.e. row by row.
-    Q_DECL_CONSTEXPR AffineTransformationT(
+    constexpr AffineTransformationT(
                         T m11, T m12, T m13, T m14,
                         T m21, T m22, T m23, T m24,
-                        T m31, T m32, T m33, T m34)
+                        T m31, T m32, T m33, T m34) noexcept
         : std::array<Vector_3<T>,4>{{
             Vector_3<T>(m11,m21,m31),
             Vector_3<T>(m12,m22,m32),
@@ -112,12 +112,12 @@ public:
             Vector_3<T>(m14,m24,m34)}} {}
 
     /// \brief Constructor that initializes the matrix from four column vectors.
-    Q_DECL_CONSTEXPR AffineTransformationT(const column_type& c1, const column_type& c2, const column_type& c3, const column_type& c4)
+    constexpr AffineTransformationT(const column_type& c1, const column_type& c2, const column_type& c3, const column_type& c4) noexcept
         : std::array<Vector_3<T>,4>{{c1, c2, c3, c4}} {}
 
     /// \brief Initializes the matrix to the null matrix.
     /// All matrix elements are set to zero by this constructor.
-    Q_DECL_CONSTEXPR AffineTransformationT(Zero)
+    constexpr AffineTransformationT(Zero) noexcept
         : std::array<Vector_3<T>,4>{{
             typename Vector_3<T>::Zero(),
             typename Vector_3<T>::Zero(),
@@ -126,7 +126,7 @@ public:
 
     /// \brief Initializes the matrix to the identity matrix.
     /// All diagonal elements are set to one and all off-diagonal elements are set to zero.
-    Q_DECL_CONSTEXPR AffineTransformationT(Identity)
+    constexpr AffineTransformationT(Identity) noexcept
         : std::array<Vector_3<T>,4>{{
             Vector_3<T>(T(1),T(0),T(0)),
             Vector_3<T>(T(0),T(1),T(0)),
@@ -135,11 +135,11 @@ public:
 
     /// \brief Initializes the 3x4 matrix from a 3x3 matrix.
     /// The translation vector (4th column) is set to zero.
-    explicit Q_DECL_CONSTEXPR AffineTransformationT(const Matrix_3<T>& tm)
+    explicit constexpr AffineTransformationT(const Matrix_3<T>& tm) noexcept
         : std::array<Vector_3<T>,4>{{tm.column(0), tm.column(1), tm.column(2), typename Vector_3<T>::Zero()}} {}
 
     /// Conversion constructor from a Qt matrix.
-    Q_DECL_CONSTEXPR AffineTransformationT(const QMatrix4x4& m)
+    constexpr AffineTransformationT(const QMatrix4x4& m) noexcept
         : std::array<Vector_3<T>,4>{{
             Vector_3<T>(m(0,0),m(1,0),m(2,0)),
             Vector_3<T>(m(0,1),m(1,1),m(2,1)),
@@ -148,7 +148,7 @@ public:
 
     /// \brief Casts the matrix to a matrix with another data type.
     template<typename U>
-    Q_DECL_CONSTEXPR auto toDataType() const -> std::conditional_t<!std::is_same_v<T,U>, AffineTransformationT<U>, const AffineTransformationT<T>&> {
+    constexpr auto toDataType() const noexcept -> std::conditional_t<!std::is_same_v<T,U>, AffineTransformationT<U>, const AffineTransformationT<T>&> {
         if constexpr(!std::is_same_v<T,U>)
             return AffineTransformationT<U>(
                 static_cast<U>((*this)(0,0)), static_cast<U>((*this)(0,1)), static_cast<U>((*this)(0,2)), static_cast<U>((*this)(0,3)),
@@ -159,67 +159,67 @@ public:
     }
 
     /// \brief Returns the number of rows of this matrix.
-    static Q_DECL_CONSTEXPR size_type row_count() { return 3; }
+    static constexpr size_type row_count() { return 3; }
 
     /// \brief Returns the number of columns of this matrix.
-    static Q_DECL_CONSTEXPR size_type col_count() { return 4; }
+    static constexpr size_type col_count() { return 4; }
 
     /// \brief Returns a matrix element.
     /// \param row The row of the element.
     /// \param col The column of the element.
     /// \return The value of the matrix element.
-    inline Q_DECL_CONSTEXPR T operator()(size_type row, size_type col) const {
+    inline constexpr T operator()(size_type row, size_type col) const {
         return (*this)[col][row];
     }
 
     /// \brief Returns a modifiable reference to a matrix element.
     /// \param row The row of the element.
     /// \param col The column of the element.
-    inline Q_DECL_CONSTEXPR T& operator()(size_type row, size_type col) {
+    inline constexpr T& operator()(size_type row, size_type col) {
         return (*this)[col][row];
     }
 
     /// \brief Returns a column vector of the matrix.
     /// \param col The index of the column.
     /// \return A vector of three elements.
-    inline Q_DECL_CONSTEXPR const column_type& column(size_type col) const {
+    inline constexpr const column_type& column(size_type col) const {
         return (*this)[col];
     }
 
     /// \brief Returns a modifiable reference to a column vector of the matrix.
     /// \param col The column to return.
     /// \return A reference to a vector of three elements.
-    inline Q_DECL_CONSTEXPR column_type& column(size_type col) {
+    inline constexpr column_type& column(size_type col) {
         return (*this)[col];
     }
 
     /// \brief Returns a row of the matrix.
     /// \param row The row to return.
     /// \return A vector of four elements.
-    Q_DECL_CONSTEXPR Vector_4<T> row(size_type row) const {
+    constexpr Vector_4<T> row(size_type row) const {
         return { (*this)[0][row], (*this)[1][row], (*this)[2][row], (*this)[3][row] };
     }
 
     /// \brief Returns the translational part of the transformation, which is stored in the fourth column.
-    Q_DECL_CONSTEXPR const column_type& translation() const { return column(3); }
+    constexpr const column_type& translation() const { return column(3); }
 
     /// \brief Returns a modifiable reference to the translational part of the transformation, which is stored in the fourth column.
-    Q_DECL_CONSTEXPR column_type& translation() { return column(3); }
+    constexpr column_type& translation() { return column(3); }
 
     /// \brief Sets all elements of the matrix to zero.
-    Q_DECL_CONSTEXPR void setZero() {
+    constexpr void setZero() {
         for(size_type i = 0; i < col_count(); i++)
             (*this)[i].setZero();
     }
 
     /// \brief Sets all elements of the matrix to zero.
-    Q_DECL_CONSTEXPR AffineTransformationT& operator=(Zero) {
+    constexpr AffineTransformationT& operator=(Zero) {
         setZero();
         return *this;
     }
 
     /// \brief Sets the matrix to the identity matrix.
-    Q_DECL_CONSTEXPR void setIdentity() {
+    constexpr void setIdentity() {
         (*this)[0] = Vector_3<T>(T(1),T(0),T(0));
         (*this)[1] = Vector_3<T>(T(0),T(1),T(0));
         (*this)[2] = Vector_3<T>(T(0),T(0),T(1));
@@ -227,19 +227,19 @@ public:
     }
 
     /// \brief Sets the matrix to the identity matrix.
-    Q_DECL_CONSTEXPR AffineTransformationT& operator=(Identity) {
+    constexpr AffineTransformationT& operator=(Identity) {
         setIdentity();
         return *this;
     }
 
     /// Returns a pointer to the 12 elements of the matrix (stored in column-major order).
-    Q_DECL_CONSTEXPR const element_type* elements() const {
+    constexpr const element_type* elements() const {
         OVITO_STATIC_ASSERT(sizeof(*this) == sizeof(element_type)*12);
         return column(0).data();
     }
 
     /// Returns a pointer to the 12 elements of the matrix (stored in column-major order).
-    Q_DECL_CONSTEXPR element_type* elements() {
+    constexpr element_type* elements() {
         OVITO_STATIC_ASSERT(sizeof(*this) == sizeof(element_type)*12);
         return column(0).data();
     }
@@ -248,22 +248,34 @@ public:
 
     /// \brief Compares two matrices for exact equality.
     /// \return true if all elements are equal; false otherwise.
-    Q_DECL_CONSTEXPR bool operator==(const AffineTransformationT& b) const {
+    constexpr bool operator==(const AffineTransformationT& b) const {
         return (b[0] == (*this)[0]) && (b[1] == (*this)[1]) && (b[2] == (*this)[2]) && (b[3] == (*this)[3]);
     }
 
     /// \brief Compares two matrices for inequality.
     /// \return true if not all elements are equal; false if all are equal.
-    Q_DECL_CONSTEXPR bool operator!=(const AffineTransformationT& b) const {
+    constexpr bool operator!=(const AffineTransformationT& b) const {
         return !(*this == b);
     }
 
+    /// \brief Compares this matrix to the zero matrix for exact equality.
+    /// \return true if all matrix elements are zero; false otherwise.
+    constexpr bool operator==(Zero) const {
+        return ((*this)[0] == typename column_type::Zero()) && ((*this)[1] == typename column_type::Zero()) && ((*this)[2] == typename column_type::Zero()) && ((*this)[3] == typename column_type::Zero());
+    }
+
+    /// \brief Compares this matrix to the zero matrix for inequality.
+    /// \return true if any matrix element is non-zero; false otherwise.
+    constexpr bool operator!=(Zero) const {
+        return ((*this)[0] != typename column_type::Zero()) || ((*this)[1] != typename column_type::Zero()) || ((*this)[2] != typename column_type::Zero()) || ((*this)[3] != typename column_type::Zero());
+    }
+
     /// \brief Tests if two matrices are equal within a given tolerance.
     /// \param m The matrix to compare to.
     /// \param tolerance A non-negative threshold for the equality test. The two matrices are considered equal if
     ///        the element-wise differences are all less than this tolerance value.
     /// \return \c true if this matrix is equal to \a m within the given tolerance; \c false otherwise.
-    Q_DECL_CONSTEXPR inline bool equals(const AffineTransformationT& m, T tolerance = FloatTypeEpsilon<T>()) const {
+    constexpr inline bool equals(const AffineTransformationT& m, T tolerance = FloatTypeEpsilon<T>()) const {
         for(size_type i = 0; i < col_count(); i++)
             if(!column(i).equals(m.column(i), tolerance)) return false;
         return true;
@@ -272,7 +284,7 @@ public:
     /// \brief Test if the matrix is zero within a given tolerance.
     /// \param tolerance A non-negative threshold.
     /// \return \c true if the absolute value of each matrix element is all smaller than \a tolerance.
-    Q_DECL_CONSTEXPR inline bool isZero(T tolerance = FloatTypeEpsilon<T>()) const {
+    constexpr inline bool isZero(T tolerance = FloatTypeEpsilon<T>()) const {
         for(size_type i = 0; i < col_count(); i++)
             if(!column(i).isZero(tolerance)) return false;
         return true;
@@ -281,7 +293,7 @@ public:
     ////////////////////////////////// Computations ///////////////////////////////////
 
     /// \brief Computes the determinant of the matrix.
-    Q_DECL_CONSTEXPR inline T determinant() const {
+    constexpr inline T determinant() const {
         return(((*this)[0][0]*(*this)[1][1] - (*this)[0][1]*(*this)[1][0])*((*this)[2][2])
               -((*this)[0][0]*(*this)[1][2] - (*this)[0][2]*(*this)[1][0])*((*this)[2][1])
               +((*this)[0][1]*(*this)[1][2] - (*this)[0][2]*(*this)[1][1])*((*this)[2][0]));
@@ -289,7 +301,7 @@ public:
 
     /// \brief Computes the inverse of the matrix.
     /// \throw Exception if the matrix is not invertible because it is singular.
-    Q_DECL_CONSTEXPR AffineTransformationT inverse() const {
+    constexpr AffineTransformationT inverse() const {
         // Compute inverse of 3x3 sub-matrix.
         // Then multiply with inverse translation.
         T det = determinant();
@@ -319,7 +331,7 @@ public:
     /// \return \c false if the matrix is not invertible because it is singular; \c true if the inverse has been calculated
     ///         and was stored in \a result.
     /// \sa determinant()
-    Q_DECL_CONSTEXPR bool inverse(AffineTransformationT& result, T epsilon = T(1e-16)) const {
+    constexpr bool inverse(AffineTransformationT& result, T epsilon = T(1e-16)) const {
         T det = determinant();
         if(std::abs(det) <= epsilon) return false;
         result = AffineTransformationT(
@@ -350,7 +362,7 @@ public:
     ///
     /// where |V| denotes length of vector V and A*B denotes dot
     /// product of vectors A and B.
-    Q_DECL_CONSTEXPR void orthonormalize() {
+    constexpr void orthonormalize() {
 
         // Compute q0.
         (*this)[0].normalize();
@@ -375,7 +387,7 @@ public:
     /// \param p The point to transform with the matrix. It is implicitly extended to a 4-vector with the last element being 1.
     /// \param index The component (0-2) of the transformed point to return.
     /// \return ((*this)*p)[index]
-    inline Q_DECL_CONSTEXPR T prodrow(const Point_3<T>& p, typename Point_3<T>::size_type index) const {
+    inline constexpr T prodrow(const Point_3<T>& p, typename Point_3<T>::size_type index) const {
         return (*this)[0][index] * p[0] + (*this)[1][index] * p[1] + (*this)[2][index] * p[2] + (*this)[3][index];
     }
 
@@ -383,12 +395,12 @@ public:
     /// \param v The vector to transform with the matrix. It is implicitly extended to a 4-vector with the last element being 0.
     /// \param index The component (0-2) of the transformed vector to return.
     /// \return ((*this)*v)[index]
-    inline Q_DECL_CONSTEXPR T prodrow(const Vector_3<T>& v, typename Vector_3<T>::size_type index) const {
+    inline constexpr T prodrow(const Vector_3<T>& v, typename Vector_3<T>::size_type index) const {
         return (*this)[0][index] * v[0] + (*this)[1][index] * v[1] + (*this)[2][index] * v[2];
     }
 
     /// Returns the upper left 3x3 submatrix of this 3x4 matrix containing only the linear transformation but not the translation.
-    inline Q_DECL_CONSTEXPR const Matrix_3<T>& linear() const {
+    inline constexpr const Matrix_3<T>& linear() const {
         OVITO_STATIC_ASSERT(sizeof(Matrix_3<T>) * 4 / 3 == sizeof(*this));
         return *reinterpret_cast<const Matrix_3<T>*>(this);
     }
@@ -397,7 +409,7 @@ public:
 
     /// \brief Generates a matrix describing a rotation around the X axis.
     /// \param angle The rotation angle in radians.
-    static Q_DECL_CONSTEXPR inline AffineTransformationT rotationX(T angle) {
+    static constexpr inline AffineTransformationT rotationX(T angle) {
         const T c = cos(angle);
         const T s = sin(angle);
         return AffineTransformationT{T(1), T(0), T(0), T(0),
@@ -407,7 +419,7 @@ public:
 
     /// \brief Generates a matrix describing a rotation around the Y axis.
     /// \param angle The rotation angle in radians.
-    static Q_DECL_CONSTEXPR inline AffineTransformationT rotationY(T angle) {
+    static constexpr inline AffineTransformationT rotationY(T angle) {
         const T c = cos(angle);
         const T s = sin(angle);
         return AffineTransformationT{   c, T(0),    s, T(0),
@@ -417,7 +429,7 @@ public:
 
     /// \brief Generates a matrix describing a rotation around the Z axis.
     /// \param angle The rotation angle in radians.
-    static Q_DECL_CONSTEXPR inline AffineTransformationT rotationZ(T angle) {
+    static constexpr inline AffineTransformationT rotationZ(T angle) {
         const T c = cos(angle);
         const T s = sin(angle);
         return AffineTransformationT{   c,   -s, T(0), T(0),
@@ -426,32 +438,40 @@ public:
     }
 
     /// Generates a pure rotation matrix from an axis-angle representation.
-    static Q_DECL_CONSTEXPR AffineTransformationT rotation(const RotationT<T>& rot);
+    static constexpr AffineTransformationT rotation(const RotationT<T>& rot);
 
     /// Generates a pure rotation matrix from a quaternion.
-    static Q_DECL_CONSTEXPR AffineTransformationT rotation(const QuaternionT<T>& q);
+    static constexpr AffineTransformationT rotation(const QuaternionT<T>& q);
 
     /// Generates a pure translation matrix.
-    static Q_DECL_CONSTEXPR AffineTransformationT translation(const Vector_3<T>& t) {
+    static constexpr AffineTransformationT translation(const Vector_3<T>& t) {
         return AffineTransformationT(T(1), T(0), T(0), t.x(),
                                      T(0), T(1), T(0), t.y(),
                                      T(0), T(0), T(1), t.z());
     }
 
-    /// Generates a diagonal scaling matrix.
+    /// Generates a diagonal matrix, i.e., a uniform scaling matrix.
     /// \param s The value of the three diagonal elements.
-    static Q_DECL_CONSTEXPR AffineTransformationT scaling(T s) {
+    static constexpr AffineTransformationT scaling(T s) {
         return AffineTransformationT(
                          s, T(0), T(0), T(0),
-                         T(0),    s, T(0), T(0),
-                         T(0), T(0),    s, T(0));
+                         T(0), s, T(0), T(0),
+                         T(0), T(0), s, T(0));
+    }
+
+    /// Generates a diagonal scaling matrix.
+    static constexpr AffineTransformationT diagonal(T s1, T s2, T s3) {
+        return AffineTransformationT(
+                         s1, T(0), T(0), T(0),
+                         T(0), s2, T(0), T(0),
+                         T(0), T(0), s3, T(0));
     }
 
     /// Generates a pure scaling matrix.
-    static Q_DECL_CONSTEXPR AffineTransformationT scaling(const ScalingT<T>& scaling);
+    static constexpr AffineTransformationT scaling(const ScalingT<T>& scaling);
 
     /// Generates pure shear matrix (shear in the x-y plane).
-    static Q_DECL_CONSTEXPR AffineTransformationT shear(T gammaX, T gammaY) {
+    static constexpr AffineTransformationT shear(T gammaX, T gammaY) {
         return AffineTransformationT(
                          T(1), T(0), gammaX, T(0),
                          T(0), T(1), gammaY, T(0),
@@ -459,7 +479,7 @@ public:
     }
 
     /// Generates a matrix from an OpenGL matrix.
-    static Q_DECL_CONSTEXPR AffineTransformationT fromOpenGL(const T tm[16]) {
+    static constexpr AffineTransformationT fromOpenGL(const T tm[16]) {
         OVITO_ASSERT(tm[3] == 0 && tm[7] == 0 && tm[11] == 0 && tm[15] == 1);
         return AffineTransformationT{tm[0], tm[4], tm[8], tm[12],
                                      tm[1], tm[5], tm[9], tm[13],
@@ -472,7 +492,7 @@ public:
     /// \param upVector A vector specifying the up direction that determines the rotation of the camera
     ///                 around the view axis.
     /// \return A transformation from world space to view space.
-    static Q_DECL_CONSTEXPR AffineTransformationT lookAt(const Point_3<T>& camera, const Point_3<T>& target, const Vector_3<T>& upVector) {
+    static constexpr AffineTransformationT lookAt(const Point_3<T>& camera, const Point_3<T>& target, const Vector_3<T>& upVector) {
         return lookAlong(camera, target - camera, upVector);
     }
 
@@ -482,7 +502,7 @@ public:
     /// \param upVector A vector specifying the up direction that determines the rotation of the camera
     ///                 around the view axis.
     /// \return A transformation from world space to view space.
-    static Q_DECL_CONSTEXPR AffineTransformationT lookAlong(const Point_3<T>& camera, const Vector_3<T>& direction, const Vector_3<T>& upVector) {
+    static constexpr AffineTransformationT lookAlong(const Point_3<T>& camera, const Vector_3<T>& direction, const Vector_3<T>& upVector) {
         auto zaxis = -direction.normalized();
         auto xaxis = upVector.cross(zaxis);
         if(xaxis == typename Vector_3<T>::Zero()) {
@@ -507,7 +527,7 @@ public:
     /// \return \c true if the matrix is orthogonal; \c false otherwise.
     ///
     /// The matrix A is a pure rotation/reflection matrix if A * A^T = I and the translation is zero.
-    Q_DECL_CONSTEXPR bool isOrthogonalMatrix(T epsilon = FloatTypeEpsilon<T>()) const {
+    constexpr bool isOrthogonalMatrix(T epsilon = FloatTypeEpsilon<T>()) const {
         return
             translation().isZero(epsilon) &&
             (std::abs((*this)[0][0]*(*this)[1][0] + (*this)[0][1]*(*this)[1][1] + (*this)[0][2]*(*this)[1][2]) <= epsilon) &&
@@ -524,16 +544,25 @@ public:
     /// The matrix A is a pure rotation matrix if:
     ///   1. det(A) = 1  and
     ///   2. A * A^T = I
-    Q_DECL_CONSTEXPR bool isRotationMatrix(T epsilon = FloatTypeEpsilon<T>()) const {
+    constexpr bool isRotationMatrix(T epsilon = FloatTypeEpsilon<T>()) const {
         return isOrthogonalMatrix(epsilon) && (std::abs(determinant() - T(1)) <= epsilon);
     }
 
     /// \brief Tests whether the matrix is a pure translation matrix.
     /// \return \c true if the matrix is a pure rotation matrix; \c false otherwise.
-    Q_DECL_CONSTEXPR bool isTranslationMatrix() const {
+    constexpr bool isTranslationMatrix() const {
         return linear() == typename Matrix_3<T>::Identity();
     }
 
+    /// \brief Tests whether the matrix describes a pure uniform scaling.
+    /// \return \c true if the matrix is a pure uniform scaling matrix; \c false otherwise.
+    constexpr bool isUniformScaling() const {
+        const auto s = (*this)(0,0);
+        return (*this)(1,1) == s && (*this)(2,2) == s &&
+                (*this)(0,1) == 0 && (*this)(0,2) == 0 && (*this)(1,2) == 0 && (*this)(1,0) == 0 && (*this)(2,0) == 0 && (*this)(2,1) == 0 &&
+                translation() == typename Vector_3<T>::Zero();
+    }
+
     /// \brief Converts the matrix to a Qt 4x4 matrix.
     operator QMatrix4x4() const {
         return QMatrix4x4(
@@ -556,7 +585,7 @@ public:
 /// Computes the product of a 3x4 matrix and a Vector3 (which is automatically extended to a 4-vector with the last element being 0).
 /// \relates AffineTransformationT
 template<typename T>
-inline Q_DECL_CONSTEXPR Vector_3<T> operator*(const AffineTransformationT<T>& m, const Vector_3<T>& v)
+inline constexpr Vector_3<T> operator*(const AffineTransformationT<T>& m, const Vector_3<T>& v)
 {
     return Vector_3<T>{ m(0,0) * v[0] + m(0,1) * v[1] + m(0,2) * v[2],
                         m(1,0) * v[0] + m(1,1) * v[1] + m(1,2) * v[2],
@@ -566,7 +595,7 @@ inline Q_DECL_CONSTEXPR Vector_3<T> operator*(const AffineTransformationT<T>& m,
 /// Computes the product of a 3x4 matrix and a Point3 (which is extended to a 4-vector with the last element being 1).
 /// \relates AffineTransformationT
 template<typename T>
-inline Q_DECL_CONSTEXPR Point_3<T> operator*(const AffineTransformationT<T>& m, const Point_3<T>& p)
+inline constexpr Point_3<T> operator*(const AffineTransformationT<T>& m, const Point_3<T>& p)
 {
     return Point_3<T>{ m(0,0) * p[0] + m(0,1) * p[1] + m(0,2) * p[2] + m(0,3),
                         m(1,0) * p[0] + m(1,1) * p[1] + m(1,2) * p[2] + m(1,3),
@@ -576,7 +605,7 @@ inline Q_DECL_CONSTEXPR Point_3<T> operator*(const AffineTransformationT<T>& m,
 /// Computes the product of two 3x4 matrices. The last row of the extended 4x4 matrix is assumed to be (0,0,0,1).
 /// \relates AffineTransformationT
 template<typename T>
-inline Q_DECL_CONSTEXPR AffineTransformationT<T> operator*(const AffineTransformationT<T>& a, const AffineTransformationT<T>& b)
+inline constexpr AffineTransformationT<T> operator*(const AffineTransformationT<T>& a, const AffineTransformationT<T>& b)
 {
 #if 1
     return AffineTransformationT<T>(
@@ -610,7 +639,7 @@ inline Q_DECL_CONSTEXPR AffineTransformationT<T> operator*(const AffineTransform
 /// Multiplies a 3x4 matrix with a scalar.
 /// \relates AffineTransformationT
 template<typename T>
-inline Q_DECL_CONSTEXPR AffineTransformationT<T> operator*(const AffineTransformationT<T>& a, T s)
+inline constexpr AffineTransformationT<T> operator*(const AffineTransformationT<T>& a, T s)
 {
     return { a.column(0)*s, a.column(1)*s, a.column(2)*s, a.column(3)*s };
 }
@@ -618,7 +647,7 @@ inline Q_DECL_CONSTEXPR AffineTransformationT<T> operator*(const AffineTransform
 /// Multiplies a 3x4 matrix with a scalar.
 /// \relates AffineTransformationT
 template<typename T>
-inline Q_DECL_CONSTEXPR AffineTransformationT<T> operator*(T s, const AffineTransformationT<T>& a)
+inline constexpr AffineTransformationT<T> operator*(T s, const AffineTransformationT<T>& a)
 {
     return a * s;
 }
@@ -626,7 +655,7 @@ inline Q_DECL_CONSTEXPR AffineTransformationT<T> operator*(T s, const AffineTran
 /// Add two 3x4 matrices.
 /// \relates AffineTransformationT
 template<typename T>
-inline Q_DECL_CONSTEXPR AffineTransformationT<T> operator+(const AffineTransformationT<T>& a, const AffineTransformationT<T>& b)
+inline constexpr AffineTransformationT<T> operator+(const AffineTransformationT<T>& a, const AffineTransformationT<T>& b)
 {
     return { a.column(0) + b.column(0), a.column(1) + b.column(1), a.column(2) + b.column(2), a.column(3) + b.column(3) };
 }
@@ -634,7 +663,7 @@ inline Q_DECL_CONSTEXPR AffineTransformationT<T> operator+(const AffineTransform
 /// Incremental addition of two 3x4 matrices.
 /// \relates AffineTransformationT
 template<typename T>
-inline Q_DECL_CONSTEXPR AffineTransformationT<T>& operator+=(AffineTransformationT<T>& a, const AffineTransformationT<T>& b)
+inline constexpr AffineTransformationT<T>& operator+=(AffineTransformationT<T>& a, const AffineTransformationT<T>& b)
 {
     for(size_t col = 0; col < a.col_count(); col++)
         a.column(col) += b.column(col);
@@ -644,7 +673,7 @@ inline Q_DECL_CONSTEXPR AffineTransformationT<T>& operator+=(AffineTransformatio
 /// Incremental subtraction of two 3x4 matrices.
 /// \relates AffineTransformationT
 template<typename T>
-inline Q_DECL_CONSTEXPR AffineTransformationT<T>& operator-=(AffineTransformationT<T>& a, const AffineTransformationT<T>& b)
+inline constexpr AffineTransformationT<T>& operator-=(AffineTransformationT<T>& a, const AffineTransformationT<T>& b)
 {
     for(size_t col = 0; col < a.col_count(); col++)
         a.column(col) -= b.column(col);
@@ -654,7 +683,7 @@ inline Q_DECL_CONSTEXPR AffineTransformationT<T>& operator-=(AffineTransformatio
 /// Subtracts 3x4 matrix from another matrix.
 /// \relates AffineTransformationT
 template<typename T>
-inline Q_DECL_CONSTEXPR AffineTransformationT<T> operator-(const AffineTransformationT<T>& a, const AffineTransformationT<T>& b)
+inline constexpr AffineTransformationT<T> operator-(const AffineTransformationT<T>& a, const AffineTransformationT<T>& b)
 {
     return { a.column(0) - b.column(0), a.column(1) - b.column(1), a.column(2) - b.column(2), a.column(3) - b.column(3) };
 }
@@ -662,7 +691,7 @@ inline Q_DECL_CONSTEXPR AffineTransformationT<T> operator-(const AffineTransform
 /// Computes the product of a 3x3 matrix and a 3x4 matrix.
 /// \relates AffineTransformationT
 template<typename T>
-inline Q_DECL_CONSTEXPR AffineTransformationT<T> operator*(const Matrix_3<T>& a, const AffineTransformationT<T>& b)
+inline constexpr AffineTransformationT<T> operator*(const Matrix_3<T>& a, const AffineTransformationT<T>& b)
 {
 #if 1
     return AffineTransformationT<T>(
@@ -693,7 +722,7 @@ inline Q_DECL_CONSTEXPR AffineTransformationT<T> operator*(const Matrix_3<T>& a,
 /// Computes the product of a 3x4 matrix and a 3x3 matrix.
 /// \relates AffineTransformationT
 template<typename T>
-inline Q_DECL_CONSTEXPR AffineTransformationT<T> operator*(const AffineTransformationT<T>& a, const Matrix_3<T>& b)
+inline constexpr AffineTransformationT<T> operator*(const AffineTransformationT<T>& a, const Matrix_3<T>& b)
 {
 #if 1
     return AffineTransformationT<T>(
@@ -726,7 +755,7 @@ inline Q_DECL_CONSTEXPR AffineTransformationT<T> operator*(const AffineTransform
 
 // Generates a pure rotation matrix around the given axis.
 template<typename T>
-inline Q_DECL_CONSTEXPR AffineTransformationT<T> AffineTransformationT<T>::rotation(const RotationT<T>& rot)
+inline constexpr AffineTransformationT<T> AffineTransformationT<T>::rotation(const RotationT<T>& rot)
 {
     T c = cos(rot.angle());
     T s = sin(rot.angle());
@@ -749,7 +778,7 @@ inline Q_DECL_CONSTEXPR AffineTransformationT<T> AffineTransformationT<T>::rotat
 
 // Generates a pure rotation matrix from a quaternion.
 template<typename T>
-inline Q_DECL_CONSTEXPR AffineTransformationT<T> AffineTransformationT<T>::rotation(const QuaternionT<T>& q)
+inline constexpr AffineTransformationT<T> AffineTransformationT<T>::rotation(const QuaternionT<T>& q)
 {
 #ifdef OVITO_DEBUG
     if(std::abs(q.dot(q) - T(1)) > FloatTypeEpsilon<T>())
@@ -768,7 +797,7 @@ inline Q_DECL_CONSTEXPR AffineTransformationT<T> AffineTransformationT<T>::rotat
 
 // Generates a pure scaling matrix.
 template<typename T>
-inline Q_DECL_CONSTEXPR AffineTransformationT<T> AffineTransformationT<T>::scaling(const ScalingT<T>& scaling)
+inline constexpr AffineTransformationT<T> AffineTransformationT<T>::scaling(const ScalingT<T>& scaling)
 {
     Matrix_3<T> U = Matrix_3<T>::rotation(scaling.Q);
     Matrix_3<T> K = Matrix_3<T>(scaling.S.x(), T(0), T(0),
