@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -22,7 +22,7 @@
 
 #include <ovito/core/Core.h>
 #include <ovito/core/viewport/Viewport.h>
-#include <ovito/core/rendering/RenderSettings.h>
+#include <ovito/core/rendering/FrameGraph.h>
 #include <ovito/core/dataset/DataSet.h>
 #include <ovito/core/utilities/units/UnitsManager.h>
 #include <ovito/core/app/Application.h>
@@ -30,7 +30,8 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(CoordinateTripodOverlay);
+IMPLEMENT_CREATABLE_OVITO_CLASS(CoordinateTripodOverlay);
+OVITO_CLASSINFO(CoordinateTripodOverlay, "DisplayName", "Coordinate tripod");
 DEFINE_PROPERTY_FIELD(CoordinateTripodOverlay, alignment);
 DEFINE_PROPERTY_FIELD(CoordinateTripodOverlay, tripodSize);
 DEFINE_PROPERTY_FIELD(CoordinateTripodOverlay, lineWidth);
@@ -71,34 +72,16 @@ SET_PROPERTY_FIELD_LABEL(CoordinateTripodOverlay, outlineEnabled, "Enable outlin
 SET_PROPERTY_FIELD_LABEL(CoordinateTripodOverlay, perspectiveDistortion, "Perspective distortion");
 SET_PROPERTY_FIELD_UNITS(CoordinateTripodOverlay, offsetX, PercentParameterUnit);
 SET_PROPERTY_FIELD_UNITS(CoordinateTripodOverlay, offsetY, PercentParameterUnit);
-SET_PROPERTY_FIELD_UNITS_AND_MINIMUM(CoordinateTripodOverlay, tripodSize, FloatParameterUnit, 1e-4);
-SET_PROPERTY_FIELD_UNITS_AND_MINIMUM(CoordinateTripodOverlay, lineWidth, FloatParameterUnit, 1e-4);
-SET_PROPERTY_FIELD_UNITS_AND_MINIMUM(CoordinateTripodOverlay, fontSize, FloatParameterUnit, 0);
-
-/******************************************************************************
-* Constructor.
-******************************************************************************/
-CoordinateTripodOverlay::CoordinateTripodOverlay(ObjectInitializationFlags flags) : ViewportOverlay(flags),
-        _alignment(Qt::AlignLeft | Qt::AlignBottom),
-        _tripodSize(0.075), _lineWidth(0.06), _offsetX(0), _offsetY(0),
-        _fontSize(0.4),
-        _axis1Enabled(true), _axis2Enabled(true), _axis3Enabled(true), _axis4Enabled(false),
-        _axis1Label("x"), _axis2Label("y"), _axis3Label("z"), _axis4Label("w"),
-        _axis1Dir(1,0,0), _axis2Dir(0,1,0), _axis3Dir(0,0,1), _axis4Dir(sqrt(0.5),sqrt(0.5),0),
-        _axis1Color(1,0,0), _axis2Color(0,0.8,0), _axis3Color(0.2,0.2,1), _axis4Color(1,0,1),
-        _tripodStyle(FlatArrows),
-        _outlineColor(1,1,1),
-        _outlineEnabled(false),
-        _perspectiveDistortion(false)
-{
-}
+SET_PROPERTY_FIELD_UNITS_AND_RANGE(CoordinateTripodOverlay, tripodSize, FloatParameterUnit, 1e-4, 1);
+SET_PROPERTY_FIELD_UNITS_AND_RANGE(CoordinateTripodOverlay, lineWidth, FloatParameterUnit, 1e-4, 1);
+SET_PROPERTY_FIELD_UNITS_AND_RANGE(CoordinateTripodOverlay, fontSize, FloatParameterUnit, 0, 1);
 
 /******************************************************************************
 * Is called when the value of a property of this object has changed.
 ******************************************************************************/
 void CoordinateTripodOverlay::propertyChanged(const PropertyFieldDescriptor* field)
 {
-    if(field == PROPERTY_FIELD(alignment) && !isBeingLoaded() && !isAboutToBeDeleted() && !isUndoingOrRedoing() && ExecutionContext::isInteractive()) {
+    if(field == PROPERTY_FIELD(alignment) && !isBeingLoaded() && !isBeingDeleted() && !isUndoingOrRedoing() && this_task::isInteractive()) {
         // Automatically reset offset to zero when user changes the alignment of the overlay in the viewport.
         setOffsetX(0);
         setOffsetY(0);
@@ -109,16 +92,20 @@ void CoordinateTripodOverlay::propertyChanged(const PropertyFieldDescriptor* fie
 /******************************************************************************
 * Lets the overlay paint its contents into the framebuffer.
 ******************************************************************************/
-void CoordinateTripodOverlay::render(SceneRenderer* renderer, const QRect& logicalViewportRect, const QRect& physicalViewportRect)
+std::variant<PipelineStatus, Future<PipelineStatus>> CoordinateTripodOverlay::render(FrameGraph& frameGraph, FrameGraph::RenderingCommandGroup& commandGroup, const QRect& logicalViewportRect, const QRect& physicalViewportRect, const ViewProjectionParameters& noninteractiveProjParams, const Scene* scene)
 {
     // Check alignment parameter.
-    checkAlignmentParameterValue(alignment());
+    if(!frameGraph.isInteractive())
+        checkAlignmentParameterValue(alignment());
+
+    if(this->tripodSize() > 1.0)
+        return PipelineStatus(PipelineStatus::Error, tr("Coordinate tripod size is too large. Maximum size factor is 1."));
 
     FloatType tripodSize = this->tripodSize() * physicalViewportRect.height();
-    if(tripodSize <= 0) return;
+    if(tripodSize <= 0) return {};
 
     FloatType lineWidth = this->lineWidth() * tripodSize;
-    if(lineWidth <= 0) return;
+    if(lineWidth <= 0) return {};
 
     FloatType arrowSize = FloatType(0.17);
 
@@ -133,14 +120,12 @@ void CoordinateTripodOverlay::render(SceneRenderer* renderer, const QRect& logic
     else if(alignment() & Qt::AlignBottom) origin.ry() += physicalViewportRect.height() - margin;
     else if(alignment() & Qt::AlignVCenter) origin.ry() += FloatType(0.5) * physicalViewportRect.height();
 
-    const ViewProjectionParameters& projParams = renderer->projParams();
-
     // Project axes to view space.
     Vector3 axisDirs[4] = {
-            projParams.viewMatrix * axis1Dir(),
-            projParams.viewMatrix * axis2Dir(),
-            projParams.viewMatrix * axis3Dir(),
-            projParams.viewMatrix * axis4Dir()
+            noninteractiveProjParams.viewMatrix * axis1Dir(),
+            noninteractiveProjParams.viewMatrix * axis2Dir(),
+            noninteractiveProjParams.viewMatrix * axis3Dir(),
+            noninteractiveProjParams.viewMatrix * axis4Dir()
     };
 
     // Get axis colors.
@@ -167,44 +152,47 @@ void CoordinateTripodOverlay::render(SceneRenderer* renderer, const QRect& logic
             axis3Label(),
             axis4Label()
     };
+
+    if(this->fontSize() > 1.0)
+        return PipelineStatus(PipelineStatus::Error, tr("Coordinate label font size is too large."));
+
     QFont font = this->font();
     qreal fontSize = tripodSize * std::max(0.0, (double)this->fontSize());
     if(fontSize != 0)
-        font.setPointSizeF(fontSize / renderer->devicePixelRatio()); // Font size if always in logical units.
+        font.setPointSizeF(fontSize / frameGraph.devicePixelRatio()); // Font size if always in logical units.
 
     auto renderSolidJoint = [&]() {
         // Look up the image primitive for the axis arrow in the cache.
-        auto& [imagePrimitive, offset] = renderer->visCache().get<std::tuple<ImagePrimitive, QPointF>>(
+        const auto& [image, offset] = frameGraph.visCache().lookup<std::tuple<QImage, QPointF>>(
             RendererResourceKey<struct SolidJointImageCache, Matrix3, FloatType>{
-                projParams.viewMatrix.linear(),
+                noninteractiveProjParams.viewMatrix.linear(),
                 lineWidth
+            },
+            [&](QImage& image, QPointF& offset) {
+                // Compute bounding box of joint.
+                FloatType margin = sqrt(3.0) * lineWidth;
+                QRectF imageRect = QRectF(-margin, -margin, 2*margin, 2*margin);
+
+                // Convert bounding box to physical units.
+                QRect pixelBounds = imageRect.toAlignedRect();
+
+                // Draw the joint into an image buffer, which will be cached.
+                image = QImage(pixelBounds.width(), pixelBounds.height(), frameGraph.preferredImageFormat());
+                image.fill(0);
+                QPainter painter(&image);
+                painter.setRenderHint(QPainter::Antialiasing);
+                painter.setWindow(pixelBounds);
+                paintSolidJoint(painter, QPointF(0,0), noninteractiveProjParams.viewMatrix, lineWidth);
+                painter.end(); // Release the QImage we've been painting into.
+                offset = imageRect.topLeft();
             });
 
-        // Render joint.
-        if(imagePrimitive.image().isNull()) {
-            // Compute bounding box of joint.
-            FloatType margin = sqrt(3.0) * lineWidth;
-            QRectF imageRect = QRectF(-margin, -margin, 2*margin, 2*margin);
-
-            // Convert bounding box to physical units.
-            QRect pixelBounds = imageRect.toAlignedRect();
-
-            // Draw the joint into an image buffer, which will be cached.
-            QImage textureImage(pixelBounds.width(), pixelBounds.height(), renderer->preferredImageFormat());
-            textureImage.fill(0);
-            QPainter painter(&textureImage);
-            painter.setRenderHint(QPainter::Antialiasing);
-            painter.setWindow(pixelBounds);
-            paintSolidJoint(painter, QPointF(0,0), projParams.viewMatrix, lineWidth);
-            painter.end(); // Release the QImage we've been painting into.
-            offset = imageRect.topLeft();
-            imagePrimitive.setImage(std::move(textureImage));
-        }
-
         // Render the prepared image into the output framebuffer.
         QPoint alignedPos = (origin + offset).toPoint();
-        imagePrimitive.setRectWindow(QRect(alignedPos, imagePrimitive.image().size()));
-        renderer->renderImage(imagePrimitive);
+        std::unique_ptr<ImagePrimitive> imagePrimitive = std::make_unique<ImagePrimitive>();
+        imagePrimitive->setImage(image);
+        imagePrimitive->setRectWindow(QRect(alignedPos, image.size()));
+        commandGroup.addPrimitivePreprojected(std::move(imagePrimitive));
     };
 
     // Render axis arrows.
@@ -221,90 +209,90 @@ void CoordinateTripodOverlay::render(SceneRenderer* renderer, const QRect& logic
         Vector2 dir2d(dir3d.x(), dir3d.y());
 
         // Apply perspective distortion to tripod axis.
-        if(perspectiveDistortion() && projParams.isPerspective) {
+        if(perspectiveDistortion() && noninteractiveProjParams.isPerspective) {
             // Calculate the tripod's origin in normalized device coordinates.
             FloatType vporiginX =  (origin.x() - physicalViewportRect.left()) / physicalViewportRect.width()  * 2.0 - 1.0;
             FloatType vporiginY = -(origin.y() - physicalViewportRect.top())  / physicalViewportRect.height() * 2.0 + 1.0;
-            FloatType distance = projParams.zfar;
+            FloatType distance = noninteractiveProjParams.zfar;
             // Calculate the screen-space points on the near and far clipping planes.
-            Point3 p1 = projParams.inverseProjectionMatrix * Point3(vporiginX, vporiginY, 1);
-            Point3 p2 = projParams.inverseProjectionMatrix * Point3(vporiginX, vporiginY, 0);
+            Point3 p1 = noninteractiveProjParams.inverseProjectionMatrix * Point3(vporiginX, vporiginY, 1);
+            Point3 p2 = noninteractiveProjParams.inverseProjectionMatrix * Point3(vporiginX, vporiginY, 0);
             Vector3 rayDir = (p1 - p2).safelyNormalized();
             // View-space position of the tripod's origin.
             Point3 viewSpaceOrigin = Point3::Origin() + rayDir * (distance / -rayDir.z());
             // View-space position of the tripod's tip. Scale the axis length with the distance from the viewer.
-            Point3 viewSpaceTip = viewSpaceOrigin + (std::tan(projParams.fieldOfView / 2) * distance * axisDirs[axis]);
+            Point3 viewSpaceTip = viewSpaceOrigin + (std::tan(noninteractiveProjParams.fieldOfView / 2) * distance * axisDirs[axis]);
             // Screen-space position of the tripod's tip.
-            Point3 screenSpaceTip = projParams.projectionMatrix * viewSpaceTip;
+            Point3 screenSpaceTip = noninteractiveProjParams.projectionMatrix * viewSpaceTip;
             // Screen-space direction of the tripod axis.
             dir2d = Point2(screenSpaceTip.x(), screenSpaceTip.y()) - Point2(vporiginX, vporiginY);
             dir2d.y() = -dir2d.y();
-            dir2d.x() /= projParams.aspectRatio;
+            dir2d.x() /= noninteractiveProjParams.aspectRatio;
             dir2d *= this->tripodSize() * physicalViewportRect.height();
         }
 
         FloatType labelMargin = lineWidth * 2.5;
 
         // Look up the image primitive for the axis arrow in the cache.
-        auto& [imagePrimitive, offset, addedMargin] = renderer->visCache().get<std::tuple<ImagePrimitive, QPointF, FloatType>>(
+        const auto& [image, offset, addedMargin] = frameGraph.visCache().lookup<std::tuple<QImage, QPointF, FloatType>>(
             RendererResourceKey<struct ArrowAxisImageCache, TripodStyle, Vector3, Vector2, FloatType, Color>{
                 tripodStyle(),
                 dir3d,
                 dir2d,
                 lineWidth,
                 axisColors[axis]
+            },
+            [&](QImage& image, QPointF& offset, FloatType& addedMargin) {
+                // Compute bounding box of arrow.
+                QRectF imageRect = QRectF(0, 0, dir2d.x(), dir2d.y()).normalized();
+                FloatType margin = std::max(lineWidth, (tripodStyle() == FlatArrows) ? (arrowSize * tripodSize) : 0.0);
+                imageRect.adjust(-margin, -margin, margin, margin);
+
+                // Convert bounding box to physical units.
+                QRect pixelBounds = imageRect.toAlignedRect();
+
+                // Draw the arrow into an image buffer, which will be cached.
+                image = QImage(pixelBounds.width(), pixelBounds.height(), frameGraph.preferredImageFormat());
+                image.fill(0);
+                QPainter painter(&image);
+                painter.setRenderHint(QPainter::Antialiasing);
+                painter.setWindow(pixelBounds);
+                QBrush brush(axisColors[axis]);
+                QPen pen(axisColors[axis]);
+                pen.setWidthF(lineWidth);
+                pen.setJoinStyle(Qt::MiterJoin);
+                pen.setCapStyle(Qt::RoundCap);
+                painter.setPen(pen);
+                painter.setBrush(brush);
+                if(tripodStyle() == FlatArrows)
+                    addedMargin = paintFlatArrow(painter, dir2d, arrowSize, lineWidth, tripodSize, QPointF(0,0));
+                else if(tripodStyle() == SolidArrows)
+                    addedMargin += paintSolidArrow(painter, dir2d, dir3d, arrowSize, lineWidth, tripodSize, QPointF(0,0));
+                painter.end(); // Release the QImage we've been painting into.
+                offset = imageRect.topLeft();
             });
 
-        // Render axis arrow.
-        if(imagePrimitive.image().isNull()) {
-            // Compute bounding box of arrow.
-            QRectF imageRect = QRectF(0, 0, dir2d.x(), dir2d.y()).normalized();
-            FloatType margin = std::max(lineWidth, (tripodStyle() == FlatArrows) ? (arrowSize * tripodSize) : 0.0);
-            imageRect.adjust(-margin, -margin, margin, margin);
-
-            // Convert bounding box to physical units.
-            QRect pixelBounds = imageRect.toAlignedRect();
-
-            // Draw the arrow into an image buffer, which will be cached.
-            QImage textureImage(pixelBounds.width(), pixelBounds.height(), renderer->preferredImageFormat());
-            textureImage.fill(0);
-            QPainter painter(&textureImage);
-            painter.setRenderHint(QPainter::Antialiasing);
-            painter.setWindow(pixelBounds);
-            QBrush brush(axisColors[axis]);
-            QPen pen(axisColors[axis]);
-            pen.setWidthF(lineWidth);
-            pen.setJoinStyle(Qt::MiterJoin);
-            pen.setCapStyle(Qt::RoundCap);
-            painter.setPen(pen);
-            painter.setBrush(brush);
-            if(tripodStyle() == FlatArrows)
-                addedMargin = paintFlatArrow(painter, dir2d, arrowSize, lineWidth, tripodSize, QPointF(0,0));
-            else if(tripodStyle() == SolidArrows)
-                addedMargin += paintSolidArrow(painter, dir2d, dir3d, arrowSize, lineWidth, tripodSize, QPointF(0,0));
-            painter.end(); // Release the QImage we've been painting into.
-            offset = imageRect.topLeft();
-            imagePrimitive.setImage(std::move(textureImage));
-        }
-
         // Paint the prepared image into the output framebuffer.
         labelMargin += addedMargin;
         QPoint alignedPos = (origin + offset).toPoint();
-        imagePrimitive.setRectWindow(QRect(alignedPos, imagePrimitive.image().size()));
-        renderer->renderImage(imagePrimitive);
+        std::unique_ptr<ImagePrimitive> imagePrimitive = std::make_unique<ImagePrimitive>();
+        imagePrimitive->setImage(image);
+        imagePrimitive->setRectWindow(QRect(alignedPos, image.size()));
+        commandGroup.addPrimitivePreprojected(std::move(imagePrimitive));
 
         // Render axis label.
         if(fontSize != 0 && !labels[axis].isEmpty()) {
-            TextPrimitive textPrimitive;
-            textPrimitive.setText(labels[axis]);
-            textPrimitive.setFont(font);
-            textPrimitive.setColor(axisColors[axis]);
-            if(outlineEnabled()) textPrimitive.setOutlineColor(outlineColor());
-            textPrimitive.setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
-            textPrimitive.setUseTightBox(true);
-            textPrimitive.setTextFormat(Qt::AutoText);
+            std::unique_ptr<TextPrimitive> textPrimitive = std::make_unique<TextPrimitive>();
+            textPrimitive->setText(labels[axis]);
+            textPrimitive->setFont(font);
+            textPrimitive->setColor(axisColors[axis]);
+            if(outlineEnabled())
+                textPrimitive->setOutlineColor(outlineColor());
+            textPrimitive->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
+            textPrimitive->setUseTightBox(true);
+            textPrimitive->setTextFormat(Qt::AutoText);
 
-            QRectF textRect = textPrimitive.queryLocalBounds(renderer->devicePixelRatio());
+            QRectF textRect = textPrimitive->queryLocalBounds(frameGraph.devicePixelRatio());
             textRect.moveTopLeft(QPointF(-textRect.width() / 2, -textRect.height() / 2));
             textRect.translate(origin + QPointF(dir2d.x(), dir2d.y()));
             if(dir2d.isZero() && orderedAxes.size() >= 2) {
@@ -325,14 +313,16 @@ void CoordinateTripodOverlay::render(SceneRenderer* renderer, const QRect& logic
                 dir2d_normalized.resize(labelMargin);
                 textRect.translate(dir2d_normalized.x(), dir2d_normalized.y());
             }
-            textPrimitive.setPositionWindow(Point2(textRect.center().x(), textRect.center().y()));
-            renderer->renderText(textPrimitive);
+            textPrimitive->setPositionWindow(Point2(textRect.center().x(), textRect.center().y()));
+            commandGroup.addPrimitivePreprojected(std::move(textPrimitive));
         }
     }
 
     if(tripodStyle() == SolidArrows && lastZ < 0) {
         renderSolidJoint();
     }
+
+    return {};
 }
 
 /******************************************************************************
