@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -21,9 +21,9 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 
 #include <ovito/crystalanalysis/CrystalAnalysis.h>
-#include <ovito/crystalanalysis/objects/DislocationNetworkObject.h>
+#include <ovito/crystalanalysis/objects/DislocationNetwork.h>
 #include <ovito/crystalanalysis/objects/DislocationVis.h>
-#include <ovito/crystalanalysis/objects/ClusterGraphObject.h>
+#include <ovito/crystalanalysis/objects/ClusterGraph.h>
 #include <ovito/crystalanalysis/modifier/dxa/DislocationAnalysisEngine.h>
 #include <ovito/mesh/surface/SurfaceMesh.h>
 #include <ovito/mesh/surface/SurfaceMeshVis.h>
@@ -33,7 +33,8 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(CAImporter);
+IMPLEMENT_CREATABLE_OVITO_CLASS(CAImporter);
+OVITO_CLASSINFO(CAImporter, "DisplayName", "CA File");
 
 /******************************************************************************
 * Checks if the given file has format that can be read by this importer.
@@ -56,17 +57,19 @@ bool CAImporter::OOMetaClass::checkFileFormat(const FileHandle& file) const
 /******************************************************************************
 * Scans the data file and builds a list of source frames.
 ******************************************************************************/
-void CAImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::Frame>& frames)
+void CAImporter::discoverFramesInFile(const FileHandle& fileHandle, QVector<FileSourceImporter::Frame>& frames) const
 {
-    CompressedTextReader stream(fileHandle());
-    setProgressText(tr("Scanning CA file %1").arg(stream.filename()));
-    setProgressMaximum(stream.underlyingSize());
+    CompressedTextReader stream(fileHandle);
 
-    Frame frame(fileHandle());
-    QString filename = fileHandle().sourceUrl().fileName();
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Scanning CA file %1").arg(stream.filename()));
+    progress.setMaximum(stream.underlyingSize());
+
+    Frame frame(fileHandle);
+    QString filename = fileHandle.sourceUrl().fileName();
     int frameNumber = 0;
 
-    while(!stream.eof() && !isCanceled()) {
+    while(!stream.eof() && !this_task::isCanceled()) {
 
         if(frameNumber == 0) {
             frame.byteOffset = stream.byteOffset();
@@ -87,9 +90,10 @@ void CAImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::F
         while(!stream.eof()) {
             frame.byteOffset = stream.byteOffset();
             stream.readLineTrimLeft();
-            if(stream.lineStartsWith("CA_FILE_VERSION ")) break;
+            if(stream.lineStartsWith("CA_FILE_VERSION "))
+                break;
             if((stream.lineNumber() % 4096) == 0)
-                setProgressValue(stream.underlyingByteOffset());
+                progress.setValue(stream.underlyingByteOffset());
         }
     }
 }
@@ -99,9 +103,11 @@ void CAImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::F
 ******************************************************************************/
 void CAImporter::FrameLoader::loadFile()
 {
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Reading CA file %1").arg(fileHandle().toString()));
+
     // Open file for reading.
     CompressedTextReader stream(fileHandle());
-    setProgressText(tr("Reading CA file %1").arg(fileHandle().toString()));
 
     // Read file header.
     stream.readLine();
@@ -141,8 +147,8 @@ void CAImporter::FrameLoader::loadFile()
     int numClusterTransitions = 0;
     int numDislocationSegments = 0;
     SurfaceMesh* defectSurface = nullptr;
-    ClusterGraphPtr clusterGraph = std::make_shared<ClusterGraph>();
-    std::shared_ptr<DislocationNetwork> dislocations;
+    DataOORef<DislocationNetwork> dislocations;
+    DataOORef<ClusterGraph> clusterGraph = DataOORef<ClusterGraph>::create();
     QVector<PatternInfo> patterns;
 
     while(!stream.eof()) {
@@ -245,11 +251,10 @@ void CAImporter::FrameLoader::loadFile()
             // Read cluster list.
             if(sscanf(stream.line(), "CLUSTERS %i", &numClusters) != 1)
                 throw Exception(tr("Failed to parse file. Invalid number of clusters in line %1.").arg(stream.lineNumber()));
-            setProgressText(tr("Reading clusters"));
-            setProgressMaximum(numClusters);
+            progress.setText(tr("Reading clusters"));
+            progress.setMaximum(numClusters);
             for(int index = 0; index < numClusters; index++) {
-                if(!setProgressValueIntermittent(index))
-                    return;
+                progress.setValueIntermittent(index);
                 if(fileFormatVersion <= 4) {
                     int patternId = 0, clusterId = 0, clusterProc = 0;
                     stream.readLine();
@@ -322,11 +327,10 @@ void CAImporter::FrameLoader::loadFile()
             // Read cluster transition list.
             if(sscanf(stream.line(), "CLUSTER_TRANSITIONS %i", &numClusterTransitions) != 1)
                 throw Exception(tr("Failed to parse file. Invalid number of cluster transitions in line %1.").arg(stream.lineNumber()));
-            setProgressText(tr("Reading cluster transitions"));
-            setProgressMaximum(numClusterTransitions);
+            progress.setText(tr("Reading cluster transitions"));
+            progress.setMaximum(numClusterTransitions);
             for(int index = 0; index < numClusterTransitions; index++) {
-                if(!setProgressValueIntermittent(index))
-                    return;
+                progress.setValueIntermittent(index);
                 int clusterIndex1, clusterIndex2;
                 if(sscanf(stream.readLine(), "TRANSITION %i %i", &clusterIndex1, &clusterIndex2) != 2 || clusterIndex1 >= numClusters || clusterIndex2 >= numClusters) {
                     throw Exception(tr("Failed to parse file. Invalid cluster transition in line %1.").arg(stream.lineNumber()));
@@ -346,12 +350,18 @@ void CAImporter::FrameLoader::loadFile()
             // Read dislocations list.
             if(sscanf(stream.line(), "DISLOCATIONS %i", &numDislocationSegments) != 1)
                 throw Exception(tr("Failed to parse file. Invalid number of dislocation segments in line %1.").arg(stream.lineNumber()));
-            setProgressText(tr("Reading dislocations"));
-            setProgressMaximum(numDislocationSegments);
-            dislocations = std::make_shared<DislocationNetwork>(clusterGraph);
+            progress.setText(tr("Reading dislocations"));
+            progress.setMaximum(numDislocationSegments);
+            if(const DislocationNetwork* existingDislocationsObj = state().getObject<DislocationNetwork>()) {
+                dislocations = state().makeMutable(existingDislocationsObj);
+            }
+            else {
+                dislocations = state().createObject<DislocationNetwork>(pipelineNode());
+            }
+            dislocations->setClusterGraph(clusterGraph);
+            dislocations->segments().clear();
             for(int index = 0; index < numDislocationSegments; index++) {
-                if(!setProgressValueIntermittent(index))
-                    return;
+                progress.setValueIntermittent(index);
                 int segmentId;
                 if(sscanf(stream.readLine(), "%i", &segmentId) != 1)
                     throw Exception(tr("Failed to parse file. Invalid segment ID in line %1.").arg(stream.lineNumber()));
@@ -440,12 +450,12 @@ void CAImporter::FrameLoader::loadFile()
             int numDefectMeshVertices;
             if(sscanf(stream.line(), "DEFECT_MESH_VERTICES %i", &numDefectMeshVertices) != 1 || numDefectMeshVertices < 0)
                 throw Exception(tr("Failed to parse file. Invalid number of defect mesh vertices in line %1.").arg(stream.lineNumber()));
-            setProgressText(tr("Reading defect surface"));
-            setProgressMaximum(numDefectMeshVertices);
+            progress.setText(tr("Reading defect surface"));
+            progress.setMaximum(numDefectMeshVertices);
             meshBuilder.createVertices(numDefectMeshVertices);
             BufferWriteAccess<Point3, access_mode::read_write> vertexPositions(meshBuilder.mutableVertexProperty(SurfaceMeshVertices::PositionProperty));
             for(int index = 0; index < numDefectMeshVertices; index++) {
-                if(!setProgressValueIntermittent(index)) return;
+                progress.setValueIntermittent(index);
                 Point3 p;
                 if(sscanf(stream.readLine(), FLOATTYPE_SCANF_STRING " " FLOATTYPE_SCANF_STRING " " FLOATTYPE_SCANF_STRING, &p.x(), &p.y(), &p.z()) != 3)
                     throw Exception(tr("Failed to parse file. Invalid point in line %1.").arg(stream.lineNumber()));
@@ -458,11 +468,10 @@ void CAImporter::FrameLoader::loadFile()
             int numDefectMeshFacets;
             if(sscanf(stream.line(), "DEFECT_MESH_FACETS %i", &numDefectMeshFacets) != 1 || numDefectMeshFacets < 0)
                 throw Exception(tr("Failed to parse file. Invalid number of defect mesh facets in line %1.").arg(stream.lineNumber()));
-            setProgressMaximum(numDefectMeshFacets * 2);
+            progress.setMaximum(numDefectMeshFacets * 2);
             SurfaceMeshBuilder::FaceGrower faceGrower(meshBuilder);
             for(int index = 0; index < numDefectMeshFacets; index++) {
-                if(!setProgressValueIntermittent(index))
-                    return;
+                progress.setValueIntermittent(index);
                 int v[3];
                 if(sscanf(stream.readLine(), "%i %i %i", &v[0], &v[1], &v[2]) != 3
                         || v[0] < 0 || v[0] >= meshBuilder.vertexCount()
@@ -474,8 +483,7 @@ void CAImporter::FrameLoader::loadFile()
 
             // Read facet adjacency information.
             for(int index = 0; index < numDefectMeshFacets; index++) {
-                if(!setProgressValueIntermittent(index + numDefectMeshFacets))
-                    return;
+                progress.setValueIntermittent(index + numDefectMeshFacets);
                 int v[3];
                 if(sscanf(stream.readLine(), "%i %i %i", &v[0], &v[1], &v[2]) != 3)
                     throw Exception(tr("Failed to parse file. Invalid triangle adjacency info in line %1.").arg(stream.lineNumber()));
@@ -515,7 +523,6 @@ void CAImporter::FrameLoader::loadFile()
 
     std::vector<size_t> structureCounts;
     if(clusterGraph) {
-
         // Count how many atoms of each structure type exist by summing the cluster atom counts.
         for(const Cluster* cluster : clusterGraph->clusters()) {
             if(cluster->structure < 0) continue;
@@ -523,33 +530,15 @@ void CAImporter::FrameLoader::loadFile()
                 structureCounts.resize(cluster->structure + 1, 0);
             structureCounts[cluster->structure] += cluster->atomCount;
         }
-
-        // Insert cluster graph.
-        ClusterGraphObject* clusterGraphObj;
-        if(const ClusterGraphObject* existingClusterGraphObj = state().getObject<ClusterGraphObject>()) {
-            clusterGraphObj = state().makeMutable(existingClusterGraphObj);
-        }
-        else {
-            clusterGraphObj = state().createObject<ClusterGraphObject>(pipelineNode());
-        }
-        clusterGraphObj->setStorage(std::move(clusterGraph));
     }
 
     // Insert dislocations.
     if(dislocations) {
-        DislocationNetworkObject* dislocationNetwork;
-        if(const DislocationNetworkObject* existingDislocationsObj = state().getObject<DislocationNetworkObject>()) {
-            dislocationNetwork = state().makeMutable(existingDislocationsObj);
-        }
-        else {
-            dislocationNetwork = state().createObject<DislocationNetworkObject>(pipelineNode());
-        }
-        dislocationNetwork->setDomain(simulationCell());
-        dislocationNetwork->setStorage(dislocations);
+        dislocations->setDomain(simulationCell());
 
         // Update structure catalog.
         for(int i = 0; i < patterns.size(); i++) {
-            if(dislocationNetwork->structureByName(patterns[i].longName))
+            if(dislocations->structureByName(patterns[i].longName))
                 continue;
 
             DataOORef<MicrostructurePhase> pattern = DataOORef<MicrostructurePhase>::create();
@@ -560,33 +549,31 @@ void CAImporter::FrameLoader::loadFile()
             pattern->setNumericId(patterns[i].id);
             pattern->setCrystalSymmetryClass(patterns[i].symmetryType);
             pattern->freezeInitialParameterValues({SHADOW_PROPERTY_FIELD(ElementType::name), SHADOW_PROPERTY_FIELD(ElementType::color), SHADOW_PROPERTY_FIELD(MicrostructurePhase::shortName), SHADOW_PROPERTY_FIELD(MicrostructurePhase::dimensionality), SHADOW_PROPERTY_FIELD(MicrostructurePhase::crystalSymmetryClass)});
-            dislocationNetwork->addCrystalStructure(pattern);
+            dislocations->addCrystalStructure(pattern);
 
             // Add Burgers vector families.
             for(int j = 0; j < patterns[i].burgersVectorFamilies.size(); j++) {
-                DataOORef<BurgersVectorFamily> family = DataOORef<BurgersVectorFamily>::create();
-                family->setNumericId(patterns[i].burgersVectorFamilies[j].id);
-                family->setColor(patterns[i].burgersVectorFamilies[j].color);
-                family->setName(patterns[i].burgersVectorFamilies[j].name);
-                family->setBurgersVector(patterns[i].burgersVectorFamilies[j].burgersVector);
-                family->freezeInitialParameterValues({SHADOW_PROPERTY_FIELD(ElementType::name), SHADOW_PROPERTY_FIELD(ElementType::color), SHADOW_PROPERTY_FIELD(BurgersVectorFamily::burgersVector)});
-                pattern->addBurgersVectorFamily(family);
+                pattern->createBurgersVectorFamily(
+                    patterns[i].burgersVectorFamilies[j].id,
+                    patterns[i].burgersVectorFamilies[j].name,
+                    patterns[i].burgersVectorFamilies[j].burgersVector,
+                    patterns[i].burgersVectorFamilies[j].color);
             }
 
             // Make sure there always is a default family.
             if(pattern->burgersVectorFamilies().empty())
-                pattern->addBurgersVectorFamily(DataOORef<BurgersVectorFamily>::create());
+                pattern->createBurgersVectorFamily();
         }
 
         // Determine the main crystal structure of the system, which has the most atoms.
         const MicrostructurePhase* mainStructure = nullptr;
         if(!structureCounts.empty()) {
             int maxStructure = std::distance(structureCounts.begin(), std::max_element(structureCounts.begin(), structureCounts.end()));
-            mainStructure = dislocationNetwork->structureById(maxStructure);
+            mainStructure = dislocations->structureById(maxStructure);
         }
 
         // Compute dislocation line statistics.
-        DislocationAnalysisEngine::generateDislocationStatistics(pipelineNode(), state(), dislocationNetwork, true, mainStructure);
+        DislocationAnalysisEngine::generateDislocationStatistics(pipelineNode(), state(), dislocations, true, mainStructure);
     }
 
     state().setStatus(tr("Number of dislocations: %1").arg(numDislocationSegments));
