@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -22,8 +22,10 @@
 
 #include <ovito/crystalanalysis/CrystalAnalysis.h>
 #include <ovito/core/utilities/concurrent/Task.h>
+#include <ovito/core/utilities/concurrent/ParallelFor.h>
 #include "DislocationTracer.h"
 #include "InterfaceMesh.h"
+#include "TriangleTetrahedronIntersectionTest.h"
 
 namespace Ovito {
 
@@ -55,7 +57,7 @@ void DislocationTracer::discardCircuit(BurgersCircuit* circuit)
 * trial Burgers circuits. Identified dislocation segments are converted to
 * a continuous line representation.
 ******************************************************************************/
-bool DislocationTracer::traceDislocationSegments(ProgressingTask& operation)
+void DislocationTracer::traceDislocationSegments(TaskProgress& progress)
 {
     if(_maxBurgersCircuitSize < 3 || _maxBurgersCircuitSize > _maxExtendedBurgersCircuitSize)
         throw Exception("Invalid maximum circuit size parameter(s).");
@@ -65,13 +67,12 @@ bool DislocationTracer::traceDislocationSegments(ProgressingTask& operation)
     std::vector<int> subStepWeights(_maxExtendedBurgersCircuitSize - 2);
     for(int i = 0; i < subStepWeights.size(); i++)
         subStepWeights[i] = (i+3)*(i+3);
-    operation.beginProgressSubStepsWithWeights(subStepWeights);
+    progress.beginSubSteps(std::move(subStepWeights));
 
     mesh().clearFaceFlag(0);
 
     // Incrementally extend search radius for new Burgers circuits and extend existing segments by enlarging
     // the maximum circuit size until segments meet at a junction.
-    size_t numJunctions = 0;
     for(int circuitLength = 3; circuitLength <= _maxExtendedBurgersCircuitSize; circuitLength++) {
 
         // Extend existing segments with dangling ends.
@@ -86,12 +87,11 @@ bool DislocationTracer::traceDislocationSegments(ProgressingTask& operation)
         // Find dislocation segments by generating trial Burgers circuits on the interface mesh
         // and then moving them in both directions along the dislocation segment.
         if(circuitLength <= _maxBurgersCircuitSize && (circuitLength % 2) != 0) {
-            if(!findPrimarySegments(circuitLength, operation))
-                return false;
+            findPrimarySegments(circuitLength, progress);
         }
 
         // Join segments forming dislocation junctions.
-        numJunctions += joinSegments(circuitLength);
+        joinSegments(circuitLength);
 
         // Store circuits of dangling ends.
         if(circuitLength >= _maxBurgersCircuitSize) {
@@ -106,14 +106,10 @@ bool DislocationTracer::traceDislocationSegments(ProgressingTask& operation)
         }
 
         if(circuitLength < _maxExtendedBurgersCircuitSize)
-            operation.nextProgressSubStep();
+            progress.nextSubStep();
     }
 
-    //qDebug() << "Number of dislocation segments:" << network()->segments().size();
-    //qDebug() << "Number of dislocation junctions:" << numJunctions;
-
-    operation.endProgressSubSteps();
-    return !operation.isCanceled();
+    progress.endSubSteps();
 }
 
 /******************************************************************************
@@ -181,6 +177,42 @@ void DislocationTracer::finishDislocationSegments(int crystalStructure)
 
         segment->flipOrientation();
     }
+
+#if 0
+    {
+        const DelaunayTessellation& tessellation = mesh().tessellation();
+        const InterfaceMesh& m = mesh();
+
+        std::ofstream stream("tets.data", std::ios_base::trunc);
+
+        stream << tessellation.points().size() << "\n";
+        for(const auto& point : tessellation.points()) {
+            stream << point.x() << " " << point.y() << " " << point.z() << "\n";
+        }
+        size_t count = 0;
+        for(size_t cell = 0; cell < tessellation.numberOfTetrahedra(); ++cell) {
+            const auto& dislocInfo = tessellation.getDislocCoreInfo(cell);
+            DislocationSegment* segment = static_cast<DislocationSegment*>(dislocInfo.first);
+            if(segment &&
+               std::any_of(std::execution::par_unseq, segment->line.cbegin(), segment->line.cend(),
+                           [&dislocInfo, &m](const Point3& p) { return m.wrapVector(p - dislocInfo.second).squaredLength() < 1e-6; })) {
+                count++;
+            }
+        }
+        stream << count << "\n";
+        for(size_t cell = 0; cell < tessellation.numberOfTetrahedra(); ++cell) {
+            const auto& dislocInfo = tessellation.getDislocCoreInfo(cell);
+            DislocationSegment* segment = static_cast<DislocationSegment*>(dislocInfo.first);
+            if(segment &&
+               std::any_of(std::execution::par_unseq, segment->line.cbegin(), segment->line.cend(),
+                           [&dislocInfo, &m](const Point3& p) { return m.wrapVector(p - dislocInfo.second).squaredLength() < 1e-6; })) {
+                stream << tessellation.cellVertex(cell, 0) << " " << tessellation.cellVertex(cell, 1) << " "
+                       << tessellation.cellVertex(cell, 2) << " " << tessellation.cellVertex(cell, 3) << " " << segment->id << "\n";
+            }
+        }
+        stream.close();
+    }
+#endif
 }
 
 /**
@@ -214,14 +246,14 @@ struct BurgersCircuitSearchStruct
 * Then moves the Burgers circuit in both directions along the dislocation
 * segment until the maximum circuit size has been reached.
 ******************************************************************************/
-bool DislocationTracer::findPrimarySegments(int maxBurgersCircuitSize, ProgressingTask& operation)
+void DislocationTracer::findPrimarySegments(int maxBurgersCircuitSize, TaskProgress& progress)
 {
     int searchDepth =  (maxBurgersCircuitSize - 1) / 2;
     OVITO_ASSERT(searchDepth >= 1);
 
     MemoryPool<BurgersCircuitSearchStruct> structPool;
 
-    operation.setProgressMaximum(mesh().vertexCount());
+    progress.setMaximum(mesh().vertexCount());
     int progressCounter = 0;
 
     // Find an appropriate start node for the recursive search.
@@ -230,8 +262,7 @@ bool DislocationTracer::findPrimarySegments(int maxBurgersCircuitSize, Progressi
         OVITO_ASSERT(startNode.burgersSearchStruct == nullptr);
 
         // Update progress indicator.
-        if(!operation.setProgressValueIntermittent(progressCounter++))
-            return false;
+        progress.setValueIntermittent(progressCounter++);
 
         // The first node is the seed of our recursive walk.
         // It is mapped to the origin of the reference lattice.
@@ -246,7 +277,7 @@ bool DislocationTracer::findPrimarySegments(int maxBurgersCircuitSize, Progressi
 
         // This is the cluster we work in.
         OVITO_ASSERT(startNode.edges()->clusterTransition);
-        Cluster* cluster = startNode.edges()->clusterTransition->cluster1;
+        Q_DECL_UNUSED Cluster* cluster = startNode.edges()->clusterTransition->cluster1;
         OVITO_ASSERT(cluster && cluster->id != 0);
 
         bool foundBurgersCircuit = false;
@@ -324,7 +355,7 @@ bool DislocationTracer::findPrimarySegments(int maxBurgersCircuitSize, Progressi
         structPool.clear(true);
     }
 
-    return !operation.isCanceled();
+    this_task::throwIfCanceled();
 }
 
 /******************************************************************************
@@ -541,9 +572,7 @@ BurgersCircuit* DislocationTracer::buildReverseCircuit(BurgersCircuit* forwardCi
         OVITO_ASSERT((edge2->clusterVector + oppositeEdge2->clusterTransition->tm * oppositeEdge2->clusterVector).isZero(CA_LATTICE_VECTOR_EPSILON));
 
         if(facet1 != facet2) {
-            InterfaceMesh::Edge* outerEdge1 = oppositeEdge1->nextFaceEdge()->oppositeEdge();
             InterfaceMesh::Edge* innerEdge1 = oppositeEdge1->prevFaceEdge()->oppositeEdge();
-            InterfaceMesh::Edge* outerEdge2 = oppositeEdge2->prevFaceEdge()->oppositeEdge();
             InterfaceMesh::Edge* innerEdge2 = oppositeEdge2->nextFaceEdge()->oppositeEdge();
             OVITO_ASSERT(innerEdge1 != nullptr && innerEdge2 != nullptr);
             OVITO_ASSERT(innerEdge1->vertex1() == edge1->vertex2());
@@ -612,11 +641,11 @@ void DislocationTracer::traceSegment(DislocationSegment& segment, DislocationNod
 
     // Advance circuit as far as possible.
     for(;;) {
-
         // During each iteration, first shorten circuit as much as possible.
         // Pick a random start edge to distribute the removal of edges
         // over the whole circuit.
-        int edgeIndex = std::uniform_int_distribution<>(0, circuit.edgeCount - 1)(_rng);
+        int edgeIndex = boost::random::uniform_int_distribution<>(0, circuit.edgeCount - 1)(_rng);
+
         InterfaceMesh::Edge* firstEdge = circuit.getEdge(edgeIndex);
 
         InterfaceMesh::Edge* edge0 = firstEdge;
@@ -663,7 +692,8 @@ void DislocationTracer::traceSegment(DislocationSegment& segment, DislocationNod
 
         // Pick a random start edge to distribute the insertion of new edges
         // over the whole circuit.
-        edgeIndex = std::uniform_int_distribution<>(0, circuit.edgeCount - 1)(_rng);
+        edgeIndex = boost::random::uniform_int_distribution<>(0, circuit.edgeCount - 1)(_rng);
+
         firstEdge = circuit.getEdge(edgeIndex);
 
         edge0 = firstEdge;
@@ -930,13 +960,98 @@ void DislocationTracer::appendLinePoint(DislocationNode& node)
         segment.line.push_front(newPoint);
         segment.coreSize.push_front(coreSize);
     }
+
     node.circuit->numPreliminaryPoints++;
+    if(!_markCoreAtoms) {
+        return;
+    }
+
+    // Implementation of the core atom marking algorithm:
+
+    // One-time initialization of spatial query structure for Delaunay cells and auxiliary data array per "defective" Delaunay cell.
+    DelaunayTessellation& tessellation = mesh().tessellation();
+    FloatType alpha = 3.5 * mesh().elasticMapping().structureAnalysis().maximumNeighborDistance();
+    if(!_spatialQuery) {
+        _spatialQuery.emplace(tessellation, alpha);
+        _cellDataForCoreAtomIdentification.resize(_spatialQuery->numCells(), std::make_pair(nullptr, false));
+    }
+
+    // Collect cap triangles and computed their combined bounding box.
+    InterfaceMesh::Edge* edge = node.circuit->firstEdge;
+    _triangles.clear();
+    Box3 bbox;
+    do {
+        _triangles.push_back({newPoint + mesh().wrapVector(edge->vertex1()->pos() - newPoint),
+                              newPoint + mesh().wrapVector(edge->vertex2()->pos() - newPoint), newPoint});
+
+        for(size_t i = 0; i < 3; ++i) {
+            bbox.addPoint(_triangles.back()[i]);
+        }
+        edge = edge->nextCircuitEdge;
+    }
+    while(edge != node.circuit->firstEdge);
+
+    // Obtain the list of all Delaunay cells intersecting with the bounding box
+    _spatialQuery->getOverlappingCells(bbox, _ranges);
+
+    using bBox = DelaunayTessellationSpatialQuery::bBox;
+
+    // Work chunk size of 256 is used as threshold for parallelization and has been determined from initial testing.
+    // Parallel loop over tetrahedrons to be tested for intersection with the cap polygon.
+    // Inner loop over each triangle until an intersection is detected.
+    parallelFor(_ranges.size(), 256, TaskProgress::Ignore, [&](size_t idx) {
+
+        // Grab bbox and cell
+        const bBox& bbox = _ranges[idx];
+        OVITO_ASSERT(bbox.max_corner().cell == bbox.min_corner().cell);
+        size_t cell = bbox.max_corner().cell;
+        int cellIdx = tessellation.getUserField(cell);
+        OVITO_ASSERT(cellIdx == -1 || cellIdx < _cellDataForCoreAtomIdentification.size());
+
+        // Skip over tetrahedrons that have already been assigned to some dislocation.
+        if(cellIdx == -1 || _cellDataForCoreAtomIdentification[cellIdx].first) {
+            return;
+        }
+
+        // Get tetrahedron coordinates -> unwrapped as newPoint should be in the
+        // real or ghost layer region of the tessellation containing tetrahedrons
+        std::array<Point3, 4> tet;
+        for(size_t t = 0; t < tet.size(); ++t) {
+            tet[t] = tessellation.vertexPosition(tessellation.cellVertex(cell, t));
+        }
+
+        // Intersection test
+        for(const std::array<Point3, 3>& triangle : _triangles) {
+            if(TetrahedronTriangleIntersection::test(tet, triangle)) {
+                // Tag tetrahedron as belonging to the current dislocation line end.
+                _cellDataForCoreAtomIdentification[cellIdx] = std::make_pair(&node, !node.circuit->segmentMeshCap.empty());
+                return;
+            }
+        }
+    });
+
+#if 0
+    {
+        std::ofstream stream("slices.data", std::ios_base::app);
+        InterfaceMesh::Edge* edge = node.circuit->firstEdge;
+        do {
+            Point3 p1 = newPoint + mesh().wrapVector(edge->vertex1()->pos() - newPoint);
+            stream << p1.x() << " " << p1.y() << " " << p1.z() << " ";
+            Point3 p2 = newPoint + mesh().wrapVector(edge->vertex2()->pos() - newPoint);
+            stream << p2.x() << " " << p2.y() << " " << p2.z() << " ";
+            stream << newPoint.x() << " " << newPoint.y() << " " << newPoint.z() << "\n";
+            edge = edge->nextCircuitEdge;
+        } while(edge != node.circuit->firstEdge);
+        stream.close();
+#endif
 }
 
 /******************************************************************************
-* Determines whether two Burgers circuits intersect.
-******************************************************************************/
-void DislocationTracer::circuitCircuitIntersection(InterfaceMesh::Edge* circuitAEdge1, InterfaceMesh::Edge* circuitAEdge2, InterfaceMesh::Edge* circuitBEdge1, InterfaceMesh::Edge* circuitBEdge2, int& goingOutside, int& goingInside)
+ * Determines whether two Burgers circuits intersect.
+ ******************************************************************************/
+void DislocationTracer::circuitCircuitIntersection(InterfaceMesh::Edge* circuitAEdge1, InterfaceMesh::Edge* circuitAEdge2,
+                                                   InterfaceMesh::Edge* circuitBEdge1, InterfaceMesh::Edge* circuitBEdge2,
+                                                   int& goingOutside, int& goingInside)
 {
     OVITO_ASSERT(circuitAEdge2->vertex1() == circuitBEdge2->vertex1());
     OVITO_ASSERT(circuitAEdge1->vertex2() == circuitBEdge2->vertex1());
@@ -985,8 +1100,8 @@ void DislocationTracer::circuitCircuitIntersection(InterfaceMesh::Edge* circuitA
 }
 
 /******************************************************************************
-* Look for dislocation segments whose circuits touch each other.
-******************************************************************************/
+ * Look for dislocation segments whose circuits touch each other.
+ ******************************************************************************/
 size_t DislocationTracer::joinSegments(int maxCircuitLength)
 {
     // First iteration over all dangling circuits.
@@ -1009,12 +1124,11 @@ size_t DislocationTracer::joinSegments(int maxCircuitLength)
                 createSecondarySegment(edge, circuit, maxCircuitLength);
 
                 // Skip edges to the end of the unvisited interval.
-                while(edge->oppositeEdge()->circuit == nullptr && edge != circuit->firstEdge)
-                    edge = edge->nextCircuitEdge;
+                while(edge->oppositeEdge()->circuit == nullptr && edge != circuit->firstEdge) edge = edge->nextCircuitEdge;
             }
-            else edge = edge->nextCircuitEdge;
-        }
-        while(edge != circuit->firstEdge);
+            else
+                edge = edge->nextCircuitEdge;
+        } while(edge != circuit->firstEdge);
     }
 
     // Second pass over all dangling nodes.
@@ -1045,8 +1159,7 @@ size_t DislocationTracer::joinSegments(int maxCircuitLength)
                 }
             }
             edge = edge->nextCircuitEdge;
-        }
-        while(edge != circuit->firstEdge);
+        } while(edge != circuit->firstEdge);
     }
 
     // Count number of created dislocation junctions.
@@ -1118,8 +1231,7 @@ size_t DislocationTracer::joinSegments(int maxCircuitLength)
                 }
                 armNode->circuit->numPreliminaryPoints = 0;
                 armNode = armNode->junctionRing;
-            }
-            while(armNode != node);
+            } while(armNode != node);
             numJunctions++;
         }
         else {
@@ -1194,8 +1306,7 @@ size_t DislocationTracer::joinSegments(int maxCircuitLength)
                         segment1->coreSize.insert(segment1->coreSize.end(), segment2->coreSize.rbegin() + 1, segment2->coreSize.rend());
                     }
                     if(shiftVector != Vector3::Zero()) {
-                        for(auto p = segment1->line.end() - segment2->line.size() + 1; p != segment1->line.end(); ++p)
-                            *p -= shiftVector;
+                        for(auto p = segment1->line.end() - segment2->line.size() + 1; p != segment1->line.end(); ++p) *p -= shiftVector;
                     }
                 }
                 farEnd2->segment = segment1;
@@ -1210,15 +1321,14 @@ size_t DislocationTracer::joinSegments(int maxCircuitLength)
     }
 
     // Clean up list of dangling nodes. Remove joined nodes.
-    _danglingNodes.erase(std::remove_if(_danglingNodes.begin(), _danglingNodes.end(),
-            [](DislocationNode* node) { return !node->isDangling(); }), _danglingNodes.end());
+    std::erase_if(_danglingNodes, [](DislocationNode* node) { return !node->isDangling(); });
 
     return numJunctions;
 }
 
 /******************************************************************************
-* Creates a new dislocation segment at an incomplete junction.
-******************************************************************************/
+ * Creates a new dislocation segment at an incomplete junction.
+ ******************************************************************************/
 void DislocationTracer::createSecondarySegment(InterfaceMesh::Edge* firstEdge, BurgersCircuit* outerCircuit, int maxCircuitLength)
 {
     OVITO_ASSERT(firstEdge->circuit == outerCircuit);
@@ -1237,7 +1347,6 @@ void DislocationTracer::createSecondarySegment(InterfaceMesh::Edge* firstEdge, B
         for(;;) {
             OVITO_ASSERT(edge->circuit == nullptr);
             InterfaceMesh::Edge* oppositeEdge = edge->oppositeEdge();
-            InterfaceMesh::Face* oppositeFacet = oppositeEdge->face();
             InterfaceMesh::Edge* nextEdge = oppositeEdge->prevFaceEdge();
             OVITO_ASSERT(nextEdge->vertex2() == oppositeEdge->vertex1());
             OVITO_ASSERT(nextEdge->vertex2() == edge->vertex2());
@@ -1256,24 +1365,17 @@ void DislocationTracer::createSecondarySegment(InterfaceMesh::Edge* firstEdge, B
         edgeSum += edge->physicalVector;
         burgersVector += frankRotation * edge->clusterVector;
         if(!baseCluster) baseCluster = edge->clusterTransition->cluster1;
-        if(!edge->clusterTransition->isSelfTransition())
-            frankRotation = frankRotation * edge->clusterTransition->reverse->tm;
-        if(edge == circuitStart)
-            break;
+        if(!edge->clusterTransition->isSelfTransition()) frankRotation = frankRotation * edge->clusterTransition->reverse->tm;
+        if(edge == circuitStart) break;
         circuitEnd = edge;
         edgeCount++;
 
-        if(edgeCount > maxCircuitLength)
-            break;
+        if(edgeCount > maxCircuitLength) break;
     }
 
     // Create secondary segment only for dislocations (b != 0) and small enough dislocation cores.
-    if(numCircuits == 1
-            || edgeCount > maxCircuitLength
-            || burgersVector.isZero(CA_LATTICE_VECTOR_EPSILON)
-            || edgeSum.isZero(CA_ATOM_VECTOR_EPSILON) == false
-            || !frankRotation.equals(Matrix3::Identity(), CA_TRANSITION_MATRIX_EPSILON)) {
-
+    if(numCircuits == 1 || edgeCount > maxCircuitLength || burgersVector.isZero(CA_LATTICE_VECTOR_EPSILON) ||
+       edgeSum.isZero(CA_ATOM_VECTOR_EPSILON) == false || !frankRotation.equals(Matrix3::Identity(), CA_TRANSITION_MATRIX_EPSILON)) {
         // Discard unused circuit.
         edge = circuitStart;
         for(;;) {
@@ -1297,8 +1399,7 @@ void DislocationTracer::createSecondarySegment(InterfaceMesh::Edge* firstEdge, B
         OVITO_ASSERT(edge->circuit == nullptr);
         edge->circuit = forwardCircuit;
         edge = edge->nextCircuitEdge;
-    }
-    while(edge != circuitStart);
+    } while(edge != circuitStart);
     OVITO_ASSERT(forwardCircuit->countEdges() == forwardCircuit->edgeCount);
 
     // Do all the rest.
