@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -58,8 +58,12 @@ void bitmapSort(iterator begin, iterator end, int max)
 /******************************************************************************
 * Constructor.
 ******************************************************************************/
-StructureAnalysis::StructureAnalysis(ConstPropertyPtr positions, const SimulationCell* simCell,
-        LatticeStructureType inputCrystalType, ConstPropertyPtr particleSelection,
+StructureAnalysis::StructureAnalysis(
+        ConstPropertyPtr positions,
+        const SimulationCell* simCell,
+        LatticeStructureType inputCrystalType,
+        ConstPropertyPtr particleSelection,
+        ClusterGraph* clusterGraph,
         PropertyPtr outputStructures, std::vector<Matrix3> preferredCrystalOrientations,
         bool identifyPlanarDefects) :
     _positions(positions),
@@ -71,14 +75,18 @@ StructureAnalysis::StructureAnalysis(ConstPropertyPtr positions, const Simulatio
     _atomClusters(Particles::OOClass().createStandardProperty(DataBuffer::Initialized, positions->size(), Particles::ClusterProperty)),
     _atomClustersArray(_atomClusters),
     _atomSymmetryPermutations(positions->size()),
-    _clusterGraph(std::make_shared<ClusterGraph>()),
+    _clusterGraph(clusterGraph),
     _preferredCrystalOrientations(std::move(preferredCrystalOrientations)),
     _identifyPlanarDefects(identifyPlanarDefects)
 {
-    static bool initialized = false;
-    if(!initialized) {
-        initializeListOfStructures();
-        initialized = true;
+    // One-time initialization of precomputed structure information.
+    static std::mutex mutex;
+    {
+        std::lock_guard<std::mutex> lock(mutex);
+        if(!latticeStructure(LATTICE_FCC).coordStructure) {
+            initializeListOfStructures();
+            OVITO_ASSERT(latticeStructure(LATTICE_FCC).coordStructure);
+        }
     }
 
     if(_positions->size() > std::numeric_limits<int>::max())
@@ -132,7 +140,7 @@ void StructureAnalysis::initializeListOfStructures()
     for(int ni1 = 0; ni1 < 12; ni1++) {
         _coordinationStructures[COORD_FCC].neighborArray.setNeighborBond(ni1, ni1, false);
         for(int ni2 = ni1 + 1; ni2 < 12; ni2++) {
-            bool bonded = (fccVec[ni1] - fccVec[ni2]).length() < (sqrt(0.5f)+1.0)*0.5;
+            bool bonded = (fccVec[ni1] - fccVec[ni2]).length() < (std::sqrt(0.5f)+1.0)*0.5;
             _coordinationStructures[COORD_FCC].neighborArray.setNeighborBond(ni1, ni2, bonded);
         }
         _coordinationStructures[COORD_FCC].cnaSignatures[ni1] = 0;
@@ -171,7 +179,7 @@ void StructureAnalysis::initializeListOfStructures()
     for(int ni1 = 0; ni1 < 12; ni1++) {
         _coordinationStructures[COORD_HCP].neighborArray.setNeighborBond(ni1, ni1, false);
         for(int ni2 = ni1 + 1; ni2 < 12; ni2++) {
-            bool bonded = (hcpVec[ni1] - hcpVec[ni2]).length() < (sqrt(0.5)+1.0)*0.5;
+            bool bonded = (hcpVec[ni1] - hcpVec[ni2]).length() < (std::sqrt(0.5)+1.0)*0.5;
             _coordinationStructures[COORD_HCP].neighborArray.setNeighborBond(ni1, ni2, bonded);
         }
         _coordinationStructures[COORD_HCP].cnaSignatures[ni1] = (hcpVec[ni1].z() == 0) ? 1 : 0;
@@ -382,7 +390,7 @@ void StructureAnalysis::initializeListOfStructures()
         // Find symmetry permutations.
         std::vector<int> permutation(latticeStruct->latticeVectors.size());
         std::vector<int> lastPermutation(latticeStruct->latticeVectors.size(), -1);
-        std::iota(permutation.begin(), permutation.end(), 0);
+        boost::algorithm::iota(permutation, 0);
         SymmetryPermutation symmetryPermutation;
         do {
             int changedFrom = std::mismatch(permutation.begin(), permutation.end(), lastPermutation.begin()).first - permutation.begin();
@@ -409,9 +417,11 @@ void StructureAnalysis::initializeListOfStructures()
             }
             if(invalidFrom == coordStruct.numNeighbors) {
                 std::copy(permutation.begin(), permutation.begin() + coordStruct.numNeighbors, symmetryPermutation.permutation.begin());
+#ifdef OVITO_DEBUG
                 for(const auto& entry : latticeStruct->permutations) {
                     OVITO_ASSERT(!entry.transformation.equals(symmetryPermutation.transformation));
                 }
+#endif
                 latticeStruct->permutations.push_back(symmetryPermutation);
             }
             else {
@@ -435,7 +445,6 @@ void StructureAnalysis::initializeListOfStructures()
                     }
                 }
                 OVITO_ASSERT(latticeStruct->permutations[s1].product.size() == s2 + 1);
-                Matrix3 inverseProduct = latticeStruct->permutations[s2].transformation.inverse() * latticeStruct->permutations[s1].transformation;
                 for(int i = 0; i < latticeStruct->permutations.size(); i++) {
                     if(latticeStruct->permutations[i].transformation.equals(product)) {
                         latticeStruct->permutations[s1].inverseProduct.push_back(i);
@@ -451,18 +460,16 @@ void StructureAnalysis::initializeListOfStructures()
 /******************************************************************************
 * Identifies the atomic structures.
 ******************************************************************************/
-bool StructureAnalysis::identifyStructures()
+void StructureAnalysis::identifyStructures(TaskProgress& progress, const SimulationCell* simulationCell)
 {
     // Prepare the neighbor list.
     int maxNeighborListSize = std::min((int)_neighborListsSize + 1, (int)MAX_NEIGHBORS);
-    NearestNeighborFinder neighFinder(maxNeighborListSize);
-    if(!neighFinder.prepare(positions(), cell(), _particleSelection.buffer()))
-        return false;
+    NearestNeighborFinder neighFinder(maxNeighborListSize, positions(), simulationCell, _particleSelection.buffer());
 
     // Identify local structure around each particle.
     _maximumNeighborDistance = 0;
 
-    return parallelForWithProgress(positions()->size(), [this, &neighFinder](size_t index) {
+    parallelFor(positions()->size(), 1024, progress, [this, &neighFinder](size_t index) {
         determineLocalStructure(neighFinder, index);
     });
 }
@@ -731,12 +738,10 @@ void StructureAnalysis::determineLocalStructure(NearestNeighborFinder& neighList
             for(int i = 0; i < nn; i++) {
                 const Vector3& neighborVector = neighborVectors[neighborMapping[i]];
                 // Check if neighbor vectors spans more than half of a periodic simulation cell.
-                if(cell()) {
-                    for(size_t dim = 0; dim < 3; dim++) {
-                        if(cell()->hasPbc(dim)) {
-                            if(std::abs(cell()->inverseMatrix().prodrow(neighborVector, dim)) >= FloatType(0.5)+FLOATTYPE_EPSILON)
-                                StructureAnalysis::generateCellTooSmallError(dim);
-                        }
+                for(size_t dim = 0; dim < 3; dim++) {
+                    if(cell().hasPbc(dim)) {
+                        if(std::abs(cell().reciprocalCellMatrix().prodrow(neighborVector, dim)) >= FloatType(0.5)+FLOATTYPE_EPSILON)
+                            StructureAnalysis::generateCellTooSmallError(dim);
                     }
                 }
                 setNeighbor(particleIndex, i, neighborIndices[neighborMapping[i]]);
@@ -761,12 +766,11 @@ void StructureAnalysis::determineLocalStructure(NearestNeighborFinder& neighList
 /******************************************************************************
 * Combines adjacent atoms to clusters.
 ******************************************************************************/
-bool StructureAnalysis::buildClusters()
+void StructureAnalysis::buildClusters(TaskProgress& progress)
 {
-    OVITO_ASSERT(Task::current() && Task::current()->isProgressingTask());
-    ProgressingTask& operation = static_cast<ProgressingTask&>(*Task::current());
-    operation.setProgressMaximum(positions()->size());
-    int progressCounter = 0;
+    progress.setMaximum(positions()->size());
+
+    qlonglong progressCounter = 0;
     BufferReadAccess<Point3> positionsArray(positions());
 
     // Iterate over atoms, looking for those that have not been visited yet.
@@ -800,8 +804,7 @@ bool StructureAnalysis::buildClusters()
             atomsToVisit.pop_front();
 
             // Update progress indicator.
-            if(!operation.setProgressValueIntermittent(++progressCounter))
-                return false;
+            progress.setValueIntermittent(++progressCounter);
 
             // Look up symmetry permutation of current atom.
             int symmetryPermutationIndex = _atomSymmetryPermutations[currentAtomIndex];
@@ -817,8 +820,7 @@ bool StructureAnalysis::buildClusters()
 
                 // Add vector pair to matrices for computing the cluster orientation.
                 const Vector3& latticeVector = latticeStructure.latticeVectors[permutation[neighborIndex]];
-                Vector3 spatialVector = positionsArray[neighborAtomIndex] - positionsArray[currentAtomIndex];
-                if(cell()) spatialVector = cell()->wrapVector(spatialVector);
+                Vector3 spatialVector = cell().wrapVector(positionsArray[neighborAtomIndex] - positionsArray[currentAtomIndex]);
                 for(size_t i = 0; i < 3; i++) {
                     for(size_t j = 0; j < 3; j++) {
                         orientationV(i,j) += (double)(latticeVector[j] * latticeVector[i]);
@@ -927,28 +929,24 @@ bool StructureAnalysis::buildClusters()
     }
 
 //  qInfo() << "Number of clusters:" << (clusterGraph()->clusters().size() - 1);
-
-    return !operation.isCanceled();
 }
 
 /******************************************************************************
 * Determines the transition matrices between clusters.
 ******************************************************************************/
-bool StructureAnalysis::connectClusters()
+void StructureAnalysis::connectClusters(TaskProgress& progress)
 {
-    OVITO_ASSERT(Task::current() && Task::current()->isProgressingTask());
-    ProgressingTask& operation = static_cast<ProgressingTask&>(*Task::current());
-    operation.setProgressMaximum(positions()->size());
+    progress.setMaximum(positions()->size());
 
     for(size_t atomIndex = 0; atomIndex < positions()->size(); atomIndex++) {
         int clusterId = _atomClustersArray[atomIndex];
-        if(clusterId == 0) continue;
+        if(clusterId == 0)
+            continue;
         Cluster* cluster1 = clusterGraph()->findCluster(clusterId);
         OVITO_ASSERT(cluster1);
 
         // Update progress indicator.
-        if(!operation.setProgressValueIntermittent(atomIndex))
-            return false;
+        progress.setValueIntermittent(atomIndex);
 
         // Look up symmetry permutation of current atom.
         int structureType = _structureTypesArray[atomIndex];
@@ -1033,17 +1031,13 @@ bool StructureAnalysis::connectClusters()
     }
 
 //  qInfo() << "Number of cluster transitions:" << clusterGraph()->clusterTransitions().size();
-
-    return !operation.isCanceled();
 }
 
 /******************************************************************************
 * Combines clusters to super clusters.
 ******************************************************************************/
-bool StructureAnalysis::formSuperClusters()
+void StructureAnalysis::formSuperClusters()
 {
-    OVITO_ASSERT(Task::current());
-    Task& operation = *Task::current();
     size_t oldTransitionCount = clusterGraph()->clusterTransitions().size();
 
     for(size_t clusterIndex = 0; clusterIndex < clusterGraph()->clusters().size(); clusterIndex++) {
@@ -1051,8 +1045,7 @@ bool StructureAnalysis::formSuperClusters()
         cluster->rank = 0;
         if(cluster->id == 0) continue;
 
-        if(operation.isCanceled())
-            return false;
+        this_task::throwIfCanceled();
 
         OVITO_ASSERT(cluster->parentTransition == nullptr);
         if(cluster->structure != _inputCrystalType) {
@@ -1114,8 +1107,7 @@ bool StructureAnalysis::formSuperClusters()
         Cluster* parentCluster2 = getParentGrain(t->cluster2);
         if(parentCluster1 == parentCluster2) continue;
 
-        if(operation.isCanceled())
-            return false;
+        this_task::throwIfCanceled();
 
         ClusterTransition* parentTransition = t;
         if(parentCluster2 != t->cluster2) {
@@ -1141,8 +1133,7 @@ bool StructureAnalysis::formSuperClusters()
     for(Cluster* cluster : clusterGraph()->clusters()) {
         getParentGrain(cluster);
     }
-
-    return !operation.isCanceled();
+    this_task::throwIfCanceled();
 }
 
 }   // End of namespace
