@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -27,7 +27,6 @@
 #include <ovito/stdobj/simcell/SimulationCell.h>
 #include <ovito/stdobj/properties/Property.h>
 #include <ovito/mesh/surface/SurfaceMeshBuilder.h>
-#include <ovito/core/utilities/concurrent/ProgressingTask.h>
 #include <ovito/delaunay/DelaunayTessellation.h>
 
 namespace Ovito {
@@ -57,7 +56,7 @@ public:
 
     /// Constructor.
     ManifoldConstructionHelper(DelaunayTessellation& tessellation, SurfaceMeshBuilder& outputMesh, FloatType alpha, bool createRegions,
-            const Property* positions) : _tessellation(tessellation), _mesh(outputMesh), _alpha(alpha), _createRegions(createRegions), _positions(positions) { OVITO_ASSERT(_tessellation.simCell()); }
+            const Property* positions, TaskProgress& progress) : _tessellation(tessellation), _mesh(outputMesh), _alpha(alpha), _createRegions(createRegions), _positions(positions), _progress(progress) {}
 
     /// Returns the number of filled regions that have been identified.
     SurfaceMesh::size_type filledRegionCount() const { return _filledRegionCount; }
@@ -67,37 +66,31 @@ public:
 
     /// This is the main function, which constructs the manifold triangle mesh.
     template<typename CellRegionFunc, typename PrepareMeshFaceFunc = DefaultPrepareMeshFaceFunc, typename PrepareMeshVertexFunc = DefaultPrepareMeshVertexFunc>
-    bool construct(CellRegionFunc&& determineCellRegion, ProgressingTask& operation,
-            PrepareMeshFaceFunc&& prepareMeshFaceFunc = PrepareMeshFaceFunc(), PrepareMeshVertexFunc&& prepareMeshVertexFunc = PrepareMeshVertexFunc())
+    void construct(CellRegionFunc&& determineCellRegion, PrepareMeshFaceFunc&& prepareMeshFaceFunc = PrepareMeshFaceFunc(), PrepareMeshVertexFunc&& prepareMeshVertexFunc = PrepareMeshVertexFunc())
     {
         // Algorithm is divided into several sub-steps.
         if(_createRegions)
-            operation.beginProgressSubStepsWithWeights({1,8,2,1});
+            _progress.beginSubSteps({1,8,2,1});
         else
-            operation.beginProgressSubStepsWithWeights({1,1,2});
+            _progress.beginSubSteps({1,1,2});
 
         // Assign tetrahedra to spatial regions.
-        if(!classifyTetrahedra(std::move(determineCellRegion), operation))
-            return false;
-
-        operation.nextProgressSubStep();
+        classifyTetrahedra(std::move(determineCellRegion));
+        _progress.nextSubStep();
 
         // Group connected tetrahedra into volumetric regions.
         if(_createRegions) {
-            if(!formFilledRegions(operation))
-                return false;
-            operation.nextProgressSubStep();
+            formFilledRegions();
+            _progress.nextSubStep();
         }
 
         // Create triangle facets at interfaces between two different regions.
-        if(!createInterfaceFacets(std::move(prepareMeshFaceFunc), std::move(prepareMeshVertexFunc), operation))
-            return false;
-
-        operation.nextProgressSubStep();
+        createInterfaceFacets(std::move(prepareMeshFaceFunc), std::move(prepareMeshVertexFunc));
+        _progress.nextSubStep();
 
         // Connect triangles with one another to form a closed manifold.
-        if(!linkHalfedges(operation))
-            return false;
+        linkHalfedges();
+        this_task::throwIfCanceled();
 
 #ifdef OVITO_DEBUG
         // Verify that generated manifold connectivity is correct when a two-sided mesh was created.
@@ -111,22 +104,20 @@ public:
         }
 #endif
 
-        operation.endProgressSubSteps();
-
-        return !operation.isCanceled();
+        _progress.endSubSteps();
     }
 
     /// Finds disconnected empty regions and computes their volumes.
-    bool formEmptyRegions(ProgressingTask& operation) {
-        operation.beginProgressSubStepsWithWeights({1,1});
+    void formEmptyRegions() {
+        _progress.beginSubSteps({1,1});
 
         // Count number of identified regions.
         _emptyRegionCount = 0;
 
         // Flags indicating which periodic cell directions are connected by a surface through the cell boundary.
-        const SimulationCell* simCell = _tessellation.simCell();
+        const SimulationCellData simCell = _tessellation.simCell();
         bool surfaceCrossesBoundaries[3] = { false, false, false };
-        bool detectBoundaryCrossings = simCell->hasPbc();
+        bool detectBoundaryCrossings = simCell.hasPbc();
 
         // Stack of faces to visit. Used for implementing recursive algorithm.
         std::deque<SurfaceMesh::face_index> facesToProcess;
@@ -138,7 +129,8 @@ public:
         BufferReadAccess<Point3> vertexPositions(_mesh.expectVertexProperty(SurfaceMeshVertices::PositionProperty));
 
         // Identify disconnected components of the surface mesh bordering to an empty region.
-        operation.setProgressMaximum(_mesh.faceCount() / 2); // Note: Dividing by two, because only every other face is oriented towards the empty region.
+        _progress.setMaximum(_mesh.faceCount() / 2); // Note: Dividing by two, because only every other face is oriented towards the empty region.
+        size_t progressValue = 0;
         for(SurfaceMesh::face_index face = 0; face < _mesh.faceCount(); face++) {
             // Look for mesh faces that are not adjacent to a filled region and which have not been visited yet.
             SurfaceMesh::region_index faceRegion = faceRegions[face];
@@ -148,15 +140,13 @@ public:
             // Found a first seed face. Start a new mesh component.
             faceRegion = _filledRegionCount + (_emptyRegionCount++);
             faceRegions[face] = faceRegion;
-            size_t faceCount = 1;
 
             facesToProcess.push_back(face);
             do {
                 // Take next face from the stack.
                 SurfaceMesh::face_index currentFace = facesToProcess.front();
                 facesToProcess.pop_front();
-                if(!operation.incrementProgressValue())
-                    return false;
+                _progress.setValueIntermittent(progressValue++);
 
                 // Visit neighbors of current face.
                 SurfaceMesh::edge_index firstEdge = _mesh.firstFaceEdge(currentFace);
@@ -169,11 +159,11 @@ public:
                         const Point3& p2 = vertexPositions[_mesh.vertex2(edge)];
                         Vector3 delta = p2 - p1;
                         for(size_t dim = 0; dim < 3; dim++) {
-                            if(!surfaceCrossesBoundaries[dim] && simCell->hasPbc(dim)) {
+                            if(!surfaceCrossesBoundaries[dim] && simCell.hasPbc(dim)) {
                                 // The edge is crossing the periodic boundary if it spans more than half of the simulation cell in that direction.
-                                if(std::abs(simCell->inverseMatrix().prodrow(delta, dim)) >= FloatType(0.5)) {
+                                if(std::abs(simCell.reciprocalCellMatrix().prodrow(delta, dim)) >= FloatType(0.5)) {
                                     surfaceCrossesBoundaries[dim] = true;
-                                    detectBoundaryCrossings = (simCell->hasPbc(0) && !surfaceCrossesBoundaries[0]) || (simCell->hasPbc(1) && !surfaceCrossesBoundaries[1]) || (simCell->hasPbc(2) && !surfaceCrossesBoundaries[2]);
+                                    detectBoundaryCrossings = (simCell.hasPbc(0) && !surfaceCrossesBoundaries[0]) || (simCell.hasPbc(1) && !surfaceCrossesBoundaries[1]) || (simCell.hasPbc(2) && !surfaceCrossesBoundaries[2]);
                                 }
                             }
                         }
@@ -190,7 +180,6 @@ public:
                         faceRegions[neighborFace] = faceRegion;
                         // Put on recursive stack.
                         facesToProcess.push_back(neighborFace);
-                        faceCount++;
                     }
 
                     edge = _mesh.nextFaceEdge(edge);
@@ -200,7 +189,7 @@ public:
             while(!facesToProcess.empty());
         }
 
-        operation.nextProgressSubStep();
+        _progress.nextSubStep();
         std::deque<DelaunayTessellation::CellHandle> cellsToProcess;
 
         // Disjoint set data structure for merging empty regions:
@@ -208,8 +197,8 @@ public:
         std::vector<size_t> regionSizes(_emptyRegionCount);
         std::vector<double> regionVolumes(_emptyRegionCount, 0.0);
         boost::dynamic_bitset<> regionIsExterior(_emptyRegionCount);
-        std::iota(regionParents.begin(), regionParents.end(), (size_t)0);
-        std::fill(regionSizes.begin(), regionSizes.end(), 1);
+        boost::algorithm::iota(regionParents, (size_t)0);
+        boost::fill(regionSizes, 1);
 
         // "Find" part of Union-Find:
         auto findRegion = [&](size_t index) {
@@ -248,9 +237,9 @@ public:
         SurfaceMesh::region_index splitPeriodicRegion = SurfaceMesh::InvalidIndex;
 
         // Loop over all cells to cluster them.
-        operation.setProgressMaximum(_tessellation.numberOfTetrahedra() - _numFilledCells);
-        for(DelaunayTessellation::CellIterator cellIter = _tessellation.begin_cells(); cellIter != _tessellation.end_cells() && !operation.isCanceled(); ++cellIter) {
-            DelaunayTessellation::CellHandle cell = *cellIter;
+        _progress.setMaximum(_tessellation.numberOfTetrahedra() - _numFilledCells);
+        progressValue = 0;
+        for(DelaunayTessellation::CellHandle cell : _tessellation.cells()) {
 
             // Only consider finite cells.
             if(!_tessellation.isFiniteCell(cell))
@@ -288,8 +277,7 @@ public:
                 // Take next tetrahedron from the stack.
                 DelaunayTessellation::CellHandle currentCell = cellsToProcess.front();
                 cellsToProcess.pop_front();
-                if(!operation.incrementProgressValue())
-                    return false;
+                _progress.setValueIntermittent(progressValue++);
 
                 // Flags indicating whether the current tetrahedron crosses the cell boundaries.
                 std::array<bool,3> cellCrossesBoundaries = {false, false, false};
@@ -299,9 +287,9 @@ public:
 
                 // Detect whether the region is an exterior empty region, i.e. it is adjacent to the open boundaries of the simulation cell.
                 touchesOpenBoundaries |=
-                    (cellCrossesBoundaries[0] && !_tessellation.simCell()->pbcX()) ||
-                    (cellCrossesBoundaries[1] && !_tessellation.simCell()->pbcY()) ||
-                    (cellCrossesBoundaries[2] && !_tessellation.simCell()->pbcZ());
+                    (cellCrossesBoundaries[0] && !simCell.hasPbc(0)) ||
+                    (cellCrossesBoundaries[1] && !simCell.hasPbc(1)) ||
+                    (cellCrossesBoundaries[2] && !simCell.hasPbc(2));
 
                 // Stop at cells that are completely outside of the simulation box.
                 if(overlapVolume == 0)
@@ -312,7 +300,7 @@ public:
                 // that is not crossed by the surface mesh. In this case, we need to merge the two empty
                 // regions that were created separately on either side of the simulation box boundary.
                 for(size_t dim = 0; dim < 3; dim++) {
-                    if(cellCrossesBoundaries[dim] && !surfaceCrossesBoundaries[dim] && simCell->hasPbc(dim)) {
+                    if(cellCrossesBoundaries[dim] && !surfaceCrossesBoundaries[dim] && simCell.hasPbc(dim)) {
                         if(splitPeriodicRegion == SurfaceMesh::InvalidIndex)
                             splitPeriodicRegion = emptyRegion;
                         else
@@ -364,8 +352,7 @@ public:
             // Mark the region as exterior/interior empty space.
             regionIsExterior[index] |= touchesOpenBoundaries;
         }
-        if(operation.isCanceled())
-            return false;
+        this_task::throwIfCanceled();
 
         // Create the "Exterior" region property in the output mesh.
         Property* regionPropertyIsExterior = _mesh.createRegionProperty(DataBuffer::Initialized, SurfaceMeshRegions::IsExteriorProperty);
@@ -398,8 +385,7 @@ public:
         }
 
         // Update tesselation user field to reflect the compressed empty region IDs
-        for(DelaunayTessellation::CellIterator cellIter = _tessellation.begin_cells(); cellIter != _tessellation.end_cells(); ++cellIter) {
-            DelaunayTessellation::CellHandle cell = *cellIter;
+        for(DelaunayTessellation::CellHandle cell : _tessellation.cells()) {
             int currentRegionId = _tessellation.getUserField(cell);
             if(currentRegionId >= _filledRegionCount) {
                 _tessellation.setUserField(cell, regionMapping[findRegion(currentRegionId - _filledRegionCount)]);
@@ -409,34 +395,31 @@ public:
         // Create a single space-filling empty region if there is no filled region at all.
         if(_emptyRegionCount == 0 && _filledRegionCount == 0) {
             regionGrower.grow(1);
-            BufferWriteAccess<FloatType, access_mode::discard_write>{regionPropertyVolume}[0] = simCell->volume3D();
-            BufferWriteAccess<SelectionIntType, access_mode::discard_write>{regionPropertyIsExterior}[0] = (!simCell->pbcX() || !simCell->pbcY() || !simCell->pbcZ());
+            BufferWriteAccess<FloatType, access_mode::discard_write>{regionPropertyVolume}[0] = simCell.volume3D();
+            BufferWriteAccess<SelectionIntType, access_mode::discard_write>{regionPropertyIsExterior}[0] = (!simCell.hasPbc(0) || !simCell.hasPbc(1) || !simCell.hasPbc(2));
             _emptyRegionCount = 1;
         }
 
-        operation.endProgressSubSteps();
-        return !operation.isCanceled();
+        _progress.endSubSteps();
     }
 
 private:
 
     /// Assigns each tetrahedron to a region.
     template<typename CellRegionFunc>
-    bool classifyTetrahedra(CellRegionFunc&& determineCellRegion, ProgressingTask& operation)
+    void classifyTetrahedra(CellRegionFunc&& determineCellRegion)
     {
-        operation.setProgressMaximum(_tessellation.numberOfTetrahedra());
+        _progress.setMaximum(_tessellation.numberOfTetrahedra());
 
         _numFilledCells = 0;
         size_t progressCounter = 0;
         _mesh.setSpaceFillingRegion(SurfaceMesh::InvalidIndex);
         bool spaceFillingRegionUndetermined = true;
         bool isSpaceFilling = true;
-        for(DelaunayTessellation::CellIterator cellIter = _tessellation.begin_cells(); cellIter != _tessellation.end_cells(); ++cellIter) {
-            DelaunayTessellation::CellHandle cell = *cellIter;
+        for(DelaunayTessellation::CellHandle cell : _tessellation.cells()) {
 
             // Update progress indicator.
-            if(!operation.setProgressValueIntermittent(progressCounter++))
-                return false;
+            _progress.setValueIntermittent(progressCounter++);
 
             // Alpha-shape criterion: This determines whether the Delaunay tetrahedron is part of a filled region.
             bool isFilledTetrehedron = false;
@@ -447,14 +430,14 @@ private:
                 else {
                     // If the alpha test is inconclusive (which may happen if the element is a sliver tetrahedron),
                     // then we check the surrounding tetrahedra. Only if all four neighbors are classified as filled or inconclusive,
-                    // then we accept the sliver tetradron as filled too.
+                    // then we accept the sliver tetrahedron as filled too.
                     int f = 0;
                     for(; f < 4; f++) {
                         DelaunayTessellation::CellHandle adjacentCell = _tessellation.mirrorFacet(cell, f).first;
                         if(!_tessellation.isFiniteCell(adjacentCell))
                             break;
                         auto adjacentAlphaTestResult = _tessellation.alphaTest(adjacentCell, _alpha);
-                        if(adjacentAlphaTestResult && !*adjacentAlphaTestResult)
+                        if(adjacentAlphaTestResult.has_value() && !adjacentAlphaTestResult.value())
                             break;
                     }
                     if(f == 4)
@@ -496,8 +479,6 @@ private:
                 _tessellation.setCellIndex(cell, -1);
             }
         }
-
-        return !operation.isCanceled();
     }
 
     /// Computes the volume of a Delaunay tetrahedron.
@@ -511,16 +492,16 @@ private:
     }
 
     /// Aggregates Delaunay tetrahedra into connected regions.
-    bool formFilledRegions(ProgressingTask& operation)
+    void formFilledRegions()
     {
-        operation.beginProgressSubStepsWithWeights({2,3,1});
+        _progress.beginSubSteps({2,3,1});
 
         // Create a lookup map that allows retrieving the primary image of a Delaunay cell for a triangular face formed by three particles.
-        if(!createCellMap(operation))
-            return false;
+        createCellMap();
 
-        operation.nextProgressSubStep();
-        operation.setProgressMaximum(_tessellation.numberOfTetrahedra());
+        _progress.nextSubStep();
+        _progress.setMaximum(_tessellation.numberOfTetrahedra());
+        size_t progressValue = 0;
 
         // Identify connected tetrahedra to create regions if no regions were previously defined.
         if(_mesh.regionCount() == 0) {
@@ -530,8 +511,7 @@ private:
             // Loop over all Delaunay cells to cluster them into connected components.
             // All filled cells have initially a user field value of 0.
             std::deque<size_t> toProcess;
-            for(DelaunayTessellation::CellIterator cellIter = _tessellation.begin_cells(); cellIter != _tessellation.end_cells() && !operation.isCanceled(); ++cellIter) {
-                DelaunayTessellation::CellHandle cell = *cellIter;
+            for(DelaunayTessellation::CellHandle cell : _tessellation.cells()) {
 
                 // Skip empty cells and cells that have already been assigned to a region.
                 if(_tessellation.getUserField(cell) != 0)
@@ -553,8 +533,7 @@ private:
                 do {
                     DelaunayTessellation::CellHandle currentCell = toProcess.front();
                     toProcess.pop_front();
-                    if(!operation.incrementProgressValue())
-                        return false;
+                    _progress.setValueIntermittent(progressValue++);
 
                     // Add the volume of the current cell to the total region volume.
                     regionVolume += cellVolume(currentCell);
@@ -566,7 +545,7 @@ private:
                         // Note that we reverse their order to find the opposite face.
                         std::array<size_t, 3> vertices;
                         for(int v = 0; v < 3; v++)
-                            vertices[v] = _tessellation.vertexIndex(_tessellation.cellVertex(currentCell, DelaunayTessellation::cellFacetVertexIndex(f, 2-v)));
+                            vertices[v] = _tessellation.inputPointIndex(_tessellation.cellVertex(currentCell, DelaunayTessellation::cellFacetVertexIndex(f, 2-v)));
 
                         // Bring vertices into a well-defined order, which can be used as lookup key to find the adjacent tetrahedron.
                         reorderFaceVertices(vertices);
@@ -584,7 +563,7 @@ private:
                 while(!toProcess.empty());
 
                 // Create a spatial region in the output mesh.
-                SurfaceMesh::region_index regionId = regionVolumes.size();
+                Q_DECL_UNUSED SurfaceMesh::region_index regionId = regionVolumes.size();
                 regionVolumes.push_back(regionVolume);
                 OVITO_ASSERT(regionId + 1 == currentRegionId);
             }
@@ -595,10 +574,10 @@ private:
 
             if(_mesh.regionCount() > 0) {
                 // Shift filled region IDs to start at index 0.
-                for(DelaunayTessellation::CellIterator cellIter = _tessellation.begin_cells(); cellIter != _tessellation.end_cells(); ++cellIter) {
-                    int region = _tessellation.getUserField(*cellIter);
+                for(DelaunayTessellation::CellHandle cell : _tessellation.cells()) {
+                    int region = _tessellation.getUserField(cell);
                     if(region > 0)
-                        _tessellation.setUserField(*cellIter, region - 1);
+                        _tessellation.setUserField(cell, region - 1);
                 }
             }
         }
@@ -608,8 +587,8 @@ private:
 
             // Filled mesh regions have already been predefined by the caller.
             // We just need to compute the volume of each spatial region.
-            for(DelaunayTessellation::CellIterator cellIter = _tessellation.begin_cells(); cellIter != _tessellation.end_cells() && !operation.isCanceled(); ++cellIter) {
-                DelaunayTessellation::CellHandle cell = *cellIter;
+            for(DelaunayTessellation::CellHandle cell : _tessellation.cells()) {
+                this_task::throwIfCanceled();
 
                 // Skip empty cells.
                 SurfaceMesh::region_index region = _tessellation.getUserField(cell);
@@ -624,22 +603,20 @@ private:
             }
         }
 
-        operation.nextProgressSubStep();
+        _progress.nextSubStep();
 
         _filledRegionCount = _mesh.regionCount();
         if(_filledRegionCount != 0) {
             // Copy assigned region IDs from primary tetrahedra to ghost tetrahedra.
-            operation.setProgressMaximum(_tessellation.numberOfTetrahedra());
-            for(DelaunayTessellation::CellIterator cellIter = _tessellation.begin_cells(); cellIter != _tessellation.end_cells(); ++cellIter) {
-                DelaunayTessellation::CellHandle cell = *cellIter;
+            _progress.setMaximum(_tessellation.numberOfTetrahedra());
+            for(DelaunayTessellation::CellHandle cell : _tessellation.cells()) {
                 if(_tessellation.isGhostCell(cell) && _tessellation.getUserField(cell) != SurfaceMesh::InvalidIndex) {
-                    if(!operation.setProgressValueIntermittent(cell))
-                        return false;
+                    _progress.setValueIntermittent(cell);
 
                     // Get the 3 vertices of the first face of the tet.
                     std::array<size_t, 3> vertices;
                     for(int v = 0; v < 3; v++)
-                        vertices[v] = _tessellation.vertexIndex(_tessellation.cellVertex(cell, DelaunayTessellation::cellFacetVertexIndex(0, v)));
+                        vertices[v] = _tessellation.inputPointIndex(_tessellation.cellVertex(cell, DelaunayTessellation::cellFacetVertexIndex(0, v)));
 
                     // Bring vertices into a well-defined order, which can be used as lookup key.
                     reorderFaceVertices(vertices);
@@ -651,18 +628,15 @@ private:
                 }
             }
         }
-        operation.endProgressSubSteps();
-
-        return true;
+        _progress.endSubSteps();
     }
 
     /// Creates a lookup map that allows to retrieve the primary Delaunay cell image that belongs to a
     /// triangular face formed by three particles.
-    bool createCellMap(ProgressingTask& operation)
+    void createCellMap()
     {
-        operation.setProgressMaximum(_tessellation.numberOfTetrahedra());
-        for(DelaunayTessellation::CellIterator cellIter = _tessellation.begin_cells(); cellIter != _tessellation.end_cells(); ++cellIter) {
-            DelaunayTessellation::CellHandle cell = *cellIter;
+        _progress.setMaximum(_tessellation.numberOfTetrahedra());
+        for(DelaunayTessellation::CellHandle cell : _tessellation.cells()) {
 
             // Skip cells that belong to the exterior region.
             if(_tessellation.getUserField(cell) == SurfaceMesh::InvalidIndex)
@@ -673,8 +647,7 @@ private:
                 continue;
 
             // Update progress indicator.
-            if(!operation.setProgressValueIntermittent(cell))
-                return false;
+            _progress.setValueIntermittent(cell);
 
             // Loop over the 4 facets of the cell.
             for(int f = 0; f < 4; f++) {
@@ -682,24 +655,25 @@ private:
                 // Get the 3 vertices of the facet.
                 std::array<size_t, 3> vertices;
                 for(int v = 0; v < 3; v++)
-                    vertices[v] = _tessellation.vertexIndex(_tessellation.cellVertex(cell, DelaunayTessellation::cellFacetVertexIndex(f, v)));
+                    vertices[v] = _tessellation.inputPointIndex(_tessellation.cellVertex(cell, DelaunayTessellation::cellFacetVertexIndex(f, v)));
 
                 // Bring vertices into a well-defined order, which can be used as lookup key.
                 reorderFaceVertices(vertices);
 
                 OVITO_ASSERT(_cellLookupMap.find(vertices) == _cellLookupMap.end());
 
-                // Add facet and its adjacent cell to the loopup map.
+                // Add facet and its adjacent cell to the lookup map.
                 _cellLookupMap.emplace(vertices, cell);
             }
         }
-        return !operation.isCanceled();
     }
 
     /// Constructs the triangle facets that separate different regions in the tetrahedral mesh.
     template<typename PrepareMeshFaceFunc, typename PrepareMeshVertexFunc>
-    bool createInterfaceFacets(PrepareMeshFaceFunc&& prepareMeshFaceFunc, PrepareMeshVertexFunc&& prepareMeshVertexFunc, ProgressingTask& operation)
+    void createInterfaceFacets(PrepareMeshFaceFunc&& prepareMeshFaceFunc, PrepareMeshVertexFunc&& prepareMeshVertexFunc)
     {
+        const SimulationCellData simCell = _tessellation.simCell();
+
         // Stores the triangle mesh vertices created for the vertices of the tetrahedral mesh.
         std::vector<SurfaceMesh::vertex_index> vertexMap(_positions.size(), SurfaceMesh::InvalidIndex);
         _tetrahedraFaceList.clear();
@@ -711,11 +685,10 @@ private:
         // Create the per-face region array, which will dynamically grow.
         BufferWriteAccessAndRef<SurfaceMesh::region_index, access_mode::discard_write> faceRegions = SurfaceMeshFaces::OOClass().createStandardProperty(DataBuffer::Uninitialized, 0, SurfaceMeshFaces::RegionProperty);
 
-        operation.setProgressMaximum(_numFilledCells);
+        _progress.setMaximum(_numFilledCells);
         SurfaceMeshTopology* topo = _mesh.mutableTopology();
 
-        for(DelaunayTessellation::CellIterator cellIter = _tessellation.begin_cells(); cellIter != _tessellation.end_cells(); ++cellIter) {
-            DelaunayTessellation::CellHandle cell = *cellIter;
+        for(DelaunayTessellation::CellHandle cell : _tessellation.cells()) {
 
             // Consider only filled local tetrahedra.
             if(_tessellation.getCellIndex(cell) == -1)
@@ -724,19 +697,18 @@ private:
             OVITO_ASSERT(filledRegion != SurfaceMesh::InvalidIndex);
 
             // Update progress indicator.
-            if(!operation.setProgressValueIntermittent(_tessellation.getCellIndex(cell)))
-                return false;
+            _progress.setValueIntermittent(_tessellation.getCellIndex(cell));
 
             Point3 unwrappedVerts[4];
             for(int i = 0; i < 4; i++)
                 unwrappedVerts[i] = _tessellation.vertexPosition(_tessellation.cellVertex(cell, i));
 
             // Check validity of tessellation.
-            // Delaunay edges (of filled tetrahedro) should never span more than half of the simulation box in periodic directions.
+            // Delaunay edges (of filled tetrahedron) should never span more than half of the simulation box in periodic directions.
             Vector3 ad = unwrappedVerts[0] - unwrappedVerts[3];
             Vector3 bd = unwrappedVerts[1] - unwrappedVerts[3];
             Vector3 cd = unwrappedVerts[2] - unwrappedVerts[3];
-            if(_tessellation.simCell()->isWrappedVector(ad) || _tessellation.simCell()->isWrappedVector(bd) || _tessellation.simCell()->isWrappedVector(cd))
+            if(simCell.isWrappedVector(ad) || simCell.isWrappedVector(bd) || simCell.isWrappedVector(cd))
                 throw Exception("Cannot construct manifold. Simulation cell length is too small for the given probe sphere radius parameter.");
 
             // Iterate over the four faces of the tetrahedron cell.
@@ -755,7 +727,7 @@ private:
                 std::array<size_t,3> vertexIndices;
                 for(int v = 0; v < 3; v++) {
                     vertexHandles[v] = _tessellation.cellVertex(cell, DelaunayTessellation::cellFacetVertexIndex(f, _flipOrientation ? v : (2-v)));
-                    size_t vertexIndex = vertexIndices[v] = _tessellation.vertexIndex(vertexHandles[v]);
+                    size_t vertexIndex = vertexIndices[v] = _tessellation.inputPointIndex(vertexHandles[v]);
                     OVITO_ASSERT(vertexIndex < vertexMap.size());
                     if(vertexMap[vertexIndex] == SurfaceMesh::InvalidIndex) {
                         vertexMap[vertexIndex] = topo->createVertex();
@@ -779,7 +751,7 @@ private:
                     std::array<size_t,3> reverseVertexIndices;
                     for(int v = 0; v < 3; v++) {
                         vertexHandles[v] = _tessellation.cellVertex(adjacentCell, DelaunayTessellation::cellFacetVertexIndex(mirrorFacet.second, _flipOrientation ? v : (2-v)));
-                        size_t vertexIndex = reverseVertexIndices[v] = _tessellation.vertexIndex(vertexHandles[v]);
+                        size_t vertexIndex = reverseVertexIndices[v] = _tessellation.inputPointIndex(vertexHandles[v]);
                         OVITO_ASSERT(vertexIndex < vertexMap.size());
                         OVITO_ASSERT(vertexMap[vertexIndex] != SurfaceMesh::InvalidIndex);
                         facetVertices[v] = vertexMap[vertexIndex];
@@ -815,8 +787,6 @@ private:
 
         // Store the per-face region information in the mesh.
         _mesh.mutableFaces()->setContent(topo->faceCount(), { static_object_cast<Property>(faceRegions.take()) });
-
-        return !operation.isCanceled();
     }
 
     SurfaceMesh::face_index findAdjacentFace(DelaunayTessellation::CellHandle cell, int f, int e, bool reverse = false)
@@ -830,7 +800,7 @@ private:
             vertexIndex1 = DelaunayTessellation::cellFacetVertexIndex(f, (e+1)%3);
             vertexIndex2 = DelaunayTessellation::cellFacetVertexIndex(f, e);
         }
-        DelaunayTessellation::FacetCirculator circulator_start = _tessellation.incident_facets(cell, vertexIndex1, vertexIndex2, cell, f);
+        DelaunayTessellation::FacetCirculator circulator_start = _tessellation.incidentFacets(cell, vertexIndex1, vertexIndex2, cell, f);
         DelaunayTessellation::FacetCirculator circulator = circulator_start;
         OVITO_ASSERT((*circulator).first == cell);
         OVITO_ASSERT((*circulator).second == f);
@@ -870,25 +840,24 @@ private:
         return adjacentFace;
     }
 
-    bool linkHalfedges(ProgressingTask& operation)
+    void linkHalfedges()
     {
-        operation.setProgressMaximum(_tetrahedraFaceList.size());
+        _progress.setMaximum(_tetrahedraFaceList.size());
 
 #ifdef OVITO_DEBUG
         BufferReadAccess<SurfaceMesh::region_index> faceRegions(_mesh.expectFaceProperty(SurfaceMeshFaces::RegionProperty));
 #endif
 
         auto tet = _tetrahedraFaceList.cbegin();
-        for(DelaunayTessellation::CellIterator cellIter = _tessellation.begin_cells(); cellIter != _tessellation.end_cells(); ++cellIter) {
-            DelaunayTessellation::CellHandle cell = *cellIter;
+        for(DelaunayTessellation::CellHandle cell : _tessellation.cells()) {
 
             // Look for filled cells being adjacent to at least one mesh face.
-            if(_tessellation.getCellIndex(cell) == -1) continue;
+            if(_tessellation.getCellIndex(cell) == -1)
+                continue;
             OVITO_ASSERT(_tetrahedraFaceList.cbegin() + _tessellation.getCellIndex(cell) == tet);
 
             // Update progress indicator.
-            if(!operation.setProgressValueIntermittent(_tessellation.getCellIndex(cell)))
-                return false;
+            _progress.setValueIntermittent(_tessellation.getCellIndex(cell));
 
             // Visit the mesh faces adjacent to the current cell.
             for(int f = 0; f < 4; f++) {
@@ -938,7 +907,8 @@ private:
 
         // Set up manifold pointers at edges of the mesh.
         if(_createRegions) {
-            for(SurfaceMesh::edge_index edge1 = 0; edge1 < _mesh.edgeCount() && !operation.isCanceled(); edge1++) {
+            for(SurfaceMesh::edge_index edge1 = 0; edge1 < _mesh.edgeCount(); edge1++) {
+                this_task::throwIfCanceled();
                 // Link surface manifolds.
                 SurfaceMesh::edge_index oppositeEdge = _mesh.oppositeEdge(edge1);
                 SurfaceMesh::face_index adjacentFace = _mesh.adjacentFace(oppositeEdge);
@@ -951,8 +921,6 @@ private:
                 }
             }
         }
-
-        return !operation.isCanceled();
     }
 
     SurfaceMesh::face_index findCellFace(const std::pair<DelaunayTessellation::CellHandle,int>& facet)
@@ -967,7 +935,7 @@ private:
             std::array<size_t,3> faceVerts;
             for(size_t i = 0; i < 3; i++) {
                 int vertexIndex = DelaunayTessellation::cellFacetVertexIndex(facet.second, _flipOrientation ? i : (2-i));
-                faceVerts[i] = _tessellation.vertexIndex(_tessellation.cellVertex(cell, vertexIndex));
+                faceVerts[i] = _tessellation.inputPointIndex(_tessellation.cellVertex(cell, vertexIndex));
             }
             reorderFaceVertices(faceVerts);
             if(auto item = _faceLookupMap.find(faceVerts); item != _faceLookupMap.end())
@@ -997,7 +965,7 @@ private:
         Point3 reducedVertexPositions[4];
         for(int v = 0; v < 4; v++) {
             const Point3& vpos = vertexPositions[v] = _tessellation.vertexPosition(_tessellation.cellVertex(cell, v));
-            const Point3& rp = reducedVertexPositions[v] = _tessellation.simCell()->absoluteToReduced(vpos);
+            const Point3& rp = reducedVertexPositions[v] = _tessellation.simCell().absoluteToReduced(vpos);
             if(rp.x() < 0.0 || rp.x() > 1.0) {
                 isCompletelyInsideBox = false;
                 outsideDir[0] = true;
@@ -1131,6 +1099,9 @@ private:
         SurfaceMeshBuilder meshBuilder(_convexHullMesh);
         meshBuilder.clearMesh();
         meshBuilder.constructConvexHull(std::vector<Point3>(lineSegments, lineSegments + numPoints), SurfaceMesh::InvalidIndex);
+#ifdef OVITO_DEBUG
+        _convexHullMesh->verifyMeshIntegrity();
+#endif
 
         // The convex hull may be empty if the input point set is degenerate.
         if(meshBuilder.faceCount() == 0)
@@ -1155,7 +1126,7 @@ private:
             }
         }
 
-        return (convexVolume / 6.0) * _tessellation.simCell()->volume3D();
+        return (convexVolume / 6.0) * _tessellation.simCell().volume3D();
     }
 
 private:
@@ -1170,7 +1141,7 @@ private:
     /// of a two-sided surface mesh.
     bool _createRegions;
 
-    /// Counts the number of tetrehedral cells that belong to the filled regions.
+    /// Counts the number of tetrahedral cells that belong to the filled regions.
     size_t _numFilledCells = 0;
 
     /// Number of filled regions that have been identified.
@@ -1199,6 +1170,9 @@ private:
 
     /// Working data structure used in calculateVolumeOverlap() for computing the volume of a truncated tetrahedral cell.
     DataOORef<SurfaceMesh> _convexHullMesh;
+
+    /// The progress reporting channel.
+    TaskProgress& _progress;
 };
 
 }   // End of namespace
