@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -28,7 +28,7 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(LAMMPSGridDumpImporter);
+IMPLEMENT_CREATABLE_OVITO_CLASS(LAMMPSGridDumpImporter);
 
 /******************************************************************************
 * Checks if the given file has format that can be read by this importer.
@@ -60,17 +60,19 @@ bool LAMMPSGridDumpImporter::OOMetaClass::checkFileFormat(const FileHandle& file
 /******************************************************************************
 * Scans the data file and builds a list of source frames.
 ******************************************************************************/
-void LAMMPSGridDumpImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::Frame>& frames)
+void LAMMPSGridDumpImporter::discoverFramesInFile(const FileHandle& fileHandle, QVector<FileSourceImporter::Frame>& frames) const
 {
-    CompressedTextReader stream(fileHandle());
-    setProgressText(tr("Scanning LAMMPS grid dump file %1").arg(fileHandle().toString()));
-    setProgressMaximum(stream.underlyingSize());
+    CompressedTextReader stream(fileHandle);
+
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Scanning LAMMPS grid dump file %1").arg(fileHandle.toString()));
+    progress.setMaximum(stream.underlyingSize());
 
     unsigned long long timestep = 0;
     size_t numVoxels = 0;
-    Frame frame(fileHandle());
+    Frame frame(fileHandle);
 
-    while(!stream.eof() && !isCanceled()) {
+    while(!stream.eof() && !this_task::isCanceled()) {
         qint64 byteOffset = stream.byteOffset();
         int lineNumber = stream.lineNumber();
 
@@ -105,8 +107,8 @@ void LAMMPSGridDumpImporter::FrameFinder::discoverFramesInFile(QVector<FileSourc
             else if(stream.lineStartsWith("ITEM: GRID CELLS")) {
                 for(size_t i = 0; i < numVoxels; i++) {
                     stream.readLine();
-                    if(!setProgressValueIntermittent(stream.underlyingByteOffset()))
-                        return;
+                    // Update progress bar and check for cancellation.
+                    progress.setValueIntermittent(stream.underlyingByteOffset());
                 }
                 break;
             }
@@ -132,7 +134,8 @@ void LAMMPSGridDumpImporter::FrameFinder::discoverFramesInFile(QVector<FileSourc
 ******************************************************************************/
 void LAMMPSGridDumpImporter::FrameLoader::loadFile()
 {
-    setProgressText(tr("Reading LAMMPS grid dump file %1").arg(fileHandle().toString()));
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Reading LAMMPS grid dump file %1").arg(fileHandle().toString()));
 
     // Open file for reading.
     CompressedTextReader stream(fileHandle(), frame().byteOffset, frame().lineNumber);
@@ -225,7 +228,7 @@ void LAMMPSGridDumpImporter::FrameLoader::loadFile()
                     throw Exception(tr("LAMMPS grid dump file parsing error. Invalid grid size in line %1:\n%2").arg(stream.lineNumber()).arg(stream.lineString()));
                 numVoxels = (size_t)nx * (size_t)ny * (size_t)nz;
                 gridDims = {nx, ny, nz};
-                setProgressMaximum(numVoxels);
+                progress.setMaximum(numVoxels);
                 break;
             }
             else if(stream.lineStartsWith("ITEM: GRID CELLS")) {
@@ -272,7 +275,7 @@ void LAMMPSGridDumpImporter::FrameLoader::loadFile()
                             gridIdentifier = tokens[1];
                     }
 
-                    columnMapping.mapCustomColumn(i, Property::makePropertyNameValid(propertyName), dataType, vectorComponent);
+                    columnMapping.mapColumnToUserProperty(i, Property::makePropertyNameValid(propertyName), dataType, vectorComponent);
                     columnMapping[i].columnName = fileColumnNames[i];
                 }
 
@@ -297,7 +300,8 @@ void LAMMPSGridDumpImporter::FrameLoader::loadFile()
                 int lineNumber = stream.lineNumber() + 1;
                 try {
                     for(size_t i = 0; i < numVoxels; i++, lineNumber++) {
-                        if(!setProgressValueIntermittent(i)) return;
+                        // Update progress bar and check for cancellation.
+                        progress.setValueIntermittent(i);
                         if(!s)
                             columnParser.readElement(i, stream.readLine());
                         else
@@ -330,7 +334,7 @@ void LAMMPSGridDumpImporter::FrameLoader::loadFile()
             else if(stream.lineStartsWith("ITEM:")) {
                 // For the sake of forward compatibility, we ignore unknown ITEM sections.
                 // Skip lines until the next "ITEM:" is reached.
-                while(!stream.eof() && !isCanceled()) {
+                while(!stream.eof() && !this_task::isCanceled()) {
                     stream.readLine();
                     if(stream.lineStartsWith("ITEM:"))
                         break;
