@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -29,6 +29,7 @@
 #include <ovito/gui/base/viewport/ViewportInputMode.h>
 #include <ovito/gui/base/viewport/ViewportInputManager.h>
 #include <ovito/core/app/UserInterface.h>
+#include <ovito/core/app/Application.h>
 #include <ovito/core/app/undo/UndoStack.h>
 #include <ovito/gui/base/actions/ViewportModeAction.h>
 #include "ActionManager.h"
@@ -42,17 +43,18 @@ ActionManager::ActionManager(QObject* parent, UserInterface& userInterface) : QA
 {
     // Actions need to be updated whenever a new dataset is loaded or the current selection changes.
     connect(&userInterface.datasetContainer(), &DataSetContainer::dataSetChanged, this, &ActionManager::onDataSetChanged);
-    connect(&userInterface.datasetContainer(), &DataSetContainer::animationSettingsReplaced, this, &ActionManager::onAnimationSettingsReplaced);
     connect(&userInterface.datasetContainer(), &DataSetContainer::selectionChangeComplete, this, &ActionManager::onSelectionChangeComplete);
-    connect(&userInterface.datasetContainer(), &DataSetContainer::viewportConfigReplaced, this, &ActionManager::onViewportConfigurationReplaced);
+    connect(&userInterface.datasetContainer(), &DataSetContainer::animationIntervalChanged, this, &ActionManager::onAnimationIntervalChanged);
+    connect(&userInterface.datasetContainer(), &DataSetContainer::maximizedViewportChanged, this, &ActionManager::onMaximizedViewportChanged);
+    connect(&userInterface.datasetContainer(), &DataSetContainer::viewportLayoutChanged, this, &ActionManager::onViewportLayoutChanged);
 
     createCommandAction(ACTION_QUIT, tr("Quit"), "file_quit", tr("Quit the application."));
     createCommandAction(ACTION_FILE_OPEN, tr("Load Session State..."), "file_open", tr("Load a previously saved session from a file."), QKeySequence::Open);
     createCommandAction(ACTION_FILE_SAVE, tr("Save Session State"), "file_save", tr("Save the current program session to a file."), QKeySequence::Save);
     createCommandAction(ACTION_FILE_SAVEAS, tr("Save Session State As..."), "file_save_as", tr("Save the current program session to a new file."), QKeySequence::SaveAs);
-    createCommandAction(ACTION_FILE_IMPORT, tr("Load File..."), "file_import", tr("Import data from a file on this computer."), Qt::Key_I | Qt::CTRL);
-    createCommandAction(ACTION_FILE_REMOTE_IMPORT, tr("Load Remote File"), "file_import_remote", tr("Import a file from a remote location."), Qt::Key_I | Qt::CTRL | Qt::SHIFT);
-    createCommandAction(ACTION_FILE_EXPORT, tr("Export File..."), "file_export", tr("Export data to a file."), Qt::Key_E | Qt::CTRL);
+    createCommandAction(ACTION_FILE_IMPORT, tr("Load File..."), "file_import", tr("Import data from a file on this computer."), QKeyCombination(Qt::CTRL, Qt::Key_I));
+    createCommandAction(ACTION_FILE_REMOTE_IMPORT, tr("Load Remote File"), "file_import_remote", tr("Import a file from a remote location."), QKeyCombination(Qt::CTRL | Qt::SHIFT, Qt::Key_I));
+    createCommandAction(ACTION_FILE_EXPORT, tr("Export File..."), "file_export", tr("Export data to a file."), QKeyCombination(Qt::CTRL, Qt::Key_E));
     createCommandAction(ACTION_FILE_NEW_WINDOW, tr("New Program Window"), "file_new_window", tr("Open another OVITO program window."), QKeySequence::New);
     createCommandAction(ACTION_HELP_ABOUT, tr("About OVITO"), "application_about", tr("Show information about this software."));
     createCommandAction(ACTION_HELP_SHOW_ONLINE_HELP, tr("User Manual"), "help_user_manual", tr("Open the OVITO user manual."), QKeySequence::HelpContents);
@@ -91,6 +93,9 @@ ActionManager::ActionManager(QObject* parent, UserInterface& userInterface) : QA
 #ifndef OVITO_BUILD_PROFESSIONAL
     createNewPipelineAcion->setText(createNewPipelineAcion->text() + QStringLiteral(" (Pro)"));
     clonePipelineAction->setText(clonePipelineAction->text() + QStringLiteral(" (Pro)"));
+#else
+    (void)createNewPipelineAcion;
+    (void)clonePipelineAction;
 #endif
     createCommandAction(ACTION_EDIT_RENAME_PIPELINE, tr("Rename Pipeline..."), "edit_rename_pipeline", tr("Assign a new name to the selected pipeline."));
     createCommandAction(ACTION_EDIT_DELETE, tr("Delete Pipeline"), "edit_delete_pipeline", tr("Delete the selected object from the scene."));
@@ -109,6 +114,7 @@ ActionManager::ActionManager(QObject* parent, UserInterface& userInterface) : QA
     createCommandAction(ACTION_VIEWPORT_ZOOM_SCENE_EXTENTS_ALL, tr("Zoom Scene Extents All"), nullptr, tr("Zoom all viewports to show everything."));
     createCommandAction(ACTION_VIEWPORT_ZOOM_SELECTION_EXTENTS, tr("Zoom Selection Extents"), nullptr, tr("Zoom active viewport to show the selected objects."));
     createCommandAction(ACTION_VIEWPORT_ZOOM_SELECTION_EXTENTS_ALL, tr("Zoom Selection Extents All"), nullptr, tr("Zoom all viewports to show the selected objects."));
+    createCommandAction(ACTION_CONFIGURE_VIEWPORT_GRAPHICS, tr("Configure Graphics..."), nullptr, tr("Change graphics settings for real-time interactive viewports."));
 
     if(ViewportInputManager* vpInputManager = userInterface.viewportInputManager()) {
         createViewportModeAction(ACTION_VIEWPORT_ZOOM, vpInputManager->zoomMode(), tr("Zoom"), "viewport_mode_zoom", tr("Activate zoom mode."));
@@ -121,8 +127,8 @@ ActionManager::ActionManager(QObject* parent, UserInterface& userInterface) : QA
 
     createCommandAction(ACTION_GOTO_START_OF_ANIMATION, tr("Go to Start of Animation"), "animation_goto_start", tr("Jump to first frame of the animation."), Qt::Key_Home);
     createCommandAction(ACTION_GOTO_END_OF_ANIMATION, tr("Go to End of Animation"), "animation_goto_end", tr("Jump to the last frame of the animation."), Qt::Key_End);
-    createCommandAction(ACTION_GOTO_PREVIOUS_FRAME, tr("Go to Previous Frame"), "animation_goto_previous_frame", tr("Move time slider one animation frame backward."), Qt::Key_Left | Qt::ALT);
-    createCommandAction(ACTION_GOTO_NEXT_FRAME, tr("Go to Next Frame"), "animation_goto_next_frame", tr("Move time slider one animation frame forward."), Qt::Key_Right | Qt::ALT);
+    createCommandAction(ACTION_GOTO_PREVIOUS_FRAME, tr("Go to Previous Frame"), "animation_goto_previous_frame", tr("Move time slider one animation frame backward."), QKeyCombination(Qt::ALT, Qt::Key_Left));
+    createCommandAction(ACTION_GOTO_NEXT_FRAME, tr("Go to Next Frame"), "animation_goto_next_frame", tr("Move time slider one animation frame forward."), QKeyCombination(Qt::ALT, Qt::Key_Right));
     createCommandAction(ACTION_START_ANIMATION_PLAYBACK, tr("Start Animation Playback"), "animation_play", tr("Start playing the animation in the viewports."));
     createCommandAction(ACTION_STOP_ANIMATION_PLAYBACK, tr("Stop Animation Playback"), "animation_stop", tr("Stop playing the animation in the viewports."));
     createCommandAction(ACTION_ANIMATION_SETTINGS, tr("Animation Settings"), "animation_settings", tr("Open the animation settings dialog."));
@@ -162,21 +168,6 @@ void ActionManager::onDataSetChanged(DataSet* newDataSet)
     getAction(ACTION_AUTO_KEY_MODE_TOGGLE)->setChecked(false);
 }
 
-/******************************************************************************
-* This is called when new animation settings have been loaded.
-******************************************************************************/
-void ActionManager::onAnimationSettingsReplaced(AnimationSettings* newAnimationSettings)
-{
-    disconnect(_animationIntervalChangedConnection);
-    if(newAnimationSettings) {
-        _animationIntervalChangedConnection = connect(newAnimationSettings, &AnimationSettings::intervalChanged, this, &ActionManager::onAnimationIntervalChanged);
-        onAnimationIntervalChanged(newAnimationSettings->firstFrame(), newAnimationSettings->lastFrame());
-    }
-    else {
-        onAnimationIntervalChanged(0, 0);
-    }
-}
-
 /******************************************************************************
 * This is called when the active animation interval has changed.
 ******************************************************************************/
@@ -194,21 +185,19 @@ void ActionManager::onAnimationIntervalChanged(int firstFrame, int lastFrame)
 }
 
 /******************************************************************************
-* This is called when new viewport configuration has been loaded.
+* This is called when a different viewport become the maximized one.
 ******************************************************************************/
-void ActionManager::onViewportConfigurationReplaced(ViewportConfiguration* newViewportConfiguration)
+void ActionManager::onMaximizedViewportChanged(Viewport* maximizedViewport)
 {
-    disconnect(_maximizedViewportChangedConnection);
-    QAction* maximizeViewportAction = getAction(ACTION_VIEWPORT_MAXIMIZE);
-    if(newViewportConfiguration) {
-        maximizeViewportAction->setChecked(newViewportConfiguration->maximizedViewport() != nullptr);
-        _maximizedViewportChangedConnection = connect(newViewportConfiguration, &ViewportConfiguration::maximizedViewportChanged, maximizeViewportAction, [maximizeViewportAction](Viewport* maximizedViewport) {
-            maximizeViewportAction->setChecked(maximizedViewport != nullptr);
-        });
-    }
-    else {
-        maximizeViewportAction->setChecked(false);
-    }
+    getAction(ACTION_VIEWPORT_MAXIMIZE)->setChecked(maximizedViewport != nullptr);
+}
+
+/******************************************************************************
+* This is called when the viewport layout changes.
+******************************************************************************/
+void ActionManager::onViewportLayoutChanged(ViewportConfiguration* viewportConfig)
+{
+    getAction(ACTION_VIEWPORT_MAXIMIZE)->setEnabled(viewportConfig && viewportConfig->layoutRootCell() && !viewportConfig->layoutRootCell()->children().empty());
 }
 
 /******************************************************************************
@@ -227,7 +216,7 @@ void ActionManager::onSelectionChangeComplete(SelectionSet* selection)
 void ActionManager::addAction(QAction* action)
 {
     OVITO_CHECK_POINTER(action);
-    OVITO_ASSERT_MSG(action->parent() == this || findAction(action->objectName()) == nullptr, "ActionManager::addAction()", "There is already an action with the same ID.");
+    OVITO_ASSERT_MSG(action->parent() == this || findAction(action->objectName()) == nullptr, "ActionManager::addAction()", qPrintable(QStringLiteral("There is already an action with the same ID: %1").arg(action->objectName())));
     OVITO_ASSERT(!_actions.contains(action));
 
     // Make the action a child of this object.
@@ -276,9 +265,9 @@ QAction* ActionManager::createCommandAction(const QString& id, const QString& ti
 /******************************************************************************
 * Creates and registers a new viewport mode action with the ActionManager.
 ******************************************************************************/
-QAction* ActionManager::createViewportModeAction(const QString& id, ViewportInputMode* inputHandler, const QString& title, const char* iconPath, const QString& statusTip, const QKeySequence& shortcut)
+QAction* ActionManager::createViewportModeAction(const QString& id, OORef<ViewportInputMode> inputHandler, const QString& title, const char* iconPath, const QString& statusTip, const QKeySequence& shortcut)
 {
-    QAction* action = new ViewportModeAction(userInterface(), title, this, inputHandler);
+    QAction* action = new ViewportModeAction(userInterface(), title, this, std::move(inputHandler));
     action->setObjectName(id);
     if(!shortcut.isEmpty())
         action->setShortcut(shortcut);
@@ -353,8 +342,8 @@ void ActionManager::on_EditDelete_triggered()
         // Get active scene.
         if(Scene* scene = userInterface().datasetContainer().activeScene()) {
             // Delete all nodes in selection set.
-            for(SceneNode* node : scene->selection()->nodes())
-                node->deleteSceneNode();
+            while(!scene->selection()->nodes().empty())
+                scene->selection()->nodes().front()->requestObjectDeletion();
 
             // Automatically select one of the remaining nodes.
             if(scene->children().isEmpty() == false)
@@ -363,6 +352,28 @@ void ActionManager::on_EditDelete_triggered()
     });
 }
 
+/******************************************************************************
+* Handles user clicks on an action link.
+******************************************************************************/
+void ActionManager::handleActionLink(const QString& link)
+{
+    if(link.startsWith("action:")) {
+        QString actionId = link.mid(7);
+        if(QAction* action = findAction(actionId)) {
+            action->trigger();
+        }
+        else {
+            userInterface().reportError(tr("Action not found: %1").arg(actionId));
+        }
+    }
+    else if(link.startsWith("manual:")) {
+        openHelpTopic(link);
+    }
+    else {
+        userInterface().reportError(tr("Invalid action link: %1").arg(link));
+    }
+}
+
 /******************************************************************************
 * Shows the online manual and opens the given help page.
 ******************************************************************************/
@@ -370,7 +381,7 @@ void ActionManager::openHelpTopic(const QString& helpTopicId)
 {
     // Determine the filesystem path where OVITO's documentation files are installed.
 #ifndef Q_OS_WASM
-    QDir prefixDir(QCoreApplication::applicationDirPath());
+    QDir prefixDir(Application::instance()->applicationDirPath());
     QDir helpDir = QDir(prefixDir.absolutePath() + QChar('/') + QStringLiteral(OVITO_DOCUMENTATION_PATH));
     QUrl url;
 #else
@@ -437,7 +448,7 @@ void ActionManager::openHelpTopic(const QString& helpTopicId)
     // Solution is to generate a temporary HTML file which redirects to the actual help page including the # fragment.
     // See also https://forums.madcapsoftware.com/viewtopic.php?f=9&t=28376#p130613
     // and https://stackoverflow.com/questions/26305322/shellexecute-fails-for-local-html-or-file-urls
-#if defined(Q_OS_WIN) || defined(Q_OS_MAC)
+#if defined(Q_OS_WIN) || defined(Q_OS_MACOS)
     if(url.isLocalFile() && url.hasFragment()) {
         static QTemporaryFile* temporaryHtmlFile = nullptr;
         if(temporaryHtmlFile)
@@ -447,7 +458,7 @@ void ActionManager::openHelpTopic(const QString& helpTopicId)
             // Write a small HTML file that just contains a redirect directive to the actual help page including the # fragment.
             QTextStream(temporaryHtmlFile) << QStringLiteral("<html><meta http-equiv=Refresh content=\"0; url=%1\"><body><a href=\"%1\">Continue to help topic...</a></body></html>").arg(url.toString(QUrl::FullyEncoded));
             temporaryHtmlFile->close();
-            // Let the web brwoser ppen the redirect page instead of the original help page.
+            // Let the web browser open the redirect page instead of the original help page.
             url = QUrl::fromLocalFile(temporaryHtmlFile->fileName());
         }
     }
