@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -55,7 +55,7 @@ PipelineListModel::PipelineListModel(UserInterface& userInterface, QObject* pare
     _selectionModel = new QItemSelectionModel(this);
 
     // Connect signals and slots.
-    connect(&_selectedPipeline, &RefTargetListener<Pipeline>::notificationEvent, this, &PipelineListModel::onPipelineEvent);
+    _selectedPipeline.connect(this, &PipelineListModel::onPipelineEvent);
     connect(&userInterface.datasetContainer(), &DataSetContainer::selectionChangeComplete, this, &PipelineListModel::onSceneSelectionChangeComplete);
     connect(_selectionModel, &QItemSelectionModel::selectionChanged, this, &PipelineListModel::onSelectionModelChanged);
     connect(this, &PipelineListModel::selectedItemChanged, this, &PipelineListModel::updateActions);
@@ -120,7 +120,8 @@ PipelineListItem* PipelineListModel::selectedItem() const
 ******************************************************************************/
 void PipelineListModel::onSceneSelectionChangeComplete(SelectionSet* selection)
 {
-    Pipeline* pipeline = selection ? dynamic_object_cast<Pipeline>(selection->firstNode()) : nullptr;
+    _selectedSceneNode = selection ? selection->firstNode() : nullptr;
+    Pipeline* pipeline = _selectedSceneNode ? _selectedSceneNode->pipeline() : nullptr;
     if(pipeline != selectedPipeline()) {
         _selectedPipeline.setTarget(pipeline);
         if(pipeline)
@@ -232,7 +233,7 @@ void PipelineListModel::refreshList()
 
     // Determine the currently selected objects and select them again after the list has been rebuilt.
     // _nextObjectToSelect may have been set to replace the selection.
-    if(!_nextObjectToSelect && _previouslySelectedPipeline.data() == selectedPipeline()) {
+    if(!_nextObjectToSelect && _previouslySelectedPipeline.lock().get() == selectedPipeline()) {
         for(const QModelIndex& idx : _selectionModel->selectedRows()) {
             OVITO_ASSERT(idx.isValid() && idx.row() < items().size());
             _previouslySelectedItems.push_back(items()[idx.row()]);
@@ -322,19 +323,23 @@ void PipelineListModel::refreshList()
 
 /******************************************************************************
 * Create the pipeline editor entries for the subjects of the given
-* object (and their subobjects).
+* object (and their sub-objects).
 ******************************************************************************/
 void PipelineListModel::createListItemsForSubobjects(const DataObject* dataObj, PipelineListItem* parentItem)
 {
-    if(dataObj->showInPipelineEditor() && dataObj->editableProxy()) {
+    DataObject::PipelineEditorObjectListMode mode = dataObj->pipelineEditorObjectListMode();
+
+    if(dataObj->editableProxy() && (mode == DataObject::PipelineEditorObjectListMode::Show || mode == DataObject::PipelineEditorObjectListMode::ShowIncludingSubObjects)) {
         parentItem = appendListItem(dataObj->editableProxy(), PipelineListItem::DataObject, parentItem);
     }
 
-    // Recursively visit the sub-objects of the data object.
-    dataObj->visitSubObjects([&](const DataObject* subObject) {
-        createListItemsForSubobjects(subObject, parentItem);
-        return false;
-    });
+    if(mode == DataObject::PipelineEditorObjectListMode::ShowIncludingSubObjects || mode == DataObject::PipelineEditorObjectListMode::Hide) {
+        // Recursively visit the sub-objects of the data object.
+        dataObj->visitSubObjects([&](const DataObject* subObject) {
+            createListItemsForSubobjects(subObject, parentItem);
+            return false;
+        });
+    }
 }
 
 /******************************************************************************
@@ -407,11 +412,11 @@ PipelineListItem* PipelineListModel::appendListItem(RefTarget* object, PipelineL
 }
 
 /******************************************************************************
-* Handles notification events generated by the selected pipeline node.
+* Handles notification events generated by the selected pipeline.
 ******************************************************************************/
 void PipelineListModel::onPipelineEvent(RefTarget* source, const ReferenceEvent& event)
 {
-    // Update the entire modification list if the PipelineSceneNode has been assigned a new
+    // Update the entire modification list if the Pipeline has been assigned a new
     // data object, or if the list of visual elements has changed.
     if(event.type() == ReferenceEvent::ReferenceChanged
         || event.type() == ReferenceEvent::ReferenceAdded
@@ -468,7 +473,7 @@ void PipelineListModel::applyModifiers(const QVector<OORef<Modifier>>& modifiers
                 modNode->setModifier(modifier);
                 modNode->setInput(pnode);
                 modNode->setModifierGroup(modifierGroup);
-                modifier->initializeModifier(ModifierInitializationRequest(time, modNode));
+                modifier->initializeModifier(ModifierInitializationRequest(time, false, true, modNode));
                 setNextObjectToSelect(modNode);
                 for(RefMaker* dependent : dependentsList) {
                     if(ModificationNode* predecessorModNode = dynamic_object_cast<ModificationNode>(dependent)) {
@@ -489,7 +494,7 @@ void PipelineListModel::applyModifiers(const QVector<OORef<Modifier>>& modifiers
 
     // Insert modifiers at the end of the selected pipelines.
     for(int index = modifiers.size() - 1; index >= 0; --index) {
-        ModificationNode* modNode = selectedPipeline()->applyModifier(time, modifiers[index]);
+        ModificationNode* modNode = selectedPipeline()->applyModifier(time, true, modifiers[index]);
         if(group)
             modNode->setModifierGroup(group);
         else
@@ -551,7 +556,7 @@ void PipelineListModel::deleteModificationNode(ModificationNode* node)
                 }
             }
         });
-        node->deleteReferenceObject();
+        node->requestObjectDeletion();
     });
 
     // Invalidate the items list of the model.
@@ -579,132 +584,187 @@ void PipelineListModel::iconAnimationFrameChanged()
 ******************************************************************************/
 QVariant PipelineListModel::data(const QModelIndex& index, int role) const
 {
-    OVITO_ASSERT(index.row() >= 0 && index.row() < _items.size());
+    QModelRoleData roleData(role);
+    multiData(index, roleData);
+    return roleData.data();
+}
 
+/******************************************************************************
+* Fills the roleDataSpan with the requested data for the given index.
+******************************************************************************/
+void PipelineListModel::multiData(const QModelIndex& index, QModelRoleDataSpan roleDataSpan) const
+{
+    OVITO_ASSERT(index.row() >= 0 && index.row() < _items.size());
     PipelineListItem* item = this->item(index.row());
 
-    if(role == Qt::DisplayRole || role == TitleRole) {
-        // Indent modifiers that are part of a group.
-        if(item->itemType() == PipelineListItem::Modifier) {
-            if(ModificationNode* modNode = dynamic_object_cast<ModificationNode>(item->object())) {
-                if(modNode->modifierGroup())
+    for(QModelRoleData& roleData : roleDataSpan) {
+        int role = roleData.role();
+        if(role == Qt::DisplayRole || role == TitleRole) {
+            // Indent modifiers that are part of a group.
+            if(item->itemType() == PipelineListItem::Modifier) {
+                if(ModificationNode* modNode = dynamic_object_cast<ModificationNode>(item->object())) {
+                    if(modNode->modifierGroup()) {
 #ifndef Q_OS_WIN
-                    return QStringLiteral(" ") + item->title();
+                        roleData.setData(QStringLiteral(" ") + item->title());
 #else
-                    return QStringLiteral("   ") + item->title();
+                        roleData.setData(QStringLiteral("   ") + item->title());
 #endif
+                        continue;
+                    }
+                }
+            }
+            roleData.setData(item->title());
+            continue;
+        }
+        else if(role == Qt::EditRole) {
+            roleData.setData(item->title());
+            continue;
+        }
+        else if(role == ItemTypeRole) {
+            roleData.setData(item->itemType());
+            continue;
+        }
+        else if(role == IsCollapsedRole) {
+            if(item->itemType() == PipelineListItem::ModifierGroup) {
+                roleData.setData(static_object_cast<ModifierGroup>(item->object())->isCollapsed());
+                continue;
             }
         }
-        return item->title();
-    }
-    else if(role == Qt::EditRole) {
-        return item->title();
-    }
-    else if(role == ItemTypeRole) {
-        return item->itemType();
-    }
-    else if(role == IsCollapsedRole) {
-        if(item->itemType() == PipelineListItem::ModifierGroup)
-            return static_object_cast<ModifierGroup>(item->object())->isCollapsed();
-    }
-    else if(role == StatusInfoRole) {
-        if(Pipeline* pipeline = selectedPipeline()) {
-            QVariant v;
-            if(_userInterface.handleExceptions([&] {
-                v = item->shortInfo(pipeline);
-            })) return v;
-        }
-    }
-    else if(role == Qt::DecorationRole) {
-        // This role is only used by the QWidgets GUI.
-        if(item->itemType() == PipelineListItem::ModifierGroup) {
-            if(!static_object_cast<ModifierGroup>(item->object())->isCollapsed())
-                return _modifierGroupExpanded;
-        }
-        if(item->isObjectActive()) {
-            const_cast<QMovie&>(_statusPendingIcon).start();
-            return QVariant::fromValue(_statusPendingIcon.currentPixmap());
-        }
-        if(item->itemType() == PipelineListItem::ModifierGroup) {
-            if(item->status().type() == PipelineStatus::Success)
-                return _modifierGroupCollapsed;
-        }
-        if(item->isObjectItem()) {
-            switch(item->status().type()) {
-            case PipelineStatus::Warning: return QVariant::fromValue(_statusWarningIcon);
-            case PipelineStatus::Error: return QVariant::fromValue(_statusErrorIcon);
-            default: return QVariant::fromValue(_statusNoneIcon);
+        else if(role == StatusInfoRole) {
+            if(dynamic_object_cast<ActiveObject>(item->object())) {
+                if(SceneNode* sceneNode = selectedSceneNode()) {
+                    if(_userInterface.handleExceptions<true>([&] {
+                        roleData.setData(item->shortInfo(sceneNode));
+                    })) continue;
+                }
             }
         }
-    }
-    else if(role == PipelineListModel::DecorationRole) {
-        // This role is only used by the QML GUI.
-        if(item->itemType() == PipelineListItem::ModifierGroup) {
-            if(!static_object_cast<ModifierGroup>(item->object())->isCollapsed())
-                return QStringLiteral("modify_modifier_group_expanded");
-        }
-        if(item->itemType() == PipelineListItem::ModifierGroup) {
-            if(item->status().type() == PipelineStatus::Success)
-                return QStringLiteral("modify_modifier_group_collapsed");
-        }
-        if(item->isObjectItem()) {
-            switch(item->status().type()) {
-            case PipelineStatus::Warning: return QStringLiteral("qrc:/guibase/mainwin/status/status_warning.png");
-            case PipelineStatus::Error: return QStringLiteral("qrc:/guibase/mainwin/status/status_error.png");
-            default: return QStringLiteral("qrc:/guibase/mainwin/status/status_none.png");
+        else if(role == Qt::DecorationRole) {
+            // This role is only used by the QWidgets GUI.
+            if(item->itemType() == PipelineListItem::ModifierGroup) {
+                if(!static_object_cast<ModifierGroup>(item->object())->isCollapsed()) {
+                    roleData.setData(_modifierGroupExpanded);
+                    continue;
+                }
+            }
+            if(item->isObjectActive()) {
+                _statusPendingIcon.start();
+                roleData.setData(_statusPendingIcon.currentPixmap());
+                continue;
+            }
+            if(item->itemType() == PipelineListItem::ModifierGroup) {
+                if(item->status().type() == PipelineStatus::Success) {
+                    roleData.setData(_modifierGroupCollapsed);
+                    continue;
+                }
+            }
+            if(item->isObjectItem()) {
+                switch(item->status().type()) {
+                case PipelineStatus::Warning: roleData.setData(_statusWarningIcon); break;
+                case PipelineStatus::Error: roleData.setData(_statusErrorIcon); break;
+                default: roleData.setData(_statusNoneIcon); break;
+                }
+                continue;
             }
         }
-        return QString();
-    }
-    else if(role == Qt::ToolTipRole || role == PipelineListModel::ToolTipRole) {
-        return QVariant::fromValue(item->status().text());
-    }
-    else if(role == Qt::CheckStateRole) {
-        if(ModificationNode* node = dynamic_object_cast<ModificationNode>(item->object()))
-            return (node->modifier() && node->modifier()->isEnabled()) ? Qt::Checked : Qt::Unchecked;
-        else if(ActiveObject* object = dynamic_object_cast<ActiveObject>(item->object())) {
-            if(item->itemType() != PipelineListItem::DataSource)
-                return object->isEnabled() ? Qt::Checked : Qt::Unchecked;
+        else if(role == PipelineListModel::DecorationRole) {
+            // This role is only used by the QML GUI.
+            if(item->itemType() == PipelineListItem::ModifierGroup) {
+                if(!static_object_cast<ModifierGroup>(item->object())->isCollapsed()) {
+                    roleData.setData(QStringLiteral("modify_modifier_group_expanded"));
+                    continue;
+                }
+            }
+            if(item->itemType() == PipelineListItem::ModifierGroup) {
+                if(item->status().type() == PipelineStatus::Success) {
+                    roleData.setData(QStringLiteral("modify_modifier_group_collapsed"));
+                    continue;
+                }
+            }
+            if(item->isObjectItem()) {
+                switch(item->status().type()) {
+                case PipelineStatus::Warning: roleData.setData(QStringLiteral("qrc:/guibase/mainwin/status/status_warning.png")); break;
+                case PipelineStatus::Error: roleData.setData(QStringLiteral("qrc:/guibase/mainwin/status/status_error.png")); break;
+                default: roleData.setData(QStringLiteral("qrc:/guibase/mainwin/status/status_none.png"));
+                }
+                continue;
+            }
         }
-    }
-    else if(role == CheckedRole) {
-        if(ModificationNode* node = dynamic_object_cast<ModificationNode>(item->object()))
-            return node->modifier() && node->modifier()->isEnabled();
-        else if(ActiveObject* object = dynamic_object_cast<ActiveObject>(item->object())) {
-            if(item->itemType() != PipelineListItem::DataSource)
-                return object->isEnabled();
+        else if(role == Qt::ToolTipRole || role == PipelineListModel::ToolTipRole) {
+            roleData.setData(item->status().text());
+            continue;
         }
-        return false;
-    }
-    else if(role == Qt::TextAlignmentRole) {
-        if(!item->isObjectItem()) {
-            return Qt::AlignCenter;
+        else if(role == Qt::CheckStateRole) {
+            if(ModificationNode* node = dynamic_object_cast<ModificationNode>(item->object())) {
+                roleData.setData((node->modifier() && node->modifier()->isEnabled()) ? Qt::Checked : Qt::Unchecked);
+                continue;
+            }
+            else if(ActiveObject* object = dynamic_object_cast<ActiveObject>(item->object())) {
+                if(item->itemType() != PipelineListItem::DataSource) {
+                    roleData.setData(object->isEnabled() ? Qt::Checked : Qt::Unchecked);
+                    continue;
+                }
+            }
+            else if(item->itemType() == PipelineListItem::DeletedVisualElement) {
+                roleData.setData(Qt::Unchecked); // This is to prevent the checkbox element from temporarily disappearing while the list of visual elements is being regenerated.
+                continue;
+            }
         }
-    }
-    else if(role == Qt::BackgroundRole) {
-        if(!item->isObjectItem()) {
-            if(item->itemType() != PipelineListItem::PipelineBranch)
-                return _sectionHeaderBackgroundBrush;
-            else
-                return QBrush(Qt::lightGray, Qt::Dense6Pattern);
+        else if(role == CheckedRole) {
+            if(ModificationNode* node = dynamic_object_cast<ModificationNode>(item->object())) {
+                roleData.setData(node->modifier() && node->modifier()->isEnabled());
+                continue;
+            }
+            else if(ActiveObject* object = dynamic_object_cast<ActiveObject>(item->object())) {
+                if(item->itemType() != PipelineListItem::DataSource) {
+                    roleData.setData(object->isEnabled());
+                    continue;
+                }
+            }
+            roleData.setData(false);
+            continue;
         }
+        else if(role == Qt::TextAlignmentRole) {
+            if(!item->isObjectItem()) {
+                roleData.setData(Qt::AlignCenter);
+                continue;
+            }
+        }
+        else if(role == Qt::BackgroundRole) {
+            if(!item->isObjectItem()) {
+                if(item->itemType() != PipelineListItem::PipelineBranch)
+                    roleData.setData(_sectionHeaderBackgroundBrush);
+                else
+                    roleData.setData(QBrush(Qt::lightGray, Qt::Dense6Pattern));
+                continue;
+            }
+        }
+        else if(role == Qt::ForegroundRole) {
+            if(!item->isObjectItem()) {
+                roleData.setData(_sectionHeaderForegroundBrush);
+                continue;
+            }
+            else if(item->itemType() == PipelineListItem::Modifier && static_object_cast<ModificationNode>(item->object())->modifierAndGroupEnabled() == false) {
+                roleData.setData(_disabledForegroundBrush);
+                continue;
+            }
+            else if(item->itemType() == PipelineListItem::ModifierGroup && static_object_cast<ModifierGroup>(item->object())->isEnabled() == false) {
+                roleData.setData(_disabledForegroundBrush);
+                continue;
+            }
+        }
+        else if(role == Qt::FontRole) {
+            if(!item->isObjectItem()) {
+                roleData.setData(_sectionHeaderFont);
+                continue;
+            }
+            else if(isSharedObject(item->object())) {
+                roleData.setData(_sharedObjectFont);
+                continue;
+            }
+        }
+        roleData.clearData();
     }
-    else if(role == Qt::ForegroundRole) {
-        if(!item->isObjectItem())
-            return _sectionHeaderForegroundBrush;
-        else if(item->itemType() == PipelineListItem::Modifier && static_object_cast<ModificationNode>(item->object())->modifierAndGroupEnabled() == false)
-            return _disabledForegroundBrush;
-        else if(item->itemType() == PipelineListItem::ModifierGroup && static_object_cast<ModifierGroup>(item->object())->isEnabled() == false)
-            return _disabledForegroundBrush;
-    }
-    else if(role == Qt::FontRole) {
-        if(!item->isObjectItem())
-            return _sectionHeaderFont;
-        else if(isSharedObject(item->object()))
-            return _sharedObjectFont;
-    }
-
-    return {};
 }
 
 /******************************************************************************
@@ -721,7 +781,7 @@ bool PipelineListModel::setData(const QModelIndex& index, const QVariant& value,
             return true;
         }
         else if(ModificationNode* node = dynamic_object_cast<ModificationNode>(item->object())) {
-            _userInterface.performTransaction((value.toInt() != Qt::Unchecked) ? tr("Enable modifier") : tr("Disable modifier"), [node, &value, index, role, this]() {
+            _userInterface.performTransaction((value.toInt() != Qt::Unchecked) ? tr("Enable modifier") : tr("Disable modifier"), [node, &value]() {
                 if(node->modifier())
                     node->modifier()->setEnabled(value.toInt() != Qt::Unchecked);
             });
@@ -792,6 +852,7 @@ Qt::ItemFlags PipelineListModel::flags(const QModelIndex& index) const
                 return Qt::ItemIsDropEnabled;
             case PipelineListItem::DeletedObject:
             case PipelineListItem::DeletedDataObject:
+            case PipelineListItem::DeletedVisualElement:
                 return QAbstractListModel::flags(index); // Keep entries with deleted objects selectable to not loose the current selection before the model is updated.
             default:
                 return Qt::NoItemFlags;
@@ -1449,7 +1510,6 @@ PipelineNode* PipelineListModel::makeElementIndependentImpl(PipelineNode* pipeli
     // When arriving at the selected ModificationNode, duplicate the modifier too
     // in case it is being shared by multiple pipelines.
     while(currentNode) {
-        PipelineNode* nextNode = nullptr;
         if(ModificationNode* modNode = dynamic_object_cast<ModificationNode>(currentNode)) {
 
             // Clone all modification nodes along the way if they are shared by multiple pipeline branches.
@@ -1511,17 +1571,17 @@ void PipelineListModel::toggleModifierGroup()
         // But first make sure the modifier nodes aren't already part of an existing group.
         existingGroup = modNode->modifierGroup();
         if(!existingGroup) {
-            // Create a new group.
-            OORef<ModifierGroup> group = OORef<ModifierGroup>::create();
             _userInterface.performTransaction(tr("Create modifier group"), [&]() {
+                // Create a new group.
+                OORef<ModifierGroup> group = OORef<ModifierGroup>::create();
                 for(RefTarget* obj : objects) {
                     if(ModificationNode* modNode = dynamic_object_cast<ModificationNode>(obj)) {
                         modNode->setModifierGroup(group);
                     }
                 }
+                setNextObjectToSelect(group);
+                refreshList();
             });
-            setNextObjectToSelect(group);
-            refreshList();
             return;
         }
     }
@@ -1537,7 +1597,7 @@ void PipelineListModel::toggleModifierGroup()
                 if(modNode->modifierGroup() == existingGroup)
                     modNode->setModifierGroup(nullptr);
             }
-            existingGroup->deleteReferenceObject();
+            existingGroup->requestObjectDeletion();
         });
     }
     refreshList();
