@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -23,11 +23,14 @@
 #include <ovito/gui/base/GUIBase.h>
 #include <ovito/core/app/UserInterface.h>
 #include <ovito/core/viewport/Viewport.h>
+#include <ovito/core/viewport/ViewportWindow.h>
 #include <ovito/core/viewport/ViewportSettings.h>
 #include <ovito/core/viewport/ViewportConfiguration.h>
 #include <ovito/core/dataset/animation/AnimationSettings.h>
 #include <ovito/core/dataset/data/camera/AbstractCameraObject.h>
+#include <ovito/core/dataset/data/camera/AbstractCameraSource.h>
 #include <ovito/core/dataset/data/BufferAccess.h>
+#include <ovito/core/dataset/DataSetContainer.h>
 #include <ovito/core/dataset/scene/Scene.h>
 #include <ovito/core/app/undo/UndoableOperation.h>
 #include <ovito/core/dataset/DataSet.h>
@@ -36,6 +39,13 @@
 
 namespace Ovito {
 
+IMPLEMENT_ABSTRACT_OVITO_CLASS(NavigationMode);
+IMPLEMENT_ABSTRACT_OVITO_CLASS(OrbitMode);
+IMPLEMENT_ABSTRACT_OVITO_CLASS(PanMode);
+IMPLEMENT_ABSTRACT_OVITO_CLASS(ZoomMode);
+IMPLEMENT_ABSTRACT_OVITO_CLASS(FOVMode);
+IMPLEMENT_ABSTRACT_OVITO_CLASS(PickOrbitCenterMode);
+
 /******************************************************************************
 * This is called by the system after the input handler has
 * become the active handler.
@@ -43,7 +53,8 @@ namespace Ovito {
 void NavigationMode::activated(bool temporaryActivation)
 {
     _temporaryActivation = temporaryActivation;
-    inputManager()->addViewportGizmo(inputManager()->pickOrbitCenterMode());
+    if(!temporaryActivation)
+        inputManager()->addViewportGizmo(inputManager()->pickOrbitCenterMode());
     ViewportInputMode::activated(temporaryActivation);
 }
 
@@ -58,6 +69,7 @@ void NavigationMode::deactivated(bool temporary)
         _viewport->setCameraTransformation(_oldCameraTM);
         _viewport->setFieldOfView(_oldFieldOfView);
         _undoTransaction.cancel();
+        _viewport->removeViewportGizmo(inputManager()->pickOrbitCenterMode());
         _viewport = nullptr;
     }
     inputManager()->removeViewportGizmo(inputManager()->pickOrbitCenterMode());
@@ -67,28 +79,28 @@ void NavigationMode::deactivated(bool temporary)
 /******************************************************************************
 * Applies a step-wise change of the view orientation.
 ******************************************************************************/
-void NavigationMode::discreteStep(ViewportWindowInterface* vpwin, QPointF delta)
+void NavigationMode::discreteStep(ViewportWindow* vpwin, QPointF delta)
 {
     Viewport* vp = vpwin->viewport();
     if(_viewport == nullptr) {
-        std::swap(_viewport, vp);
+        _viewport = vp;
         _startPoint = QPointF(0,0);
         _oldCameraTM = _viewport->cameraTransformation();
         _oldCameraPosition = _viewport->cameraPosition();
         _oldCameraDirection = _viewport->cameraDirection();
         _oldFieldOfView = _viewport->fieldOfView();
-        _oldViewMatrix = _viewport->projectionParams().viewMatrix;
-        _oldInverseViewMatrix = _viewport->projectionParams().inverseViewMatrix;
+        _oldViewMatrix = vpwin->projectionParams().viewMatrix;
+        _oldInverseViewMatrix = vpwin->projectionParams().inverseViewMatrix;
         _currentOrbitCenter = _viewport->orbitCenter();
     }
     modifyView(vpwin, vpwin->viewport(), delta, true);
-    std::swap(_viewport, vp);
+    _viewport = {};
 }
 
 /******************************************************************************
 * Handles the mouse down event for the given viewport.
 ******************************************************************************/
-void NavigationMode::mousePressEvent(ViewportWindowInterface* vpwin, QMouseEvent* event)
+void NavigationMode::mousePressEvent(ViewportWindow* vpwin, QMouseEvent* event)
 {
     if(event->button() == Qt::RightButton) {
         ViewportInputMode::mousePressEvent(vpwin, event);
@@ -102,21 +114,25 @@ void NavigationMode::mousePressEvent(ViewportWindowInterface* vpwin, QMouseEvent
         _oldCameraPosition = _viewport->cameraPosition();
         _oldCameraDirection = _viewport->cameraDirection();
         _oldFieldOfView = _viewport->fieldOfView();
-        _oldViewMatrix = _viewport->projectionParams().viewMatrix;
-        _oldInverseViewMatrix = _viewport->projectionParams().inverseViewMatrix;
+        _oldViewMatrix = vpwin->projectionParams().viewMatrix;
+        _oldInverseViewMatrix = vpwin->projectionParams().inverseViewMatrix;
         _currentOrbitCenter = _viewport->orbitCenter();
         _undoTransaction.begin(inputManager()->userInterface(), tr("Modify camera"));
+        if(_temporaryActivation)
+            _viewport->addViewportGizmo(inputManager()->pickOrbitCenterMode());
     }
 }
 
 /******************************************************************************
 * Handles the mouse up event for the given viewport.
 ******************************************************************************/
-void NavigationMode::mouseReleaseEvent(ViewportWindowInterface* vpwin, QMouseEvent* event)
+void NavigationMode::mouseReleaseEvent(ViewportWindow* vpwin, QMouseEvent* event)
 {
     if(_viewport) {
         // Commit view change.
         _undoTransaction.commit();
+        if(_temporaryActivation)
+            _viewport->removeViewportGizmo(inputManager()->pickOrbitCenterMode());
         _viewport = nullptr;
 
         if(_temporaryActivation)
@@ -127,7 +143,7 @@ void NavigationMode::mouseReleaseEvent(ViewportWindowInterface* vpwin, QMouseEve
 /******************************************************************************
 * Is called when a viewport looses the input focus.
 ******************************************************************************/
-void NavigationMode::focusOutEvent(ViewportWindowInterface* vpwin, QFocusEvent* event)
+void NavigationMode::focusOutEvent(ViewportWindow* vpwin, QFocusEvent* event)
 {
     if(_viewport) {
         if(_temporaryActivation)
@@ -138,7 +154,7 @@ void NavigationMode::focusOutEvent(ViewportWindowInterface* vpwin, QFocusEvent*
 /******************************************************************************
 * Handles the mouse move event for the given viewport.
 ******************************************************************************/
-void NavigationMode::mouseMoveEvent(ViewportWindowInterface* vpwin, QMouseEvent* event)
+void NavigationMode::mouseMoveEvent(ViewportWindow* vpwin, QMouseEvent* event)
 {
     if(_viewport == vpwin->viewport()) {
         QPointF pos = getMousePosition(event);
@@ -147,9 +163,6 @@ void NavigationMode::mouseMoveEvent(ViewportWindowInterface* vpwin, QMouseEvent*
         inputManager()->userInterface().performActions(_undoTransaction, [&] {
             modifyView(vpwin, _viewport, pos - _startPoint, false);
         });
-
-        // Force immediate viewport repaint.
-        inputManager()->userInterface().processViewportUpdateRequests();
     }
 }
 
@@ -158,8 +171,8 @@ void NavigationMode::mouseMoveEvent(ViewportWindowInterface* vpwin, QMouseEvent*
 ******************************************************************************/
 PipelineNode* NavigationMode::getViewportCamera(Viewport* vp)
 {
-    if(vp->viewNode() && vp->viewType() == Viewport::VIEW_SCENENODE) {
-        return vp->viewNode()->source();
+    if(vp->viewNode() && vp->viewNode()->pipeline() && vp->viewType() == Viewport::VIEW_SCENENODE) {
+        return vp->viewNode()->pipeline()->source();
     }
     return nullptr;
 }
@@ -170,12 +183,12 @@ PipelineNode* NavigationMode::getViewportCamera(Viewport* vp)
 /******************************************************************************
 * Computes the new view matrix based on the new mouse position.
 ******************************************************************************/
-void PanMode::modifyView(ViewportWindowInterface* vpwin, Viewport* vp, QPointF delta, bool discreteStep)
+void PanMode::modifyView(ViewportWindow* vpwin, Viewport* vp, QPointF delta, bool discreteStep)
 {
     FloatType scaling;
     FloatType normalization = discreteStep ? 20.0 : vpwin->viewportWindowDeviceIndependentSize().height();
     if(vp->isPerspectiveProjection())
-        scaling = FloatType(10) * vp->nonScalingSize(_currentOrbitCenter) / normalization;
+        scaling = FloatType(10) * vpwin->projectionParams().nonScalingSize(_currentOrbitCenter, vpwin->viewportWindowDeviceIndependentSize()) / normalization;
     else
         scaling = FloatType(2) * _oldFieldOfView / normalization;
     FloatType deltaX = -scaling * delta.x();
@@ -186,18 +199,16 @@ void PanMode::modifyView(ViewportWindowInterface* vpwin, Viewport* vp, QPointF d
     }
     else {
         // Get parent's system.
-        TimeInterval iv;
         const AffineTransformation& parentSys =
-                vp->viewNode()->parentNode()->getWorldTransform(vp->scene()->animationSettings()->currentTime(), iv);
+                vp->viewNode()->parentNode()->getWorldTransform(vp->currentTime());
 
         // Move node in parent's system.
         vp->viewNode()->transformationController()->translate(
-                vp->scene()->animationSettings()->currentTime(), displacement, parentSys.inverse());
+                vp->currentTime(), displacement, parentSys.inverse());
 
         // If it's a target camera, move target as well.
         if(vp->viewNode()->lookatTargetNode()) {
-            vp->viewNode()->lookatTargetNode()->transformationController()->translate(
-                    vp->scene()->animationSettings()->currentTime(), displacement, parentSys.inverse());
+            vp->viewNode()->lookatTargetNode()->transformationController()->translate(vp->currentTime(), displacement, parentSys.inverse());
         }
     }
 }
@@ -208,7 +219,7 @@ void PanMode::modifyView(ViewportWindowInterface* vpwin, Viewport* vp, QPointF d
 /******************************************************************************
 * Computes the new view matrix based on the new mouse position.
 ******************************************************************************/
-void ZoomMode::modifyView(ViewportWindowInterface* vpwin, Viewport* vp, QPointF delta, bool discreteStep)
+void ZoomMode::modifyView(ViewportWindow* vpwin, Viewport* vp, QPointF delta, bool discreteStep)
 {
     if(vp->isPerspectiveProjection()) {
         FloatType amount =  FloatType(-5) * sceneSizeFactor(vp) * delta.y();
@@ -216,20 +227,18 @@ void ZoomMode::modifyView(ViewportWindowInterface* vpwin, Viewport* vp, QPointF
             vp->setCameraPosition(_oldCameraPosition + _oldCameraDirection.resized(amount));
         }
         else {
-            TimeInterval iv;
-            const AffineTransformation& sys = vp->viewNode()->getWorldTransform(vp->scene()->animationSettings()->currentTime(), iv);
+            const AffineTransformation& sys = vp->viewNode()->getWorldTransform(vp->currentTime());
             vp->viewNode()->transformationController()->translate(
-                    vp->scene()->animationSettings()->currentTime(), Vector3(0,0,-amount), sys);
+                    vp->currentTime(), Vector3(0,0,-amount), sys);
         }
     }
     else {
-        if(PipelineNode* cameraSource = getViewportCamera(vp)) {
-            FloatType oldFOV = cameraSource->property("zoom").value<FloatType>();
-            FloatType newFOV = oldFOV * (FloatType)exp(0.003 * delta.y());
-            cameraSource->setProperty("zoom", QVariant::fromValue(newFOV));
+        if(AbstractCameraSource* cameraSource = dynamic_object_cast<AbstractCameraSource>(getViewportCamera(vp))) {
+            FloatType newFOV = cameraSource->zoom() * (FloatType)std::exp(0.003 * delta.y());
+            cameraSource->setZoom(newFOV);
         }
         else {
-            FloatType newFOV = _oldFieldOfView * (FloatType)exp(0.003 * delta.y());
+            FloatType newFOV = _oldFieldOfView * (FloatType)std::exp(0.003 * delta.y());
             vp->setFieldOfView(newFOV);
         }
     }
@@ -243,7 +252,7 @@ FloatType ZoomMode::sceneSizeFactor(Viewport* vp)
 {
     OVITO_CHECK_OBJECT_POINTER(vp);
     if(vp->scene()) {
-        Box3 sceneBoundingBox = vp->scene()->worldBoundingBox(vp->scene()->animationSettings()->currentTime(), vp);
+        Box3 sceneBoundingBox = vp->scene()->worldBoundingBox(vp->currentTime(), vp);
         if(!sceneBoundingBox.isEmpty())
             return sceneBoundingBox.size().length() * FloatType(5e-4);
     }
@@ -260,21 +269,20 @@ void ZoomMode::zoom(Viewport* vp, FloatType steps, UserInterface& ui)
             vp->setCameraPosition(vp->cameraPosition() + vp->cameraDirection().resized(sceneSizeFactor(vp) * steps));
         }
         else {
-            vp->setFieldOfView(vp->fieldOfView() * exp(-steps * FloatType(1e-3)));
+            vp->setFieldOfView(vp->fieldOfView() * std::exp(-steps * FloatType(1e-3)));
         }
     }
     else {
         ui.performTransaction(tr("Zoom viewport"), [this, steps, vp]() {
             if(vp->isPerspectiveProjection()) {
                 FloatType amount = sceneSizeFactor(vp) * steps;
-                TimeInterval iv;
-                const AffineTransformation& sys = vp->viewNode()->getWorldTransform(vp->scene()->animationSettings()->currentTime(), iv);
-                vp->viewNode()->transformationController()->translate(vp->scene()->animationSettings()->currentTime(), Vector3(0,0,-amount), sys);
+                const AffineTransformation& sys = vp->viewNode()->getWorldTransform(vp->currentTime());
+                vp->viewNode()->transformationController()->translate(vp->currentTime(), Vector3(0,0,-amount), sys);
             }
             else {
-                if(PipelineNode* cameraSource = getViewportCamera(vp)) {
-                    FloatType oldFOV = cameraSource->property("zoom").value<FloatType>();
-                    cameraSource->setProperty("zoom", QVariant::fromValue(oldFOV * exp(-steps * FloatType(1e-3))));
+                if(AbstractCameraSource* cameraSource = dynamic_object_cast<AbstractCameraSource>(getViewportCamera(vp))) {
+                    FloatType oldFOV = cameraSource->zoom();
+                    cameraSource->setZoom(oldFOV * std::exp(-steps * FloatType(1e-3)));
                 }
             }
         });
@@ -287,11 +295,11 @@ void ZoomMode::zoom(Viewport* vp, FloatType steps, UserInterface& ui)
 /******************************************************************************
 * Computes the new field of view based on the new mouse position.
 ******************************************************************************/
-void FOVMode::modifyView(ViewportWindowInterface* vpwin, Viewport* vp, QPointF delta, bool discreteStep)
+void FOVMode::modifyView(ViewportWindow* vpwin, Viewport* vp, QPointF delta, bool discreteStep)
 {
     FloatType oldFOV = _oldFieldOfView;
-    if(PipelineNode* cameraSource = getViewportCamera(vp)) {
-        oldFOV = cameraSource->property(vp->isPerspectiveProjection() ? "fov" : "zoom").value<FloatType>();
+    if(AbstractCameraSource* cameraSource = dynamic_object_cast<AbstractCameraSource>(getViewportCamera(vp))) {
+        oldFOV = vp->isPerspectiveProjection() ? cameraSource->fov() : cameraSource->zoom();
     }
 
     FloatType newFOV;
@@ -304,8 +312,11 @@ void FOVMode::modifyView(ViewportWindowInterface* vpwin, Viewport* vp, QPointF d
         newFOV = oldFOV * (FloatType)exp(FloatType(6e-3) * delta.y());
     }
 
-    if(PipelineNode* cameraSource = getViewportCamera(vp)) {
-        cameraSource->setProperty(vp->isPerspectiveProjection() ? "fov" : "zoom", QVariant::fromValue(newFOV));
+    if(AbstractCameraSource* cameraSource = dynamic_object_cast<AbstractCameraSource>(getViewportCamera(vp))) {
+        if(vp->isPerspectiveProjection())
+            cameraSource->setFov(newFOV);
+        else
+            cameraSource->setZoom(newFOV);
     }
     else {
         vp->setFieldOfView(newFOV);
@@ -318,12 +329,12 @@ void FOVMode::modifyView(ViewportWindowInterface* vpwin, Viewport* vp, QPointF d
 /******************************************************************************
 * Computes the new view matrix based on the new mouse position.
 ******************************************************************************/
-void OrbitMode::modifyView(ViewportWindowInterface* vpwin, Viewport* vp, QPointF delta, bool discreteStep)
+void OrbitMode::modifyView(ViewportWindow* vpwin, Viewport* vp, QPointF delta, bool discreteStep)
 {
     if(vp->viewType() < Viewport::VIEW_ORTHO)
         vp->setViewType(Viewport::VIEW_ORTHO, true);
 
-    FloatType speed = discreteStep ? 0.05 : (5.0 / vp->windowSize().height());
+    FloatType speed = discreteStep ? 0.05 : (5.0 / vpwin->viewportWindowDeviceIndependentSize().height());
     FloatType deltaTheta = speed * delta.x();
     FloatType deltaPhi = -speed * delta.y();
 
@@ -334,12 +345,7 @@ void OrbitMode::modifyView(ViewportWindowInterface* vpwin, Viewport* vp, QPointF
         const Matrix3& coordSys = ViewportSettings::getSettings().coordinateSystemOrientation();
         Vector3 v = _oldViewMatrix * coordSys.column(2);
 
-        FloatType theta, phi;
-        if(v.x() == 0 && v.y() == 0)
-            theta = FLOATTYPE_PI;
-        else
-            theta = atan2(v.x(), v.y());
-        phi = atan2(sqrt(v.x() * v.x() + v.y() * v.y()), v.z());
+        FloatType phi = std::atan2(sqrt(v.x() * v.x() + v.y() * v.y()), v.z());
 
         // Restrict rotation to keep the major axis pointing upward (prevent camera from becoming upside down).
         if(phi + deltaPhi < FLOATTYPE_EPSILON)
@@ -360,7 +366,7 @@ void OrbitMode::modifyView(ViewportWindowInterface* vpwin, Viewport* vp, QPointF
         }
         else {
             Controller* ctrl = vp->viewNode()->transformationController();
-            AnimationTime time = vp->scene()->animationSettings()->currentTime();
+            AnimationTime time = vp->currentTime();
             Rotation rotX(Vector3(1,0,0), deltaPhi, false);
             ctrl->rotate(time, rotX, _oldInverseViewMatrix);
             Rotation rotZ(ViewportSettings::getSettings().upVector(), -deltaTheta);
@@ -383,7 +389,7 @@ void OrbitMode::modifyView(ViewportWindowInterface* vpwin, Viewport* vp, QPointF
         }
         else {
             Controller* ctrl = vp->viewNode()->transformationController();
-            AnimationTime time = vp->scene()->animationSettings()->currentTime();
+            AnimationTime time = vp->currentTime();
             Rotation rot = Rotation(Vector3(0,1,0), -deltaTheta, false) * Rotation(Vector3(1,0,0), deltaPhi, false);
             ctrl->rotate(time, rot, _oldInverseViewMatrix);
             Vector3 translation = t2 - (Matrix3::rotation(rot) * t2);
@@ -398,7 +404,7 @@ void OrbitMode::modifyView(ViewportWindowInterface* vpwin, Viewport* vp, QPointF
 /******************************************************************************
 * Sets the orbit rotation center to the space location under given mouse coordinates.
 ******************************************************************************/
-bool PickOrbitCenterMode::pickOrbitCenter(ViewportWindowInterface* vpwin, const QPointF& pos)
+bool PickOrbitCenterMode::pickOrbitCenter(ViewportWindow* vpwin, const QPointF& pos)
 {
     Point3 p;
     Viewport* vp = vpwin->viewport();
@@ -420,7 +426,7 @@ bool PickOrbitCenterMode::pickOrbitCenter(ViewportWindowInterface* vpwin, const
 /******************************************************************************
 * Handles the mouse down events for a Viewport.
 ******************************************************************************/
-void PickOrbitCenterMode::mousePressEvent(ViewportWindowInterface* vpwin, QMouseEvent* event)
+void PickOrbitCenterMode::mousePressEvent(ViewportWindow* vpwin, QMouseEvent* event)
 {
     if(event->button() == Qt::LeftButton) {
         if(pickOrbitCenter(vpwin, getMousePosition(event)))
@@ -432,7 +438,7 @@ void PickOrbitCenterMode::mousePressEvent(ViewportWindowInterface* vpwin, QMouse
 /******************************************************************************
 * Is called when the user moves the mouse while the operation is not active.
 ******************************************************************************/
-void PickOrbitCenterMode::mouseMoveEvent(ViewportWindowInterface* vpwin, QMouseEvent* event)
+void PickOrbitCenterMode::mouseMoveEvent(ViewportWindow* vpwin, QMouseEvent* event)
 {
     ViewportInputMode::mouseMoveEvent(vpwin, event);
 
@@ -453,11 +459,10 @@ void PickOrbitCenterMode::mouseMoveEvent(ViewportWindowInterface* vpwin, QMouseE
 * Finds the closest intersection point between a ray originating from the
 * current mouse cursor position and the whole scene.
 ******************************************************************************/
-bool PickOrbitCenterMode::findIntersection(ViewportWindowInterface* vpwin, const QPointF& mousePos, Point3& intersectionPoint)
+bool PickOrbitCenterMode::findIntersection(ViewportWindow* vpwin, const QPointF& mousePos, Point3& intersectionPoint)
 {
-    ViewportPickResult pickResult = vpwin->pick(mousePos);
-    if(pickResult.isValid()) {
-        intersectionPoint = pickResult.hitLocation();
+    if(std::optional<ViewportWindow::PickResult> pickResult = vpwin->pick(mousePos)) {
+        intersectionPoint = pickResult->hitLocation();
         return true;
     }
     return false;
@@ -466,19 +471,13 @@ bool PickOrbitCenterMode::findIntersection(ViewportWindowInterface* vpwin, const
 /******************************************************************************
 * Lets the input mode render its overlay content in a viewport.
 ******************************************************************************/
-void PickOrbitCenterMode::renderOverlay3D(Viewport* vp, SceneRenderer* renderer)
+void PickOrbitCenterMode::renderOverlay(Viewport* vp, ViewportWindow* vpWin, FrameGraph& frameGraph, DataSet* dataset)
 {
-    if(!renderer->isImagePass() || !vp->scene())
-        return;
-
-    // Render center of rotation.
-    Point3 center = vp->orbitCenter();
-    FloatType symbolSize = vp->nonScalingSize(center);
-    renderer->setWorldTransform(AffineTransformation::translation(center - Point3::Origin()) * AffineTransformation::scaling(symbolSize));
-
-    if(!renderer->isBoundingBoxPass()) {
-        auto& orbitCenterMarker = renderer->visCache().get<CylinderPrimitive>(RendererResourceKey<struct OrbitGlyphCache>{});
-        if(!orbitCenterMarker.basePositions()) {
+    // Prepare the cylinder graphics primitive for rendering the orbit center marker as a crosshair symbol.
+    // The graphics primitive is immutable and can be reused across multiple frames.
+    const auto& orbitCenterMarker = frameGraph.visCache().lookup<CylinderPrimitive>(
+        RendererResourceKey<struct OrbitGlyphCache>{},
+        [&](CylinderPrimitive& orbitCenterMarker) {
             BufferFactory<Point3G> basePositions(3);
             BufferFactory<Point3G> headPositions(3);
             BufferFactory<ColorG> colors(3);
@@ -490,13 +489,26 @@ void PickOrbitCenterMode::renderOverlay3D(Viewport* vp, SceneRenderer* renderer)
             orbitCenterMarker.setUniformWidth(0.1);
             orbitCenterMarker.setPositions(basePositions.take(), headPositions.take());
             orbitCenterMarker.setColors(colors.take());
-        }
-        renderer->renderCylinders(orbitCenterMarker);
-    }
-    else {
-        // Add marker to bounding box.
-        renderer->addToLocalBoundingBox(Box3(Point3::Origin(), symbolSize));
+        });
+
+    // Determine current center of rotation.
+    // Note: Always using the active viewport's orbit center here to show a consistent picture across all viewports.
+    Viewport* activeViewport = vp;
+    if(dataset) {
+        if(dataset->viewportConfig() && dataset->viewportConfig()->activeViewport())
+            activeViewport = dataset->viewportConfig()->activeViewport();
     }
+    const Point3 center = activeViewport->orbitCenter();
+
+    // The cross-hair should appear always the same size on the screen.
+    const FloatType symbolSize = frameGraph.projectionParams().nonScalingSize(center, vpWin->viewportWindowDeviceIndependentSize());
+
+    // Emit the graphics primitive.
+    frameGraph.addCommandGroup(FrameGraph::SceneLayer).addPrimitiveNonpickable(
+        std::make_unique<CylinderPrimitive>(orbitCenterMarker),
+        AffineTransformation::translation(center - Point3::Origin()) * AffineTransformation::scaling(symbolSize),
+        Box3(Point3(-1), Point3(1)))
+            .setExcludeFromLighting(true);
 }
 
 }   // End of namespace
