@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -23,17 +23,20 @@
 #include <ovito/gui/desktop/GUI.h>
 #include <ovito/gui/desktop/mainwin/MainWindow.h>
 #include <ovito/gui/desktop/mainwin/OvitoStyle.h>
-#include <ovito/gui/desktop/dataset/GuiDataSetContainer.h>
 #include <ovito/gui/desktop/dialogs/MessageDialog.h>
 #include <ovito/gui/base/actions/ActionManager.h>
 #include <ovito/core/app/undo/UndoStack.h>
+#include <ovito/core/dataset/DataSet.h>
+#include <ovito/core/dataset/DataSetContainer.h>
 #include "GuiApplication.h"
 
-#ifdef Q_OS_LINUX
-    #include <QDBusConnection>
-    #include <QDBusMessage>
-    #include <QDBusVariant>
-    #include <QDBusReply>
+#if QT_VERSION < QT_VERSION_CHECK(6, 5, 0)
+    #ifdef Q_OS_LINUX
+        #include <QDBusConnection>
+        #include <QDBusMessage>
+        #include <QDBusVariant>
+        #include <QDBusReply>
+    #endif
 #endif
 
 // Registers the embedded Qt resource files embedded in a statically linked executable at application startup.
@@ -51,7 +54,7 @@ namespace Ovito {
 /******************************************************************************
 * Constructor.
 ******************************************************************************/
-GuiApplication::GuiApplication() : StandaloneApplication(_fileManager), _fileManager(StandaloneApplication::taskManager())
+GuiApplication::GuiApplication()
 {
     // Register Qt resources.
     ::registerQtResources();
@@ -80,26 +83,12 @@ bool GuiApplication::processCommandLineParameters()
         return false;
 
     // Check if program was started in console mode.
-    if(!_cmdLineParser.isSet("nogui")) {
-        // Enable GUI mode by default.
-        setGuiMode(true);
-        _headlessMode = false;
-    }
-    else {
+    if(_cmdLineParser.isSet("nogui")) {
         // Activate console mode.
-        setGuiMode(false);
-#if defined(Q_OS_LINUX)
-        // On Linux, run in headless mode by default - unless explicitly requested otherwise (in which case an X server is required).
-        if(qEnvironmentVariableIsSet("OVITO_GUI_MODE") && qgetenv("OVITO_GUI_MODE") != "0") {
-            _headlessMode = false;
-        }
-#elif defined(Q_OS_MACOS)
+        setRunMode(Application::TerminalMode);
+#if defined(Q_OS_MACOS)
         // Don't let Qt move the app to the foreground when running in console mode.
         ::setenv("QT_MAC_DISABLE_FOREGROUND_APPLICATION_TRANSFORM", "1", 1);
-        _headlessMode = false;
-#elif defined(Q_OS_WIN)
-        // On Windows, there is always an OpenGL implementation available for offscreen rendering.
-        _headlessMode = false;
 #endif
     }
 
@@ -109,37 +98,43 @@ bool GuiApplication::processCommandLineParameters()
 /******************************************************************************
 * Create the global instance of the right QCoreApplication derived class.
 ******************************************************************************/
-void GuiApplication::createQtApplication(int& argc, char** argv)
+QCoreApplication* GuiApplication::createQtApplicationImpl(bool supportGui, int& argc, char** argv)
 {
-    // Verify that the OpenGLSceneRenderer class has registered the right default surface format.
+    // Verify that the OpenGLRenderer class has registered the right default surface format.
     OVITO_ASSERT(QSurfaceFormat::defaultFormat().depthBufferSize() == 24 && QSurfaceFormat::defaultFormat().stencilBufferSize() == 1);
 
-    if(headlessMode()) {
-        StandaloneApplication::createQtApplication(argc, argv);
+    QCoreApplication* qtApp = nullptr;
+    if(!supportGui) {
+        qtApp = StandaloneApplication::createQtApplicationImpl(supportGui, argc, argv);
     }
     else {
-        // OVITO prefers the "C" locale over the system's default locale.
-        QLocale::setDefault(QLocale::c());
-
 #ifndef Q_OS_MACOS
         QGuiApplication::setHighDpiScaleFactorRoundingPolicy(Qt::HighDpiScaleFactorRoundingPolicy::RoundPreferFloor);
 #endif
 
 #if defined(Q_OS_WIN) && QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
-        // Use Fusion UI style on Windows (Qt 6.5+) to enable dark mode support (can be changed by the user in the application settings).
-        QSettings settings;
-        if(settings.value("ui/automatic_dark_mode", false).toBool()) {
+        // Use Fusion UI style on Windows to enable dark mode support (can be changed by the user in the application settings).
+        // If dark mode is not enabled in the application settings (default), use the classic Windows Vista UI style.
+        if(automaticallyEnableDarkMode()) {
             qputenv("QT_QPA_PLATFORM", "windows:darkmode=2");
             QApplication::setStyle("Fusion");
         }
+        else {
+            qputenv("QT_QPA_PLATFORM", "windows:darkmode=0");
+            QApplication::setStyle("windowsvista");
+        }
 #elif defined(Q_OS_LINUX)
+        // Prefer XCB platform plugin over Wayland.
+        if(!qEnvironmentVariableIsSet("QT_QPA_PLATFORM"))
+            qputenv("QT_QPA_PLATFORM", "xcb");
+
         // Always enforce Fusion UI style on Linux.
         qunsetenv("QT_STYLE_OVERRIDE");
         QApplication::setStyle("Fusion");
 #endif
 
         // Create the global application object.
-        new QApplication(argc, argv);
+        qtApp = new QApplication(argc, argv);
 
 #if !defined(Q_OS_WIN) || QT_VERSION < QT_VERSION_CHECK(6, 5, 0)
         // Install our customized UI style.
@@ -148,10 +143,21 @@ void GuiApplication::createQtApplication(int& argc, char** argv)
 
         // Verify that a global sharing OpenGL context has been created by the Qt application as requested.
         OVITO_ASSERT(QOpenGLContext::globalShareContext() != nullptr);
+
+#ifdef OVITO_SSH_CLIENT
+        if(Application::guiEnabled()) {
+            fileManager().registerAskUserForPasswordImpl(&GuiApplication::askUserForPassword);
+            fileManager().registerAskUserForKbiResponseImpl(&GuiApplication::askUserForKbiResponse);
+            fileManager().registerAskUserForKeyPassphraseImpl(&GuiApplication::askUserForKeyPassphrase);
+            fileManager().registerDetectedUnknownSshServerImpl(&GuiApplication::detectedUnknownSshServer);
+        }
+#endif
     }
 
     // Process events sent to the Qt application by the OS.
-    QCoreApplication::instance()->installEventFilter(this);
+    qtApp->installEventFilter(this);
+
+    return qtApp;
 }
 
 #if defined(Q_OS_LINUX) && QT_VERSION < QT_VERSION_CHECK(6, 5, 0)
@@ -194,8 +200,11 @@ static void activateThemeColors(bool dark)
 ******************************************************************************/
 MainThreadOperation GuiApplication::startupApplication()
 {
-    if(guiMode()) {
-        // Set up graphical user interface.
+    OVITO_ASSERT(this_task::get());
+
+    if(Application::guiEnabled()) {
+        // Set up Qt event loop.
+        createQtApplication(true);
 
         // Activate icon theme that matches the current UI theme.
         bool darkTheme = usingDarkTheme();
@@ -257,25 +266,23 @@ MainThreadOperation GuiApplication::startupApplication()
         mainWindowIcon.addFile(":/guibase/mainwin/window_icon_16.png");
         QGuiApplication::setWindowIcon(mainWindowIcon);
 
-        // Create the main window.
-        std::shared_ptr<MainWindow> mainWin = std::make_shared<MainWindow>();
-        mainWin->keepAliveUntilShutdown();
+        if(Application::runMode() == Application::AppMode) {
+            // Create the main window.
+            OORef<MainWindow> mainWin = OORef<MainWindow>::create();
+            mainWin->keepAliveUntilShutdown();
 
-        // Make the application shutdown as soon as the last main window has been closed.
-        QGuiApplication::setQuitOnLastWindowClosed(true);
+            // Show the main window.
+            mainWin->setUpdatesEnabled(false);
+            mainWin->restoreMainWindowGeometry();
+            mainWin->restoreLayout();
+            mainWin->setUpdatesEnabled(true);
 
-        // Show the main window.
-        mainWin->setUpdatesEnabled(false);
-        mainWin->restoreMainWindowGeometry();
-        mainWin->restoreLayout();
-        mainWin->setUpdatesEnabled(true);
-
-        return MainThreadOperation(ExecutionContext::Type::Interactive, *mainWin, false);
-    }
-    else {
-        // Use this application's command line user interface.
-        return MainThreadOperation(ExecutionContext::Type::Interactive, *this, false);
+            return MainThreadOperation(*mainWin, MainThreadOperation::Kind::Isolated);
+        }
     }
+
+    // Run application in non-gui mode.
+    return MainThreadOperation(*this, MainThreadOperation::Kind::Isolated);
 }
 
 /******************************************************************************
@@ -283,12 +290,7 @@ MainThreadOperation GuiApplication::startupApplication()
 ******************************************************************************/
 void GuiApplication::postStartupInitialization()
 {
-    // This is to quit the application's event loop right after we are done executing the startup actions
-    // (only when running in console mode). In GUI mode, the main window will keep the event loop going.
-    QEventLoopLocker eventLoopLocker;
-
-    GuiApplication::initializeUserInterface(ExecutionContext::current().ui(), cmdLineParser().positionalArguments());
-
+    GuiApplication::initializeUserInterface(*this_task::ui(), cmdLineParser().positionalArguments());
     StandaloneApplication::postStartupInitialization();
 }
 
@@ -297,6 +299,8 @@ void GuiApplication::postStartupInitialization()
 ******************************************************************************/
 void GuiApplication::initializeUserInterface(UserInterface& userInterface, const QStringList& arguments)
 {
+    OVITO_ASSERT(this_task::isInteractive());
+
     DataSetContainer& datasetContainer = userInterface.datasetContainer();
 
     // Load session state file specified on the command line.
@@ -304,10 +308,9 @@ void GuiApplication::initializeUserInterface(UserInterface& userInterface, const
         QString startupFilename = arguments.front();
         if(startupFilename.endsWith(".ovito", Qt::CaseInsensitive)) {
             try {
-                MainThreadOperation loadOperation(true);
-                if(OORef<DataSet> dataset = datasetContainer.loadDataset(startupFilename)) {
+                OORef<DataSet> dataset = DataSet::createFromFile(startupFilename);
+                if(userInterface.checkLoadedDataset(dataset))
                     datasetContainer.setCurrentSet(std::move(dataset));
-                }
             }
             catch(const Exception& ex) {
                 userInterface.reportError(ex);
@@ -317,12 +320,12 @@ void GuiApplication::initializeUserInterface(UserInterface& userInterface, const
 
     // If no .ovito state file was specified on the command line, load
     // the user's default state from the standard location.
-    if(datasetContainer.currentSet() == nullptr && Application::instance()->guiMode()) {
+    if(datasetContainer.currentSet() == nullptr && Application::runMode() == Application::AppMode) {
         QString defaultsFilePath = QStandardPaths::locate(QStandardPaths::AppDataLocation, QStringLiteral("defaults.ovito"));
         if(!defaultsFilePath.isEmpty()) {
             try {
-                MainThreadOperation loadOperation(true);
-                if(OORef<DataSet> dataset = datasetContainer.loadDataset(defaultsFilePath)) {
+                OORef<DataSet> dataset = DataSet::createFromFile(defaultsFilePath);
+                if(userInterface.checkLoadedDataset(dataset)) {
                     dataset->setFilePath({});
                     datasetContainer.setCurrentSet(std::move(dataset));
                 }
@@ -335,7 +338,7 @@ void GuiApplication::initializeUserInterface(UserInterface& userInterface, const
 
     // Create an empty dataset if nothing has been loaded.
     if(datasetContainer.currentSet() == nullptr) {
-        datasetContainer.newDataset();
+        datasetContainer.setCurrentSet(OORef<DataSet>::create());
     }
 
     // Import data file(s) specified on the command line.
@@ -348,23 +351,18 @@ void GuiApplication::initializeUserInterface(UserInterface& userInterface, const
             else
                 importUrls.push_back(Application::instance()->fileManager().urlFromUserInput(importFilename));
         }
-        try {
+        userInterface.handleExceptions([&](){
             if(!importUrls.empty()) {
                 if(numSessionFiles)
                     throw Exception(tr("Detected incompatible arguments: Cannot open a session state file and a simulation data file at the same time."));
-                if(GuiDataSetContainer* guiContainer = dynamic_object_cast<GuiDataSetContainer>(&datasetContainer)) {
-                    MainThreadOperation importOperation(true);
-                    guiContainer->importFiles(std::move(importUrls));
-                }
+                if(MainWindow* mainWindow = dynamic_object_cast<MainWindow>(&userInterface))
+                    mainWindow->importFiles(std::move(importUrls));
                 else
                     throw Exception(tr("Cannot import data files from the command line when running in console mode."));
             }
             if(numSessionFiles > 1)
                 throw Exception(tr("Detected incompatible arguments: Cannot open multiple session state files at the same time."));
-        }
-        catch(const Exception& ex) {
-            userInterface.reportError(ex);
-        }
+        });
 
         // Make sure we start with a clean undo stack at application startup.
         if(userInterface.undoStack())
@@ -380,28 +378,24 @@ bool GuiApplication::eventFilter(QObject* watched, QEvent* event)
     if(event->type() == QEvent::FileOpen) {
         QFileOpenEvent* openEvent = static_cast<QFileOpenEvent*>(event);
 
-        // Only use an existing main window to load the imported file if there is no existing scene content. Otherwise, open a new window.
+        // Only use an existing main window to load the imported file if it has no existing scene content. Otherwise, open a new window.
         MainWindow* mainWindow = nullptr;
-        for(QWidget* widget : QApplication::topLevelWidgets()) {
-            if(MainWindow* mw = qobject_cast<MainWindow*>(widget)) {
-                if(!mw->datasetContainer().activeScene() || mw->datasetContainer().activeScene()->children().empty()) {
-                    mainWindow = mw;
-                    break;
-                }
+        MainWindow::visitMainWindows([&](MainWindow* mw) {
+            if(!mw->datasetContainer().activeScene() || mw->datasetContainer().activeScene()->children().empty()) {
+                mainWindow = mw;
             }
-        }
+        });
 
         if(mainWindow) {
             mainWindow->handleExceptions([&] {
                 if(openEvent->file().endsWith(".ovito", Qt::CaseInsensitive)) {
-                    if(!mainWindow->datasetContainer().askForSaveChanges())
-                        return;
-                    if(OORef<DataSet> dataset = mainWindow->datasetContainer().loadDataset(openEvent->file())) {
+                    mainWindow->askForSaveChanges();
+                    OORef<DataSet> dataset = DataSet::createFromFile(openEvent->file());
+                    if(mainWindow->checkLoadedDataset(dataset))
                         mainWindow->datasetContainer().setCurrentSet(std::move(dataset));
-                    }
                 }
                 else {
-                    mainWindow->datasetContainer().importFiles({openEvent->url()});
+                    mainWindow->importFiles({openEvent->url()});
                 }
             });
         }
@@ -424,8 +418,11 @@ void GuiApplication::reportError(const Exception& ex, bool blocking)
     // Always display errors in the terminal window.
     Application::reportError(ex, blocking);
 
-    // In GUI mode, display a message box (application modal).
-    if(guiMode()) {
+    // In app mode, display a message box (application modal).
+    if(Application::runMode() == Application::AppMode) {
+        // Require a Qt event loop to show message box.
+        createQtApplication(true);
+
         MessageDialog msgbox;
         msgbox.setWindowTitle(tr("Error - %1").arg(applicationName()));
         msgbox.setStandardButtons(QMessageBox::Ok);
@@ -458,7 +455,7 @@ void GuiApplication::reportError(const Exception& ex, bool blocking)
 ******************************************************************************/
 bool GuiApplication::automaticallyEnableDarkMode()
 {
-#if defined(Q_OS_LINUX) || defined(Q_OS_MAC)
+#if defined(Q_OS_LINUX) || defined(Q_OS_MACOS)
     return true;
 #else
     #if QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
@@ -467,7 +464,6 @@ bool GuiApplication::automaticallyEnableDarkMode()
         return false;
     #endif
 #endif
-
 }
 
 /******************************************************************************
@@ -538,4 +534,55 @@ bool GuiApplication::detectDarkTheme() const
 }
 #endif
 
+#ifdef OVITO_SSH_CLIENT
+
+/******************************************************************************
+* Asks the user for the login password for a SSH server.
+******************************************************************************/
+bool GuiApplication::askUserForPassword(const QString& hostname, const QString& username, QString& password)
+{
+    bool ok;
+    password = QInputDialog::getText(nullptr, tr("SSH Password Authentication"),
+        tr("<p>OVITO is connecting to remote host <b>%1</b> via SSH.</p></p>Please enter the password for user <b>%2</b>:</p>").arg(hostname.toHtmlEscaped()).arg(username.toHtmlEscaped()),
+        QLineEdit::Password, password, &ok);
+    return ok;
+}
+
+/******************************************************************************
+* Asks the user for the passphrase for a private SSH key.
+******************************************************************************/
+bool GuiApplication::askUserForKeyPassphrase(const QString& hostname, const QString& prompt, QString& passphrase)
+{
+    bool ok;
+    passphrase = QInputDialog::getText(nullptr, tr("SSH Remote Connection"),
+        tr("<p>OVITO is connecting to remote host <b>%1</b> via SSH.</p><p>%2</p>").arg(hostname.toHtmlEscaped()).arg(prompt.toHtmlEscaped()),
+        QLineEdit::Password, passphrase, &ok);
+    return ok;
+}
+
+/******************************************************************************
+* Asks the user for the answer to a keyboard-interactive question sent by the SSH server.
+******************************************************************************/
+bool GuiApplication::askUserForKbiResponse(const QString& hostname, const QString& username, const QString& instruction, const QString& question, bool showAnswer, QString& answer)
+{
+    bool ok;
+    answer = QInputDialog::getText(nullptr, tr("SSH Keyboard-Interactive Authentication"),
+        tr("<p>OVITO is connecting to remote host <b>%1</b> via SSH.</p></p>Please enter your response to the following question sent by the SSH server:</p><p>%2 <b>%3</b></p>").arg(hostname.toHtmlEscaped()).arg(instruction.toHtmlEscaped()).arg(question.toHtmlEscaped()),
+        showAnswer ? QLineEdit::Normal : QLineEdit::Password, QString(), &ok);
+    return ok;
+}
+
+/******************************************************************************
+* Informs the user about an unknown SSH host.
+******************************************************************************/
+bool GuiApplication::detectedUnknownSshServer(const QString& hostname, const QString& unknownHostMessage, const QString& hostPublicKeyHash)
+{
+    return MessageDialog::question(nullptr, tr("SSH Unknown Remote Host"),
+        tr("<p>OVITO is connecting to unknown remote host <b>%1</b> via SSH.</p><p>%2</p><p>Host key fingerprint is %3</p><p>Are you sure you want to continue connecting?</p>")
+        .arg(hostname.toHtmlEscaped()).arg(unknownHostMessage.toHtmlEscaped()).arg(hostPublicKeyHash),
+        QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes;
+}
+
+#endif
+
 }   // End of namespace
