@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -25,6 +25,7 @@
 #include <ovito/gui/desktop/properties/BooleanActionParameterUI.h>
 #include <ovito/gui/desktop/properties/IntegerParameterUI.h>
 #include <ovito/gui/desktop/properties/SubObjectParameterUI.h>
+#include <ovito/gui/desktop/widgets/general/EnterLineEdit.h>
 #include <ovito/gui/desktop/dialogs/ModalPropertiesEditorDialog.h>
 #include <ovito/gui/desktop/dialogs/ImportFileDialog.h>
 #include <ovito/gui/desktop/dialogs/ImportRemoteFileDialog.h>
@@ -41,7 +42,7 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(FileSourceEditor);
+IMPLEMENT_CREATABLE_OVITO_CLASS(FileSourceEditor);
 SET_OVITO_OBJECT_EDITOR(FileSource, FileSourceEditor);
 
 /******************************************************************************
@@ -69,7 +70,7 @@ void FileSourceEditor::createUI(const RolloutInsertionParameters& rolloutParams)
     toolbar->addAction(QIcon::fromTheme("file_import_object_reload"), tr("Reload file"), this, SLOT(onReloadFrame()));
     toolbar->addAction(QIcon::fromTheme("file_import_object_refresh_animation"), tr("Update trajectory frames"), this, SLOT(onReloadAnimation()));
     QAction* preloadTrajAction = toolbar->addAction(QIcon::fromTheme("file_cache_pipeline_output"), tr("Load entire trajectory into memory"));
-    BooleanActionParameterUI* preloadTrajectoryUI = new BooleanActionParameterUI(this, PROPERTY_FIELD(FileSource::pipelineTrajectoryCachingEnabled), preloadTrajAction);
+    createParamUI<BooleanActionParameterUI>(PROPERTY_FIELD(FileSource::pipelineTrajectoryCachingEnabled), preloadTrajAction);
 
     QGroupBox* sourceBox = new QGroupBox(tr("Data source"), rollout);
     layout->addWidget(sourceBox);
@@ -98,7 +99,7 @@ void FileSourceEditor::createUI(const RolloutInsertionParameters& rolloutParams)
     gridlayout2->setContentsMargins(4,4,4,4);
     gridlayout2->setVerticalSpacing(2);
     gridlayout2->setColumnStretch(1, 1);
-    _wildcardPatternTextbox = new QLineEdit();
+    _wildcardPatternTextbox = new EnterLineEdit();
     connect(_wildcardPatternTextbox, &QLineEdit::returnPressed, this, &FileSourceEditor::onWildcardPatternEntered);
 
     label = new QLabel(tr("Search pattern:"));
@@ -106,7 +107,7 @@ void FileSourceEditor::createUI(const RolloutInsertionParameters& rolloutParams)
     gridlayout2->addWidget(label, 0, 0);
     gridlayout2->addWidget(_wildcardPatternTextbox, 0, 1);
 
-    BooleanParameterUI* autoGenerateFilePatternUI = new BooleanParameterUI(this, PROPERTY_FIELD(FileSource::autoGenerateFilePattern));
+    BooleanParameterUI* autoGenerateFilePatternUI = createParamUI<BooleanParameterUI>(PROPERTY_FIELD(FileSource::autoGenerateFilePattern));
     autoGenerateFilePatternUI->checkBox()->setText(tr("auto-generate"));
     gridlayout2->addWidget(autoGenerateFilePatternUI->checkBox(), 1, 0);
     maxLabelWidth = std::max(autoGenerateFilePatternUI->checkBox()->sizeHint().width(), maxLabelWidth);
@@ -174,18 +175,13 @@ void FileSourceEditor::createUI(const RolloutInsertionParameters& rolloutParams)
     sublayout->addWidget(_statusLabel);
 
     // Show settings editor of importer class.
-    new SubObjectParameterUI(this, PROPERTY_FIELD(FileSource::importer), rolloutParams.after(rollout));
+    createParamUI<SubObjectParameterUI>(PROPERTY_FIELD(FileSource::importer), rolloutParams.after(rollout));
 
     // Whenever a new FileSource gets loaded into the editor:
-    connect(this, &PropertiesEditor::contentsReplaced, this, [this, con = QMetaObject::Connection()](RefTarget* editObject) mutable {
-        disconnect(con);
-
+    connect(this, &PropertiesEditor::contentsReplaced, this, [this](RefTarget* editObject) mutable {
         // Update displayed information.
         updateFramesList();
         updateDisplayedInformation();
-
-        // Update the frames list displayed in the UI whenever it changes.
-        con = editObject ? connect(static_object_cast<FileSource>(editObject), &FileSource::framesListChanged, this, &FileSourceEditor::updateFramesList) : QMetaObject::Connection();
     });
 }
 
@@ -275,7 +271,7 @@ void FileSourceEditor::onPickRemoteInputFile()
 /******************************************************************************
 * Loads a new file into the FileSource.
 ******************************************************************************/
-bool FileSourceEditor::importNewFile(FileSource* fileSource, const QUrl& url, OvitoClassPtr importerType, const QString& importerFormat)
+void FileSourceEditor::importNewFile(FileSource* fileSource, const QUrl& url, OvitoClassPtr importerType, const QString& importerFormat)
 {
     OORef<FileImporter> importer;
 
@@ -284,10 +280,7 @@ bool FileSourceEditor::importNewFile(FileSource* fileSource, const QUrl& url, Ov
 
         // Detect file format.
         Future<OORef<FileImporter>> importerFuture = FileImporter::autodetectFileFormat(url, fileSource->importer());
-        if(!importerFuture.waitForFinished())
-            return false;
-
-        importer = importerFuture.result();
+        importer = importerFuture.blockForResult();
         if(!importer)
             throw Exception(tr("Could not detect the format of the file to be imported. The format might not be supported."));
     }
@@ -301,7 +294,7 @@ bool FileSourceEditor::importNewFile(FileSource* fileSource, const QUrl& url, Ov
             // Create a new importer instance.
             importer = static_object_cast<FileImporter>(importerType->createInstance());
             if(!importer)
-                return false;
+                return;
         }
         importer->setSelectedFileFormat(importerFormat);
     }
@@ -326,7 +319,7 @@ bool FileSourceEditor::importNewFile(FileSource* fileSource, const QUrl& url, Ov
             "<p>In either case, modifiers you have added to the pipeline will be preserved.</p>"));
         int result = msgBox.exec();
         if(result == QMessageBox::Cancel)
-            return false; // Operation canceled by user.
+            this_task::cancelAndThrow(); // Operation canceled by user.
         else if(result == QMessageBox::Yes)
             keepExistingDataCollection = true;
     }
@@ -337,14 +330,14 @@ bool FileSourceEditor::importNewFile(FileSource* fileSource, const QUrl& url, Ov
         if(editorClass && editorClass->isDerivedFrom(FileImporterEditor::OOClass())) {
             OORef<FileImporterEditor> editor = dynamic_object_cast<FileImporterEditor>(editorClass->createInstance());
             if(editor) {
-                if(!editor->inspectNewFile(newImporter, url, mainWindow()))
-                    return false;
+                editor->inspectNewFile(newImporter, url, mainWindow());
+                this_task::throwIfCanceled();
             }
         }
     }
 
     // Set the new input location.
-    return fileSource->setSource({url}, std::move(newImporter), false, keepExistingDataCollection);
+    fileSource->setSource({url}, std::move(newImporter), false, keepExistingDataCollection);
 }
 
 /******************************************************************************
@@ -370,14 +363,19 @@ void FileSourceEditor::onReloadAnimation()
     if(FileSource* fileSource = static_object_cast<FileSource>(editObject())) {
         handleExceptions([&] {
             // Let the FileSource update the list of source animation frames.
-            // After the update is complete, jump to the last one of the newly added animation frames.
-            int oldFrameCount = fileSource->frames().size();
-            fileSource->updateListOfFrames(true).finally(*fileSource, [fileSource, oldFrameCount, anim=OORef<AnimationSettings>(mainWindow().datasetContainer().activeAnimationSettings())](Task& task) noexcept {
-                if(!task.isCanceled() && fileSource->frames().size() > oldFrameCount && anim) {
-                    AnimationTime time = fileSource->sourceFrameToAnimationTime(fileSource->frames().size() - 1);
-                    anim->setCurrentFrame(time.frame());
-                }
-            });
+            auto future = fileSource->updateListOfFrames(true);
+
+            // After the trajectory update is complete, jump to the last one of the newly added animation frames.
+            OORef<AnimationSettings> anim = mainWindow().datasetContainer().activeAnimationSettings();
+            if(anim && anim->currentFrame() == anim->lastFrame()) {
+                int oldFrameCount = fileSource->frames().size();
+                future.finally(ObjectExecutor(fileSource), [fileSource, oldFrameCount, anim=std::move(anim)](Task& task) noexcept {
+                    if(!task.isCanceled() && fileSource->frames().size() > oldFrameCount) {
+                        AnimationTime time = fileSource->sourceFrameToAnimationTime(fileSource->frames().size() - 1);
+                        anim->setCurrentFrame(time.frame());
+                    }
+                });
+            }
         });
     }
 }
@@ -449,7 +447,7 @@ void FileSourceEditor::updateDisplayedInformation()
 
     if(_timeSeriesLabel) {
         if(!fileSource->frames().empty())
-            _timeSeriesLabel->setText(tr("Showing frame %1 of %2").arg(fileSource->dataCollectionFrame()+1).arg(fileSource->frames().count()));
+            _timeSeriesLabel->setText(tr("Showing frame %1 of %2").arg(fileSource->dataCollectionFrame() + 1).arg(fileSource->frames().count()));
         else
             _timeSeriesLabel->setText(tr("No frames available"));
     }
@@ -539,6 +537,9 @@ bool FileSourceEditor::referenceEvent(RefTarget* source, const ReferenceEvent& e
                 QTimer::singleShot(200, this, &FileSourceEditor::updateDisplayedInformation);
             }
         }
+        else if(event.type() == ReferenceEvent::AnimationFramesChanged) {
+            updateFramesList();
+        }
     }
     return PropertiesEditor::referenceEvent(source, event);
 }
