@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -25,6 +25,7 @@
 #include <ovito/core/utilities/units/UnitsManager.h>
 #include <ovito/core/viewport/ViewportSettings.h>
 #include <ovito/gui/desktop/widgets/general/SpinnerWidget.h>
+#include <ovito/gui/desktop/widgets/general/EnterLineEdit.h>
 #include <ovito/gui/desktop/mainwin/MainWindow.h>
 #include <ovito/gui/base/actions/ActionManager.h>
 #include "AdjustViewDialog.h"
@@ -41,19 +42,17 @@ AdjustViewDialog::AdjustViewDialog(MainWindow& mainWindow, Viewport* viewport, Q
     setFeatures(QDockWidget::DockWidgetFloatable | QDockWidget::DockWidgetClosable);
     setAllowedAreas(Qt::NoDockWidgetArea);
     setFloating(true);
-    setAttribute(Qt::WA_DeleteOnClose);
+    setAttribute(Qt::WA_DeleteOnClose); // Make sure AdjustViewDialog instance gets deleted when the dialog is closed.
+
     QWidget* widget = new QWidget();
     setWidget(widget);
 
-    OVITO_ASSERT(viewport->window());
-
     _oldViewType = viewport->viewType();
     _oldCameraTM = viewport->cameraTransformation();
     _oldFOV = viewport->fieldOfView();
 
     _viewportListener.setTarget(viewport);
-
-    connect(&_viewportListener, &RefTargetListenerBase::notificationEvent, this, [this](RefTarget* source, const ReferenceEvent& event) {
+    _viewportListener.connect(this, [this](RefTarget* source, const ReferenceEvent& event) {
         // Update the values displayed in the dialog when the viewport camera is moved by the user.
         if(event.type() == ReferenceEvent::TargetChanged)
             updateGUI();
@@ -86,32 +85,32 @@ AdjustViewDialog::AdjustViewDialog(MainWindow& mainWindow, Viewport* viewport, Q
     fieldLayout = new QHBoxLayout();
     fieldLayout->setContentsMargins(0,0,0,0);
     fieldLayout->setSpacing(0);
-    textBox = new QLineEdit();
+    textBox = new EnterLineEdit();
     _camPosXSpinner->setTextBox(textBox);
     fieldLayout->addWidget(textBox);
     fieldLayout->addWidget(_camPosXSpinner);
     gridLayout->addLayout(fieldLayout, 0, 1);
-    connect(_camPosXSpinner, &SpinnerWidget::spinnerValueChanged, this, &AdjustViewDialog::onAdjustCamera);
+    connect(_camPosXSpinner, &SpinnerWidget::valueChanged, this, &AdjustViewDialog::onAdjustCamera);
 
     fieldLayout = new QHBoxLayout();
     fieldLayout->setContentsMargins(0,0,0,0);
     fieldLayout->setSpacing(0);
-    textBox = new QLineEdit();
+    textBox = new EnterLineEdit();
     _camPosYSpinner->setTextBox(textBox);
     fieldLayout->addWidget(textBox);
     fieldLayout->addWidget(_camPosYSpinner);
     gridLayout->addLayout(fieldLayout, 0, 2);
-    connect(_camPosYSpinner, &SpinnerWidget::spinnerValueChanged, this, &AdjustViewDialog::onAdjustCamera);
+    connect(_camPosYSpinner, &SpinnerWidget::valueChanged, this, &AdjustViewDialog::onAdjustCamera);
 
     fieldLayout = new QHBoxLayout();
     fieldLayout->setContentsMargins(0,0,0,0);
     fieldLayout->setSpacing(0);
-    textBox = new QLineEdit();
+    textBox = new EnterLineEdit();
     _camPosZSpinner->setTextBox(textBox);
     fieldLayout->addWidget(textBox);
     fieldLayout->addWidget(_camPosZSpinner);
     gridLayout->addLayout(fieldLayout, 0, 3);
-    connect(_camPosZSpinner, &SpinnerWidget::spinnerValueChanged, this, &AdjustViewDialog::onAdjustCamera);
+    connect(_camPosZSpinner, &SpinnerWidget::valueChanged, this, &AdjustViewDialog::onAdjustCamera);
 
     QGroupBox* viewDirBox = new QGroupBox(tr("View direction"));
     mainLayout->addWidget(viewDirBox);
@@ -132,32 +131,32 @@ AdjustViewDialog::AdjustViewDialog(MainWindow& mainWindow, Viewport* viewport, Q
     fieldLayout = new QHBoxLayout();
     fieldLayout->setContentsMargins(0,0,0,0);
     fieldLayout->setSpacing(0);
-    textBox = new QLineEdit();
+    textBox = new EnterLineEdit();
     _camDirXSpinner->setTextBox(textBox);
     fieldLayout->addWidget(textBox);
     fieldLayout->addWidget(_camDirXSpinner);
     gridLayout->addLayout(fieldLayout, 0, 1);
-    connect(_camDirXSpinner, &SpinnerWidget::spinnerValueChanged, this, &AdjustViewDialog::onAdjustCamera);
+    connect(_camDirXSpinner, &SpinnerWidget::valueChanged, this, &AdjustViewDialog::onAdjustCamera);
 
     fieldLayout = new QHBoxLayout();
     fieldLayout->setContentsMargins(0,0,0,0);
     fieldLayout->setSpacing(0);
-    textBox = new QLineEdit();
+    textBox = new EnterLineEdit();
     _camDirYSpinner->setTextBox(textBox);
     fieldLayout->addWidget(textBox);
     fieldLayout->addWidget(_camDirYSpinner);
     gridLayout->addLayout(fieldLayout, 0, 2);
-    connect(_camDirYSpinner, &SpinnerWidget::spinnerValueChanged, this, &AdjustViewDialog::onAdjustCamera);
+    connect(_camDirYSpinner, &SpinnerWidget::valueChanged, this, &AdjustViewDialog::onAdjustCamera);
 
     fieldLayout = new QHBoxLayout();
     fieldLayout->setContentsMargins(0,0,0,0);
     fieldLayout->setSpacing(0);
-    textBox = new QLineEdit();
+    textBox = new EnterLineEdit();
     _camDirZSpinner->setTextBox(textBox);
     fieldLayout->addWidget(textBox);
     fieldLayout->addWidget(_camDirZSpinner);
     gridLayout->addLayout(fieldLayout, 0, 3);
-    connect(_camDirZSpinner, &SpinnerWidget::spinnerValueChanged, this, &AdjustViewDialog::onAdjustCamera);
+    connect(_camDirZSpinner, &SpinnerWidget::valueChanged, this, &AdjustViewDialog::onAdjustCamera);
 
     QGroupBox* upDirBox = new QGroupBox(tr("Up direction"));
     mainLayout->addWidget(upDirBox);
@@ -187,7 +186,7 @@ AdjustViewDialog::AdjustViewDialog(MainWindow& mainWindow, Viewport* viewport, Q
     fieldLayout = new QHBoxLayout();
     fieldLayout->setContentsMargins(0,0,0,0);
     fieldLayout->setSpacing(0);
-    textBox = new QLineEdit();
+    textBox = new EnterLineEdit();
     _rollAngleSpinner->setTextBox(textBox);
     fieldLayout->addWidget(textBox);
     fieldLayout->addWidget(_rollAngleSpinner);
@@ -195,13 +194,13 @@ AdjustViewDialog::AdjustViewDialog(MainWindow& mainWindow, Viewport* viewport, Q
     hLayout->addStretch(1);
     gridLayout->addLayout(hLayout, 1, 0, 1, 4);
 
-    connect(_constrainRotationBtn, &QRadioButton::clicked, this, [this](bool checked) {
+    connect(_constrainRotationBtn, &QRadioButton::clicked, this, [](bool checked) {
         ViewportSettings::getSettings().setConstrainCameraRotation(checked);
     });
-    connect(_rollAngleBtn, &QRadioButton::clicked, this, [this](bool checked) {
+    connect(_rollAngleBtn, &QRadioButton::clicked, this, [](bool checked) {
         ViewportSettings::getSettings().setConstrainCameraRotation(!checked);
     });
-    connect(_rollAngleSpinner, &SpinnerWidget::spinnerValueChanged, this, &AdjustViewDialog::onAdjustCamera);
+    connect(_rollAngleSpinner, &SpinnerWidget::valueChanged, this, &AdjustViewDialog::onAdjustCamera);
 
     gridLayout->addWidget(new QLabel(tr("XYZ (read-only):")), 2, 0);
     _upDirXSpinner = new SpinnerWidget();
@@ -214,13 +213,13 @@ AdjustViewDialog::AdjustViewDialog(MainWindow& mainWindow, Viewport* viewport, Q
     fieldLayout = new QHBoxLayout();
     fieldLayout->setContentsMargins(0,0,0,0);
     fieldLayout->setSpacing(0);
-    textBox = new QLineEdit();
+    textBox = new EnterLineEdit();
     _upDirXSpinner->setTextBox(textBox);
     fieldLayout->addWidget(textBox);
     fieldLayout->addWidget(_upDirXSpinner);
     gridLayout->addLayout(fieldLayout, 2, 1);
 #if 0
-    connect(_upDirXSpinner, &SpinnerWidget::spinnerValueChanged, this, &AdjustViewDialog::onAdjustCamera);
+    connect(_upDirXSpinner, &SpinnerWidget::valueChanged, this, &AdjustViewDialog::onAdjustCamera);
 #else
     _upDirXSpinner->hide();
     textBox->setReadOnly(true);
@@ -229,13 +228,13 @@ AdjustViewDialog::AdjustViewDialog(MainWindow& mainWindow, Viewport* viewport, Q
     fieldLayout = new QHBoxLayout();
     fieldLayout->setContentsMargins(0,0,0,0);
     fieldLayout->setSpacing(0);
-    textBox = new QLineEdit();
+    textBox = new EnterLineEdit();
     _upDirYSpinner->setTextBox(textBox);
     fieldLayout->addWidget(textBox);
     fieldLayout->addWidget(_upDirYSpinner);
     gridLayout->addLayout(fieldLayout, 2, 2);
 #if 0
-    connect(_upDirYSpinner, &SpinnerWidget::spinnerValueChanged, this, &AdjustViewDialog::onAdjustCamera);
+    connect(_upDirYSpinner, &SpinnerWidget::valueChanged, this, &AdjustViewDialog::onAdjustCamera);
 #else
     _upDirYSpinner->hide();
     textBox->setReadOnly(true);
@@ -244,13 +243,13 @@ AdjustViewDialog::AdjustViewDialog(MainWindow& mainWindow, Viewport* viewport, Q
     fieldLayout = new QHBoxLayout();
     fieldLayout->setContentsMargins(0,0,0,0);
     fieldLayout->setSpacing(0);
-    textBox = new QLineEdit();
+    textBox = new EnterLineEdit();
     _upDirZSpinner->setTextBox(textBox);
     fieldLayout->addWidget(textBox);
     fieldLayout->addWidget(_upDirZSpinner);
     gridLayout->addLayout(fieldLayout, 2, 3);
 #if 0
-    connect(_camDirZSpinner, &SpinnerWidget::spinnerValueChanged, this, &AdjustViewDialog::onAdjustCamera);
+    connect(_camDirZSpinner, &SpinnerWidget::valueChanged, this, &AdjustViewDialog::onAdjustCamera);
 #else
     _upDirZSpinner->hide();
     textBox->setReadOnly(true);
@@ -279,12 +278,12 @@ AdjustViewDialog::AdjustViewDialog(MainWindow& mainWindow, Viewport* viewport, Q
     fieldLayout = new QHBoxLayout();
     fieldLayout->setContentsMargins(0,0,0,0);
     fieldLayout->setSpacing(0);
-    textBox = new QLineEdit();
+    textBox = new EnterLineEdit();
     _camFOVAngleSpinner->setTextBox(textBox);
     fieldLayout->addWidget(textBox);
     fieldLayout->addWidget(_camFOVAngleSpinner);
     gridLayout->addLayout(fieldLayout, 1, 2);
-    connect(_camFOVAngleSpinner, &SpinnerWidget::spinnerValueChanged, this, &AdjustViewDialog::onAdjustCamera);
+    connect(_camFOVAngleSpinner, &SpinnerWidget::valueChanged, this, &AdjustViewDialog::onAdjustCamera);
 
     _camParallel = new QRadioButton(tr("Parallel:"));
     connect(_camParallel, &QRadioButton::clicked, this, &AdjustViewDialog::onAdjustCamera);
@@ -301,18 +300,18 @@ AdjustViewDialog::AdjustViewDialog(MainWindow& mainWindow, Viewport* viewport, Q
     fieldLayout = new QHBoxLayout();
     fieldLayout->setContentsMargins(0,0,0,0);
     fieldLayout->setSpacing(0);
-    textBox = new QLineEdit();
+    textBox = new EnterLineEdit();
     _camFOVSpinner->setTextBox(textBox);
     fieldLayout->addWidget(textBox);
     fieldLayout->addWidget(_camFOVSpinner);
     gridLayout->addLayout(fieldLayout, 3, 2);
-    connect(_camFOVSpinner, &SpinnerWidget::spinnerValueChanged, this, &AdjustViewDialog::onAdjustCamera);
+    connect(_camFOVSpinner, &SpinnerWidget::valueChanged, this, &AdjustViewDialog::onAdjustCamera);
 
     QDialogButtonBox* buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel | QDialogButtonBox::Help, Qt::Horizontal, this);
     connect(buttonBox, &QDialogButtonBox::accepted, this, &AdjustViewDialog::close);
     connect(buttonBox, &QDialogButtonBox::rejected, this, &AdjustViewDialog::onCancel);
     connect(buttonBox, &QDialogButtonBox::helpRequested, &mainWindow, [&mainWindow]() {
-        mainWindow.actionManager()->openHelpTopic("manual:viewports.adjust_view_dialog");
+        mainWindow.actionManager()->openHelpTopic(QStringLiteral("manual:viewports.adjust_view_dialog"));
     });
     mainLayout->addWidget(buttonBox);
 
