@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -62,7 +62,7 @@ public:
         spinner->setMinValue(_minValue);
         spinner->setMaxValue(_maxValue);
         layout->addWidget(spinner);
-        connect(spinner, &SpinnerWidget::spinnerValueChanged, [this,container]() {
+        connect(spinner, &SpinnerWidget::valueChanged, [this,container]() {
             Q_EMIT const_cast<NumericalItemDelegate*>(this)->commitData(container);
         });
         return container;
@@ -101,15 +101,15 @@ public:
         _ctrl.setTarget(ctrl);
         _ctrlType = ctrl->controllerType();
         _keys.setTargets(ctrl->keys());
-        connect(&_ctrl, &RefTargetListener<KeyframeController>::notificationEvent, this, &AnimationKeyModel::onCtrlEvent);
-        connect(&_keys, &VectorRefTargetListener<AnimationKey>::notificationEvent, this, &AnimationKeyModel::onKeyEvent);
+        _ctrl.connect(this, &AnimationKeyModel::onCtrlEvent);
+        _keys.connect(this, &AnimationKeyModel::onKeyEvent);
     }
 
     /// Returns the animation controller being edited.
     KeyframeController* ctrl() const { return _ctrl.target(); }
 
     /// Returns the list of animation keys.
-    const QVector<AnimationKey*>& keys() const { return _keys.targets(); }
+    const auto& keys() const { return _keys.targets(); }
 
     /// Returns the number of rows.
     virtual int rowCount(const QModelIndex& parent = QModelIndex()) const override {
@@ -473,7 +473,7 @@ void AnimationKeyEditorDialog::onDeleteKey()
 {
     QModelIndexList selection = _tableWidget->selectionModel()->selectedRows();
     _mainWindow.performActions(*this, [&]() {
-        QVector<AnimationKey*> keysToDelete;
+        QVector<OORef<AnimationKey>> keysToDelete;
         for(const QModelIndex& index : selection) {
             OVITO_ASSERT(index.row() >= 0 && index.row() < ctrl()->keys().size());
             keysToDelete.push_back(ctrl()->keys()[index.row()]);
