@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -22,6 +22,7 @@
 
 #include <ovito/gui/desktop/GUI.h>
 #include <ovito/gui/desktop/mainwin/MainWindow.h>
+#include <ovito/gui/desktop/widgets/general/EnterLineEdit.h>
 #include <ovito/gui/base/actions/ActionManager.h>
 #include <ovito/core/dataset/pipeline/ModificationNode.h>
 #include <ovito/core/dataset/scene/Pipeline.h>
@@ -33,8 +34,8 @@ namespace Ovito {
 /******************************************************************************
 * Constructor.
 ******************************************************************************/
-ClonePipelineDialog::ClonePipelineDialog(MainWindow& mainWindow, Pipeline* pipeline, QWidget* parent) :
-    QDialog(parent), _mainWindow(mainWindow), _originalPipeline(pipeline)
+ClonePipelineDialog::ClonePipelineDialog(MainWindow& mainWindow, SceneNode* sceneNode, QWidget* parent) :
+    QDialog(parent), _mainWindow(mainWindow), _originalSceneNode(sceneNode), _originalPipeline(sceneNode->pipeline())
 {
     setWindowTitle(tr("Clone pipeline"));
 
@@ -80,15 +81,15 @@ ClonePipelineDialog::ClonePipelineDialog(MainWindow& mainWindow, Pipeline* pipel
     mainLayout->addWidget(nameBox);
     sublayout2 = new QHBoxLayout(nameBox);
     sublayout2->setSpacing(2);
-    _originalNameEdit = new QLineEdit(nameBox);
-    _cloneNameEdit = new QLineEdit(nameBox);
+    _originalNameEdit = new EnterLineEdit(nameBox);
+    _cloneNameEdit = new EnterLineEdit(nameBox);
     sublayout2->addWidget(new QLabel(tr("Original:")));
     sublayout2->addWidget(_originalNameEdit, 1);
     sublayout2->addSpacing(10);
     sublayout2->addWidget(new QLabel(tr("Clone:")));
     sublayout2->addWidget(_cloneNameEdit, 1);
-    _originalNameEdit->setPlaceholderText(pipeline->objectTitle());
-    _cloneNameEdit->setPlaceholderText(pipeline->objectTitle());
+    _originalNameEdit->setPlaceholderText(sceneNode->objectTitle());
+    _cloneNameEdit->setPlaceholderText(sceneNode->objectTitle());
 
     QDialogButtonBox* buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel | QDialogButtonBox::Help, Qt::Horizontal, this);
     connect(buttonBox, &QDialogButtonBox::accepted, this, &ClonePipelineDialog::onAccept);
@@ -111,6 +112,8 @@ ClonePipelineDialog::ClonePipelineDialog(MainWindow& mainWindow, Pipeline* pipel
                             "  background-color: rgb(230,180,180); "
                             "}");
     mainLayout->insertWidget(0, noticeWidget);
+#else
+    buttonBox->button(QDialogButtonBox::Ok)->setDefault(true);
 #endif
 
     resize(sizeHint());
@@ -169,7 +172,7 @@ void ClonePipelineDialog::initializeGraphicsScene()
 
     QGraphicsSimpleTextItem* textItem;
 
-    auto addShadowEffect = [this](QGraphicsItem* item) {
+    auto addShadowEffect = [](QGraphicsItem* item) {
 #if 0   // Shadows diabled to work around Qt bug https://bugreports.qt.io/browse/QTBUG-65035
         QGraphicsDropShadowEffect* effect = new QGraphicsDropShadowEffect(this);
         effect->setOffset(3);
@@ -426,11 +429,12 @@ void ClonePipelineDialog::onAccept()
 
         // Clone the scene node.
         CloneHelper cloneHelper;
+        OORef<SceneNode> clonedSceneNode = cloneHelper.cloneObject(_originalSceneNode, false);
         OORef<Pipeline> clonedPipeline = cloneHelper.cloneObject(_originalPipeline, false);
         OVITO_ASSERT(clonedPipeline->head() == _originalPipeline->head());
 
         // The scene we are working in.
-        Scene* scene = _originalPipeline->scene();
+        Scene* scene = _originalSceneNode->scene();
 
         // Clone the pipeline nodes.
         OORef<PipelineNode> precedingNode;
@@ -461,34 +465,35 @@ void ClonePipelineDialog::onAccept()
             }
         }
         clonedPipeline->setHead(precedingNode);
+        clonedSceneNode->setPipeline(clonedPipeline);
 
-        // Give the cloned pipeline the user-defined name.
+        // Give the cloned scene node the user-defined name.
         QString nodeName = _cloneNameEdit->text().trimmed();
         if(nodeName.isEmpty() == false)
-            clonedPipeline->setSceneNodeName(nodeName);
+            clonedSceneNode->setSceneNodeName(nodeName);
 
-        // Give the original pipeline the new user-defined name.
+        // Give the original scene node the new user-defined name.
         nodeName = _originalNameEdit->text().trimmed();
         if(nodeName.isEmpty() == false)
-            _originalPipeline->setSceneNodeName(nodeName);
+            _originalSceneNode->setSceneNodeName(nodeName);
 
-        // Translate cloned pipeline in scene.
+        // Translate cloned node in scene.
         int displacementMode = _displacementDirectionGroup->checkedAction()->data().toInt();
         if(displacementMode != -1) {
             AnimationTime time = scene ? scene->animationSettings()->currentTime() : AnimationTime(0);
-            const Box3& bbox = _originalPipeline->worldBoundingBox(time);
+            const Box3& bbox = _originalSceneNode->worldBoundingBox(time);
             Vector3 translation = Vector3::Zero();
             translation[displacementMode] = bbox.size(displacementMode) + FloatType(0.2) * bbox.size().length();
-            clonedPipeline->transformationController()->translate(time, translation, AffineTransformation::Identity());
+            clonedSceneNode->transformationController()->translate(time, translation, AffineTransformation::Identity());
         }
 
-        // Insert cloned pipeline into scene.
-        if(SceneNode* parentNode = _originalPipeline->parentNode())
-            parentNode->addChildNode(clonedPipeline);
+        // Insert cloned scene node into scene.
+        if(SceneNode* parentNode = _originalSceneNode->parentNode())
+            parentNode->addChildNode(clonedSceneNode);
 
-        // Select cloned pipeline.
+        // Select cloned scene node.
         if(scene)
-            scene->selection()->setNode(clonedPipeline);
+            scene->selection()->setNode(clonedSceneNode);
     });
     accept();
 }
