@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -23,8 +23,8 @@
 #include <ovito/gui/desktop/GUI.h>
 #include <ovito/gui/desktop/mainwin/MainWindow.h>
 #include <ovito/gui/desktop/viewport/ViewportMenu.h>
+#include <ovito/gui/vpwidget/WidgetViewportWindow.h>
 #include <ovito/gui/base/actions/ActionManager.h>
-#include <ovito/gui/base/viewport/BaseViewportWindow.h>
 #include <ovito/gui/base/viewport/ViewportInputMode.h>
 #include <ovito/gui/base/viewport/ViewportInputManager.h>
 #include <ovito/core/viewport/ViewportSettings.h>
@@ -47,10 +47,22 @@ ViewportsPanel::ViewportsPanel(MainWindow& mainWindow) : _mainWindow(mainWindow)
     // Track viewport input mode changes.
     connect(mainWindow.viewportInputManager(), &ViewportInputManager::inputModeChanged, this, &ViewportsPanel::onInputModeChanged);
 
+    // Show a context menu when the user clicks a viewport's caption.
+    connect(mainWindow.viewportInputManager(), &ViewportInputManager::contextMenuRequested, this, &ViewportsPanel::onViewportMenuRequested);
+
     // Repaint when auto-key animation mode is toggled.
     connect(mainWindow.actionManager()->getAction(ACTION_AUTO_KEY_MODE_TOGGLE), &QAction::toggled, this, qOverload<>(&ViewportsPanel::update));
 
-    // Prevent the viewports from collpasing and disappearing completely.
+    // Repaint the viewport borders when another viewport has been activated.
+    connect(&mainWindow.datasetContainer(), &DataSetContainer::activeViewportChanged, this, qOverload<>(&ViewportsPanel::update));
+
+    // Update layout when a viewport has been maximized or restored.
+    connect(&mainWindow.datasetContainer(), &DataSetContainer::maximizedViewportChanged, this, &ViewportsPanel::invalidateWindowLayout);
+
+    // Update the viewport window positions when the viewport layout is modified.
+    connect(&mainWindow.datasetContainer(), &DataSetContainer::viewportLayoutChanged, this, &ViewportsPanel::invalidateWindowLayout);
+
+    // Prevent the viewports from collapsing and disappearing completely.
     setMinimumSize(40, 40);
 
     // Set the background color of the panel.
@@ -65,92 +77,63 @@ ViewportsPanel::ViewportsPanel(MainWindow& mainWindow) : _mainWindow(mainWindow)
 }
 
 /******************************************************************************
-* Factory method which creates a new viewport window widget. Depending on the
-* user's settings this can be either a OpenGL or a Vulkan window.
+* Factory method which creates a new viewport window widget.
 ******************************************************************************/
-BaseViewportWindow* ViewportsPanel::createViewportWindow(Viewport& vp, MainWindow& mainWindow, QWidget* parent)
+OORef<WidgetViewportWindow> ViewportsPanel::createViewportWindow(Viewport& vp, QWidget* parent)
 {
+    // If a critical error has occurred before, do not try creating a window again.
+    if(_windowCreationIsBroken)
+        return {};
+
     // Select the viewport window implementation to use.
-    QSettings settings;
+    OvitoClassPtr windowClass = ViewportWindow::getInteractiveWindowImplementationClass();
 
-    QByteArray selectedGraphicsApi = qgetenv("OVITO_VIEWPORT_RENDERER");
-#if 0
-    if(selectedGraphicsApi.isEmpty())
-        selectedGraphicsApi = settings.value("rendering/selected_graphics_api").toString().toUtf8();
-#endif
+    // Instantiate the selected viewport window implementation.
+    if(windowClass) {
+        OORef<WidgetViewportWindow> window = static_object_cast<WidgetViewportWindow>(windowClass->createInstance());
+        window->initializeWindow(&vp, _mainWindow, parent);
 
-    const QMetaObject* viewportImplementation = nullptr;
-#if QT_VERSION < QT_VERSION_CHECK(6, 5, 0)
-    for(const QMetaObject* metaType : ViewportWindowInterface::registry()) {
-#else
-    ViewportWindowInterface* (*viewportWindowConstructor)(Viewport*, UserInterface*, QWidget*) = nullptr;
-    for(auto [metaType, constructor] : ViewportWindowInterface::registry()) {
-#endif
-        if(qstrcmp(metaType->className(), "Ovito::OpenGLViewportWindow") == 0) {
-            viewportImplementation = metaType;
-#if QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
-            viewportWindowConstructor = constructor;
-#endif
-        }
-        else if(qstrcmp(metaType->className(), "Ovito::VulkanViewportWindow") == 0 && selectedGraphicsApi.compare("vulkan", Qt::CaseInsensitive) == 0) {
-            viewportImplementation = metaType;
-#if QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
-            viewportWindowConstructor = constructor;
-#endif
-            break;
-        }
-#ifdef OVITO_BUILD_PROFESSIONAL
-        else if(qstrcmp(metaType->className(), "Ovito::AnariViewportWindow") == 0 && selectedGraphicsApi.compare("anari", Qt::CaseInsensitive) == 0) {
-            viewportImplementation = metaType;
-#if QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
-            viewportWindowConstructor = constructor;
-#endif
-            break;
-        }
-#endif
+        // Handle fatal rendering errors.
+        connect(window.get(), &ViewportWindow::fatalError, this, [this](Exception& ex) {
+            if(_windowCreationErrorOccurred)
+                return;
+            _windowCreationErrorOccurred = true;
+            // Automatically switch back to the default OpenGL backend if there is a problem with the current backend.
+            if(!ViewportWindow::revertToDefaultInteractiveWindowImplementation()) {
+                // When already using the default OpenGL backend, do not try to use it again for the current program session.
+                ex.prependGeneralMessage(tr("There is a critical problem with the interactive viewport windows."));
+                _windowCreationIsBroken = true;
+            }
+            _mainWindow.reportError(ex, true);
+            QMetaObject::invokeMethod(this, "recreateViewportWindows", Qt::QueuedConnection);
+        });
+
+        return window;
     }
 
-    if(viewportImplementation) {
-#if QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
-        ViewportWindowInterface* obj = viewportWindowConstructor(&vp, &mainWindow, parent);
-        return static_cast<BaseViewportWindow*>(obj);
-#else
-        qRegisterMetaType<UserInterface*>("UserInterfacePtr");
-        QObject* obj = viewportImplementation->newInstance(Q_ARG(Viewport*, &vp), Q_ARG(UserInterface*, &mainWindow), Q_ARG(QWidget*, parent));
-        OVITO_ASSERT(dynamic_cast<BaseViewportWindow*>(obj));
-        return dynamic_cast<BaseViewportWindow*>(obj);
-#endif
-    }
+    return {};
+}
 
+/******************************************************************************
+* Returns the window that is associated with the given viewport (if any).
+******************************************************************************/
+WidgetViewportWindow* ViewportsPanel::viewportWindow(Viewport* vp)
+{
+    // Check if the viewport already has an associated window.
+    for(const auto& window : _viewportWindows) {
+        if(window->viewport() == vp)
+            return window;
+    }
     return nullptr;
 }
 
 /******************************************************************************
-* Returns the widget that is associated with the given viewport.
+* Returns the widget that hosts the given viewport.
 ******************************************************************************/
 QWidget* ViewportsPanel::viewportWidget(Viewport* vp)
 {
-    OVITO_ASSERT(_viewportConfig != nullptr);
-
-    // Create the viewport window if it hasn't been created for this viewport yet.
-    if(!vp->window() && !_graphicsInitializationErrorOccurred) {
-        try {
-            BaseViewportWindow* viewportWindow = createViewportWindow(*vp, _mainWindow, this);
-            if(!viewportWindow || !viewportWindow->widget())
-                throw Exception(tr("Failed to create viewport window or there is no realtime graphics implementation available. Please check your OVITO installation and the graphics capabilities of your system."));
-            if(_viewportConfig->activeViewport() == vp)
-                viewportWindow->widget()->setFocus();
-            // Show a context menu when the user clicks the viewport caption.
-            connect(vp, &Viewport::contextMenuRequested, this, &ViewportsPanel::onViewportMenuRequested);
-        }
-        catch(const Exception& ex) {
-            _graphicsInitializationErrorOccurred = true;
-            _mainWindow.reportError(ex, true);
-            return nullptr;
-        }
-    }
-
-    if(BaseViewportWindow* window = dynamic_cast<BaseViewportWindow*>(vp->window()))
+    // Check if the viewport already has an associated window.
+    if(WidgetViewportWindow* window = viewportWindow(vp))
         return window->widget();
 
     return nullptr;
@@ -159,21 +142,17 @@ QWidget* ViewportsPanel::viewportWidget(Viewport* vp)
 /******************************************************************************
 * Displays the context menu for a viewport window.
 ******************************************************************************/
-void ViewportsPanel::onViewportMenuRequested(const QPoint& pos)
+void ViewportsPanel::onViewportMenuRequested(ViewportWindow* viewportWindow, const QPoint& pos)
 {
-    // Get the viewport that emitted the signal.
-    Viewport* viewport = qobject_cast<Viewport*>(sender());
-    OVITO_ASSERT(viewport);
+    // Get the viewport's widget.
+    if(QWidget* widget = viewportWidget(viewportWindow->viewport())) {
 
-    // Get the viewport's window.
-    BaseViewportWindow* vpwin = dynamic_cast<BaseViewportWindow*>(viewport->window());
-    OVITO_ASSERT(vpwin && vpwin->widget() && vpwin->widget()->parentWidget() == this);
+        // Create the context menu for the viewport.
+        ViewportMenu contextMenu(_mainWindow, viewportWindow, widget);
 
-    // Create the context menu for the viewport.
-    ViewportMenu contextMenu(_mainWindow, viewport, vpwin->widget());
-
-    // Show menu.
-    contextMenu.show(pos);
+        // Show menu.
+        contextMenu.show(pos);
+    }
 }
 
 /******************************************************************************
@@ -181,22 +160,10 @@ void ViewportsPanel::onViewportMenuRequested(const QPoint& pos)
 ******************************************************************************/
 void ViewportsPanel::onViewportConfigurationReplaced(ViewportConfiguration* newViewportConfiguration)
 {
-    disconnect(_activeViewportChangedConnection);
-    disconnect(_maximizedViewportChangedConnection);
-    disconnect(_viewportLayoutChangedConnection);
     _viewportConfig = newViewportConfiguration;
 
     // Create the interactive viewport windows.
     recreateViewportWindows();
-
-    if(_viewportConfig) {
-        // Repaint the viewport borders when another viewport has been activated.
-        _activeViewportChangedConnection = connect(_viewportConfig, &ViewportConfiguration::activeViewportChanged, this, qOverload<>(&ViewportsPanel::update));
-        // Update layout when a viewport has been maximized.
-        _maximizedViewportChangedConnection = connect(_viewportConfig, &ViewportConfiguration::maximizedViewportChanged, this, &ViewportsPanel::invalidateWindowLayout);
-        // Update the viewport window positions when the viewport layout is modified.
-        _viewportLayoutChangedConnection = connect(_viewportConfig, &ViewportConfiguration::viewportLayoutChanged, this, &ViewportsPanel::invalidateWindowLayout);
-    }
 }
 
 /******************************************************************************
@@ -204,19 +171,24 @@ void ViewportsPanel::onViewportConfigurationReplaced(ViewportConfiguration* newV
 ******************************************************************************/
 void ViewportsPanel::recreateViewportWindows()
 {
-    // Delete all existing viewport widgets first.
-#if QT_VERSION >= QT_VERSION_CHECK(6, 3, 0)
-    for(QWidget* widget : findChildren<QWidget*>(Qt::FindDirectChildrenOnly))
-#else
-    for(QWidget* widget : findChildren<QWidget*>(QString{}, Qt::FindDirectChildrenOnly))
-#endif
-        delete widget;
+    // Delete all widgets of the existing viewport windows immediately.
+    for(auto& window : _viewportWindows)
+        delete window->widget();
 
-    if(_viewportConfig) {
-        // Layout viewport widgets.
-        // This function implicitly creates the Qt widgets for all viewports.
-        layoutViewports();
-    }
+    // Release all viewport window objects.
+    _viewportWindows.clear();
+
+    // Now all child widgets should be gone.
+    OVITO_ASSERT(findChildren<QWidget*>(Qt::FindDirectChildrenOnly).empty());
+
+    // Reset error flag.
+    _windowCreationErrorOccurred = false;
+
+    // Inform listeners.
+    Q_EMIT interactiveWindowImplementationChanged();
+
+    // This implicitly creates the new windows for the viewports by calling createViewportWindows().
+    layoutViewports();
 }
 
 /******************************************************************************
@@ -239,10 +211,8 @@ void ViewportsPanel::onViewportModeCursorChanged(const QCursor& cursor)
 {
     if(!_viewportConfig) return;
 
-    for(Viewport* vp : _viewportConfig->viewports()) {
-        if(ViewportWindowInterface* window = vp->window()) {
-            window->setCursor(cursor);
-        }
+    for(const auto& window : _viewportWindows) {
+        window->setCursor(cursor);
     }
 }
 
@@ -254,10 +224,10 @@ void ViewportsPanel::paintEvent(QPaintEvent* event)
     if(!_viewportConfig) return;
 
     // Get the active viewport and its associated Qt widget.
-    Viewport* vp = _viewportConfig->activeViewport();
-    if(!vp) return;
-    QWidget* vpWidget = viewportWidget(vp);
-    if(!vpWidget || vpWidget->isHidden()) return;
+    Viewport* activeViewport = _viewportConfig->activeViewport();
+    QWidget* activeWidget = activeViewport ? viewportWidget(activeViewport) : nullptr;
+    if(!activeWidget || activeWidget->isHidden())
+        return;
 
     QPainter painter(this);
 
@@ -280,7 +250,7 @@ void ViewportsPanel::paintEvent(QPaintEvent* event)
         // Render a border around the active viewport.
         painter.setPen((QColor)borderColor);
         painter.setBrush(Qt::NoBrush);
-        QRect rect = vpWidget->geometry();
+        QRect rect = activeWidget->geometry();
         rect.adjust(-1, -1, 0, 0);
         painter.drawRect(rect);
         rect.adjust(-1, -1, 1, 1);
@@ -305,22 +275,68 @@ void ViewportsPanel::resizeEvent(QResizeEvent* event)
 }
 
 /******************************************************************************
-* Requests a relayout of the viewport windows.
+* Requests a re-layout of the viewport windows.
 ******************************************************************************/
 void ViewportsPanel::invalidateWindowLayout()
 {
-    if(!_relayoutRequested) {
-        _relayoutRequested = true;
+    if(!_layoutRequested) {
+        _layoutRequested = true;
         QMetaObject::invokeMethod(this, "layoutViewports", Qt::QueuedConnection);
     }
 }
 
+/******************************************************************************
+* Creates the physical viewport windows for the viewports of the current viewport configuration.
+******************************************************************************/
+void ViewportsPanel::createViewportWindows()
+{
+    if(!_viewportConfig)
+        return;
+
+    // Get the list of all viewports that are part of the current viewport configuration.
+    const auto& viewports = _viewportConfig->viewports();
+
+    // Cleanup: Delete stale viewport windows belonging to viewports that have been removed from the current viewport configuration.
+    std::erase_if(_viewportWindows, [&](const auto& window) {
+        return boost::algorithm::none_of(viewports, [&](Viewport* vp) { return window->viewport() == vp; });
+    });
+
+    // Create new viewport windows for viewports that don't have one yet.
+    for(Viewport* viewport : viewports) {
+        if(!_windowCreationErrorOccurred && !viewportWindow(viewport)) {
+            bool success = _mainWindow.handleExceptions([&] {
+                OORef<WidgetViewportWindow> viewportWindow = createViewportWindow(*viewport, this);
+                if(!viewportWindow || !viewportWindow->widget())
+                    throw Exception(tr("Failed to create viewport window or there is no realtime graphics implementation available. Please check your OVITO installation and the graphics capabilities of your system."));
+                if(_viewportConfig && _viewportConfig->activeViewport() == viewport)
+                    viewportWindow->widget()->setFocus();
+                _viewportWindows.push_back(viewportWindow);
+            });
+            if(!success)
+                _windowCreationErrorOccurred = true;
+        }
+    }
+
+    // Automatically activate a different viewport if the currently active one has been hidden.
+    if(_viewportConfig->maximizedViewport() && !viewportWindow(_viewportConfig->maximizedViewport())) {
+        _mainWindow.handleExceptions([&] {
+            _viewportConfig->setMaximizedViewport(viewports.empty() ? nullptr : viewports.front());
+            _viewportConfig->setActiveViewport(_viewportConfig->maximizedViewport());
+        });
+    }
+    if(_viewportConfig->activeViewport() && !viewportWindow(_viewportConfig->activeViewport())) {
+        _mainWindow.handleExceptions([&] {
+            _viewportConfig->setActiveViewport(viewports.empty() ? nullptr : viewports.front());
+        });
+    }
+}
+
 /******************************************************************************
 * Performs the layout of the viewport windows.
 ******************************************************************************/
 void ViewportsPanel::layoutViewports()
 {
-    _relayoutRequested = false;
+    _layoutRequested = false;
     _splitterRegions.clear();
     _hoveredSplitter = -1;
     _highlightSplitter = false;
@@ -328,20 +344,15 @@ void ViewportsPanel::layoutViewports()
     if(!_viewportConfig)
         return;
 
+    // Create the physical viewport windows for the viewports of the current viewport configuration.
+    createViewportWindows();
+
     // Get the list of all viewports.
     const auto& viewports = _viewportConfig->viewports();
 
-    // Delete stale viewport widgets belonging to removed viewports.
-    for(QObject* childWidget : children()) {
-        OVITO_ASSERT(childWidget->isWidgetType());
-        if(boost::algorithm::none_of(viewports, [&](Viewport* vp) { return viewportWidget(vp) == childWidget; })) {
-            delete childWidget;
-        }
-    }
-
     // Get the viewport that is currently maximized.
     if(Viewport* maximizedViewport = _viewportConfig->maximizedViewport()) {
-        // If there is a maximized viewport, hide all other viewport windows.
+        // If there is a maximized viewport, hide all other viewport windows (only if they are not a free-floating viewport window).
         for(Viewport* viewport : viewports) {
             if(QWidget* widget = viewportWidget(viewport)) {
                 if(widget->parentWidget() == this) {
@@ -359,22 +370,9 @@ void ViewportsPanel::layoutViewports()
         }
     }
     else {
-        // Perform a reculation of the nested layout.
+        // Perform a recalculation of the nested layout.
         layoutViewportsRecursive(_viewportConfig->layoutRootCell(), rect());
     }
-
-    // Automatically activate a different viewport if the currently active one has been hidden.
-    if(_viewportConfig->maximizedViewport() && !_viewportConfig->maximizedViewport()->window()) {
-        _mainWindow.handleExceptions([&] {
-            _viewportConfig->setMaximizedViewport(viewports.empty() ? nullptr : viewports.front());
-            _viewportConfig->setActiveViewport(_viewportConfig->maximizedViewport());
-        });
-    }
-    if(_viewportConfig->activeViewport() && !_viewportConfig->activeViewport()->window()) {
-        _mainWindow.handleExceptions([&] {
-            _viewportConfig->setActiveViewport(viewports.empty() ? nullptr : viewports.front());
-        });
-    }
 }
 
 /******************************************************************************
@@ -473,9 +471,9 @@ void ViewportsPanel::mousePressEvent(QMouseEvent* event)
 void ViewportsPanel::mouseMoveEvent(QMouseEvent* event)
 {
     if(_draggedSplitter != -1) {
-        // Temporarily block the viewportLayoutChanged() signal from the ViewportConfiguration to avoid
+        // Temporarily block the viewportLayoutChanged() signal from the DataSetContainer to avoid
         // an unnecessary relayout of the viewport windows while resetting the undo operation.
-        QSignalBlocker signalBlocker(_viewportConfig);
+        QSignalBlocker signalBlocker(&_mainWindow.datasetContainer());
         _undoTransaction.revert();
         signalBlocker.unblock();
 
@@ -667,53 +665,54 @@ bool ViewportsPanel::onKeyShortcut(QKeyEvent* event)
     if(qobject_cast<QAbstractItemView*>(focusWidget))
         return false;
 
-    // Get the viewport the input pertains to.
-    Viewport* vp = _viewportConfig ? _viewportConfig->activeViewport() : nullptr;
-    if(!vp) return false;
+    // Get the viewport window the input pertains to.
+    ViewportWindow* vpwin = viewportWindow(_viewportConfig ? _viewportConfig->activeViewport() : nullptr);
+    if(!vpwin)
+        return false;
 
     qreal delta = 1.0;
     if(event->key() == Qt::Key_Left) {
         _mainWindow.performTransaction(tr("Move camera"), [&] {
             if(!(event->modifiers() & Qt::ShiftModifier))
-                _mainWindow.viewportInputManager()->orbitMode()->discreteStep(vp->window(), QPointF(-delta, 0));
+                _mainWindow.viewportInputManager()->orbitMode()->discreteStep(vpwin, QPointF(-delta, 0));
             else
-                _mainWindow.viewportInputManager()->panMode()->discreteStep(vp->window(), QPointF(-delta, 0));
+                _mainWindow.viewportInputManager()->panMode()->discreteStep(vpwin, QPointF(-delta, 0));
         });
         return true;
     }
     else if(event->key() == Qt::Key_Right) {
         _mainWindow.performTransaction(tr("Move camera"), [&] {
             if(!(event->modifiers() & Qt::ShiftModifier))
-                _mainWindow.viewportInputManager()->orbitMode()->discreteStep(vp->window(), QPointF(delta, 0));
+                _mainWindow.viewportInputManager()->orbitMode()->discreteStep(vpwin, QPointF(delta, 0));
             else
-                _mainWindow.viewportInputManager()->panMode()->discreteStep(vp->window(), QPointF(delta, 0));
+                _mainWindow.viewportInputManager()->panMode()->discreteStep(vpwin, QPointF(delta, 0));
         });
         return true;
     }
     else if(event->key() == Qt::Key_Up) {
         _mainWindow.performTransaction(tr("Move camera"), [&] {
             if(!(event->modifiers() & Qt::ShiftModifier))
-                _mainWindow.viewportInputManager()->orbitMode()->discreteStep(vp->window(), QPointF(0, -delta));
+                _mainWindow.viewportInputManager()->orbitMode()->discreteStep(vpwin, QPointF(0, -delta));
             else
-                _mainWindow.viewportInputManager()->panMode()->discreteStep(vp->window(), QPointF(0, -delta));
+                _mainWindow.viewportInputManager()->panMode()->discreteStep(vpwin, QPointF(0, -delta));
         });
         return true;
     }
     else if(event->key() == Qt::Key_Down) {
         _mainWindow.performTransaction(tr("Move camera"), [&] {
             if(!(event->modifiers() & Qt::ShiftModifier))
-                _mainWindow.viewportInputManager()->orbitMode()->discreteStep(vp->window(), QPointF(0, delta));
+                _mainWindow.viewportInputManager()->orbitMode()->discreteStep(vpwin, QPointF(0, delta));
             else
-                _mainWindow.viewportInputManager()->panMode()->discreteStep(vp->window(), QPointF(0, delta));
+                _mainWindow.viewportInputManager()->panMode()->discreteStep(vpwin, QPointF(0, delta));
         });
         return true;
     }
     else if(event->matches(QKeySequence::ZoomIn)) {
-        _mainWindow.viewportInputManager()->zoomMode()->zoom(vp, 50, _mainWindow);
+        _mainWindow.viewportInputManager()->zoomMode()->zoom(vpwin->viewport(), 50, _mainWindow);
         return true;
     }
     else if(event->matches(QKeySequence::ZoomOut)) {
-        _mainWindow.viewportInputManager()->zoomMode()->zoom(vp, -50, _mainWindow);
+        _mainWindow.viewportInputManager()->zoomMode()->zoom(vpwin->viewport(), -50, _mainWindow);
         return true;
     }
     return false;
