@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -23,7 +23,6 @@
 #include <ovito/gui/desktop/GUI.h>
 #include <ovito/core/dataset/pipeline/ModificationNode.h>
 #include <ovito/core/dataset/pipeline/Modifier.h>
-#include <ovito/core/dataset/pipeline/ModifierTemplates.h>
 #include <ovito/core/dataset/scene/Pipeline.h>
 #include <ovito/core/dataset/scene/SelectionSet.h>
 #include <ovito/core/viewport/ViewportConfiguration.h>
@@ -33,6 +32,7 @@
 #include <ovito/core/oo/CloneHelper.h>
 #include <ovito/gui/base/actions/ActionManager.h>
 #include <ovito/gui/base/mainwin/PipelineListModel.h>
+#include <ovito/gui/base/mainwin/templates/ModifierTemplates.h>
 #include <ovito/gui/desktop/app/GuiApplication.h>
 #include <ovito/gui/desktop/mainwin/MainWindow.h>
 #include <ovito/gui/desktop/dialogs/ModifierTemplatesPage.h>
@@ -40,12 +40,10 @@
 #include "CommandPanel.h"
 #include "ModifyCommandPage.h"
 
-#include <QtNetwork>
-
 namespace Ovito {
 
 /******************************************************************************
-* Initializes the modify tab.
+* Initializes the command panel tab.
 ******************************************************************************/
 ModifyCommandPage::ModifyCommandPage(MainWindow& mainWindow, QWidget* parent) : QWidget(parent), _mainWindow(mainWindow)
 {
@@ -59,24 +57,20 @@ ModifyCommandPage::ModifyCommandPage(MainWindow& mainWindow, QWidget* parent) :
     public:
         using QComboBox::QComboBox;
         virtual void showPopup() override {
-            static_cast<ModifierListModel*>(model())->updateActionState();
+            static_cast<AvailableModifiersModel*>(model())->updateActionState();
             QComboBox::showPopup();
         }
     };
     _modifierSelector = new ModifierListBox(this);
     layout->addWidget(_modifierSelector, 1, 0, 1, 1);
     _modifierSelector->setSizeAdjustPolicy(QComboBox::AdjustToContents);
-    _modifierSelector->setModel(new ModifierListModel(this, mainWindow, _pipelineListModel));
+    _modifierSelector->setModel(new AvailableModifiersModel(this, mainWindow, _pipelineListModel));
     _modifierSelector->setMaxVisibleItems(0xFFFF);
-    connect(_modifierSelector, qOverload<int>(&QComboBox::activated), this, [this](int index) {
-        QComboBox* selector = static_cast<QComboBox*>(sender());
-        static_cast<ModifierListModel*>(selector->model())->insertModifierByIndex(index);
-        selector->setCurrentIndex(0);
-    });
+    connect(_modifierSelector, qOverload<int>(&QComboBox::activated), this, &ModifyCommandPage::onInsertNewModifier);
 
     class PipelineListView : public QListView {
     public:
-        PipelineListView(QWidget* parent) : QListView(parent) {}
+        using QListView::QListView;
         virtual QSize sizeHint() const override { return QSize(256, 260); }
     protected:
         virtual bool edit(const QModelIndex& index, QAbstractItemView::EditTrigger trigger, QEvent* event) override {
@@ -171,7 +165,7 @@ ModifyCommandPage::ModifyCommandPage(MainWindow& mainWindow, QWidget* parent) :
     editToolbar->addSeparator();
     editToolbar->addAction(actionManager->getAction(ACTION_PIPELINE_TOGGLE_MODIFIER_GROUP));
 
-    QAction* manageModifierTemplatesAction = actionManager->createCommandAction(ACTION_MODIFIER_MANAGE_TEMPLATES, tr("Manage Modifier Templates..."), "modify_modifier_save_preset", tr("Open the dialog that lets you manage the saved modifier templates."));
+    QAction* manageModifierTemplatesAction = actionManager->createCommandAction(ACTION_MODIFIER_MANAGE_MODIFIER_TEMPLATES, tr("Manage Modifier Templates..."), "modify_modifier_save_preset", tr("Open the dialog that lets you manage the saved modifier templates."));
     connect(manageModifierTemplatesAction, &QAction::triggered, [&mainWindow]() {
         ApplicationSettingsDialog dlg(mainWindow, &ModifierTemplatesPage::OOClass());
         dlg.exec();
@@ -241,6 +235,23 @@ void ModifyCommandPage::saveLayout()
     settings.setValue("splitter", _splitter->saveState());
 }
 
+/******************************************************************************
+* Is called when the user has selected a modifier from drop-down list of available modifiers.
+******************************************************************************/
+void ModifyCommandPage::onInsertNewModifier(int index)
+{
+    if(index == availableModifiersModel()->getMoreExtensionsItemIndex()) {
+        if(QAction* action = _mainWindow.actionManager()->getAction(ACTION_SCRIPTING_EXTENSIONS_GALLERY_MODIFIERS))
+            action->trigger();
+        else
+            QDesktopServices::openUrl(QStringLiteral("https://www.ovito.org/extensions/"));
+    }
+    else {
+        availableModifiersModel()->insertModifierByIndex(index);
+    }
+    _modifierSelector->setCurrentIndex(0);
+}
+
 /******************************************************************************
 * Is called when a new modification list item has been selected, or if the currently
 * selected item has changed.
@@ -253,10 +264,6 @@ void ModifyCommandPage::onSelectedItemChanged()
 
     if(selectedObject != _propertiesPanel->editObject()) {
         _propertiesPanel->setEditObject(selectedObject);
-
-        // Request a viewport update whenever a new item in the pipeline editor is selected,
-        // because the currently selected modifier may render gizmos in the viewports.
-        _mainWindow.updateViewports();
     }
 
     // Whenever no object is selected, show information about the program.
@@ -323,63 +330,9 @@ void ModifyCommandPage::createAboutPanel()
     newsPage = QByteArray((const char *)res.data(), (int)res.size());
 #endif
 
+    // Display cached newsPage
     _aboutRollout = _propertiesPanel->addRollout(rollout, Application::applicationName());
     showProgramNotice(QString::fromUtf8(newsPage.constData()));
-
-#if !defined(OVITO_BUILD_APPSTORE_VERSION)
-    if(settings.value("updates/check_for_updates", true).toBool()) {
-        QString operatingSystemString;
-#if defined(Q_OS_MACOS)
-        operatingSystemString = QStringLiteral("macosx");
-#elif defined(Q_OS_WIN) || defined(Q_OS_CYGWIN)
-        operatingSystemString = QStringLiteral("win");
-#elif defined(Q_OS_LINUX) || defined(Q_OS_UNIX)
-        operatingSystemString = QStringLiteral("linux");
-#else
-        operatingSystemString = QStringLiteral("other");
-#endif
-
-        QString programEdition;
-#if defined(OVITO_BUILD_BASIC)
-        programEdition = QStringLiteral("basic/");
-#elif defined(OVITO_BUILD_PRO)
-        programEdition = QStringLiteral("pro/");
-#endif
-
-        // Fetch newest web page from web server.
-        QString urlString = QString("https://www.ovito.org/appnews/v%1.%2.%3/%4?ovito=000000000000000000&OS=%5%6")
-                .arg(Application::applicationVersionMajor())
-                .arg(Application::applicationVersionMinor())
-                .arg(Application::applicationVersionRevision())
-                .arg(programEdition)
-                .arg(operatingSystemString)
-                .arg(QT_POINTER_SIZE*8);
-        QNetworkAccessManager* networkAccessManager = Application::instance()->networkAccessManager();
-        QNetworkReply* networkReply = networkAccessManager->get(QNetworkRequest(QUrl(urlString)));
-        connect(networkReply, &QNetworkReply::finished, this, &ModifyCommandPage::onWebRequestFinished);
-    }
-#endif
-}
-
-/******************************************************************************
-* Is called by the system when fetching the news web page from the server is
-* completed.
-******************************************************************************/
-void ModifyCommandPage::onWebRequestFinished()
-{
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(QObject::sender());
-    if(reply->error() == QNetworkReply::NoError) {
-        QByteArray page = reply->readAll();
-        reply->close();
-        if(page.startsWith("<html><!--OVITO-->")) {
-
-            showProgramNotice(QString::fromUtf8(page.constData()));
-
-            QSettings settings;
-            settings.setValue("news/cached_webpage", page);
-        }
-    }
-    reply->deleteLater();
 }
 
 /******************************************************************************
@@ -390,10 +343,13 @@ void ModifyCommandPage::showProgramNotice(const QString& htmlPage)
     QString finalText = htmlPage;
 
 #if defined(OVITO_DEVELOPMENT_BUILD_DATE)
-    QString previewNotice = tr("<h4>Preview version notice</h4><p style=\"background-color: rgb(230,180,180); color: black;\">You are using an early development build of %1, which was created on %2.</p> "
-            "<p style=\"background-color: rgb(230,180,180); color: black;\">Remember to install the final release of %1 as soon as it becomes available on our website <a href=\"https://www.ovito.org/\">www.ovito.org</a>.</p>")
-        .arg(Application::applicationName())
-        .arg(QStringLiteral(OVITO_DEVELOPMENT_BUILD_DATE));
+    const QString previewNotice =
+        tr("<h4>Preview version notice</h4><p style=\"background-color: rgb(230,180,180); color: black;\">You are using an early "
+           "development build of %1, which was created on %2.</p> "
+           "<p style=\"background-color: rgb(230,180,180); color: black;\">Remember to install the final release of %1 as soon as it "
+           "becomes available on our website <a href=\"https://www.ovito.org/\">www.ovito.org</a>.</p>")
+            .arg(Application::applicationName())
+            .arg(QStringLiteral(OVITO_DEVELOPMENT_BUILD_DATE));
     finalText.replace(QStringLiteral("<p>&nbsp;</p>"), previewNotice);
 #endif
 
