@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -23,6 +23,7 @@
 #include <ovito/gui/desktop/GUI.h>
 #include <ovito/gui/desktop/mainwin/MainWindow.h>
 #include <ovito/core/app/PluginManager.h>
+#include <ovito/core/dataset/DataSetContainer.h>
 #include "DataInspectorPanel.h"
 #include "DataInspectionApplet.h"
 
@@ -39,7 +40,7 @@ DataInspectorPanel::DataInspectorPanel(MainWindow& mainWindow) :
     // Create data inspection applets.
     for(OvitoClassPtr clazz : PluginManager::instance().listClasses(DataInspectionApplet::OOClass())) {
         OORef<DataInspectionApplet> applet = static_object_cast<DataInspectionApplet>(clazz->createInstance());
-        applet->setParent(this);
+        applet->setInspectorPanel(this);
         _applets.push_back(std::move(applet));
     }
     // Give applets a fixed ordering.
@@ -82,7 +83,7 @@ DataInspectorPanel::DataInspectorPanel(MainWindow& mainWindow) :
     _appletContainer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Ignored);
     _appletContainer->setMinimumWidth(10);
     _appletContainer->resize(0,0);
-    QLabel* label = new QLabel(tr("No data available or no object selected."));
+    QLabel* label = new QLabel(tr("This panel will list the output of the currently selected pipeline."));
     label->setAlignment(Qt::AlignCenter);
     _appletContainer->addWidget(label);
     for(DataInspectionApplet* applet : _applets)
@@ -98,6 +99,9 @@ DataInspectorPanel::DataInspectorPanel(MainWindow& mainWindow) :
     connect(_scenePreparation.get(), &ScenePreparation::scenePreparationStarted, this, &DataInspectorPanel::onScenePreparationStarted);
     connect(_scenePreparation.get(), &ScenePreparation::scenePreparationFinished, this, &DataInspectorPanel::onScenePreparationFinished);
 
+    // Keep requesting output from the scene pipeline(s) indefinitely.
+    _scenePreparation->setAutoRestart(true);
+
     updateTabsList();
 }
 
@@ -164,17 +168,22 @@ void DataInspectorPanel::open()
 ******************************************************************************/
 void DataInspectorPanel::onSceneSelectionChanged(SelectionSet* selection)
 {
-    // Find the first selected Pipeline and make it the active node:
-    Pipeline* selectedNode = nullptr;
+    // Find the first selected Pipeline:
+    SceneNode* selectedSceneNode = nullptr;
+    Pipeline* selectedPipeline = nullptr;
     if(selection) {
         for(SceneNode* node : selection->nodes()) {
-            selectedNode = dynamic_object_cast<Pipeline>(node);
-            if(selectedNode) break;
+            if(node->pipeline()) {
+                selectedSceneNode = node;
+                selectedPipeline = node->pipeline();
+                break;
+            }
         }
     }
-    if(selectedNode != _selectedPipeline) {
-        _selectedPipeline = selectedNode;
-        Q_EMIT selectedPipelineChanged(selectedNode);
+    _selectedSceneNode = selectedSceneNode;
+    if(selectedPipeline != _selectedPipeline) {
+        _selectedPipeline = selectedPipeline;
+        Q_EMIT selectedPipelineChanged(selectedPipeline);
         updateInspector();
     }
 }
@@ -241,7 +250,7 @@ void DataInspectorPanel::resizeEvent(QResizeEvent* event)
 }
 
 /******************************************************************************
-* Updates the contents displayed in the data inpector.
+* Updates the contents displayed in the data inspector.
 ******************************************************************************/
 void DataInspectorPanel::updateInspector()
 {
@@ -251,7 +260,7 @@ void DataInspectorPanel::updateInspector()
     // Update the list of visible tabs.
     updateTabsList();
 
-    // Update content displayed by the current inpector tab page.
+    // Update content displayed by the current inspector tab page.
     if(_inspectorActive) {
         if(_activeAppletIndex >= 0 && _activeAppletIndex < _applets.size()) {
             _applets[_activeAppletIndex]->updateDisplay();
@@ -320,17 +329,13 @@ void DataInspectorPanel::updateTabsList()
 ******************************************************************************/
 bool DataInspectorPanel::updatePipelineOutput()
 {
+    _pipelineOutput.reset();
     if(selectedPipeline()) {
         if(AnimationSettings* anim = mainWindow().datasetContainer().activeAnimationSettings()) {
-            if(mainWindow().handleExceptions([&] {
-                _pipelineOutput = selectedPipeline()->evaluatePipelineSynchronous(anim->currentTime(), true);
-            })) {
-                return true;
-            }
+            _pipelineOutput = selectedPipeline()->getCachedPipelineOutput(anim->currentTime(), false);
         }
     }
-    _pipelineOutput.reset();
-    return false;
+    return (bool)_pipelineOutput;
 }
 
 /******************************************************************************
@@ -366,15 +371,8 @@ void DataInspectorPanel::onCurrentPageChanged(int index)
 /******************************************************************************
 * Selects a specific data object in the data inspector.
 ******************************************************************************/
-bool DataInspectorPanel::selectDataObject(PipelineNode* createdByNode, const QString& objectIdentifierHint, const QVariant& modeHint)
+bool DataInspectorPanel::selectDataObject(const PipelineNode* createdByNode, const QString& objectIdentifierHint, const QVariant& modeHint)
 {
-    // Obtain the output of the currently selected pipeline.
-    if(!updatePipelineOutput())
-        return false;
-
-    // Update the list of displayed tabs.
-    updateTabsList();
-
     for(int appletIndex = 0; appletIndex < _applets.size(); appletIndex++) {
         if(_appletsToTabs[appletIndex] == -1) continue;
         DataInspectionApplet* applet = _applets[appletIndex];
