@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -27,10 +27,11 @@
 #include <ovito/core/dataset/pipeline/Modifier.h>
 #include <ovito/core/dataset/scene/Pipeline.h>
 #include <ovito/core/dataset/data/DataVis.h>
+#include <ovito/core/viewport/overlays/ViewportOverlay.h>
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(PropertiesEditor);
+IMPLEMENT_ABSTRACT_OVITO_CLASS(PropertiesEditor);
 DEFINE_REFERENCE_FIELD(PropertiesEditor, editObject);
 
 /******************************************************************************
@@ -87,11 +88,15 @@ void PropertiesEditor::initialize(PropertiesPanel* container, const RolloutInser
 }
 
 /******************************************************************************
-* The virtual destructor.
+* This method is called when the reference counter of this OvitoObject
+* has reached zero.
 ******************************************************************************/
-PropertiesEditor::~PropertiesEditor()
+void PropertiesEditor::aboutToBeDeleted()
 {
-    clearAllReferences();
+    // Detach from edited object. This is necessary to send the editingStopped signal to the edited object.
+    setEditObject(nullptr);
+
+    RefMaker::aboutToBeDeleted();
 }
 
 /******************************************************************************
@@ -103,6 +108,8 @@ void PropertiesEditor::setEditObject(RefTarget* newObject)
             "PropertiesEditor::setEditObject()", "This properties editor was not made for this object class.");
 
     _editObject.set(this, PROPERTY_FIELD(editObject), newObject);
+
+    OVITO_ASSERT(!newObject || newObject->isBeingEdited());
 }
 
 /******************************************************************************
@@ -168,10 +175,18 @@ QWidget* PropertiesEditor::parentWindow() const
 void PropertiesEditor::disableRollout(QWidget* rolloutWidget, const QString& noticeText)
 {
     rolloutWidget->setEnabled(false);
+    showNotice(rolloutWidget, noticeText);
+}
+
+/******************************************************************************
+* Inserts a notice text at the top of the given rollout.
+******************************************************************************/
+void PropertiesEditor::showNotice(QWidget* rolloutWidget, const QString& noticeText)
+{
     if(Rollout* rollout = container()->findRolloutFromWidget(rolloutWidget)) {
         rollout->setNotice(noticeText);
         // Force a re-layout of the rollouts.
-        QTimer::singleShot(100, container(), &RolloutContainer::updateRollouts);
+        QTimer::singleShot(60, container(), &RolloutContainer::updateRollouts);
     }
 }
 
@@ -183,6 +198,11 @@ bool PropertiesEditor::referenceEvent(RefTarget* source, const ReferenceEvent& e
     if(source == editObject()) {
         if(event.type() == ReferenceEvent::TargetChanged) {
             Q_EMIT contentsChanged(source);
+
+            // If it's a ViewportOverlay being edited and its selected pipeline has changed, emit a signal.
+            if(static_cast<const TargetChangedEvent&>(event).field() == PROPERTY_FIELD(ViewportOverlay::pipeline)) {
+                emitPipelineInputChangedSignal(this);
+            }
         }
         else if(event.type() == ReferenceEvent::PipelineCacheUpdated) {
             emitPipelineOutputChangedSignal(this);
@@ -200,25 +220,22 @@ bool PropertiesEditor::referenceEvent(RefTarget* source, const ReferenceEvent& e
 void PropertiesEditor::referenceReplaced(const PropertyFieldDescriptor* field, RefTarget* oldTarget, RefTarget* newTarget, int listIndex)
 {
     if(field == PROPERTY_FIELD(editObject)) {
-        if(oldTarget) oldTarget->unsetObjectEditingFlag();
-        if(newTarget) newTarget->setObjectEditingFlag();
+        if(oldTarget) {
+            oldTarget->editingStopped(mainWindow());
+        }
         Q_EMIT contentsReplaced(editObject());
         Q_EMIT contentsChanged(editObject());
-        emitPipelineInputChangedSignal(this);
-        emitPipelineOutputChangedSignal(this);
+        if(newTarget) {
+            newTarget->editingStarted(mainWindow());
+        }
+        if(!isBeingDeleted()) {
+            emitPipelineInputChangedSignal(this, mainWindow());
+            emitPipelineOutputChangedSignal(this, mainWindow());
+        }
     }
     RefMaker::referenceReplaced(field, oldTarget, newTarget, listIndex);
 }
 
-/******************************************************************************
-* Changes the value of a non-animatable property field of the object being edited.
-******************************************************************************/
-void PropertiesEditor::changePropertyFieldValue(const PropertyFieldDescriptor* field, const QVariant& newValue)
-{
-    if(editObject())
-        editObject()->setPropertyFieldValue(field, newValue);
-}
-
 /******************************************************************************
 * Returns the current input data from the upstream pipeline.
 ******************************************************************************/
@@ -226,29 +243,22 @@ PipelineFlowState PropertiesEditor::getPipelineInput() const
 {
     // When editing a modifier, request pipeline input state from the modifier node being edited in the parent editor.
     if(ModificationNode* modNode = dynamic_object_cast<ModificationNode>(editObject())) {
-        if(std::optional<AnimationTime> time = currentAnimationTime()) {
-            PipelineFlowState state;
-            handleExceptions([&] {
-                state = modNode->evaluateInputSynchronous(PipelineEvaluationRequest(*time));
-            });
-            return state;
+        return modNode->getCachedPipelineNodeInput(currentAnimationTime());
+    }
+
+    // When editing a DataVis element, request pipeline state from the selected pipeline.
+    if(DataVis* vis = dynamic_object_cast<DataVis>(editObject())) {
+        if(Pipeline* pipeline = selectedPipeline()) {
+            OVITO_ASSERT(vis->pipelines(true).contains(pipeline));
+            OVITO_ASSERT(pipeline->visElements().contains(vis));
+            return pipeline->getCachedPipelineOutput(currentAnimationTime());
         }
     }
 
-    // When editing a DataVis element, request pipeline state from the selected scene node.
-    if(DataVis* vis = dynamic_object_cast<DataVis>(editObject())) {
-        if(SelectionSet* selection = mainWindow().datasetContainer().activeSelectionSet()) {
-            if(Pipeline* pipeline = dynamic_object_cast<Pipeline>(selection->firstNode())) {
-                if(std::optional<AnimationTime> time = currentAnimationTime()) {
-                    OVITO_ASSERT(vis->pipelines(true).contains(pipeline));
-                    OVITO_ASSERT(pipeline->visElements().contains(vis));
-                    PipelineFlowState state;
-                    handleExceptions([&] {
-                        state = pipeline->evaluatePipelineSynchronous(*time, false);
-                    });
-                    return state;
-                }
-            }
+    // When editing a ViewportOverlay, request state from the associated pipeline.
+    if(ViewportOverlay* overlay = dynamic_object_cast<ViewportOverlay>(editObject())) {
+        if(overlay->pipeline()) {
+            return overlay->pipeline()->getCachedPipelineOutput(currentAnimationTime());
         }
     }
 
@@ -272,24 +282,25 @@ std::vector<PipelineFlowState> PropertiesEditor::getPipelineInputs() const
 
     // When editing a modifier, get the pipeline state from the modifier applications.
     if(Modifier* modifier = dynamic_object_cast<Modifier>(editObject())) {
-        if(std::optional<AnimationTime> time = currentAnimationTime()) {
-            PipelineEvaluationRequest request(*time);
-            for(ModificationNode* node : modifier->nodes()) {
-                handleExceptions([&] {
-                    inputStates.push_back(node->evaluateInputSynchronous(request));
-                });
-            }
+        for(ModificationNode* node : modifier->nodes()) {
+            if(PipelineFlowState state = node->getCachedPipelineNodeInput(currentAnimationTime()))
+                inputStates.push_back(std::move(state));
         }
     }
 
     // When editing a DataVis element, get the pipeline state from the scene nodes.
     if(DataVis* vis = dynamic_object_cast<DataVis>(editObject())) {
-        if(std::optional<AnimationTime> time = currentAnimationTime()) {
-            for(Pipeline* pipeline : vis->pipelines(true)) {
-                handleExceptions([&] {
-                    inputStates.push_back(pipeline->evaluatePipelineSynchronous(*time, false));
-                });
-            }
+        for(Pipeline* pipeline : vis->pipelines(true)) {
+            if(PipelineFlowState state = pipeline->getCachedPipelineOutput(currentAnimationTime()))
+                inputStates.push_back(std::move(state));
+        }
+    }
+
+    // When editing a ViewportOverlay, request state from the associated pipeline.
+    if(ViewportOverlay* overlay = dynamic_object_cast<ViewportOverlay>(editObject())) {
+        if(overlay->pipeline()) {
+            if(PipelineFlowState state = overlay->pipeline()->getCachedPipelineOutput(currentAnimationTime()))
+                inputStates.push_back(std::move(state));
         }
     }
 
@@ -308,11 +319,7 @@ PipelineFlowState PropertiesEditor::getPipelineOutput() const
     }
     else if(ModificationNode* node = dynamic_object_cast<ModificationNode>(editObject())) {
         // Request pipeline output state from the modifier application.
-        PipelineFlowState state;
-        handleExceptions([&] {
-            state = node->evaluateSynchronous(PipelineEvaluationRequest(currentAnimationTime()));
-        });
-        return state;
+        return node->getCachedPipelineNodeOutput(currentAnimationTime());
     }
     return {};
 }
@@ -352,17 +359,13 @@ ConstDataObjectRefPath PropertiesEditor::getVisDataObjectPath() const
     if(DataVis* vis = dynamic_object_cast<DataVis>(editObject())) {
         // We'll now try to find the DataObject this DataVis element is associated with.
         // Let's start looking in the output data collection of the currently selected pipeline scene node.
-        if(SelectionSet* selection = mainWindow().datasetContainer().activeSelectionSet()) {
-            if(Pipeline* pipeline = dynamic_object_cast<Pipeline>(selection->firstNode())) {
-                std::vector<ConstDataObjectPath> dataObjectPaths;
-                handleExceptions([&] {
-                    const PipelineFlowState& state = pipeline->evaluatePipelineSynchronous(currentAnimationTime(), false);
-                    dataObjectPaths = pipeline->getDataObjectsForVisElement(state, vis);
-                });
-                if(!dataObjectPaths.empty()) {
-                    // Return just the first path from the list.
-                    return ConstDataObjectRefPath(dataObjectPaths.front().begin(), dataObjectPaths.front().end());
-                }
+        if(Pipeline* pipeline = selectedPipeline()) {
+            std::vector<ConstDataObjectPath> dataObjectPaths;
+            const PipelineFlowState& state = pipeline->getCachedPipelineOutput(currentAnimationTime());
+            dataObjectPaths = pipeline->getDataObjectsForVisElement(state, vis);
+            if(!dataObjectPaths.empty()) {
+                // Return just the first path from the list.
+                return ConstDataObjectRefPath(dataObjectPaths.front().begin(), dataObjectPaths.front().end());
             }
         }
         return {};
