@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -25,18 +25,23 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(RefTargetListParameterUI);
+IMPLEMENT_ABSTRACT_OVITO_CLASS(RefTargetListParameterUI);
 DEFINE_VECTOR_REFERENCE_FIELD(RefTargetListParameterUI, targets);
 
 /******************************************************************************
-* The constructor.
+* Constructor.
 ******************************************************************************/
-RefTargetListParameterUI::RefTargetListParameterUI(PropertiesEditor* parentEditor, const PropertyFieldDescriptor* refField, const RolloutInsertionParameters& rolloutParams, OvitoClassPtr defaultEditorClass)
-    : ParameterUI(parentEditor), _refField(refField), _rolloutParams(rolloutParams), _defaultEditorClass(defaultEditorClass)
+void RefTargetListParameterUI::initializeObject(PropertiesEditor* parentEditor, const PropertyFieldDescriptor* refField, const RolloutInsertionParameters& rolloutParams, OvitoClassPtr defaultEditorClass)
 {
     OVITO_ASSERT(refField);
     OVITO_ASSERT_MSG(refField->isVector(), "RefTargetListParameterUI constructor", "The reference field bound to this parameter UI must be a vector reference field.");
 
+    ParameterUI::initializeObject(parentEditor);
+
+    _refField = refField;
+    _rolloutParams = rolloutParams;
+    _defaultEditorClass = defaultEditorClass;
+
     _model = new ListViewModel(this);
 
     if(_defaultEditorClass)
@@ -49,7 +54,6 @@ RefTargetListParameterUI::RefTargetListParameterUI(PropertiesEditor* parentEdito
 RefTargetListParameterUI::~RefTargetListParameterUI()
 {
     _subEditor = nullptr;
-    clearAllReferences();
 
     // Release GUI controls.
     delete _viewWidget;
@@ -285,7 +289,6 @@ bool RefTargetListParameterUI::referenceEvent(RefTarget* source, const Reference
         }
         else if(event.type() == ReferenceEvent::ReferenceRemoved) {
             const ReferenceFieldEvent& refevent = static_cast<const ReferenceFieldEvent&>(event);
-            OVITO_ASSERT(false);
             if(refevent.field() == referenceField()) {
                 int rowIndex = _targetToRow[refevent.index()];
                 if(refevent.oldTarget())
