@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -33,11 +33,16 @@
 #include <ovito/core/dataset/scene/SelectionSet.h>
 #include <ovito/core/viewport/ViewportConfiguration.h>
 #include <ovito/core/viewport/Viewport.h>
-#include <ovito/core/viewport/ViewportWindowInterface.h>
+#include <ovito/core/viewport/ViewportWindow.h>
 #include "XFormModes.h"
 
 namespace Ovito {
 
+IMPLEMENT_ABSTRACT_OVITO_CLASS(XFormMode);
+DEFINE_REFERENCE_FIELD(XFormMode, selectedNode);
+IMPLEMENT_ABSTRACT_OVITO_CLASS(MoveMode);
+IMPLEMENT_ABSTRACT_OVITO_CLASS(RotateMode);
+
 /******************************************************************************
 * This is called by the system after the input handler has
 * become the active handler.
@@ -59,14 +64,14 @@ void XFormMode::activated(bool temporaryActivation)
 ******************************************************************************/
 void XFormMode::deactivated(bool temporary)
 {
-    if(viewport()) {
+    if(viewportWindow()) {
         // Restore old state if change has not been committed.
         _undoTransaction.cancel();
-        _viewport = nullptr;
+        _viewportWindow = nullptr;
     }
     disconnect(&inputManager()->datasetContainer(), &DataSetContainer::selectionChangeComplete, this, &XFormMode::onSelectionChangeComplete);
     disconnect(&inputManager()->datasetContainer(), &DataSetContainer::currentFrameChanged, this, &XFormMode::onCurrentFrameChanged);
-    _selectedNode.setTarget(nullptr);
+    setSelectedNode(nullptr);
     onSelectionChangeComplete(nullptr);
     ViewportInputMode::deactivated(temporary);
 }
@@ -76,12 +81,12 @@ void XFormMode::deactivated(bool temporary)
 ******************************************************************************/
 void XFormMode::onSelectionChangeComplete(SelectionSet* selection)
 {
-    MainWindow* mainWindow = dynamic_cast<MainWindow*>(&inputManager()->userInterface());
+    MainWindow* mainWindow = dynamic_object_cast<MainWindow>(&inputManager()->userInterface());
     CoordinateDisplayWidget* coordDisplay = mainWindow ? mainWindow->coordinateDisplay() : nullptr;
 
     if(selection) {
         if(selection->nodes().size() == 1) {
-            _selectedNode.setTarget(selection->nodes().front());
+            setSelectedNode(selection->nodes().front());
             if(coordDisplay) {
                 updateCoordinateDisplay(coordDisplay);
                 coordDisplay->activate(undoDisplayName());
@@ -91,7 +96,7 @@ void XFormMode::onSelectionChangeComplete(SelectionSet* selection)
             return;
         }
     }
-    _selectedNode.setTarget(nullptr);
+    setSelectedNode(nullptr);
     if(coordDisplay) {
         disconnect(coordDisplay, &CoordinateDisplayWidget::valueEntered, this, &XFormMode::onCoordinateValueEntered);
         disconnect(coordDisplay, &CoordinateDisplayWidget::animatePressed, this, &XFormMode::onAnimateTransformationButton);
@@ -102,12 +107,13 @@ void XFormMode::onSelectionChangeComplete(SelectionSet* selection)
 /******************************************************************************
 * Is called when the selected scene node generates a notification event.
 ******************************************************************************/
-void XFormMode::onSceneNodeEvent(RefTarget* source, const ReferenceEvent& event)
+bool XFormMode::referenceEvent(RefTarget* source, const ReferenceEvent& event)
 {
-    if(event.type() == ReferenceEvent::TransformationChanged) {
-        if(MainWindow* mainWindow = dynamic_cast<MainWindow*>(&inputManager()->userInterface()))
+    if(source == selectedNode() && event.type() == SceneNode::TransformationChanged) {
+        if(MainWindow* mainWindow = dynamic_object_cast<MainWindow>(&inputManager()->userInterface()))
             updateCoordinateDisplay(mainWindow->coordinateDisplay());
     }
+    return ViewportInputMode::referenceEvent(source, event);
 }
 
 /******************************************************************************
@@ -115,26 +121,26 @@ void XFormMode::onSceneNodeEvent(RefTarget* source, const ReferenceEvent& event)
 ******************************************************************************/
 void XFormMode::onCurrentFrameChanged(int frame)
 {
-    if(MainWindow* mainWindow = dynamic_cast<MainWindow*>(&inputManager()->userInterface()))
+    if(MainWindow* mainWindow = dynamic_object_cast<MainWindow>(&inputManager()->userInterface()))
         updateCoordinateDisplay(mainWindow->coordinateDisplay());
 }
 
 /******************************************************************************
 * Handles the mouse down event for the given viewport.
 ******************************************************************************/
-void XFormMode::mousePressEvent(ViewportWindowInterface* vpwin, QMouseEvent* event)
+void XFormMode::mousePressEvent(ViewportWindow* vpwin, QMouseEvent* event)
 {
     if(event->button() == Qt::LeftButton) {
-        if(viewport() == nullptr) {
+        if(viewportWindow() == nullptr) {
 
             // Select object under mouse cursor.
-            ViewportPickResult pickResult = vpwin->pick(getMousePosition(event));
-            if(pickResult.isValid()) {
-                _viewport = vpwin->viewport();
+            std::optional<ViewportWindow::PickResult> pickResult = vpwin->pick(getMousePosition(event));
+            if(pickResult && vpwin->viewport()->scene()) {
+                _viewportWindow = vpwin;
                 _startPoint = getMousePosition(event);
                 _undoTransaction.begin(inputManager()->userInterface(), undoDisplayName());
                 inputManager()->userInterface().performActions(_undoTransaction, [&] {
-                    viewport()->scene()->selection()->setNode(pickResult.pipeline());
+                    viewportWindow()->viewport()->scene()->selection()->setNode(pickResult->sceneNode());
                 });
                 _undoSelectionOperation = _undoTransaction.snapshot();
                 startXForm();
@@ -143,10 +149,10 @@ void XFormMode::mousePressEvent(ViewportWindowInterface* vpwin, QMouseEvent* eve
         return;
     }
     else if(event->button() == Qt::RightButton) {
-        if(viewport()) {
+        if(viewportWindow()) {
             // Restore old state when aborting the operation.
             _undoTransaction.cancel();
-            _viewport = nullptr;
+            _viewportWindow = nullptr;
             return;
         }
     }
@@ -156,12 +162,12 @@ void XFormMode::mousePressEvent(ViewportWindowInterface* vpwin, QMouseEvent* eve
 /******************************************************************************
 * Handles the mouse up event for the given viewport.
 ******************************************************************************/
-void XFormMode::mouseReleaseEvent(ViewportWindowInterface* vpwin, QMouseEvent* event)
+void XFormMode::mouseReleaseEvent(ViewportWindow* vpwin, QMouseEvent* event)
 {
-    if(viewport()) {
+    if(viewportWindow()) {
         // Commit change.
         _undoTransaction.commit();
-        _viewport = nullptr;
+        _viewportWindow = nullptr;
     }
     ViewportInputMode::mouseReleaseEvent(vpwin, event);
 }
@@ -169,27 +175,24 @@ void XFormMode::mouseReleaseEvent(ViewportWindowInterface* vpwin, QMouseEvent* e
 /******************************************************************************
 * Handles the mouse move event for the given viewport.
 ******************************************************************************/
-void XFormMode::mouseMoveEvent(ViewportWindowInterface* vpwin, QMouseEvent* event)
+void XFormMode::mouseMoveEvent(ViewportWindow* vpwin, QMouseEvent* event)
 {
-    if(viewport() == vpwin->viewport()) {
+    if(viewportWindow() == vpwin) {
         // Take the current mouse cursor position to make the input mode
         // look more responsive. The cursor position recorded when the mouse event was
         // generates may be too old.
         _currentPoint = vpwin->getCurrentMousePos();
 
         // Revert the previous x-form operation.
-        _undoTransaction.revertTo(_undoSelectionOperation);
-
-        inputManager()->userInterface().performActions(_undoTransaction, [&] {
-            doXForm();
-        });
-
-        // Force immediate viewport repaints.
-        inputManager()->userInterface().processViewportUpdateRequests();
+        if(_undoTransaction.revertTo(_undoSelectionOperation)) {
+            inputManager()->userInterface().performActions(_undoTransaction, [&] {
+                doXForm();
+            });
+        }
     }
     else {
         // Change mouse cursor while hovering over an object.
-        setCursor(vpwin->pick(getMousePosition(event)).isValid() ? _xformCursor : QCursor());
+        setCursor(vpwin->pick(getMousePosition(event)) ? _xformCursor : QCursor{});
     }
     ViewportInputMode::mouseMoveEvent(vpwin, event);
 }
@@ -197,12 +200,12 @@ void XFormMode::mouseMoveEvent(ViewportWindowInterface* vpwin, QMouseEvent* even
 /******************************************************************************
 * Is called when a viewport looses the input focus.
 ******************************************************************************/
-void XFormMode::focusOutEvent(ViewportWindowInterface* vpwin, QFocusEvent* event)
+void XFormMode::focusOutEvent(ViewportWindow* vpwin, QFocusEvent* event)
 {
-    if(viewport()) {
+    if(viewportWindow()) {
         // Restore old state if change has not been committed.
         _undoTransaction.cancel();
-        _viewport = nullptr;
+        _viewportWindow = nullptr;
     }
 }
 
@@ -211,18 +214,17 @@ void XFormMode::focusOutEvent(ViewportWindowInterface* vpwin, QFocusEvent* event
 ******************************************************************************/
 Point3 XFormMode::transformationCenter()
 {
-    OVITO_ASSERT(viewport());
-    OVITO_ASSERT(viewport()->scene());
-    if(!viewport() || !viewport()->scene())
+    OVITO_ASSERT(viewportWindow());
+    OVITO_ASSERT(viewportWindow()->viewport()->scene());
+    if(!viewportWindow() || !viewportWindow()->viewport()->scene())
         return Point3::Origin();
 
     Point3 center = Point3::Origin();
-    SelectionSet* selection = viewport()->scene()->selection();
+    SelectionSet* selection = viewportWindow()->viewport()->scene()->selection();
     if(selection && !selection->nodes().empty()) {
-        TimeInterval interval;
-        AnimationTime time = viewport()->scene()->animationSettings()->currentTime();
+        AnimationTime time = viewportWindow()->viewport()->currentTime();
         for(SceneNode* node : selection->nodes()) {
-            const AffineTransformation& nodeTM = node->getWorldTransform(time, interval);
+            const AffineTransformation& nodeTM = node->getWorldTransform(time);
             center += nodeTM.translation();
         }
         center /= (FloatType)selection->nodes().size();
@@ -235,7 +237,7 @@ Point3 XFormMode::transformationCenter()
 *******************************************************************************/
 AffineTransformation XFormMode::transformationSystem()
 {
-    return viewport()->gridMatrix();
+    return viewportWindow()->viewport()->gridMatrix();
 }
 
 /******************************************************************************
@@ -245,7 +247,7 @@ void MoveMode::startXForm()
 {
     _translationSystem = transformationSystem();
     _initialPoint = Point3::Origin();
-    viewport()->snapPoint(_startPoint, _initialPoint, _translationSystem);
+    viewportWindow()->snapPoint(_startPoint, _initialPoint, _translationSystem);
 }
 
 /******************************************************************************
@@ -254,31 +256,27 @@ void MoveMode::startXForm()
 void MoveMode::doXForm()
 {
     Point3 point2;
-    if(viewport()->snapPoint(_currentPoint, point2, _translationSystem)) {
+    if(viewportWindow()->snapPoint(_currentPoint, point2, _translationSystem)) {
 
         // Get movement in world space.
         _delta = _translationSystem * (point2 - _initialPoint);
 
         // Apply transformation to selected nodes.
-        applyXForm(viewport()->scene()->animationSettings()->currentTime(), viewport()->scene()->selection()->nodes(), 1);
+        applyXForm(viewportWindow()->viewport()->currentTime(), viewportWindow()->viewport()->scene()->selection()->nodes(), 1);
     }
 }
 
 /******************************************************************************
 * Applies the current transformation to a set of nodes.
 ******************************************************************************/
-void MoveMode::applyXForm(AnimationTime time, const QVector<SceneNode*>& nodeSet, FloatType multiplier)
+void MoveMode::applyXForm(AnimationTime time, const QVector<OORef<SceneNode>>& nodeSet, FloatType multiplier)
 {
     for(SceneNode* node : nodeSet) {
         OVITO_CHECK_OBJECT_POINTER(node);
         OVITO_CHECK_OBJECT_POINTER(node->transformationController());
 
-        // Get node's transformation system.
-        AffineTransformation transformSystem = _translationSystem;
-
         // Get parent's system.
-        TimeInterval iv;
-        const AffineTransformation& translationSys = node->parentNode()->getWorldTransform(time, iv);
+        const AffineTransformation& translationSys = node->parentNode()->getWorldTransform(time);
 
         // Move node in parent's system.
         node->transformationController()->translate(time, _delta * multiplier, translationSys.inverse());
@@ -290,9 +288,9 @@ void MoveMode::applyXForm(AnimationTime time, const QVector<SceneNode*>& nodeSet
 ******************************************************************************/
 void MoveMode::updateCoordinateDisplay(CoordinateDisplayWidget* coordDisplay)
 {
-    if(_selectedNode.target()) {
+    if(selectedNode()) {
         coordDisplay->setUnit(coordDisplay->mainWindow().unitsManager().worldUnit());
-        if(Controller* ctrl = _selectedNode.target()->transformationController()) {
+        if(Controller* ctrl = selectedNode()->transformationController()) {
             if(AnimationSettings* anim = inputManager()->datasetContainer().activeAnimationSettings()) {
                 TimeInterval iv;
                 Vector3 translation;
@@ -311,8 +309,8 @@ void MoveMode::updateCoordinateDisplay(CoordinateDisplayWidget* coordDisplay)
 ******************************************************************************/
 void MoveMode::onCoordinateValueEntered(int component, FloatType value)
 {
-    if(_selectedNode.target()) {
-        if(Controller* ctrl = _selectedNode.target()->transformationController()) {
+    if(selectedNode()) {
+        if(Controller* ctrl = selectedNode()->transformationController()) {
             if(AnimationSettings* anim = inputManager()->datasetContainer().activeAnimationSettings()) {
                 TimeInterval iv;
                 Vector3 translation;
@@ -330,10 +328,10 @@ void MoveMode::onCoordinateValueEntered(int component, FloatType value)
 ******************************************************************************/
 void MoveMode::onAnimateTransformationButton()
 {
-    if(_selectedNode.target()) {
-        if(PRSTransformationController* prs_ctrl = dynamic_object_cast<PRSTransformationController>(_selectedNode.target()->transformationController())) {
+    if(selectedNode()) {
+        if(PRSTransformationController* prs_ctrl = dynamic_object_cast<PRSTransformationController>(selectedNode()->transformationController())) {
             if(KeyframeController* ctrl = dynamic_object_cast<KeyframeController>(prs_ctrl->positionController())) {
-                if(MainWindow* mainWindow = dynamic_cast<MainWindow*>(&inputManager()->userInterface())) {
+                if(MainWindow* mainWindow = dynamic_object_cast<MainWindow>(&inputManager()->userInterface())) {
                     AnimationKeyEditorDialog dlg(ctrl, PROPERTY_FIELD(PRSTransformationController::positionController), mainWindow, *mainWindow);
                     dlg.exec();
                 }
@@ -356,19 +354,18 @@ void RotateMode::startXForm()
 void RotateMode::doXForm()
 {
     FloatType angle1 = (FloatType)(_currentPoint.y() - _startPoint.y()) / 100;
-    FloatType angle2 = (FloatType)(_currentPoint.x() - _startPoint.x()) / 100;
 
     // Constrain rotation to z-axis.
     _rotation = Rotation(Vector3(0,0,1), angle1);
 
     // Apply transformation to selected nodes.
-    applyXForm(viewport()->scene()->animationSettings()->currentTime(), viewport()->scene()->selection()->nodes(), 1);
+    applyXForm(viewportWindow()->viewport()->currentTime(), viewportWindow()->viewport()->scene()->selection()->nodes(), 1);
 }
 
 /******************************************************************************
 * Applies the current transformation to a set of nodes.
 ******************************************************************************/
-void RotateMode::applyXForm(AnimationTime time, const QVector<SceneNode*>& nodeSet, FloatType multiplier)
+void RotateMode::applyXForm(AnimationTime time, const QVector<OORef<SceneNode>>& nodeSet, FloatType multiplier)
 {
     for(SceneNode* node : nodeSet) {
         OVITO_CHECK_OBJECT_POINTER(node);
@@ -379,8 +376,7 @@ void RotateMode::applyXForm(AnimationTime time, const QVector<SceneNode*>& nodeS
         transformSystem.translation() = _transformationCenter - Point3::Origin();
 
         // Make transformation system relative to parent's tm.
-        TimeInterval iv;
-        const AffineTransformation& parentTM = node->parentNode()->getWorldTransform(time, iv);
+        const AffineTransformation& parentTM = node->parentNode()->getWorldTransform(time);
         transformSystem = transformSystem * parentTM.inverse();
 
         // Rotate node in transformation system.
@@ -394,9 +390,9 @@ void RotateMode::applyXForm(AnimationTime time, const QVector<SceneNode*>& nodeS
 ******************************************************************************/
 void RotateMode::updateCoordinateDisplay(CoordinateDisplayWidget* coordDisplay)
 {
-    if(_selectedNode.target()) {
+    if(selectedNode()) {
         coordDisplay->setUnit(coordDisplay->mainWindow().unitsManager().angleUnit());
-        if(Controller* ctrl = _selectedNode.target()->transformationController()) {
+        if(Controller* ctrl = selectedNode()->transformationController()) {
             if(AnimationSettings* anim = inputManager()->datasetContainer().activeAnimationSettings()) {
                 TimeInterval iv;
                 Rotation rotation;
@@ -416,10 +412,9 @@ void RotateMode::updateCoordinateDisplay(CoordinateDisplayWidget* coordDisplay)
 ******************************************************************************/
 void RotateMode::onCoordinateValueEntered(int component, FloatType value)
 {
-    if(_selectedNode.target()) {
-        if(Controller* ctrl = _selectedNode.target()->transformationController()) {
-            if(MainWindow* mainWindow = dynamic_cast<MainWindow*>(&inputManager()->userInterface())) {
-                TimeInterval iv;
+    if(selectedNode()) {
+        if(Controller* ctrl = selectedNode()->transformationController()) {
+            if(MainWindow* mainWindow = dynamic_object_cast<MainWindow>(&inputManager()->userInterface())) {
                 CoordinateDisplayWidget* coordDisplay = mainWindow->coordinateDisplay();
                 Vector3 euler = coordDisplay->getValues();
                 Rotation rotation = Rotation::fromEuler(Vector3(euler[2], euler[1], euler[0]), Matrix3::szyx);
@@ -437,12 +432,12 @@ void RotateMode::onCoordinateValueEntered(int component, FloatType value)
 ******************************************************************************/
 void RotateMode::onAnimateTransformationButton()
 {
-    if(_selectedNode.target()) {
-        PRSTransformationController* prs_ctrl = dynamic_object_cast<PRSTransformationController>(_selectedNode.target()->transformationController());
+    if(selectedNode()) {
+        PRSTransformationController* prs_ctrl = dynamic_object_cast<PRSTransformationController>(selectedNode()->transformationController());
         if(prs_ctrl) {
             KeyframeController* ctrl = dynamic_object_cast<KeyframeController>(prs_ctrl->rotationController());
             if(ctrl) {
-                if(MainWindow* mainWindow = dynamic_cast<MainWindow*>(&inputManager()->userInterface())) {
+                if(MainWindow* mainWindow = dynamic_object_cast<MainWindow>(&inputManager()->userInterface())) {
                     AnimationKeyEditorDialog dlg(ctrl, PROPERTY_FIELD(PRSTransformationController::rotationController), mainWindow, *mainWindow);
                     dlg.exec();
                 }
