@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -61,8 +61,8 @@ AnimationTrackBar::AnimationTrackBar(MainWindow& mainWindow, AnimationTimeSlider
     connect(&mainWindow.datasetContainer(), &DataSetContainer::currentFrameChanged, this, qOverload<>(&AnimationTrackBar::update));
     connect(&mainWindow.datasetContainer(), &DataSetContainer::timeFormatChanged, this, qOverload<>(&AnimationTrackBar::update));
     connect(&mainWindow.datasetContainer(), &DataSetContainer::selectionChangeComplete, this, &AnimationTrackBar::onRebuildControllerList);
-    connect(&_objects, &VectorRefTargetListener<RefTarget>::notificationEvent, this, &AnimationTrackBar::onObjectNotificationEvent);
-    connect(&_controllers, &VectorRefTargetListener<KeyframeController>::notificationEvent, this, &AnimationTrackBar::onControllerNotificationEvent);
+    _objects.connect(this, &AnimationTrackBar::onObjectNotificationEvent);
+    _controllers.connect(this, &AnimationTrackBar::onControllerNotificationEvent);
 }
 
 /******************************************************************************
@@ -188,7 +188,8 @@ void AnimationTrackBar::onRebuildControllerList()
     // Traverse object graphs of selected scene nodes to find all animation controllers.
     if(SelectionSet* selection = mainWindow().datasetContainer().activeSelectionSet()) {
         for(SceneNode* node : selection->nodes()) {
-            if(Pipeline* pipeline = dynamic_object_cast<Pipeline>(node))
+            findControllers(node);
+            if(Pipeline* pipeline = node->pipeline())
                 findControllers(pipeline);
         }
     }
@@ -258,7 +259,7 @@ void AnimationTrackBar::onObjectNotificationEvent(RefTarget* source, const Refer
 {
     // Rebuild the complete controller list whenever the reference object changes.
     if(event.type() == ReferenceEvent::ReferenceChanged || event.type() == ReferenceEvent::ReferenceAdded || event.type() == ReferenceEvent::ReferenceRemoved) {
-        if(!_objects.targets().empty() && !static_cast<const PropertyFieldEvent&>(event).field()->flags().testFlag(PROPERTY_FIELD_NO_SUB_ANIM)) {
+        if(!_isDragging && !_objects.targets().empty() && !static_cast<const PropertyFieldEvent&>(event).field()->flags().testFlag(PROPERTY_FIELD_NO_SUB_ANIM)) {
             _objects.clear();
             if(!_deferredUpdateScheduled) {
                 _deferredUpdateScheduled = true;
@@ -479,9 +480,10 @@ void AnimationTrackBar::showKeyContextMenu(const QPoint& pos, const QVector<Anim
     QAction* jumpToTimeAction = contextMenu.addAction(tr("Jump to key"));
     if(clickedKeys.empty() == false) {
         int frame = clickedKeys.front()->time().frame();
-        connect(jumpToTimeAction, &QAction::triggered, animSettings(), [this, frame]() {
+        connect(jumpToTimeAction, &QAction::triggered, this, [this, frame]() {
             mainWindow().handleExceptions([&] {
-                animSettings()->setCurrentFrame(frame);
+                if(animSettings())
+                    animSettings()->setCurrentFrame(frame);
             });
         });
     }
