@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -102,7 +102,7 @@ void SpinnerWidget::onTextChanged()
 
     try {
         if(textBox()->text() == _originalText) return;
-        if(textBox()->text().isEmpty() && std::isfinite(standardValue())) {
+        if(textBox()->text().isEmpty() && hasStandardValue()) {
             setFloatValue(standardValue(), true);
         }
         else if(unit()) {
@@ -157,7 +157,7 @@ void SpinnerWidget::setFloatValue(FloatType newVal, bool emitChangeSignal)
 #ifdef OVITO_DEBUG
             QPointer<SpinnerWidget> self(this);
 #endif
-            Q_EMIT spinnerValueChanged();
+            spinnerValueChanged();
 #ifdef OVITO_DEBUG
             OVITO_ASSERT(!self.isNull());
 #endif
@@ -186,7 +186,7 @@ void SpinnerWidget::setIntValue(int newValInt, bool emitChangeSignal)
 #ifdef OVITO_DEBUG
             QPointer<SpinnerWidget> self(this);
 #endif
-            Q_EMIT spinnerValueChanged();
+            spinnerValueChanged();
 #ifdef OVITO_DEBUG
             OVITO_ASSERT(!self.isNull());
 #endif
@@ -288,7 +288,7 @@ void SpinnerWidget::mousePressEvent(QMouseEvent* event)
 #ifdef OVITO_DEBUG
             QPointer<SpinnerWidget> self(this);
 #endif
-            Q_EMIT spinnerDragAbort();
+            spinnerDragAbort();
 #ifdef OVITO_DEBUG
             OVITO_ASSERT(!self.isNull());
 #endif
@@ -313,7 +313,7 @@ void SpinnerWidget::mouseReleaseEvent(QMouseEvent* event)
 #ifdef OVITO_DEBUG
             QPointer<SpinnerWidget> self(this);
 #endif
-            Q_EMIT spinnerDragStop();
+            spinnerDragStop();
 #ifdef OVITO_DEBUG
             OVITO_ASSERT(!self.isNull());
 #endif
@@ -358,7 +358,7 @@ void SpinnerWidget::mouseMoveEvent(QMouseEvent* event)
                 _lowerBtnPressed = true;
                 _lastMouseY = _startMouseY = mapToGlobal(event->pos()).y();
                 update();
-                Q_EMIT spinnerDragStart();
+                spinnerDragStart();
             }
         }
         else if(!_upperBtnPressed && _lowerBtnPressed) {
@@ -366,43 +366,52 @@ void SpinnerWidget::mouseMoveEvent(QMouseEvent* event)
                 _upperBtnPressed = true;
                 _lastMouseY = _startMouseY = mapToGlobal(event->pos()).y();
                 update();
-                Q_EMIT spinnerDragStart();
+                spinnerDragStart();
             }
         }
         else {
-            QPoint cursorPos = QCursor::pos();
-            int screenY = cursorPos.y();
+            const QPoint cursorPos = QCursor::pos();
+            const int screenY = cursorPos.y();
             if(screenY != _lastMouseY) {
-                int screenHeight = screen()->size().height();
-                if(screenY <= 5 && _lastMouseY == screenHeight - 1) return;
-                if(screenY >= screenHeight - 5 && _lastMouseY == 0) return;
+                // Get current screen top and bottom positions
+                // QT convention has greater Y values pointing down
+                const int screenMaxY = screen()->geometry().bottom();
+                const int screenMinY = screen()->geometry().top();
+                OVITO_ASSERT(screenMinY < screenMaxY);
+                // Distance from the edge at which warpping occurs
+                constexpr int borderWidth = 5;
+
+                if(screenY <= screenMinY + borderWidth && _lastMouseY == screenMaxY - 1) return;
+                if(screenY >= screenMaxY - borderWidth && _lastMouseY == screenMinY) return;
 
                 FloatType newVal = _oldValue + _currentStepSize * (FloatType)(_startMouseY - screenY) * FloatType(0.1);
                 if(unit())
                     newVal = unit()->roundValue(newVal);
 
-                if(screenY < _lastMouseY && screenY <= 5) {
-                    _lastMouseY = screenHeight - 1;
+                // Accessibility access is required on macOS to move the mouse
+                bool accessibilty = false;
+                if(MainWindow* mainWindow = qobject_cast<MainWindow*>(window())) {
+                    accessibilty = mainWindow->checkAccessibilityAccess(this);
+                }
+
+                if(accessibilty && screenY < _lastMouseY && screenY <= screenMinY + borderWidth) {
+                    _lastMouseY = screenMaxY - 1;
                     _startMouseY += _lastMouseY - screenY;
                     QCursor::setPos(cursorPos.x(), _lastMouseY);
                 }
-                else if(screenY > _lastMouseY && screenY >= screenHeight - 5) {
-                    _lastMouseY = 0;
+                else if(accessibilty && screenY > _lastMouseY && screenY >= screenMaxY - borderWidth) {
+                    _lastMouseY = screenMinY;
                     _startMouseY += _lastMouseY - screenY;
                     QCursor::setPos(cursorPos.x(), _lastMouseY);
                 }
-                else _lastMouseY = screenY;
+                else {
+                    _lastMouseY = screenY;
+                }
 
                 if(newVal != floatValue()) {
                     setFloatValue(newVal, true);
-
-                    // Repaint viewports for immediate visual feedback when changing a parameter.
-                    if(MainWindow* mainWindow = qobject_cast<MainWindow*>(window()))
-                        mainWindow->processViewportUpdateRequests();
-
-                    // Also repaint text box for immediate visual updates.
                     if(textBox())
-                        textBox()->repaint();
+                        textBox()->update();
                 }
             }
         }
@@ -419,7 +428,7 @@ void SpinnerWidget::focusOutEvent(QFocusEvent* event)
 #ifdef OVITO_DEBUG
         QPointer<SpinnerWidget> self(this);
 #endif
-        Q_EMIT spinnerDragAbort();
+        spinnerDragAbort();
 #ifdef OVITO_DEBUG
         OVITO_ASSERT(!self.isNull());
 #endif
@@ -431,4 +440,62 @@ void SpinnerWidget::focusOutEvent(QFocusEvent* event)
     QWidget::focusOutEvent(event);
 }
 
+/******************************************************************************
+* Called after the spinner's value has been changed by the user.
+******************************************************************************/
+void SpinnerWidget::spinnerValueChanged()
+{
+    if(_userInterface) {
+        // Create an undoable transaction to record the change or
+        // use the existing transaction if the user is dragging the spinner.
+        std::optional<UndoableTransaction> transaction;
+        if(_undoTransaction.operation()) {
+            auto val = _value;
+            _undoTransaction.revert(); // Note: This may revert the spinner's value to an old value.
+            _value = val;
+        }
+        else
+            transaction.emplace(*_userInterface, _undoOperationName);
+
+        // Perform the parameter change (including exception handling).
+        bool success = _userInterface->performActions(transaction ? *transaction : _undoTransaction, [&]() {
+            Q_EMIT valueChanged();
+        });
+
+        // Commit the transaction.
+        if(success && transaction)
+            transaction->commit();
+    }
+    else {
+        Q_EMIT valueChanged();
+    }
+}
+
+/******************************************************************************
+* Called when the user has started a drag operation.
+******************************************************************************/
+void SpinnerWidget::spinnerDragStart()
+{
+    if(_userInterface)
+        _undoTransaction.begin(*_userInterface, _undoOperationName);
+}
+
+/******************************************************************************
+* Called when the user has finished the drag operation.
+******************************************************************************/
+void SpinnerWidget::spinnerDragStop()
+{
+    if(_undoTransaction.operation())
+        _undoTransaction.commit();
+}
+
+/******************************************************************************
+* Called when the user has aborted the drag operation.
+******************************************************************************/
+void SpinnerWidget::spinnerDragAbort()
+{
+    if(_undoTransaction.operation())
+        _undoTransaction.cancel();
+}
+
 }   // End of namespace
