@@ -137,6 +137,10 @@ std::vector<ParaViewVTMBlockInfo> ParaViewVTMImporter::loadVTMFile(const FileHan
 
                 xml.skipCurrentElement();
             }
+            else if(xml.name().compare(QStringLiteral("FieldData")) == 0) {
+                // Skip <FieldData> elements.
+                xml.skipCurrentElement();
+            }
             else {
                 xml.raiseError(tr("Unexpected XML element <%1>.").arg(xml.name().toString()));
             }
@@ -200,6 +204,8 @@ Future<PipelineFlowState> ParaViewVTMImporter::loadFrame(const LoadOperationRequ
         ParaViewVTMBlockInfo blockInfo;
         /// Plugin filters processing the datasets referenced by the VTM file.
         std::vector<OORef<ParaViewVTMFileFilter>> filters;
+        /// List of referenced files that have been ignored by OVITO.
+        std::vector<QUrl> ignoredFiles;
     };
 
     // Copy the pipeline request into an extended data structure, which can be modified.
@@ -222,7 +228,8 @@ Future<PipelineFlowState> ParaViewVTMImporter::loadFrame(const LoadOperationRequ
         std::move(modifiedRequest), std::move(blockDatasets), DeferredObjectExecutor(request.pipelineNode),
         [](const ParaViewVTMBlockInfo& blockInfo, ExtendedLoadRequest& request) {
             // We can skip empty datasets which are not associated with a VTK file.
-            if(blockInfo.location.isEmpty()) return Future<void>::createImmediateEmpty();
+            if(blockInfo.location.isEmpty())
+                return Future<void>::createImmediateEmpty();
 
             // Set up the load request submitted to the FileSourceImporter.
             request.dataBlockPrefix = blockInfo.blockPath.back();
@@ -232,7 +239,8 @@ Future<PipelineFlowState> ParaViewVTMImporter::loadFrame(const LoadOperationRequ
                  0);  // Append data (instead of replacing it) when loading subsequent partial blocks of a piece-wise (parallel) dataset.
 
             OORef<const PipelineNode> fileSource = request.pipelineNode.lock();
-            if(!fileSource) throw Exception(QStringLiteral("Object requesting the data import has been deleted."));
+            if(!fileSource)
+                throw Exception(QStringLiteral("Object requesting the data import has been deleted."));
 
             // Retrieve the data file, then detect its format.
             // Note: FileImporter::autodetectFileFormat() may only be called from the main thread.
@@ -258,8 +266,13 @@ Future<PipelineFlowState> ParaViewVTMImporter::loadFrame(const LoadOperationRequ
                         // This currently works only for FileSourceImporters. Files handled by other kinds of importers will be skipped.
                         // VTK dataset blocks using a file format not supported by OVITO are silently ignored.
                         OORef<FileSourceImporter> importer = dynamic_object_cast<FileSourceImporter>(FileImporter::autodetectFileFormat(file));
-                        if(!importer)
+                        if(!importer) {
+                            // Gather list of unsupported files that have been skipped,
+                            // unless it's the "Domain" block from an Aspherix VTM file, which we ignore silently.
+                            if(request.blockInfo.blockPath.size() != 1 || request.blockInfo.blockPath[0] != QStringLiteral("Domain"))
+                                request.ignoredFiles.push_back(file.sourceUrl());
                             return Future<void>::createImmediateEmpty();
+                        }
 
                         // Remember the current status returned by the loading operations completed so far.
                         // We will prepend this existing status text to the one generated by the current file importer.
@@ -323,6 +336,14 @@ Future<PipelineFlowState> ParaViewVTMImporter::loadFrame(const LoadOperationRequ
         for(const auto& filter : request.filters)
             filter->postprocessDatasets(request);
 
+        if(!request.ignoredFiles.empty()) {
+            // Report the files that were ignored by OVITO.
+            QStringList fileNames;
+            for(const QUrl& url : request.ignoredFiles)
+                fileNames.push_back(url.fileName());
+            request.state.combineStatus(PipelineStatus::Success, tr("Skipped unsupported data file(s) referenced in the VTM file: %1").arg(fileNames.join(QStringLiteral(", "))));
+        }
+
         // Return just the PipelineFlowState to the caller.
         return std::move(request.state);
     });
