@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -187,11 +187,6 @@ public:
     /// Constructor that takes an existing SurfaceMesh object.
     explicit SurfaceMeshBuilder(SurfaceMesh* mesh);
 
-#ifdef OVITO_DEBUG
-    /// Destructor.
-    ~SurfaceMeshBuilder();
-#endif
-
     /// Resets the surface mesh structure by discarding all existing vertices, faces and regions.
     void clearMesh();
 
@@ -269,8 +264,8 @@ public:
     vertex_index createVerticesRange(CoordinatesRange coordRange) {
         auto nverts = std::distance(std::begin(coordRange), std::end(coordRange));
         vertex_index startIndex = createVertices(nverts);
-        bool no_init = (startIndex == 0);
-        BufferWriteAccess<Point3, access_mode::write> vertexPositions(mutableVertexProperty(SurfaceMeshVertices::PositionProperty, no_init ? DataBuffer::Uninitialized : DataBuffer::Initialized), no_init);
+        DataBuffer::BufferInitialization initMode = (startIndex == 0) ? DataBuffer::Uninitialized : DataBuffer::Initialized;
+        BufferWriteAccess<Point3, access_mode::write> vertexPositions(mutableVertexProperty(SurfaceMeshVertices::PositionProperty, initMode), initMode);
         boost::copy(std::forward<CoordinatesRange>(coordRange), std::next(vertexPositions.begin(), startIndex));
         return startIndex;
     }
@@ -302,19 +297,19 @@ public:
 
     /// Attaches an existing property object to the vertices of the mesh.
     void addVertexProperty(const Property* property) {
-        OVITO_ASSERT(property->type() == 0 || vertices()->getProperty(property->type()) == nullptr);
+        OVITO_ASSERT(!property->isStandardProperty() || vertices()->getProperty(property->typeId()) == nullptr);
         mutableVertices()->addProperty(property);
     }
 
     /// Attaches an existing property object to the faces of the mesh.
     void addFaceProperty(const Property* property) {
-        OVITO_ASSERT(property->type() == 0 || faces()->getProperty(property->type()) == nullptr);
+        OVITO_ASSERT(!property->isStandardProperty() || faces()->getProperty(property->typeId()) == nullptr);
         mutableFaces()->addProperty(property);
     }
 
     /// Attaches an existing property object to the regions of the mesh.
     void addRegionProperty(const Property* property) {
-        OVITO_ASSERT(property->type() == 0 || regions()->getProperty(property->type()) == nullptr);
+        OVITO_ASSERT(!property->isStandardProperty() || regions()->getProperty(property->typeId()) == nullptr);
         mutableRegions()->addProperty(property);
     }
 
@@ -401,7 +396,7 @@ public:
     void flipFaces() { mutableTopology()->flipFaces(); }
 
     /// Fairs the surface mesh.
-    bool smoothMesh(int numIterations, ProgressingTask& task, FloatType k_PB = FloatType(0.1), FloatType lambda = FloatType(0.5));
+    void smoothMesh(int numIterations, TaskProgress& progress, FloatType k_PB = FloatType(0.1), FloatType lambda = FloatType(0.5));
 
     /// Splits a face along the edge given by the second vertices of two of its border edges.
     edge_index splitFace(edge_index edge1, edge_index edge2, FaceGrower& faceGrower);
@@ -452,8 +447,8 @@ public:
     /// Computes the void, exterior, and total volumes from the per region volume properties.
     [[nodiscard]] AggregateVolumes computeAggregateVolumes() const;
 
-    /// Set the volume of external regions to infinity if the simulation cell is non-periodic.
-    void nonPBCexternalVolume();
+    /// Set the volume of the external region to infinity if the simulation cell is non-periodic in at least one spatial direction.
+    void setExternalRegionVolumeInfinityIfNonPeriodic();
 
 private:
 
