@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -39,14 +39,13 @@ namespace Ovito {
  * Each vertex has a pointer to the first half-edge originating from it.
  * Each face has a pointer to one of the half-edges adjacent to it.
  *
- * Note that this class stores only the topolgy of the mesh, i.e. the connectivty of vertices, half-edges
+ * Note that this class stores only the topology of the mesh, i.e. the connectivity of vertices, half-edges
  * and faces. The embedding of the mesh into three-dimensional space, i.e. the vertex coordinates,
  * are not managed by the class and must be kept in a separate data array.
  */
 class OVITO_MESH_EXPORT SurfaceMeshTopology : public DataObject
 {
     OVITO_CLASS(SurfaceMeshTopology)
-    Q_CLASSINFO("DisplayName", "Surface mesh topology");
 
 public:
 
@@ -67,9 +66,6 @@ public:
 
 public:
 
-    /// Constructor creating an empty SurfaceMeshTopology object.
-    Q_INVOKABLE SurfaceMeshTopology(ObjectInitializationFlags flags) : DataObject(flags) {}
-
     /// Removes all faces, edges and vertices from this mesh.
     void clear();
 
@@ -379,7 +375,7 @@ public:
         return faceEdgeCount(firstFaceEdge(face));
     }
 
-    /// Inserts a vertex in the midle of an existing edge.
+    /// Inserts a vertex in the middle of an existing edge.
     void splitEdge(edge_index edge, vertex_index vertex);
 
 protected:
