@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -56,7 +56,8 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(AMBERNetCDFImporter);
+IMPLEMENT_CREATABLE_OVITO_CLASS(AMBERNetCDFImporter);
+OVITO_CLASSINFO(AMBERNetCDFImporter, "DisplayName", "NetCDF");
 DEFINE_PROPERTY_FIELD(AMBERNetCDFImporter, useCustomColumnMapping);
 DEFINE_PROPERTY_FIELD(AMBERNetCDFImporter, customColumnMapping);
 SET_PROPERTY_FIELD_LABEL(AMBERNetCDFImporter, useCustomColumnMapping, "Custom file column mapping");
@@ -95,7 +96,7 @@ bool AMBERNetCDFImporter::OOMetaClass::checkFileFormat(const FileHandle& file) c
         // Particle data may be stored in a subgroup named "AMBER" instead of the root group.
         int amber_ncid = tmp_ncid;
         if(nc_inq_ncid(tmp_ncid, "AMBER", &amber_ncid) == NC_NOERR) {
-            amber_ncid = amber_ncid;
+            (void)amber_ncid;
         }
 
         // Make sure we have the right file conventions.
@@ -120,15 +121,14 @@ bool AMBERNetCDFImporter::OOMetaClass::checkFileFormat(const FileHandle& file) c
 /******************************************************************************
 * Scans the data file and builds a list of source frames.
 ******************************************************************************/
-void AMBERNetCDFImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::Frame>& frames)
+void AMBERNetCDFImporter::discoverFramesInFile(const FileHandle& fileHandle, QVector<FileSourceImporter::Frame>& frames) const
 {
-    QString filename = QDir::toNativeSeparators(fileHandle().localFilePath());
+    QString filename = QDir::toNativeSeparators(fileHandle.localFilePath());
     if(filename.isEmpty())
         throw Exception(tr("The NetCDF file reader supports reading only from physical files. Cannot read data from an in-memory buffer."));
 
     // Only serial access to NetCDF functions is allowed, because they are not thread-safe.
-    NetCDFExclusiveAccess locker(this);
-    if(!locker.isLocked()) return;
+    NetCDFExclusiveAccess locker;
 
     // Open the input NetCDF file.
     int ncid;
@@ -149,7 +149,7 @@ void AMBERNetCDFImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceIm
     NCERR( nc_inq_dimlen(ncid, frame_dim, &nFrames) );
     NCERR( nc_close(root_ncid) );
 
-    Frame frame(fileHandle());
+    Frame frame(fileHandle);
     for(size_t i = 0; i < nFrames; i++) {
         frame.lineNumber = i;
         frame.label = tr("Frame %1").arg(i);
@@ -282,7 +282,7 @@ ParticleInputColumnMapping AMBERNetCDFImporter::NetCDFFile::detectColumnMapping(
                     _coordinatesVar = varId;
             }
             else {
-                qDebug() << "Skipping NetCDF variable " << name << " because data type is not known.";
+                qWarning() << "Skipping NetCDF variable " << name << " because data type is not known.";
             }
         }
     }
@@ -348,7 +348,8 @@ bool AMBERNetCDFImporter::NetCDFFile::detectDims(int movieFrame, int particleCou
 ******************************************************************************/
 void AMBERNetCDFImporter::FrameLoader::loadFile()
 {
-    setProgressText(tr("Reading NetCDF file %1").arg(fileHandle().toString()));
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Reading NetCDF file %1").arg(fileHandle().toString()));
 
     QString filename = QDir::toNativeSeparators(fileHandle().localFilePath());
     if(filename.isEmpty())
@@ -358,8 +359,7 @@ void AMBERNetCDFImporter::FrameLoader::loadFile()
     size_t movieFrame = frame().lineNumber;
 
     // Only serial access to NetCDF functions is allowed, because they are not thread-safe.
-    NetCDFExclusiveAccess locker(this);
-    if(!locker.isLocked()) return;
+    NetCDFExclusiveAccess locker;
 
     // Open the NetCDF file for reading.
     NetCDFFile ncFile;
@@ -467,10 +467,10 @@ void AMBERNetCDFImporter::FrameLoader::loadFile()
     }
     simulationCell()->setCellMatrix(AffineTransformation(va, vb, vc, Vector3(o[0], o[1], o[2])));
 
-    // Report to user.
-    beginProgressSubSteps(columnMapping.size());
+    // Report progress to user.
+    progress.beginSubSteps(columnMapping.size());
 
-    // We inpsect the particle coordinate array in the NetCDF first before any properties are loaded
+    // We inspect the particle coordinate array in the NetCDF first before any properties are loaded
     // in order to determine the number of particles (which might actually be lower than the size of the "atoms" dimension).
 
     // Retrieve NetCDF variable meta-information.
@@ -495,14 +495,13 @@ void AMBERNetCDFImporter::FrameLoader::loadFile()
     // Now iterate over all NetCDF variables and load the appropriate frame data.
     std::vector<Property*> loadedProperties;
     for(const InputColumnInfo& column : columnMapping) {
-        if(isCanceled())
-            return;
+        this_task::throwIfCanceled();
         if(&column != &columnMapping.front())
-            nextProgressSubStep();
+            progress.nextSubStep();
 
         Property* property = nullptr;
         QString columnName = column.columnName;
-        QString propertyName = column.property.name();
+        auto propertyName = column.property.name();
         int dataType = column.dataType;
         if(dataType == QMetaType::Void)
             continue;
@@ -522,7 +521,7 @@ void AMBERNetCDFImporter::FrameLoader::loadFile()
             continue;
 
         // Create property to load this information into.
-        Particles::Type propertyType = (Particles::Type)column.property.type();
+        Particles::Type propertyType = (Particles::Type)column.property.standardTypeId(&Particles::OOClass());
         if(propertyType != Particles::UserProperty) {
             // Create standard property.
             property = particles()->createProperty(DataBuffer::Initialized, propertyType);
@@ -563,15 +562,14 @@ void AMBERNetCDFImporter::FrameLoader::loadFile()
                 size_t totalCount = countp[particleCountDim];
                 size_t remaining = totalCount;
                 countp[particleCountDim] = 1000000;
-                setProgressMaximum(totalCount / countp[particleCountDim] + 1);
+                progress.setMaximum(totalCount / countp[particleCountDim] + 1);
                 for(size_t chunk = 0; chunk < totalCount; chunk += countp[particleCountDim], startp[particleCountDim] += countp[particleCountDim]) {
                     countp[particleCountDim] = std::min(countp[particleCountDim], remaining);
                     remaining -= countp[particleCountDim];
                     OVITO_ASSERT(countp[particleCountDim] >= 1);
                     OVITO_STATIC_ASSERT(sizeof(T) == sizeof(U));
                     NCERRI( nc_get_vara(ncFile._ncid, varId, startp, countp, reinterpret_cast<U*>(propertyArray.begin() + chunk * property->componentCount())), tr("(While reading variable '%1'.)").arg(columnName) );
-                    if(!incrementProgressValue())
-                        return false;
+                    progress.incrementValue();
                 }
                 OVITO_ASSERT(remaining == 0);
             }
@@ -588,7 +586,7 @@ void AMBERNetCDFImporter::FrameLoader::loadFile()
                 return;
 
             // Create particles types if this is the typed property.
-            if(OvitoClassPtr elementTypeClass = Particles::OOClass().typedPropertyElementClass(property->type())) {
+            if(OvitoClassPtr elementTypeClass = Particles::OOClass().typedPropertyElementClass(property->typeId())) {
 
                 // Create particle types.
                 for(int ptype : BufferReadAccess<int32_t>(property)) {
@@ -614,7 +612,7 @@ void AMBERNetCDFImporter::FrameLoader::loadFile()
                 return;
         }
         else {
-            qDebug() << "Warning: Skipping field '" << columnName << "' of NetCDF file because it has an unrecognized data type.";
+            qWarning() << "Warning: Skipping field '" << columnName << "' of NetCDF file because it has an unrecognized data type.";
         }
     }
 
@@ -625,7 +623,7 @@ void AMBERNetCDFImporter::FrameLoader::loadFile()
             particles()->removeProperty(property);
     }
 
-    endProgressSubSteps();
+    progress.endSubSteps();
 
     // If the input file does not contain simulation cell size, use bounding box of particles as simulation cell.
     if(!pbc[0] || !pbc[1] || !pbc[2]) {
@@ -691,12 +689,12 @@ InputColumnInfo AMBERNetCDFImporter::mapVariableToColumn(const QString& name, in
     // Only map to standard property if data layout matches.
     if(standardType != Particles::UserProperty) {
         if(componentCount == Particles::OOClass().standardPropertyComponentCount(standardType)) {
-            column.mapStandardColumn(&Particles::OOClass(), standardType);
+            column.mapToStandardProperty(&Particles::OOClass(), standardType);
             return column;
         }
     }
 
-    column.mapCustomColumn(&Particles::OOClass(), Property::makePropertyNameValid(name), dataType);
+    column.mapToUserProperty(Property::makePropertyNameValid(name), dataType);
     return column;
 }
 
@@ -708,7 +706,7 @@ Future<ParticleInputColumnMapping> AMBERNetCDFImporter::inspectFileHeader(const
 {
     // Retrieve file.
     return Application::instance()->fileManager().fetchUrl(frame.sourceFile)
-        .then(*this, [](const FileHandle& fileHandle) {
+        .then(ObjectExecutor(this), [](const FileHandle& fileHandle) {
             QString filename = QDir::toNativeSeparators(fileHandle.localFilePath());
             if(filename.isEmpty())
                 throw Exception(tr("The NetCDF file reader supports reading only from physical files. Cannot read data from an in-memory buffer."));
@@ -725,6 +723,18 @@ Future<ParticleInputColumnMapping> AMBERNetCDFImporter::inspectFileHeader(const
         });
 }
 
+/******************************************************************************
+* Is called when the value of a non-animatable property field of this RefMaker has changed.
+******************************************************************************/
+void AMBERNetCDFImporter::propertyChanged(const PropertyFieldDescriptor* field)
+{
+    ParticleImporter::propertyChanged(field);
+
+    if((field == PROPERTY_FIELD(customColumnMapping) || field == PROPERTY_FIELD(useCustomColumnMapping)) && !isBeingLoaded()) {
+        requestReload();
+    }
+}
+
 /******************************************************************************
  * Saves the class' contents to the given stream.
  *****************************************************************************/
