@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -23,8 +23,9 @@
 #include <ovito/core/Core.h>
 #include <ovito/core/dataset/data/BufferAccess.h>
 #include <ovito/core/dataset/DataSet.h>
+#include <ovito/core/rendering/MeshPrimitive.h>
 #include <ovito/core/utilities/SortZipped.h>
-#include "OpenGLSceneRenderer.h"
+#include "OpenGLRenderingJob.h"
 #include "OpenGLShaderHelper.h"
 
 namespace Ovito {
@@ -32,9 +33,9 @@ namespace Ovito {
 /******************************************************************************
 * Renders a triangle mesh.
 ******************************************************************************/
-void OpenGLSceneRenderer::renderMeshImplementation(const MeshPrimitive& primitive)
+void OpenGLRenderingJob::renderMeshImplementation(const MeshPrimitive& primitive, const FrameGraph::RenderingCommand& command)
 {
-    QOpenGLTexture* colorMapTexture = nullptr;
+    const OpenGLTexture* colorMapTexture = nullptr;
 
     // Make sure there is something to be rendered. Otherwise, step out early.
     if(!primitive.mesh() || primitive.mesh()->faceCount() == 0)
@@ -42,12 +43,9 @@ void OpenGLSceneRenderer::renderMeshImplementation(const MeshPrimitive& primitiv
     if(primitive.useInstancedRendering() && primitive.perInstanceTMs()->size() == 0)
         return;
 
-    rebindVAO();
-    OVITO_REPORT_OPENGL_ERRORS(this);
-
     // Render wireframe lines.
     if(primitive.emphasizeEdges() && !isPickingPass())
-        renderMeshWireframeImplementation(primitive);
+        renderMeshWireframeImplementation(primitive, false);
 
     // The mesh object to be rendered.
     const TriangleMesh& mesh = *primitive.mesh();
@@ -57,17 +55,15 @@ void OpenGLSceneRenderer::renderMeshImplementation(const MeshPrimitive& primitiv
         qWarning() << "WARNING: OpenGL renderer - Mesh to be rendered has too many faces, exceeding device limits.";
         return;
     }
-    if(primitive.useInstancedRendering() && primitive.perInstanceTMs()->size() > std::numeric_limits<int32_t>::max() / (3 * sizeof(Vector_4<float>))) {
+    if(primitive.useInstancedRendering() && primitive.perInstanceTMs()->size() > std::numeric_limits<int32_t>::max() / (3 * sizeof(Vector4F))) {
         qWarning() << "WARNING: OpenGL renderer - Number of mesh instances to be rendered exceeds device limits.";
         return;
     }
 
     // Decide whether per-pixel pseudo-color mapping is used.
     bool renderWithPseudoColorMapping = false;
-    if(primitive.pseudoColorMapping().isValid() && !isPickingPass() && !primitive.useInstancedRendering()) {
-        if(!mesh.hasVertexColors() && mesh.hasVertexPseudoColors())
-            renderWithPseudoColorMapping = true;
-        else if(!mesh.hasFaceColors() && mesh.hasFacePseudoColors())
+    if(primitive.pseudoColorMapping().isValid() && !isPickingPass() && !primitive.useInstancedRendering() && !mesh.hasVertexColors() && !mesh.hasFaceColors()) {
+        if(mesh.hasVertexPseudoColors() || mesh.hasFacePseudoColors())
             renderWithPseudoColorMapping = true;
     }
 
@@ -83,7 +79,7 @@ void OpenGLSceneRenderer::renderMeshImplementation(const MeshPrimitive& primitiv
         shader.setInstanceCount(1);
     }
     else {
-        OVITO_ASSERT(!renderWithPseudoColorMapping); // Note: Color mapping has not been implemented yet for instanced mesh primtives.
+        OVITO_ASSERT(!renderWithPseudoColorMapping); // Note: Color mapping has not been implemented yet for instanced mesh primitives.
         if(!isPickingPass()) {
             if(!primitive.perInstanceColors())
                 shader.load("mesh_instanced", "mesh/mesh_instanced.vert", "mesh/mesh_instanced.frag");
@@ -99,14 +95,15 @@ void OpenGLSceneRenderer::renderMeshImplementation(const MeshPrimitive& primitiv
 
     // Are we rendering a semi-transparent mesh?
     bool useBlending = !isPickingPass() && !primitive.isFullyOpaque() && !orderIndependentTransparency();
-    if(useBlending) shader.enableBlending();
+    if(useBlending)
+        shader.enableBlending();
 
     // Turn back-face culling off if requested.
     if(!primitive.cullFaces()) {
         OVITO_CHECK_OPENGL(this, glDisable(GL_CULL_FACE));
     }
 
-    // Apply optional positive depth-offset to mesh faces to make the wireframe lines fully visible.
+    // Apply optional positive depth offset to mesh faces to make the wireframe lines fully visible.
     if(primitive.emphasizeEdges() && !isPickingPass()) {
         OVITO_CHECK_OPENGL(this, glEnable(GL_POLYGON_OFFSET_FILL));
         OVITO_CHECK_OPENGL(this, glPolygonOffset(1.0f, 1.0f));
@@ -114,21 +111,24 @@ void OpenGLSceneRenderer::renderMeshImplementation(const MeshPrimitive& primitiv
 
     // Pass picking base ID to shader.
     if(isPickingPass()) {
-        shader.setPickingBaseId(registerSubObjectIDs(primitive.useInstancedRendering() ? primitive.perInstanceTMs()->size() : mesh.faceCount()));
+        shader.setPickingBaseId(objectPickingMap()->allocateObjectPickingIDs(command, primitive.useInstancedRendering() ? primitive.perInstanceTMs()->size() : mesh.faceCount()));
     }
 
+    bool highlightSelectedFaces = frameGraph()->isInteractive() && !isPickingPass();
+
     // The lookup key for the buffer cache.
-    RendererResourceKey<struct MeshBufferCache, DataOORef<const TriangleMesh>, std::vector<ColorA>, ColorA, Color> meshCacheKey{
+    RendererResourceKey<struct MeshBufferCache, DataOORef<const TriangleMesh>, std::vector<ColorA>, ColorA, Color, bool, bool> meshCacheKey{
         primitive.mesh(),
         primitive.materialColors(),
         primitive.uniformColor(),
-        primitive.faceSelectionColor()
+        primitive.faceSelectionColor(),
+        highlightSelectedFaces,
+        renderWithPseudoColorMapping
     };
 
     // Upload vertex buffer to GPU memory.
     QOpenGLBuffer meshBuffer = shader.createCachedBuffer(std::move(meshCacheKey), sizeof(MeshPrimitive::RenderVertex), QOpenGLBuffer::VertexBuffer, OpenGLShaderHelper::PerVertex, [&](void* buffer, BufferReadAccess<int32_t> subset) {
         OVITO_ASSERT(!subset);
-        bool highlightSelectedFaces = isInteractive() && !isPickingPass();
         primitive.generateRenderableVertices(reinterpret_cast<MeshPrimitive::RenderVertex*>(buffer), highlightSelectedFaces, renderWithPseudoColorMapping);
     });
 
@@ -161,7 +161,7 @@ void OpenGLSceneRenderer::renderMeshImplementation(const MeshPrimitive& primitiv
         shader.setUniformValue("color_range_max", maxValue);
 
         // Upload color map as a 1-d OpenGL texture.
-        colorMapTexture = OpenGLResourceManager::instance()->uploadColorMap(primitive.pseudoColorMapping().gradient(), currentResourceFrame());
+        colorMapTexture = &uploadColorMap(primitive.pseudoColorMapping().gradient());
         colorMapTexture->bind();
     }
 
@@ -170,9 +170,9 @@ void OpenGLSceneRenderer::renderMeshImplementation(const MeshPrimitive& primitiv
         QOpenGLBuffer instanceTMBuffer = getMeshInstanceTMBuffer(primitive, shader);
 
         // Bind buffer with the instance matrices.
-        shader.bindBuffer(instanceTMBuffer, "instance_tm_row1", GL_FLOAT, 4, 3 * sizeof(Vector_4<float>), 0 * sizeof(Vector_4<float>), OpenGLShaderHelper::PerInstance);
-        shader.bindBuffer(instanceTMBuffer, "instance_tm_row2", GL_FLOAT, 4, 3 * sizeof(Vector_4<float>), 1 * sizeof(Vector_4<float>), OpenGLShaderHelper::PerInstance);
-        shader.bindBuffer(instanceTMBuffer, "instance_tm_row3", GL_FLOAT, 4, 3 * sizeof(Vector_4<float>), 2 * sizeof(Vector_4<float>), OpenGLShaderHelper::PerInstance);
+        shader.bindBuffer(instanceTMBuffer, "instance_tm_row1", GL_FLOAT, 4, 3 * sizeof(Vector4F), 0 * sizeof(Vector4F), OpenGLShaderHelper::PerInstance);
+        shader.bindBuffer(instanceTMBuffer, "instance_tm_row2", GL_FLOAT, 4, 3 * sizeof(Vector4F), 1 * sizeof(Vector4F), OpenGLShaderHelper::PerInstance);
+        shader.bindBuffer(instanceTMBuffer, "instance_tm_row3", GL_FLOAT, 4, 3 * sizeof(Vector4F), 2 * sizeof(Vector4F), OpenGLShaderHelper::PerInstance);
 
         if(primitive.perInstanceColors() && !isPickingPass()) {
             // Upload the per-instance colors to GPU memory.
@@ -192,7 +192,7 @@ void OpenGLSceneRenderer::renderMeshImplementation(const MeshPrimitive& primitiv
 
         // Assuming that the input mesh is convex, render semi-transparent triangles in two passes:
         // First, render triangles facing away from the viewer, then render triangles facing toward the viewer.
-        // Each time we pass the entire triangle list to OpenGL and use OpenGL's backface/frontfrace culling
+        // Each time we pass the entire triangle list to OpenGL and use OpenGL's backface/frontface culling
         // option to render the right subset of triangles.
         if(!primitive.cullFaces()) {
             // First pass is only needed if backface culling is not active.
@@ -211,17 +211,25 @@ void OpenGLSceneRenderer::renderMeshImplementation(const MeshPrimitive& primitiv
         // Create a buffer for an indexed drawing command to render the triangles in back-to-front order.
 
         // Viewing direction in object space:
-        const Vector3 direction = modelViewTM().inverse().column(2);
+        const Vector3 direction = modelViewTM().linear().inverse().column(2);
+
+        // Coarsen the direction vector's precision for real-time rendering to reduce the frequency at
+        // which the particles must be reordered when moving the camera.
+        Vector3 coarseDirection = direction;
+        if(frameGraph()->isInteractive()) {
+            for(size_t dim = 0; dim < 3; dim++)
+                coarseDirection[dim] = std::round(2 * direction[dim]);
+        }
 
         // The caching key for the index buffer.
-        RendererResourceKey<struct DepthSortingCache, DataOORef<const TriangleMesh>, Vector3> indexBufferCacheKey{ primitive.mesh(), direction };
+        RendererResourceKey<struct DepthSortingCache, DataOORef<const TriangleMesh>, Vector3> indexBufferCacheKey{ primitive.mesh(), coarseDirection };
 
         // Create index buffer with three entries per triangle face.
         QOpenGLBuffer indexBuffer = shader.createCachedBuffer(std::move(indexBufferCacheKey), sizeof(GLsizei), QOpenGLBuffer::IndexBuffer, OpenGLShaderHelper::PerVertex, [&](void* buffer, BufferReadAccess<int32_t> subset) {
             OVITO_ASSERT(!subset);
 
             // Compute each face's center point.
-            std::vector<Vector_3<float>> faceCenters(mesh.faceCount());
+            std::vector<Vector3F> faceCenters(mesh.faceCount());
             auto tc = faceCenters.begin();
             for(auto face = mesh.faces().cbegin(); face != mesh.faces().cend(); ++face, ++tc) {
                 // Compute centroid of triangle.
@@ -235,16 +243,16 @@ void OpenGLSceneRenderer::renderMeshImplementation(const MeshPrimitive& primitiv
 
             // Next, compute distance of each face from the camera along the viewing direction (=camera z-axis).
             std::vector<FloatType> distances(mesh.faceCount());
-            boost::transform(faceCenters, distances.begin(), [direction = direction.toDataType<float>()](const Vector_3<float>& v) {
+            boost::transform(faceCenters, distances.begin(), [direction = direction.toDataType<float>()](const Vector3F& v) {
                 return direction.dot(v);
             });
 
             // Create index array with all face indices.
             std::vector<uint32_t> sortedIndices(mesh.faceCount());
-            std::iota(sortedIndices.begin(), sortedIndices.end(), (uint32_t)0);
+            boost::algorithm::iota(sortedIndices, (uint32_t)0);
 
             // Sort face indices with respect to distance (back-to-front order).
-            std::sort(sortedIndices.begin(), sortedIndices.end(), [&](uint32_t a, uint32_t b) {
+            boost::sort(sortedIndices, [&](uint32_t a, uint32_t b) {
                 return distances[a] < distances[b];
             });
 
@@ -271,27 +279,35 @@ void OpenGLSceneRenderer::renderMeshImplementation(const MeshPrimitive& primitiv
         // Render the mesh instances in back-to-front order.
 
         // Viewing direction in object space:
-        const Vector3 direction = modelViewTM().inverse().column(2);
+        const Vector3 direction = modelViewTM().linear().inverse().column(2);
+
+        // Coarsen the direction vector's precision for real-time rendering to reduce the frequency at
+        // which the particles must be reordered when moving the camera.
+        Vector3 coarseDirection = direction;
+        if(frameGraph()->isInteractive()) {
+            for(size_t dim = 0; dim < 3; dim++)
+                coarseDirection[dim] = std::round(2 * direction[dim]);
+        }
 
         // The caching key for the ordering of the primitives.
-        RendererResourceKey<struct OrderingCache, ConstDataBufferPtr, Vector3> orderingCacheKey{ primitive.perInstanceTMs(), direction };
+        RendererResourceKey<struct OrderingCache, ConstDataBufferPtr, Vector3> orderingCacheKey{ primitive.perInstanceTMs(), coarseDirection };
 
         // Render the primitives.
-        shader.drawReordered(GL_TRIANGLES, std::move(orderingCacheKey), [&](span<GLuint> sortedIndices) {
+        shader.drawReordered(GL_TRIANGLES, std::move(orderingCacheKey), [&](std::span<GLuint> sortedIndices) {
             OVITO_ASSERT(sortedIndices.size() == shader.instanceCount());
 
             // First, compute distance of each instance from the camera along the viewing direction (=camera z-axis).
             std::vector<GraphicsFloatType> distances(sortedIndices.size());
             if(primitive.perInstanceTMs()->dataType() == DataBuffer::Float32) {
-                const Vector_3<float> directionFloat = direction.toDataType<float>();
-                boost::transform(sortedIndices, distances.begin(), [directionFloat, tmArray = BufferReadAccess<AffineTransformationT<float>>(primitive.perInstanceTMs())](size_t i) {
+                const Vector3F directionFloat = direction.toDataType<float>();
+                std::transform(sortedIndices.begin(), sortedIndices.end(), distances.begin(), [directionFloat, tmArray = BufferReadAccess<AffineTransformationT<float>>(primitive.perInstanceTMs())](size_t i) {
                     return directionFloat.dot(tmArray[i].translation());
                 });
             }
             else {
                 // Viewing direction in object space:
                 const Vector_3<double> directionDouble = direction.toDataType<double>();
-                boost::transform(sortedIndices, distances.begin(), [directionDouble, tmArray = BufferReadAccess<AffineTransformationT<double>>(primitive.perInstanceTMs())](size_t i) {
+                std::transform(sortedIndices.begin(), sortedIndices.end(), distances.begin(), [directionDouble, tmArray = BufferReadAccess<AffineTransformationT<double>>(primitive.perInstanceTMs())](size_t i) {
                     return directionDouble.dot(tmArray[i].translation());
                 });
             }
@@ -307,16 +323,15 @@ void OpenGLSceneRenderer::renderMeshImplementation(const MeshPrimitive& primitiv
     }
 
     // Unbind color mapping texture.
-    if(colorMapTexture) {
+    if(colorMapTexture)
         colorMapTexture->release();
-    }
 }
 
 /******************************************************************************
 * Prepares the OpenGL buffer with the per-instance transformation matrices for
 * rendering a set of meshes.
 ******************************************************************************/
-QOpenGLBuffer OpenGLSceneRenderer::getMeshInstanceTMBuffer(const MeshPrimitive& primitive, OpenGLShaderHelper& shader)
+QOpenGLBuffer OpenGLRenderingJob::getMeshInstanceTMBuffer(const MeshPrimitive& primitive, OpenGLShaderHelper& shader)
 {
     OVITO_ASSERT(primitive.useInstancedRendering());
     OVITO_ASSERT(primitive.perInstanceTMs());
@@ -325,9 +340,9 @@ QOpenGLBuffer OpenGLSceneRenderer::getMeshInstanceTMBuffer(const MeshPrimitive&
     RendererResourceKey<struct InstanceTMCache, ConstDataBufferPtr> cacheKey(primitive.perInstanceTMs());
 
     // Upload the per-instance TMs to GPU memory.
-    return shader.createCachedBuffer(std::move(cacheKey), 3 * sizeof(Vector_4<float>), QOpenGLBuffer::VertexBuffer, OpenGLShaderHelper::PerInstance, [&](void* buffer, BufferReadAccess<int32_t> subset) {
+    return shader.createCachedBuffer(std::move(cacheKey), 3 * sizeof(Vector4F), QOpenGLBuffer::VertexBuffer, OpenGLShaderHelper::PerInstance, [&](void* buffer, BufferReadAccess<int32_t> subset) {
         OVITO_ASSERT(!subset);
-        Vector_4<float>* row = reinterpret_cast<Vector_4<float>*>(buffer);
+        Vector4F* row = reinterpret_cast<Vector4F*>(buffer);
         if(primitive.perInstanceTMs()->dataType() == DataBuffer::Float32) {
             for(const AffineTransformationT<float>& tm : BufferReadAccess<AffineTransformationT<float>>(primitive.perInstanceTMs())) {
                 *row++ = tm.row(0);
@@ -348,47 +363,75 @@ QOpenGLBuffer OpenGLSceneRenderer::getMeshInstanceTMBuffer(const MeshPrimitive&
 /******************************************************************************
 * Generates the wireframe line elements for the visible edges of a mesh.
 ******************************************************************************/
-ConstDataBufferPtr OpenGLSceneRenderer::generateMeshWireframeLines(const MeshPrimitive& primitive)
+ConstDataBufferPtr OpenGLRenderingJob::generateMeshWireframeLines(const MeshPrimitive& primitive)
 {
     OVITO_ASSERT(primitive.emphasizeEdges());
     OVITO_ASSERT(primitive.mesh());
 
     // Cache the wireframe geometry generated for the current mesh.
-    RendererResourceKey<struct WireframeCache, DataOORef<const TriangleMesh>> cacheKey(primitive.mesh());
-    ConstDataBufferPtr& wireframeLines = OpenGLResourceManager::instance()->lookup<ConstDataBufferPtr>(std::move(cacheKey), currentResourceFrame());
-
-    if(!wireframeLines) {
-        wireframeLines = primitive.generateWireframeLines();
-    }
-
-    return wireframeLines;
+    return currentResourceFrame().lookup<ConstDataBufferPtr>(
+        RendererResourceKey<struct WireframeCache, DataOORef<const TriangleMesh>>{ primitive.mesh() },
+        [&](ConstDataBufferPtr& wireframeLines) {
+            wireframeLines = primitive.generateWireframeLines();
+        });
 }
 
 /******************************************************************************
 * Renders just the edges of a triangle mesh as a wireframe model.
 ******************************************************************************/
-void OpenGLSceneRenderer::renderMeshWireframeImplementation(const MeshPrimitive& primitive)
+void OpenGLRenderingJob::renderMeshWireframeImplementation(const MeshPrimitive& primitive, bool renderAsTriangles)
 {
     OVITO_ASSERT(!isPickingPass());
 
     OpenGLShaderHelper shader(this);
-    if(!primitive.useInstancedRendering())
-        shader.load("mesh_wireframe", "mesh/mesh_wireframe.vert", "mesh/mesh_wireframe.frag");
-    else
-        shader.load("mesh_wireframe_instanced", "mesh/mesh_wireframe_instanced.vert", "mesh/mesh_wireframe_instanced.frag");
+    if(!renderAsTriangles) {
+        if(!primitive.useInstancedRendering())
+            shader.load("mesh_wireframe", "mesh/mesh_wireframe.vert", "mesh/mesh_wireframe.frag");
+        else
+            shader.load("mesh_wireframe_instanced", "mesh/mesh_wireframe_instanced.vert", "mesh/mesh_wireframe_instanced.frag");
+    }
+    else {
+        if(!primitive.useInstancedRendering())
+            shader.load("mesh_wireframe_tri", "mesh/mesh_wireframe_tri.vert", "mesh/mesh_wireframe_tri.frag");
+        else
+            shader.load("mesh_wireframe_tri_instanced", "mesh/mesh_wireframe_tri_instanced.vert", "mesh/mesh_wireframe_tri_instanced.frag");
+    }
 
     bool useBlending = (primitive.uniformColor().a() < 1.0) && !orderIndependentTransparency();
-    if(useBlending) shader.enableBlending();
+    if(useBlending)
+        shader.enableBlending();
 
     // Pass uniform line color to fragment shader as a uniform value.
     ColorA wireframeColor(0.1, 0.1, 0.1, primitive.uniformColor().a());
     shader.setUniformValue("color", wireframeColor);
 
-    // Get the wireframe lines geometry.
+    // Get the wireframe lines geometry data.
     ConstDataBufferPtr wireframeLinesBuffer = generateMeshWireframeLines(primitive);
+    static_assert(std::is_same_v<Point3G, Point3F>);
+
+    // When rendering the lines as pairs of triangles, we need to replicate the line data 6 times,
+    // because each line segments is rendered using 6 triangle vertices.
+    if(renderAsTriangles) {
+        int orgCount = wireframeLinesBuffer->size();
+        wireframeLinesBuffer = currentResourceFrame().lookup<ConstDataBufferPtr>(
+            RendererResourceKey<struct WireframeTriangleDataCache, ConstDataBufferPtr>{ wireframeLinesBuffer },
+            [&](ConstDataBufferPtr& replicatedBuffer) {
+                BufferFactory<Point3G> replicatedAccess(6 * wireframeLinesBuffer->size());
+                BufferReadAccess<Point3G> originalAccess(wireframeLinesBuffer);
+                auto pout = replicatedAccess.begin();
+                for(auto pin = originalAccess.cbegin(); pin != originalAccess.cend(); pin += 2) {
+                    for(int i = 0; i < 6; i++) {
+                        *pout++ = pin[0];
+                        *pout++ = pin[1];
+                    }
+                }
+                replicatedBuffer = replicatedAccess.take();
+            });
+    }
+
+    // Configure shader.
     shader.setVerticesPerInstance(wireframeLinesBuffer->size());
     shader.setInstanceCount(primitive.useInstancedRendering() ? primitive.perInstanceTMs()->size() : 1);
-
     if(shader.verticesPerInstance() > std::numeric_limits<int32_t>::max() / shader.instanceCount() / wireframeLinesBuffer->stride()) {
         qWarning() << "WARNING: OpenGL renderer - Wireframe mesh consists of too many lines, exceeding device limits (verts per instance:" << shader.verticesPerInstance() << ", instance count:" << shader.instanceCount() << ", stride:" << wireframeLinesBuffer->stride() << ").";
         return;
@@ -396,20 +439,32 @@ void OpenGLSceneRenderer::renderMeshWireframeImplementation(const MeshPrimitive&
 
     // Bind vertex buffer for wireframe vertex positions.
     QOpenGLBuffer buffer = shader.uploadDataBuffer(wireframeLinesBuffer, OpenGLShaderHelper::PerVertex);
-    shader.bindBuffer(buffer, "position", GL_FLOAT, 3, sizeof(Point_3<float>), 0, OpenGLShaderHelper::PerVertex);
+    if(!renderAsTriangles) {
+        shader.bindBuffer(buffer, "position", GL_FLOAT, 3, sizeof(Point3F), 0, OpenGLShaderHelper::PerVertex);
+    }
+    else {
+        shader.bindBuffer(buffer, "position_from", GL_FLOAT, 3, 2 * sizeof(Point3F), 0, OpenGLShaderHelper::PerVertex);
+        shader.bindBuffer(buffer, "position_to", GL_FLOAT, 3, 2 * sizeof(Point3F), sizeof(Point3F), OpenGLShaderHelper::PerVertex);
+        // Compute line width in viewport space.
+        constexpr float lineWidth = 1.0f;
+        shader.setUniformValue("line_thickness", lineWidth / framebufferSize().height());
+    }
 
     // Bind vertex buffer for instance TMs.
     if(primitive.useInstancedRendering()) {
         // Upload the per-instance TMs to GPU memory.
         QOpenGLBuffer instanceTMBuffer = getMeshInstanceTMBuffer(primitive, shader);
         // Bind buffer with the instance matrices.
-        shader.bindBuffer(instanceTMBuffer, "instance_tm_row1", GL_FLOAT, 4, 3 * sizeof(Vector_4<float>), 0 * sizeof(Vector_4<float>), OpenGLShaderHelper::PerInstance);
-        shader.bindBuffer(instanceTMBuffer, "instance_tm_row2", GL_FLOAT, 4, 3 * sizeof(Vector_4<float>), 1 * sizeof(Vector_4<float>), OpenGLShaderHelper::PerInstance);
-        shader.bindBuffer(instanceTMBuffer, "instance_tm_row3", GL_FLOAT, 4, 3 * sizeof(Vector_4<float>), 2 * sizeof(Vector_4<float>), OpenGLShaderHelper::PerInstance);
+        shader.bindBuffer(instanceTMBuffer, "instance_tm_row1", GL_FLOAT, 4, 3 * sizeof(Vector4F), 0 * sizeof(Vector4F), OpenGLShaderHelper::PerInstance);
+        shader.bindBuffer(instanceTMBuffer, "instance_tm_row2", GL_FLOAT, 4, 3 * sizeof(Vector4F), 1 * sizeof(Vector4F), OpenGLShaderHelper::PerInstance);
+        shader.bindBuffer(instanceTMBuffer, "instance_tm_row3", GL_FLOAT, 4, 3 * sizeof(Vector4F), 2 * sizeof(Vector4F), OpenGLShaderHelper::PerInstance);
     }
 
     // Draw lines.
-    shader.draw(GL_LINES);
+    if(!renderAsTriangles)
+        shader.draw(GL_LINES);
+    else
+        shader.draw(GL_TRIANGLES);
 }
 
 }   // End of namespace
