@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -21,9 +21,10 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 
 #include <ovito/core/Core.h>
+#include <ovito/core/rendering/ParticlePrimitive.h>
 #include <ovito/core/utilities/SortZipped.h>
 #include <ovito/core/utilities/concurrent/ParallelFor.h>
-#include "OpenGLSceneRenderer.h"
+#include "OpenGLRenderingJob.h"
 #include "OpenGLShaderHelper.h"
 
 namespace Ovito {
@@ -31,7 +32,7 @@ namespace Ovito {
 /******************************************************************************
 * Renders a set of particles.
 ******************************************************************************/
-void OpenGLSceneRenderer::renderParticlesImplementation(const ParticlePrimitive& primitive)
+void OpenGLRenderingJob::renderParticlesImplementation(const ParticlePrimitive& primitive, const FrameGraph::RenderingCommand& command)
 {
     // Make sure there is something to be rendered. Otherwise, step out early.
     if(!primitive.positions() || primitive.positions()->size() == 0)
@@ -39,9 +40,6 @@ void OpenGLSceneRenderer::renderParticlesImplementation(const ParticlePrimitive&
     if(primitive.indices() && primitive.indices()->size() == 0)
         return;
 
-    rebindVAO();
-    OVITO_REPORT_OPENGL_ERRORS(this);
-
     // Activate the right OpenGL shader program.
     OpenGLShaderHelper shader(this);
     switch(primitive.particleShape()) {
@@ -210,7 +208,7 @@ void OpenGLSceneRenderer::renderParticlesImplementation(const ParticlePrimitive&
         shader.enableSubsetRendering(primitive.indices());
 
     // Check VBO size limits.
-    if(shader.instanceCount() > std::numeric_limits<int32_t>::max() / shader.verticesPerInstance() / sizeof(Vector_4<float>)) {
+    if(shader.instanceCount() > std::numeric_limits<int32_t>::max() / shader.verticesPerInstance() / sizeof(Vector4F)) {
         qWarning() << "WARNING: OpenGL renderer - Trying to render too many particles at once, exceeding device limits.";
         return;
     }
@@ -222,13 +220,13 @@ void OpenGLSceneRenderer::renderParticlesImplementation(const ParticlePrimitive&
 
     // Pass picking base ID to shader.
     if(isPickingPass()) {
-        shader.setPickingBaseId(registerSubObjectIDs(primitive.positions()->size()/*, primitive.indices()*/));
+        shader.setPickingBaseId(objectPickingMap()->allocateObjectPickingIDs(command, primitive.positions()->size()));
     }
     OVITO_REPORT_OPENGL_ERRORS(this);
 
     // Upload particle coordinates.
     QOpenGLBuffer positionsBuffer = shader.uploadDataBuffer(primitive.positions(), OpenGLShaderHelper::PerInstance);
-    shader.bindBuffer(positionsBuffer, "position", GL_FLOAT, 3, sizeof(Point_3<float>), 0, OpenGLShaderHelper::PerInstance);
+    shader.bindBuffer(positionsBuffer, "position", GL_FLOAT, 3, sizeof(Point3F), 0, OpenGLShaderHelper::PerInstance);
 
     // Upload particle radii.
     if(primitive.radii()) {
@@ -244,7 +242,7 @@ void OpenGLSceneRenderer::renderParticlesImplementation(const ParticlePrimitive&
         // Upload particle colors.
         if(primitive.colors()) {
             QOpenGLBuffer colorsBuffer = shader.uploadDataBuffer(primitive.colors(), OpenGLShaderHelper::PerInstance);
-            shader.bindBuffer(colorsBuffer, "color", GL_FLOAT, 3, sizeof(Point_3<float>), 0, OpenGLShaderHelper::PerInstance);
+            shader.bindBuffer(colorsBuffer, "color", GL_FLOAT, 3, sizeof(Point3F), 0, OpenGLShaderHelper::PerInstance);
         }
         else {
             shader.unbindBuffer("color");
@@ -278,7 +276,7 @@ void OpenGLSceneRenderer::renderParticlesImplementation(const ParticlePrimitive&
         // Upload aspherical shapes.
         if(primitive.asphericalShapes()) {
             QOpenGLBuffer asphericalShapesBuffer = shader.uploadDataBuffer(primitive.asphericalShapes(), OpenGLShaderHelper::PerInstance);
-            shader.bindBuffer(asphericalShapesBuffer, "aspherical_shape", GL_FLOAT, 3, sizeof(Vector_3<float>), 0, OpenGLShaderHelper::PerInstance);
+            shader.bindBuffer(asphericalShapesBuffer, "aspherical_shape", GL_FLOAT, 3, sizeof(Vector3F), 0, OpenGLShaderHelper::PerInstance);
         }
         else {
             shader.unbindBuffer("aspherical_shape");
@@ -288,7 +286,7 @@ void OpenGLSceneRenderer::renderParticlesImplementation(const ParticlePrimitive&
         // Upload orientations.
         if(primitive.orientations()) {
             QOpenGLBuffer orientationsBuffer = shader.uploadDataBuffer(primitive.orientations(), OpenGLShaderHelper::PerInstance);
-            shader.bindBuffer(orientationsBuffer, "orientation", GL_FLOAT, 4, sizeof(QuaternionT<float>), 0, OpenGLShaderHelper::PerInstance);
+            shader.bindBuffer(orientationsBuffer, "orientation", GL_FLOAT, 4, sizeof(QuaternionF), 0, OpenGLShaderHelper::PerInstance);
         }
         else {
             shader.unbindBuffer("orientation");
@@ -301,7 +299,7 @@ void OpenGLSceneRenderer::renderParticlesImplementation(const ParticlePrimitive&
         // Upload roundness values.
         if(primitive.roundness()) {
             QOpenGLBuffer roundnessBuffer = shader.uploadDataBuffer(primitive.roundness(), OpenGLShaderHelper::PerInstance);
-            shader.bindBuffer(roundnessBuffer, "roundness", GL_FLOAT, 2, sizeof(Vector_2<float>), 0, OpenGLShaderHelper::PerInstance);
+            shader.bindBuffer(roundnessBuffer, "roundness", GL_FLOAT, 2, sizeof(Vector2F), 0, OpenGLShaderHelper::PerInstance);
         }
         else {
             shader.unbindBuffer("roundness");
@@ -318,12 +316,12 @@ void OpenGLSceneRenderer::renderParticlesImplementation(const ParticlePrimitive&
         OVITO_ASSERT(!isPickingPass() && !orderIndependentTransparency());
 
         // Viewing direction in object space:
-        const Vector3 direction = modelViewTM().inverse().column(2);
+        const Vector3 direction = modelViewTM().linear().inverse().column(2);
 
-        // Coarsen the direction vector's precision to reduce the frequency at
+        // Coarsen the direction vector's precision for real-time rendering to reduce the frequency at
         // which the particles must be reordered when moving the camera.
         Vector3 coarseDirection = direction;
-        if(isInteractive()) {
+        if(frameGraph()->isInteractive()) {
             for(size_t dim = 0; dim < 3; dim++)
                 coarseDirection[dim] = std::round(2 * direction[dim]);
         }
@@ -336,27 +334,17 @@ void OpenGLSceneRenderer::renderParticlesImplementation(const ParticlePrimitive&
         };
 
         // Render primitives.
-        shader.drawReordered(GL_TRIANGLE_STRIP, std::move(orderingCacheKey), [&](span<GLuint> sortedIndices) {
+        shader.drawReordered(GL_TRIANGLE_STRIP, std::move(orderingCacheKey), [&](std::span<GLuint> sortedIndices) {
 
             // First, compute distance of each particle from the camera along the viewing direction (=camera z-axis).
             std::vector<GraphicsFloatType> distances(sortedIndices.size());
             if(primitive.positions()->dataType() == DataBuffer::Float64) {
-                if(sortedIndices.size() < 10000) {
-                    // Serial code version (used for small datasets).
-                    boost::transform(sortedIndices, distances.begin(), [direction = direction.toDataType<double>(), positionsArray = BufferReadAccess<Vector_3<double>>(primitive.positions())](size_t i) {
-                        return static_cast<GraphicsFloatType>(direction.dot(positionsArray[i]));
-                    });
-                }
-                else {
-                    // Parallelized code version.
-                    BufferReadAccess<Vector_3<double>> positionsArray(primitive.positions());
-                    parallelFor(sortedIndices.size(), [&, direction = direction.toDataType<double>()](size_t i) {
-                        distances[i] = static_cast<GraphicsFloatType>(direction.dot(positionsArray[sortedIndices[i]]));
-                    });
-                }
+                std::transform(sortedIndices.begin(), sortedIndices.end(), distances.begin(), [direction = direction.toDataType<double>(), positionsArray = BufferReadAccess<Vector_3<double>>(primitive.positions())](size_t i) {
+                    return static_cast<GraphicsFloatType>(direction.dot(positionsArray[i]));
+                });
             }
             else if(primitive.positions()->dataType() == DataBuffer::Float32) {
-                boost::transform(sortedIndices, distances.begin(), [direction = direction.toDataType<float>(), positionsArray = BufferReadAccess<Vector_3<float>>(primitive.positions())](size_t i) {
+                std::transform(sortedIndices.begin(), sortedIndices.end(), distances.begin(), [direction = direction.toDataType<float>(), positionsArray = BufferReadAccess<Vector3F>(primitive.positions())](size_t i) {
                     return static_cast<GraphicsFloatType>(direction.dot(positionsArray[i]));
                 });
             }
@@ -366,8 +354,6 @@ void OpenGLSceneRenderer::renderParticlesImplementation(const ParticlePrimitive&
             Ovito::sort_zipped(distances, sortedIndices);
         });
     }
-
-    OVITO_REPORT_OPENGL_ERRORS(this);
 }
 
 }   // End of namespace
