@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -29,19 +29,16 @@ namespace Ovito {
 /******************************************************************************
 * Loads a shader program.
 ******************************************************************************/
-void OpenGLShaderHelper::load(const QString& id, const QString& vertexShaderFile, const QString& fragmentShaderFile, const QString& geometryShaderFile)
+void OpenGLShaderHelper::load(const QString& id, const QString& vertexShaderFile, const QString& fragmentShaderFile, const QString& geometryShaderFile, const QString& shaderPathPrefix)
 {
     if(_shader)
         _shader->release();
 
-    // Prepend this to paths when loading GLSL shaders from the resource file.
-    QString prefix = QStringLiteral(":/openglrenderer/glsl/");
-
     // Compile the shader program.
     _shader = _renderer->loadShaderProgram(id,
-        prefix + vertexShaderFile,
-        prefix + fragmentShaderFile,
-        !geometryShaderFile.isEmpty() ? prefix + geometryShaderFile : QString());
+        shaderPathPrefix + vertexShaderFile,
+        shaderPathPrefix + fragmentShaderFile,
+        !geometryShaderFile.isEmpty() ? (shaderPathPrefix + geometryShaderFile) : QString());
     OVITO_REPORT_OPENGL_ERRORS(_renderer);
 
     // Are we using a geometry shader?
@@ -49,18 +46,28 @@ void OpenGLShaderHelper::load(const QString& id, const QString& vertexShaderFile
 
     // Bind the OpenGL shader program.
     if(!_shader->bind())
-        throw SceneRenderer::RendererException(QStringLiteral("Failed to bind OpenGL shader '%1'.").arg(id));
+        throw RendererException(QStringLiteral("Failed to bind OpenGL shader '%1'.").arg(id));
     OVITO_REPORT_OPENGL_ERRORS(_renderer);
 
     // Set shader uniforms.
-    OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("modelview_projection_matrix", static_cast<QMatrix4x4>(_renderer->projParams().projectionMatrix * _renderer->modelViewTM())));
-    OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("projection_matrix", static_cast<QMatrix4x4>(_renderer->projParams().projectionMatrix)));
-    OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("inverse_projection_matrix", static_cast<QMatrix4x4>(_renderer->projParams().inverseProjectionMatrix)));
-    OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("modelview_matrix", static_cast<QMatrix4x4>(Matrix4(_renderer->modelViewTM()))));
-    Matrix3 normalTM;
-    if(!_renderer->modelViewTM().linear().inverse(normalTM))
-        normalTM.setIdentity();
-    OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("normal_tm", static_cast<QMatrix4x4>(Matrix4(normalTM.transposed()))));
+    if(!_renderer->_preprojectedCoordinates) {
+        const ViewProjectionParameters& projParams = _renderer->frameGraph()->projectionParams();
+        OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("modelview_projection_matrix", static_cast<QMatrix4x4>(projParams.projectionMatrix * _renderer->modelViewTM())));
+        OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("projection_matrix", static_cast<QMatrix4x4>(projParams.projectionMatrix)));
+        OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("inverse_projection_matrix", static_cast<QMatrix4x4>(projParams.inverseProjectionMatrix)));
+        OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("modelview_matrix", static_cast<QMatrix4x4>(Matrix4(_renderer->modelViewTM()))));
+        Matrix3 normalTM;
+        if(!_renderer->modelViewTM().linear().inverse(normalTM))
+            normalTM.setIdentity();
+        OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("normal_tm", static_cast<QMatrix4x4>(Matrix4(normalTM.transposed()))));
+    }
+    else {
+        OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("modelview_projection_matrix", QMatrix4x4{}));
+        OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("projection_matrix", QMatrix4x4{}));
+        OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("inverse_projection_matrix", QMatrix4x4{}));
+        OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("modelview_matrix", QMatrix4x4{}));
+        OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("normal_tm", QMatrix4x4{}));
+    }
 
     // Set up uniform constant data arrays.
     int unitCubeTriangleStripUniform = _shader->uniformLocation("unit_cube_triangle_strip");
@@ -139,9 +146,9 @@ void OpenGLShaderHelper::load(const QString& id, const QString& vertexShaderFile
     }
 
     // Get current viewport rectangle.
-    const QRect& vpRect = _renderer->viewportRect();
-    OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("viewport_origin", (float)vpRect.x(), (float)vpRect.y()));
-    OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("inverse_viewport_size", 2.0f / vpRect.width(), 2.0f / vpRect.height()));
+    const QSize& vpSize = _renderer->framebufferSize();
+    OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("viewport_origin", 0.0f, 0.0f));
+    OVITO_CHECK_OPENGL(_renderer, _shader->setUniformValue("inverse_viewport_size", 2.0f / vpSize.width(), 2.0f / vpSize.height()));
 
     // Need to render only the front-facing sides of the geometry.
     OVITO_CHECK_OPENGL(_renderer, _renderer->glCullFace(GL_BACK));
@@ -184,7 +191,7 @@ QOpenGLBuffer OpenGLShaderHelper::createCachedBufferImpl(GLsizei elementSize, QO
     QOpenGLBuffer bufferObject(usage);
     bufferObject.setUsagePattern(QOpenGLBuffer::StaticDraw);
     if(!bufferObject.create())
-        throw SceneRenderer::RendererException(QStringLiteral("Failed to create OpenGL buffer object."));
+        throw RendererException(QStringLiteral("Failed to create OpenGL buffer object."));
 
     if(!bufferObject.bind()) {
         qWarning() << "QOpenGLBuffer::bind() failed in function OpenGLShaderHelper::createCachedBufferImpl()";
@@ -218,7 +225,7 @@ QOpenGLBuffer OpenGLShaderHelper::createCachedBufferImpl(GLsizei elementSize, QO
 
     // Check if the requested vertex buffer size exceeds limits.
     if(exceedsLimits)
-        throw SceneRenderer::RendererException(QStringLiteral("OpenGL buffer allocation failed, because requested size exceeds limits."));
+        throw RendererException(QStringLiteral("OpenGL buffer allocation failed, because requested size exceeds limits."));
 
     // Allocate the buffer memory.
     bufferObject.allocate(bufferSize);
@@ -227,7 +234,7 @@ QOpenGLBuffer OpenGLShaderHelper::createCachedBufferImpl(GLsizei elementSize, QO
     // Fill the buffer with data.
     void* p = bufferObject.map(QOpenGLBuffer::WriteOnly);
     if(p == nullptr)
-        throw SceneRenderer::RendererException(QStringLiteral("Failed to map memory of newly created OpenGL buffer object of size %1 bytes.").arg(bufferSize));
+        throw RendererException(QStringLiteral("Failed to map memory of newly created OpenGL buffer object of size %1 bytes.").arg(bufferSize));
 #else
     // WebGL 1/OpenGL ES 2.0 does not support mapping a GL buffer to memory.
     // Need to emulate the map() method by providing a temporary memory buffer on the host.
@@ -295,7 +302,7 @@ QOpenGLBuffer OpenGLShaderHelper::uploadDataBuffer(const ConstDataBufferPtr& dat
 
     if(uploadDataTypeSize == 0) {
         OVITO_ASSERT(false);
-        throw SceneRenderer::RendererException(QStringLiteral("Cannot create OpenGL buffer object for DataBuffer with data type %1.").arg(dataBuffer->dataType()));
+        throw RendererException(QStringLiteral("Cannot create OpenGL buffer object for DataBuffer with data type %1.").arg(dataBuffer->dataType()));
     }
 
     GLsizei elementSize = 0;
@@ -317,7 +324,7 @@ QOpenGLBuffer OpenGLShaderHelper::uploadDataBuffer(const ConstDataBufferPtr& dat
 
     if(elementSize == 0) {
         OVITO_ASSERT(false);
-        throw SceneRenderer::RendererException(QStringLiteral("Cannot create OpenGL buffer object for DataBuffer with data type %1.").arg(dataBuffer->dataType()));
+        throw RendererException(QStringLiteral("Cannot create OpenGL buffer object for DataBuffer with data type %1.").arg(dataBuffer->dataType()));
     }
 
     // Create an OpenGL buffer object and fill it with the data from the DataBuffer.
@@ -366,7 +373,7 @@ void OpenGLShaderHelper::bindBuffer(QOpenGLBuffer& buffer, const char* attribute
     int attrIndex = _shader->attributeLocation(attributeName);
     if(attrIndex < 0) {
         qWarning() << "OpenGLShaderHelper::bindBuffer() failed for shader" << _shader->objectName() << ": attribute with name" << attributeName << "does not exist in shader.";
-        throw SceneRenderer::RendererException(QStringLiteral("Attribute with name %1 does not exist in OpenGL shader program '%2'.").arg(attributeName).arg(_shader->objectName()));
+        throw RendererException(QStringLiteral("Attribute with name %1 does not exist in OpenGL shader program '%2'.").arg(attributeName).arg(_shader->objectName()));
     }
     bindBuffer(buffer, attrIndex, type, tupleSize, stride, offset, inputRate);
 }
@@ -383,7 +390,7 @@ void OpenGLShaderHelper::bindBuffer(QOpenGLBuffer& buffer, int attrIndex, GLenum
     OVITO_ASSERT(buffer.isCreated());
     if(!buffer.bind()) {
         qWarning() << "OpenGLShaderHelper::bindBuffer() failed for shader" << _shader->objectName();
-        throw SceneRenderer::RendererException(QStringLiteral("Failed to bind OpenGL vertex buffer for shader '%1'.").arg(_shader->objectName()));
+        throw RendererException(QStringLiteral("Failed to bind OpenGL vertex buffer for shader '%1'.").arg(_shader->objectName()));
     }
     OVITO_CHECK_OPENGL(_renderer, _shader->setAttributeBuffer(attrIndex, type, offset, tupleSize, stride));
     OVITO_CHECK_OPENGL(_renderer, _shader->enableAttributeArray(attrIndex));
@@ -403,7 +410,7 @@ void OpenGLShaderHelper::unbindBuffer(const char* attributeName)
     int attrIndex = _shader->attributeLocation(attributeName);
     if(attrIndex < 0) {
         qWarning() << "OpenGLShaderHelper::unbindBuffer() failed for shader" << _shader->objectName() << ": attribute with name" << attributeName << "does not exist in shader.";
-        throw SceneRenderer::RendererException(QStringLiteral("Attribute with name %1 does not exist in OpenGL shader program '%2'.").arg(attributeName).arg(_shader->objectName()));
+        throw RendererException(QStringLiteral("Attribute with name %1 does not exist in OpenGL shader program '%2'.").arg(attributeName).arg(_shader->objectName()));
     }
     unbindBuffer(attrIndex);
 }
@@ -466,7 +473,7 @@ void OpenGLShaderHelper::draw(GLenum mode)
 
             // Bind index buffer.
             if(!ibo.bind())
-                throw SceneRenderer::RendererException(QStringLiteral("Failed to bind OpenGL index buffer for shader '%1'.").arg(shaderObject().objectName()));
+                throw RendererException(QStringLiteral("Failed to bind OpenGL index buffer for shader '%1'.").arg(shaderObject().objectName()));
 
             // Use native command for indexed drawing.
             OVITO_CHECK_OPENGL(_renderer, _renderer->glDrawElements(GL_POINTS, renderInstanceCount, GL_UNSIGNED_INT, nullptr));
@@ -479,50 +486,42 @@ void OpenGLShaderHelper::draw(GLenum mode)
             OVITO_ASSERT(_renderer->glversion() >= QT_VERSION_CHECK(4, 3, 0));
             OVITO_ASSERT(_renderer->glMultiDrawArraysIndirect != nullptr);
 
-            // The caching key for the indirect drawing buffers.
-            RendererResourceKey<struct IndirectDrawingCache, ConstDataBufferPtr, int> indirectDrawingCacheKey{
-                _instancesSubset,
-                verticesPerInstance()
-            };
-
             // Look up indirect drawing buffer.
-            QOpenGLBuffer& indirectBuffer =
-                OpenGLResourceManager::instance()->lookup<QOpenGLBuffer>(std::move(indirectDrawingCacheKey),
-                    _renderer->currentResourceFrame());
+            const QOpenGLBuffer& indirectBuffer = _renderer->currentResourceFrame().lookup<QOpenGLBuffer>(
+                RendererResourceKey<struct IndirectDrawingCache, ConstDataBufferPtr, int>{_instancesSubset, verticesPerInstance()},
+                [&](QOpenGLBuffer& indirectBuffer) {
+                    // Data structure used by the glMultiDrawArraysIndirect() command:
+                    struct DrawArraysIndirectCommand {
+                        GLuint count;
+                        GLuint instanceCount;
+                        GLuint first;
+                        GLuint baseInstance;
+                    };
 
-            // Data structure used by the glMultiDrawArraysIndirect() command:
-            struct DrawArraysIndirectCommand {
-                GLuint count;
-                GLuint instanceCount;
-                GLuint first;
-                GLuint baseInstance;
-            };
-
-            // Check if this indirect drawing buffer has already been uploaded to the GPU.
-            if(!indirectBuffer.isCreated()) {
-                indirectBuffer = createCachedBufferImpl(sizeof(DrawArraysIndirectCommand), static_cast<QOpenGLBuffer::Type>(GL_DRAW_INDIRECT_BUFFER), PerInstance, [&](void* buffer, BufferReadAccess<int32_t> subset) {
-                    OVITO_ASSERT(!subset);
-                    // Fill the buffer with DrawArraysIndirectCommand records.
-                    DrawArraysIndirectCommand* dst = reinterpret_cast<DrawArraysIndirectCommand*>(buffer);
-                    for(auto index : BufferReadAccess<int32_t>(_instancesSubset)) {
-                        dst->count = verticesPerInstance();
-                        dst->instanceCount = 1;
-                        dst->first = 0;
-                        dst->baseInstance = index;
-                        ++dst;
-                    }
+                    // Check if this indirect drawing buffer has already been uploaded to the GPU.
+                    indirectBuffer = createCachedBufferImpl(sizeof(DrawArraysIndirectCommand), static_cast<QOpenGLBuffer::Type>(GL_DRAW_INDIRECT_BUFFER), PerInstance, [&](void* buffer, BufferReadAccess<int32_t> subset) {
+                        OVITO_ASSERT(!subset);
+                        // Fill the buffer with DrawArraysIndirectCommand records.
+                        DrawArraysIndirectCommand* dst = reinterpret_cast<DrawArraysIndirectCommand*>(buffer);
+                        for(auto index : BufferReadAccess<int32_t>(_instancesSubset)) {
+                            dst->count = verticesPerInstance();
+                            dst->instanceCount = 1;
+                            dst->first = 0;
+                            dst->baseInstance = index;
+                            ++dst;
+                        }
+                    });
                 });
-            }
 
             // Bind the indirect drawing GL buffer.
-            if(!indirectBuffer.bind())
-                throw SceneRenderer::RendererException(QStringLiteral("Failed to bind OpenGL indirect drawing buffer for shader '%1'.").arg(shaderObject().objectName()));
+            if(!const_cast<QOpenGLBuffer&>(indirectBuffer).bind())
+                throw RendererException(QStringLiteral("Failed to bind OpenGL indirect drawing buffer for shader '%1'.").arg(shaderObject().objectName()));
 
             // Draw instances in indexed order.
             OVITO_CHECK_OPENGL(_renderer, _renderer->glMultiDrawArraysIndirect(mode, nullptr, renderInstanceCount, 0));
 
             // Done.
-            indirectBuffer.release();
+            const_cast<QOpenGLBuffer&>(indirectBuffer).release();
         }
         else if(usingInstancedArrays()) {
             // Use native command for instanced drawing.
@@ -535,95 +534,6 @@ void OpenGLShaderHelper::draw(GLenum mode)
     }
 }
 
-/******************************************************************************
-* Renders the primtives using a geometry shader in a specified order.
-******************************************************************************/
-void OpenGLShaderHelper::drawReorderedGeometryShader(QOpenGLBuffer& indexBuffer, std::function<void(span<GLuint>)>&& computeOrderingFunc)
-{
-    // The number of instances to be rendered.
-    GLsizei renderInstanceCount = _instancesSubset ? _instancesSubset->size() : instanceCount();
-
-    // Check if this index buffer has already been uploaded to the GPU.
-    if(!indexBuffer.isCreated()) {
-        indexBuffer = createCachedBufferImpl(sizeof(GLsizei), QOpenGLBuffer::IndexBuffer, PerInstance, [&](void* buffer, BufferReadAccess<int32_t> subset) {
-            OVITO_ASSERT(!subset);
-            auto sortedIndices = span(static_cast<GLuint*>(buffer), renderInstanceCount);
-            if(!_instancesSubset)
-                std::iota(sortedIndices.begin(), sortedIndices.end(), (GLuint)0);
-            else
-                boost::copy(BufferReadAccess<int32_t>(_instancesSubset), sortedIndices.begin());
-            // Call user function to generate the element ordering.
-            std::move(computeOrderingFunc)(sortedIndices);
-        });
-    }
-
-    // Bind index buffer.
-    if(!indexBuffer.bind())
-        throw SceneRenderer::RendererException(QStringLiteral("Failed to bind OpenGL index buffer for shader '%1'.").arg(shaderObject().objectName()));
-
-    // Draw point primitives in sorted order.
-    OVITO_CHECK_OPENGL(_renderer, _renderer->glDrawElements(GL_POINTS, renderInstanceCount, GL_UNSIGNED_INT, nullptr));
-
-    indexBuffer.release();
-}
-
-#ifndef Q_OS_WASM
-/******************************************************************************
-* Issues a drawing command with an ordering of the instances.
-******************************************************************************/
-void OpenGLShaderHelper::drawReorderedOpenGL4(GLenum mode, QOpenGLBuffer& indirectBuffer, std::function<void(span<GLuint>)>&& computeOrderingFunc)
-{
-    // On OpenGL 4.3+ contexts, use glMultiDrawArraysIndirect() to render the instances in a prescribed order.
-    OVITO_ASSERT(_renderer->glversion() >= QT_VERSION_CHECK(4, 3, 0));
-    OVITO_ASSERT(_renderer->glMultiDrawArraysIndirect != nullptr);
-
-    // The number of instances to be rendered.
-    GLsizei renderInstanceCount = _instancesSubset ? _instancesSubset->size() : instanceCount();
-
-    // Data structure used by the glMultiDrawArraysIndirect() command:
-    struct DrawArraysIndirectCommand {
-        GLuint count;
-        GLuint instanceCount;
-        GLuint first;
-        GLuint baseInstance;
-    };
-
-    // Check if this indirect drawing buffer has already been uploaded to the GPU.
-    if(!indirectBuffer.isCreated()) {
-        indirectBuffer = createCachedBufferImpl(sizeof(DrawArraysIndirectCommand), static_cast<QOpenGLBuffer::Type>(GL_DRAW_INDIRECT_BUFFER), PerInstance, [&](void* buffer, BufferReadAccess<int32_t> subset) {
-            OVITO_ASSERT(!subset);
-            auto sortedIndices = span(static_cast<GLuint*>(buffer), renderInstanceCount);
-            if(!_instancesSubset)
-                std::iota(sortedIndices.begin(), sortedIndices.end(), (GLuint)0);
-            else
-                boost::copy(BufferReadAccess<int32_t>(_instancesSubset), sortedIndices.begin());
-            // Call user function to generate the element ordering.
-            std::move(computeOrderingFunc)(sortedIndices);
-
-            // Fill the buffer with DrawArraysIndirectCommand records.
-            DrawArraysIndirectCommand* dst = reinterpret_cast<DrawArraysIndirectCommand*>(buffer);
-            for(auto index : sortedIndices) {
-                dst->count = verticesPerInstance();
-                dst->instanceCount = 1;
-                dst->first = 0;
-                dst->baseInstance = index;
-                ++dst;
-            }
-        });
-    }
-
-    // Bind the indirect drawing GL buffer.
-    if(!indirectBuffer.bind())
-        throw SceneRenderer::RendererException(QStringLiteral("Failed to bind OpenGL indirect drawing buffer for shader '%1'.").arg(shaderObject().objectName()));
-
-    // Draw instances in sorted order.
-    OVITO_CHECK_OPENGL(_renderer, _renderer->glMultiDrawArraysIndirect(mode, nullptr, renderInstanceCount, 0));
-
-    // Done.
-    indirectBuffer.release();
-}
-#endif
-
 /******************************************************************************
 * Makes the gl_VertexID and gl_InstanceID special variables available in
 * older OpenGL implementations.
@@ -634,35 +544,39 @@ void OpenGLShaderHelper::setupVertexAndInstanceIDOpenGL2()
         // In GLSL 1.20, the 'gl_VertexID' and 'gl_InstanceID' special variables are not available.
         // Then we have to emulate them by providing a buffer-backed vertex attribute named 'vertexID' instead.
 
-        std::pair<QOpenGLBuffer, GLsizei>& buf = OpenGLResourceManager::instance()->lookup<std::pair<QOpenGLBuffer, GLsizei>>(RendererResourceKey<struct VertexIDCache>{}, _renderer->currentResourceFrame());
+        // Store the VBO in the cache.
+        auto& [vbo, vboSize] = const_cast<std::tuple<QOpenGLBuffer, GLsizei>&>(_renderer->currentResourceFrame().lookup<std::tuple<QOpenGLBuffer, GLsizei>>(
+            RendererResourceKey<struct VertexIDCache>{},
+            [&](QOpenGLBuffer& vbo, GLsizei& vboSize) { vboSize = 0; }));
 
-        if(!buf.first.isCreated() || buf.second < verticesPerInstance() * instanceCount()) {
-            buf.second = verticesPerInstance() * instanceCount();
-            buf.first = QOpenGLBuffer(QOpenGLBuffer::VertexBuffer);
-            buf.first.setUsagePattern(QOpenGLBuffer::StaticDraw);
-            if(!buf.first.create() || !buf.first.bind())
-                throw SceneRenderer::RendererException(QStringLiteral("Failed to create OpenGL buffer object."));
-            OVITO_CHECK_OPENGL(_renderer, buf.first.allocate(buf.second * sizeof(float)));
+        // Recreate/resize the buffer if necessary.
+        if(!vbo.isCreated() || vboSize < verticesPerInstance() * instanceCount()) {
+            vboSize = verticesPerInstance() * instanceCount();
+            vbo = QOpenGLBuffer(QOpenGLBuffer::VertexBuffer);
+            vbo.setUsagePattern(QOpenGLBuffer::StaticDraw);
+            if(!vbo.create() || !vbo.bind())
+                throw RendererException(QStringLiteral("Failed to create OpenGL buffer object."));
+            OVITO_CHECK_OPENGL(_renderer, vbo.allocate(vboSize * sizeof(float)));
 #ifndef Q_OS_WASM
-            void* p = buf.first.map(QOpenGLBuffer::WriteOnly);
+            void* p = vbo.map(QOpenGLBuffer::WriteOnly);
             if(p == nullptr)
-                throw SceneRenderer::RendererException(QStringLiteral("Failed to map memory of newly created OpenGL vertexID buffer of size %1 bytes.").arg(buf.second * sizeof(float)));
-            std::iota(reinterpret_cast<float*>(p), reinterpret_cast<float*>(p) + buf.second, 0);
-            OVITO_CHECK_OPENGL(_renderer, buf.first.unmap());
+                throw RendererException(QStringLiteral("Failed to map memory of newly created OpenGL vertexID buffer of size %1 bytes.").arg(vboSize * sizeof(float)));
+            std::iota(reinterpret_cast<float*>(p), reinterpret_cast<float*>(p) + vboSize, 0);
+            OVITO_CHECK_OPENGL(_renderer, vbo.unmap());
 #else
             // WebGL 1/OpenGL ES 2.0 does not support mapping a GL buffer to memory.
             // Need to emulate the map() method by providing a temporary memory buffer on the host.
-            std::vector<float> stagingBuffer(buf.second);
-            std::iota(stagingBuffer.begin(), stagingBuffer.end(), 0);
-            OVITO_CHECK_OPENGL(_renderer, buf.first.write(0, stagingBuffer.data(), buf.second * sizeof(float)));
+            std::vector<float> stagingBuffer(vboSize);
+            boost::algorithm::iota(stagingBuffer, 0);
+            OVITO_CHECK_OPENGL(_renderer, vbo.write(0, stagingBuffer.data(), vboSize * sizeof(float)));
 #endif
         }
         else {
-            OVITO_CHECK_OPENGL(_renderer, buf.first.bind());
+            OVITO_CHECK_OPENGL(_renderer, vbo.bind());
         }
         OVITO_CHECK_OPENGL(_renderer, _shader->setAttributeBuffer("vertexID", GL_FLOAT, 0, 1, 0));
         OVITO_CHECK_OPENGL(_renderer, _shader->enableAttributeArray("vertexID"));
-        OVITO_CHECK_OPENGL(_renderer, buf.first.release());
+        OVITO_CHECK_OPENGL(_renderer, vbo.release());
     }
 
     // This is needed to compute the special shader variable 'gl_VertexID' correctly when instanced arrays are not supported:
@@ -672,7 +586,7 @@ void OpenGLShaderHelper::setupVertexAndInstanceIDOpenGL2()
 }
 
 /******************************************************************************
-* Implemention of the drawArrays() method for OpenGL 2.x.
+* Implementation of the drawArrays() method for OpenGL 2.x.
 ******************************************************************************/
 void OpenGLShaderHelper::drawOpenGL2(GLenum mode, GLsizei renderInstanceCount)
 {
@@ -686,20 +600,20 @@ void OpenGLShaderHelper::drawOpenGL2(GLenum mode, GLsizei renderInstanceCount)
     else if(renderInstanceCount > 1) {
         // Use glMultiDrawArrays() if available to draw all instances in one go.
         if(_renderer->glMultiDrawArrays) {
-            RendererResourceKey<struct IndexArrayCache, GLsizei, GLsizei> indexArrayCacheKey{ renderInstanceCount, verticesPerInstance() };
-            std::pair<std::vector<GLint>, std::vector<GLsizei>>& indexArrays = OpenGLResourceManager::instance()->lookup<std::pair<std::vector<GLint>, std::vector<GLsizei>>>(indexArrayCacheKey, _renderer->currentResourceFrame());
-            if(indexArrays.first.empty()) {
-                // Fill the two arrays needed for glMultiDrawArrays().
-                indexArrays.second.resize(renderInstanceCount, verticesPerInstance());
-                indexArrays.first.resize(renderInstanceCount);
-                GLint index = 0;
-                for(GLint& f : indexArrays.first)
-                    f = (index++) * verticesPerInstance();
-            }
-            OVITO_ASSERT(indexArrays.first.size() == renderInstanceCount);
-            OVITO_ASSERT(indexArrays.second.size() == renderInstanceCount);
-            OVITO_ASSERT(indexArrays.second.front() == verticesPerInstance());
-            OVITO_CHECK_OPENGL(_renderer, _renderer->glMultiDrawArrays(mode, indexArrays.first.data(), indexArrays.second.data(), renderInstanceCount));
+            const auto& [indexFirst, indexCount] = _renderer->currentResourceFrame().lookup<std::tuple<std::vector<GLint>, std::vector<GLsizei>>>(
+                RendererResourceKey<struct IndexArrayCache, GLsizei, GLsizei>{ renderInstanceCount, verticesPerInstance() },
+                [&](std::vector<GLint>& indexFirst, std::vector<GLsizei>& indexCount) {
+                    // Fill the two arrays needed for glMultiDrawArrays().
+                    indexCount.resize(renderInstanceCount, verticesPerInstance());
+                    indexFirst.resize(renderInstanceCount);
+                    GLint index = 0;
+                    for(GLint& f : indexFirst)
+                        f = (index++) * verticesPerInstance();
+                });
+            OVITO_ASSERT(indexFirst.size() == renderInstanceCount);
+            OVITO_ASSERT(indexCount.size() == renderInstanceCount);
+            OVITO_ASSERT(indexCount.front() == verticesPerInstance());
+            OVITO_CHECK_OPENGL(_renderer, _renderer->glMultiDrawArrays(mode, indexFirst.data(), indexCount.data(), renderInstanceCount));
         }
         else {
             // If glMultiDrawArrays() is not available, fall back to drawing each instance with an individual glDrawArrays() call.
@@ -713,7 +627,7 @@ void OpenGLShaderHelper::drawOpenGL2(GLenum mode, GLsizei renderInstanceCount)
 /******************************************************************************
 * Issues a drawing command with an ordering of the instances.
 ******************************************************************************/
-void OpenGLShaderHelper::drawReorderedOpenGL2or3(GLenum mode, std::pair<std::vector<GLint>, std::vector<GLsizei>>& indirectBuffers, std::function<void(span<GLuint>)>&& computeOrderingFunc)
+void OpenGLShaderHelper::drawReorderedOpenGL2or3(GLenum mode, std::pair<std::vector<GLint>, std::vector<GLsizei>>& indirectBuffers, std::function<void(std::span<GLuint>)>&& computeOrderingFunc)
 {
     // This method is called for OpenGL versions before 3.3, when glMultiDrawArraysIndirect() and instanced arrays are not available.
 
