@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -31,7 +31,6 @@
 #include <ovito/gui/desktop/properties/BooleanParameterUI.h>
 #include <ovito/gui/desktop/properties/VariantComboBoxParameterUI.h>
 #include <ovito/gui/desktop/dialogs/ImportFileDialog.h>
-#include <ovito/gui/desktop/utilities/concurrent/ProgressDialog.h>
 #include <ovito/gui/desktop/mainwin/MainWindow.h>
 #include <ovito/core/app/PluginManager.h>
 #include <ovito/core/dataset/io/FileSourceImporter.h>
@@ -40,7 +39,7 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(ParticleTypeEditor);
+IMPLEMENT_CREATABLE_OVITO_CLASS(ParticleTypeEditor);
 SET_OVITO_OBJECT_EDITOR(ParticleType, ParticleTypeEditor);
 
 /******************************************************************************
@@ -73,7 +72,7 @@ void ParticleTypeEditor::createUI(const RolloutInsertionParameters& rolloutParam
     });
 
     // Type name.
-    StringParameterUI* namePUI = new StringParameterUI(this, PROPERTY_FIELD(ParticleType::name));
+    StringParameterUI* namePUI = createParamUI<StringParameterUI>(PROPERTY_FIELD(ParticleType::name));
     gridLayout->addWidget(new QLabel(tr("Name:")), 1, 0);
     gridLayout->addWidget(namePUI->textBox(), 1, 1);
 
@@ -94,19 +93,19 @@ void ParticleTypeEditor::createUI(const RolloutInsertionParameters& rolloutParam
     layout1->addWidget(appearanceBox);
 
     // Display color parameter.
-    ColorParameterUI* colorPUI = new ColorParameterUI(this, PROPERTY_FIELD(ParticleType::color));
+    ColorParameterUI* colorPUI = createParamUI<ColorParameterUI>(PROPERTY_FIELD(ParticleType::color));
     gridLayout->addWidget(colorPUI->label(), 0, 0);
     gridLayout->addWidget(colorPUI->colorPicker(), 0, 1);
 
     // Display radius parameter.
-    FloatParameterUI* displayRadiusPUI = new FloatParameterUI(this, PROPERTY_FIELD(ParticleType::radius));
+    FloatParameterUI* displayRadiusPUI = createParamUI<FloatParameterUI>(PROPERTY_FIELD(ParticleType::radius));
     gridLayout->addWidget(displayRadiusPUI->label(), 1, 0);
     gridLayout->addLayout(displayRadiusPUI->createFieldLayout(), 1, 1);
     displayRadiusPUI->spinner()->setStandardValue(0.0);
     displayRadiusPUI->textBox()->setPlaceholderText(tr("‹unspecified›"));
 
     // Shape type.
-    VariantComboBoxParameterUI* particleShapeUI = new VariantComboBoxParameterUI(this, PROPERTY_FIELD(ParticleType::shape));
+    VariantComboBoxParameterUI* particleShapeUI = createParamUI<VariantComboBoxParameterUI>(PROPERTY_FIELD(ParticleType::shape));
     particleShapeUI->comboBox()->addItem(tr("‹unspecified›"), QVariant::fromValue((int)ParticlesVis::Default));
     particleShapeUI->comboBox()->addItem(QIcon(":/particles/icons/particle_shape_sphere.png"), tr("Sphere/Ellipsoid"), QVariant::fromValue((int)ParticlesVis::Sphere));
     particleShapeUI->comboBox()->addItem(QIcon(":/particles/icons/particle_shape_circle.png"), tr("Circle"), QVariant::fromValue((int)ParticlesVis::Circle));
@@ -123,7 +122,7 @@ void ParticleTypeEditor::createUI(const RolloutInsertionParameters& rolloutParam
         // Loads the default parameter value.
         [](ParticleType* ptype) { ptype->setColor(ElementType::getDefaultColor(ptype->ownerProperty(), ptype->nameOrNumericId(), ptype->numericId(), true)); },
         // Saves the current parameter value as new default preset.
-        [](const ParticleType* ptype) { ElementType::setDefaultColor(ParticlePropertyReference(Particles::TypeProperty), ptype->nameOrNumericId(), ptype->color()); },
+        [](const ParticleType* ptype) { ElementType::setDefaultColor(OwnerPropertyRef(&Particles::OOClass(), Particles::TypeProperty), ptype->nameOrNumericId(), ptype->color()); },
         // Determines if the current parameter value differs from the saved default value or not.
         [](const ParticleType* ptype) { return (ptype->color() == ElementType::getDefaultColor(ptype->ownerProperty(), ptype->nameOrNumericId(), ptype->numericId(), true)); }
     );
@@ -132,11 +131,11 @@ void ParticleTypeEditor::createUI(const RolloutInsertionParameters& rolloutParam
     // Display radius presets menu.
     QToolButton* displayRadiusPresetsMenuButton = createPresetsMenuButton(tr("display radius"),
         // Loads the default parameter value.
-        [](ParticleType* ptype) { ptype->setRadius(ParticleType::getDefaultParticleRadius(static_cast<Particles::Type>(ptype->ownerProperty().type()), ptype->nameOrNumericId(), ptype->numericId(), true, ParticleType::DisplayRadius)); },
+        [](ParticleType* ptype) { ptype->setRadius(ParticleType::getDefaultParticleRadius(static_cast<Particles::Type>(ptype->ownerProperty().typeId()), ptype->nameOrNumericId(), ptype->numericId(), true, ParticleType::DisplayRadius)); },
         // Saves the current parameter value as new default preset.
         [](const ParticleType* ptype) { ParticleType::setDefaultParticleRadius(Particles::TypeProperty, ptype->nameOrNumericId(), ptype->radius(), ParticleType::DisplayRadius); },
         // Determines if the current parameter value differs from the saved default value or not.
-        [](const ParticleType* ptype) { return (ptype->radius() == ParticleType::getDefaultParticleRadius(static_cast<Particles::Type>(ptype->ownerProperty().type()), ptype->nameOrNumericId(), ptype->numericId(), true, ParticleType::DisplayRadius)); }
+        [](const ParticleType* ptype) { return (ptype->radius() == ParticleType::getDefaultParticleRadius(static_cast<Particles::Type>(ptype->ownerProperty().typeId()), ptype->nameOrNumericId(), ptype->numericId(), true, ParticleType::DisplayRadius)); }
     );
     gridLayout->addWidget(displayRadiusPresetsMenuButton, 1, 2);
 
@@ -151,11 +150,11 @@ void ParticleTypeEditor::createUI(const RolloutInsertionParameters& rolloutParam
     QPushButton* loadShapeBtn = new QPushButton(tr("Load geometry file..."));
     loadShapeBtn->setToolTip(tr("Loads a mesh file to be used as shape for this particle type."));
     gridLayout->addWidget(loadShapeBtn, 0, 0, 1, 2);
-    BooleanParameterUI* highlightEdgesUI = new BooleanParameterUI(this, PROPERTY_FIELD(ParticleType::highlightShapeEdges));
+    BooleanParameterUI* highlightEdgesUI = createParamUI<BooleanParameterUI>(PROPERTY_FIELD(ParticleType::highlightShapeEdges));
     gridLayout->addWidget(highlightEdgesUI->checkBox(), 1, 0, 1, 2);
-    BooleanParameterUI* shapeBackfaceCullingUI = new BooleanParameterUI(this, PROPERTY_FIELD(ParticleType::shapeBackfaceCullingEnabled));
+    BooleanParameterUI* shapeBackfaceCullingUI = createParamUI<BooleanParameterUI>(PROPERTY_FIELD(ParticleType::shapeBackfaceCullingEnabled));
     gridLayout->addWidget(shapeBackfaceCullingUI->checkBox(), 2, 0, 1, 2);
-    BooleanParameterUI* shapeUseMeshColorUI = new BooleanParameterUI(this, PROPERTY_FIELD(ParticleType::shapeUseMeshColor));
+    BooleanParameterUI* shapeUseMeshColorUI = createParamUI<BooleanParameterUI>(PROPERTY_FIELD(ParticleType::shapeUseMeshColor));
     gridLayout->addWidget(shapeUseMeshColorUI->checkBox(), 3, 0, 1, 2);
 
     // Show/hide controls for user-defined shapes depending on the selected shape type.
@@ -184,38 +183,7 @@ void ParticleTypeEditor::createUI(const RolloutInsertionParameters& rolloutParam
     });
 
     // Shape load button.
-    connect(loadShapeBtn, &QPushButton::clicked, this, [this]() {
-        if(OORef<ParticleType> ptype = static_object_cast<ParticleType>(editObject())) {
-
-            performTransaction(tr("Load particle shape"), [&](MainThreadOperation& operation) {
-                QUrl selectedFile;
-                const FileImporterClass* fileImporterClass = nullptr;
-                QString fileImporterFormat;
-
-                // Put code in a block: Need to release dialog before loading the input file.
-                {
-                    // Build list of file importers that can import triangle meshes.
-                    QVector<const FileImporterClass*> meshImporters;
-                    for(const FileImporterClass* importerClass : PluginManager::instance().metaclassMembers<FileSourceImporter>()) {
-                        if(importerClass->importsDataType(TriangleMesh::OOClass()))
-                            meshImporters.push_back(importerClass);
-                    }
-
-                    // Let the user select a geometry file to import.
-                    ImportFileDialog fileDialog(meshImporters, &mainWindow(), tr("Load geometry file"), false, QStringLiteral("particle_shape_mesh"));
-                    if(fileDialog.exec() != QDialog::Accepted)
-                        return;
-
-                    selectedFile = fileDialog.urlToImport();
-                    std::tie(fileImporterClass, fileImporterFormat) = fileDialog.selectedFileImporter();
-                }
-
-                // Load the geometry from the selected file.
-                ProgressDialog progressDialog(container(), tr("Loading geometry file"));
-                ptype->loadShapeMesh(selectedFile, MainThreadOperation(true), fileImporterClass, fileImporterFormat);
-            });
-        }
-    });
+    connect(loadShapeBtn, &QPushButton::clicked, this, &ParticleTypeEditor::onLoadParticleShape);
 
     // Physical properties group.
     QGroupBox* physicalBox = new QGroupBox(tr("Physical properties"), rollout);
@@ -225,10 +193,10 @@ void ParticleTypeEditor::createUI(const RolloutInsertionParameters& rolloutParam
     layout1->addWidget(physicalBox);
 
     // Mass parameter.
-    FloatParameterUI* massPUI = new FloatParameterUI(this, PROPERTY_FIELD(ParticleType::mass));
+    FloatParameterUI* massPUI = createParamUI<FloatParameterUI>(PROPERTY_FIELD(ParticleType::mass));
     gridLayout->addWidget(massPUI->label(), 0, 0);
     gridLayout->addLayout(massPUI->createFieldLayout(), 0, 1);
-    // Reset mass paramter - can't use createPresetsMenuButton because we only
+    // Reset mass parameter - can't use createPresetsMenuButton because we only
     // offer reset but not the other options
     // Don't use PROPERTY_FIELD_RESETTABLE to give custom (better) tooltip
     MenuToolButton* presetsMenuButton = new MenuToolButton();
@@ -241,7 +209,7 @@ void ParticleTypeEditor::createUI(const RolloutInsertionParameters& rolloutParam
         connect(loadPresetAction, &QAction::triggered, this, [this, parameterName]() {
             if(ParticleType* ptype = static_object_cast<ParticleType>(editObject())) {
                 performTransaction(tr("Reset particle type %1").arg(parameterName), [&]() {
-                    ptype->setMass(ParticleType::getDefaultParticleMass(static_cast<Particles::Type>(ptype->ownerProperty().type()),
+                    ptype->setMass(ParticleType::getDefaultParticleMass(static_cast<Particles::Type>(ptype->ownerProperty().typeId()),
                                                                         ptype->nameOrNumericId(), ptype->numericId(), false));
                     mainWindow().showStatusBarMessage(
                         tr("Reset %1 of particle type '%2' to default value.").arg(parameterName).arg(ptype->nameOrNumericId()), 4000);
@@ -255,7 +223,7 @@ void ParticleTypeEditor::createUI(const RolloutInsertionParameters& rolloutParam
     massPUI->textBox()->setPlaceholderText(tr("‹unspecified›"));
 
     // VDW radius parameter.
-    FloatParameterUI* vdwRadiusPUI = new FloatParameterUI(this, PROPERTY_FIELD(ParticleType::vdwRadius));
+    FloatParameterUI* vdwRadiusPUI = createParamUI<FloatParameterUI>(PROPERTY_FIELD(ParticleType::vdwRadius));
     gridLayout->addWidget(vdwRadiusPUI->label(), 1, 0);
     gridLayout->addLayout(vdwRadiusPUI->createFieldLayout(), 1, 1);
     vdwRadiusPUI->spinner()->setStandardValue(0.0);
@@ -264,11 +232,11 @@ void ParticleTypeEditor::createUI(const RolloutInsertionParameters& rolloutParam
     // VDW radius presets menu.
     QToolButton* vdwRadiusPresetsMenuButton = createPresetsMenuButton(tr("VdW radius"),
         // Loads the default parameter value.
-        [](ParticleType* ptype) { ptype->setVdwRadius(ParticleType::getDefaultParticleRadius(static_cast<Particles::Type>(ptype->ownerProperty().type()), ptype->nameOrNumericId(), ptype->numericId(), true, ParticleType::VanDerWaalsRadius)); },
+        [](ParticleType* ptype) { ptype->setVdwRadius(ParticleType::getDefaultParticleRadius(static_cast<Particles::Type>(ptype->ownerProperty().typeId()), ptype->nameOrNumericId(), ptype->numericId(), true, ParticleType::VanDerWaalsRadius)); },
         // Saves the current parameter value as new default preset.
         [](const ParticleType* ptype) { ParticleType::setDefaultParticleRadius(Particles::TypeProperty, ptype->nameOrNumericId(), ptype->vdwRadius(), ParticleType::VanDerWaalsRadius); },
         // Determines if the current parameter value differs from the saved default value or not.
-        [](const ParticleType* ptype) { return (ptype->vdwRadius() == ParticleType::getDefaultParticleRadius(static_cast<Particles::Type>(ptype->ownerProperty().type()), ptype->nameOrNumericId(), ptype->numericId(), true, ParticleType::VanDerWaalsRadius)); }
+        [](const ParticleType* ptype) { return (ptype->vdwRadius() == ParticleType::getDefaultParticleRadius(static_cast<Particles::Type>(ptype->ownerProperty().typeId()), ptype->nameOrNumericId(), ptype->numericId(), true, ParticleType::VanDerWaalsRadius)); }
     );
     gridLayout->addWidget(vdwRadiusPresetsMenuButton, 1, 2);
 }
@@ -300,7 +268,7 @@ QToolButton* ParticleTypeEditor::createPresetsMenuButton(const QString& paramete
             mainWindow().showStatusBarMessage(tr("Stored current %1 as default for particle type '%2'.").arg(parameterName).arg(ptype->nameOrNumericId()), 4000);
         }
     });
-    presetsMenuButton->createMenuSeperator();
+    presetsMenuButton->createMenuSeparator();
     QAction* editPresetAction = presetsMenuButton->createAction(QIcon::fromTheme("application_preferences"), tr("Edit presets..."));
     connect(editPresetAction, &QAction::triggered, this, [this]() {
         ApplicationSettingsDialog dlg(mainWindow(), &ParticleSettingsPage::OOClass());
@@ -325,4 +293,49 @@ QToolButton* ParticleTypeEditor::createPresetsMenuButton(const QString& paramete
     return presetsMenuButton;
 }
 
+/******************************************************************************
+* Called when the user wants to pick and load a mesh-based particle shape from disk.
+******************************************************************************/
+void ParticleTypeEditor::onLoadParticleShape()
+{
+    OORef<ParticleType> ptype = static_object_cast<ParticleType>(editObject());
+    if(!ptype)
+        return;
+
+    handleExceptions([&]() {
+        // Build list of file importers that can import triangle meshes.
+        std::vector<const FileImporterClass*> meshImporters;
+        for(const FileImporterClass* importerClass : PluginManager::instance().metaclassMembers<FileSourceImporter>()) {
+            if(importerClass->importsDataType(TriangleMesh::OOClass()))
+                meshImporters.push_back(importerClass);
+        }
+
+        // Let the user select a geometry file to import.
+        ImportFileDialog fileDialog(meshImporters, &mainWindow(), tr("Load geometry file"), false, QStringLiteral("particle_shape_mesh"));
+        if(fileDialog.exec() != QDialog::Accepted)
+            return;
+
+        auto [fileImporterClass, fileImporterFormat] = fileDialog.selectedFileImporter();
+
+        // Asynchronously load the geometry from the selected file.
+        auto future = ptype->loadShapeMesh(fileDialog.urlToImport(), fileImporterClass, fileImporterFormat).then(ObjectExecutor(ptype), [this, ptype](DataOORef<TriangleMesh> shapeMesh) {
+            performTransaction(tr("Load particle shape"), [&]() {
+                // Assign the mesh-based shape to the particle type.
+                ptype->setShapeMesh(std::move(shapeMesh));
+
+                // Also switch the particle type's visualization mode to mesh-based.
+                ptype->setShape(ParticlesVis::Mesh);
+
+                // Determine whether the mesh is a closed manifold.
+                // If not, we should turn off back-face culling.
+                if(ptype->shapeMesh() && !ptype->shapeMesh()->isClosed())
+                    ptype->setShapeBackfaceCullingEnabled(false);
+            });
+        });
+
+        // Show a progress dialog while performing the whole operation. The dialog will self-destruct afterwards.
+        ProgressDialog::showForFuture(std::move(future), mainWindow(), parentWindow(), tr("Loading geometry file"));
+    });
+}
+
 }   // End of namespace
