@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -38,7 +38,7 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(ParticleImporter);
+IMPLEMENT_ABSTRACT_OVITO_CLASS(ParticleImporter);
 DEFINE_PROPERTY_FIELD(ParticleImporter, sortParticles);
 DEFINE_PROPERTY_FIELD(ParticleImporter, generateBonds);
 DEFINE_PROPERTY_FIELD(ParticleImporter, recenterCell);
@@ -54,9 +54,11 @@ void ParticleImporter::propertyChanged(const PropertyFieldDescriptor* field)
     FileSourceImporter::propertyChanged(field);
 
     if(field == PROPERTY_FIELD(sortParticles) || field == PROPERTY_FIELD(generateBonds) || field == PROPERTY_FIELD(recenterCell)) {
-        // Reload input file(s) when these options are changed by the user.
-        // But there is no need to refetch the data file(s) from the remote location. Reparsing the cached files is sufficient.
-        requestReload();
+        if(!isBeingLoaded()) {
+            // Reload input file(s) when these options are changed by the user.
+            // But there is no need to refetch the data file(s) from the remote location. Re-parsing the cached files is sufficient.
+            requestReload();
+        }
     }
 }
 
@@ -223,7 +225,7 @@ void ParticleImporter::FrameLoader::setDihedralCount(size_t count)
 }
 
 /******************************************************************************
-* Creates an impropers containerobject (if the bond count is non-zero) and adjusts the
+* Creates an impropers container object (if the bond count is non-zero) and adjusts the
 * number of elements of the property container.
 ******************************************************************************/
 void ParticleImporter::FrameLoader::setImproperCount(size_t count)
@@ -273,15 +275,17 @@ void ParticleImporter::FrameLoader::generateBondPeriodicImageProperty()
 /******************************************************************************
 * Generates ad-hoc bonds between atoms based on their van der Waals radii.
 ******************************************************************************/
-void ParticleImporter::FrameLoader::generateBonds()
+void ParticleImporter::FrameLoader::generateBonds(TaskProgress& progress)
 {
-    if(isCanceled()) return;
-    if(!_particles) return;
+    this_task::throwIfCanceled();
+    if(!_particles)
+        return;
 
     // Get the type particle property.
     const Property* typeProperty = _particles->getProperty(Particles::TypeProperty);
     const Property* positionProperty = _particles->getProperty(Particles::PositionProperty);
-    if(!typeProperty || !positionProperty) return;
+    if(!typeProperty || !positionProperty)
+        return;
 
     // Do not delete the generated bonds again in FrameLoader::loadFile().
     setKeepExistingTopology(true);
@@ -311,19 +315,17 @@ void ParticleImporter::FrameLoader::generateBonds()
     if(maxCutoff == 0.0)
         return;
     FloatType minCutoffSquared = 1e-10 * maxCutoff * maxCutoff;
-    setProgressText(tr("Generating bonds"));
+    progress.setText(tr("Generating bonds"));
 
-    // Prepare the neighbor list.
-    CutoffNeighborFinder neighborFinder;
-    if(!neighborFinder.prepare(maxCutoff, positionProperty, state().getObject<SimulationCell>(), {}))
-        return;
+    // Prepare the neighbor finder.
+    CutoffNeighborFinder neighborFinder(maxCutoff, positionProperty, state().getObject<SimulationCell>(), {});
 
     BufferReadAccess<int32_t> particleTypesArray(typeProperty);
 
     // Multi-threaded loop over all particles, each thread producing a partial bonds list.
     size_t particleCount = positionProperty->size();
-    auto partialBondsLists = parallelForCollect<std::vector<Bond>>(particleCount, [&](size_t particleIndex, std::vector<Bond>& bondList) {
-        // Kernel called for each particle: Iterate over the particle's neighbors withing the cutoff range.
+    auto partialBondsLists = parallelForCollect<std::vector<Bond>>(particleCount, 4096, progress, [&](size_t particleIndex, std::vector<Bond>& bondList) {
+        // Kernel called for each particle: Iterate over the particle's neighbors within the cutoff range.
         for(CutoffNeighborFinder::Query neighborQuery(neighborFinder, particleIndex); !neighborQuery.atEnd(); neighborQuery.next()) {
             int type1 = particleTypesArray[particleIndex];
             int type2 = particleTypesArray[neighborQuery.current()];
@@ -340,10 +342,8 @@ void ParticleImporter::FrameLoader::generateBonds()
             }
         }
     });
-    if(isCanceled())
-        return;
 
-    // Create Bonds.
+    // Create bonds.
     setBondCount(boost::accumulate(partialBondsLists, (size_t)0, [](size_t n, const std::vector<Bond>& bonds) { return n + bonds.size(); }));
     BufferWriteAccess<ParticleIndexPair, access_mode::discard_write> bondTopologyProperty = this->bonds()->createProperty(Bonds::TopologyProperty);
     Property* bondTypeProperty = this->bonds()->createProperty(Bonds::TypeProperty);
@@ -371,7 +371,7 @@ void ParticleImporter::FrameLoader::generateBonds()
 ******************************************************************************/
 void ParticleImporter::FrameLoader::computeVelocityMagnitude()
 {
-    if(!_particles || isCanceled())
+    if(!_particles || this_task::isCanceled())
         return;
 
     if(BufferReadAccess<Vector3> velocityVectors = _particles->getProperty(Particles::VelocityProperty)) {
@@ -390,7 +390,7 @@ void ParticleImporter::FrameLoader::computeVelocityMagnitude()
 ******************************************************************************/
 void ParticleImporter::FrameLoader::correctOffcenterCell()
 {
-    if(isCanceled())
+    if(this_task::isCanceled())
         return;
 
     // Check if a simulation cell has been defined. It must be periodic in all directions.
@@ -439,7 +439,7 @@ void ParticleImporter::FrameLoader::correctOffcenterCell()
 ******************************************************************************/
 void ParticleImporter::FrameLoader::recenterSimulationCell()
 {
-    if(isCanceled())
+    if(this_task::isCanceled())
         return;
 
     SimulationCell* simulationCell = state().getMutableObject<SimulationCell>();
@@ -465,7 +465,7 @@ void ParticleImporter::FrameLoader::recenterSimulationCell()
 ******************************************************************************/
 void ParticleImporter::FrameLoader::loadFile()
 {
-    if(isCanceled())
+    if(this_task::isCanceled())
         return;
 
     StandardFrameLoader::loadFile();
@@ -486,6 +486,14 @@ void ParticleImporter::FrameLoader::loadFile()
         if(!_impropers) setImproperCount(0);
     }
 
+    // Precompute checksum of Particle Identifier property to speed up
+    // the detection of particle reorderings.
+    if(_particles) {
+        if(const Property* identifiers = _particles->getProperty(Particles::IdentifierProperty)) {
+            (void)identifiers->checksum();
+        }
+    }
+
 #ifdef OVITO_DEBUG
     if(_particles) _particles->verifyIntegrity();
     if(_bonds) _bonds->verifyIntegrity();
@@ -498,7 +506,7 @@ void ParticleImporter::FrameLoader::loadFile()
 /******************************************************************************
 * Is called when importing multiple files of different formats.
 ******************************************************************************/
-bool ParticleImporter::importFurtherFiles(Scene* scene, std::vector<std::pair<QUrl, OORef<FileImporter>>> sourceUrlsAndImporters, ImportMode importMode, bool autodetectFileSequences, MultiFileImportMode multiFileImportMode, Pipeline* pipeline)
+Future<OORef<Pipeline>> ParticleImporter::importFurtherFiles(OORef<Scene> scene, std::vector<std::pair<QUrl, OORef<FileImporter>>> sourceUrlsAndImporters, ImportMode importMode, bool autodetectFileSequences, MultiFileImportMode multiFileImportMode, Pipeline* pipeline)
 {
     OVITO_ASSERT(!sourceUrlsAndImporters.empty());
     OORef<ParticleImporter> nextImporter = dynamic_object_cast<ParticleImporter>(sourceUrlsAndImporters.front().second);
@@ -522,18 +530,19 @@ bool ParticleImporter::importFurtherFiles(Scene* scene, std::vector<std::pair<QU
         sourceUrlsAndImporters.erase(sourceUrlsAndImporters.begin(), iter);
 
         // Set the input file location(s) and importer.
-        if(!fileSource->setSource(std::move(sourceUrls), nextImporter, autodetectFileSequences))
-            return {};
+        fileSource->setSource(std::move(sourceUrls), nextImporter, autodetectFileSequences);
+        this_task::throwIfCanceled();
 
         // Create a modifier for injecting the trajectory data into the existing pipeline.
         OORef<LoadTrajectoryModifier> loadTrjMod = OORef<LoadTrajectoryModifier>::create();
         loadTrjMod->setTrajectorySource(std::move(fileSource));
-        pipeline->applyModifier(scene->animationSettings()->currentTime(), std::move(loadTrjMod));
+        pipeline->applyModifier(scene->animationSettings()->currentTime(), true, std::move(loadTrjMod));
 
         if(sourceUrlsAndImporters.empty())
-            return true;
+            return Future<OORef<Pipeline>>::createImmediateEmpty();
     }
-    return FileSourceImporter::importFurtherFiles(scene, std::move(sourceUrlsAndImporters), importMode, autodetectFileSequences, multiFileImportMode, pipeline);
+
+    return FileSourceImporter::importFurtherFiles(std::move(scene), std::move(sourceUrlsAndImporters), importMode, autodetectFileSequences, multiFileImportMode, pipeline);
 }
 
 }   // End of namespace
