@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -28,7 +28,8 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(DCDImporter);
+IMPLEMENT_CREATABLE_OVITO_CLASS(DCDImporter);
+OVITO_CLASSINFO(DCDImporter, "DisplayName", "DCD");
 
 static void swap_endianess(qint32* data, size_t ndata = 1)
 {
@@ -265,11 +266,12 @@ bool DCDImporter::OOMetaClass::checkFileFormat(const FileHandle &file) const
 /******************************************************************************
  * Scans the data file and builds a list of source frames.
  ******************************************************************************/
-void DCDImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::Frame> &frames)
+void DCDImporter::discoverFramesInFile(const FileHandle& fileHandle, QVector<FileSourceImporter::Frame>& frames) const
 {
-    setProgressText(tr("Scanning file %1").arg(fileHandle().toString()));
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Scanning file %1").arg(fileHandle.toString()));
 
-    std::unique_ptr<QIODevice> device = fileHandle().createIODevice();
+    std::unique_ptr<QIODevice> device = fileHandle.createIODevice();
     if(!device->open(QIODevice::ReadOnly))
         throw Exception(tr("Failed to open file: %1").arg(device->errorString()));
 
@@ -291,7 +293,7 @@ void DCDImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::
             nframes = header.nsets;
     }
 
-    Frame frame(fileHandle());
+    Frame frame(fileHandle);
     for(int i = 0; i < nframes; i++) {
         frame.byteOffset = i;
         frame.label = tr("Timestep %1").arg(header.istart + i * header.nsavc);
@@ -304,9 +306,10 @@ void DCDImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::
 ******************************************************************************/
 void DCDImporter::FrameLoader::loadFile()
 {
-    // Open file for reading.
-    setProgressText(tr("Reading DCD file %1").arg(fileHandle().toString()));
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Reading DCD file %1").arg(fileHandle().toString()));
 
+    // Open file for reading.
     std::unique_ptr<QIODevice> device = fileHandle().createIODevice();
     if(!device->open(QIODevice::ReadOnly))
         throw Exception(tr("Failed to open DCD file: %1").arg(device->errorString()));
@@ -418,6 +421,7 @@ void DCDImporter::FrameLoader::loadFile()
     if((header.charmm & DCDHeader::DCD_IS_CHARMM) && (header.charmm & DCDHeader::DCD_HAS_EXTRA_BLOCK)) {
         AffineTransformation cell = AffineTransformation::Identity();
         double cos_alpha, cos_beta, cos_gamma, sin_gamma;
+        bool explictCellVectors = false;
         if(unitcell[1] >= -1.0 && unitcell[1] <= 1.0 && unitcell[3] >= -1.0 && unitcell[3] <= 1.0 && unitcell[4] >= -1.0 && unitcell[4] <= 1.0) {
             // The file was generated by Charmm, or by NAMD > 2.5, with the angle
             // cosines of the periodic cell angles written to the DCD file.
@@ -434,6 +438,7 @@ void DCDImporter::FrameLoader::loadFile()
             cell.column(0) = Vector3(unitcell[0], unitcell[1], unitcell[3]);
             cell.column(1) = Vector3(unitcell[1], unitcell[2], unitcell[4]);
             cell.column(2) = Vector3(unitcell[3], unitcell[4], unitcell[5]);
+            explictCellVectors = true;
         }
         else {
             // This file was likely generated by NAMD 2.5 and the periodic cell
@@ -443,27 +448,29 @@ void DCDImporter::FrameLoader::loadFile()
             cos_gamma = std::cos(qDegreesToRadians(unitcell[1]));
             sin_gamma = std::sin(qDegreesToRadians(unitcell[1]));
         }
-        double a = unitcell[0];
-        double b = unitcell[2];
-        double c = unitcell[5];
-        if(cos_alpha == 0.0 && cos_beta == 0.0 && cos_gamma == 0.0) {
-            cell(0,0) = a;
-            cell(1,1) = b;
-            cell(2,2) = c;
-        }
-        else if(cos_alpha == 0.0 && cos_beta == 0.0) {
-            cell(0,0) = a;
-            cell(0,1) = b * cos_gamma;
-            cell(1,1) = b * sin_gamma;
-            cell(2,2) = c;
-        }
-        else {
-            cell(0,0) = a;
-            cell(0,1) = b * cos_gamma;
-            cell(1,1) = b * sin_gamma;
-            cell(0,2) = c * cos_beta;
-            cell(1,2) = c * (cos_alpha - cos_beta * cos_gamma) / sin_gamma;
-            cell(2,2) = std::sqrt(c * c - cell(0,2) * cell(0,2) - cell(1,2) * cell(1,2));
+        if(!explictCellVectors) {
+            double a = unitcell[0];
+            double b = unitcell[2];
+            double c = unitcell[5];
+            if(cos_alpha == 0.0 && cos_beta == 0.0 && cos_gamma == 0.0) {
+                cell(0,0) = a;
+                cell(1,1) = b;
+                cell(2,2) = c;
+            }
+            else if(cos_alpha == 0.0 && cos_beta == 0.0) {
+                cell(0,0) = a;
+                cell(0,1) = b * cos_gamma;
+                cell(1,1) = b * sin_gamma;
+                cell(2,2) = c;
+            }
+            else {
+                cell(0,0) = a;
+                cell(0,1) = b * cos_gamma;
+                cell(1,1) = b * sin_gamma;
+                cell(0,2) = c * cos_beta;
+                cell(1,2) = c * (cos_alpha - cos_beta * cos_gamma) / sin_gamma;
+                cell(2,2) = std::sqrt(c * c - cell(0,2) * cell(0,2) - cell(1,2) * cell(1,2));
+            }
         }
         simulationCell()->setCellMatrix(cell);
         simulationCell()->setPbcFlags(true, true, true);
