@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -32,7 +32,8 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(GroImporter);
+IMPLEMENT_CREATABLE_OVITO_CLASS(GroImporter);
+OVITO_CLASSINFO(GroImporter, "DisplayName", "GRO");
 
 /******************************************************************************
 * Checks if the given file has format that can be read by this importer.
@@ -118,17 +119,19 @@ bool GroImporter::OOMetaClass::checkFileFormat(const FileHandle& file) const
 /******************************************************************************
 * Scans the data file and builds a list of source frames.
 ******************************************************************************/
-void GroImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::Frame>& frames)
+void GroImporter::discoverFramesInFile(const FileHandle& fileHandle, QVector<FileSourceImporter::Frame>& frames) const
 {
-    CompressedTextReader stream(fileHandle());
-    setProgressText(tr("Scanning file %1").arg(fileHandle().toString()));
-    setProgressMaximum(stream.underlyingSize());
+    CompressedTextReader stream(fileHandle);
+
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Scanning file %1").arg(fileHandle.toString()));
+    progress.setMaximum(stream.underlyingSize());
 
     int frameNumber = 0;
-    QString filename = fileHandle().sourceUrl().fileName();
-    Frame frame(fileHandle());
+    QString filename = fileHandle.sourceUrl().fileName();
+    Frame frame(fileHandle);
 
-    while(!stream.eof() && !isCanceled()) {
+    while(!stream.eof() && !this_task::isCanceled()) {
         frame.byteOffset = stream.byteOffset();
         frame.lineNumber = stream.lineNumber();
         stream.recordSeekPoint();
@@ -159,8 +162,9 @@ void GroImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::
         // Skip atom lines.
         for(unsigned long long i = 0; i < numParticlesLong; i++) {
             stream.readLine();
-            if(!setProgressValueIntermittent(stream.underlyingByteOffset()))
-                return;
+            // Update progress bar and check for user cancellation.
+            if((i % 0x10000) == 0)
+                progress.setValue(stream.underlyingByteOffset());
         }
 
         // Skip cell geometry line.
@@ -173,7 +177,8 @@ void GroImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::
 ******************************************************************************/
 void GroImporter::FrameLoader::loadFile()
 {
-    setProgressText(tr("Reading Gromacs file %1").arg(fileHandle().toString()));
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Reading Gromacs file %1").arg(fileHandle().toString()));
 
     // Open file for reading.
     CompressedTextReader stream(fileHandle(), frame().byteOffset, frame().lineNumber);
@@ -195,7 +200,7 @@ void GroImporter::FrameLoader::loadFile()
     }
     if(numParticles > (unsigned long long)std::numeric_limits<int>::max())
         throw Exception(tr("Too many atoms in Gromacs file. This program version can read files with up to %1 atoms only.").arg(std::numeric_limits<int>::max()));
-    setProgressMaximum(numParticles);
+    progress.setMaximum(numParticles);
     setParticleCount(numParticles);
 
     // Create particle properties.
@@ -220,7 +225,8 @@ void GroImporter::FrameLoader::loadFile()
     int atomBaseNumber = 0;
     int residueBaseNumber = 0;
     for(size_t i = 0; i < numParticles; i++) {
-        if(!setProgressValueIntermittent(i)) return;
+        // Update progress bar and check for user cancellation.
+        progress.setValueIntermittent(i);
         const char* token = stream.readLine();
 
         // Parse residue number (5 characters).
@@ -443,7 +449,7 @@ void GroImporter::FrameLoader::loadFile()
 
     // Generate ad-hoc bonds between atoms based on their van der Waals radii.
     if(_generateBonds)
-        generateBonds();
+        generateBonds(progress);
     else
         setBondCount(0);
 
