@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -486,7 +486,7 @@ public:
                 std::copy(tempBuffer.begin(), tempBuffer.end(), buffer);
             }
             else {
-                qDebug() << "Unexpected chunk data type:" << chunk->type;
+                qWarning() << "Unexpected GSD chunk data type:" << chunk->type;
                 OVITO_ASSERT(false);
                 errCode = -1;
             }
