@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -39,7 +39,8 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(GSDImporter);
+IMPLEMENT_CREATABLE_OVITO_CLASS(GSDImporter);
+OVITO_CLASSINFO(GSDImporter, "DisplayName", "GSD");
 DEFINE_PROPERTY_FIELD(GSDImporter, roundingResolution);
 SET_PROPERTY_FIELD_LABEL(GSDImporter, roundingResolution, "Shape rounding resolution");
 SET_PROPERTY_FIELD_UNITS_AND_RANGE(GSDImporter, roundingResolution, IntegerParameterUnit, 1, 6);
@@ -51,7 +52,7 @@ void GSDImporter::propertyChanged(const PropertyFieldDescriptor* field)
 {
     ParticleImporter::propertyChanged(field);
 
-    if(field == PROPERTY_FIELD(roundingResolution)) {
+    if(field == PROPERTY_FIELD(roundingResolution) && !isBeingLoaded()) {
         // Clear shape cache and reload GSD file when the rounding resolution is changed.
         _cacheSynchronization.lockForWrite();
         _particleShapeCache.clear();
@@ -106,12 +107,13 @@ DataOORef<const TriangleMesh> GSDImporter::lookupParticleShapeInCache(const QByt
 /******************************************************************************
 * Scans the input file for simulation timesteps.
 ******************************************************************************/
-void GSDImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::Frame>& frames)
+void GSDImporter::discoverFramesInFile(const FileHandle& fileHandle, QVector<FileSourceImporter::Frame>& frames) const
 {
-    setProgressText(tr("Scanning file %1").arg(fileHandle().toString()));
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Scanning file %1").arg(fileHandle.toString()));
 
     // First close text stream, we don't need it here.
-    QString filename = QDir::toNativeSeparators(fileHandle().localFilePath());
+    QString filename = QDir::toNativeSeparators(fileHandle.localFilePath());
     if(filename.isEmpty())
         throw Exception(tr("The GSD file reader supports reading only from physical files. Cannot read data from an in-memory buffer."));
 
@@ -123,7 +125,7 @@ void GSDImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::
 #endif
     uint64_t nFrames = gsd.numerOfFrames();
 
-    Frame frame(fileHandle());
+    Frame frame(fileHandle);
     for(uint64_t i = 0; i < nFrames; i++) {
         uint64_t simulationStep = gsd.readOptionalScalar<uint64_t>("configuration/step", i, std::numeric_limits<uint64_t>::max());
         if(simulationStep != std::numeric_limits<uint64_t>::max())
@@ -140,7 +142,8 @@ void GSDImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::
 ******************************************************************************/
 void GSDImporter::FrameLoader::loadFile()
 {
-    setProgressText(tr("Reading GSD file %1").arg(fileHandle().toString()));
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Reading GSD file %1").arg(fileHandle().toString()));
 
     // Open GSD file for reading.
     QString filename = QDir::toNativeSeparators(fileHandle().localFilePath());
@@ -156,9 +159,6 @@ void GSDImporter::FrameLoader::loadFile()
     if(qstrcmp(gsd.schemaName(), "hoomd") != 0)
         throw Exception(tr("Failed to open GSD file for reading. File schema must be 'hoomd', but found '%1'.").arg(gsd.schemaName()));
 
-    // Parse number of frames in file.
-    uint64_t nFrames = gsd.numerOfFrames();
-
     // The animation frame to read from the GSD file.
     uint64_t frameNumber = frame().byteOffset;
 
@@ -201,7 +201,7 @@ void GSDImporter::FrameLoader::loadFile()
             gsd.readFloatArray("particles/position", frameNumber, posProperty.begin(), numParticles, posProperty.componentCount());
         else
             posProperty.take()->fill<Point3>(Point3::Origin());
-        if(isCanceled()) return;
+        this_task::throwIfCanceled();
     }
 
     {
@@ -215,14 +215,14 @@ void GSDImporter::FrameLoader::loadFile()
             gsd.readIntArray("particles/typeid", frameNumber, BufferWriteAccess<int32_t, access_mode::discard_write>(typeProperty).begin(), numParticles);
         else
             typeProperty->fill<int32_t>(0);
-        if(isCanceled()) return;
+        this_task::throwIfCanceled();
     }
 
     // Parse particle shape information.
     QByteArrayList particleTypeShapes = gsd.readStringTable("particles/type_shapes", frameNumber);
     if(particleTypeShapes.size() == particleTypeNames.size()) {
         for(int i = 0; i < particleTypeShapes.size(); i++) {
-            if(isCanceled()) return;
+            this_task::throwIfCanceled();
             parseParticleShape(i, particleTypeShapes[i]);
         }
     }
@@ -266,12 +266,12 @@ void GSDImporter::FrameLoader::loadFile()
     const Vector3 defaultMomentInertia(0,0,0);
     readOptionalProperty(gsd, "particles/moment_inertia", frameNumber, Particles::UserProperty, particles(), &defaultMomentInertia, sizeof(defaultMomentInertia));
 
-    if(isCanceled()) return;
+    this_task::throwIfCanceled();
 
     // Read any user-defined particle properties.
     const char* chunkName = gsd.findMatchingChunkName("log/particles/", nullptr);
     while(chunkName) {
-        if(isCanceled()) return;
+        this_task::throwIfCanceled();
         readOptionalProperty(gsd, chunkName, frameNumber, Particles::UserProperty, particles(), nullptr, 0);
         chunkName = gsd.findMatchingChunkName("log/particles/", chunkName);
     }
@@ -279,8 +279,15 @@ void GSDImporter::FrameLoader::loadFile()
     // Read any user-defined log chunks and add them to the global attributes dictionary.
     chunkName = gsd.findMatchingChunkName("log/", nullptr);
     while(chunkName) {
-        QString key = QString::fromUtf8(chunkName + 4); // Cut off "log/" prefix
-        state().setAttribute(key, gsd.readVariant(chunkName, frameNumber), pipelineNode());
+        QString key = QString::fromUtf8(chunkName + 4); // Remove "log/" prefix.
+        // Skip chunks that are imported as per-particle/bond/angle/etc. property.
+        if(!key.startsWith("particles/") &&
+                !key.startsWith("bonds/") &&
+                !key.startsWith("angles/") &&
+                !key.startsWith("dihedrals/") &&
+                !key.startsWith("impropers/")) {
+            state().setAttribute(key, gsd.readVariant(chunkName, frameNumber), pipelineNode());
+        }
         chunkName = gsd.findMatchingChunkName("log/", chunkName);
     }
 
@@ -291,7 +298,7 @@ void GSDImporter::FrameLoader::loadFile()
         // Read bonds list.
         std::vector<uint32_t> bondList(numBonds * 2);
         gsd.readIntArray("bonds/group", frameNumber, bondList.data(), numBonds, 2);
-        if(isCanceled()) return;
+        this_task::throwIfCanceled();
 
         // Convert to OVITO format.
         BufferWriteAccess<ParticleIndexPair, access_mode::discard_write> bondTopologyProperty = bonds()->createProperty(Bonds::TopologyProperty);
@@ -306,7 +313,7 @@ void GSDImporter::FrameLoader::loadFile()
         }
         bondTopologyProperty.reset();
         generateBondPeriodicImageProperty();
-        if(isCanceled()) return;
+        this_task::throwIfCanceled();
 
         // Read types.
         if(gsd.hasChunk("bonds/types", frameNumber)) {
@@ -328,13 +335,13 @@ void GSDImporter::FrameLoader::loadFile()
             else {
                 bondTypeProperty->fill<int32_t>(0);
             }
-            if(isCanceled()) return;
+            this_task::throwIfCanceled();
         }
 
         // Read any user-defined properties.
         const char* chunkName = gsd.findMatchingChunkName("log/bonds/", nullptr);
         while(chunkName) {
-            if(isCanceled()) return;
+            this_task::throwIfCanceled();
             readOptionalProperty(gsd, chunkName, frameNumber, Bonds::UserProperty, bonds(), nullptr, 0);
             chunkName = gsd.findMatchingChunkName("log/bonds/", chunkName);
         }
@@ -347,7 +354,7 @@ void GSDImporter::FrameLoader::loadFile()
         // Read angles list.
         std::vector<uint32_t> groupList(numAngles * 3);
         gsd.readIntArray("angles/group", frameNumber, groupList.data(), numAngles, 3);
-        if(isCanceled()) return;
+        this_task::throwIfCanceled();
 
         // Convert to OVITO format.
         BufferWriteAccess<ParticleIndexTriplet, access_mode::discard_write> topologyProperty = angles()->createProperty(Angles::TopologyProperty);
@@ -360,7 +367,7 @@ void GSDImporter::FrameLoader::loadFile()
             }
         }
         topologyProperty.reset();
-        if(isCanceled()) return;
+        this_task::throwIfCanceled();
 
         // Read types.
         if(gsd.hasChunk("angles/types", frameNumber)) {
@@ -382,13 +389,13 @@ void GSDImporter::FrameLoader::loadFile()
             else {
                 typeProperty->fill<int32_t>(0);
             }
-            if(isCanceled()) return;
+            this_task::throwIfCanceled();
         }
 
         // Read any user-defined properties.
         const char* chunkName = gsd.findMatchingChunkName("log/angles/", nullptr);
         while(chunkName) {
-            if(isCanceled()) return;
+            this_task::throwIfCanceled();
             readOptionalProperty(gsd, chunkName, frameNumber, Angles::UserProperty, angles(), nullptr, 0);
             chunkName = gsd.findMatchingChunkName("log/angles/", chunkName);
         }
@@ -401,7 +408,7 @@ void GSDImporter::FrameLoader::loadFile()
         // Read dihedrals list.
         std::vector<uint32_t> groupList(numDihedrals * 4);
         gsd.readIntArray("dihedrals/group", frameNumber, groupList.data(), numDihedrals, 4);
-        if(isCanceled()) return;
+        this_task::throwIfCanceled();
 
         // Convert to OVITO format.
         BufferWriteAccess<ParticleIndexQuadruplet, access_mode::discard_write> topologyProperty = dihedrals()->createProperty(Dihedrals::TopologyProperty);
@@ -414,7 +421,7 @@ void GSDImporter::FrameLoader::loadFile()
             }
         }
         topologyProperty.reset();
-        if(isCanceled()) return;
+        this_task::throwIfCanceled();
 
         // Read types.
         if(gsd.hasChunk("dihedrals/types", frameNumber)) {
@@ -436,13 +443,13 @@ void GSDImporter::FrameLoader::loadFile()
             else {
                 typeProperty->fill<int32_t>(0);
             }
-            if(isCanceled()) return;
+            this_task::throwIfCanceled();
         }
 
         // Read any user-defined properties.
         const char* chunkName = gsd.findMatchingChunkName("log/dihedrals/", nullptr);
         while(chunkName) {
-            if(isCanceled()) return;
+            this_task::throwIfCanceled();
             readOptionalProperty(gsd, chunkName, frameNumber, Dihedrals::UserProperty, dihedrals(), nullptr, 0);
             chunkName = gsd.findMatchingChunkName("log/dihedrals/", chunkName);
         }
@@ -455,7 +462,7 @@ void GSDImporter::FrameLoader::loadFile()
         // Read impropers list.
         std::vector<uint32_t> groupList(numImpropers * 4);
         gsd.readIntArray("impropers/group", frameNumber, groupList.data(), numImpropers, 4);
-        if(isCanceled()) return;
+        this_task::throwIfCanceled();
 
         // Convert to OVITO format.
         BufferWriteAccess<ParticleIndexQuadruplet, access_mode::discard_write> topologyProperty = impropers()->createProperty(Impropers::TopologyProperty);
@@ -468,7 +475,7 @@ void GSDImporter::FrameLoader::loadFile()
             }
         }
         topologyProperty.reset();
-        if(isCanceled()) return;
+        this_task::throwIfCanceled();
 
         // Read types.
         if(gsd.hasChunk("impropers/types", frameNumber)) {
@@ -490,13 +497,13 @@ void GSDImporter::FrameLoader::loadFile()
             else {
                 typeProperty->fill<int32_t>(0);
             }
-            if(isCanceled()) return;
+            this_task::throwIfCanceled();
         }
 
         // Read any user-defined properties.
         const char* chunkName = gsd.findMatchingChunkName("log/impropers/", nullptr);
         while(chunkName) {
-            if(isCanceled()) return;
+            this_task::throwIfCanceled();
             readOptionalProperty(gsd, chunkName, frameNumber, Impropers::UserProperty, impropers(), nullptr, 0);
             chunkName = gsd.findMatchingChunkName("log/impropers/", chunkName);
         }
@@ -529,10 +536,29 @@ Property* GSDImporter::FrameLoader::readOptionalProperty(GSDFile& gsd, const cha
         }
         else {
             QString propertyName = QString::fromUtf8(chunkName);
-            int slashPos = propertyName.lastIndexOf(QChar('/'));
-            if(slashPos != -1) propertyName.remove(0, slashPos + 1);
+            if(propertyName.startsWith("log/particles/"))
+                propertyName.remove(0, 14);
+            else if(propertyName.startsWith("log/bonds/"))
+                propertyName.remove(0, 10);
+            else if(propertyName.startsWith("log/angles/"))
+                propertyName.remove(0, 11);
+            else if(propertyName.startsWith("log/dihedrals/"))
+                propertyName.remove(0, 14);
+            else if(propertyName.startsWith("log/impropers/"))
+                propertyName.remove(0, 14);
             std::pair<int, size_t> dataTypeAndComponents = gsd.getChunkDataTypeAndComponentCount(chunkName);
             prop = container->createProperty(Property::makePropertyNameValid(propertyName), dataTypeAndComponents.first, dataTypeAndComponents.second);
+
+            // Automatically attach a VectorVis element to the property if it has 3 components and likely represents a force or velocity vector.
+            if(!prop->visElement() && container == particles() && prop->componentCount() == 3 && prop->dataType() == Property::Float64 && (propertyName.contains("force", Qt::CaseInsensitive) || propertyName.contains("velocity", Qt::CaseInsensitive))) {
+                OORef<VectorVis> vis = OORef<VectorVis>::create();
+                vis->setObjectTitle(propertyName);
+                vis->setEnabled(false);
+                vis->setReverseArrowDirection(false);
+                vis->setArrowPosition(VectorVis::Base);
+                vis->freezeInitialParameterValues({SHADOW_PROPERTY_FIELD(ActiveObject::title), SHADOW_PROPERTY_FIELD(ActiveObject::isEnabled), SHADOW_PROPERTY_FIELD(VectorVis::reverseArrowDirection), SHADOW_PROPERTY_FIELD(VectorVis::arrowPosition)});
+                prop->addVisElement(std::move(vis));
+            }
         }
         if(prop->dataType() == Property::Float32)
             gsd.readFloatArray(chunkName, frameNumber, BufferWriteAccess<float*, access_mode::discard_write>(prop).begin(), container->elementCount(), prop->componentCount());
@@ -551,7 +577,7 @@ Property* GSDImporter::FrameLoader::readOptionalProperty(GSDFile& gsd, const cha
         const char* found_name = gsd.findMatchingChunkName(chunkName, nullptr);
         if(found_name && qstrcmp(found_name, chunkName) == 0) {
             // If the GSD file contains the requested chunk in some other trajectory frame(s), just not in the current frame, then
-            // fill the property array with the default value for that chunk as specified by the HOOMD standard.
+            // fill the property array with the default value for that chunk as specified by the HOOMD schema.
             if(propertyType != Property::GenericUserProperty) {
                 prop = container->createProperty(DataBuffer::Uninitialized, propertyType);
             }
@@ -565,7 +591,7 @@ Property* GSDImporter::FrameLoader::readOptionalProperty(GSDFile& gsd, const cha
             }
             if(prop->stride() == defaultValueSize) {
                 RawBufferAccess<access_mode::discard_write> access(prop);
-                std::byte* dest = access.data();
+                auto* dest = access.data();
                 for(size_t i = 0; i < prop->size(); i++, dest += defaultValueSize) {
                     std::memcpy(dest, defaultValue, defaultValueSize);
                 }
@@ -579,24 +605,24 @@ Property* GSDImporter::FrameLoader::readOptionalProperty(GSDFile& gsd, const cha
 }
 
 /******************************************************************************
-* Assigns a mesh-based shape to a particle type.
+* Sets the shape of a particle type.
 ******************************************************************************/
-void GSDImporter::FrameLoader::setParticleTypeShape(int typeId, DataOORef<const TriangleMesh> shapeMesh)
+void GSDImporter::FrameLoader::setParticleTypeShape(int typeId, ParticlesVis::ParticleShape shape, DataOORef<const TriangleMesh> shapeMesh, FloatType radius)
 {
     const Property* existingTypeProperty = particles()->expectProperty(Particles::TypeProperty);
     const ParticleType* existingType = static_object_cast<ParticleType>(existingTypeProperty->elementType(typeId));
     OVITO_ASSERT(existingType);
 
     // Check whether the shape mesh is already assigned to the existing particle type.
-    if(!existingType || existingType->shapeMesh() == shapeMesh)
+    if(!existingType || (existingType->shape() == shape && existingType->shapeMesh() == shapeMesh))
         return;
 
     // Assign the shape to the particle type.
     Property* typeProperty = particles()->makeMutable(existingTypeProperty);
     ParticleType* mutableType = typeProperty->makeMutable(existingType);
+    mutableType->setShape(shape);
     mutableType->setShapeMesh(std::move(shapeMesh));
-    mutableType->setShape(ParticlesVis::ParticleShape::Mesh);
-    mutableType->setRadius(1.0);
+    mutableType->setRadius(radius);
     mutableType->freezeInitialParameterValues({SHADOW_PROPERTY_FIELD(ParticleType::radius), SHADOW_PROPERTY_FIELD(ParticleType::shape)});
 }
 
@@ -609,7 +635,7 @@ void GSDImporter::FrameLoader::parseParticleShape(int typeId, const QByteArray&
     DataOORef<const TriangleMesh> cacheShapeMesh = _importer->lookupParticleShapeInCache(shapeSpecString);
     if(cacheShapeMesh) {
         // Assign shape to particle type.
-        setParticleTypeShape(typeId, std::move(cacheShapeMesh));
+        setParticleTypeShape(typeId, ParticlesVis::ParticleShape::Mesh, std::move(cacheShapeMesh));
         return; // No need to parse the JSON string again.
     }
 
@@ -781,7 +807,7 @@ void GSDImporter::FrameLoader::parsePolygonShape(int typeId, QJsonObject definit
     _importer->storeParticleShapeInCache(shapeSpecString, triMesh);
 
     // Assign shape to particle type.
-    setParticleTypeShape(typeId, std::move(triMesh));
+    setParticleTypeShape(typeId, ParticlesVis::ParticleShape::Mesh, std::move(triMesh));
 }
 
 /******************************************************************************
@@ -816,7 +842,7 @@ static void tessellateCornerFacet(SurfaceMesh::face_index seedFace, int recursiv
             midpoint += vertexGrower.vertexPosition(mesh.vertex2(edge)) - Point3::Origin();
             Vector3 normal = (midpoint * FloatType(0.5)) - center;
             normal.normalizeSafely();
-            SurfaceMesh::vertex_index new_v = mesh.splitEdge(edge, center + normal * roundingRadius, vertexGrower);
+            mesh.splitEdge(edge, center + normal * roundingRadius, vertexGrower);
             vertexNormals.push_back(normal);
         }
         edgeList.clear();
@@ -863,8 +889,45 @@ void GSDImporter::FrameLoader::parseConvexPolyhedronShape(int typeId, QJsonObjec
             vertex[c] = coordinateArray[c].toDouble();
         vertices.push_back(vertex);
     }
-    if(vertices.size() < 4)
-        throw Exception(tr("Invalid 'ConvexPolyhedron' particle shape definition in GSD file: Number of vertices must be at least 4."));
+
+    // Parse rounding radius.
+    FloatType roundingRadius = definition.value("rounding_radius").toDouble();
+    if(roundingRadius < 0)
+        throw Exception(tr("Invalid 'rounding_radius' field in 'ConvexPolyhedron' particle shape definition in GSD file. Value must not be negative."));
+
+    if(vertices.size() <= 0) {
+        throw Exception(tr("Invalid 'ConvexPolyhedron' particle shape definition in GSD file: Number of vertices must be positive."));
+    }
+    else if(vertices.size() == 1) {
+        if(vertices[0] != Point3::Origin())
+            throw Exception(tr("Unsupported 'ConvexPolyhedron' particle shape definition in GSD file: single sphere must be centered on the origin."));
+        setParticleTypeShape(typeId, ParticlesVis::ParticleShape::Sphere, {}, roundingRadius);
+        return;
+    }
+    else if(vertices.size() == 2) {
+        if(vertices[0].x() != 0 || vertices[0].y() != 0 || vertices[1].x() != 0 || vertices[1].y() != 0 || vertices[0].z() != -vertices[1].z())
+            throw Exception(tr("Unsupported 'ConvexPolyhedron' particle shape definition in GSD file: OVITO supports only spherocylinders aligned with the z-axis and centered on the origin. If you require support for more general shapes, please contact the OVITO development team."));
+        if(roundingRadius <= 0)
+            throw Exception(tr("Invalid rounding radius for 'ConvexPolyhedron' particle shape with 2 vertices (spherocylinder): Radius must be positive."));
+
+        setParticleTypeShape(typeId, ParticlesVis::ParticleShape::Spherocylinder);
+
+        // Create the 'Aspherical Shape' particle property if it doesn't exist yet.
+        BufferWriteAccess<Vector3G, access_mode::read_write> ashapeProperty = particles()->createProperty(DataBuffer::Initialized, Particles::AsphericalShapeProperty);
+
+        // Assign the [a,b,c] values to those particles which are of the given type.
+        BufferReadAccess<int32_t> typeProperty = particles()->expectProperty(Particles::TypeProperty);
+        const Vector3G abc(roundingRadius, 0, std::abs(vertices[1].z() - vertices[0].z()));
+        for(size_t i = 0; i < typeProperty.size(); i++) {
+            if(typeProperty[i] == typeId)
+                ashapeProperty[i] = abc;
+        }
+
+        return;
+    }
+    else if(vertices.size() == 3) {
+        throw Exception(tr("Unsupported 'ConvexPolyhedron' particle shape in GSD file: OVITO supports only ConvexPolyhedron shapes with a number of vertices other than 3. If you require support for additional shapes, please contact the OVITO development team."));
+    }
 
     // Construct the convex hull of the vertices.
     // This yields a half-edge surface mesh data structure.
@@ -875,8 +938,6 @@ void GSDImporter::FrameLoader::parseConvexPolyhedronShape(int typeId, QJsonObjec
         meshBuilder.joinCoplanarFaces();
     }
 
-    // Parse rounding radius.
-    FloatType roundingRadius = definition.value("rounding_radius").toDouble();
     std::vector<Vector3> vertexNormals;
     if(roundingRadius > 0) {
         DataOORef<SurfaceMesh> roundedMeshObj = DataOORef<SurfaceMesh>::create(ObjectInitializationFlag::DontCreateVisElement);
@@ -960,7 +1021,7 @@ void GSDImporter::FrameLoader::parseConvexPolyhedronShape(int typeId, QJsonObjec
                     SurfaceMesh::face_index new_f = roundedMeshFaceGrower.createFace();
                     SurfaceMesh::edge_index edge = e;
                     do {
-                        SurfaceMesh::edge_index new_e = roundedMesh.createOppositeEdge(edge, new_f);
+                        roundedMesh.createOppositeEdge(edge, new_f);
                         edge = roundedMesh.prevFaceEdge(roundedMesh.oppositeEdge(roundedMesh.prevFaceEdge(roundedMesh.oppositeEdge(roundedMesh.prevFaceEdge(edge)))));
                     }
                     while(edge != e);
@@ -1016,7 +1077,7 @@ void GSDImporter::FrameLoader::parseConvexPolyhedronShape(int typeId, QJsonObjec
     _importer->storeParticleShapeInCache(shapeSpecString, triMesh);
 
     // Assign shape to particle type.
-    setParticleTypeShape(typeId, std::move(triMesh));
+    setParticleTypeShape(typeId, ParticlesVis::ParticleShape::Mesh, std::move(triMesh));
 }
 
 /******************************************************************************
@@ -1075,7 +1136,7 @@ void GSDImporter::FrameLoader::parseMeshShape(int typeId, QJsonObject definition
     _importer->storeParticleShapeInCache(shapeSpecString, triMesh);
 
     // Assign shape to particle type.
-    setParticleTypeShape(typeId, std::move(triMesh));
+    setParticleTypeShape(typeId, ParticlesVis::ParticleShape::Mesh, std::move(triMesh));
 }
 
 /******************************************************************************
@@ -1148,7 +1209,7 @@ void GSDImporter::FrameLoader::parseSphereUnionShape(int typeId, QJsonObject def
     _importer->storeParticleShapeInCache(shapeSpecString, triMesh);
 
     // Assign shape to particle type.
-    setParticleTypeShape(typeId, std::move(triMesh));
+    setParticleTypeShape(typeId, ParticlesVis::ParticleShape::Mesh, std::move(triMesh));
 }
 
 }   // End of namespace
