@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -29,7 +29,8 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(LAMMPSTextDumpImporter);
+IMPLEMENT_CREATABLE_OVITO_CLASS(LAMMPSTextDumpImporter);
+OVITO_CLASSINFO(LAMMPSTextDumpImporter, "DisplayName", "LAMMPS Dump");
 DEFINE_PROPERTY_FIELD(LAMMPSTextDumpImporter, useCustomColumnMapping);
 DEFINE_PROPERTY_FIELD(LAMMPSTextDumpImporter, customColumnMapping);
 SET_PROPERTY_FIELD_LABEL(LAMMPSTextDumpImporter, useCustomColumnMapping, "Custom file column mapping");
@@ -65,17 +66,19 @@ bool LAMMPSTextDumpImporter::OOMetaClass::checkFileFormat(const FileHandle& file
 /******************************************************************************
 * Scans the data file and builds a list of source frames.
 ******************************************************************************/
-void LAMMPSTextDumpImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::Frame>& frames)
+void LAMMPSTextDumpImporter::discoverFramesInFile(const FileHandle& fileHandle, QVector<FileSourceImporter::Frame>& frames) const
 {
-    CompressedTextReader stream(fileHandle());
-    setProgressText(tr("Scanning LAMMPS dump file %1").arg(fileHandle().toString()));
-    setProgressMaximum(stream.underlyingSize());
+    CompressedTextReader stream(fileHandle);
+
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Scanning LAMMPS dump file %1").arg(fileHandle.toString()));
+    progress.setMaximum(stream.underlyingSize());
 
     unsigned long long timestep = 0;
     size_t numParticles = 0;
-    Frame frame(fileHandle());
+    Frame frame(fileHandle);
 
-    while(!stream.eof() && !isCanceled()) {
+    while(!stream.eof() && !this_task::isCanceled()) {
         qint64 byteOffset = stream.byteOffset();
         int lineNumber = stream.lineNumber();
 
@@ -113,8 +116,8 @@ void LAMMPSTextDumpImporter::FrameFinder::discoverFramesInFile(QVector<FileSourc
             else if(stream.lineStartsWith("ITEM: ATOMS")) {
                 for(size_t i = 0; i < numParticles; i++) {
                     stream.readLine();
-                    if(!setProgressValueIntermittent(stream.underlyingByteOffset()))
-                        return;
+                    // Update progress bar and check for user cancellation.
+                    progress.setValueIntermittent(stream.underlyingByteOffset());
                 }
                 break;
             }
@@ -141,7 +144,8 @@ void LAMMPSTextDumpImporter::FrameFinder::discoverFramesInFile(QVector<FileSourc
 ******************************************************************************/
 void LAMMPSTextDumpImporter::FrameLoader::loadFile()
 {
-    setProgressText(tr("Reading LAMMPS dump file %1").arg(fileHandle().toString()));
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Reading LAMMPS dump file %1").arg(fileHandle().toString()));
 
     // Open file for reading.
     CompressedTextReader stream(fileHandle(), frame().byteOffset, frame().lineNumber);
@@ -178,35 +182,58 @@ void LAMMPSTextDumpImporter::FrameLoader::loadFile()
 
                 numParticles = (size_t)u;
                 setParticleCount(numParticles);
-                setProgressMaximum(u);
+                progress.setMaximum(u);
                 break;
             }
             else if(stream.lineStartsWith("ITEM: BOX BOUNDS xy xz yz")) {
-
                 // Parse optional boundary condition flags.
                 QStringList tokens = FileImporter::splitString(stream.lineString().mid(qstrlen("ITEM: BOX BOUNDS xy xz yz")));
-                if(tokens.size() >= 3)
-                    simulationCell()->setPbcFlags(tokens[0] == "pp", tokens[1] == "pp", tokens[2] == "pp");
+                if(tokens.size() >= 3) simulationCell()->setPbcFlags(tokens[0] == "pp", tokens[1] == "pp", tokens[2] == "pp");
 
                 // Parse triclinic simulation box.
                 FloatType tiltFactors[3];
                 Box3 simBox;
                 for(int k = 0; k < 3; k++) {
-                    if(sscanf(stream.readLine(), FLOATTYPE_SCANF_STRING " " FLOATTYPE_SCANF_STRING " " FLOATTYPE_SCANF_STRING, &simBox.minc[k], &simBox.maxc[k], &tiltFactors[k]) != 3)
-                        throw Exception(tr("Invalid box size in line %1 of LAMMPS dump file: %2").arg(stream.lineNumber()).arg(stream.lineString()));
+                    if(sscanf(stream.readLine(), FLOATTYPE_SCANF_STRING " " FLOATTYPE_SCANF_STRING " " FLOATTYPE_SCANF_STRING,
+                              &simBox.minc[k], &simBox.maxc[k], &tiltFactors[k]) != 3)
+                        throw Exception(
+                            tr("Invalid box size in line %1 of LAMMPS dump file: %2").arg(stream.lineNumber()).arg(stream.lineString()));
                 }
 
                 // LAMMPS only stores the outer bounding box of the simulation cell in the dump file.
                 // We have to determine the size of the actual triclinic cell.
-                simBox.minc.x() -= std::min(std::min(std::min(tiltFactors[0], tiltFactors[1]), tiltFactors[0]+tiltFactors[1]), (FloatType)0);
-                simBox.maxc.x() -= std::max(std::max(std::max(tiltFactors[0], tiltFactors[1]), tiltFactors[0]+tiltFactors[1]), (FloatType)0);
+                simBox.minc.x() -=
+                    std::min(std::min(std::min(tiltFactors[0], tiltFactors[1]), tiltFactors[0] + tiltFactors[1]), (FloatType)0);
+                simBox.maxc.x() -=
+                    std::max(std::max(std::max(tiltFactors[0], tiltFactors[1]), tiltFactors[0] + tiltFactors[1]), (FloatType)0);
                 simBox.minc.y() -= std::min(tiltFactors[2], (FloatType)0);
                 simBox.maxc.y() -= std::max(tiltFactors[2], (FloatType)0);
-                simulationCell()->setCellMatrix(AffineTransformation(
-                        Vector3(simBox.sizeX(), 0, 0),
-                        Vector3(tiltFactors[0], simBox.sizeY(), 0),
-                        Vector3(tiltFactors[1], tiltFactors[2], simBox.sizeZ()),
-                        simBox.minc - Point3::Origin()));
+                simulationCell()->setCellMatrix(
+                    AffineTransformation(Vector3(simBox.sizeX(), 0, 0), Vector3(tiltFactors[0], simBox.sizeY(), 0),
+                                         Vector3(tiltFactors[1], tiltFactors[2], simBox.sizeZ()), simBox.minc - Point3::Origin()));
+                break;
+            }
+            else if(stream.lineStartsWith("ITEM: BOX BOUNDS abc origin")) {
+                // Parse general triclinic simulation box.
+                // Format:
+                // ITEM: BOX BOUNDS abc origin [boundary-strings]
+                // avec[0] avec[1] avec[2] origin[0]
+                // bvec[0] bvec[1] bvec[2] origin[1]
+                // cvec[0] cvec[1] cvec[2] origin[2]
+                QStringList tokens = FileImporter::splitString(stream.lineString().sliced(qstrlen("ITEM: BOX BOUNDS abc origin")));
+                if(tokens.size() >= 3) {
+                    simulationCell()->setPbcFlags(tokens[0] == "pp", tokens[1] == "pp", tokens[2] == "pp");
+                }
+                AffineTransformation simCell;
+                for(int k = 0; k < 3; k++) {
+                    if(sscanf(stream.readLine(),
+                              FLOATTYPE_SCANF_STRING " " FLOATTYPE_SCANF_STRING " " FLOATTYPE_SCANF_STRING " " FLOATTYPE_SCANF_STRING,
+                              &simCell[k][0], &simCell[k][1], &simCell[k][2], &simCell[3][k]) != 4)
+                        throw Exception(tr("Invalid cell vectors in line %1 of LAMMPS dump file: %2")
+                                            .arg(stream.lineNumber())
+                                            .arg(stream.lineString()));
+                }
+                simulationCell()->setCellMatrix(simCell);
                 break;
             }
             else if(stream.lineStartsWith("ITEM: BOX BOUNDS")) {
@@ -263,7 +290,8 @@ void LAMMPSTextDumpImporter::FrameLoader::loadFile()
                 int lineNumber = stream.lineNumber() + 1;
                 try {
                     for(size_t i = 0; i < numParticles; i++, lineNumber++) {
-                        if(!setProgressValueIntermittent(i)) return;
+                        // Update progress bar and check for user cancellation.
+                        progress.setValueIntermittent(i);
                         if(!s)
                             columnParser.readElement(i, stream.readLine());
                         else
@@ -302,7 +330,7 @@ void LAMMPSTextDumpImporter::FrameLoader::loadFile()
             else if(stream.lineStartsWith("ITEM:")) {
                 // For the sake of forward compatibility, we ignore unknown ITEM sections.
                 // Skip lines until the next "ITEM:" is reached.
-                while(!stream.eof() && !isCanceled()) {
+                while(!stream.eof() && !this_task::isCanceled()) {
                     stream.readLine();
                     if(stream.lineStartsWith("ITEM:"))
                         break;
@@ -330,7 +358,7 @@ void LAMMPSTextDumpImporter::FrameLoader::postprocessParticleProperties(const QS
         // the type of particle coordinates. Reduced coordinates are found in columns
         // "xs, ys, zs" or "xsu, ysu, zsu".
         for(int i = 0; i < (int)columnMapping.size() && i < fileColumnNames.size(); i++) {
-            if(columnMapping[i].property.type() == Particles::PositionProperty) {
+            if(columnMapping[i].property.isStandardProperty(&Particles::OOClass(), Particles::PositionProperty)) {
                 reducedCoordinates = (
                         fileColumnNames[i] == "xs" || fileColumnNames[i] == "xsu" ||
                         fileColumnNames[i] == "ys" || fileColumnNames[i] == "ysu" ||
@@ -357,11 +385,17 @@ void LAMMPSTextDumpImporter::FrameLoader::postprocessParticleProperties(const QS
     }
 
     if(reducedCoordinates) {
-        // Convert all atom coordinates from reduced to absolute (Cartesian) format.
-        if(BufferWriteAccess<Point3, access_mode::read_write> posProperty = particles()->getMutableProperty(Particles::PositionProperty)) {
+        if(Property* positions = particles()->getMutableProperty(Particles::PositionProperty)) {
             const AffineTransformation simCell = simulationCell()->cellMatrix();
-            for(Point3& p : posProperty)
+#ifdef OVITO_USE_SYCL
+            SyclBufferAccess<Point3, access_mode::read_write>{positions}.for_each([simCell](Point3& p) {
                 p = simCell * p;
+            });
+#else
+            // Convert all atom coordinates from reduced to absolute (Cartesian) format.
+            for(Point3& p : BufferWriteAccess<Point3, access_mode::read_write>{positions})
+                p = simCell * p;
+#endif
         }
     }
 
@@ -369,10 +403,16 @@ void LAMMPSTextDumpImporter::FrameLoader::postprocessParticleProperties(const QS
         // If a "diameter" column was loaded and stored in the "Radius" particle property,
         // we need to divide values by two.
         for(int i = 0; i < (int)columnMapping.size() && i < fileColumnNames.size(); i++) {
-            if(columnMapping[i].property.type() == Particles::RadiusProperty && fileColumnNames[i] == "diameter") {
-                if(BufferWriteAccess<GraphicsFloatType, access_mode::read_write> radiusProperty = particles()->getMutableProperty(Particles::RadiusProperty)) {
-                    for(auto& r : radiusProperty)
+            if(columnMapping[i].property.isStandardProperty(&Particles::OOClass(), Particles::RadiusProperty) && fileColumnNames[i] == "diameter") {
+                if(Property* radii = particles()->getMutableProperty(Particles::RadiusProperty)) {
+#ifdef OVITO_USE_SYCL
+                    SyclBufferAccess<GraphicsFloatType, access_mode::read_write>{radii}.for_each([](GraphicsFloatType& r) {
                         r *= GraphicsFloatType(0.5);
+                    });
+#else
+                    for(auto& r : BufferWriteAccess<GraphicsFloatType, access_mode::read_write>{radii})
+                        r *= GraphicsFloatType(0.5);
+#endif
                 }
                 break;
             }
@@ -380,15 +420,19 @@ void LAMMPSTextDumpImporter::FrameLoader::postprocessParticleProperties(const QS
 
         // Same for the "c_diameter[1..3]" columns or "shapex/shapey/shapez" columns being mapped to the "Aspherical Shape" property.
         for(int i = 0; i < (int)columnMapping.size() && i < fileColumnNames.size(); i++) {
-            if(columnMapping[i].property.type() == Particles::AsphericalShapeProperty &&
+            if(columnMapping[i].property.isStandardProperty(&Particles::OOClass(), Particles::AsphericalShapeProperty) &&
                 (fileColumnNames[i] == "c_diameter[1]" || fileColumnNames[i] == "c_diameter[2]" || fileColumnNames[i] == "c_diameter[3]" ||
                     fileColumnNames[i] == "shapex" || fileColumnNames[i] == "shapey" || fileColumnNames[i] == "shapez")) {
-                if(BufferWriteAccess<Vector3G, access_mode::read_write> shapeProperty = particles()->getMutableProperty(Particles::AsphericalShapeProperty)) {
-                    for(auto& s : shapeProperty) {
-                        s.x() *= GraphicsFloatType(0.5);
-                        s.y() *= GraphicsFloatType(0.5);
-                        s.z() *= GraphicsFloatType(0.5);
+                if(Property* shapeProperty = particles()->getMutableProperty(Particles::AsphericalShapeProperty)) {
+#ifdef OVITO_USE_SYCL
+                    SyclBufferAccess<Vector3G, access_mode::read_write>{shapeProperty}.for_each([](Vector3G& s) {
+                        s *= GraphicsFloatType(0.5);
+                    });
+#else
+                    for(auto& s : BufferWriteAccess<Vector3G, access_mode::read_write>{shapeProperty}) {
+                        s *= GraphicsFloatType(0.5);
                     }
+#endif
                 }
                 break;
             }
@@ -397,9 +441,9 @@ void LAMMPSTextDumpImporter::FrameLoader::postprocessParticleProperties(const QS
 
     // Detect dimensionality of system. It's a 2D system if no file column has been mapped to the Position.Z particle property (but Position.X/Y are present).
     if(std::none_of(columnMapping.begin(), columnMapping.end(), [](const InputColumnInfo& column) {
-        return column.property.type() == Particles::PositionProperty && column.property.vectorComponent() == 2;
+        return column.property.isStandardProperty(&Particles::OOClass(), Particles::PositionProperty) && column.property.componentIndex(&Particles::OOClass()) == 2;
     }) && std::any_of(columnMapping.begin(), columnMapping.end(), [](const InputColumnInfo& column) {
-        return column.property.type() == Particles::PositionProperty && column.property.vectorComponent() != 2;
+        return column.property.isStandardProperty(&Particles::OOClass(), Particles::PositionProperty) && column.property.componentIndex(&Particles::OOClass()) != 2;
     })) {
         simulationCell()->setIs2D(true);
     }
@@ -420,77 +464,77 @@ ParticleInputColumnMapping LAMMPSTextDumpImporter::generateAutomaticColumnMappin
     for(int i = 0; i < columnNames.size(); i++) {
         QString name = columnNames[i].toLower();
         columnMapping[i].columnName = columnNames[i];
-        if(name == "x" || name == "xu" || name == "coordinates") columnMapping.mapStandardColumn(i, Particles::PositionProperty, 0);
-        else if(name == "y" || name == "yu") columnMapping.mapStandardColumn(i, Particles::PositionProperty, 1);
-        else if(name == "z" || name == "zu") columnMapping.mapStandardColumn(i, Particles::PositionProperty, 2);
-        else if(name == "xs" || name == "xsu") { columnMapping.mapStandardColumn(i, Particles::PositionProperty, 0); }
-        else if(name == "ys" || name == "ysu") { columnMapping.mapStandardColumn(i, Particles::PositionProperty, 1); }
-        else if(name == "zs" || name == "zsu") { columnMapping.mapStandardColumn(i, Particles::PositionProperty, 2); }
-        else if(name == "vx" || name == "velocities") columnMapping.mapStandardColumn(i, Particles::VelocityProperty, 0);
-        else if(name == "vy") columnMapping.mapStandardColumn(i, Particles::VelocityProperty, 1);
-        else if(name == "vz") columnMapping.mapStandardColumn(i, Particles::VelocityProperty, 2);
-        else if(name == "id") columnMapping.mapStandardColumn(i, Particles::IdentifierProperty);
-        else if(name == "element") columnMapping.mapStandardColumn(i, Particles::TypeProperty);
+        if(name == "x" || name == "xu" || name == "coordinates") columnMapping.mapColumnToStandardProperty(i, Particles::PositionProperty, 0);
+        else if(name == "y" || name == "yu") columnMapping.mapColumnToStandardProperty(i, Particles::PositionProperty, 1);
+        else if(name == "z" || name == "zu") columnMapping.mapColumnToStandardProperty(i, Particles::PositionProperty, 2);
+        else if(name == "xs" || name == "xsu") { columnMapping.mapColumnToStandardProperty(i, Particles::PositionProperty, 0); }
+        else if(name == "ys" || name == "ysu") { columnMapping.mapColumnToStandardProperty(i, Particles::PositionProperty, 1); }
+        else if(name == "zs" || name == "zsu") { columnMapping.mapColumnToStandardProperty(i, Particles::PositionProperty, 2); }
+        else if(name == "vx" || name == "velocities") columnMapping.mapColumnToStandardProperty(i, Particles::VelocityProperty, 0);
+        else if(name == "vy") columnMapping.mapColumnToStandardProperty(i, Particles::VelocityProperty, 1);
+        else if(name == "vz") columnMapping.mapColumnToStandardProperty(i, Particles::VelocityProperty, 2);
+        else if(name == "id") columnMapping.mapColumnToStandardProperty(i, Particles::IdentifierProperty);
+        else if(name == "element") columnMapping.mapColumnToStandardProperty(i, Particles::TypeProperty);
         else if(name == "type") {
-            if(!columnMapping.mapStandardColumn(i, Particles::TypeProperty)) {
+            if(!columnMapping.mapColumnToStandardProperty(i, Particles::TypeProperty)) {
                 // Give precedence of the 'type' column over the 'element' column.
                 for(int j = 0; j < i; j++) {
                     if(columnNames[j].compare(QStringLiteral("element"), Qt::CaseInsensitive) == 0) {
                         columnMapping[j].unmap();
-                        columnMapping.mapStandardColumn(i, Particles::TypeProperty);
+                        columnMapping.mapColumnToStandardProperty(i, Particles::TypeProperty);
                         break;
                     }
                 }
             }
         }
-        else if(name == "radius" || name == "diameter") columnMapping.mapStandardColumn(i, Particles::RadiusProperty);
-        else if(name == "mol") columnMapping.mapStandardColumn(i, Particles::MoleculeProperty);
-        else if(name == "q") columnMapping.mapStandardColumn(i, Particles::ChargeProperty);
-        else if(name == "ix") columnMapping.mapStandardColumn(i, Particles::PeriodicImageProperty, 0);
-        else if(name == "iy") columnMapping.mapStandardColumn(i, Particles::PeriodicImageProperty, 1);
-        else if(name == "iz") columnMapping.mapStandardColumn(i, Particles::PeriodicImageProperty, 2);
-        else if(name == "fx" || name == "forces") columnMapping.mapStandardColumn(i, Particles::ForceProperty, 0);
-        else if(name == "fy") columnMapping.mapStandardColumn(i, Particles::ForceProperty, 1);
-        else if(name == "fz") columnMapping.mapStandardColumn(i, Particles::ForceProperty, 2);
-        else if(name == "mux") columnMapping.mapStandardColumn(i, Particles::DipoleOrientationProperty, 0);
-        else if(name == "muy") columnMapping.mapStandardColumn(i, Particles::DipoleOrientationProperty, 1);
-        else if(name == "muz") columnMapping.mapStandardColumn(i, Particles::DipoleOrientationProperty, 2);
-        else if(name == "mu") columnMapping.mapStandardColumn(i, Particles::DipoleMagnitudeProperty);
-        else if(name == "omegax") columnMapping.mapStandardColumn(i, Particles::AngularVelocityProperty, 0);
-        else if(name == "omegay") columnMapping.mapStandardColumn(i, Particles::AngularVelocityProperty, 1);
-        else if(name == "omegaz") columnMapping.mapStandardColumn(i, Particles::AngularVelocityProperty, 2);
-        else if(name == "angmomx") columnMapping.mapStandardColumn(i, Particles::AngularMomentumProperty, 0);
-        else if(name == "angmomy") columnMapping.mapStandardColumn(i, Particles::AngularMomentumProperty, 1);
-        else if(name == "angmomz") columnMapping.mapStandardColumn(i, Particles::AngularMomentumProperty, 2);
-        else if(name == "tqx") columnMapping.mapStandardColumn(i, Particles::TorqueProperty, 0);
-        else if(name == "tqy") columnMapping.mapStandardColumn(i, Particles::TorqueProperty, 1);
-        else if(name == "tqz") columnMapping.mapStandardColumn(i, Particles::TorqueProperty, 2);
-        else if(name == "c_cna" || name == "pattern") columnMapping.mapStandardColumn(i, Particles::StructureTypeProperty);
-        else if(name == "c_epot") columnMapping.mapStandardColumn(i, Particles::PotentialEnergyProperty);
-        else if(name == "c_kpot") columnMapping.mapStandardColumn(i, Particles::KineticEnergyProperty);
-        else if(name == "c_stress[1]") columnMapping.mapStandardColumn(i, Particles::StressTensorProperty, 0);
-        else if(name == "c_stress[2]") columnMapping.mapStandardColumn(i, Particles::StressTensorProperty, 1);
-        else if(name == "c_stress[3]") columnMapping.mapStandardColumn(i, Particles::StressTensorProperty, 2);
-        else if(name == "c_stress[4]") columnMapping.mapStandardColumn(i, Particles::StressTensorProperty, 3);
-        else if(name == "c_stress[5]") columnMapping.mapStandardColumn(i, Particles::StressTensorProperty, 4);
-        else if(name == "c_stress[6]") columnMapping.mapStandardColumn(i, Particles::StressTensorProperty, 5);
-        else if(name == "c_orient[1]" || name == "quati") columnMapping.mapStandardColumn(i, Particles::OrientationProperty, 0);
-        else if(name == "c_orient[2]" || name == "quatj") columnMapping.mapStandardColumn(i, Particles::OrientationProperty, 1);
-        else if(name == "c_orient[3]" || name == "quatk") columnMapping.mapStandardColumn(i, Particles::OrientationProperty, 2);
-        else if(name == "c_orient[4]" || name == "quatw") columnMapping.mapStandardColumn(i, Particles::OrientationProperty, 3);
-        else if(name == "c_shape[1]" || name == "c_diameter[1]" || name == "shapex") columnMapping.mapStandardColumn(i, Particles::AsphericalShapeProperty, 0);
-        else if(name == "c_shape[2]" || name == "c_diameter[2]" || name == "shapey") columnMapping.mapStandardColumn(i, Particles::AsphericalShapeProperty, 1);
-        else if(name == "c_shape[3]" || name == "c_diameter[3]" || name == "shapez") columnMapping.mapStandardColumn(i, Particles::AsphericalShapeProperty, 2);
+        else if(name == "radius" || name == "diameter") columnMapping.mapColumnToStandardProperty(i, Particles::RadiusProperty);
+        else if(name == "mol") columnMapping.mapColumnToStandardProperty(i, Particles::MoleculeProperty);
+        else if(name == "q") columnMapping.mapColumnToStandardProperty(i, Particles::ChargeProperty);
+        else if(name == "ix") columnMapping.mapColumnToStandardProperty(i, Particles::PeriodicImageProperty, 0);
+        else if(name == "iy") columnMapping.mapColumnToStandardProperty(i, Particles::PeriodicImageProperty, 1);
+        else if(name == "iz") columnMapping.mapColumnToStandardProperty(i, Particles::PeriodicImageProperty, 2);
+        else if(name == "fx" || name == "forces") columnMapping.mapColumnToStandardProperty(i, Particles::ForceProperty, 0);
+        else if(name == "fy") columnMapping.mapColumnToStandardProperty(i, Particles::ForceProperty, 1);
+        else if(name == "fz") columnMapping.mapColumnToStandardProperty(i, Particles::ForceProperty, 2);
+        else if(name == "mux") columnMapping.mapColumnToStandardProperty(i, Particles::DipoleOrientationProperty, 0);
+        else if(name == "muy") columnMapping.mapColumnToStandardProperty(i, Particles::DipoleOrientationProperty, 1);
+        else if(name == "muz") columnMapping.mapColumnToStandardProperty(i, Particles::DipoleOrientationProperty, 2);
+        else if(name == "mu") columnMapping.mapColumnToStandardProperty(i, Particles::DipoleMagnitudeProperty);
+        else if(name == "omegax") columnMapping.mapColumnToStandardProperty(i, Particles::AngularVelocityProperty, 0);
+        else if(name == "omegay") columnMapping.mapColumnToStandardProperty(i, Particles::AngularVelocityProperty, 1);
+        else if(name == "omegaz") columnMapping.mapColumnToStandardProperty(i, Particles::AngularVelocityProperty, 2);
+        else if(name == "angmomx") columnMapping.mapColumnToStandardProperty(i, Particles::AngularMomentumProperty, 0);
+        else if(name == "angmomy") columnMapping.mapColumnToStandardProperty(i, Particles::AngularMomentumProperty, 1);
+        else if(name == "angmomz") columnMapping.mapColumnToStandardProperty(i, Particles::AngularMomentumProperty, 2);
+        else if(name == "tqx") columnMapping.mapColumnToStandardProperty(i, Particles::TorqueProperty, 0);
+        else if(name == "tqy") columnMapping.mapColumnToStandardProperty(i, Particles::TorqueProperty, 1);
+        else if(name == "tqz") columnMapping.mapColumnToStandardProperty(i, Particles::TorqueProperty, 2);
+        else if(name == "c_cna" || name == "pattern") columnMapping.mapColumnToStandardProperty(i, Particles::StructureTypeProperty);
+        else if(name == "c_epot") columnMapping.mapColumnToStandardProperty(i, Particles::PotentialEnergyProperty);
+        else if(name == "c_kpot") columnMapping.mapColumnToStandardProperty(i, Particles::KineticEnergyProperty);
+        else if(name == "c_stress[1]") columnMapping.mapColumnToStandardProperty(i, Particles::StressTensorProperty, 0);
+        else if(name == "c_stress[2]") columnMapping.mapColumnToStandardProperty(i, Particles::StressTensorProperty, 1);
+        else if(name == "c_stress[3]") columnMapping.mapColumnToStandardProperty(i, Particles::StressTensorProperty, 2);
+        else if(name == "c_stress[4]") columnMapping.mapColumnToStandardProperty(i, Particles::StressTensorProperty, 3);
+        else if(name == "c_stress[5]") columnMapping.mapColumnToStandardProperty(i, Particles::StressTensorProperty, 4);
+        else if(name == "c_stress[6]") columnMapping.mapColumnToStandardProperty(i, Particles::StressTensorProperty, 5);
+        else if(name == "c_orient[1]" || name == "quati") columnMapping.mapColumnToStandardProperty(i, Particles::OrientationProperty, 0);
+        else if(name == "c_orient[2]" || name == "quatj") columnMapping.mapColumnToStandardProperty(i, Particles::OrientationProperty, 1);
+        else if(name == "c_orient[3]" || name == "quatk") columnMapping.mapColumnToStandardProperty(i, Particles::OrientationProperty, 2);
+        else if(name == "c_orient[4]" || name == "quatw") columnMapping.mapColumnToStandardProperty(i, Particles::OrientationProperty, 3);
+        else if(name == "c_shape[1]" || name == "c_diameter[1]" || name == "shapex") columnMapping.mapColumnToStandardProperty(i, Particles::AsphericalShapeProperty, 0);
+        else if(name == "c_shape[2]" || name == "c_diameter[2]" || name == "shapey") columnMapping.mapColumnToStandardProperty(i, Particles::AsphericalShapeProperty, 1);
+        else if(name == "c_shape[3]" || name == "c_diameter[3]" || name == "shapez") columnMapping.mapColumnToStandardProperty(i, Particles::AsphericalShapeProperty, 2);
         else {
             // Automatically map columns to standard OVITO particle properties.
             bool isStandardProperty = false;
             const static QRegularExpression invalidCharacters(QStringLiteral("[^A-Za-z\\d_]"));
             for(auto entry = Particles::OOClass().standardPropertyIds().cbegin(), end = Particles::OOClass().standardPropertyIds().cend(); entry != end; ++entry) {
-                const auto componentCount = Particles::OOClass().standardPropertyComponentCount(entry.value());
+                const auto componentCount = Particles::OOClass().standardPropertyComponentCount(entry->second);
                 for(size_t component = 0; component < componentCount; component++) {
-                    QString propertyName = entry.key();
+                    QString propertyName = entry->first;
                     propertyName.remove(invalidCharacters); // LAMMPS dump file format does not support column names containing spaces.
-                    const QStringList& componentNames = Particles::OOClass().standardPropertyComponentNames(entry.value());
+                    const QStringList& componentNames = Particles::OOClass().standardPropertyComponentNames(entry->second);
                     QString propertyName2;
                     if(!componentNames.empty()) {
                         OVITO_ASSERT(!componentNames[component].contains(invalidCharacters));
@@ -499,7 +543,7 @@ ParticleInputColumnMapping LAMMPSTextDumpImporter::generateAutomaticColumnMappin
                         propertyName += componentNames[component];
                     }
                     if(propertyName.compare(name, Qt::CaseInsensitive) == 0 || propertyName2.compare(name, Qt::CaseInsensitive) == 0) {
-                        columnMapping.mapStandardColumn(i, (Particles::Type)entry.value(), component);
+                        columnMapping.mapColumnToStandardProperty(i, (Particles::Type)entry->second, component);
                         isStandardProperty = true;
                         break;
                     }
@@ -509,12 +553,24 @@ ParticleInputColumnMapping LAMMPSTextDumpImporter::generateAutomaticColumnMappin
             }
             // If automatic mapping to one of the standard properties was unsuccessful, read the file column as a user-defined property.
             if(!isStandardProperty)
-                columnMapping.mapCustomColumn(i, Property::makePropertyNameValid(name), Property::FloatDefault);
+                columnMapping.mapColumnToUserProperty(i, Property::makePropertyNameValid(name), Property::FloatDefault);
         }
     }
     return columnMapping;
 }
 
+/******************************************************************************
+* Is called when the value of a non-animatable property field of this RefMaker has changed.
+******************************************************************************/
+void LAMMPSTextDumpImporter::propertyChanged(const PropertyFieldDescriptor* field)
+{
+    ParticleImporter::propertyChanged(field);
+
+    if((field == PROPERTY_FIELD(customColumnMapping) || field == PROPERTY_FIELD(useCustomColumnMapping)) && !isBeingLoaded()) {
+        requestReload();
+    }
+}
+
 /******************************************************************************
  * Saves the class' contents to the given stream.
  *****************************************************************************/
@@ -545,8 +601,6 @@ void LAMMPSTextDumpImporter::loadFromStream(ObjectLoadStream& stream)
 ******************************************************************************/
 Future<ParticleInputColumnMapping> LAMMPSTextDumpImporter::inspectFileHeader(const Frame& frame)
 {
-    activateCLocale();
-
     // Retrieve file.
     return Application::instance()->fileManager().fetchUrl(frame.sourceFile)
         .then([](const FileHandle& fileHandle) {
