@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -93,7 +93,8 @@ inline size_t copy_line_from_stream<Ovito::CompressedTextReader&>(char* line, in
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(PDBImporter);
+IMPLEMENT_CREATABLE_OVITO_CLASS(PDBImporter);
+OVITO_CLASSINFO(PDBImporter, "DisplayName", "PDB");
 
 /******************************************************************************
 * Checks if the given file has format that can be read by this importer.
@@ -121,23 +122,21 @@ bool PDBImporter::OOMetaClass::checkFileFormat(const FileHandle& file) const
 /******************************************************************************
 * Scans the data file and builds a list of source frames.
 ******************************************************************************/
-void PDBImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::Frame>& frames)
+void PDBImporter::discoverFramesInFile(const FileHandle& fileHandle, QVector<FileSourceImporter::Frame>& frames) const
 {
-    CompressedTextReader stream(fileHandle());
-    setProgressText(tr("Scanning PDB file %1").arg(stream.filename()));
-    setProgressMaximum(stream.underlyingSize());
+    CompressedTextReader stream(fileHandle);
 
-    Frame frame(fileHandle());
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Scanning PDB file %1").arg(stream.filename()));
+    progress.setMaximum(stream.underlyingSize());
+
+    Frame frame(fileHandle);
     bool endOnPreviousLine = false;
     while(!stream.eof()) {
-
-        if(isCanceled())
-            return;
-
         stream.readLine();
 
-        if(!setProgressValueIntermittent(stream.underlyingByteOffset()))
-            return;
+        // Update progress bar and check for user cancellation.
+        progress.setValueIntermittent(stream.underlyingByteOffset());
 
         if(stream.lineStartsWithToken("ENDMDL")) {
             frames.push_back(frame);
@@ -165,11 +164,6 @@ void PDBImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::
             endOnPreviousLine = false;
         }
     }
-
-    if(frames.empty()) {
-        // It's not a trajectory file. Report just a single frame.
-        frames.push_back(Frame(fileHandle()));
-    }
 }
 
 /******************************************************************************
@@ -177,7 +171,8 @@ void PDBImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::
 ******************************************************************************/
 void PDBImporter::FrameLoader::loadFile()
 {
-    setProgressText(tr("Reading PDB file %1").arg(fileHandle().toString()));
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Reading PDB file %1").arg(fileHandle().toString()));
 
     // Open file for reading.
     CompressedTextReader stream(fileHandle(), frame().byteOffset, frame().lineNumber);
@@ -185,7 +180,7 @@ void PDBImporter::FrameLoader::loadFile()
     try {
         // Parse the PDB file's contents.
         gemmi::Structure structure = gemmi::pdb_impl::read_pdb_from_stream(stream, qPrintable(frame().sourceFile.path()), gemmi::PdbReadOptions());
-        if(isCanceled()) return;
+        this_task::throwIfCanceled();
 
         // Import PDB metadata fields as global attributes.
         for(const auto& m : structure.info) {
@@ -225,7 +220,7 @@ void PDBImporter::FrameLoader::loadFile()
         }
 
         structure.merge_chain_parts();
-        if(isCanceled()) return;
+        this_task::throwIfCanceled();
 
         if(structure.models.empty())
             throw Exception(tr("PDB parsing error: No structural models."));
@@ -262,7 +257,7 @@ void PDBImporter::FrameLoader::loadFile()
         bool hasOccupancy = false;
         for(const gemmi::Chain& chain : model.chains) {
             for(const gemmi::Residue& residue : chain.residues) {
-                if(isCanceled()) return;
+                this_task::throwIfCanceled();
                 int residueTypeId = (residue.name.empty() == false) ? addNamedType(Particles::OOClass(), residueTypeProperty, QLatin1String(residue.name.c_str(), residue.name.size()))->numericId() : 0;
                 for(const gemmi::Atom& atom : residue.atoms) {
                     // Atomic position.
@@ -283,8 +278,7 @@ void PDBImporter::FrameLoader::loadFile()
                 }
             }
         }
-        if(isCanceled())
-            return;
+        this_task::throwIfCanceled();
         typeAccess.reset();
         atomNameAccess.reset();
         residueTypeAccess.reset();
@@ -312,9 +306,9 @@ void PDBImporter::FrameLoader::loadFile()
 
         // Parse unit cell if available.
         //
-        // Gemmi provides the UnitCell::is_crystal() method to determin whether (periodic) cell information
+        // Gemmi provides the UnitCell::is_crystal() method to determine whether (periodic) cell information
         // is available. But it turned out to be too strict. Atomsk writes PDB files containing unity
-        // SCALEn records, which make Gemmi intrept these files as non-periodic. We replace the check
+        // SCALEn records, which make Gemmi interpret these files as non-periodic. We replace the check
         // with a simpler criterion.
         // See https://www.ovito.org/forum/topic/polyhedral-visualazation/#postid-4244.
         if(structure.cell.a != 1.0 || structure.cell.b != 1.0 || structure.cell.c != 1.0 || structure.cell.alpha != 90.0 || structure.cell.beta != 90.0 || structure.cell.gamma != 90.0) {
@@ -383,7 +377,7 @@ void PDBImporter::FrameLoader::loadFile()
 
     // Generate ad-hoc bonds between atoms based on their van der Waals radii.
     if(_generateBonds)
-        generateBonds();
+        generateBonds(progress);
     else
         setBondCount(0);
 
