@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -28,10 +28,12 @@
 #include <ovito/core/utilities/io/NumberParsing.h>
 #include <ovito/core/utilities/io/CompressedTextReader.h>
 #include "POSCARImporter.h"
+#include <ovito/grid/modifier/CreateIsosurfaceModifier.h>
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(POSCARImporter);
+IMPLEMENT_CREATABLE_OVITO_CLASS(POSCARImporter);
+OVITO_CLASSINFO(POSCARImporter, "DisplayName", "POSCAR");
 
 /******************************************************************************
 * Checks if the given file has format that can be read by this importer.
@@ -92,20 +94,22 @@ bool POSCARImporter::shouldScanFileForFrames(const QUrl& sourceUrl) const
 /******************************************************************************
 * Scans the data file and builds a list of source frames.
 ******************************************************************************/
-void POSCARImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::Frame>& frames)
+void POSCARImporter::discoverFramesInFile(const FileHandle& fileHandle, QVector<FileSourceImporter::Frame>& frames) const
 {
-    CompressedTextReader stream(fileHandle());
-    setProgressText(tr("Scanning file %1").arg(fileHandle().toString()));
-    setProgressMaximum(stream.underlyingSize());
+    CompressedTextReader stream(fileHandle);
+
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Scanning file %1").arg(fileHandle.toString()));
+    progress.setMaximum(stream.underlyingSize());
 
     int frameNumber = 0;
     QStringList atomTypeNames;
     QVector<int> atomCounts;
-    QString filename = fileHandle().sourceUrl().fileName();
+    QString filename = fileHandle.sourceUrl().fileName();
 
     // Read frames.
-    Frame frame(fileHandle());
-    while(!stream.eof() && !isCanceled()) {
+    Frame frame(fileHandle);
+    while(!stream.eof() && !this_task::isCanceled()) {
         frame.byteOffset = stream.byteOffset();
         frame.lineNumber = stream.lineNumber();
         frame.parserData = QVariant::fromValue(true);
@@ -167,8 +171,8 @@ void POSCARImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporte
         }
         frames.push_back(frame);
 
-        if(!setProgressValueIntermittent(stream.underlyingByteOffset()))
-            return;
+        // Update progress bar and check for user cancellation.
+        progress.setValueIntermittent(stream.underlyingByteOffset());
     }
 }
 
@@ -177,7 +181,8 @@ void POSCARImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporte
 ******************************************************************************/
 void POSCARImporter::FrameLoader::loadFile()
 {
-    setProgressText(tr("Reading VASP file %1").arg(fileHandle().toString()));
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Reading VASP file %1").arg(fileHandle().toString()));
 
     // Open file for reading.
     CompressedTextReader stream(fileHandle(), frame().byteOffset, frame().lineNumber);
@@ -303,7 +308,7 @@ void POSCARImporter::FrameLoader::loadFile()
         }
         else if(!stream.eof()) {
             // Parse charge density grid.
-            statusString += readDensityGrid(stream);
+            statusString += readDensityGrid(stream, progress);
         }
     }
     posAccess.reset();
@@ -313,7 +318,7 @@ void POSCARImporter::FrameLoader::loadFile()
 
     // Generate ad-hoc bonds between atoms based on their van der Waals radii.
     if(_generateBonds)
-        generateBonds();
+        generateBonds(progress);
     else
         setBondCount(0);
 
@@ -348,10 +353,59 @@ void POSCARImporter::parseAtomTypeNamesAndCounts(CompressedTextReader& stream, Q
     }
 }
 
+/******************************************************************************
+ * This method is called when the pipeline scene node for the FileSource is created.
+ * It adds the Create Isosurface modifier to the pipeline.
+ ******************************************************************************/
+Future<void> POSCARImporter::setupPipeline(OORef<Pipeline> pipeline, OORef<FileSource> importObj)
+{
+    // Add CreateIsosurfaceModifier to pipeline (only in GUI mode).
+    if(!this_task::isInteractive())
+        co_return;
+
+    // Get current time
+    AnimationTime time = this_task::ui()->datasetContainer().currentAnimationTime();
+
+    PipelineFlowState data;
+    try {
+        // Evaluate pipeline and ignore any errors
+        PipelineEvaluationResult future = pipeline->evaluatePipeline(PipelineEvaluationRequest(time));
+        data = co_await FutureAwaiter(ObjectExecutor(this), std::move(future).asFuture());
+    }
+    catch(const Exception&) {
+        co_return;
+    }
+
+    // Get charge density grid from data object -> skip if not found
+    const ConstDataObjectPath path = data.getObject<VoxelGrid>(QStringLiteral("charge-density"));
+    const VoxelGrid* grid = path.lastAs<VoxelGrid>();
+    if(!grid) co_return;
+
+    // Get charge density property from grid -> skip if not found
+    BufferReadAccess<FloatType> chargeAccess(grid->getProperty(QStringLiteral("Charge Density")));
+    if(!chargeAccess || chargeAccess.size() == 0) co_return;
+
+    // Mean will be used as default iso level
+    FloatType mean = std::accumulate(chargeAccess.begin(), chargeAccess.end(), (FloatType)0) / (FloatType)chargeAccess.size();
+
+    // Don't create animation keys for isolevel and surface transparency parameters.
+    AnimationSuspender animSuspender(*this_task::ui());
+
+    // Create and configure the CreateIsosurfaceModifier
+    OORef<CreateIsosurfaceModifier> mod = OORef<CreateIsosurfaceModifier>::create();
+    mod->setIsolevel(mean);
+    mod->setSubject(path);
+    mod->setSourceProperty(QStringLiteral("Charge Density"));
+    mod->surfaceMeshVis()->setSurfaceTransparency(0.333);
+
+    // Add modifier to pipeline
+    pipeline->applyModifier(time, false, mod);
+}
+
 /******************************************************************************
 * Parses a charge density grid.
 ******************************************************************************/
-QString POSCARImporter::FrameLoader::readDensityGrid(CompressedTextReader& stream)
+QString POSCARImporter::FrameLoader::readDensityGrid(CompressedTextReader& stream, TaskProgress& progress)
 {
     QString statusString;
 
@@ -374,7 +428,7 @@ QString POSCARImporter::FrameLoader::readDensityGrid(CompressedTextReader& strea
     voxelGrid->setContent(gridSize[0] * gridSize[1] * gridSize[2], {});
 
     // Parse spin up + spin down density.
-    if(!readFieldQuantity(stream, voxelGrid, tr("Charge density")))
+    if(!readFieldQuantity(stream, voxelGrid, QStringLiteral("Charge Density"), progress))
         return {};
     statusString += tr("\nCharge density grid: %1 x %2 x %3").arg(gridSize[0]).arg(gridSize[1]).arg(gridSize[2]);
 
@@ -384,7 +438,7 @@ QString POSCARImporter::FrameLoader::readDensityGrid(CompressedTextReader& strea
         if(sscanf(stream.readLine(), "%zu %zu %zu", &gridSize[0], &gridSize[1], &gridSize[2]) == 3) {
             if(gridSize != voxelGrid->shape())
                 throw Exception(tr("Inconsistent voxel grid dimensions in line %1").arg(stream.lineNumber()));
-            magnetizationDensityX = readFieldQuantity(stream, voxelGrid, tr("Magnetization density"));
+            magnetizationDensityX = readFieldQuantity(stream, voxelGrid, QStringLiteral("Magnetization Density"), progress);
             if(!magnetizationDensityX) return {};
             statusString += tr("\nMagnetization density grid: %1 x %2 x %3").arg(gridSize[0]).arg(gridSize[1]).arg(gridSize[2]);
             break;
@@ -398,7 +452,7 @@ QString POSCARImporter::FrameLoader::readDensityGrid(CompressedTextReader& strea
         if(sscanf(stream.readLine(), "%zu %zu %zu", &gridSize[0], &gridSize[1], &gridSize[2]) == 3) {
             if(gridSize != voxelGrid->shape())
                 throw Exception(tr("Inconsistent voxel grid dimensions in line %1").arg(stream.lineNumber()));
-            magnetizationDensityY = readFieldQuantity(stream, voxelGrid, tr("Magnetization density"));
+            magnetizationDensityY = readFieldQuantity(stream, voxelGrid, QStringLiteral("Magnetization Density"), progress);
             if(!magnetizationDensityY) return {};
             break;
         }
@@ -407,14 +461,14 @@ QString POSCARImporter::FrameLoader::readDensityGrid(CompressedTextReader& strea
         if(sscanf(stream.readLine(), "%zu %zu %zu", &gridSize[0], &gridSize[1], &gridSize[2]) == 3) {
             if(gridSize != voxelGrid->shape())
                 throw Exception(tr("Inconsistent voxel grid dimensions in line %1").arg(stream.lineNumber()));
-            magnetizationDensityZ = readFieldQuantity(stream, voxelGrid, tr("Magnetization density"));
+            magnetizationDensityZ = readFieldQuantity(stream, voxelGrid, QStringLiteral("Magnetization Density"), progress);
             if(!magnetizationDensityZ) return {};
             break;
         }
     }
 
     if(magnetizationDensityX && magnetizationDensityY && magnetizationDensityZ) {
-        BufferWriteAccess<FloatType*, access_mode::discard_write> vectorMagnetization = voxelGrid->createProperty(tr("Magnetization density"), DataBuffer::FloatDefault, 3, QStringList() << "X" << "Y" << "Z");
+        BufferWriteAccess<FloatType*, access_mode::discard_write> vectorMagnetization = voxelGrid->createProperty(QStringLiteral("Magnetization Density"), DataBuffer::FloatDefault, 3, QStringList() << "X" << "Y" << "Z");
         boost::copy(BufferReadAccess<FloatType>(magnetizationDensityX), vectorMagnetization.componentRange(0).begin());
         boost::copy(BufferReadAccess<FloatType>(magnetizationDensityY), vectorMagnetization.componentRange(1).begin());
         boost::copy(BufferReadAccess<FloatType>(magnetizationDensityZ), vectorMagnetization.componentRange(2).begin());
@@ -427,13 +481,13 @@ QString POSCARImporter::FrameLoader::readDensityGrid(CompressedTextReader& strea
 /******************************************************************************
 * Parses the values of one field quantity.
 ******************************************************************************/
-Property* POSCARImporter::FrameLoader::readFieldQuantity(CompressedTextReader& stream, VoxelGrid* grid, const QString& name)
+Property* POSCARImporter::FrameLoader::readFieldQuantity(CompressedTextReader& stream, VoxelGrid* grid, const QString& name, TaskProgress& progress)
 {
     Property* fieldProperty = grid->createProperty(name, DataBuffer::FloatDefault);
     BufferWriteAccess<FloatType*, access_mode::discard_read_write> fieldAccess(fieldProperty);
     const char* s = stream.readLine();
     auto* data = fieldAccess.begin();
-    setProgressMaximum(fieldProperty->size());
+    progress.setMaximum(fieldProperty->size());
     FloatType cellVolume = std::abs(simulationCell()->cellMatrix().determinant());
     for(size_t i = 0; i < fieldProperty->size(); i++, ++data) {
         const char* token;
@@ -450,7 +504,8 @@ Property* POSCARImporter::FrameLoader::readFieldQuantity(CompressedTextReader& s
         if(*s != '\0')
             s++;
 
-        if(!setProgressValueIntermittent(i)) return nullptr;
+        // Update progress bar and check for user cancellation.
+        progress.setValueIntermittent(i);
     }
     return fieldProperty;
 }
