@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -35,7 +35,8 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(XSFImporter);
+IMPLEMENT_CREATABLE_OVITO_CLASS(XSFImporter);
+OVITO_CLASSINFO(XSFImporter, "DisplayName", "XSF");
 
 /******************************************************************************
 * Checks if the given file has format that can be read by this importer.
@@ -68,14 +69,16 @@ bool XSFImporter::OOMetaClass::checkFileFormat(const FileHandle& file) const
 /******************************************************************************
 * Scans the data file and builds a list of source frames.
 ******************************************************************************/
-void XSFImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::Frame>& frames)
+void XSFImporter::discoverFramesInFile(const FileHandle& fileHandle, QVector<FileSourceImporter::Frame>& frames) const
 {
-    CompressedTextReader stream(fileHandle());
-    setProgressText(tr("Scanning XSF file %1").arg(stream.filename()));
-    setProgressMaximum(stream.underlyingSize());
+    CompressedTextReader stream(fileHandle);
+
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Scanning XSF file %1").arg(stream.filename()));
+    progress.setMaximum(stream.underlyingSize());
 
     int nFrames = 1;
-    while(!stream.eof() && !isCanceled()) {
+    while(!stream.eof()) {
         const char* line = stream.readLineTrimLeft(1024);
         if(boost::algorithm::starts_with(line, "ANIMSTEPS")) {
             if(sscanf(line, "ANIMSTEPS %i", &nFrames) != 1 || nFrames < 1)
@@ -85,11 +88,11 @@ void XSFImporter::FrameFinder::discoverFramesInFile(QVector<FileSourceImporter::
         else if(line[0] != '#') {
             break;
         }
-        setProgressValueIntermittent(stream.underlyingByteOffset());
+        progress.setValueIntermittent(stream.underlyingByteOffset());
     }
 
-    Frame frame(fileHandle());
-    QString filename = fileHandle().sourceUrl().fileName();
+    Frame frame(fileHandle);
+    QString filename = fileHandle.sourceUrl().fileName();
     for(int i = 0; i < nFrames; i++) {
         frame.lineNumber = i;
         frame.label = tr("%1 (Frame %2)").arg(filename).arg(i);
@@ -104,16 +107,19 @@ void XSFImporter::FrameLoader::loadFile()
 {
     // Open file for reading.
     CompressedTextReader stream(fileHandle());
-    setProgressText(tr("Reading XSF file %1").arg(fileHandle().toString()));
+
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Reading XSF file %1").arg(fileHandle().toString()));
 
     // The animation frame number to load from the XSF file.
     int frameNumber = frame().lineNumber + 1;
 
     VoxelGrid* voxelGrid = nullptr;
     VoxelGridVis* newVoxelGridVis = nullptr;
+    bool containsSimulationCellInfo = false;
 
     while(!stream.eof()) {
-        if(isCanceled()) return;
+        this_task::throwIfCanceled();
         const char* line = stream.readLineTrimLeft(1024);
         if(boost::algorithm::starts_with(line, "ATOMS")) {
 
@@ -143,7 +149,7 @@ void XSFImporter::FrameLoader::loadFile()
                     forces.resize(coords.size(), Vector3::Zero());
                     forces.back() = f;
                 }
-                if(isCanceled()) return;
+                this_task::throwIfCanceled();
             }
             if(coords.empty())
                 throw Exception(tr("Invalid ATOMS section in line %1 of XSF file.").arg(stream.lineNumber()));
@@ -172,7 +178,7 @@ void XSFImporter::FrameLoader::loadFile()
             state().setStatus(tr("%1 atoms").arg(coords.size()));
 
             // If the input file does not contain simulation cell info,
-            // Use bounding box of particles as simulation cell.
+            // Use the axis-aligned bounding box of particles as a simulation cell.
             Box3 boundingBox;
             boundingBox.addPoints(posAccess);
             simulationCell()->setCellMatrix(AffineTransformation(
@@ -206,6 +212,7 @@ void XSFImporter::FrameLoader::loadFile()
                     throw Exception(tr("Invalid cell vector in XSF file at line %1").arg(stream.lineNumber()));
             }
             simulationCell()->setCellMatrix(cell);
+            containsSimulationCellInfo = true;
         }
         else if(boost::algorithm::starts_with(line, "PRIMCOORD")) {
             int anim;
@@ -234,14 +241,14 @@ void XSFImporter::FrameLoader::loadFile()
             // Prepare the file column to particle property mapping.
             ParticleInputColumnMapping columnMapping;
             columnMapping.resize(nfields + 1);
-            columnMapping.mapStandardColumn(0, Particles::TypeProperty);
-            columnMapping.mapStandardColumn(1, Particles::PositionProperty, 0);
-            columnMapping.mapStandardColumn(2, Particles::PositionProperty, 1);
-            columnMapping.mapStandardColumn(3, Particles::PositionProperty, 2);
+            columnMapping.mapColumnToStandardProperty(0, Particles::TypeProperty);
+            columnMapping.mapColumnToStandardProperty(1, Particles::PositionProperty, 0);
+            columnMapping.mapColumnToStandardProperty(2, Particles::PositionProperty, 1);
+            columnMapping.mapColumnToStandardProperty(3, Particles::PositionProperty, 2);
             if(nfields == 6) {
-                columnMapping.mapStandardColumn(4, Particles::ForceProperty, 0);
-                columnMapping.mapStandardColumn(5, Particles::ForceProperty, 1);
-                columnMapping.mapStandardColumn(6, Particles::ForceProperty, 2);
+                columnMapping.mapColumnToStandardProperty(4, Particles::ForceProperty, 0);
+                columnMapping.mapColumnToStandardProperty(5, Particles::ForceProperty, 1);
+                columnMapping.mapColumnToStandardProperty(6, Particles::ForceProperty, 2);
             }
 
             // Jump back to start of atoms list.
@@ -249,15 +256,16 @@ void XSFImporter::FrameLoader::loadFile()
 
             // Parse atoms data.
             InputColumnReader columnParser(*this, columnMapping, particles());
-            setProgressMaximum(natoms);
+            progress.setMaximum(natoms);
             for(size_t i = 0; i < natoms; i++) {
-                if(!setProgressValueIntermittent(i)) return;
                 try {
                     columnParser.readElement(i, stream.readLine());
                 }
                 catch(Exception& ex) {
                     throw ex.prependGeneralMessage(tr("Parsing error in line %1 of XSF file.").arg(atomsLineNumber + i));
                 }
+                // Update progress bar and check for user cancellation.
+                progress.setValueIntermittent(i);
             }
             columnParser.sortElementTypes();
             columnParser.reset();
@@ -270,7 +278,7 @@ void XSFImporter::FrameLoader::loadFile()
                     if(type->name().isEmpty() && typeId >= 0 && typeId < ParticleType::NUMBER_OF_PREDEFINED_PARTICLE_TYPES) {
                         ElementType* mutableType = typeProperty->makeMutable(type);
                         mutableType->setName(ParticleType::getPredefinedParticleTypeName(static_cast<ParticleType::PredefinedParticleType>(typeId)));
-                        mutableType->initializeType(ParticlePropertyReference(typeProperty));
+                        mutableType->initializeType(OwnerPropertyRef(&Particles::OOClass(), typeProperty));
                     }
                 }
             }
@@ -321,16 +329,25 @@ void XSFImporter::FrameLoader::loadFile()
             }
             if(voxelGrid->domain()) {
                 voxelGrid->mutableDomain()->setCellMatrix(cell);
+                if(!containsSimulationCellInfo)
+                    simulationCell()->setCellMatrix(cell);
             }
             else {
-                DataOORef<SimulationCell> simCell = DataOORef<SimulationCell>::create(cell, true, true, true, false);
-                simCell->setCreatedByNode(pipelineNode());
-                voxelGrid->setDomain(std::move(simCell));
+                if(containsSimulationCellInfo) {
+                    DataOORef<SimulationCell> simCell = DataOORef<SimulationCell>::create(cell, true, true, true, false);
+                    simCell->setCreatedByNode(pipelineNode());
+                    voxelGrid->setDomain(std::move(simCell));
+                }
+                else {
+                    simulationCell()->setCellMatrix(cell);
+                    simulationCell()->setPbcFlags(true, true, true);
+                    voxelGrid->setDomain(simulationCell());
+                }
             }
 
             BufferWriteAccess<FloatType, access_mode::discard_read_write> fieldQuantity = voxelGrid->createProperty(name, DataBuffer::FloatDefault);
             FloatType* data = fieldQuantity.begin();
-            setProgressMaximum(fieldQuantity.size());
+            progress.setMaximum(fieldQuantity.size());
             const char* s = "";
             for(size_t i = 0; i < fieldQuantity.size(); i++, ++data) {
                 const char* token;
@@ -346,13 +363,14 @@ void XSFImporter::FrameLoader::loadFile()
                 if(*s != '\0')
                     s++;
 
-                if(!setProgressValueIntermittent(i)) return;
+                // Update progress bar and check for user cancellation.
+                progress.setValueIntermittent(i);
             }
 
             // Automatically select the property for pseudo-coloring of the grid and adjust the value range.
             // But only if this is the first time the file reader is loading the file.
-            if(newVoxelGridVis && newVoxelGridVis->colorMapping()->sourceProperty().isNull() && fieldQuantity.size() != 0) {
-                newVoxelGridVis->colorMapping()->setSourceProperty(VoxelPropertyReference(name));
+            if(newVoxelGridVis && !newVoxelGridVis->colorMapping()->sourceProperty() && fieldQuantity.size() != 0) {
+                newVoxelGridVis->colorMapping()->setSourceProperty(name);
                 auto [min, max] = std::minmax_element(fieldQuantity.cbegin(), fieldQuantity.cend());
                 newVoxelGridVis->colorMapping()->setStartValue(*min);
                 newVoxelGridVis->colorMapping()->setEndValue(*max);
