@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -23,14 +23,14 @@
 #include <ovito/particles/Particles.h>
 #include <ovito/particles/objects/Particles.h>
 #include <ovito/core/dataset/pipeline/ModificationNode.h>
-#include <ovito/core/dataset/pipeline/PipelineEvaluation.h>
+#include <ovito/core/dataset/pipeline/PipelineEvaluationRequest.h>
 #include <ovito/core/dataset/animation/AnimationSettings.h>
 #include <ovito/core/utilities/units/UnitsManager.h>
 #include "ReferenceConfigurationModifier.h"
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(ReferenceConfigurationModifier);
+IMPLEMENT_ABSTRACT_OVITO_CLASS(ReferenceConfigurationModifier);
 DEFINE_REFERENCE_FIELD(ReferenceConfigurationModifier, referenceConfiguration);
 DEFINE_PROPERTY_FIELD(ReferenceConfigurationModifier, affineMapping);
 DEFINE_PROPERTY_FIELD(ReferenceConfigurationModifier, useMinimumImageConvention);
@@ -46,19 +46,7 @@ SET_PROPERTY_FIELD_LABEL(ReferenceConfigurationModifier, referenceFrameOffset, "
 SET_PROPERTY_FIELD_UNITS_AND_MINIMUM(ReferenceConfigurationModifier, referenceFrameNumber, IntegerParameterUnit, 0);
 
 // This class can be removed in a future version of OVITO:
-IMPLEMENT_OVITO_CLASS(ReferenceConfigurationModifierApplication);
-
-/******************************************************************************
-* Constructs the modifier object.
-******************************************************************************/
-ReferenceConfigurationModifier::ReferenceConfigurationModifier(ObjectInitializationFlags flags) : AsynchronousModifier(flags),
-    _affineMapping(NO_MAPPING),
-    _useReferenceFrameOffset(false),
-    _referenceFrameNumber(0),
-    _referenceFrameOffset(-1),
-    _useMinimumImageConvention(true)
-{
-}
+IMPLEMENT_CREATABLE_OVITO_CLASS(ReferenceConfigurationModifierApplication);
 
 /******************************************************************************
 * Asks the modifier whether it can be applied to the given input data.
@@ -69,45 +57,58 @@ bool ReferenceConfigurationModifier::OOMetaClass::isApplicableTo(const DataColle
 }
 
 /******************************************************************************
-* Determines the time interval over which a computed pipeline state will remain valid.
+* Throws an exception if the pipeline stage cannot be evaluated at this time.
+* This is called by the system to catch user mistakes that would lead to infinite recursion.
 ******************************************************************************/
-TimeInterval ReferenceConfigurationModifier::validityInterval(const ModifierEvaluationRequest& request) const
+void ReferenceConfigurationModifier::preEvaluationCheck(const PipelineEvaluationRequest& request) const
 {
-    TimeInterval iv = AsynchronousModifier::validityInterval(request);
+    // Walk up the reference config pipeline and ask each step if evaluation is allowed at this time.
+    if(referenceConfiguration())
+        referenceConfiguration()->preEvaluationCheck(request);
+}
+
+/******************************************************************************
+ * Is called by the pipeline system before a new modifier evaluation begins.
+ ******************************************************************************/
+void ReferenceConfigurationModifier::preevaluateModifier(const ModifierEvaluationRequest& request, PipelineEvaluationResult::EvaluationTypes& evaluationTypes, TimeInterval& validityInterval) const
+{
+    // Indicate that we will do different computations depending on whether the pipeline is evaluated in interactive mode or not.
+    if(request.interactiveMode())
+        evaluationTypes = PipelineEvaluationResult::EvaluationType::Interactive;
+    else
+        evaluationTypes = PipelineEvaluationResult::EvaluationType::Noninteractive;
 
     if(useReferenceFrameOffset()) {
         // Results will only be valid for the duration of the current frame when using a relative offset.
-        iv.intersect(request.time());
+        validityInterval.intersect(request.time());
     }
-    return iv;
 }
 
 /******************************************************************************
 * Asks the modifier for the set of animation time intervals that should be
 * cached by the upstream pipeline.
 ******************************************************************************/
-void ReferenceConfigurationModifier::inputCachingHints(TimeIntervalUnion& cachingIntervals, ModificationNode* node)
+void ReferenceConfigurationModifier::inputCachingHints(ModifierEvaluationRequest& request)
 {
-    AsynchronousModifier::inputCachingHints(cachingIntervals, node);
-
     // Only need to communicate caching hints when reference configuration is provided by the upstream pipeline.
     if(!referenceConfiguration()) {
         if(useReferenceFrameOffset()) {
             // When using a relative reference configuration, we need to build the corresponding set of shifted time intervals.
-            TimeIntervalUnion originalIntervals = cachingIntervals;
-            for(const TimeInterval& iv : originalIntervals) {
-                int startFrame = node->animationTimeToSourceFrame(iv.start());
-                int endFrame = node->animationTimeToSourceFrame(iv.end());
-                AnimationTime shiftedStartTime = node->sourceFrameToAnimationTime(startFrame + referenceFrameOffset());
-                AnimationTime shiftedEndTime = node->sourceFrameToAnimationTime(endFrame + referenceFrameOffset());
-                cachingIntervals.add(TimeInterval(shiftedStartTime, shiftedEndTime));
+            for(const TimeInterval& iv : TimeIntervalUnion(request.cachingIntervals())) {
+                int startFrame = request.modificationNode()->animationTimeToSourceFrame(iv.start());
+                int endFrame = request.modificationNode()->animationTimeToSourceFrame(iv.end());
+                AnimationTime shiftedStartTime = request.modificationNode()->sourceFrameToAnimationTime(startFrame + referenceFrameOffset());
+                AnimationTime shiftedEndTime = request.modificationNode()->sourceFrameToAnimationTime(endFrame + referenceFrameOffset());
+                request.mutableCachingIntervals().add(TimeInterval(shiftedStartTime, shiftedEndTime));
             }
         }
         else {
             // When using a static reference configuration, ask the upstream pipeline to cache the corresponding animation frame.
-            cachingIntervals.add(node->sourceFrameToAnimationTime(referenceFrameNumber()));
+            request.mutableCachingIntervals().add(request.modificationNode()->sourceFrameToAnimationTime(referenceFrameNumber()));
         }
     }
+
+    Modifier::inputCachingHints(request);
 }
 
 /******************************************************************************
@@ -117,7 +118,7 @@ void ReferenceConfigurationModifier::inputCachingHints(TimeIntervalUnion& cachin
 ******************************************************************************/
 void ReferenceConfigurationModifier::restrictInputValidityInterval(TimeInterval& iv) const
 {
-    AsynchronousModifier::restrictInputValidityInterval(iv);
+    Modifier::restrictInputValidityInterval(iv);
 
     if(!referenceConfiguration()) {
         // If the upstream pipeline changes, all computed output frames of the modifier become invalid.
@@ -126,7 +127,7 @@ void ReferenceConfigurationModifier::restrictInputValidityInterval(TimeInterval&
 }
 
 /******************************************************************************
-* Is called when a RefTarget referenced by this object has generated an event.
+* Is called when a RefTarget referenced by this object generated an event.
 ******************************************************************************/
 bool ReferenceConfigurationModifier::referenceEvent(RefTarget* source, const ReferenceEvent& event)
 {
@@ -136,30 +137,35 @@ bool ReferenceConfigurationModifier::referenceEvent(RefTarget* source, const Ref
         notifyTargetChanged();
         return false;
     }
-    return AsynchronousModifier::referenceEvent(source, event);
+    return Modifier::referenceEvent(source, event);
 }
 
 /******************************************************************************
-* Creates and initializes a computation engine that will compute the
-* modifier's results.
+* Modifies the input data.
 ******************************************************************************/
-Future<AsynchronousModifier::EnginePtr> ReferenceConfigurationModifier::createEngine(const ModifierEvaluationRequest& request, const PipelineFlowState& input)
+Future<PipelineFlowState> ReferenceConfigurationModifier::evaluateModifier(const ModifierEvaluationRequest& request, PipelineFlowState&& state)
 {
+    // In interactive mode, do not perform a real computation. Instead, reuse an old result from the cached state if available.
+    if(request.interactiveMode()) {
+        if(PipelineFlowState cachedState = request.modificationNode()->getCachedPipelineNodeOutput(request.time(), true)) {
+            Particles* particles = state.expectMutableObject<Particles>();
+            particles->verifyIntegrity();
+            return reuseCachedState(request, particles, std::move(state), cachedState);
+        }
+        return std::move(state);
+    }
+
     // What is the reference frame number to use?
-    TimeInterval validityInterval = input.stateValidity();
     int referenceFrame;
     if(useReferenceFrameOffset()) {
         // Determine the current frame, preferably from the marker attribute stored in the pipeline flow state.
         // If the source frame attribute is not present, fall back to inferring it from the current animation time.
-        int currentFrame = input.data() ? input.data()->sourceFrame() : -1;
+        int currentFrame = state.data() ? state.data()->sourceFrame() : -1;
         if(currentFrame < 0)
             currentFrame = request.modificationNode()->animationTimeToSourceFrame(request.time());
 
         // Use frame offset relative to current configuration.
         referenceFrame = currentFrame + referenceFrameOffset();
-
-        // Results will only be valid for the duration of the current frame.
-        validityInterval.intersect(request.time());
     }
     else {
         // Use a constant, user-specified frame as reference configuration.
@@ -167,15 +173,12 @@ Future<AsynchronousModifier::EnginePtr> ReferenceConfigurationModifier::createEn
     }
 
     // Obtain the reference positions of the particles, either from the upstream pipeline or from a user-specified reference data source.
-    SharedFuture<PipelineFlowState> refState;
+    PipelineEvaluationResult refState;
     if(!referenceConfiguration()) {
-        // Convert frame to animation time.
-        AnimationTime referenceTime = request.modificationNode()->sourceFrameToAnimationTime(referenceFrame);
-
         // Set up the pipeline request for obtaining the reference configuration.
-        PipelineEvaluationRequest referenceRequest = request;
-        referenceRequest.setTime(referenceTime);
-        inputCachingHints(referenceRequest.modifiableCachingIntervals(), request.modificationNode());
+        ModifierEvaluationRequest referenceRequest = request;
+        referenceRequest.setTime(request.modificationNode()->sourceFrameToAnimationTime(referenceFrame));
+        inputCachingHints(referenceRequest);
 
         // Send the request to the upstream pipeline.
         refState = request.modificationNode()->evaluateInput(referenceRequest);
@@ -202,12 +205,12 @@ Future<AsynchronousModifier::EnginePtr> ReferenceConfigurationModifier::createEn
         }
         else {
             // Create an empty state for the reference configuration if it is yet to be specified by the user.
-            refState = Future<PipelineFlowState>::createImmediateEmplace();
+            refState = PipelineFlowState{};
         }
     }
 
     // Wait for the reference configuration to become available.
-    return refState.then(*this, [this, request, input = input, referenceFrame, validityInterval](const PipelineFlowState& referenceInput) {
+    return refState.then(ObjectExecutor(this), [this, request, state = std::move(state), referenceFrame](const PipelineFlowState& referenceInput) mutable {
 
         // Make sure the obtained reference configuration is valid and ready to use.
         if(referenceInput.status().type() == PipelineStatus::Error)
@@ -224,21 +227,28 @@ Future<AsynchronousModifier::EnginePtr> ReferenceConfigurationModifier::createEn
         }
 
         // Let subclass create the compute engine.
-        return createEngineInternal(request, std::move(input), referenceInput, validityInterval);
+        std::unique_ptr<Engine> engine = createEngine(request, state, referenceInput);
+
+        // Perform the actual calculation in a separate thread.
+        return asyncLaunch([
+                state = std::move(state),
+                engine = std::move(engine)]() mutable
+        {
+            // Run the algorithm.
+            engine->perform(state);
+            return std::move(state);
+        });
     });
 }
 
 /******************************************************************************
 * Constructor.
 ******************************************************************************/
-ReferenceConfigurationModifier::RefConfigEngineBase::RefConfigEngineBase(
-    const ModifierEvaluationRequest& request,
-    const TimeInterval& validityInterval,
+ReferenceConfigurationModifier::Engine::Engine(
     ConstPropertyPtr positions, const SimulationCell* simCell,
     ConstPropertyPtr refPositions, const SimulationCell* simCellRef,
     ConstPropertyPtr identifiers, ConstPropertyPtr refIdentifiers,
     AffineMappingType affineMapping, bool useMinimumImageConvention) :
-    Engine(request, validityInterval),
     _positions(std::move(positions)),
     _refPositions(std::move(refPositions)),
     _identifiers(std::move(identifiers)),
@@ -282,7 +292,7 @@ ReferenceConfigurationModifier::RefConfigEngineBase::RefConfigEngineBase(
 * Determines the mapping between particles in the reference configuration and
 * the current configuration and vice versa.
 ******************************************************************************/
-bool ReferenceConfigurationModifier::RefConfigEngineBase::buildParticleMapping(bool requireCompleteCurrentToRefMapping, bool requireCompleteRefToCurrentMapping)
+void ReferenceConfigurationModifier::Engine::buildParticleMapping(bool requireCompleteCurrentToRefMapping, bool requireCompleteRefToCurrentMapping)
 {
     // Build particle-to-particle index maps.
     _currentToRefIndexMap.resize(positions()->size());
@@ -300,9 +310,7 @@ bool ReferenceConfigurationModifier::RefConfigEngineBase::buildParticleMapping(b
                 throw Exception(tr("Particles with duplicate identifiers detected in reference configuration."));
             index++;
         }
-
-        if(isCanceled())
-            return false;
+        this_task::throwIfCanceled();
 
         // Check for duplicate identifiers in current configuration
         std::map<IdentifierIntType, size_t> currentMap;
@@ -313,9 +321,7 @@ bool ReferenceConfigurationModifier::RefConfigEngineBase::buildParticleMapping(b
                 throw Exception(tr("Particles with duplicate identifiers detected in current configuration."));
             index++;
         }
-
-        if(isCanceled())
-            return false;
+        this_task::throwIfCanceled();
 
         // Build index maps.
         auto id = identifiersArray.cbegin();
@@ -329,9 +335,7 @@ bool ReferenceConfigurationModifier::RefConfigEngineBase::buildParticleMapping(b
                 mappedIndex = std::numeric_limits<size_t>::max();
             ++id;
         }
-
-        if(isCanceled())
-            return false;
+        this_task::throwIfCanceled();
 
         id = refIdentifiersArray.cbegin();
         for(auto& mappedIndex : _refToCurrentIndexMap) {
@@ -352,11 +356,11 @@ bool ReferenceConfigurationModifier::RefConfigEngineBase::buildParticleMapping(b
 
         // When particle identifiers are not available, assume the storage order of particles in the
         // reference configuration and the current configuration are the same and use trivial 1-to-1 mapping.
-        std::iota(_refToCurrentIndexMap.begin(), _refToCurrentIndexMap.end(), size_t(0));
-        std::iota(_currentToRefIndexMap.begin(), _currentToRefIndexMap.end(), size_t(0));
+        boost::algorithm::iota(_refToCurrentIndexMap, size_t(0));
+        boost::algorithm::iota(_currentToRefIndexMap, size_t(0));
     }
 
-    return !isCanceled();
+    this_task::throwIfCanceled();
 }
 
 }   // End of namespace
