@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -29,15 +29,18 @@
 #include <ovito/stdobj/simcell/SimulationCell.h>
 #include <ovito/core/dataset/pipeline/ModificationNode.h>
 #include <ovito/core/dataset/DataSet.h>
+#include <ovito/core/dataset/data/AttributeDataObject.h>
 #include <ovito/core/utilities/units/UnitsManager.h>
-#include <ovito/core/app/Application.h>
 #include "ClusterAnalysisModifier.h"
 
 #include <boost/range/combine.hpp>
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(ClusterAnalysisModifier);
+IMPLEMENT_CREATABLE_OVITO_CLASS(ClusterAnalysisModifier);
+OVITO_CLASSINFO(ClusterAnalysisModifier, "DisplayName", "Cluster analysis");
+OVITO_CLASSINFO(ClusterAnalysisModifier, "Description", "Decompose a particle-based structure into disconnected clusters.");
+OVITO_CLASSINFO(ClusterAnalysisModifier, "ModifierCategory", "Analysis");
 DEFINE_PROPERTY_FIELD(ClusterAnalysisModifier, neighborMode);
 DEFINE_PROPERTY_FIELD(ClusterAnalysisModifier, cutoff);
 DEFINE_PROPERTY_FIELD(ClusterAnalysisModifier, onlySelectedParticles);
@@ -56,21 +59,6 @@ SET_PROPERTY_FIELD_LABEL(ClusterAnalysisModifier, computeRadiusOfGyration, "Comp
 SET_PROPERTY_FIELD_LABEL(ClusterAnalysisModifier, colorParticlesByCluster, "Color particles by cluster");
 SET_PROPERTY_FIELD_UNITS_AND_MINIMUM(ClusterAnalysisModifier, cutoff, WorldParameterUnit, 0);
 
-/******************************************************************************
-* Constructs the modifier object.
-******************************************************************************/
-ClusterAnalysisModifier::ClusterAnalysisModifier(ObjectInitializationFlags flags) : AsynchronousModifier(flags),
-    _cutoff(3.2),
-    _onlySelectedParticles(false),
-    _sortBySize(false),
-    _neighborMode(CutoffRange),
-    _unwrapParticleCoordinates(false),
-    _computeCentersOfMass(false),
-    _computeRadiusOfGyration(false),
-    _colorParticlesByCluster(false)
-{
-}
-
 /******************************************************************************
 * Asks the modifier whether it can be applied to the given input data.
 ******************************************************************************/
@@ -80,17 +68,58 @@ bool ClusterAnalysisModifier::OOMetaClass::isApplicableTo(const DataCollection&
 }
 
 /******************************************************************************
-* Creates and initializes a computation engine that will compute the modifier's results.
+ * Is called by the pipeline system before a new modifier evaluation begins.
+ ******************************************************************************/
+void ClusterAnalysisModifier::preevaluateModifier(const ModifierEvaluationRequest& request, PipelineEvaluationResult::EvaluationTypes& evaluationTypes, TimeInterval& validityInterval) const
+{
+    // Indicate that we will do different computations depending on whether the pipeline is evaluated in interactive mode or not.
+    if(request.interactiveMode())
+        evaluationTypes = PipelineEvaluationResult::EvaluationType::Interactive;
+    else
+        evaluationTypes = PipelineEvaluationResult::EvaluationType::Noninteractive;
+}
+
+/******************************************************************************
+* Modifies the input data.
 ******************************************************************************/
-Future<AsynchronousModifier::EnginePtr> ClusterAnalysisModifier::createEngine(const ModifierEvaluationRequest& request, const PipelineFlowState& input)
+Future<PipelineFlowState> ClusterAnalysisModifier::evaluateModifier(const ModifierEvaluationRequest& request, PipelineFlowState&& state)
 {
     // Get the current particle positions.
-    const Particles* particles = input.expectObject<Particles>();
+    Particles* particles = state.expectMutableObject<Particles>();
     particles->verifyIntegrity();
+
+    // In interactive mode, do not perform a real computation. Instead, reuse old results if available in the pipeline cache.
+    if(request.interactiveMode()) {
+        if(PipelineFlowState cachedState = request.modificationNode()->getCachedPipelineNodeOutput(request.time(), true)) {
+            if(const Particles* cachedParticles = cachedState.getObject<Particles>()) {
+
+                particles->tryToAdoptProperties(cachedParticles, {
+                    cachedParticles->getProperty(Particles::ClusterProperty),
+                    colorParticlesByCluster() ? cachedParticles->getProperty(Particles::ColorProperty) : nullptr,
+                    unwrapParticleCoordinates() ? cachedParticles->getProperty(Particles::PositionProperty) : nullptr,
+                }, {particles});
+
+                if(unwrapParticleCoordinates()) {
+                    if(particles->bonds() && cachedParticles->bonds() && cachedParticles->bonds()->elementCount() == particles->bonds()->elementCount()) {
+                        if(const Property* cachedImages = cachedParticles->bonds()->getProperty(Bonds::PeriodicImageProperty)) {
+                            particles->makeBondsMutable()->createProperty(cachedImages);
+                        }
+                    }
+                }
+            }
+            if(const DataTable* cachedTable = cachedState.getObjectBy<DataTable>(request.modificationNode(), QStringLiteral("clusters"))) {
+                state.addObject(cachedTable);
+            }
+            // Adopt all global attributes computed by the modifier from the cached state.
+            state.adoptAttributesFrom(cachedState, request.modificationNode());
+        }
+        return std::move(state);
+    }
+
     const Property* posProperty = particles->expectProperty(Particles::PositionProperty);
 
-    // Get simulation cell.
-    const SimulationCell* inputCell = input.expectObject<SimulationCell>();
+    // Get simulation cell (optional).
+    const SimulationCell* inputCell = state.getObject<SimulationCell>();
 
     // Get particle selection.
     const Property* selectionProperty = onlySelectedParticles() ? particles->expectProperty(Particles::SelectionProperty) : nullptr;
@@ -99,7 +128,7 @@ Future<AsynchronousModifier::EnginePtr> ClusterAnalysisModifier::createEngine(co
     PropertyPtr periodicImageBondProperty;
     if(unwrapParticleCoordinates() && particles->bonds()) {
         // Create a copy of the input bond PBC vectors so that it is safe to modify them.
-        periodicImageBondProperty = ConstPropertyPtr(particles->bonds()->getProperty(Bonds::PeriodicImageProperty)).makeCopy();
+        periodicImageBondProperty = ConstPropertyPtr::makeCopy(particles->bonds()->getProperty(Bonds::PeriodicImageProperty));
         // If no PBC vectors are present, create ad-hoc vectors initialized to zero.
         if(!periodicImageBondProperty)
             periodicImageBondProperty = Bonds::OOClass().createStandardProperty(DataBuffer::Initialized, particles->bonds()->elementCount(), Bonds::PeriodicImageProperty);
@@ -141,16 +170,17 @@ Future<AsynchronousModifier::EnginePtr> ClusterAnalysisModifier::createEngine(co
     }
 
     // Create engine object. Pass all relevant modifier parameters to the engine as well as the input data.
+    std::unique_ptr<ClusterAnalysisEngine> engine;
     if(neighborMode() == CutoffRange) {
         const Property* bondTopology = (periodicImageBondProperty && particles->bonds()) ? particles->bonds()->getProperty(Bonds::TopologyProperty) : nullptr;
-        return std::make_shared<CutoffClusterAnalysisEngine>(
-            request,
-            particles,
+        engine = std::make_unique<CutoffClusterAnalysisEngine>(
+            request.modificationNode(),
             posProperty,
             std::move(masses),
             inputCell,
             sortBySize(),
             unwrapParticleCoordinates(),
+            colorParticlesByCluster(),
             computeCentersOfMass(),
             computeRadiusOfGyration(),
             selectionProperty,
@@ -160,14 +190,14 @@ Future<AsynchronousModifier::EnginePtr> ClusterAnalysisModifier::createEngine(co
     }
     else if(neighborMode() == Bonding) {
         particles->expectBonds()->verifyIntegrity();
-        return std::make_shared<BondClusterAnalysisEngine>(
-            request,
-            particles,
+        engine = std::make_unique<BondClusterAnalysisEngine>(
+            request.modificationNode(),
             posProperty,
             std::move(masses),
             inputCell,
             sortBySize(),
             unwrapParticleCoordinates(),
+            colorParticlesByCluster(),
             computeCentersOfMass(),
             computeRadiusOfGyration(),
             selectionProperty,
@@ -177,6 +207,18 @@ Future<AsynchronousModifier::EnginePtr> ClusterAnalysisModifier::createEngine(co
     else {
         throw Exception(tr("Invalid cluster neighbor mode"));
     }
+
+    // Perform the calculation in a separate thread.
+    return asyncLaunch([
+            state = std::move(state),
+            engine = std::move(engine)]() mutable
+    {
+        // Compute the clustering.
+        engine->perform();
+        this_task::throwIfCanceled();
+        engine->applyResults(state);
+        return std::move(state);
+    });
 }
 
 /******************************************************************************
@@ -184,16 +226,16 @@ Future<AsynchronousModifier::EnginePtr> ClusterAnalysisModifier::createEngine(co
 ******************************************************************************/
 void ClusterAnalysisModifier::ClusterAnalysisEngine::perform()
 {
-    setProgressText(tr("Performing cluster analysis"));
+    TaskProgress progress(this_task::ui());
+    progress.setText(tr("Performing cluster analysis"));
 
     // Initialize.
     particleClusters()->fill<int64_t>(-1);
     std::vector<Point3> centersOfMass;
 
     // Perform the actual clustering.
-    doClustering(centersOfMass);
-    if(isCanceled())
-        return;
+    doClustering(centersOfMass, progress);
+    this_task::throwIfCanceled();
 
     // Copy center-of-mass coordinates from local array to output property storage.
     if(_centersOfMass) {
@@ -215,7 +257,7 @@ void ClusterAnalysisModifier::ClusterAnalysisEngine::perform()
 
         // Visit all input particles again.
         size_t particleCount = positions()->size();
-        setProgressMaximum(particleCount);
+        progress.setMaximum(particleCount);
         for(size_t particleIndex = 0; particleIndex < particleCount; particleIndex++) {
 
             // Skip particles that do not belong to any cluster.
@@ -223,8 +265,7 @@ void ClusterAnalysisModifier::ClusterAnalysisEngine::perform()
                 continue;
 
             // Update progress indicator.
-            if(!setProgressValueIntermittent(particleIndex))
-                return;
+            progress.setValueIntermittent(particleIndex);
 
             size_t clusterIndex = particleClusters[particleIndex] - 1;
 
@@ -260,18 +301,18 @@ void ClusterAnalysisModifier::ClusterAnalysisEngine::perform()
     if(_periodicImageBondProperty && _periodicImageBondProperty->size() == bondTopology()->size()) {
         OVITO_ASSERT(_unwrappedPositions);
 
-        if(!cell() || !cell()->hasPbcCorrected()) {
+        if(!cell().hasPbc()) {
             // No wrapping of bonds needed if simulation cell is non-periodic.
             _periodicImageBondProperty.reset();
         }
         else {
-            const std::array<bool, 3> pbcFlags = cell()->pbcFlagsCorrected();
+            const std::array<bool, 3> pbcFlags = cell().pbcFlags();
 
             // If any particles have been unwrapped by the modifier, update the PBC vectors
             // of the incident bonds accordingly.
             BufferReadAccess<Point3> positionsArray(positions());
             BufferReadAccess<Point3> unwrappedPositionsArray(_unwrappedPositions);
-            const AffineTransformation inverseSimCell = cell()->inverseMatrix();
+            const AffineTransformation inverseSimCell = cell().reciprocalCellMatrix();
             BufferWriteAccess<Vector3I, access_mode::read_write> pbcArray(_periodicImageBondProperty);
             Vector3I* pbcVec = pbcArray.begin();
             for(const ParticleIndexPair& bond : BufferReadAccess<ParticleIndexPair>(bondTopology())) {
@@ -287,8 +328,7 @@ void ClusterAnalysisModifier::ClusterAnalysisEngine::perform()
                 }
                 ++pbcVec;
             }
-            if(isCanceled())
-                return;
+            this_task::throwIfCanceled();
         }
     }
 
@@ -299,8 +339,7 @@ void ClusterAnalysisModifier::ClusterAnalysisEngine::perform()
         if(id != 0)
             clusterSizeArray[id-1]++;
     }
-    if(isCanceled())
-        return;
+    this_task::throwIfCanceled();
 
     // Create custer ID property.
     _clusterIds->resize(numClusters(), true);
@@ -311,11 +350,11 @@ void ClusterAnalysisModifier::ClusterAnalysisEngine::perform()
 
         // Determine new cluster ordering.
         std::vector<size_t> mapping(clusterSizeArray.size());
-        std::iota(mapping.begin(), mapping.end(), size_t(0));
-        std::sort(mapping.begin(), mapping.end(), [&](auto a, auto b) {
+        boost::algorithm::iota(mapping, size_t(0));
+        boost::sort(mapping, [&](auto a, auto b) {
             return clusterSizeArray[a] > clusterSizeArray[b];
         });
-        std::sort(clusterSizeArray.begin(), clusterSizeArray.end(), std::greater<>());
+        boost::sort(clusterSizeArray, std::greater<>());
         setLargestClusterSize(clusterSizeArray[0]);
 
         // Reorder centers of mass.
@@ -339,29 +378,19 @@ void ClusterAnalysisModifier::ClusterAnalysisEngine::perform()
         for(auto& id : BufferWriteAccess<int64_t, access_mode::read_write>(particleClusters()))
             id = inverseMapping[id];
     }
-
-    // Release data that is no longer needed.
-    _positions.reset();
-    _selection.reset();
-    _bondTopology.reset();
-    _masses.reset();
-    if(!_unwrapParticleCoordinates)
-        _unwrappedPositions.reset();
 }
 
 /******************************************************************************
 * Performs the actual clustering algorithm.
 ******************************************************************************/
-void ClusterAnalysisModifier::CutoffClusterAnalysisEngine::doClustering(std::vector<Point3>& centersOfMass)
+void ClusterAnalysisModifier::CutoffClusterAnalysisEngine::doClustering(std::vector<Point3>& centersOfMass, TaskProgress& progress)
 {
     // Prepare the neighbor finder.
-    CutoffNeighborFinder neighborFinder;
-    if(!neighborFinder.prepare(cutoff(), positions(), cell(), selection()))
-        return;
+    CutoffNeighborFinder neighborFinder(cutoff(), positions(), cell(), selection());
 
     size_t particleCount = positions()->size();
-    setProgressMaximum(particleCount);
-    size_t progress = 0;
+    progress.setMaximum(particleCount);
+    size_t progressVal = 0;
 
     BufferWriteAccess<int64_t, access_mode::read_write> particleClusters(this->particleClusters());
     BufferReadAccess<SelectionIntType> selectionData(selection());
@@ -374,7 +403,7 @@ void ClusterAnalysisModifier::CutoffClusterAnalysisEngine::doClustering(std::vec
         // Skip unselected particles that are not included in the analysis.
         if(selectionData && !selectionData[seedParticleIndex]) {
             particleClusters[seedParticleIndex] = 0;
-            progress++;
+            progressVal++;
             continue;
         }
 
@@ -394,8 +423,7 @@ void ClusterAnalysisModifier::CutoffClusterAnalysisEngine::doClustering(std::vec
         toProcess.push_back(seedParticleIndex);
 
         do {
-            if(!setProgressValueIntermittent(progress++))
-                return;
+            progress.setValueIntermittent(progressVal++);
 
             size_t currentParticle = toProcess.front();
             toProcess.pop_front();
@@ -433,11 +461,11 @@ void ClusterAnalysisModifier::CutoffClusterAnalysisEngine::doClustering(std::vec
 /******************************************************************************
 * Performs the actual clustering algorithm.
 ******************************************************************************/
-void ClusterAnalysisModifier::BondClusterAnalysisEngine::doClustering(std::vector<Point3>& centersOfMass)
+void ClusterAnalysisModifier::BondClusterAnalysisEngine::doClustering(std::vector<Point3>& centersOfMass, TaskProgress& progress)
 {
     size_t particleCount = positions()->size();
-    setProgressMaximum(particleCount);
-    size_t progress = 0;
+    progress.setMaximum(particleCount);
+    size_t progressVal = 0;
 
     // Prepare particle bond map.
     ParticleBondMap bondMap(bondTopology());
@@ -454,7 +482,7 @@ void ClusterAnalysisModifier::BondClusterAnalysisEngine::doClustering(std::vecto
         // Skip unselected particles that are not included in the analysis.
         if(selectionData && !selectionData[seedParticleIndex]) {
             particleClusters[seedParticleIndex] = 0;
-            progress++;
+            progressVal++;
             continue;
         }
 
@@ -474,8 +502,7 @@ void ClusterAnalysisModifier::BondClusterAnalysisEngine::doClustering(std::vecto
         toProcess.push_back(seedParticleIndex);
 
         do {
-            if(!setProgressValueIntermittent(progress++))
-                return;
+            progress.setValueIntermittent(progressVal++);
 
             size_t currentParticle = toProcess.front();
             toProcess.pop_front();
@@ -497,8 +524,7 @@ void ClusterAnalysisModifier::BondClusterAnalysisEngine::doClustering(std::vecto
                 toProcess.push_back(neighborIndex);
 
                 if(unwrappedCoordinates) {
-                    Vector3 delta = unwrappedCoordinates[neighborIndex] - unwrappedCoordinates[currentParticle];
-                    if(cell()) delta = cell()->wrapVector(delta);
+                    Vector3 delta = cell().wrapVector(unwrappedCoordinates[neighborIndex] - unwrappedCoordinates[currentParticle]);
                     unwrappedCoordinates[neighborIndex] = unwrappedCoordinates[currentParticle] + delta;
                     FloatType weight = particleMassesData ? particleMassesData[neighborIndex] : FloatType(1);
                     centerOfMass += weight * (unwrappedCoordinates[neighborIndex] - Point3::Origin());
@@ -526,23 +552,19 @@ void ClusterAnalysisModifier::BondClusterAnalysisEngine::doClustering(std::vecto
 /******************************************************************************
 * Injects the computed results of the engine into the data pipeline.
 ******************************************************************************/
-void ClusterAnalysisModifier::ClusterAnalysisEngine::applyResults(const ModifierEvaluationRequest& request, PipelineFlowState& state)
+void ClusterAnalysisModifier::ClusterAnalysisEngine::applyResults(PipelineFlowState& state)
 {
-    ClusterAnalysisModifier* modifier = static_object_cast<ClusterAnalysisModifier>(request.modifier());
     Particles* particles = state.expectMutableObject<Particles>();
 
-    if(_inputFingerprint.hasChanged(particles))
-        throw Exception(tr("Cached modifier results are obsolete, because the number or the storage order of input particles has changed."));
-
     // Output the cluster assignment.
     particles->createProperty(particleClusters());
 
     // Give clusters a random color.
-    if(modifier->colorParticlesByCluster()) {
+    if(_colorParticlesByCluster) {
         // Assign random colors to clusters.
         std::vector<ColorG> clusterColors(numClusters() + 1);
         std::default_random_engine rng(1);
-        std::uniform_real_distribution<FloatType> uniform_dist(0, 1);
+        boost::random::uniform_real_distribution<FloatType> uniform_dist(0, 1);
         boost::generate(clusterColors, [&]() { return ColorG::fromHSV(static_cast<GraphicsFloatType>(uniform_dist(rng)), 1.0f - static_cast<GraphicsFloatType>(uniform_dist(rng)) * 0.4f, 1.0f - static_cast<GraphicsFloatType>(uniform_dist(rng)) * 0.3f); });
         // Special color for particles not part of any cluster:
         clusterColors[0] = ColorG(0.8f, 0.8f, 0.8f);
@@ -556,7 +578,7 @@ void ClusterAnalysisModifier::ClusterAnalysisEngine::applyResults(const Modifier
     }
 
     // Output unwrapped particle coordinates.
-    if(modifier->unwrapParticleCoordinates() && _unwrappedPositions) {
+    if(_unwrapParticleCoordinates && _unwrappedPositions) {
         particles->createProperty(_unwrappedPositions);
 
         // Correct the PBC flags of the bonds if particles have been unwrapped.
@@ -565,28 +587,28 @@ void ClusterAnalysisModifier::ClusterAnalysisEngine::applyResults(const Modifier
         }
     }
 
-    state.addAttribute(QStringLiteral("ClusterAnalysis.cluster_count"), QVariant::fromValue(numClusters()), request.modificationNode());
-    if(modifier->sortBySize())
-        state.addAttribute(QStringLiteral("ClusterAnalysis.largest_size"), QVariant::fromValue(largestClusterSize()), request.modificationNode());
+    state.addAttribute(QStringLiteral("ClusterAnalysis.cluster_count"), QVariant::fromValue(numClusters()), createdByNode());
+    if(_sortBySize)
+        state.addAttribute(QStringLiteral("ClusterAnalysis.largest_size"), QVariant::fromValue(largestClusterSize()), createdByNode());
 
     // Output a data table with the cluster list.
-    DataTable* table = state.createObject<DataTable>(QStringLiteral("clusters"), request.modificationNode(), DataTable::Scatter, tr("Cluster list"), _clusterSizes, _clusterIds);
+    DataTable* table = state.createObject<DataTable>(QStringLiteral("clusters"), createdByNode(), DataTable::Scatter, tr("Cluster list"), _clusterSizes, _clusterIds);
 
     // Output centers of mass.
-    if(modifier->computeCentersOfMass() && _centersOfMass)
+    if(_centersOfMass)
         table->createProperty(_centersOfMass);
 
     // Output radii of gyration.
-    if(modifier->computeRadiusOfGyration() && _radiiOfGyration)
+    if(_radiiOfGyration)
         table->createProperty(_radiiOfGyration);
 
     // Output gyration tensors.
-    if(modifier->computeRadiusOfGyration() && _gyrationTensors)
+    if(_gyrationTensors)
         table->createProperty(_gyrationTensors);
 
     PipelineStatus status(
-        tr("Found %n cluster(s).", "", numClusters()),
-        numClusters() == 1 ? tr("1 cluster") : tr("%n clusters", "", numClusters()));
+        tr("Found %1 cluster(s).").arg(numClusters()),
+        numClusters() == 1 ? tr("1 cluster") : tr("%1 clusters").arg(numClusters()));
 
     if(_hasZeroWeightCluster) {
         status.setType(PipelineStatus::Warning);
