@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -190,8 +190,16 @@ public:
         return qRadiansToDegrees(disorientation);
     }
 
-    /// Creates the algorithm object.
-    PTMAlgorithm();
+    /// \brief Initializes the PTMAlgorithm with the input system of particles.
+    /// \param positions The particle coordinates.
+    /// \param cell The simulation cell information.
+    /// \param selection Per-particle selection flags determining which particles are included in the neighbor search (optional).
+    /// \throw Exception on error.
+    PTMAlgorithm(BufferReadAccess<Point3> positions, const SimulationCell* cell, BufferReadAccess<SelectionIntType> selection = {}) :
+            NearestNeighborFinder(MAX_INPUT_NEIGHBORS, std::move(positions), cell, std::move(selection))
+    {
+        ptm_initialize_global();
+    }
 
     /// Sets the threshold for the RMSD that must not be exceeded for a structure match to be valid.
     /// A zero cutoff value turns off the threshold filtering. The default threshold value is 0.1.
@@ -225,17 +233,6 @@ public:
         _identifyOrdering = (_particleTypes != nullptr);
     }
 
-    /// \brief Initializes the PTMAlgorithm with the input system of particles.
-    /// \param positions The particle coordinates.
-    /// \param cell The simulation cell information.
-    /// \param selection Per-particle selection flags determining which particles are included in the neighbor search (optional).
-    /// \return \c false when the operation has been canceled by the user;
-    ///         \c true on success.
-    /// \throw Exception on error.
-    bool prepare(BufferReadAccess<Point3> positions, const SimulationCell* cell, BufferReadAccess<SelectionIntType> selection = {}) {
-        return NearestNeighborFinder::prepare(std::move(positions), cell, std::move(selection));
-    }
-
     /// This nested class performs a PTM calculation on a single input particle.
     /// It is thread-safe to use several Kernel objects concurrently, initialized from the same PTMAlgorithm object.
     /// The Kernel object performs the PTM analysis and yields the identified structure type and, if a match has been detected,
@@ -298,7 +295,6 @@ public:
 
             FloatType minDist = std::numeric_limits<FloatType>::infinity();;
             FloatType distances[PTM_MAX_INPUT_POINTS];
-            //qDebug() << "_env.num:" << _env.num << PTM_MAX_INPUT_POINTS << minDist;
             OVITO_ASSERT(_env.num <= PTM_MAX_INPUT_POINTS);
             for (int i=1;i<_env.num;i++) {
                 FloatType dx = _env.points[i][0];
