@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -35,7 +35,10 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(CreateBondsModifier);
+IMPLEMENT_CREATABLE_OVITO_CLASS(CreateBondsModifier);
+OVITO_CLASSINFO(CreateBondsModifier, "DisplayName", "Create bonds");
+OVITO_CLASSINFO(CreateBondsModifier, "Description", "Creates bonds between particles.");
+OVITO_CLASSINFO(CreateBondsModifier, "ModifierCategory", "Visualization");
 DEFINE_PROPERTY_FIELD(CreateBondsModifier, cutoffMode);
 DEFINE_PROPERTY_FIELD(CreateBondsModifier, uniformCutoff);
 DEFINE_PROPERTY_FIELD(CreateBondsModifier, pairwiseCutoffs);
@@ -61,21 +64,16 @@ SET_PROPERTY_FIELD_UNITS_AND_MINIMUM(CreateBondsModifier, minimumCutoff, WorldPa
 SET_PROPERTY_FIELD_UNITS_AND_MINIMUM(CreateBondsModifier, vdwPrefactor, PercentParameterUnit, 0);
 
 /******************************************************************************
-* Constructs the modifier object.
+* Constructor.
 ******************************************************************************/
-CreateBondsModifier::CreateBondsModifier(ObjectInitializationFlags flags) : AsynchronousModifier(flags),
-    _cutoffMode(UniformCutoff),
-    _uniformCutoff(3.2),
-    _onlyIntraMoleculeBonds(false),
-    _minimumCutoff(0),
-    _autoDisableBondDisplay(true),
-    _skipHydrogenHydrogenBonds(true),
-    _vdwPrefactor(0.6) // Value 0.6 has been adopted from VMD source code.
+void CreateBondsModifier::initializeObject(ObjectInitializationFlags flags)
 {
+    Modifier::initializeObject(flags);
+
     if(!flags.testFlag(ObjectInitializationFlag::DontInitializeObject)) {
         // Create the bond type that will be assigned to the newly created bonds.
         setBondType(OORef<BondType>::create(flags));
-        bondType()->initializeType(BondPropertyReference(Bonds::TypeProperty));
+        bondType()->initializeType(OwnerPropertyRef(&Bonds::OOClass(), Bonds::TypeProperty));
 
         // Create the vis element for rendering the bonds generated by the modifier.
         setBondsVis(OORef<BondsVis>::create(flags));
@@ -83,7 +81,7 @@ CreateBondsModifier::CreateBondsModifier(ObjectInitializationFlags flags) : Asyn
 }
 
 /******************************************************************************
-* Is called when a RefTarget referenced by this object has generated an event.
+* Is called when a RefTarget referenced by this object generated an event.
 ******************************************************************************/
 bool CreateBondsModifier::referenceEvent(RefTarget* source, const ReferenceEvent& event)
 {
@@ -92,7 +90,7 @@ bool CreateBondsModifier::referenceEvent(RefTarget* source, const ReferenceEvent
         // again in the future.
         setAutoDisableBondDisplay(false);
     }
-    return AsynchronousModifier::referenceEvent(source, event);
+    return Modifier::referenceEvent(source, event);
 }
 
 /******************************************************************************
@@ -138,10 +136,10 @@ FloatType CreateBondsModifier::getPairwiseCutoff(const QVariant& typeA, const QV
 ******************************************************************************/
 void CreateBondsModifier::initializeModifier(const ModifierInitializationRequest& request)
 {
-    AsynchronousModifier::initializeModifier(request);
+    Modifier::initializeModifier(request);
 
     int bondTypeId = 1;
-    const PipelineFlowState& input = request.modificationNode()->evaluateInputSynchronous(request);
+    const PipelineFlowState& input = request.modificationNode()->evaluateInput(request).blockForResult();
     if(const Particles* particles = input.getObject<Particles>()) {
         // Adopt the upstream BondsVis object if there already is one.
         // Also choose a unique numeric bond type ID, which does not conflict with any existing bond type.
@@ -155,7 +153,7 @@ void CreateBondsModifier::initializeModifier(const ModifierInitializationRequest
         }
 
         // Initialize the pair-wise cutoffs based on the van der Waals radii of the particle types.
-        if(ExecutionContext::isInteractive() && pairwiseCutoffs().empty()) {
+        if(this_task::isInteractive() && pairwiseCutoffs().empty()) {
             if(const Property* typeProperty = particles->getProperty(Particles::TypeProperty)) {
                 PairwiseCutoffsList cutoffList;
                 for(const ElementType* type1 : typeProperty->elementTypes()) {
@@ -181,7 +179,7 @@ void CreateBondsModifier::initializeModifier(const ModifierInitializationRequest
     }
     if(bondType() && bondType()->numericId() == 0) {
         bondType()->setNumericId(bondTypeId);
-        bondType()->initializeType(BondPropertyReference(Bonds::TypeProperty));
+        bondType()->initializeType(OwnerPropertyRef(&Bonds::OOClass(), Bonds::TypeProperty));
     }
 }
 
@@ -203,22 +201,19 @@ const ElementType* CreateBondsModifier::lookupParticleType(const Property* typeP
 }
 
 /******************************************************************************
-* Creates and initializes a computation engine that will compute the
-* modifier's results.
+* Modifies the input data.
 ******************************************************************************/
-Future<AsynchronousModifier::EnginePtr> CreateBondsModifier::createEngine(const ModifierEvaluationRequest& request, const PipelineFlowState& input)
+Future<PipelineFlowState> CreateBondsModifier::evaluateModifier(const ModifierEvaluationRequest& request, PipelineFlowState&& state)
 {
     // Get modifier input.
-    const Particles* particles = input.expectObject<Particles>();
+    Particles* particles = state.expectMutableObject<Particles>();
     particles->verifyIntegrity();
-    const SimulationCell* simCell = input.expectObject<SimulationCell>();
-    const Property* posProperty = particles->expectProperty(Particles::PositionProperty);
 
     // The neighbor list cutoff.
     FloatType maxCutoff = uniformCutoff();
     // The list of per-type VdW radii.
     std::vector<FloatType> typeVdWRadiusMap;
-    // Flags indicating which type(s) are hydrogens.
+    // Flags indicating which particle type(s) are hydrogens.
     std::vector<bool> isHydrogenType;
 
     // Build table of pair-wise cutoff radii.
@@ -284,168 +279,131 @@ Future<AsynchronousModifier::EnginePtr> CreateBondsModifier::createEngine(const
     const Property* moleculeProperty = onlyIntraMoleculeBonds() ? particles->getProperty(Particles::MoleculeProperty) : nullptr;
 
     // Create the bonds object that will store the generated bonds.
-    DataOORef<Bonds> bondsObject;
-    if(particles->bonds()) {
-        bondsObject = DataOORef<Bonds>::makeCopy(particles->bonds());
-        bondsObject->verifyIntegrity();
-    }
-    else {
-        bondsObject = DataOORef<Bonds>::create(ObjectInitializationFlag::DontCreateVisElement);
-        bondsObject->setCreatedByNode(request.modificationNode());
-        bondsObject->setVisElement(bondsVis());
+    if(!particles->bonds()) {
+        DataOORef<Bonds> bondsObj = DataOORef<Bonds>::create(ObjectInitializationFlag::DontCreateVisElement);
+        bondsObj->setCreatedByNode(request.modificationNode());
+        bondsObj->setVisElement(bondsVis());
+        particles->setBonds(std::move(bondsObj));
     }
 
-    // Pass a deep copy of the original bond type to the data pipeline.
-    DataOORef<BondType> clonedBondType = DataOORef<BondType>::makeDeepCopy(bondType());
+    // Perform the main work in a separate thread.
+    return asyncLaunch([
+            state = std::move(state),
+            particles,
+            maxCutoff,
+            minCutoff = minimumCutoff(),
+            moleculeIDs = moleculeProperty,
+            particleTypes = typeProperty,
+            bondType = DataOORef<BondType>::makeDeepCopy(bondType()), // Note: Passing a deep copy of the original bond type to the data pipeline.
+            pairCutoffsSquared = std::move(pairCutoffSquaredTable),
+            typeVdWRadiusMap = std::move(typeVdWRadiusMap),
+            vdwPrefactor = vdwPrefactor(),
+            isHydrogenType = std::move(isHydrogenType),
+            autoDisableBondDisplay = autoDisableBondDisplay(),
+            createdByNode = request.modificationNodeWeak()]() mutable
+    {
+        TaskProgress progress(this_task::ui());
+        progress.setText(tr("Generating bonds"));
 
-    // Create engine object. Pass all relevant modifier parameters to the engine as well as the input data.
-    return std::make_shared<BondsEngine>(request, particles, posProperty,
-            typeProperty, simCell, std::move(bondsObject), std::move(clonedBondType), particles, cutoffMode(),
-            maxCutoff, minimumCutoff(), std::move(pairCutoffSquaredTable),
-            std::move(typeVdWRadiusMap), vdwPrefactor(), moleculeProperty, std::move(isHydrogenType));
-}
+        OVITO_ASSERT(state.data()->dataReferenceCount() == 1);
+        OVITO_ASSERT(state.data()->isSafeToModify());
+        OVITO_ASSERT(particles->isSafeToModify());
+        Bonds* bonds = particles->makeBondsMutable();
+        bonds->verifyIntegrity();
 
-/******************************************************************************
-* Performs the actual analysis. This method is executed in a worker thread.
-******************************************************************************/
-void CreateBondsModifier::BondsEngine::perform()
-{
-    setProgressText(tr("Generating bonds"));
+        // Prepare the neighbor finder.
+        CutoffNeighborFinder neighborFinder(maxCutoff, particles->expectProperty(Particles::PositionProperty), state.getObject<SimulationCell>(), {});
 
-    // Prepare the neighbor list.
-    CutoffNeighborFinder neighborFinder;
-    if(!neighborFinder.prepare(_maxCutoff, _positions, _simCell, {}))
-        return;
+        // The lower bond length cutoff squared.
+        FloatType minCutoffSquared = minCutoff * minCutoff;
 
-    // The lower bond length cutoff squared.
-    FloatType minCutoffSquared = _minCutoff * _minCutoff;
+        BufferReadAccess<IdentifierIntType> moleculeIDsArray(moleculeIDs);
+        BufferReadAccess<int32_t> particleTypesArray(particleTypes);
 
-    BufferReadAccess<IdentifierIntType> moleculeIDsArray(_moleculeIDs);
-    BufferReadAccess<int32_t> particleTypesArray(_particleTypes);
-
-    // Generate bonds.
-    size_t particleCount = _particles->elementCount();
-    // Multi-threaded loop over all particles, each thread producing a partial bonds list.
-    auto partialBondsLists = parallelForCollect<std::vector<Bond>>(particleCount, [&](size_t particleIndex, std::vector<Bond>& bondList) {
-
-        // Get the type of the central particles.
-        int type1;
-        bool isHydrogenType1 = false;
-        if(particleTypesArray) {
-            type1 = particleTypesArray[particleIndex];
-            if(type1 < 0) return;
-            if(type1 < _isHydrogenType.size())
-                isHydrogenType1 = _isHydrogenType[type1];
-        }
-
-        // Kernel called for each particle: Iterate over the particle's neighbors withing the cutoff range.
-        for(CutoffNeighborFinder::Query neighborQuery(neighborFinder, particleIndex); !neighborQuery.atEnd(); neighborQuery.next()) {
-            if(neighborQuery.distanceSquared() < minCutoffSquared)
-                continue;
-            if(moleculeIDsArray && moleculeIDsArray[particleIndex] != moleculeIDsArray[neighborQuery.current()])
-                continue;
+        // Generate bonds.
+        size_t particleCount = particles->elementCount();
+        // Multi-threaded loop over all particles, each thread producing a partial bonds list.
+        auto partialBondsLists = parallelForCollect<std::vector<Bond>>(particleCount, 4096, progress, [&](size_t particleIndex, std::vector<Bond>& bondList) {
 
+            // Get the type of the central particles.
+            int type1;
+            bool isHydrogenType1 = false;
             if(particleTypesArray) {
-                int type2 = particleTypesArray[neighborQuery.current()];
-                if(type2 < 0) continue;
-                if(type1 < (int)_typeVdWRadiusMap.size() && type2 < (int)_typeVdWRadiusMap.size()) {
-                    // Avoid generating H-H bonds.
-                    if(isHydrogenType1 && type2 < _isHydrogenType.size()) {
-                        if(_isHydrogenType[type2])
-                            continue;
-                    }
-                    FloatType cutoff = _vdwPrefactor * (_typeVdWRadiusMap[type1] + _typeVdWRadiusMap[type2]);
-                    if(neighborQuery.distanceSquared() > cutoff*cutoff)
-                        continue;
-                }
-                else if(type1 < (int)_pairCutoffsSquared.size() && type2 < (int)_pairCutoffsSquared[type1].size()) {
-                    if(neighborQuery.distanceSquared() > _pairCutoffsSquared[type1][type2])
-                        continue;
-                }
-                else continue;
+                type1 = particleTypesArray[particleIndex];
+                if(type1 < 0) return;
+                if(type1 < isHydrogenType.size())
+                    isHydrogenType1 = isHydrogenType[type1];
             }
 
-            Bond bond = { particleIndex, neighborQuery.current(), neighborQuery.unwrappedPbcShift() };
+            // Kernel called for each particle: Iterate over the particle's neighbors within the cutoff range.
+            for(CutoffNeighborFinder::Query neighborQuery(neighborFinder, particleIndex); !neighborQuery.atEnd(); neighborQuery.next()) {
+                if(neighborQuery.distanceSquared() < minCutoffSquared)
+                    continue;
+                if(moleculeIDsArray && moleculeIDsArray[particleIndex] != moleculeIDsArray[neighborQuery.current()])
+                    continue;
 
-            // Skip every other bond to create only one bond per particle pair.
-            if(!bond.isOdd())
-                bondList.push_back(bond);
+                if(particleTypesArray) {
+                    int type2 = particleTypesArray[neighborQuery.current()];
+                    if(type2 < 0) continue;
+                    if(type1 < (int)typeVdWRadiusMap.size() && type2 < (int)typeVdWRadiusMap.size()) {
+                        // Do not generate H-H bonds (if requested).
+                        if(isHydrogenType1 && type2 < isHydrogenType.size()) {
+                            if(isHydrogenType[type2])
+                                continue;
+                        }
+                        FloatType cutoff = vdwPrefactor * (typeVdWRadiusMap[type1] + typeVdWRadiusMap[type2]);
+                        if(neighborQuery.distanceSquared() > cutoff*cutoff)
+                            continue;
+                    }
+                    else if(type1 < (int)pairCutoffsSquared.size() && type2 < (int)pairCutoffsSquared[type1].size()) {
+                        if(neighborQuery.distanceSquared() > pairCutoffsSquared[type1][type2])
+                            continue;
+                    }
+                    else continue;
+                }
+
+                Bond bond = { particleIndex, neighborQuery.current(), neighborQuery.unwrappedPbcShift() };
+
+                // Skip every other bond to create only one bond per particle pair.
+                if(!bond.isOdd())
+                    bondList.push_back(bond);
+            }
+        });
+
+        // Flatten the bonds list into a single std::vector.
+        size_t totalBondCount = boost::accumulate(partialBondsLists, (size_t)0, [](size_t n, const std::vector<Bond>& bonds) { return n + bonds.size(); });
+        std::vector<Bond> totalBondsList;
+        if(totalBondCount != 0) {
+            totalBondsList = std::move(partialBondsLists.front());
+            totalBondsList.reserve(totalBondCount);
+            std::for_each(std::next(partialBondsLists.begin()), partialBondsLists.end(), [&](const std::vector<Bond>& bonds) { totalBondsList.insert(totalBondsList.end(), bonds.begin(), bonds.end()); });
+            this_task::throwIfCanceled();
         }
+
+        // Insert bonds into Bonds container.
+        size_t numGeneratedBonds = bonds->addBonds(totalBondsList, nullptr, particles, {}, std::move(bondType));
+
+        // Output the number of newly added bonds to the pipeline.
+        state.addAttribute(QStringLiteral("CreateBonds.num_bonds"), QVariant::fromValue(numGeneratedBonds), createdByNode);
+
+        // If the total number of bonds is unusually high, we better turn off bonds display to prevent the program from freezing.
+        if(bonds->elementCount() > 2000000 && autoDisableBondDisplay && this_task::isInteractive()) {
+            if(BondsVis* vis = bonds->visElement<BondsVis>()) {
+                // Modifying the vis element must be done in the main thread.
+                launchDetached(ObjectExecutor(vis), [vis]() {
+                    this_task::ui()->performTransaction(tr("Disable bonds display"), [&]() {
+                        vis->setEnabled(false);
+                    });
+                });
+            }
+            state.setStatus(PipelineStatus(PipelineStatus::Warning, tr("Created %1 bonds, which is a lot. The display of bonds has been turned off as a precaution. You can manually turn it on again if needed.").arg(numGeneratedBonds)));
+        }
+        else {
+            state.setStatus(PipelineStatus(PipelineStatus::Success, tr("Created %1 bonds.").arg(numGeneratedBonds)));
+        }
+
+        return std::move(state);
     });
-    if(isCanceled())
-        return;
-
-    // Flatten the bonds list into a single std::vector.
-    size_t totalBondCount = boost::accumulate(partialBondsLists, (size_t)0, [](size_t n, const std::vector<Bond>& bonds) { return n + bonds.size(); });
-    std::vector<Bond>& bondsList = partialBondsLists.front();
-    bondsList.reserve(totalBondCount);
-    std::for_each(std::next(partialBondsLists.begin()), partialBondsLists.end(), [&](const std::vector<Bond>& bonds) { bondsList.insert(bondsList.end(), bonds.begin(), bonds.end()); });
-    if(isCanceled())
-        return;
-
-    // Insert bonds into Bonds.
-    _numGeneratedBonds = bonds()->addBonds(bondsList, nullptr, _particles.get(), {}, std::move(_bondType));
-
-    // Release data that is no longer needed.
-    _positions.reset();
-    _particleTypes.reset();
-    _moleculeIDs.reset();
-    _simCell.reset();
-    _particles.reset();
-    decltype(_typeVdWRadiusMap){}.swap(_typeVdWRadiusMap);
-    decltype(_pairCutoffsSquared){}.swap(_pairCutoffsSquared);
-    decltype(_isHydrogenType){}.swap(_isHydrogenType);
-}
-
-/******************************************************************************
-* Injects the computed results of the engine into the data pipeline.
-******************************************************************************/
-void CreateBondsModifier::BondsEngine::applyResults(const ModifierEvaluationRequest& request, PipelineFlowState& state)
-{
-    CreateBondsModifier* modifier = static_object_cast<CreateBondsModifier>(request.modifier());
-    OVITO_ASSERT(modifier);
-
-    // Make the parent particle system mutable.
-    Particles* particles = state.expectMutableObject<Particles>();
-
-    // Bonds have been created for a specific particle ordering. Make sure it's still the same.
-    if(_inputFingerprint.hasChanged(particles))
-        throw Exception(tr("Cached modifier results are obsolete, because the number or the storage order of input particles has changed."));
-
-    // Add our bonds to the system.
-    particles->setBonds(bonds());
-
-    // Output the number of newly added bonds to the pipeline.
-    state.addAttribute(QStringLiteral("CreateBonds.num_bonds"), QVariant::fromValue(_numGeneratedBonds), request.modificationNode());
-
-    // If the total number of bonds is unusually high, we better turn off bonds display to prevent the program from freezing.
-    size_t bondsCount = bonds()->elementCount();
-    if(bondsCount > 1000000 && modifier->autoDisableBondDisplay() && modifier->bondsVis() && ExecutionContext::isInteractive()) {
-        modifier->bondsVis()->setEnabled(false);
-        state.setStatus(PipelineStatus(PipelineStatus::Warning, tr("Created %1 bonds, which is a lot. The display of bonds has been turned off as a precaution. You can manually turn it on again if needed.").arg(_numGeneratedBonds)));
-    }
-    else {
-        state.setStatus(PipelineStatus(PipelineStatus::Success, tr("Created %1 bonds.").arg(_numGeneratedBonds)));
-    }
-}
-
-/******************************************************************************
-* This function is called from AsynchronousModifier::evaluateSynchronous() to
-* apply the results from the last asycnhronous compute engine during a
-* synchronous pipeline evaluation.
-******************************************************************************/
-bool CreateBondsModifier::applyCachedResultsSynchronous(const ModifierEvaluationRequest& request, PipelineFlowState& state)
-{
-    // If results are still available from the last pipeline evaluation, apply them to the input data.
-    if(AsynchronousModifier::applyCachedResultsSynchronous(request, state))
-        return true;
-
-    // Bonds have not been computed yet, but still add the empty Bonds to the pipeline output
-    // so that subsequent modifiers in the pipeline see it.
-    state.expectMutableObject<Particles>()->addBonds({}, bondsVis(), {}, bondType());
-    OVITO_ASSERT(state.expectObject<Particles>()->bonds());
-
-    return false;
 }
 
 }   // End of namespace
