@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -32,15 +32,20 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(LoadTrajectoryModifier);
+IMPLEMENT_CREATABLE_OVITO_CLASS(LoadTrajectoryModifier);
+OVITO_CLASSINFO(LoadTrajectoryModifier, "DisplayName", "Load trajectory");
+OVITO_CLASSINFO(LoadTrajectoryModifier, "Description", "Load atomic trajectories or dynamic bonds from a trajectory file.");
+OVITO_CLASSINFO(LoadTrajectoryModifier, "ModifierCategory", "Modification");
 DEFINE_REFERENCE_FIELD(LoadTrajectoryModifier, trajectorySource);
 SET_PROPERTY_FIELD_LABEL(LoadTrajectoryModifier, trajectorySource, "Trajectory source");
 
 /******************************************************************************
-* Constructs the modifier object.
+* Constructor.
 ******************************************************************************/
-LoadTrajectoryModifier::LoadTrajectoryModifier(ObjectInitializationFlags flags) : Modifier(flags)
+void LoadTrajectoryModifier::initializeObject(ObjectInitializationFlags flags)
 {
+    Modifier::initializeObject(flags);
+
     if(!flags.testFlag(ObjectInitializationFlag::DontInitializeObject)) {
         // Create the file source object, which will be responsible for loading
         // and caching the trajectory data.
@@ -57,35 +62,43 @@ bool LoadTrajectoryModifier::OOMetaClass::isApplicableTo(const DataCollection& i
 }
 
 /******************************************************************************
-* Determines the time interval over which a computed pipeline state will remain valid.
-******************************************************************************/
-TimeInterval LoadTrajectoryModifier::validityInterval(const ModifierEvaluationRequest& request) const
+ * Is called by the pipeline system before a new modifier evaluation begins.
+ ******************************************************************************/
+void LoadTrajectoryModifier::preevaluateModifier(const ModifierEvaluationRequest& request, PipelineEvaluationResult::EvaluationTypes& evaluationTypes, TimeInterval& validityInterval) const
 {
-    TimeInterval iv = Modifier::validityInterval(request);
+    if(trajectorySource()) {
+        // Indicate that we will do different computations depending on whether the pipeline is evaluated in interactive mode or not.
+        if(request.interactiveMode())
+            evaluationTypes = PipelineEvaluationResult::EvaluationType::Interactive;
+        else
+            evaluationTypes = PipelineEvaluationResult::EvaluationType::Noninteractive;
 
-    if(trajectorySource())
-        iv.intersect(trajectorySource()->validityInterval(request));
-
-    return iv;
+        trajectorySource()->preevaluate(request, evaluationTypes, validityInterval);
+    }
 }
 
 /******************************************************************************
 * Modifies the input data.
 ******************************************************************************/
-Future<PipelineFlowState> LoadTrajectoryModifier::evaluate(const ModifierEvaluationRequest& request, const PipelineFlowState& input)
+Future<PipelineFlowState> LoadTrajectoryModifier::evaluateModifier(const ModifierEvaluationRequest& request, PipelineFlowState&& state)
 {
-    OVITO_ASSERT(input);
-
     // Get the trajectory data source.
     if(!trajectorySource())
         throw Exception(tr("No trajectory data source has been set."));
 
+    if(request.interactiveMode()) {
+        // In interactive mode, reuse cached node output even though it might be outdated.
+        // This avoids rebuilding the particle positions too often.
+        if(PipelineFlowState cachedState = request.modificationNode()->getCachedPipelineNodeOutput(request.time(), true)) {
+            return std::move(cachedState);
+        }
+    }
+
     // Obtain the trajectory frame from the secondary pipeline.
-    SharedFuture<PipelineFlowState> trajStateFuture = trajectorySource()->evaluate(request);
+    PipelineEvaluationResult trajStateFuture = trajectorySource()->evaluate(request);
 
     // Wait for the data to become available.
-    return trajStateFuture.then(*request.modificationNode(), [state = input, request](const PipelineFlowState& trajState) mutable {
-
+    return trajStateFuture.then(ObjectExecutor(request.modificationNode()), [state = std::move(state), request](const PipelineFlowState& trajState) mutable -> Future<PipelineFlowState> {
         if(LoadTrajectoryModifier* trajModifier = dynamic_object_cast<LoadTrajectoryModifier>(request.modifier())) {
             // Make sure the obtained configuration is valid and ready to use.
             if(trajState.status().type() == PipelineStatus::Error) {
@@ -94,14 +107,14 @@ Future<PipelineFlowState> LoadTrajectoryModifier::evaluate(const ModifierEvaluat
                         throw Exception(tr("Please pick a trajectory file."));
                 }
                 state.setStatus(trajState.status());
+                return std::move(state);
             }
             else {
-                trajModifier->applyTrajectoryState(state, trajState);
-
-                // Invalidate the synchronous state cache of the modifier pipeline node.
-                // This is needed to force the pipeline system to call our evaluateSynchronous() method
-                // again next time the system request a synchronous state from the pipeline.
-                request.modificationNode()->pipelineCache().invalidateSynchronousState();
+                // Perform the heavy work in a separate thread.
+                return asyncLaunch([state = std::move(state), trajState = std::move(trajState)]() mutable {
+                    LoadTrajectoryModifier::applyTrajectoryState(state, trajState);
+                    return std::move(state);
+                });
             }
         }
 
@@ -109,17 +122,6 @@ Future<PipelineFlowState> LoadTrajectoryModifier::evaluate(const ModifierEvaluat
     });
 }
 
-/******************************************************************************
-* Modifies the input data synchronously.
-******************************************************************************/
-void LoadTrajectoryModifier::evaluateSynchronous(const ModifierEvaluationRequest& request, PipelineFlowState& state)
-{
-    if(trajectorySource()) {
-        const PipelineFlowState& trajState = trajectorySource()->evaluateSynchronous(request);
-        applyTrajectoryState(state, trajState);
-    }
-}
-
 /******************************************************************************
 * Transfers the particle positions from the trajectory frame to the current
 * pipeline input state.
@@ -159,7 +161,8 @@ void LoadTrajectoryModifier::applyTrajectoryState(PipelineFlowState& state, cons
         if(topoIdentifierProperty && trajIdentifierProperty) {
 
             // Build map of particle identifiers in trajectory dataset.
-            std::map<IdentifierIntType, size_t> refMap;
+            std::unordered_map<IdentifierIntType, size_t> refMap;
+            refMap.reserve(trajIdentifierProperty.size());
             size_t index = 0;
             for(auto id : trajIdentifierProperty) {
                 if(refMap.insert(std::make_pair(id, index++)).second == false)
@@ -246,20 +249,20 @@ void LoadTrajectoryModifier::applyTrajectoryState(PipelineFlowState& state, cons
             }
 
             // When particle identifiers are not available, use trivial 1-to-1 mapping.
-            std::iota(indexToIndexMap.begin(), indexToIndexMap.end(), size_t(0));
+            boost::algorithm::iota(indexToIndexMap, size_t(0));
         }
 
         // Transfer particle properties from the trajectory file.
         for(const Property* property : trajectoryParticles->properties()) {
-            if(property->type() == Particles::IdentifierProperty)
+            if(property->typeId() == Particles::IdentifierProperty)
                 continue;
 
             // Get or create the output particle property.
             Property* outputProperty;
             bool replacingProperty;
-            if(property->type() != Particles::UserProperty) {
-                replacingProperty = (particles->getProperty(property->type()) != nullptr);
-                outputProperty = particles->createProperty(DataBuffer::Initialized, property->type());
+            if(property->isStandardProperty()) {
+                replacingProperty = (particles->getPropertyLike(property) != nullptr);
+                outputProperty = particles->createProperty(DataBuffer::Initialized, property->typeId());
                 if(outputProperty->dataType() != property->dataType()
                     || outputProperty->componentCount() != property->componentCount())
                     continue; // Types of source property and output property are not compatible.
@@ -280,7 +283,7 @@ void LoadTrajectoryModifier::applyTrajectoryState(PipelineFlowState& state, cons
             }
         }
 
-        // Transfer box geometry.
+        // Transfer simulation cell geometry.
         const SimulationCell* topologyCell = state.getObject<SimulationCell>();
         const SimulationCell* trajectoryCell = trajState.getObject<SimulationCell>();
         if(topologyCell && trajectoryCell) {
@@ -290,68 +293,113 @@ void LoadTrajectoryModifier::applyTrajectoryState(PipelineFlowState& state, cons
 
             // Trajectories of atoms may cross periodic boundaries and if atomic positions are
             // stored in wrapped coordinates, then it becomes necessary to fix bonds using the minimum image convention.
-            const std::array<bool, 3> pbc = topologyCell->pbcFlags();
-            if((pbc[0] || pbc[1] || pbc[2]) && particles->bonds() && std::abs(simCell.determinant()) > FLOATTYPE_EPSILON) {
-                BufferReadAccess<Point3> outputPosProperty = particles->expectProperty(Particles::PositionProperty);
+            const std::array<bool, 3> pbcFlags = topologyCell->pbcFlagsCorrected();
+            if((pbcFlags[0] || pbcFlags[1] || pbcFlags[2]) && particles->bonds() && !topologyCell->isDegenerate()) {
                 const AffineTransformation inverseCellMatrix = simCell.inverse();
-                const size_t particleCount = outputPosProperty.size();
+                const size_t numParticles = particles->elementCount();
 
                 Bonds* bonds = particles->makeBondsMutable();
+#ifdef OVITO_USE_SYCL
+                if(const Property* topologyProperty = bonds->getProperty(Bonds::TopologyProperty)) {
+                    Property* periodicImageProperty = bonds->createProperty(DataBuffer::Uninitialized, Bonds::PeriodicImageProperty);
+                    if(periodicImageProperty->size() != 0) {
+                        this_task::ui()->taskManager().syclQueue().submit([&](sycl::handler& cgh) {
+                            SyclBufferAccess<ParticleIndexPair, access_mode::read> topologyAcc{topologyProperty, cgh};
+                            SyclBufferAccess<Point3, access_mode::read> posInAcc{particles->expectProperty(Particles::PositionProperty), cgh};
+                            SyclBufferAccess<Vector3I, access_mode::discard_write> imageAcc{periodicImageProperty, cgh};
+                            OVITO_SYCL_PARALLEL_FOR(cgh, LoadTrajectoryModifier_bond_imageflags)(sycl::range(topologyAcc.size()), [=](size_t bondIndex) {
+                                const auto [particleIndex1, particleIndex2] = topologyAcc[bondIndex];
+                                if(particleIndex1 < numParticles && particleIndex2 < numParticles) {
+                                    const Vector3 rv = inverseCellMatrix * (posInAcc[particleIndex1] - posInAcc[particleIndex2]);
+                                    const Vector3I iflags(
+                                        pbcFlags[0] * static_cast<Vector3I::value_type>(sycl::round(rv.x())),
+                                        pbcFlags[1] * static_cast<Vector3I::value_type>(sycl::round(rv.y())),
+                                        pbcFlags[2] * static_cast<Vector3I::value_type>(sycl::round(rv.z()))
+                                    );
+                                    imageAcc[bondIndex] = iflags;
+                                }
+                                else {
+                                    imageAcc[bondIndex].setZero();
+                                }
+                            });
+                        });
+                    }
+                }
+#else
                 if(BufferReadAccess<ParticleIndexPair> topologyProperty = bonds->getProperty(Bonds::TopologyProperty)) {
                     BufferWriteAccess<Vector3I, access_mode::discard_write> periodicImageProperty = bonds->createProperty(DataBuffer::Uninitialized, Bonds::PeriodicImageProperty);
+                    BufferReadAccess<Point3> positions = particles->expectProperty(Particles::PositionProperty);
 
                     // Wrap bonds crossing a periodic boundary by resetting their PBC shift vectors.
                     for(size_t bondIndex = 0; bondIndex < topologyProperty.size(); bondIndex++) {
                         Vector3I& pbcShift = periodicImageProperty[bondIndex];
                         size_t particleIndex1 = topologyProperty[bondIndex][0];
                         size_t particleIndex2 = topologyProperty[bondIndex][1];
-                        if(particleIndex1 >= particleCount || particleIndex2 >= particleCount) {
+                        if(particleIndex1 >= numParticles || particleIndex2 >= numParticles) {
                             pbcShift.setZero();
                             continue;
                         }
-                        const Point3& p1 = outputPosProperty[particleIndex1];
-                        const Point3& p2 = outputPosProperty[particleIndex2];
+                        const Point3& p1 = positions[particleIndex1];
+                        const Point3& p2 = positions[particleIndex2];
                         const Vector3 delta = p1 - p2;
                         for(int dim = 0; dim < 3; dim++) {
-                            pbcShift[dim] = pbc[dim] ? std::lround(inverseCellMatrix.prodrow(delta, dim)) : 0;
+                            pbcShift[dim] = pbcFlags[dim] ? std::lround(inverseCellMatrix.prodrow(delta, dim)) : 0;
                         }
                     }
                 }
+#endif
             }
         }
     }
 
-    if(const Bonds* trajectoryBonds = trajectoryParticles->bonds()) {
-        trajectoryBonds->verifyIntegrity();
+    // This generic function merges the 4 kinds of interactions (bonds, angles, dihedrals, impropers) from the trajectory dataset with the topology dataset.
+    auto mergeInteractionTopology = [&](const PropertyContainer* trajectoryInteractions, const PropertyContainer* topologyInteractions, int topologyPropertyId, int particleIdentifiersPropertyId, const QString& interactionName) {
+        if(!trajectoryInteractions)
+            return;
+
+        trajectoryInteractions->verifyIntegrity();
 
         // Create a mutable copy of the particles object.
         Particles* particles = state.expectMutableObject<Particles>();
-        particles->verifyIntegrity();
 
-        // If the trajectory file contains a bond topology, completely replace all existing bonds
-        // from the topology dataset with the new set of bonds.
-        // The topology can be specified either in the form of the "Topology" bond property (two 0-based particle indices)
-        // or in the form of the "Particle Identifiers" bond property (pairs of atom IDs).
-        const Property* bondTopology = trajectoryBonds->getProperty(Bonds::TopologyProperty);
-        const Property* bondParticleIdentifiers = trajectoryBonds->getProperty(Bonds::ParticleIdentifiersProperty);
+        // If the trajectory file contains a topology, completely replace all existing interactions
+        // from the topology dataset with the new set of interactions.
+        // The topology can be specified either in the form of the "Topology" property (two 0-based particle indices)
+        // or in the form of the "Particle Identifiers" property (tuples of atom IDs).
+        const Property* interactionTopology = trajectoryInteractions->getProperty(topologyPropertyId);
+        const Property* interactionParticleIdentifiers = trajectoryInteractions->getProperty(particleIdentifiersPropertyId);
 
-        if(bondTopology || bondParticleIdentifiers) {
-            if(particles->bonds()) {
-                // Replace the property arrays, but make sure that BondType instances
+        if(interactionTopology || interactionParticleIdentifiers) {
+            if(topologyInteractions) {
+                // Replace the property arrays, but make sure that type elements
                 // as well as the visual elements from the topology dataset are preserved.
-                particles->makeBondsMutable()->setContent(trajectoryBonds->elementCount(), trajectoryBonds->properties());
+                PropertyContainer* interactions = particles->makeMutable(topologyInteractions);
+                interactions->setContent(trajectoryInteractions->elementCount(), trajectoryInteractions->properties());
+                topologyInteractions = interactions;
             }
             else {
-                // We can simply adopt the bonds object from the trajectory dataset as a whole
-                // if the topology dataset didn't contain any bonds yet.
-                particles->setBonds(trajectoryBonds);
+                // We can simply adopt the property container from the trajectory dataset as a whole
+                // if the topology dataset didn't contain any interactions yet.
+                if(trajectoryInteractions == trajectoryParticles->bonds())
+                    particles->setBonds(static_object_cast<Bonds>(trajectoryInteractions));
+                else if(trajectoryInteractions == trajectoryParticles->angles())
+                    particles->setAngles(static_object_cast<Angles>(trajectoryInteractions));
+                else if(trajectoryInteractions == trajectoryParticles->dihedrals())
+                    particles->setDihedrals(static_object_cast<Dihedrals>(trajectoryInteractions));
+                else if(trajectoryInteractions == trajectoryParticles->impropers())
+                    particles->setImpropers(static_object_cast<Impropers>(trajectoryInteractions));
+                else
+                    throw Exception(tr("Unsupported interaction type in trajectory."));
+
+                topologyInteractions = trajectoryInteractions;
             }
 
-            // If the input bonds are defined in terms of atom IDs (i.e. bond property "Particle Identifiers"), then map the IDs
-            // to corresponding particle indices and store them in the "Topology" bond property.
-            if(bondParticleIdentifiers) {
+            // If the input interactions are defined in terms of atom IDs (i.e.property "Particle Identifiers"), then map the IDs
+            // to corresponding particle indices and store them in the "Topology" property.
+            if(interactionParticleIdentifiers) {
                 // Build map from particle identifiers to particle indices.
-                std::map<IdentifierIntType, size_t> idToIndexMap;
+                std::unordered_map<IdentifierIntType, size_t> idToIndexMap;
+                idToIndexMap.reserve(particles->elementCount());
                 if(BufferReadAccess<IdentifierIntType> particleIdentifierProperty = particles->getProperty(Particles::IdentifierProperty)) {
                     size_t index = 0;
                     for(auto id : particleIdentifierProperty) {
@@ -366,60 +414,71 @@ void LoadTrajectoryModifier::applyTrajectoryState(PipelineFlowState& state, cons
                 }
 
                 // Perform lookup of particle IDs.
-                BufferWriteAccess<ParticleIndexPair, access_mode::discard_write> bondTopologyArray = particles->makeBondsMutable()->createProperty(Bonds::TopologyProperty);
-                auto t = bondTopologyArray.begin();
-                for(const ParticleIndexPair& bond : BufferReadAccess<ParticleIndexPair>(bondParticleIdentifiers)) {
-                    auto iter1 = idToIndexMap.find(bond[0]);
-                    auto iter2 = idToIndexMap.find(bond[1]);
-                    if(iter1 == idToIndexMap.end())
-                        throw Exception(tr("Particle id %1 referenced by bond #%2 does not exist.").arg(bond[0]).arg(std::distance(bondTopologyArray.begin(), t)));
-                    if(iter2 == idToIndexMap.end())
-                        throw Exception(tr("Particle id %1 referenced by bond #%2 does not exist.").arg(bond[1]).arg(std::distance(bondTopologyArray.begin(), t)));
-                    (*t)[0] = iter1->second;
-                    (*t)[1] = iter2->second;
-                    ++t;
+                PropertyContainer* interactions = particles->makeMutable(topologyInteractions);
+                BufferWriteAccess<int64_t*, access_mode::discard_write> interactionTopologyArray = interactions->createProperty(topologyPropertyId);
+                auto t = interactionTopologyArray.begin();
+                for(const int64_t& id : BufferReadAccess<int64_t*>(interactionParticleIdentifiers)) {
+                    auto iter = idToIndexMap.find(id);
+                    if(iter == idToIndexMap.end())
+                        throw Exception(tr("Particle id %1 referenced by %2 #%3 does not exist.")
+                            .arg(id)
+                            .arg(interactionName)
+                            .arg(std::distance(interactionTopologyArray.begin(), t) / interactionTopologyArray.componentCount()));
+                    *t++ = iter->second;
                 }
+                OVITO_ASSERT(t == interactionTopologyArray.end());
 
-                // Remove the "Particle Identifiers" property from bonds again, because it is no longer needed.
-                particles->makeBondsMutable()->removeProperty(bondParticleIdentifiers);
+                // Remove the "Particle Identifiers" property from interactions again, because it is no longer needed.
+                interactions->removeProperty(interactionParticleIdentifiers);
             }
 
             // Compute the PBC shift vectors of the bonds based on current particle positions.
-            if(!trajectoryBonds->getProperty(Bonds::PeriodicImageProperty)) {
-                if(const SimulationCell* simCellObj = state.getObject<SimulationCell>()) {
-                    if(simCellObj->pbcX() || simCellObj->pbcY() || simCellObj->pbcZ()) {
-                        particles->makeBondsMutable()->generatePeriodicImageProperty(particles, simCellObj);
+            if(trajectoryInteractions == trajectoryParticles->bonds()) {
+                if(!trajectoryInteractions->getProperty(Bonds::PeriodicImageProperty)) {
+                    if(const SimulationCell* simCellObj = state.getObject<SimulationCell>()) {
+                        if(simCellObj->hasPbcCorrected()) {
+                            particles->makeBondsMutable()->generatePeriodicImageProperty(particles, simCellObj);
+                        }
                     }
                 }
             }
         }
-        else if(particles->bonds()) {
-            // If the trajectory dataset doesn't contain the "Topology" nor the "Particle Identifiers" bond property,
-            // then add the bond properties to the existing bonds from the topology dataset.
-            // This requires that the number of bonds remains constant.
-            if(trajectoryBonds->elementCount() != particles->bonds()->elementCount()) {
-                throw Exception(tr("Cannot merge bond properties of trajectory dataset with topology dataset, because numbers of bonds in the two datasets do not match."));
+        else if(topologyInteractions) {
+            // If the trajectory dataset doesn't contain the "Topology" nor the "Particle Identifiers" property,
+            // then add the properties to the existing interactions from the topology dataset.
+            // This requires that the number of interactions remains constant.
+            if(trajectoryInteractions->elementCount() != topologyInteractions->elementCount()) {
+                throw Exception(tr("Cannot merge %1 properties of trajectory dataset with topology dataset, because numbers of %1s in the two datasets do not match.").arg(interactionName));
             }
 
-            if(!trajectoryBonds->properties().empty()) {
-                Bonds* bonds = particles->makeBondsMutable();
+            if(!trajectoryInteractions->properties().empty()) {
+                PropertyContainer* interactions = particles->makeMutable(topologyInteractions);
 
-                // Add the properties to the existing bonds, overwriting existing values if necessary.
-                for(const Property* newProperty : trajectoryBonds->properties()) {
-                    const Property* existingPropertyObj = (newProperty->type() != 0) ? bonds->getProperty(newProperty->type()) : bonds->getProperty(newProperty->name());
+                // Add the properties to the existing interactions, overwriting existing values if necessary.
+                for(const Property* newProperty : trajectoryInteractions->properties()) {
+                    const Property* existingPropertyObj = interactions->getPropertyLike(newProperty);
                     if(existingPropertyObj) {
-                        bonds->makeMutable(existingPropertyObj)->copyFrom(*newProperty);
+                        interactions->makeMutable(existingPropertyObj)->copyFrom(*newProperty);
                     }
                     else {
-                        bonds->addProperty(newProperty);
+                        interactions->addProperty(newProperty);
                     }
                 }
             }
         }
         else {
-            throw Exception(tr("Neither the trajectory nor the topology dataset contain bond connectivity information."));
+            throw Exception(tr("Neither the trajectory nor the topology dataset contain %1 connectivity information.").arg(interactionName));
         }
-    }
+    };
+
+    // Merge bonds from the trajectory dataset with the topology dataset.
+    mergeInteractionTopology(trajectoryParticles->bonds(), particles->bonds(), Bonds::TopologyProperty, Bonds::ParticleIdentifiersProperty, tr("bond"));
+    // Merge angles from the trajectory dataset with the topology dataset.
+    mergeInteractionTopology(trajectoryParticles->angles(), particles->angles(), Angles::TopologyProperty, Angles::ParticleIdentifiersProperty, tr("angle"));
+    // Merge dihedrals from the trajectory dataset with the topology dataset.
+    mergeInteractionTopology(trajectoryParticles->dihedrals(), particles->dihedrals(), Dihedrals::TopologyProperty, Dihedrals::ParticleIdentifiersProperty, tr("dihedral"));
+    // Merge impropers from the trajectory dataset with the topology dataset.
+    mergeInteractionTopology(trajectoryParticles->impropers(), particles->impropers(), Impropers::TopologyProperty, Impropers::ParticleIdentifiersProperty, tr("improper"));
 
     // Merge global attributes of topology and trajectory datasets.
     // If there is a naming collision, attributes from the trajectory dataset override those from the topology dataset.
@@ -435,7 +494,7 @@ void LoadTrajectoryModifier::applyTrajectoryState(PipelineFlowState& state, cons
                 }
             }
             if(existingAttribute)
-                state.mutableData()->replaceObject(existingAttribute, attribute);
+                state.replaceObject(existingAttribute, attribute);
             else
                 state.addObject(attribute);
         }
@@ -443,7 +502,7 @@ void LoadTrajectoryModifier::applyTrajectoryState(PipelineFlowState& state, cons
 }
 
 /******************************************************************************
-* Is called when a RefTarget referenced by this object has generated an event.
+* Is called when a RefTarget referenced by this object generated an event.
 ******************************************************************************/
 bool LoadTrajectoryModifier::referenceEvent(RefTarget* source, const ReferenceEvent& event)
 {
@@ -459,11 +518,32 @@ bool LoadTrajectoryModifier::referenceEvent(RefTarget* source, const ReferenceEv
 ******************************************************************************/
 void LoadTrajectoryModifier::referenceReplaced(const PropertyFieldDescriptor* field, RefTarget* oldTarget, RefTarget* newTarget, int listIndex)
 {
-    if(field == PROPERTY_FIELD(trajectorySource) && !isBeingLoaded() && !isAboutToBeDeleted()) {
+    if(field == PROPERTY_FIELD(trajectorySource) && !isBeingLoaded() && !isBeingDeleted()) {
         // The animation length might have changed when the trajectory source has been replaced.
         notifyDependents(ReferenceEvent::AnimationFramesChanged);
     }
     Modifier::referenceReplaced(field, oldTarget, newTarget, listIndex);
 }
 
+/******************************************************************************
+* Returns a short piece of information (typically a string or color) to be
+* displayed next to the object's title in the pipeline editor.
+******************************************************************************/
+QVariant LoadTrajectoryModifier::getPipelineEditorShortInfo(Scene* scene, ModificationNode* node) const
+{
+    OVITO_ASSERT(this_task::get());
+    OVITO_ASSERT(scene);
+
+    // Display the name of the trajectory file as short info.
+    if(node && trajectorySource()) {
+        const PipelineFlowState& state = trajectorySource()->getCachedPipelineNodeOutput(scene->animationSettings()->currentTime());
+        const QString sourceFile = state.getAttributeValue(QStringLiteral("SourceFile")).toString();
+        if(!sourceFile.isEmpty()) {
+            // Extract filename from path.
+            return sourceFile.section('/', -1);
+        }
+    }
+    return {};
+}
+
 }   // End of namespace
