@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -25,14 +25,18 @@
 #include <ovito/core/dataset/animation/AnimationSettings.h>
 #include <ovito/core/dataset/DataSet.h>
 #include <ovito/core/dataset/pipeline/ModificationNode.h>
-#include <ovito/core/dataset/pipeline/PipelineEvaluation.h>
+#include <ovito/core/dataset/pipeline/PipelineEvaluationRequest.h>
 #include <ovito/core/utilities/units/UnitsManager.h>
 #include <ovito/core/app/Application.h>
 #include "SmoothTrajectoryModifier.h"
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(SmoothTrajectoryModifier);
+IMPLEMENT_CREATABLE_OVITO_CLASS(SmoothTrajectoryModifier);
+OVITO_CLASSINFO(SmoothTrajectoryModifier, "DisplayName", "Smooth trajectory");
+OVITO_CLASSINFO(SmoothTrajectoryModifier, "ClassNameAlias", "InterpolateTrajectoryModifier");
+OVITO_CLASSINFO(SmoothTrajectoryModifier, "Description", "Time-averaged particle positions using a sliding time window.");
+OVITO_CLASSINFO(SmoothTrajectoryModifier, "ModifierCategory", "Modification");
 DEFINE_PROPERTY_FIELD(SmoothTrajectoryModifier, useMinimumImageConvention);
 DEFINE_PROPERTY_FIELD(SmoothTrajectoryModifier, smoothingWindowSize);
 SET_PROPERTY_FIELD_LABEL(SmoothTrajectoryModifier, useMinimumImageConvention, "Use minimum image convention");
@@ -40,16 +44,7 @@ SET_PROPERTY_FIELD_LABEL(SmoothTrajectoryModifier, smoothingWindowSize, "Smoothi
 SET_PROPERTY_FIELD_UNITS_AND_RANGE(SmoothTrajectoryModifier, smoothingWindowSize, IntegerParameterUnit, 1, 200);
 
 // This class can be removed in a future version of OVITO:
-IMPLEMENT_OVITO_CLASS(InterpolateTrajectoryModifierApplication);
-
-/******************************************************************************
-* Constructs the modifier object.
-******************************************************************************/
-SmoothTrajectoryModifier::SmoothTrajectoryModifier(ObjectInitializationFlags flags) : Modifier(flags),
-    _useMinimumImageConvention(true),
-    _smoothingWindowSize(1)
-{
-}
+IMPLEMENT_CREATABLE_OVITO_CLASS(InterpolateTrajectoryModifierApplication);
 
 /******************************************************************************
 * Asks the modifier whether it can be applied to the given input data.
@@ -60,44 +55,49 @@ bool SmoothTrajectoryModifier::OOMetaClass::isApplicableTo(const DataCollection&
 }
 
 /******************************************************************************
-* Determines the time interval over which a computed pipeline state will remain valid.
-******************************************************************************/
-TimeInterval SmoothTrajectoryModifier::validityInterval(const ModifierEvaluationRequest& request) const
+ * Is called by the pipeline system before a new modifier evaluation begins.
+ ******************************************************************************/
+void SmoothTrajectoryModifier::preevaluateModifier(const ModifierEvaluationRequest& request, PipelineEvaluationResult::EvaluationTypes& evaluationTypes, TimeInterval& validityInterval) const
 {
-    TimeInterval iv = Modifier::validityInterval(request);
     // Interpolation results will only be valid for the duration of the current frame.
-    iv.intersect(request.time());
-    return iv;
+    validityInterval.intersect(request.time());
+
+    // Indicate that we will do different computations depending on whether the pipeline is evaluated in interactive mode or not.
+    if(smoothingWindowSize() != 1) {
+        if(request.interactiveMode())
+            evaluationTypes = PipelineEvaluationResult::EvaluationType::Interactive;
+        else
+            evaluationTypes = PipelineEvaluationResult::EvaluationType::Noninteractive;
+    }
 }
 
 /******************************************************************************
 * Asks the modifier for the set of animation time intervals that should be
 * cached by the upstream pipeline.
 ******************************************************************************/
-void SmoothTrajectoryModifier::inputCachingHints(TimeIntervalUnion& cachingIntervals, ModificationNode* node)
+void SmoothTrajectoryModifier::inputCachingHints(ModifierEvaluationRequest& request)
 {
-    Modifier::inputCachingHints(cachingIntervals, node);
-
-    TimeIntervalUnion originalIntervals = cachingIntervals;
-    for(const TimeInterval& iv : originalIntervals) {
+    for(const TimeInterval& iv : TimeIntervalUnion(request.cachingIntervals())) {
         // Round interval start down to the previous animation frame.
         // Round interval end up to the next animation frame.
-        int startFrame = node->animationTimeToSourceFrame(iv.start());
-        int endFrame = node->animationTimeToSourceFrame(iv.end());
-        if(node->sourceFrameToAnimationTime(endFrame) < iv.end())
+        int startFrame = request.modificationNode()->animationTimeToSourceFrame(iv.start());
+        int endFrame = request.modificationNode()->animationTimeToSourceFrame(iv.end());
+        if(request.modificationNode()->sourceFrameToAnimationTime(endFrame) < iv.end())
             endFrame++;
         startFrame -= (smoothingWindowSize() - 1) / 2;
         endFrame += smoothingWindowSize() / 2;
-        AnimationTime newStartTime = node->sourceFrameToAnimationTime(startFrame);
-        AnimationTime newEndTime = node->sourceFrameToAnimationTime(endFrame);
+        AnimationTime newStartTime = request.modificationNode()->sourceFrameToAnimationTime(startFrame);
+        AnimationTime newEndTime = request.modificationNode()->sourceFrameToAnimationTime(endFrame);
         OVITO_ASSERT(newStartTime <= iv.start());
         OVITO_ASSERT(newEndTime >= iv.end());
-        cachingIntervals.add(TimeInterval(newStartTime, newEndTime));
+        request.mutableCachingIntervals().add(TimeInterval(newStartTime, newEndTime));
     }
+
+    Modifier::inputCachingHints(request);
 }
 
 /******************************************************************************
-* Is called by the ModifierApplication to let the modifier adjust the
+* Is called by the ModificationNode to let the modifier adjust the
 * time interval of a TargetChanged event received from the upstream pipeline
 * before it is propagated to the downstream pipeline.
 ******************************************************************************/
@@ -112,11 +112,11 @@ void SmoothTrajectoryModifier::restrictInputValidityInterval(TimeInterval& iv) c
 /******************************************************************************
 * Modifies the input data.
 ******************************************************************************/
-Future<PipelineFlowState> SmoothTrajectoryModifier::evaluate(const ModifierEvaluationRequest& request, const PipelineFlowState& input)
+Future<PipelineFlowState> SmoothTrajectoryModifier::evaluateModifier(const ModifierEvaluationRequest& request, PipelineFlowState&& state)
 {
     // Determine the current frame, preferably from the attribute stored with the pipeline flow state.
     // If the source frame attribute is not present, fall back to inferring it from the current animation time.
-    int currentFrame = input.data() ? input.data()->sourceFrame() : -1;
+    int currentFrame = state.data() ? state.data()->sourceFrame() : -1;
     if(currentFrame < 0)
         currentFrame = request.modificationNode()->animationTimeToSourceFrame(request.time());
     AnimationTime time1 = request.modificationNode()->sourceFrameToAnimationTime(currentFrame);
@@ -124,9 +124,8 @@ Future<PipelineFlowState> SmoothTrajectoryModifier::evaluate(const ModifierEvalu
     // If we are exactly on a source frame, there is no need to interpolate between frames.
     if(time1 == request.time() && smoothingWindowSize() <= 1) {
         // The validity of the resulting state is restricted to the current animation time.
-        PipelineFlowState output = input;
-        output.intersectStateValidity(time1);
-        return output;
+        state.intersectStateValidity(time1);
+        return std::move(state);
     }
 
     if(smoothingWindowSize() == 1) {
@@ -140,13 +139,19 @@ Future<PipelineFlowState> SmoothTrajectoryModifier::evaluate(const ModifierEvalu
 
         // Wait for the second frame to become available.
         return request.modificationNode()->evaluateInput(frameRequest)
-            .then(*this, [this, request, state = input, time1, time2](const PipelineFlowState& nextState) mutable {
+            .then(ObjectExecutor(this), [this, request, state = std::move(state), time1, time2](const PipelineFlowState& nextState) mutable {
                 // Compute interpolated state.
                 interpolateState(state, nextState, request, time1, time2);
                 return std::move(state);
             });
     }
     else {
+
+        // In interactive mode, do not perform a long-running computation.
+        if(request.interactiveMode()) {
+            return std::move(state);
+        }
+
         // Perform averaging of several frames. Determine frame interval first.
         int startFrame = currentFrame - (smoothingWindowSize() - 1) / 2;
         int endFrame = currentFrame + smoothingWindowSize() / 2;
@@ -165,7 +170,7 @@ Future<PipelineFlowState> SmoothTrajectoryModifier::evaluate(const ModifierEvalu
 
         // Obtain the range of input frames from the upstream pipeline.
         return request.modificationNode()->evaluateInputMultiple(frameRequest, std::move(otherTimes))
-            .then(*this, [this, state = input, request](const std::vector<PipelineFlowState>& otherStates) mutable {
+            .then(ObjectExecutor(this), [this, state = std::move(state), request](const std::vector<PipelineFlowState>& otherStates) mutable {
                 // Compute smoothed state.
                 averageState(state, otherStates, request);
                 return std::move(state);
@@ -173,56 +178,6 @@ Future<PipelineFlowState> SmoothTrajectoryModifier::evaluate(const ModifierEvalu
     }
 }
 
-/******************************************************************************
-* Modifies the input data synchronously.
-******************************************************************************/
-void SmoothTrajectoryModifier::evaluateSynchronous(const ModifierEvaluationRequest& request, PipelineFlowState& state)
-{
-    // Determine the current frame, preferably from the attribute stored with the pipeline flow state.
-    // If the source frame attribute is not present, fall back to inferring it from the current animation time.
-    int currentFrame = state.data() ? state.data()->sourceFrame() : -1;
-    if(currentFrame < 0)
-        currentFrame = request.modificationNode()->animationTimeToSourceFrame(request.time());
-    AnimationTime time1 = request.modificationNode()->sourceFrameToAnimationTime(currentFrame);
-
-    // If we are exactly on a source frame, there is no need to interpolate between two consecutive frames.
-    if(time1 == request.time() && smoothingWindowSize() <= 1) {
-        // The validity of the resulting state is restricted to the current animation time.
-        state.intersectStateValidity(request.time());
-        return;
-    }
-
-    if(smoothingWindowSize() == 1) {
-        // Perform interpolation between two consecutive frames.
-        int nextFrame = currentFrame + 1;
-        AnimationTime time2 = request.modificationNode()->sourceFrameToAnimationTime(nextFrame);
-
-        // Get the second frame.
-        const PipelineFlowState& state2 = request.modificationNode()->evaluateInputSynchronous(PipelineEvaluationRequest(time2));
-
-        // Perform the actual interpolation calculation.
-        interpolateState(state, state2, request, time1, time2);
-    }
-    else {
-        // Perform averaging of several frames. Determine frame interval.
-        int startFrame = currentFrame - (smoothingWindowSize() - 1) / 2;
-        int endFrame = currentFrame + smoothingWindowSize() / 2;
-
-        // Obtain the range of input frames from the upstream pipeline.
-        std::vector<PipelineFlowState> otherStates;
-        otherStates.reserve(endFrame - startFrame);
-        for(int frame = startFrame; frame <= endFrame; frame++) {
-            if(frame != currentFrame) {
-                AnimationTime time2 = request.modificationNode()->sourceFrameToAnimationTime(frame);
-                otherStates.push_back(request.modificationNode()->evaluateInputSynchronous(PipelineEvaluationRequest(time2)));
-            }
-        }
-
-        // Compute smoothed state.
-        averageState(state, otherStates, request);
-    }
-}
-
 /******************************************************************************
 * Computes the interpolated state between two input states.
 ******************************************************************************/
@@ -333,8 +288,7 @@ void SmoothTrajectoryModifier::interpolateState(PipelineFlowState& state1, const
     // Interpolate all scalar and continuous particle properties.
     for(const Property* property1 : particles1->properties()) {
         if(property1->dataType() == Property::Float32 && property1->componentCount() == 1) {
-            const Property* property2 = (property1->type() != 0) ? particles2->getProperty(property1->type()) : particles2->getProperty(property1->name());
-            if(property2 && property2->dataType() == property1->dataType() && property2->componentCount() == property1->componentCount()) {
+            if(const Property* property2 = particles2->getPropertyLike(property1)) {
                 BufferWriteAccess<float, access_mode::read_write> data1 = outputParticles->makeMutable(property1);
                 BufferReadAccess<float> data2(property2);
                 if(idProperty1 && idProperty2 && !boost::equal(idProperty1, idProperty2)) {
@@ -357,8 +311,7 @@ void SmoothTrajectoryModifier::interpolateState(PipelineFlowState& state1, const
             }
         }
         else if(property1->dataType() == Property::Float64 && property1->componentCount() == 1) {
-            const Property* property2 = (property1->type() != 0) ? particles2->getProperty(property1->type()) : particles2->getProperty(property1->name());
-            if(property2 && property2->dataType() == property1->dataType() && property2->componentCount() == property1->componentCount()) {
+            if(const Property* property2 = particles2->getPropertyLike(property1)) {
                 BufferWriteAccess<double, access_mode::read_write> data1 = outputParticles->makeMutable(property1);
                 BufferReadAccess<double> data2(property2);
                 if(idProperty1 && idProperty2 && !boost::equal(idProperty1, idProperty2)) {
@@ -517,8 +470,7 @@ void SmoothTrajectoryModifier::averageState(PipelineFlowState& state1, const std
             // Average all scalar continuous properties.
             for(auto& accessor1 : outputScalarProperties32) {
                 Property* property1 = static_object_cast<Property>(accessor1.buffer());
-                const Property* property2 = (property1->type() != 0) ? particles2->getProperty(property1->type()) : particles2->getProperty(property1->name());
-                if(property2 && property2->dataType() == accessor1.dataType() && property2->componentCount() == accessor1.componentCount()) {
+                if(const Property* property2 = particles2->getPropertyLike(property1)) {
                     BufferReadAccess<float> accessor2(property2);
                     auto id = idProperty1.cbegin();
                     for(auto& v : accessor1) {
@@ -531,8 +483,7 @@ void SmoothTrajectoryModifier::averageState(PipelineFlowState& state1, const std
             }
             for(auto& accessor1 : outputScalarProperties64) {
                 Property* property1 = static_object_cast<Property>(accessor1.buffer());
-                const Property* property2 = (property1->type() != 0) ? particles2->getProperty(property1->type()) : particles2->getProperty(property1->name());
-                if(property2 && property2->dataType() == accessor1.dataType() && property2->componentCount() == accessor1.componentCount()) {
+                if(const Property* property2 = particles2->getPropertyLike(property1)) {
                     BufferReadAccess<double> accessor2(property2);
                     auto id = idProperty1.cbegin();
                     for(auto& v : accessor1) {
@@ -580,8 +531,7 @@ void SmoothTrajectoryModifier::averageState(PipelineFlowState& state1, const std
             // Average all scalar continuous properties.
             for(auto& accessor1 : outputScalarProperties32) {
                 Property* property1 = static_object_cast<Property>(accessor1.buffer());
-                const Property* property2 = (property1->type() != 0) ? particles2->getProperty(property1->type()) : particles2->getProperty(property1->name());
-                if(property2 && property2->dataType() == accessor1.dataType() && property2->componentCount() == accessor1.componentCount()) {
+                if(const Property* property2 = particles2->getPropertyLike(property1)) {
                     BufferReadAccess<float> accessor2(property2);
                     const auto* v2 = accessor2.cbegin();
                     for(auto* vout = accessor1.begin(), *vend = vout + std::min(accessor1.size(), accessor2.size()); vout != vend; ++vout, ++v2) {
@@ -591,8 +541,7 @@ void SmoothTrajectoryModifier::averageState(PipelineFlowState& state1, const std
             }
             for(auto& accessor1 : outputScalarProperties64) {
                 Property* property1 = static_object_cast<Property>(accessor1.buffer());
-                const Property* property2 = (property1->type() != 0) ? particles2->getProperty(property1->type()) : particles2->getProperty(property1->name());
-                if(property2 && property2->dataType() == accessor1.dataType() && property2->componentCount() == accessor1.componentCount()) {
+                if(const Property* property2 = particles2->getPropertyLike(property1)) {
                     BufferReadAccess<double> accessor2(property2);
                     const auto* v2 = accessor2.cbegin();
                     for(auto* vout = accessor1.begin(), *vend = vout + std::min(accessor1.size(), accessor2.size()); vout != vend; ++vout, ++v2) {
