@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -34,13 +34,17 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(Bonds);
+IMPLEMENT_CREATABLE_OVITO_CLASS(Bonds);
+OVITO_CLASSINFO(Bonds, "DisplayName", "Bonds");
+OVITO_CLASSINFO(Bonds, "ClassNameAlias", "BondsObject");  // For backward compatibility with OVITO 3.9.2
 
 /******************************************************************************
 * Constructor.
 ******************************************************************************/
-Bonds::Bonds(ObjectInitializationFlags flags) : PropertyContainer(flags)
+void Bonds::initializeObject(ObjectInitializationFlags flags)
 {
+    PropertyContainer::initializeObject(flags);
+
     // Assign the default data object identifier.
     setIdentifier(OOClass().pythonName());
 
@@ -125,16 +129,17 @@ size_t Bonds::addBonds(const std::vector<Bond>& newBonds, BondsVis* bondsVis, co
         // Insert bond type.
         if(bondTypeProperty) {
             bondTypeProperty->fill<int32_t>(bondType->numericId());
-            bondTypeProperty->addElementType(std::move(bondType));
+            if(!bondTypeProperty->elementType(bondType->numericId()))
+                bondTypeProperty->addElementType(std::move(bondType));
         }
 
         // Insert other bond properties.
         for(const auto& bprop : bondProperties) {
             OVITO_ASSERT(bprop);
             OVITO_ASSERT(bprop->size() == newBonds.size());
-            OVITO_ASSERT(bprop->type() != Bonds::TopologyProperty);
-            OVITO_ASSERT(bprop->type() != Bonds::PeriodicImageProperty);
-            OVITO_ASSERT(!bondTypeProperty || bprop->type() != Bonds::TypeProperty);
+            OVITO_ASSERT(bprop->typeId() != Bonds::TopologyProperty);
+            OVITO_ASSERT(bprop->typeId() != Bonds::PeriodicImageProperty);
+            OVITO_ASSERT(!bondTypeProperty || bprop->typeId() != Bonds::TypeProperty);
             createProperty(bprop);
         }
 
@@ -197,7 +202,7 @@ size_t Bonds::addBonds(const std::vector<Bond>& newBonds, BondsVis* bondsVis, co
 
         // Initialize property values of existing properties for new bonds.
         for(Property* bondProperty : makePropertiesMutable()) {
-            if(bondProperty->type() == Bonds::ColorProperty) {
+            if(bondProperty->typeId() == Bonds::ColorProperty) {
                 if(particles) {
                     ConstPropertyPtr bondColors;
                     if(particles->bonds() != this) {
@@ -218,13 +223,13 @@ size_t Bonds::addBonds(const std::vector<Bond>& newBonds, BondsVis* bondsVis, co
         for(const auto& bprop : bondProperties) {
             OVITO_ASSERT(bprop);
             OVITO_ASSERT(bprop->size() == newBonds.size());
-            OVITO_ASSERT(bprop->type() != Bonds::TopologyProperty);
-            OVITO_ASSERT(bprop->type() != Bonds::PeriodicImageProperty);
-            OVITO_ASSERT(!bondType || bprop->type() != Bonds::TypeProperty);
+            OVITO_ASSERT(bprop->typeId() != Bonds::TopologyProperty);
+            OVITO_ASSERT(bprop->typeId() != Bonds::PeriodicImageProperty);
+            OVITO_ASSERT(!bondType || bprop->typeId() != Bonds::TypeProperty);
 
             Property* property;
-            if(bprop->type() != Bonds::UserProperty) {
-                property = createProperty(DataBuffer::Initialized, bprop->type());
+            if(bprop->isStandardProperty()) {
+                property = createProperty(DataBuffer::Initialized, bprop->typeId());
             }
             else {
                 property = createProperty(DataBuffer::Initialized, bprop->name(), bprop->dataType(), bprop->componentCount());
@@ -267,7 +272,7 @@ PropertyPtr Bonds::OOMetaClass::createStandardPropertyInternal(DataBuffer::Buffe
         if(type == ColorProperty) {
             if(const Particles* particles = dynamic_object_cast<Particles>(containerPath[containerPath.size()-2])) {
                 ConstPropertyPtr property = particles->inputBondColors();
-                OVITO_ASSERT(property && property->size() == elementCount && property->type() == ColorProperty);
+                OVITO_ASSERT(property && property->size() == elementCount && property->typeId() == ColorProperty);
                 return std::move(property).makeMutable();
             }
         }
@@ -277,7 +282,7 @@ PropertyPtr Bonds::OOMetaClass::createStandardPropertyInternal(DataBuffer::Buffe
                 ConstPropertyPtr property = bonds->inputBondWidths();
                 OVITO_ASSERT(property);
                 OVITO_ASSERT(property->size() == elementCount);
-                OVITO_ASSERT(property->type() == WidthProperty);
+                OVITO_ASSERT(property->typeId() == WidthProperty);
                 return std::move(property).makeMutable();
             }
         }
@@ -344,10 +349,6 @@ void Bonds::OOMetaClass::initialize()
 {
     PropertyContainerClass::initialize();
 
-    // Enable automatic conversion of a BondPropertyReference to a generic PropertyReference and vice versa.
-    QMetaType::registerConverter<BondPropertyReference, PropertyReference>();
-    QMetaType::registerConverter<PropertyReference, BondPropertyReference>();
-
     setPropertyClassDisplayName(tr("Bonds"));
     setElementDescriptionName(QStringLiteral("bonds"));
     setPythonName(QStringLiteral("bonds"));
@@ -372,9 +373,9 @@ void Bonds::OOMetaClass::initialize()
 /******************************************************************************
 * Returns the default color for a numeric type ID.
 ******************************************************************************/
-Color Bonds::OOMetaClass::getElementTypeDefaultColor(const PropertyReference& property, const QString& typeName, int numericTypeId, bool loadUserDefaults) const
+Color Bonds::OOMetaClass::getElementTypeDefaultColor(const OwnerPropertyRef& property, const QString& typeName, int numericTypeId, bool loadUserDefaults) const
 {
-    if(property.type() == Bonds::TypeProperty) {
+    if(property.typeId() == Bonds::TypeProperty) {
 
         // Initial standard colors assigned to new bond types:
         static const Color defaultTypeColors[] = {
@@ -388,7 +389,7 @@ Color Bonds::OOMetaClass::getElementTypeDefaultColor(const PropertyReference& pr
             Color(1.0,  1.0,  0.7), // 7
             Color(0.97, 0.97, 0.97) // 8
         };
-        return defaultTypeColors[std::abs(numericTypeId) % (sizeof(defaultTypeColors) / sizeof(defaultTypeColors[0]))];
+        return defaultTypeColors[std::abs(numericTypeId) % std::size(defaultTypeColors)];
     }
 
     return PropertyContainerClass::getElementTypeDefaultColor(property, typeName, numericTypeId, loadUserDefaults);
@@ -397,7 +398,7 @@ Color Bonds::OOMetaClass::getElementTypeDefaultColor(const PropertyReference& pr
 /******************************************************************************
 * Returns the index of the element that was picked in a viewport.
 ******************************************************************************/
-std::pair<size_t, ConstDataObjectPath> Bonds::OOMetaClass::elementFromPickResult(const ViewportPickResult& pickResult) const
+std::pair<size_t, ConstDataObjectPath> Bonds::OOMetaClass::elementFromPickResult(const ViewportWindow::PickResult& pickResult) const
 {
     // Check if a bond was picked.
     if(BondPickInfo* pickInfo = dynamic_object_cast<BondPickInfo>(pickResult.pickInfo())) {
@@ -510,7 +511,7 @@ size_t Bonds::OOMetaClass::remapElementIndex(const ConstDataObjectPath& source,
 * Determines which elements are located within the given
 * viewport fence region (=2D polygon).
 ******************************************************************************/
-boost::dynamic_bitset<> Bonds::OOMetaClass::viewportFenceSelection(const QVector<Point2>& fence, const ConstDataObjectPath& objectPath, Pipeline* pipeline, const Matrix4& projectionTM) const
+ConstPropertyPtr Bonds::OOMetaClass::viewportFenceSelection(const QVector<Point2>& fence, const ConstDataObjectPath& objectPath, Pipeline* pipeline, const Matrix4& projectionTM) const
 {
     const Bonds* bonds = static_object_cast<Bonds>(objectPath.back());
     const Particles* particles = dynamic_object_cast<Particles>(objectPath.size() >= 2 ? objectPath[objectPath.size()-2] : nullptr);
@@ -522,49 +523,47 @@ boost::dynamic_bitset<> Bonds::OOMetaClass::viewportFenceSelection(const QVector
                 if(!bonds->visElement() || bonds->visElement()->isEnabled() == false)
                     throw Exception(tr("Cannot select bonds while the corresponding visual element is disabled. Please enable the display of bonds first."));
 
-                boost::dynamic_bitset<> fullSelection(topologyProperty.size());
-                QMutex mutex;
-                parallelForChunks(topologyProperty.size(), [&topologyProperty, &posProperty, &projectionTM, &fence, &mutex, &fullSelection](size_t startIndex, size_t chunkSize) {
-                    boost::dynamic_bitset<> selection(fullSelection.size());
-                    for(size_t index = startIndex; chunkSize != 0; chunkSize--, index++) {
-                        const ParticleIndexPair& t = topologyProperty[index];
-                        int insideCount = 0;
-                        for(size_t i = 0; i < 2; i++) {
-                            if(t[i] >= (qlonglong)posProperty.size()) continue;
-                            const Point3& p = posProperty[t[i]];
+                // Create the output property storage.
+                PropertyPtr selection = Particles::OOClass().createStandardProperty(DataBuffer::Uninitialized, topologyProperty.size(), Bonds::SelectionProperty, objectPath);
 
-                            // Project particle center to screen coordinates.
-                            Point3 projPos = projectionTM * p;
+                BufferWriteAccess<SelectionIntType, access_mode::discard_write> selectionAcc{selection};
+                parallelFor(topologyProperty.size(), 16000, TaskProgress::Ignore, [&](size_t index) {
+                    selectionAcc[index] = 0;
+                    const ParticleIndexPair& t = topologyProperty[index];
+                    int insideCount = 0;
+                    for(size_t i = 0; i < 2; i++) {
+                        if(t[i] >= (qlonglong)posProperty.size())
+                            continue;
+                        const Point3& p = posProperty[t[i]];
 
-                            // Perform z-clipping.
-                            if(std::abs(projPos.z()) >= FloatType(1))
-                                break;
+                        // Project particle center to screen coordinates.
+                        Point3 projPos = projectionTM * p;
 
-                            // Perform point-in-polygon test.
-                            int intersectionsLeft = 0;
-                            int intersectionsRight = 0;
-                            for(auto p2 = fence.constBegin(), p1 = p2 + (fence.size()-1); p2 != fence.constEnd(); p1 = p2++) {
-                                if(p1->y() == p2->y()) continue;
-                                if(projPos.y() >= p1->y() && projPos.y() >= p2->y()) continue;
-                                if(projPos.y() < p1->y() && projPos.y() < p2->y()) continue;
-                                FloatType xint = (projPos.y() - p2->y()) / (p1->y() - p2->y()) * (p1->x() - p2->x()) + p2->x();
-                                if(xint >= projPos.x())
-                                    intersectionsRight++;
-                                else
-                                    intersectionsLeft++;
-                            }
-                            if(intersectionsRight & 1)
-                                insideCount++;
+                        // Perform z-clipping.
+                        if(std::abs(projPos.z()) >= FloatType(1))
+                            break;
+
+                        // Perform point-in-polygon test.
+                        int intersectionsRight = 0;
+                        for(auto p2 = fence.constBegin(), p1 = p2 + (fence.size()-1); p2 != fence.constEnd(); p1 = p2++) {
+                            if(p1->y() == p2->y())
+                                continue;
+                            if(projPos.y() >= p1->y() && projPos.y() >= p2->y())
+                                continue;
+                            if(projPos.y() < p1->y() && projPos.y() < p2->y())
+                                continue;
+                            FloatType xint = (projPos.y() - p2->y()) / (p1->y() - p2->y()) * (p1->x() - p2->x()) + p2->x();
+                            if(xint >= projPos.x())
+                                intersectionsRight++;
                         }
-                        if(insideCount == 2)
-                            selection.set(index);
+                        if(intersectionsRight & 1)
+                            insideCount++;
                     }
-                    // Transfer thread-local results to output bit array.
-                    QMutexLocker locker(&mutex);
-                    fullSelection |= selection;
+                    if(insideCount == 2)
+                        selectionAcc[index] = 1;
                 });
 
-                return fullSelection;
+                return selection;
             }
         }
     }
@@ -574,10 +573,10 @@ boost::dynamic_bitset<> Bonds::OOMetaClass::viewportFenceSelection(const QVector
 }
 
 /******************************************************************************
-* Returns the base point and vector information for visualizing a vector
-* property from this container using a VectorVis element.
-******************************************************************************/
-std::tuple<ConstDataBufferPtr, ConstDataBufferPtr> Bonds::getVectorVisData(const ConstDataObjectPath& path, const PipelineFlowState& state, MixedKeyCache& visCache) const
+ * Returns the data for visualizing a vector property from this container using a VectorVis element.
+ ******************************************************************************/
+VectorVis::VectorData Bonds::getVectorVisData(const ConstDataObjectPath& path, const PipelineFlowState& state,
+                                              const RendererResourceCache::ResourceFrame& visCache) const
 {
     OVITO_ASSERT(path.lastAs<Bonds>(1) == this);
     verifyIntegrity();
@@ -590,39 +589,39 @@ std::tuple<ConstDataBufferPtr, ConstDataBufferPtr> Bonds::getVectorVisData(const
             const SimulationCell* simulationCell = state.getObject<SimulationCell>();
 
             // Look up the bond centers in the cache.
-            using CacheKey = RendererResourceKey<struct BondCentersCache, ConstDataObjectRef, ConstDataObjectRef>;
-            auto& basePositions = visCache.get<ConstDataBufferPtr>(CacheKey(particles, simulationCell));
-            if(!basePositions) {
-                // Compute bond centers.
-                BufferFactory<Point3> centers(elementCount());
-                BufferReadAccess<ParticleIndexPair> bondTopology(bondTopologyProperty);
-                BufferReadAccess<Vector3I> bondPeriodicImages(bondPeriodicImageProperty);
-                BufferReadAccess<Point3> positions(positionProperty);
+            const auto& basePositions = visCache.lookup<ConstDataBufferPtr>(
+                RendererResourceKey<struct BondCentersCache, ConstDataObjectRef, ConstDataObjectRef>{ particles, simulationCell },
+                [&](ConstDataBufferPtr& basePositions) {
+                    // Compute bond centers.
+                    BufferFactory<Point3> centers(elementCount());
+                    BufferReadAccess<ParticleIndexPair> bondTopology(bondTopologyProperty);
+                    BufferReadAccess<Vector3I> bondPeriodicImages(bondPeriodicImageProperty);
+                    BufferReadAccess<Point3> positions(positionProperty);
 
-                size_t particleCount = positions.size();
-                const AffineTransformation cell = simulationCell ? simulationCell->cellMatrix() : AffineTransformation::Zero();
+                    size_t particleCount = positions.size();
+                    const AffineTransformation cell = simulationCell ? simulationCell->cellMatrix() : AffineTransformation::Zero();
 
-                for(size_t bondIndex = 0; bondIndex < bondTopology.size(); bondIndex++) {
-                    size_t index1 = bondTopology[bondIndex][0];
-                    size_t index2 = bondTopology[bondIndex][1];
-                    if(index1 >= particleCount || index2 >= particleCount) {
-                        centers[bondIndex] = Point3::Origin();
-                        continue;
-                    }
+                    for(size_t bondIndex = 0; bondIndex < bondTopology.size(); bondIndex++) {
+                        size_t index1 = bondTopology[bondIndex][0];
+                        size_t index2 = bondTopology[bondIndex][1];
+                        if(index1 >= particleCount || index2 >= particleCount) {
+                            centers[bondIndex] = Point3::Origin();
+                            continue;
+                        }
 
-                    Vector3 vec = positions[index2] - positions[index1];
-                    if(bondPeriodicImageProperty) {
-                        for(size_t k = 0; k < 3; k++) {
-                            if(int d = bondPeriodicImages[bondIndex][k]) {
-                                vec += cell.column(k) * (FloatType)d;
+                        Vector3 vec = positions[index2] - positions[index1];
+                        if(bondPeriodicImageProperty) {
+                            for(size_t k = 0; k < 3; k++) {
+                                if(int d = bondPeriodicImages[bondIndex][k]) {
+                                    vec += cell.column(k) * (FloatType)d;
+                                }
                             }
                         }
+                        centers[bondIndex] = positions[index1] + FloatType(0.5) * vec;
                     }
-                    centers[bondIndex] = positions[index1] + FloatType(0.5) * vec;
-                }
-                basePositions = centers.take();
-            }
-            return { basePositions, path.lastAs<DataBuffer>() };
+                    basePositions = centers.take();
+                });
+            return {basePositions, path.lastAs<DataBuffer>(), nullptr, nullptr, nullptr};
         }
     }
     return {};
