@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -33,8 +33,9 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(BondsVis);
-IMPLEMENT_OVITO_CLASS(BondPickInfo);
+IMPLEMENT_CREATABLE_OVITO_CLASS(BondsVis);
+OVITO_CLASSINFO(BondsVis, "DisplayName", "Bonds");
+IMPLEMENT_ABSTRACT_OVITO_CLASS(BondPickInfo);
 DEFINE_PROPERTY_FIELD(BondsVis, bondWidth);
 DEFINE_PROPERTY_FIELD(BondsVis, bondColor);
 DEFINE_PROPERTY_FIELD(BondsVis, shadingMode);
@@ -45,25 +46,15 @@ SET_PROPERTY_FIELD_LABEL(BondsVis, shadingMode, "Shading mode");
 SET_PROPERTY_FIELD_LABEL(BondsVis, coloringMode, "Coloring mode");
 SET_PROPERTY_FIELD_UNITS_AND_MINIMUM(BondsVis, bondWidth, WorldParameterUnit, 0);
 
-/******************************************************************************
-* Constructor.
-******************************************************************************/
-BondsVis::BondsVis(ObjectInitializationFlags flags) : DataVis(flags),
-    _bondWidth(0.4),
-    _bondColor(0.6, 0.6, 0.6),
-    _shadingMode(NormalShading),
-    _coloringMode(ParticleBasedColoring)
-{
-}
-
 /******************************************************************************
 * Computes the bounding box of the visual element.
 ******************************************************************************/
-Box3 BondsVis::boundingBox(AnimationTime time, const ConstDataObjectPath& path, const Pipeline* pipeline, const PipelineFlowState& flowState, MixedKeyCache& visCache, TimeInterval& validityInterval)
+Box3 BondsVis::boundingBoxImmediate(AnimationTime time, const ConstDataObjectPath& path, const Pipeline* pipeline, const PipelineFlowState& flowState, TimeInterval& validityInterval)
 {
     const Bonds* bonds = path.lastAs<Bonds>(0);
     const Particles* particles = path.lastAs<Particles>(1);
-    if(!bonds || !particles) return {};
+    if(!bonds || !particles)
+        return {};
     particles->verifyIntegrity();
     bonds->verifyIntegrity();
     const Property* bondTopologyProperty = bonds->getProperty(Bonds::TopologyProperty);
@@ -72,87 +63,61 @@ Box3 BondsVis::boundingBox(AnimationTime time, const ConstDataObjectPath& path,
     const Property* positionProperty = particles->getProperty(Particles::PositionProperty);
     const SimulationCell* simulationCell = flowState.getObject<SimulationCell>();
 
-    // The key type used for caching the computed bounding box:
-    using CacheKey = RendererResourceKey<struct BondsVisBoundingBoxCache,
-        ConstDataObjectRef,     // Bond topology property
-        ConstDataObjectRef,     // Bond PBC vector property
-        ConstDataObjectRef,     // Bond width property
-        ConstDataObjectRef,     // Particle position property
-        ConstDataObjectRef,     // Simulation cell
-        FloatType               // Bond width
-    >;
+    // Compute bounding box from bond data.
+    if(bondTopologyProperty && positionProperty) {
+        Box3 bbox;
 
-    // Look up the bounding box in the vis cache.
-    auto& bbox = visCache.get<Box3>(CacheKey(
-            bondTopologyProperty,
-            bondPeriodicImageProperty,
-            bondWidthProperty,
-            positionProperty,
-            simulationCell,
-            bondWidth()));
+        BufferReadAccess<ParticleIndexPair> bondTopology(bondTopologyProperty);
+        BufferReadAccess<Vector3I> bondPeriodicImages(bondPeriodicImageProperty);
+        BufferReadAccess<Point3> positions(positionProperty);
 
-    // Check if the cached bounding box information is still up to date.
-    if(bbox.isEmpty()) {
+        size_t particleCount = positions.size();
+        const AffineTransformation cell = simulationCell ? simulationCell->cellMatrix() : AffineTransformation::Zero();
 
-        // If not, recompute bounding box from bond data.
-        if(bondTopologyProperty && positionProperty) {
+        for(size_t bondIndex = 0; bondIndex < bondTopology.size(); bondIndex++) {
+            size_t index1 = bondTopology[bondIndex][0];
+            size_t index2 = bondTopology[bondIndex][1];
+            if(index1 >= particleCount || index2 >= particleCount)
+                continue;
 
-            BufferReadAccess<ParticleIndexPair> bondTopology(bondTopologyProperty);
-            BufferReadAccess<Vector3I> bondPeriodicImages(bondPeriodicImageProperty);
-            BufferReadAccess<Point3> positions(positionProperty);
-
-            size_t particleCount = positions.size();
-            const AffineTransformation cell = simulationCell ? simulationCell->cellMatrix() : AffineTransformation::Zero();
-
-            for(size_t bondIndex = 0; bondIndex < bondTopology.size(); bondIndex++) {
-                size_t index1 = bondTopology[bondIndex][0];
-                size_t index2 = bondTopology[bondIndex][1];
-                if(index1 >= particleCount || index2 >= particleCount)
-                    continue;
-
-                bbox.addPoint(positions[index1]);
-                bbox.addPoint(positions[index2]);
-                if(bondPeriodicImages && bondPeriodicImages[bondIndex] != Vector3I::Zero()) {
-                    Vector3 vec = positions[index2] - positions[index1];
-                    const Vector3I& pbcShift = bondPeriodicImages[bondIndex];
-                    for(size_t k = 0; k < 3; k++) {
-                        if(pbcShift[k] != 0) vec += cell.column(k) * (FloatType)pbcShift[k];
-                    }
-                    bbox.addPoint(positions[index1] + (vec * FloatType(0.5)));
-                    bbox.addPoint(positions[index2] - (vec * FloatType(0.5)));
+            bbox.addPoint(positions[index1]);
+            bbox.addPoint(positions[index2]);
+            if(bondPeriodicImages && bondPeriodicImages[bondIndex] != Vector3I::Zero()) {
+                Vector3 vec = positions[index2] - positions[index1];
+                const Vector3I& pbcShift = bondPeriodicImages[bondIndex];
+                for(size_t k = 0; k < 3; k++) {
+                    if(pbcShift[k] != 0) vec += cell.column(k) * (FloatType)pbcShift[k];
                 }
+                bbox.addPoint(positions[index1] + (vec * FloatType(0.5)));
+                bbox.addPoint(positions[index2] - (vec * FloatType(0.5)));
             }
-
-            // Extend box to account for width of bonds.
-            GraphicsFloatType maxBondWidth = std::max(static_cast<GraphicsFloatType>(bondWidth()), GraphicsFloatType(0));
-            if(bondWidthProperty && bondWidthProperty->size() != 0) {
-                BufferReadAccess<GraphicsFloatType> widthArray(bondWidthProperty);
-                auto minmax = std::minmax_element(widthArray.cbegin(), widthArray.cend());
-                if(*minmax.first <= 0)
-                    maxBondWidth = std::max(maxBondWidth, *minmax.second);
-                else
-                    maxBondWidth = *minmax.second;
-            }
-            bbox = bbox.padBox(maxBondWidth / FloatType(2) * std::sqrt(FloatType(3)));
         }
+
+        // Extend box to account for width of bonds.
+        GraphicsFloatType maxBondWidth = std::max(static_cast<GraphicsFloatType>(bondWidth()), GraphicsFloatType(0));
+        if(bondWidthProperty && bondWidthProperty->size() != 0) {
+            BufferReadAccess<GraphicsFloatType> widthArray(bondWidthProperty);
+            auto minmax = std::minmax_element(widthArray.cbegin(), widthArray.cend());
+            if(*minmax.first <= 0)
+                maxBondWidth = std::max(maxBondWidth, *minmax.second);
+            else
+                maxBondWidth = *minmax.second;
+        }
+
+        return bbox.padBox(maxBondWidth / FloatType(2) * std::sqrt(FloatType(3)));
     }
-    return bbox;
+    return {};
 }
 
 /******************************************************************************
 * Lets the visualization element render the data object.
 ******************************************************************************/
-PipelineStatus BondsVis::render(AnimationTime time, const ConstDataObjectPath& path, const PipelineFlowState& flowState, SceneRenderer* renderer, const Pipeline* pipeline)
+std::variant<PipelineStatus, Future<PipelineStatus>> BondsVis::render(const ConstDataObjectPath& path, const PipelineFlowState& flowState, FrameGraph& frameGraph, const SceneNode* sceneNode)
 {
-    if(renderer->isBoundingBoxPass()) {
-        TimeInterval validityInterval;
-        renderer->addToLocalBoundingBox(boundingBox(time, path, pipeline, flowState, renderer->visCache(), validityInterval));
-        return {};
-    }
-
     const Bonds* bonds = path.lastAs<Bonds>(0);
     const Particles* particles = path.lastAs<Particles>(1);
-    if(!bonds || !particles) return {};
+    if(!bonds || !particles)
+        return {};
     particles->verifyIntegrity();
     bonds->verifyIntegrity();
     const Property* bondTopologyProperty = bonds->getProperty(Bonds::TopologyProperty);
@@ -162,7 +127,7 @@ PipelineStatus BondsVis::render(AnimationTime time, const ConstDataObjectPath& p
     const Property* bondTypeProperty = (coloringMode() == ByTypeColoring) ? bonds->getProperty(Bonds::TypeProperty) : nullptr;
     const Property* bondColorProperty = bonds->getProperty(Bonds::ColorProperty);
     const Property* bondWidthProperty = bonds->getProperty(Bonds::WidthProperty);
-    const Property* bondSelectionProperty = renderer->isInteractive() ? bonds->getProperty(Bonds::SelectionProperty) : nullptr;
+    const Property* bondSelectionProperty = frameGraph.isInteractive() ? bonds->getProperty(Bonds::SelectionProperty) : nullptr;
     const Property* transparencyProperty = bonds->getProperty(Bonds::TransparencyProperty);
 
     // Obtain particle-related properties and the vis element.
@@ -198,18 +163,15 @@ PipelineStatus BondsVis::render(AnimationTime time, const ConstDataObjectPath& p
         FloatType,              // Bond width
         Color,                  // Bond uniform color
         ColoringMode,           // Bond coloring mode
-        ShadingMode             // Bond shading mode
+        ShadingMode,            // Bond shading mode
+        bool                    // Render nodal vertices
     >;
 
-    // The data structure stored in the vis cache.
-    struct CacheValue {
-        CylinderPrimitive cylinders;
-        ParticlePrimitive vertices;
-        OORef<BondPickInfo> pickInfo;
-    };
+    // Make sure the primitive for the nodal vertices gets created if particles display is turned off or if particles are semi-transparent.
+    bool renderNodalVertices = !transparencyProperty && !bondWidthProperty && (!particleVis || particleVis->isEnabled() == false || particleTransparencyProperty != nullptr);
 
     // Lookup the rendering primitive in the vis cache.
-    auto& visCache = renderer->visCache().get<CacheValue>(CacheKey(
+    const auto& [cylinders, vertices, pickInfo] = frameGraph.visCache().lookup<std::tuple<CylinderPrimitive, ParticlePrimitive, OORef<BondPickInfo>>>(CacheKey(
             bondTopologyProperty,
             bondPeriodicImageProperty,
             positionProperty,
@@ -225,161 +187,151 @@ PipelineStatus BondsVis::render(AnimationTime time, const ConstDataObjectPath& p
             bondWidth(),
             bondColor(),
             coloringMode(),
-            shadingMode()));
+            shadingMode(),
+            renderNodalVertices),
+        [&](CylinderPrimitive& cylinders, ParticlePrimitive& vertices, OORef<BondPickInfo>& pickInfo) {
+            FloatType bondDiameter = bondWidth();
+            if(bondTopologyProperty && positionProperty && bondDiameter > 0) {
 
-    // Make sure the primitive for the nodal vertices gets created if particles display is turned off or if particles are semi-transparent.
-    bool renderNodalVertices = !transparencyProperty && !bondWidthProperty && (!particleVis || particleVis->isEnabled() == false || particleTransparencyProperty != nullptr);
-    if(renderNodalVertices && !visCache.vertices.positions())
-        visCache.cylinders.setPositions(nullptr, nullptr);
+                // Allocate buffers for the bonds geometry.
+                BufferFactory<Point3G> bondPositions1(bondTopologyProperty->size() * 2);
+                BufferFactory<Point3G> bondPositions2(bondTopologyProperty->size() * 2);
+                BufferFactory<ColorG> bondColors(bondTopologyProperty->size() * 2);
+                BufferFactory<GraphicsFloatType> bondTransparencies = transparencyProperty ? BufferFactory<GraphicsFloatType>(bondTopologyProperty->size() * 2) : BufferFactory<GraphicsFloatType>{};
+                BufferFactory<GraphicsFloatType> bondWidths = bondWidthProperty ? BufferFactory<GraphicsFloatType>(bondTopologyProperty->size() * 2) : BufferFactory<GraphicsFloatType>{};
 
-    // Check if we already have a valid rendering primitive that is up to date.
-    if(!visCache.cylinders.basePositions()) {
+                // Allocate buffers for the nodal vertices.
+                BufferFactory<ColorG> nodalColors = renderNodalVertices ? BufferFactory<ColorG>(positionProperty->size()) : BufferFactory<ColorG>{};
+                BufferFactory<GraphicsFloatType> nodalTransparencies = (renderNodalVertices && transparencyProperty) ? BufferFactory<GraphicsFloatType>(positionProperty->size()) : BufferFactory<GraphicsFloatType>{};
+                BufferFactory<int32_t> nodalIndices = renderNodalVertices ? BufferFactory<int32_t>(0) : BufferFactory<int32_t>{};
+                boost::dynamic_bitset<> visitedParticles(renderNodalVertices ? positionProperty->size() : 0);
+                OVITO_ASSERT(nodalColors || !nodalTransparencies);
 
-        FloatType bondDiameter = bondWidth();
-        if(bondTopologyProperty && positionProperty && bondDiameter > 0) {
+                // Cache some values.
+                BufferReadAccess<Point3> positions(positionProperty);
+                size_t particleCount = positions.size();
+                const AffineTransformation cell = simulationCell ? simulationCell->cellMatrix() : AffineTransformation::Zero();
 
-            // Allocate buffers for the bonds geometry.
-            BufferFactory<Point3G> bondPositions1(bondTopologyProperty->size() * 2);
-            BufferFactory<Point3G> bondPositions2(bondTopologyProperty->size() * 2);
-            BufferFactory<ColorG> bondColors(bondTopologyProperty->size() * 2);
-            BufferFactory<GraphicsFloatType> bondTransparencies = transparencyProperty ? BufferFactory<GraphicsFloatType>(bondTopologyProperty->size() * 2) : BufferFactory<GraphicsFloatType>{};
-            BufferFactory<GraphicsFloatType> bondWidths = bondWidthProperty ? BufferFactory<GraphicsFloatType>(bondTopologyProperty->size() * 2) : BufferFactory<GraphicsFloatType>{};
+                // Obtain the radii of the particles.
+                BufferReadAccessAndRef<GraphicsFloatType> particleRadii;
+                if(particleVis)
+                    particleRadii = ConstDataBufferPtr(particleVis->particleRadii(particles, false));
+                // Make sure the particle radius array has the correct length.
+                if(particleRadii && particleRadii.size() != particleCount)
+                    particleRadii.reset();
 
-            // Allocate buffers for the nodal vertices.
-            BufferFactory<ColorG> nodalColors = renderNodalVertices ? BufferFactory<ColorG>(positionProperty->size()) : BufferFactory<ColorG>{};
-            BufferFactory<GraphicsFloatType> nodalTransparencies = (renderNodalVertices && transparencyProperty) ? BufferFactory<GraphicsFloatType>(positionProperty->size()) : BufferFactory<GraphicsFloatType>{};
-            BufferFactory<int32_t> nodalIndices = renderNodalVertices ? BufferFactory<int32_t>(0) : BufferFactory<int32_t>{};
-            boost::dynamic_bitset<> visitedParticles(renderNodalVertices ? positionProperty->size() : 0);
-            OVITO_ASSERT(nodalColors || !nodalTransparencies);
+                // Determine half-bond colors.
+                std::vector<ColorG> colors = halfBondColors(particles, frameGraph.isInteractive(), coloringMode(), false);
+                OVITO_ASSERT(colors.size() == bondPositions1.size());
 
-            // Cache some values.
-            BufferReadAccess<Point3> positions(positionProperty);
-            size_t particleCount = positions.size();
-            const AffineTransformation cell = simulationCell ? simulationCell->cellMatrix() : AffineTransformation::Zero();
-
-            // Obtain the radii of the particles.
-            BufferReadAccessAndRef<GraphicsFloatType> particleRadii;
-            if(particleVis)
-                particleRadii = ConstDataBufferPtr(particleVis->particleRadii(particles, false));
-            // Make sure the particle radius array has the correct length.
-            if(particleRadii && particleRadii.size() != particleCount)
-                particleRadii.reset();
-
-            // Determine half-bond colors.
-            std::vector<ColorG> colors = halfBondColors(particles, renderer->isInteractive(), coloringMode(), false);
-            OVITO_ASSERT(colors.size() == bondPositions1.size());
-
-            size_t cylinderIndex = 0;
-            auto color = colors.cbegin();
-            BufferReadAccess<ParticleIndexPair> bonds(bondTopologyProperty);
-            BufferReadAccess<Vector3I> bondPeriodicImages(bondPeriodicImageProperty);
-            BufferReadAccess<GraphicsFloatType> bondInputTransparency(transparencyProperty);
-            BufferReadAccess<GraphicsFloatType> bondInputWidths(bondWidthProperty);
-            for(size_t bondIndex = 0; bondIndex < bonds.size(); bondIndex++) {
-                size_t particleIndex1 = bonds[bondIndex][0];
-                size_t particleIndex2 = bonds[bondIndex][1];
-                if(particleIndex1 < particleCount && particleIndex2 < particleCount) {
-                    Vector3 vec = positions[particleIndex2] - positions[particleIndex1];
-                    bool isSplitBond = false;
-                    if(bondPeriodicImageProperty) {
-                        for(size_t k = 0; k < 3; k++) {
-                            if(int d = bondPeriodicImages[bondIndex][k]) {
-                                vec += cell.column(k) * (FloatType)d;
-                                isSplitBond = true;
+                size_t cylinderIndex = 0;
+                auto color = colors.cbegin();
+                BufferReadAccess<ParticleIndexPair> bonds(bondTopologyProperty);
+                BufferReadAccess<Vector3I> bondPeriodicImages(bondPeriodicImageProperty);
+                BufferReadAccess<GraphicsFloatType> bondInputTransparency(transparencyProperty);
+                BufferReadAccess<GraphicsFloatType> bondInputWidths(bondWidthProperty);
+                for(size_t bondIndex = 0; bondIndex < bonds.size(); bondIndex++) {
+                    size_t particleIndex1 = bonds[bondIndex][0];
+                    size_t particleIndex2 = bonds[bondIndex][1];
+                    if(particleIndex1 < particleCount && particleIndex2 < particleCount) {
+                        Vector3 vec = positions[particleIndex2] - positions[particleIndex1];
+                        bool isSplitBond = false;
+                        if(bondPeriodicImageProperty) {
+                            for(size_t k = 0; k < 3; k++) {
+                                if(int d = bondPeriodicImages[bondIndex][k]) {
+                                    vec += cell.column(k) * (FloatType)d;
+                                    isSplitBond = true;
+                                }
                             }
                         }
-                    }
-                    const Vector3G& gvec = vec.toDataType<GraphicsFloatType>();
-                    GraphicsFloatType t = 0.5;
-                    GraphicsFloatType blen = gvec.length() * GraphicsFloatType(2);
-                    if(particleRadii && blen != 0) {
-                        // This calculation determines the point where to split the bond into the two half-bonds
-                        // such that the border appears halfway between the two particles, which may have two different sizes.
-                        t = GraphicsFloatType(0.5) + std::min(GraphicsFloatType(0.5), particleRadii[particleIndex1]/blen) - std::min(GraphicsFloatType(0.5), particleRadii[particleIndex2]/blen);
-                    }
-                    bondColors[cylinderIndex] = *color++;
-                    if(nodalColors && !visitedParticles.test(particleIndex1)) {
-                        nodalColors[particleIndex1] = bondColors[cylinderIndex];
-                        if(nodalTransparencies)
-                            nodalTransparencies[particleIndex1] = bondInputTransparency[bondIndex];
-                        visitedParticles.set(particleIndex1);
-                        nodalIndices.push_back(particleIndex1);
-                    }
-                    if(bondTransparencies) bondTransparencies[cylinderIndex] = bondInputTransparency[bondIndex];
-                    if(bondWidths) bondWidths[cylinderIndex] = (bondInputWidths[bondIndex] <= GraphicsFloatType(0)) ? bondDiameter : bondInputWidths[bondIndex];
-                    bondPositions1[cylinderIndex] = positions[particleIndex1].toDataType<GraphicsFloatType>();
-                    bondPositions2[cylinderIndex] = bondPositions1[cylinderIndex] + gvec * t;
-                    if(isSplitBond)
-                        swap(bondPositions1[cylinderIndex], bondPositions2[cylinderIndex]);
-                    cylinderIndex++;
+                        const Vector3G& gvec = vec.toDataType<GraphicsFloatType>();
+                        GraphicsFloatType t = 0.5;
+                        GraphicsFloatType blen = gvec.length() * GraphicsFloatType(2);
+                        if(particleRadii && blen != 0) {
+                            // This calculation determines the point where to split the bond into the two half-bonds
+                            // such that the border appears halfway between the two particles, which may have two different sizes.
+                            t = GraphicsFloatType(0.5) + std::min(GraphicsFloatType(0.5), particleRadii[particleIndex1]/blen) - std::min(GraphicsFloatType(0.5), particleRadii[particleIndex2]/blen);
+                        }
+                        bondColors[cylinderIndex] = *color++;
+                        if(nodalColors && !visitedParticles.test(particleIndex1)) {
+                            nodalColors[particleIndex1] = bondColors[cylinderIndex];
+                            if(nodalTransparencies)
+                                nodalTransparencies[particleIndex1] = bondInputTransparency[bondIndex];
+                            visitedParticles.set(particleIndex1);
+                            nodalIndices.push_back(particleIndex1);
+                        }
+                        if(bondTransparencies) bondTransparencies[cylinderIndex] = bondInputTransparency[bondIndex];
+                        if(bondWidths) bondWidths[cylinderIndex] = (bondInputWidths[bondIndex] <= GraphicsFloatType(0)) ? bondDiameter : bondInputWidths[bondIndex];
+                        bondPositions1[cylinderIndex] = positions[particleIndex1].toDataType<GraphicsFloatType>();
+                        bondPositions2[cylinderIndex] = bondPositions1[cylinderIndex] + gvec * t;
+                        if(isSplitBond)
+                            swap(bondPositions1[cylinderIndex], bondPositions2[cylinderIndex]);
+                        cylinderIndex++;
 
-                    bondColors[cylinderIndex] = *color++;
-                    if(nodalColors && !visitedParticles.test(particleIndex2)) {
-                        nodalColors[particleIndex2] = bondColors[cylinderIndex];
-                        if(nodalTransparencies)
-                            nodalTransparencies[particleIndex2] = bondInputTransparency[bondIndex];
-                        visitedParticles.set(particleIndex2);
-                        nodalIndices.push_back(particleIndex2);
+                        bondColors[cylinderIndex] = *color++;
+                        if(nodalColors && !visitedParticles.test(particleIndex2)) {
+                            nodalColors[particleIndex2] = bondColors[cylinderIndex];
+                            if(nodalTransparencies)
+                                nodalTransparencies[particleIndex2] = bondInputTransparency[bondIndex];
+                            visitedParticles.set(particleIndex2);
+                            nodalIndices.push_back(particleIndex2);
+                        }
+                        if(bondTransparencies) bondTransparencies[cylinderIndex] = bondInputTransparency[bondIndex];
+                        if(bondWidths) bondWidths[cylinderIndex] = (bondInputWidths[bondIndex] <= GraphicsFloatType(0)) ? bondDiameter : bondInputWidths[bondIndex];
+                        bondPositions1[cylinderIndex] = positions[particleIndex2].toDataType<GraphicsFloatType>();
+                        bondPositions2[cylinderIndex] = bondPositions1[cylinderIndex] - gvec * (GraphicsFloatType(1) - t);
+                        if(isSplitBond)
+                            swap(bondPositions1[cylinderIndex], bondPositions2[cylinderIndex]);
+                        cylinderIndex++;
+                    }
+                    else {
+                        bondColors[cylinderIndex] = *color++;
+                        if(bondTransparencies) bondTransparencies[cylinderIndex] = 0;
+                        if(bondWidths) bondWidths[cylinderIndex] = 0;
+                        bondPositions1[cylinderIndex] = Point3G::Origin();
+                        bondPositions2[cylinderIndex++] = Point3G::Origin();
+
+                        bondColors[cylinderIndex] = *color++;
+                        if(bondTransparencies) bondTransparencies[cylinderIndex] = 0;
+                        if(bondWidths) bondWidths[cylinderIndex] = 0;
+                        bondPositions1[cylinderIndex] = Point3G::Origin();
+                        bondPositions2[cylinderIndex++] = Point3G::Origin();
                     }
-                    if(bondTransparencies) bondTransparencies[cylinderIndex] = bondInputTransparency[bondIndex];
-                    if(bondWidths) bondWidths[cylinderIndex] = (bondInputWidths[bondIndex] <= GraphicsFloatType(0)) ? bondDiameter : bondInputWidths[bondIndex];
-                    bondPositions1[cylinderIndex] = positions[particleIndex2].toDataType<GraphicsFloatType>();
-                    bondPositions2[cylinderIndex] = bondPositions1[cylinderIndex] - gvec * (GraphicsFloatType(1) - t);
-                    if(isSplitBond)
-                        swap(bondPositions1[cylinderIndex], bondPositions2[cylinderIndex]);
-                    cylinderIndex++;
                 }
-                else {
-                    bondColors[cylinderIndex] = *color++;
-                    if(bondTransparencies) bondTransparencies[cylinderIndex] = 0;
-                    if(bondWidths) bondWidths[cylinderIndex] = 0;
-                    bondPositions1[cylinderIndex] = Point3G::Origin();
-                    bondPositions2[cylinderIndex++] = Point3G::Origin();
 
-                    bondColors[cylinderIndex] = *color++;
-                    if(bondTransparencies) bondTransparencies[cylinderIndex] = 0;
-                    if(bondWidths) bondWidths[cylinderIndex] = 0;
-                    bondPositions1[cylinderIndex] = Point3G::Origin();
-                    bondPositions2[cylinderIndex++] = Point3G::Origin();
+                cylinders.setShape(CylinderPrimitive::CylinderShape);
+                cylinders.setShadingMode(static_cast<CylinderPrimitive::ShadingMode>(shadingMode()));
+                cylinders.setRenderSingleCylinderCap(transparencyProperty != nullptr);
+                cylinders.setUniformWidth(bondDiameter);
+                cylinders.setWidths(bondWidths.take());
+                cylinders.setPositions(bondPositions1.take(), bondPositions2.take());
+                cylinders.setColors(bondColors.take());
+                cylinders.setTransparencies(bondTransparencies.take());
+
+                if(renderNodalVertices) {
+                    OVITO_ASSERT(positionProperty);
+                    vertices.setParticleShape(ParticlePrimitive::SphericalShape);
+                    vertices.setShadingMode((shadingMode() == NormalShading) ? ParticlePrimitive::NormalShading : ParticlePrimitive::FlatShading);
+                    vertices.setRenderingQuality(ParticlePrimitive::HighQuality);
+                    vertices.setPositions(positionProperty);
+                    vertices.setUniformRadius(0.5 * bondDiameter);
+                    vertices.setColors(nodalColors.take());
+                    vertices.setIndices(nodalIndices.take());
+                    vertices.setTransparencies(nodalTransparencies.take());
                 }
             }
 
-            visCache.cylinders.setShape(CylinderPrimitive::CylinderShape);
-            visCache.cylinders.setShadingMode(static_cast<CylinderPrimitive::ShadingMode>(shadingMode()));
-            visCache.cylinders.setRenderSingleCylinderCap(transparencyProperty != nullptr);
-            visCache.cylinders.setUniformWidth(bondDiameter);
-            visCache.cylinders.setWidths(bondWidths.take());
-            visCache.cylinders.setPositions(bondPositions1.take(), bondPositions2.take());
-            visCache.cylinders.setColors(bondColors.take());
-            visCache.cylinders.setTransparencies(bondTransparencies.take());
+            pickInfo = OORef<BondPickInfo>::create(particles, simulationCell);
+        });
 
-            if(renderNodalVertices) {
-                OVITO_ASSERT(positionProperty);
-                visCache.vertices.setParticleShape(ParticlePrimitive::SphericalShape);
-                visCache.vertices.setShadingMode((shadingMode() == NormalShading) ? ParticlePrimitive::NormalShading : ParticlePrimitive::FlatShading);
-                visCache.vertices.setRenderingQuality(ParticlePrimitive::HighQuality);
-                visCache.vertices.setPositions(positionProperty);
-                visCache.vertices.setUniformRadius(0.5 * bondDiameter);
-                visCache.vertices.setColors(nodalColors.take());
-                visCache.vertices.setIndices(nodalIndices.take());
-                visCache.vertices.setTransparencies(nodalTransparencies.take());
-            }
-        }
-    }
-    if(!visCache.cylinders.basePositions())
+    if(!cylinders.basePositions())
         return {};
-    if(!visCache.pickInfo)
-        visCache.pickInfo = OORef<BondPickInfo>::create(particles, simulationCell);
 
-    renderer->beginPickObject(pipeline, visCache.pickInfo);
-    renderer->renderCylinders(visCache.cylinders);
-    renderer->endPickObject();
+    FrameGraph::RenderingCommandGroup& commandGroup = frameGraph.addCommandGroup(FrameGraph::SceneLayer);
+    frameGraph.addPrimitive(commandGroup, std::make_unique<CylinderPrimitive>(cylinders), sceneNode, pickInfo);
 
-    if(visCache.vertices.positions() && renderNodalVertices) {
-        renderer->beginPickObject(pipeline);
-        renderer->renderParticles(visCache.vertices);
-        renderer->endPickObject();
-    }
+    if(renderNodalVertices)
+        frameGraph.addPrimitive(commandGroup, std::make_unique<ParticlePrimitive>(vertices), sceneNode);
 
     return {};
 }
@@ -405,12 +357,6 @@ std::vector<ColorG> BondsVis::halfBondColors(const Particles* particles, bool hi
 
     // Get particle-related properties and the vis element.
     const ParticlesVis* particleVis = particles->visElement<ParticlesVis>();
-    BufferReadAccess<ColorG> particleColorProperty;
-    const Property* particleTypeProperty = nullptr;
-    if(coloringMode == ParticleBasedColoring && particleVis) {
-        particleColorProperty = particles->getProperty(Particles::ColorProperty);
-        particleTypeProperty = particleVis->getParticleTypeColorProperty(particles);
-    }
 
     std::vector<ColorG> output(bonds->elementCount() * 2);
     const ColorG defaultColor = bondColor().toDataType<GraphicsFloatType>();
@@ -443,20 +389,19 @@ std::vector<ColorG> BondsVis::halfBondColors(const Particles* particles, bool hi
         if(bondTypeProperty && bondTypeProperty->size() * 2 == output.size()) {
             // Assign colors based on bond types.
             // Generate a lookup map for bond type colors.
-            const std::map<int, Color>& colorMap = bondTypeProperty->typeColorMap();
+            const std::map<int, ColorG>& colorMap = bondTypeProperty->typeColorMap();
             std::array<ColorG,16> colorArray;
             // Check if all type IDs are within a small, non-negative range.
             // If yes, we can use an array lookup strategy. Otherwise we have to use a dictionary lookup strategy, which is slower.
-            if(boost::algorithm::all_of(colorMap,
-                    [&colorArray](const std::map<int, Color>::value_type& i) { return i.first >= 0 && i.first < (int)colorArray.size(); })) {
+            if(boost::algorithm::all_of(colorMap, [&colorArray](const auto& i) { return i.first >= 0 && i.first < (int)colorArray.size(); })) {
                 colorArray.fill(defaultColor);
                 for(const auto& entry : colorMap)
-                    colorArray[entry.first] = entry.second.toDataType<GraphicsFloatType>();
+                    colorArray[entry.first] = entry.second;
                 // Fill color array.
                 BufferReadAccess<int32_t> bondTypeData(bondTypeProperty);
                 const int32_t* t = bondTypeData.cbegin();
                 for(auto c = output.begin(); c != output.end(); ++t) {
-                    if(*t >= 0 && *t < (int)colorArray.size()) {
+                    if(*t >= 0 && (size_t)*t < colorArray.size()) {
                         *c++ = colorArray[*t];
                         *c++ = colorArray[*t];
                     }
@@ -472,8 +417,8 @@ std::vector<ColorG> BondsVis::halfBondColors(const Particles* particles, bool hi
                 const int32_t* t = bondTypeData.cbegin();
                 for(auto c = output.begin(); c != output.end(); ++t) {
                     if(auto it = colorMap.find(*t); it != colorMap.end()) {
-                        *c++ = it->second.toDataType<GraphicsFloatType>();
-                        *c++ = it->second.toDataType<GraphicsFloatType>();
+                        *c++ = it->second;
+                        *c++ = it->second;
                     }
                     else {
                         *c++ = defaultColor;
@@ -508,7 +453,7 @@ std::vector<ColorG> BondsVis::halfBondColors(const Particles* particles, bool hi
 * Returns a human-readable string describing the picked object,
 * which will be displayed in the status bar by OVITO.
 ******************************************************************************/
-QString BondPickInfo::infoString(Pipeline* pipeline, quint32 subobjectId)
+QString BondPickInfo::infoString(const Pipeline* pipeline, uint32_t subobjectId)
 {
     QString str;
     size_t bondIndex = subobjectId / 2;
