@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -27,7 +27,8 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(ParticleType);
+IMPLEMENT_CREATABLE_OVITO_CLASS(ParticleType);
+OVITO_CLASSINFO(ParticleType, "DisplayName", "Particle type");
 DEFINE_PROPERTY_FIELD(ParticleType, radius);
 DEFINE_PROPERTY_FIELD(ParticleType, radiusIsPrescribed);
 DEFINE_PROPERTY_FIELD(ParticleType, vdwRadius);
@@ -55,56 +56,41 @@ SET_PROPERTY_FIELD_LABEL(ParticleType, mass, "Mass");
 SET_PROPERTY_FIELD_UNITS_AND_MINIMUM(ParticleType, radius, WorldParameterUnit, 0);
 SET_PROPERTY_FIELD_UNITS_AND_MINIMUM(ParticleType, vdwRadius, WorldParameterUnit, 0);
 
-/******************************************************************************
-* Constructs a new particle type.
-******************************************************************************/
-ParticleType::ParticleType(ObjectInitializationFlags flags) : ElementType(flags),
-    _radius(0),
-    _radiusIsPrescribed(false),
-    _vdwRadius(0),
-    _shape(ParticlesVis::ParticleShape::Default),
-    _highlightShapeEdges(false),
-    _shapeBackfaceCullingEnabled(true),
-    _shapeUseMeshColor(false),
-    _mass(0)
-{
-}
-
 /******************************************************************************
 * Initializes the particle type's attributes to standard values.
 ******************************************************************************/
-void ParticleType::initializeType(const PropertyReference& property, bool loadUserDefaults)
+void ParticleType::initializeType(const OwnerPropertyRef& property, bool loadUserDefaults)
 {
     ElementType::initializeType(property, loadUserDefaults);
 
     // Load standard display radius.
     // First load the hardcoded default radius and freeze it, then load the user-defined default radius.
-    setRadius(getDefaultParticleRadius(static_cast<Particles::Type>(property.type()), nameOrNumericId(), numericId(), false, DisplayRadius));
+    setRadius(getDefaultParticleRadius(static_cast<Particles::Type>(property.typeId()), nameOrNumericId(), numericId(), false, DisplayRadius));
     freezeInitialParameterValues({SHADOW_PROPERTY_FIELD(ParticleType::radius)});
     if(loadUserDefaults)
-        setRadius(getDefaultParticleRadius(static_cast<Particles::Type>(property.type()), nameOrNumericId(), numericId(), true, DisplayRadius));
+        setRadius(getDefaultParticleRadius(static_cast<Particles::Type>(property.typeId()), nameOrNumericId(), numericId(), true, DisplayRadius));
 
     // Load standard van der Waals radius.
     // First load the hardcoded default radius and freeze it, then load the user-defined default radius.
-    setVdwRadius(getDefaultParticleRadius(static_cast<Particles::Type>(property.type()), nameOrNumericId(), numericId(), false, VanDerWaalsRadius));
+    setVdwRadius(getDefaultParticleRadius(static_cast<Particles::Type>(property.typeId()), nameOrNumericId(), numericId(), false, VanDerWaalsRadius));
     freezeInitialParameterValues({SHADOW_PROPERTY_FIELD(ParticleType::vdwRadius)});
     if(loadUserDefaults)
-        setVdwRadius(getDefaultParticleRadius(static_cast<Particles::Type>(property.type()), nameOrNumericId(), numericId(), true, VanDerWaalsRadius));
+        setVdwRadius(getDefaultParticleRadius(static_cast<Particles::Type>(property.typeId()), nameOrNumericId(), numericId(), true, VanDerWaalsRadius));
 
     // Load standard mass.
     // First load the hardcoded default mass and freeze it, then load the user-defined default mass.
-    setMass(getDefaultParticleMass(static_cast<Particles::Type>(property.type()), nameOrNumericId(), numericId(), false));
+    setMass(getDefaultParticleMass(static_cast<Particles::Type>(property.typeId()), nameOrNumericId(), numericId(), false));
     freezeInitialParameterValues({SHADOW_PROPERTY_FIELD(ParticleType::mass)});
     if(loadUserDefaults)
-        setMass(getDefaultParticleMass(static_cast<Particles::Type>(property.type()), nameOrNumericId(), numericId(), true));
+        setMass(getDefaultParticleMass(static_cast<Particles::Type>(property.typeId()), nameOrNumericId(), numericId(), true));
 }
 
 /******************************************************************************
 * Creates an editable proxy object for this DataObject and synchronizes its parameters.
 ******************************************************************************/
-void ParticleType::updateEditableProxies(PipelineFlowState& state, ConstDataObjectPath& dataPath) const
+void ParticleType::updateEditableProxies(PipelineFlowState& state, ConstDataObjectPath& dataPath, bool forceProxyReplacement) const
 {
-    ElementType::updateEditableProxies(state, dataPath);
+    ElementType::updateEditableProxies(state, dataPath, forceProxyReplacement);
 
     // Note: 'this' may no longer exist at this point, because the base method implementation may
     // have already replaced it with a mutable copy.
@@ -139,71 +125,58 @@ void ParticleType::updateEditableProxies(PipelineFlowState& state, ConstDataObje
 }
 
 /******************************************************************************
- * Loads a user-defined display shape from a geometry file and assigns it to this particle type.
- ******************************************************************************/
-bool ParticleType::loadShapeMesh(const QUrl& sourceUrl, MainThreadOperation operation, const FileImporterClass* importerClass, const QString& importerFormat)
+* Loads a mesh-based shape from a geometry file (but doesn't yet assign it to the ParticleType).
+******************************************************************************/
+Future<DataOORef<TriangleMesh>> ParticleType::loadShapeMesh(const QUrl sourceUrl, const FileImporterClass* importerClass, const QString& importerFormat) const
 {
-    operation.setProgressText(tr("Loading mesh geometry file %1").arg(sourceUrl.fileName()));
+    OVITO_ASSERT(this_task::get());
+    OVITO_ASSERT(this_task::isMainThread());
 
-    // Temporarily disable undo recording while loading the geometry data.
-    DataOORef<TriangleMesh> meshObj;
-    {
-        UndoSuspender noUndo;
-
-        OORef<FileSourceImporter> importer;
-        if(!importerClass) {
-
-            // Inspect input file to detect its format.
-            Future<OORef<FileImporter>> importerFuture = FileImporter::autodetectFileFormat(sourceUrl);
-            if(!importerFuture.waitForFinished())
-                return false;
-
-            importer = dynamic_object_cast<FileSourceImporter>(importerFuture.result());
-        }
-        else {
-            importer = dynamic_object_cast<FileSourceImporter>(importerClass->createInstance());
-            if(importer)
-                importer->setSelectedFileFormat(importerFormat);
-        }
-        if(!importer)
-            throw Exception(tr("Could not detect the format of the geometry file. The format might not be supported."));
-
-        // Create a temporary FileSource for loading the geometry data from the file.
-        OORef<FileSource> fileSource = OORef<FileSource>::create();
-        fileSource->setSource({sourceUrl}, importer, false);
-        SharedFuture<PipelineFlowState> stateFuture = fileSource->evaluate(PipelineEvaluationRequest(AnimationTime(0)));
-        if(!stateFuture.waitForFinished())
-            return false;
-
-        // Check if the FileSource has provided some useful data.
-        PipelineFlowState state = stateFuture.result();
-        if(state.status().type() == PipelineStatus::Error) {
-            operation.cancel();
-            return false;
-        }
-        if(!state)
-            throw Exception(tr("The loaded geometry file does not provide any valid mesh data."));
-        meshObj = DataOORef<TriangleMesh>::makeCopy(state.expectObject<TriangleMesh>());
-
-        // Throw away any visual elements attached to the mesh object.
-        meshObj->setVisElement(nullptr);
-
-        // Show sharp edges of the mesh.
-        meshObj->determineEdgeVisibility();
-
-        // Turn on undo recording again. The final shape assignment should be recorded on the undo stack.
+    // Create the right importer for the file. May need to inspect input file to detect its format.
+    Future<OORef<FileImporter>> importerFuture;
+    if(!importerClass) {
+        importerFuture = FileImporter::autodetectFileFormat(sourceUrl);
+    }
+    else {
+        OORef<FileSourceImporter> importer = dynamic_object_cast<FileSourceImporter>(importerClass->createInstance());
+        if(importer)
+            importer->setSelectedFileFormat(importerFormat);
+        importerFuture = std::move(importer);
     }
-    setShapeMesh(std::move(meshObj));
 
-    // Also switch the particle type's visualization shape to mesh-based.
-    setShape(ParticlesVis::Mesh);
+    // Wait for format detection to complete.
+    OORef<FileSourceImporter> importer = dynamic_object_cast<FileSourceImporter>(co_await FutureAwaiter(ObjectExecutor(this), std::move(importerFuture)));
+    if(!importer)
+        throw Exception(tr("Could not detect the format of the geometry file. The format might not be supported."));
 
-    // Determine whether the mesh is a closed manifold.
-    // If not, we should turn off back-face culling.
-    if(shapeMesh() && !shapeMesh()->isClosed())
-        setShapeBackfaceCullingEnabled(false);
+    OVITO_ASSERT(!CompoundOperation::isUndoRecording()); // Be sure that our actions are not recorded on the undo stack.
 
-    return !operation.isCanceled();
+    // Create a temporary FileSource for loading the geometry data from the file.
+    OORef<FileSource> fileSource = OORef<FileSource>::create();
+    fileSource->setSource({std::move(sourceUrl)}, std::move(importer), false);
+
+    // Evaluate the FileSource to load the data and obtain a data collection.
+    PipelineFlowState state = co_await FutureAwaiter(ObjectExecutor(this), fileSource->evaluate(PipelineEvaluationRequest(AnimationTime(0), true)).asFuture());
+
+    // Some error checking.
+    if(state.status().type() == PipelineStatus::Error)
+        throw Exception(state.status().text());
+    if(!state)
+        throw Exception(tr("The loaded geometry file does not contain any valid mesh."));
+
+    // Extract the TriangleMesh object from the data collection.
+    // Then discard the rest of the data collection and make the mesh mutable.
+    DataOORef<const TriangleMesh> meshObjConst = state.expectObject<TriangleMesh>();
+    state.reset();
+    DataOORef<TriangleMesh> meshObj = std::move(meshObjConst).makeMutable();
+
+    // Throw away any visual elements attached to the mesh object.
+    meshObj->setVisElement(nullptr);
+
+    // Show sharp edges of the mesh.
+    meshObj->determineEdgeVisibility();
+
+    co_return meshObj;
 }
 
 /******************************************************************************
@@ -230,7 +203,6 @@ void ParticleType::loadFromStreamComplete(ObjectLoadStream& stream)
 // The radii for ions (Na, K, Cl, Ca, Mg, and Cs) are based on the CHARMM27 Rmin/2 parameters for (SOD, POT, CLA, CAL, MG, CES).
 //
 // Colors and covalent radii of elements marked with '//' have been adopted from OpenBabel.
-//
 const std::array<ParticleType::PredefinedChemicalType, ParticleType::NUMBER_OF_PREDEFINED_PARTICLE_TYPES> ParticleType::_predefinedParticleTypes{{
     ParticleType::PredefinedChemicalType{ QStringLiteral("X"),  Color(255.0f/255.0f, 255.0f/255.0f, 255.0f/255.0f), 0.00f, 0.00f, 0.0 },
     ParticleType::PredefinedChemicalType{ QStringLiteral("H"),  Color(255.0f/255.0f, 255.0f/255.0f, 255.0f/255.0f), 0.46f, 1.20f, 1.00794 },
@@ -264,7 +236,8 @@ const std::array<ParticleType::PredefinedChemicalType, ParticleType::NUMBER_OF_P
     ParticleType::PredefinedChemicalType{ QStringLiteral("Co"), Color(240.0f/255.0f, 144.0f/255.0f, 160.0f/255.0f), 1.25f, 2.00f, 58.9332 },
     ParticleType::PredefinedChemicalType{ QStringLiteral("Ni"), Color( 80.0f/255.0f, 208.0f/255.0f,  80.0f/255.0f), 1.25f, 1.63f, 58.6934 },
     ParticleType::PredefinedChemicalType{ QStringLiteral("Cu"), Color(200.0f/255.0f, 128.0f/255.0f,  51.0f/255.0f), 1.28f, 1.40f, 63.546 },
-    ParticleType::PredefinedChemicalType{ QStringLiteral("Zn"), Color(125.0f/255.0f, 128.0f/255.0f, 176.0f/255.0f), 1.37f, 1.39f, 65.409 },
+    ParticleType::PredefinedChemicalType{ QStringLiteral("Zn"), Color(125.0f/255.0f, 128.0f/255.0f, 176.0f/255.0f), 1.37f, 1.39f, 65.38, 65.409 }, // Zn has two atomic weights: the old mass (pre 2001) 65.409 and the updated mass 65.38 (https://www.ciaaw.org/zinc.htm)
+
     ParticleType::PredefinedChemicalType{ QStringLiteral("Ga"), Color(194.0f/255.0f, 143.0f/255.0f, 143.0f/255.0f), 1.53f, 1.07f, 69.723 },
     ParticleType::PredefinedChemicalType{ QStringLiteral("Ge"), Color(102.0f/255.0f, 143.0f/255.0f, 143.0f/255.0f), 1.22f, 2.00f, 72.64 },
 
@@ -363,7 +336,7 @@ FloatType ParticleType::getDefaultParticleRadius(Particles::Type typeClass, cons
 #ifndef OVITO_DISABLE_QSETTINGS
         // Use the type's name, property type and container class to look up the
         // default radius saved by the user.
-        const QString& settingsKey = ElementType::getElementSettingsKey(ParticlePropertyReference(typeClass),
+        const QString& settingsKey = ElementType::getElementSettingsKey(OwnerPropertyRef(&Particles::OOClass(), typeClass),
             (radiusVariant == DisplayRadius) ? QStringLiteral("radius") : QStringLiteral("vdw_radius"), particleTypeName);
         QVariant v = QSettings().value(settingsKey);
         if(v.isValid() && v.canConvert<FloatType>())
@@ -408,7 +381,7 @@ void ParticleType::setDefaultParticleRadius(Particles::Type typeClass, const QSt
 
 #ifndef OVITO_DISABLE_QSETTINGS
     QSettings settings;
-    const QString& settingsKey = ElementType::getElementSettingsKey(ParticlePropertyReference(typeClass),
+    const QString& settingsKey = ElementType::getElementSettingsKey(OwnerPropertyRef(&Particles::OOClass(), typeClass),
         (radiusVariant == DisplayRadius) ? QStringLiteral("radius") : QStringLiteral("vdw_radius"), particleTypeName);
 
     if(std::abs(getDefaultParticleRadius(typeClass, particleTypeName, 0, false, radiusVariant) - radius) > 1e-6)
@@ -453,6 +426,9 @@ QString ParticleType::guessTypeNameFromMass(FloatType mass)
         if(std::abs(predefType.mass - mass) <= tolerance) {
             return predefType.name;
         }
+        if(predefType.alternativeMass != 0 && std::abs(predefType.alternativeMass - mass) <= tolerance) {
+            return predefType.name;
+        }
     }
 
     return {};
