@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -25,13 +25,14 @@
 #include <ovito/gui/desktop/properties/BooleanRadioButtonParameterUI.h>
 #include <ovito/gui/desktop/properties/AffineTransformationParameterUI.h>
 #include <ovito/gui/desktop/properties/ModifierDelegateFixedListParameterUI.h>
+#include <ovito/gui/desktop/widgets/general/EnterLineEdit.h>
 #include <ovito/stdmod/modifiers/AffineTransformationModifier.h>
 #include <ovito/stdobj/simcell/SimulationCell.h>
 #include "AffineTransformationModifierEditor.h"
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(AffineTransformationModifierEditor);
+IMPLEMENT_CREATABLE_OVITO_CLASS(AffineTransformationModifierEditor);
 SET_OVITO_OBJECT_EDITOR(AffineTransformationModifier, AffineTransformationModifierEditor);
 
 /******************************************************************************
@@ -42,20 +43,20 @@ void AffineTransformationModifierEditor::createUI(const RolloutInsertionParamete
     // Create the first rollout.
     QWidget* rollout = rollout = createRollout(tr("Transformation"), rolloutParams, "manual:particles.modifiers.affine_transformation");
 
-    QVBoxLayout* topLayout = new QVBoxLayout(rollout);
-    topLayout->setContentsMargins(8,8,8,8);
-    topLayout->setSpacing(4);
+    QGridLayout* layout = new QGridLayout(rollout);
+    layout->setContentsMargins(4,4,4,4);
+    layout->setSpacing(2);
+    layout->setColumnMinimumWidth(0, 30);
+    layout->setColumnStretch(1, 1);
+    layout->setColumnStretch(2, 1);
+    layout->setColumnStretch(3, 1);
 
-    BooleanRadioButtonParameterUI* relativeModeUI = new BooleanRadioButtonParameterUI(this, PROPERTY_FIELD(AffineTransformationModifier::relativeMode));
+    BooleanRadioButtonParameterUI* relativeModeUI = createParamUI<BooleanRadioButtonParameterUI>(PROPERTY_FIELD(AffineTransformationModifier::relativeMode));
 
+    int layoutRow = 0;
     relativeModeUI->buttonTrue()->setText(tr("Transformation matrix:"));
-    topLayout->addWidget(relativeModeUI->buttonTrue());
-
-    QGridLayout* layout = new QGridLayout();
-    layout->setContentsMargins(30,4,4,4);
-    layout->setHorizontalSpacing(0);
-    layout->setVerticalSpacing(2);
-    topLayout->addLayout(layout);
+    layout->addWidget(relativeModeUI->buttonTrue(), layoutRow++, 0, 1, 4);
+    layout->setRowMinimumHeight(layoutRow++, 4);
 
     QGridLayout* sublayout = new QGridLayout();
     sublayout->setContentsMargins(0,0,0,0);
@@ -70,85 +71,98 @@ void AffineTransformationModifierEditor::createUI(const RolloutInsertionParamete
     enterRotationAction->setEnabled(false);
     connect(relativeModeUI->buttonTrue(), &QRadioButton::toggled, enterRotationAction, &QAction::setEnabled);
     connect(enterRotationAction, &QAction::triggered, this, &AffineTransformationModifierEditor::onEnterRotation);
-    layout->addLayout(sublayout, 0, 0, 1, 8);
+    layout->addLayout(sublayout, layoutRow++, 1, 1, 3);
 
-    for(int col = 0; col < 3; col++) {
-        layout->setColumnStretch(col*3 + 0, 1);
-        if(col < 2) layout->setColumnMinimumWidth(col*3 + 2, 4);
-        for(int row=0; row<4; row++) {
-            QLineEdit* lineEdit = new QLineEdit(rollout);
+    // Override automatic increment step sizes for the cell parameters.
+    _relativeMatrixUnits.emplace(mainWindow().unitsManager().floatIdentityUnit());
+    _relativeTranslationUnits[0].emplace(mainWindow().unitsManager().floatIdentityUnit());
+    _relativeTranslationUnits[1].emplace(mainWindow().unitsManager().floatIdentityUnit());
+    _relativeTranslationUnits[2].emplace(mainWindow().unitsManager().floatIdentityUnit());
+
+    for(int row = 0; row < 4; row++) {
+        if(row == 3) {
+            QHBoxLayout* hboxsublayout = new QHBoxLayout();
+            hboxsublayout->setContentsMargins(0,4,0,0);
+            hboxsublayout->setSpacing(4);
+            hboxsublayout->addWidget(new QLabel(tr("Translation:")));
+            BooleanParameterUI* translationReducedCoordinatesUI = createParamUI<BooleanParameterUI>(PROPERTY_FIELD(AffineTransformationModifier::translationReducedCoordinates));
+            connect(relativeModeUI->buttonTrue(), &QRadioButton::toggled, translationReducedCoordinatesUI, &BooleanParameterUI::setEnabled);
+            translationReducedCoordinatesUI->checkBox()->setText(tr("In reduced cell coordinates"));
+            hboxsublayout->addWidget(translationReducedCoordinatesUI->checkBox(), 1, Qt::AlignRight | Qt::AlignVCenter);
+            connect(translationReducedCoordinatesUI, &BooleanParameterUI::valueEntered, this, &AffineTransformationModifierEditor::onReducedCoordinatesOptionChanged);
+            layout->addLayout(hboxsublayout, layoutRow++, 1, 1, 3);
+        }
+        for(int col = 0; col < 3; col++) {
+            QHBoxLayout* sublayout = new QHBoxLayout();
+            sublayout->setContentsMargins(0,0,0,0);
+            sublayout->setSpacing(0);
+            QLineEdit* lineEdit = new EnterLineEdit(rollout);
             SpinnerWidget* spinner = new SpinnerWidget(rollout);
             lineEdit->setEnabled(false);
             spinner->setEnabled(false);
             if(row < 3) {
-                elementSpinners[row][col] = spinner;
+                relativeCellSpinners[row][col] = spinner;
                 spinner->setProperty("column", col);
                 spinner->setProperty("row", row);
+                spinner->setUnit(&_relativeMatrixUnits.value());
             }
             else {
-                elementSpinners[col][row] = spinner;
+                relativeCellSpinners[col][row] = spinner;
                 spinner->setProperty("column", row);
                 spinner->setProperty("row", col);
+                spinner->setUnit(&_relativeTranslationUnits[col].value());
             }
             spinner->setTextBox(lineEdit);
 
-            int gridRow = (row == 3) ? 5 : (row+1);
-            layout->addWidget(lineEdit, gridRow, col*3 + 0);
-            layout->addWidget(spinner, gridRow, col*3 + 1);
+            sublayout->addWidget(lineEdit, 1);
+            sublayout->addWidget(spinner);
+            layout->addLayout(sublayout, layoutRow, col+1);
 
-            connect(spinner, &SpinnerWidget::spinnerValueChanged, this, &AffineTransformationModifierEditor::onSpinnerValueChanged);
-            connect(spinner, &SpinnerWidget::spinnerDragStart, this, &AffineTransformationModifierEditor::onSpinnerDragStart);
-            connect(spinner, &SpinnerWidget::spinnerDragStop, this, &AffineTransformationModifierEditor::onSpinnerDragStop);
-            connect(spinner, &SpinnerWidget::spinnerDragAbort, this, &AffineTransformationModifierEditor::onSpinnerDragAbort);
+            spinner->enableAutomaticUndo(mainWindow(), tr("Change parameter"));
+            connect(spinner, &SpinnerWidget::valueChanged, this, &AffineTransformationModifierEditor::onSpinnerValueChanged);
             connect(relativeModeUI->buttonTrue(), &QRadioButton::toggled, spinner, &SpinnerWidget::setEnabled);
             connect(relativeModeUI->buttonTrue(), &QRadioButton::toggled, lineEdit, &QLineEdit::setEnabled);
         }
+        layoutRow++;
     }
 
-    QHBoxLayout* hboxsublayout = new QHBoxLayout();
-    hboxsublayout->setContentsMargins(0,4,0,0);
-    hboxsublayout->setSpacing(4);
+    layout->setRowMinimumHeight(layoutRow++, 6);
+    relativeModeUI->buttonFalse()->setText(tr("Target cell:"));
+    layout->addWidget(relativeModeUI->buttonFalse(), layoutRow++, 0, 1, 4);
+    layout->setRowMinimumHeight(layoutRow++, 6);
 
-    hboxsublayout->addWidget(new QLabel(tr("Translation:")));
-    layout->addLayout(hboxsublayout, 4, 0, 1, 8);
-
-    BooleanParameterUI* translationReducedCoordinatesUI = new BooleanParameterUI(this, PROPERTY_FIELD(AffineTransformationModifier::translationReducedCoordinates));
-    connect(relativeModeUI->buttonTrue(), &QRadioButton::toggled, translationReducedCoordinatesUI, &BooleanParameterUI::setEnabled);
-    translationReducedCoordinatesUI->checkBox()->setText(tr("In reduced cell coordinates"));
-    hboxsublayout->addWidget(translationReducedCoordinatesUI->checkBox(), 1, Qt::AlignRight | Qt::AlignVCenter);
-    connect(translationReducedCoordinatesUI, &BooleanParameterUI::valueEntered, this, &AffineTransformationModifierEditor::onReducedCoordinatesOptionChanged);
-
-    relativeModeUI->buttonFalse()->setText(tr("Transform to target cell:"));
-    topLayout->addWidget(relativeModeUI->buttonFalse());
-
-    layout = new QGridLayout();
-    layout->setContentsMargins(30,4,4,4);
-    layout->setHorizontalSpacing(0);
-    layout->setVerticalSpacing(2);
-    layout->setColumnStretch(0, 1);
-    layout->setColumnStretch(2, 1);
-    layout->setColumnStretch(4, 1);
-    layout->setColumnMinimumWidth(1, 4);
-    layout->setColumnMinimumWidth(3, 4);
-    topLayout->addLayout(layout);
     AffineTransformationParameterUI* destinationCellUI;
-
+    layout->addWidget(new QLabel(tr("New cell vectors:")), layoutRow++, 1, 1, 3);
+    layout->addWidget(new QLabel(tr("<b>a</b>:")), layoutRow+0, 0, 1, 1, Qt::AlignRight);
+    layout->addWidget(new QLabel(tr("<b>b</b>:")), layoutRow+1, 0, 1, 1, Qt::AlignRight);
+    layout->addWidget(new QLabel(tr("<b>c</b>:")), layoutRow+2, 0, 1, 1, Qt::AlignRight);
     for(size_t v = 0; v < 3; v++) {
-        layout->addWidget(new QLabel(tr("Cell vector %1:").arg(v+1)), v*2, 0, 1, 8);
         for(size_t r = 0; r < 3; r++) {
-            destinationCellUI = new AffineTransformationParameterUI(this, PROPERTY_FIELD(AffineTransformationModifier::targetCell), r, v);
+            destinationCellUI = createParamUI<AffineTransformationParameterUI>(PROPERTY_FIELD(AffineTransformationModifier::targetCell), r, v);
             destinationCellUI->setEnabled(false);
-            layout->addLayout(destinationCellUI->createFieldLayout(), v*2+1, r*2);
+            layout->addLayout(destinationCellUI->createFieldLayout(), layoutRow, r+1);
             connect(relativeModeUI->buttonFalse(), &QRadioButton::toggled, destinationCellUI, &AffineTransformationParameterUI::setEnabled);
+            absoluteCellSpinners[r][v] = destinationCellUI->spinner();
+            _absoluteCellUnits[r][v].emplace(mainWindow().unitsManager().getUnit(destinationCellUI->parameterUnitType()));
+            destinationCellUI->spinner()->setUnit(&_absoluteCellUnits[r][v].value());
+            destinationCellUI->textBox()->setPlaceholderText(_absoluteCellUnits[r][v]->formatValue(0));
+            destinationCellUI->spinner()->setStandardValue(0);
         }
+        layoutRow++;
     }
 
-    layout->addWidget(new QLabel(tr("Cell origin:")), 6, 0, 1, 8);
+    layout->addWidget(new QLabel(tr("New cell origin:")), layoutRow++, 1, 1, 3);
+    layout->addWidget(new QLabel(tr("<b>o</b>:")), layoutRow, 0, 1, 1, Qt::AlignRight);
     for(size_t r = 0; r < 3; r++) {
-        destinationCellUI = new AffineTransformationParameterUI(this, PROPERTY_FIELD(AffineTransformationModifier::targetCell), r, 3);
+        destinationCellUI = createParamUI<AffineTransformationParameterUI>(PROPERTY_FIELD(AffineTransformationModifier::targetCell), r, 3);
         destinationCellUI->setEnabled(false);
-        layout->addLayout(destinationCellUI->createFieldLayout(), 7, r*2);
+        layout->addLayout(destinationCellUI->createFieldLayout(), layoutRow, r+1);
         connect(relativeModeUI->buttonFalse(), &QRadioButton::toggled, destinationCellUI, &AffineTransformationParameterUI::setEnabled);
+        absoluteCellSpinners[r][3] = destinationCellUI->spinner();
+        _absoluteOriginUnits[r].emplace(mainWindow().unitsManager().getUnit(destinationCellUI->parameterUnitType()));
+        destinationCellUI->spinner()->setUnit(&_absoluteOriginUnits[r].value());
+        destinationCellUI->textBox()->setPlaceholderText(_absoluteOriginUnits[r]->formatValue(0));
+        destinationCellUI->spinner()->setStandardValue(0);
     }
 
     // Update spinner values when a new object has been loaded into the editor.
@@ -158,15 +172,18 @@ void AffineTransformationModifierEditor::createUI(const RolloutInsertionParamete
     rollout = createRollout(tr("Operate on"), rolloutParams.after(rollout), "manual:particles.modifiers.slice");
 
     // Create the rollout contents.
-    topLayout = new QVBoxLayout(rollout);
+    QVBoxLayout* topLayout = new QVBoxLayout(rollout);
     topLayout->setContentsMargins(4,4,4,4);
     topLayout->setSpacing(12);
 
-    ModifierDelegateFixedListParameterUI* delegatesPUI = new ModifierDelegateFixedListParameterUI(this, rolloutParams.after(rollout));
+    ModifierDelegateFixedListParameterUI* delegatesPUI = createParamUI<ModifierDelegateFixedListParameterUI>(rolloutParams.after(rollout));
     topLayout->addWidget(delegatesPUI->listWidget(141));
 
-    BooleanParameterUI* selectionUI = new BooleanParameterUI(this, PROPERTY_FIELD(AffineTransformationModifier::selectionOnly));
+    BooleanParameterUI* selectionUI = createParamUI<BooleanParameterUI>(PROPERTY_FIELD(AffineTransformationModifier::selectionOnly));
     topLayout->addWidget(selectionUI->checkBox());
+
+    // Whenever the pipeline input of the modifier changes, update the increment step sizes of the cell parameters.
+    connect(this, &PropertiesEditor::pipelineInputChanged, this, &AffineTransformationModifierEditor::updateParameterUnitScales);
 }
 
 /******************************************************************************
@@ -181,35 +198,59 @@ void AffineTransformationModifierEditor::updateUI()
 
     for(int row = 0; row < 3; row++) {
         for(int column = 0; column < 4; column++) {
-            if(!elementSpinners[row][column]->isDragging())
-                elementSpinners[row][column]->setFloatValue(tm(row, column));
+            if(!relativeCellSpinners[row][column]->isDragging())
+                relativeCellSpinners[row][column]->setFloatValue(tm(row, column));
         }
     }
 }
 
+/******************************************************************************
+* Auto-adjusts the increment steps of the numeric parameter spinner widgets.
+******************************************************************************/
+void AffineTransformationModifierEditor::updateParameterUnitScales()
+{
+    OVITO_ASSERT(_relativeMatrixUnits.has_value());
+
+    _relativeMatrixUnits->setScaleReference(0.1);
+    _relativeTranslationUnits[0]->setScaleReference(0.3);
+    _relativeTranslationUnits[1]->setScaleReference(0.3);
+    _relativeTranslationUnits[2]->setScaleReference(0.3);
+    _absoluteOriginUnits[0]->setScaleReference(0);
+    _absoluteOriginUnits[1]->setScaleReference(0);
+    _absoluteOriginUnits[2]->setScaleReference(0);
+
+    if(DataOORef<const SimulationCell> cell = getPipelineInput().getObject<SimulationCell>()) {
+        bool usingReducedCoords = editObject() ? static_object_cast<AffineTransformationModifier>(editObject())->translationReducedCoordinates() : false;
+
+        // If a simulation cell is available, set the translation increments proportional to the cell's dimensions - unless translation is given in reduced coordinates.
+        if(!usingReducedCoords) {
+            _relativeTranslationUnits[0]->setScaleReference(cell->cellMatrix().column(0).length());
+            _relativeTranslationUnits[1]->setScaleReference(cell->cellMatrix().column(1).length());
+            _relativeTranslationUnits[2]->setScaleReference(cell->cellMatrix().column(2).length());
+        }
+
+        // Let origin translation increments depend on the cell vector lengths.
+        _absoluteOriginUnits[0]->setScaleReference(cell->cellMatrix().column(0).length());
+        _absoluteOriginUnits[1]->setScaleReference(cell->cellMatrix().column(1).length());
+        _absoluteOriginUnits[2]->setScaleReference(cell->cellMatrix().column(2).length());
+
+        // Also let shear increments (off-diagonal cell matrix elements) depend on the cell vector lengths.
+        _absoluteCellUnits[1][0]->setScaleReference(0.3 * cell->cellMatrix().column(0).length());
+        _absoluteCellUnits[2][0]->setScaleReference(0.3 * cell->cellMatrix().column(0).length());
+        _absoluteCellUnits[0][1]->setScaleReference(0.3 * cell->cellMatrix().column(1).length());
+        _absoluteCellUnits[2][1]->setScaleReference(0.3 * cell->cellMatrix().column(1).length());
+        _absoluteCellUnits[0][2]->setScaleReference(0.3 * cell->cellMatrix().column(2).length());
+        _absoluteCellUnits[1][2]->setScaleReference(0.3 * cell->cellMatrix().column(2).length());
+    }
+}
+
 /******************************************************************************
 * Is called when the spinner value has changed.
 ******************************************************************************/
 void AffineTransformationModifierEditor::onSpinnerValueChanged()
 {
-    if(!_undoTransaction.operation()) {
-        performTransaction(tr("Change parameter"), [&] {
-            updateParameterValue();
-        });
-    }
-    else {
-        _undoTransaction.revert();
-        performActions(_undoTransaction, [&] {
-            updateParameterValue();
-        });
-    }
-}
+    // Take the value entered by the user and store it in transformation controller.
 
-/******************************************************************************
-* Takes the value entered by the user and stores it in transformation controller.
-******************************************************************************/
-void AffineTransformationModifierEditor::updateParameterValue()
-{
     AffineTransformationModifier* mod = dynamic_object_cast<AffineTransformationModifier>(editObject());
     if(!mod) return;
 
@@ -226,41 +267,21 @@ void AffineTransformationModifierEditor::updateParameterValue()
     mod->setTransformationTM(tm);
 }
 
-/******************************************************************************
-* Is called when the user begins dragging the spinner interactively.
-******************************************************************************/
-void AffineTransformationModifierEditor::onSpinnerDragStart()
-{
-    _undoTransaction.begin(mainWindow(), tr("Change parameter"));
-}
-
-/******************************************************************************
-* Is called when the user stops dragging the spinner interactively.
-******************************************************************************/
-void AffineTransformationModifierEditor::onSpinnerDragStop()
-{
-    _undoTransaction.commit();
-}
-
-/******************************************************************************
-* Is called when the user aborts dragging the spinner interactively.
-******************************************************************************/
-void AffineTransformationModifierEditor::onSpinnerDragAbort()
-{
-    _undoTransaction.cancel();
-}
-
 /******************************************************************************
 * Is called when the user switches between Cartesian and reduced cell coordinates for the translation vector.
 ******************************************************************************/
 void AffineTransformationModifierEditor::onReducedCoordinatesOptionChanged()
 {
     AffineTransformationModifier* mod = static_object_cast<AffineTransformationModifier>(editObject());
-    if(!mod) return;
+    if(!mod)
+        return;
+
+    updateParameterUnitScales();
 
     const PipelineFlowState& input = getPipelineInput();
     const SimulationCell* cell = input.getObject<SimulationCell>();
-    if(!cell) return;
+    if(!cell)
+        return;
 
     // Automatically convert translation vector to/from reduced cell coordinates.
     AffineTransformation tm = mod->transformationTM();
@@ -289,15 +310,17 @@ void AffineTransformationModifierEditor::onEnterRotation()
     AffineTransformationModifier* mod = static_object_cast<AffineTransformationModifier>(editObject());
     if(!mod) return;
 
-    _undoTransaction.begin(mainWindow(), tr("Set transformation matrix"));
-    if(!handleExceptions([&] {
+    UndoableTransaction transaction;
+    transaction.begin(mainWindow(), tr("Set transformation matrix"));
+
+    handleExceptions([&] {
         QDialog dlg(container()->window());
         dlg.setWindowTitle(tr("Enter rotation axis and angle"));
         QVBoxLayout* mainLayout = new QVBoxLayout(&dlg);
 
         QGridLayout* layout = new QGridLayout();
         layout->setContentsMargins(0,0,0,0);
-        layout->addWidget(new QLabel(tr("Rotation axis:")), 0, 0, 1, 8);
+        layout->addWidget(new QLabel(tr("3D rotation axis (xyz):")), 0, 0, 1, 8);
         layout->setColumnStretch(0, 1);
         layout->setColumnStretch(3, 1);
         layout->setColumnStretch(6, 1);
@@ -305,9 +328,9 @@ void AffineTransformationModifierEditor::onEnterRotation()
         layout->setColumnMinimumWidth(5, 4);
         layout->setVerticalSpacing(2);
         layout->setHorizontalSpacing(0);
-        QLineEdit* axisEditX = new QLineEdit();
-        QLineEdit* axisEditY = new QLineEdit();
-        QLineEdit* axisEditZ = new QLineEdit();
+        QLineEdit* axisEditX = new EnterLineEdit();
+        QLineEdit* axisEditY = new EnterLineEdit();
+        QLineEdit* axisEditZ = new EnterLineEdit();
         SpinnerWidget* axisSpinnerX = new SpinnerWidget();
         SpinnerWidget* axisSpinnerY = new SpinnerWidget();
         SpinnerWidget* axisSpinnerZ = new SpinnerWidget();
@@ -323,17 +346,17 @@ void AffineTransformationModifierEditor::onEnterRotation()
         layout->addWidget(axisSpinnerY, 1, 4);
         layout->addWidget(axisEditZ, 1, 6);
         layout->addWidget(axisSpinnerZ, 1, 7);
-        layout->addWidget(new QLabel(tr("Angle:")), 2, 0, 1, 8);
-        QLineEdit* angleEdit = new QLineEdit();
+        layout->addWidget(new QLabel(tr("Angle (degrees):")), 2, 0, 1, 8);
+        QLineEdit* angleEdit = new EnterLineEdit();
         SpinnerWidget* angleSpinner = new SpinnerWidget();
         angleSpinner->setTextBox(angleEdit);
         angleSpinner->setUnit(mainWindow().unitsManager().angleUnit());
         layout->addWidget(angleEdit, 3, 0);
         layout->addWidget(angleSpinner, 3, 1);
-        layout->addWidget(new QLabel(tr("Center of rotation:")), 4, 0, 1, 8);
-        QLineEdit* centerEditX = new QLineEdit();
-        QLineEdit* centerEditY = new QLineEdit();
-        QLineEdit* centerEditZ = new QLineEdit();
+        layout->addWidget(new QLabel(tr("Center of rotation (xyz):")), 4, 0, 1, 8);
+        QLineEdit* centerEditX = new EnterLineEdit();
+        QLineEdit* centerEditY = new EnterLineEdit();
+        QLineEdit* centerEditZ = new EnterLineEdit();
         SpinnerWidget* centerSpinnerX = new SpinnerWidget();
         SpinnerWidget* centerSpinnerY = new SpinnerWidget();
         SpinnerWidget* centerSpinnerZ = new SpinnerWidget();
@@ -398,38 +421,38 @@ void AffineTransformationModifierEditor::onEnterRotation()
             centerSpinnerZ->setFloatValue(center.z());
         }
 
-        auto updateMatrix = [mod, angleSpinner, axisSpinnerX, axisSpinnerY, axisSpinnerZ, centerSpinnerX, centerSpinnerY, centerSpinnerZ, this]() {
+        auto updateMatrix = [&]() {
             Vector3 axis(axisSpinnerX->floatValue(), axisSpinnerY->floatValue(), axisSpinnerZ->floatValue());
             if(axis == Vector3::Zero()) axis = Vector3(0,0,1);
             Vector3 center(centerSpinnerX->floatValue(), centerSpinnerY->floatValue(), centerSpinnerZ->floatValue());
             Rotation rot(axis, angleSpinner->floatValue());
             AffineTransformation tm = AffineTransformation::translation(center) * AffineTransformation::rotation(rot) * AffineTransformation::translation(-center);
-            _undoTransaction.revert();
-            performActions(_undoTransaction, [&] {
+            transaction.revert();
+            performActions(transaction, [&] {
                 mod->setTranslationReducedCoordinates(false);
                 mod->setTransformationTM(tm);
             });
         };
 
-        connect(angleSpinner, &SpinnerWidget::spinnerValueChanged, updateMatrix);
-        connect(axisSpinnerX, &SpinnerWidget::spinnerValueChanged, updateMatrix);
-        connect(axisSpinnerY, &SpinnerWidget::spinnerValueChanged, updateMatrix);
-        connect(axisSpinnerZ, &SpinnerWidget::spinnerValueChanged, updateMatrix);
-        connect(centerSpinnerX, &SpinnerWidget::spinnerValueChanged, updateMatrix);
-        connect(centerSpinnerY, &SpinnerWidget::spinnerValueChanged, updateMatrix);
-        connect(centerSpinnerZ, &SpinnerWidget::spinnerValueChanged, updateMatrix);
+        connect(angleSpinner, &SpinnerWidget::valueChanged, updateMatrix);
+        connect(axisSpinnerX, &SpinnerWidget::valueChanged, updateMatrix);
+        connect(axisSpinnerY, &SpinnerWidget::valueChanged, updateMatrix);
+        connect(axisSpinnerZ, &SpinnerWidget::valueChanged, updateMatrix);
+        connect(centerSpinnerX, &SpinnerWidget::valueChanged, updateMatrix);
+        connect(centerSpinnerY, &SpinnerWidget::valueChanged, updateMatrix);
+        connect(centerSpinnerZ, &SpinnerWidget::valueChanged, updateMatrix);
 
         QDialogButtonBox* buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
         connect(buttonBox, &QDialogButtonBox::accepted, &dlg, &QDialog::accept);
         connect(buttonBox, &QDialogButtonBox::rejected, &dlg, &QDialog::reject);
         mainLayout->addWidget(buttonBox);
         if(dlg.exec() == QDialog::Accepted) {
-            _undoTransaction.commit();
+            transaction.commit();
         }
         else {
-            _undoTransaction.cancel();
+            transaction.cancel();
         }
-    })) _undoTransaction.cancel();
+    });
 }
 
 }   // End of namespace
