@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -24,23 +24,20 @@
 #include <ovito/gui/desktop/mainwin/MainWindow.h>
 #include <ovito/gui/desktop/dialogs/LoadImageFileDialog.h>
 #include <ovito/gui/desktop/properties/FloatParameterUI.h>
-#include <ovito/gui/desktop/properties/Vector3ParameterUI.h>
+#include <ovito/gui/desktop/properties/VectorParameterUI.h>
 #include <ovito/gui/desktop/properties/ColorParameterUI.h>
 #include <ovito/gui/desktop/properties/BooleanParameterUI.h>
-#include <ovito/gui/desktop/properties/CustomParameterUI.h>
 #include <ovito/gui/desktop/properties/ModifierDelegateParameterUI.h>
 #include <ovito/gui/desktop/dialogs/SaveImageFileDialog.h>
-#include <ovito/gui/desktop/utilities/concurrent/ProgressDialog.h>
 #include <ovito/gui/base/viewport/ViewportInputMode.h>
 #include <ovito/core/app/PluginManager.h>
-#include <ovito/core/viewport/ViewportSuspender.h>
 #include <ovito/core/oo/OvitoClass.h>
 #include <ovito/core/dataset/pipeline/ModificationNode.h>
 #include "ColorCodingModifierEditor.h"
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(ColorCodingModifierEditor);
+IMPLEMENT_CREATABLE_OVITO_CLASS(ColorCodingModifierEditor);
 SET_OVITO_OBJECT_EDITOR(ColorCodingModifier, ColorCodingModifierEditor);
 
 /******************************************************************************
@@ -56,11 +53,11 @@ void ColorCodingModifierEditor::createUI(const RolloutInsertionParameters& rollo
     layout1->setContentsMargins(4,4,4,4);
     layout1->setSpacing(2);
 
-    ModifierDelegateParameterUI* delegateUI = new ModifierDelegateParameterUI(this, ColorCodingModifierDelegate::OOClass());
+    ModifierDelegateParameterUI* delegateUI = createParamUI<ModifierDelegateParameterUI>(ColorCodingModifierDelegate::OOClass());
     layout1->addWidget(new QLabel(tr("Operate on:")));
     layout1->addWidget(delegateUI->comboBox());
 
-    _sourcePropertyUI = new PropertyReferenceParameterUI(this, PROPERTY_FIELD(ColorCodingModifier::sourceProperty));
+    _sourcePropertyUI = createParamUI<PropertyReferenceParameterUI>(PROPERTY_FIELD(ColorCodingModifier::sourceProperty));
     layout1->addWidget(new QLabel(tr("Input property:")));
     layout1->addWidget(_sourcePropertyUI->comboBox());
     connect(this, &PropertiesEditor::contentsChanged, this, [this](RefTarget* editObject) {
@@ -77,9 +74,8 @@ void ColorCodingModifierEditor::createUI(const RolloutInsertionParameters& rollo
     layout1->addWidget(_colorGradientList);
     _colorGradientList->setIconSize(QSize(48,16));
     connect(_colorGradientList, qOverload<int>(&QComboBox::activated), this, &ColorCodingModifierEditor::onColorGradientSelected);
-    QVector<OvitoClassPtr> sortedColormapClassList = PluginManager::instance().listClasses(ColorCodingGradient::OOClass());
-    std::sort(sortedColormapClassList.begin(), sortedColormapClassList.end(),
-        [](OvitoClassPtr a, OvitoClassPtr b) { return QString::localeAwareCompare(a->displayName(), b->displayName()) < 0; });
+    std::vector<OvitoClassPtr> sortedColormapClassList = PluginManager::instance().listClasses(ColorCodingGradient::OOClass());
+    boost::sort(sortedColormapClassList, [](OvitoClassPtr a, OvitoClassPtr b) { return QString::localeAwareCompare(a->displayName(), b->displayName()) < 0; });
     for(OvitoClassPtr clazz : sortedColormapClassList) {
         if(clazz == &ColorCodingImageGradient::OOClass() || clazz == &ColorCodingTableGradient::OOClass())
             continue;
@@ -92,6 +88,7 @@ void ColorCodingModifierEditor::createUI(const RolloutInsertionParameters& rollo
 
     // Update color legend if another modifier has been loaded into the editor.
     connect(this, &ColorCodingModifierEditor::contentsReplaced, this, &ColorCodingModifierEditor::updateColorGradient);
+    // Update editor UI if the modifier's settings change.
     connect(this, &ColorCodingModifierEditor::contentsChanged, this, &ColorCodingModifierEditor::onModifierChanged);
 
     // Update the start/end parameters display whenever the modifier has been evaluated.
@@ -105,7 +102,7 @@ void ColorCodingModifierEditor::createUI(const RolloutInsertionParameters& rollo
     layout1->addLayout(layout2);
 
     // End value parameter.
-    _endValueUI = new FloatParameterUI(this, PROPERTY_FIELD(ColorCodingModifier::endValueController));
+    _endValueUI = createParamUI<FloatParameterUI>(PROPERTY_FIELD(ColorCodingModifier::endValue));
     layout2->addWidget(_endValueUI->label(), 0, 0);
     layout2->addLayout(_endValueUI->createFieldLayout(), 0, 1);
 
@@ -135,7 +132,7 @@ void ColorCodingModifierEditor::createUI(const RolloutInsertionParameters& rollo
     layout2->addWidget(_colorLegendLabel, 1, 1);
 
     // Start value parameter.
-    _startValueUI = new FloatParameterUI(this, PROPERTY_FIELD(ColorCodingModifier::startValueController));
+    _startValueUI = createParamUI<FloatParameterUI>(PROPERTY_FIELD(ColorCodingModifier::startValue));
     layout2->addWidget(_startValueUI->label(), 2, 0);
     layout2->addLayout(_startValueUI->createFieldLayout(), 2, 1);
 
@@ -149,8 +146,11 @@ void ColorCodingModifierEditor::createUI(const RolloutInsertionParameters& rollo
     layout2->addWidget(exportBtn, 1, 0, Qt::AlignCenter);
 
     // Auto-adjust range.
-    BooleanParameterUI* autoAdjustRangePUI = new BooleanParameterUI(this, PROPERTY_FIELD(ColorCodingModifier::autoAdjustRange));
+    BooleanParameterUI* autoAdjustRangePUI = createParamUI<BooleanParameterUI>(PROPERTY_FIELD(ColorCodingModifier::autoAdjustRange));
     layout2->addWidget(autoAdjustRangePUI->checkBox(), 3, 1);
+    // Symmetric range
+    BooleanParameterUI* symmetricRangePUI = createParamUI<BooleanParameterUI>(PROPERTY_FIELD(ColorCodingModifier::symmetricRange));
+    layout2->addWidget(symmetricRangePUI->checkBox(), 4, 1);
 
     layout1->addSpacing(8);
     _adjustRangeBtn = new QPushButton(tr("Adjust range"), rollout);
@@ -168,11 +168,11 @@ void ColorCodingModifierEditor::createUI(const RolloutInsertionParameters& rollo
     layout1->addSpacing(8);
 
     // Only selected particles/bonds.
-    BooleanParameterUI* onlySelectedPUI = new BooleanParameterUI(this, PROPERTY_FIELD(ColorCodingModifier::colorOnlySelected));
+    BooleanParameterUI* onlySelectedPUI = createParamUI<BooleanParameterUI>(PROPERTY_FIELD(ColorCodingModifier::colorOnlySelected));
     layout1->addWidget(onlySelectedPUI->checkBox());
 
     // Keep selection
-    BooleanParameterUI* keepSelectionPUI = new BooleanParameterUI(this, PROPERTY_FIELD(ColorCodingModifier::keepSelection));
+    BooleanParameterUI* keepSelectionPUI = createParamUI<BooleanParameterUI>(PROPERTY_FIELD(ColorCodingModifier::keepSelection));
     layout1->addWidget(keepSelectionPUI->checkBox());
     connect(onlySelectedPUI->checkBox(), &QCheckBox::toggled, keepSelectionPUI, &BooleanParameterUI::setEnabled);
     keepSelectionPUI->setEnabled(false);
@@ -236,17 +236,17 @@ bool ColorCodingModifierEditor::referenceEvent(RefTarget* source, const Referenc
         }
     }
     else if(source == editObject() && event.type() == ReferenceEvent::TargetChanged) {
-        if(static_cast<const ReferenceFieldEvent&>(event).field() == PROPERTY_FIELD(ColorCodingModifier::autoAdjustRange)) {
+        if((static_cast<const ReferenceFieldEvent&>(event).field() == PROPERTY_FIELD(ColorCodingModifier::autoAdjustRange))) {
             ColorCodingModifier* mod = static_object_cast<ColorCodingModifier>(editObject());
-            if(mod->autoAdjustRange() == false && isUndoRecording()) {
+            if(!mod->autoAdjustRange() && isUndoRecording()) {
                 // When the user turns off the auto-adjust option, adopt the current automatic range
                 // as the new user-defined range.
-                FloatType newMin = _lastAutoRangeMinValue;
-                FloatType newMax = _lastAutoRangeMaxValue;
-                if(std::isfinite(newMin))
-                    mod->setStartValue(newMin);
-                if(std::isfinite(newMax))
-                    mod->setEndValue(newMax);
+                auto newStartValue = _lastAutoRangeMinValue;
+                auto newEndValue = _lastAutoRangeMaxValue; // Note: Copying value, because setStartValue() will overwrite _lastAutoRangeMaxValue.
+                if(std::isfinite(newStartValue))
+                    mod->setStartValue(newStartValue);
+                if(std::isfinite(newEndValue))
+                    mod->setEndValue(newEndValue);
             }
         }
     }
@@ -254,15 +254,16 @@ bool ColorCodingModifierEditor::referenceEvent(RefTarget* source, const Referenc
 }
 
 /******************************************************************************
-* This method is called whenever the parameters of the ColoCodingModifier change.
+* This method is called whenever the parameters of the ColorCodingModifier change.
 ******************************************************************************/
 void ColorCodingModifierEditor::onModifierChanged()
 {
     ColorCodingModifier* mod = static_object_cast<ColorCodingModifier>(editObject());
 
     bool enableCustomRangeCtrls = (mod && !mod->autoAdjustRange());
+    bool symmetricRange = (mod && mod->symmetricRange());
 
-    _startValueUI->setEnabled(enableCustomRangeCtrls);
+    _startValueUI->setEnabled(enableCustomRangeCtrls && !symmetricRange);
     _endValueUI->setEnabled(enableCustomRangeCtrls);
     _adjustRangeBtn->setEnabled(enableCustomRangeCtrls);
     _adjustRangeGlobalBtn->setEnabled(enableCustomRangeCtrls);
@@ -284,14 +285,16 @@ void ColorCodingModifierEditor::onModifierChanged()
 void ColorCodingModifierEditor::autoRangeChanged()
 {
     ColorCodingModifier* mod = static_object_cast<ColorCodingModifier>(editObject());
-    if(!mod || !mod->autoAdjustRange()) return;
+    if(!mod || !mod->autoAdjustRange())
+        return;
+
     ModificationNode* modNode = modificationNode();
     if(!modNode)
         return;
 
     handleExceptions([&] {
         // Request the modifier's pipeline output.
-        const PipelineFlowState& state = modNode->evaluateSynchronous(currentAnimationTime());
+        const PipelineFlowState& state = getPipelineOutput();
 
         QVariant minValue = state.getAttributeValue(modNode, QStringLiteral("ColorCoding.RangeMin"));
         QVariant maxValue = state.getAttributeValue(modNode, QStringLiteral("ColorCoding.RangeMax"));
@@ -383,6 +386,7 @@ void ColorCodingModifierEditor::onAdjustRange()
     OVITO_CHECK_OBJECT_POINTER(mod);
 
     performTransaction(tr("Adjust range"), [&]() {
+        ProgressDialog::showForCurrentTask(mainWindow(), container(), tr("Determining property value range"));
         mod->adjustRange(currentAnimationTime());
     });
 }
@@ -396,10 +400,9 @@ void ColorCodingModifierEditor::onAdjustRangeGlobal()
     OVITO_CHECK_OBJECT_POINTER(mod);
 
     if(AnimationSettings* anim = mainWindow().datasetContainer().activeAnimationSettings()) {
-        performTransaction(tr("Adjust range"), [this, mod, firstFrame=anim->firstFrame(), lastFrame=anim->lastFrame()](MainThreadOperation& operation) {
-            ViewportSuspender noVPUpdates;
-            ProgressDialog progressDialog(container(), tr("Determining property value range"));
-            mod->adjustRangeGlobal(operation, firstFrame, lastFrame);
+        performTransaction(tr("Adjust range"), [this, mod, firstFrame=anim->firstFrame(), lastFrame=anim->lastFrame()]() {
+            ProgressDialog::showForCurrentTask(mainWindow(), container(), tr("Determining property value range"));
+            mod->adjustRangeGlobal(firstFrame, lastFrame);
         });
     }
 }
