@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -22,14 +22,14 @@
 
 #include <ovito/stdmod/gui/StdModGui.h>
 #include <ovito/stdmod/modifiers/ManualSelectionModifier.h>
-#include <ovito/stdobj/gui/widgets/PropertyContainerParameterUI.h>
 #include <ovito/core/viewport/ViewportConfiguration.h>
 #include <ovito/core/dataset/pipeline/ModificationNode.h>
 #include <ovito/core/dataset/animation/AnimationSettings.h>
-#include <ovito/core/viewport/ViewportWindowInterface.h>
+#include <ovito/core/viewport/ViewportWindow.h>
 #include <ovito/gui/desktop/mainwin/MainWindow.h>
 #include <ovito/gui/desktop/widgets/general/ViewportModeButton.h>
 #include <ovito/gui/desktop/properties/ObjectStatusDisplay.h>
+#include <ovito/gui/desktop/properties/DataObjectReferenceParameterUI.h>
 #include <ovito/gui/base/actions/ViewportModeAction.h>
 #include <ovito/gui/base/viewport/ViewportInputManager.h>
 #include <ovito/gui/base/viewport/ViewportInputMode.h>
@@ -37,7 +37,7 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(ManualSelectionModifierEditor);
+IMPLEMENT_CREATABLE_OVITO_CLASS(ManualSelectionModifierEditor);
 SET_OVITO_OBJECT_EDITOR(ManualSelectionModifier, ManualSelectionModifierEditor);
 
 /**
@@ -49,21 +49,23 @@ class PickElementMode : public ViewportInputMode
 public:
 
     /// Constructor.
-    PickElementMode(ManualSelectionModifierEditor* editor) : ViewportInputMode(editor), _editor(editor) {}
+    void initializeObject(ManualSelectionModifierEditor* editor) {
+        ViewportInputMode::initializeObject();
+        _editor = editor;
+    }
 
     /// Handles the mouse up events for a Viewport.
-    virtual void mouseReleaseEvent(ViewportWindowInterface* vpwin, QMouseEvent* event) override {
+    virtual void mouseReleaseEvent(ViewportWindow* vpwin, QMouseEvent* event) override {
         if(event->button() == Qt::LeftButton) {
             ManualSelectionModifier* mod = static_object_cast<ManualSelectionModifier>(_editor->editObject());
             if(mod && mod->subject()) {
                 // Find out what's under the mouse cursor.
-                ViewportPickResult pickResult = vpwin->pick(event->pos());
-                if(pickResult.isValid()) {
+                if(std::optional<ViewportWindow::PickResult> pickResult = vpwin->pick(event->pos())) {
                     // Look up the index of the element that was picked.
-                    std::pair<size_t, ConstDataObjectPath> indexAndContainer = mod->subject().dataClass()->elementFromPickResult(pickResult);
+                    std::pair<size_t, ConstDataObjectPath> indexAndContainer = mod->subject().dataClass()->elementFromPickResult(*pickResult);
                     if(indexAndContainer.first != std::numeric_limits<size_t>::max()) {
                         // Let the editor class handle it from here.
-                        _editor->onElementPicked(pickResult, indexAndContainer.first, indexAndContainer.second);
+                        _editor->onElementPicked(*pickResult, indexAndContainer.first, indexAndContainer.second);
                     }
                     else {
                         inputManager()->userInterface().showStatusBarMessage(tr("You did not click on an element of type '%1'.").arg(mod->subject().dataClass()->elementDescriptionName()), 1000);
@@ -75,7 +77,7 @@ public:
     }
 
     /// Handles the mouse events for a Viewport.
-    virtual void mouseMoveEvent(ViewportWindowInterface* vpwin, QMouseEvent* event) override {
+    virtual void mouseMoveEvent(ViewportWindow* vpwin, QMouseEvent* event) override {
         ViewportInputMode::mouseMoveEvent(vpwin, event);
 
         // Check if a selectable element is beneath the mouse cursor position.
@@ -83,10 +85,9 @@ public:
         ManualSelectionModifier* mod = static_object_cast<ManualSelectionModifier>(_editor->editObject());
         if(mod && mod->subject()) {
             // Find out what's under the mouse cursor.
-            ViewportPickResult pickResult = vpwin->pick(event->pos());
-            if(pickResult.isValid()) {
+            if(std::optional<ViewportWindow::PickResult> pickResult = vpwin->pick(event->pos())) {
                 // Look up the index of the element.
-                std::pair<size_t, ConstDataObjectPath> indexAndContainer = mod->subject().dataClass()->elementFromPickResult(pickResult);
+                std::pair<size_t, ConstDataObjectPath> indexAndContainer = mod->subject().dataClass()->elementFromPickResult(*pickResult);
                 if(indexAndContainer.first != std::numeric_limits<size_t>::max()) {
                     setCursor(SelectionMode::selectionCursor());
                     return;
@@ -109,7 +110,10 @@ class FenceSelectionMode : public ViewportInputMode, public ViewportGizmo
 public:
 
     /// Constructor.
-    FenceSelectionMode(ManualSelectionModifierEditor* editor) : ViewportInputMode(editor), _editor(editor) {}
+    void initializeObject(ManualSelectionModifierEditor* editor) {
+        ViewportInputMode::initializeObject();
+        _editor = editor;
+    }
 
     /// Destructor.
     virtual ~FenceSelectionMode() {
@@ -118,7 +122,7 @@ public:
     }
 
     /// Handles the mouse down events for a Viewport.
-    virtual void mousePressEvent(ViewportWindowInterface* vpwin, QMouseEvent* event) override {
+    virtual void mousePressEvent(ViewportWindow* vpwin, QMouseEvent* event) override {
         _fence.clear();
         if(event->button() == Qt::LeftButton) {
             _fence.push_back(Point2(getMousePosition(event).x(), getMousePosition(event).y())
@@ -130,7 +134,7 @@ public:
     }
 
     /// Handles the mouse move events for a Viewport.
-    virtual void mouseMoveEvent(ViewportWindowInterface* vpwin, QMouseEvent* event) override {
+    virtual void mouseMoveEvent(ViewportWindow* vpwin, QMouseEvent* event) override {
         if(!_fence.isEmpty()) {
             _fence.push_back(Point2(getMousePosition(event).x(), getMousePosition(event).y())
                     * (FloatType)vpwin->devicePixelRatio());
@@ -140,7 +144,7 @@ public:
     }
 
     /// Handles the mouse up events for a Viewport.
-    virtual void mouseReleaseEvent(ViewportWindowInterface* vpwin, QMouseEvent* event) override {
+    virtual void mouseReleaseEvent(ViewportWindow* vpwin, QMouseEvent* event) override {
         if(!_fence.isEmpty()) {
             if(_fence.size() >= 3) {
                 ElementSelectionSet::SelectionMode mode = ElementSelectionSet::SelectionReplace;
@@ -148,7 +152,7 @@ public:
                     mode = ElementSelectionSet::SelectionAdd;
                 else if(event->modifiers().testFlag(Qt::AltModifier))
                     mode = ElementSelectionSet::SelectionSubtract;
-                _editor->onFence(_fence, vpwin->viewport(), mode);
+                _editor->onFence(_fence, vpwin, mode);
             }
             _fence.clear();
             _activeViewport = nullptr;
@@ -158,9 +162,9 @@ public:
     }
 
     /// Lets the input mode render its 2d overlay content in a viewport.
-    virtual void renderOverlay2D(Viewport* vp, SceneRenderer* renderer) override {
+    virtual void renderOverlay(Viewport* vp, ViewportWindow* vpWin, FrameGraph& frameGraph, DataSet* dataset) override {
         if(isActive() && vp == _activeViewport && _fence.size() >= 2) {
-            renderer->render2DPolyline(_fence.constData(), _fence.size(), ViewportSettings::getSettings().viewportColor(ViewportSettings::COLOR_SELECTION), true);
+            frameGraph.addCommandGroup(FrameGraph::OverLayer).render2DPolyline(_fence.constData(), _fence.size(), ViewportSettings::getSettings().viewportColor(ViewportSettings::COLOR_SELECTION), true, vpWin->viewportWindowDeviceSize());
         }
     }
 
@@ -194,7 +198,7 @@ protected:
 
 private:
 
-    ManualSelectionModifierEditor* _editor;
+    ManualSelectionModifierEditor* _editor = nullptr;
     QVector<Point2> _fence;
     Viewport* _activeViewport = nullptr;
 };
@@ -217,11 +221,11 @@ void ManualSelectionModifierEditor::createUI(const RolloutInsertionParameters& r
     sublayout->setSpacing(6);
     layout->addWidget(operateOnGroup);
 
-    PropertyContainerParameterUI* pclassUI = new PropertyContainerParameterUI(this, PROPERTY_FIELD(GenericPropertyModifier::subject));
+    DataObjectReferenceParameterUI* pclassUI = createParamUI<DataObjectReferenceParameterUI>(PROPERTY_FIELD(GenericPropertyModifier::subject), PropertyContainer::OOClass());
     sublayout->addWidget(pclassUI->comboBox());
 
     // List only property containers that support element selection.
-    pclassUI->setContainerFilter([](const PropertyContainer* container) {
+    pclassUI->setObjectFilter<PropertyContainer>([](const PropertyContainer* container) {
         return container->getOOMetaClass().isValidStandardPropertyId(Property::GenericSelectionProperty)
             && container->getOOMetaClass().supportsViewportPicking();
     });
@@ -232,14 +236,14 @@ void ManualSelectionModifierEditor::createUI(const RolloutInsertionParameters& r
     sublayout->setSpacing(6);
     layout->addWidget(mouseSelectionGroup);
 
-    PickElementMode* pickElementMode = new PickElementMode(this);
+    OORef<PickElementMode> pickElementMode = OORef<PickElementMode>::create(this);
     connect(this, &QObject::destroyed, pickElementMode, &ViewportInputMode::removeMode);
-    ViewportModeAction* pickModeAction = new ViewportModeAction(mainWindow(), tr("Pick"), this, pickElementMode);
+    ViewportModeAction* pickModeAction = new ViewportModeAction(mainWindow(), tr("Pick"), this, std::move(pickElementMode));
     sublayout->addWidget(new ViewportModeButton(pickModeAction));
 
-    FenceSelectionMode* fenceMode = new FenceSelectionMode(this);
+    OORef<FenceSelectionMode> fenceMode = OORef<FenceSelectionMode>::create(this);
     connect(this, &QObject::destroyed, fenceMode, &ViewportInputMode::removeMode);
-    ViewportModeAction* fenceModeAction = new ViewportModeAction(mainWindow(), tr("Fence selection"), this, fenceMode);
+    ViewportModeAction* fenceModeAction = new ViewportModeAction(mainWindow(), tr("Fence selection"), this, std::move(fenceMode));
     sublayout->addWidget(new ViewportModeButton(fenceModeAction));
 
     // Deactivate input modes when editor is reset.
@@ -270,7 +274,7 @@ void ManualSelectionModifierEditor::createUI(const RolloutInsertionParameters& r
 
     // Status label.
     layout->addSpacing(12);
-    layout->addWidget((new ObjectStatusDisplay(this))->statusWidget());
+    layout->addWidget(createParamUI<ObjectStatusDisplay>()->statusWidget());
 }
 
 /******************************************************************************
@@ -282,9 +286,9 @@ void ManualSelectionModifierEditor::resetSelection()
     if(!mod) return;
 
     performTransaction(tr("Reset selection"), [this,mod]() {
-        PipelineEvaluationRequest request(currentAnimationTime());
+        PipelineEvaluationRequest request(currentAnimationTime(), false, true);
         for(ModificationNode* node : modificationNodes()) {
-            mod->resetSelection(node, node->evaluateInputSynchronous(request));
+            mod->resetSelection(node, node->evaluateInput(request).blockForResult());
         }
     });
 }
@@ -298,9 +302,9 @@ void ManualSelectionModifierEditor::selectAll()
     if(!mod) return;
 
     performTransaction(tr("Select all"), [this,mod]() {
-        PipelineEvaluationRequest request(currentAnimationTime());
+        PipelineEvaluationRequest request(currentAnimationTime(), false, true);
         for(ModificationNode* node : modificationNodes()) {
-            mod->selectAll(node, node->evaluateInputSynchronous(request));
+            mod->selectAll(node, node->evaluateInput(request).blockForResult());
         }
     });
 }
@@ -314,9 +318,9 @@ void ManualSelectionModifierEditor::clearSelection()
     if(!mod) return;
 
     performTransaction(tr("Clear selection"), [this,mod]() {
-        PipelineEvaluationRequest request(currentAnimationTime());
+        PipelineEvaluationRequest request(currentAnimationTime(), false, true);
         for(ModificationNode* node : modificationNodes()) {
-            mod->clearSelection(node, node->evaluateInputSynchronous(request));
+            mod->clearSelection(node, node->evaluateInput(request).blockForResult());
         }
     });
 }
@@ -330,9 +334,9 @@ void ManualSelectionModifierEditor::invertSelection()
     if(!mod) return;
 
     performTransaction(tr("Invert selection"), [this,mod]() {
-        PipelineEvaluationRequest request(currentAnimationTime());
+        PipelineEvaluationRequest request(currentAnimationTime(), false, true);
         for(ModificationNode* node : modificationNodes()) {
-            mod->invertSelection(node, node->evaluateInputSynchronous(request));
+            mod->invertSelection(node, node->evaluateInput(request).blockForResult());
         }
     });
 }
@@ -340,21 +344,21 @@ void ManualSelectionModifierEditor::invertSelection()
 /******************************************************************************
 * This is called when the user has selected an element.
 ******************************************************************************/
-void ManualSelectionModifierEditor::onElementPicked(const ViewportPickResult& pickResult, size_t elementIndex, const ConstDataObjectPath& pickedObjectPath)
+void ManualSelectionModifierEditor::onElementPicked(const ViewportWindow::PickResult& pickResult, size_t elementIndex, const ConstDataObjectPath& pickedObjectPath)
 {
     ManualSelectionModifier* mod = static_object_cast<ManualSelectionModifier>(editObject());
     if(!mod || !mod->subject()) return;
 
     performTransaction(tr("Toggle selection"), [this, mod, elementIndex, &pickedObjectPath, &pickResult]() {
-        PipelineEvaluationRequest request(currentAnimationTime());
+        PipelineEvaluationRequest request(currentAnimationTime(), false, true);
         for(ModificationNode* node : modificationNodes()) {
 
             // Make sure we are in the right data pipeline.
-            if(!node->pipelines(true).contains(pickResult.pipeline()))
+            if(!node->pipelines(true).contains(pickResult.sceneNode()->pipeline()))
                 continue;
 
             // Get the modifier's input data.
-            const PipelineFlowState& modInput = node->evaluateInputSynchronous(request);
+            const PipelineFlowState& modInput = node->evaluateInput(request).blockForResult();
             const ConstDataObjectPath& inputObjectPath = modInput.expectObject(mod->subject());
 
             // Look up the right element in the modifier's input.
@@ -376,41 +380,42 @@ void ManualSelectionModifierEditor::onElementPicked(const ViewportPickResult& pi
 /******************************************************************************
 * This is called when the user has drawn a fence around particles.
 ******************************************************************************/
-void ManualSelectionModifierEditor::onFence(const QVector<Point2>& fence, Viewport* viewport, ElementSelectionSet::SelectionMode mode)
+void ManualSelectionModifierEditor::onFence(const QVector<Point2>& fence, ViewportWindow* vpwin, ElementSelectionSet::SelectionMode mode)
 {
     ManualSelectionModifier* mod = static_object_cast<ManualSelectionModifier>(editObject());
     if(!mod || !mod->subject()) return;
 
-    performTransaction(tr("Select"), [this, mod, &fence, viewport, mode]() {
-        PipelineEvaluationRequest request(currentAnimationTime());
+    performTransaction(tr("Select"), [this, mod, &fence, vpwin, mode]() {
+        PipelineEvaluationRequest request(currentAnimationTime(), false, true);
         for(ModificationNode* node : modificationNodes()) {
 
             // Get the modifier's input data.
-            const PipelineFlowState& modInput = node->evaluateInputSynchronous(request);
+            const PipelineFlowState& modInput = node->evaluateInput(request).blockForResult();
             const ConstDataObjectPath& inputObjectPath = modInput.expectObject(mod->subject());
 
             // Iterate over the pipelines.
             // We'll need their object-to-world transformation.
             for(Pipeline* pipeline : node->pipelines(true)) {
+                for(SceneNode* sceneNode : pipeline->sceneNodes()) {
+                    // Set up projection matrix transforming elements from object space to screen space.
+                    const AffineTransformation& nodeTM = sceneNode->getWorldTransform(request.time());
+                    Matrix4 ndcToScreen = Matrix4::Identity();
+                    ndcToScreen(0,0) = 0.5 * vpwin->viewportWindowDeviceSize().width();
+                    ndcToScreen(1,1) = 0.5 * vpwin->viewportWindowDeviceSize().height();
+                    ndcToScreen(0,3) = ndcToScreen(0,0);
+                    ndcToScreen(1,3) = ndcToScreen(1,1);
+                    ndcToScreen(1,1) = -ndcToScreen(1,1);   // Vertical flip.
+                    Matrix4 projectionTM = ndcToScreen * vpwin->projectionParams().projectionMatrix * (vpwin->projectionParams().viewMatrix * nodeTM);
 
-                // Set up projection matrix transforming elements from object space to screen space.
-                TimeInterval interval;
-                const AffineTransformation& nodeTM = pipeline->getWorldTransform(request.time(), interval);
-                Matrix4 ndcToScreen = Matrix4::Identity();
-                ndcToScreen(0,0) = 0.5 * viewport->windowSize().width();
-                ndcToScreen(1,1) = 0.5 * viewport->windowSize().height();
-                ndcToScreen(0,3) = ndcToScreen(0,0);
-                ndcToScreen(1,3) = ndcToScreen(1,1);
-                ndcToScreen(1,1) = -ndcToScreen(1,1);   // Vertical flip.
-                Matrix4 projectionTM = ndcToScreen * viewport->projectionParams().projectionMatrix * (viewport->projectionParams().viewMatrix * nodeTM);
-
-                // Determine which elements are within the closed fence polygon.
-                boost::dynamic_bitset<> selection = mod->subject().dataClass()->viewportFenceSelection(fence, inputObjectPath, pipeline, projectionTM);
-                if(selection.size() != 0) {
-                    mod->setSelection(node, modInput, selection, mode);
-                }
-                else {
-                    throw Exception(tr("Sorry, making a fence-based selection is not supported for %1.").arg(mod->subject().dataClass()->elementDescriptionName()));
+                    // Determine which elements are within the closed fence polygon.
+                    ConstPropertyPtr selection = mod->subject().dataClass()->viewportFenceSelection(fence, inputObjectPath, pipeline, projectionTM);
+                    if(selection) {
+                        mod->setSelection(node, modInput, std::move(selection), mode);
+                    }
+                    else {
+                        throw Exception(tr("Sorry, making a fence-based selection is not supported for %1.").arg(mod->subject().dataClass()->elementDescriptionName()));
+                    }
+                    break;
                 }
                 break;
             }
