@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -28,7 +28,7 @@
 #include <ovito/core/dataset/pipeline/ModificationNode.h>
 #include <ovito/core/viewport/Viewport.h>
 #include <ovito/core/viewport/ViewportConfiguration.h>
-#include <ovito/core/viewport/ViewportWindowInterface.h>
+#include <ovito/core/viewport/ViewportWindow.h>
 #include <ovito/core/rendering/MarkerPrimitive.h>
 #include <ovito/core/rendering/LinePrimitive.h>
 #include <ovito/core/rendering/MeshPrimitive.h>
@@ -37,7 +37,7 @@
 #include <ovito/gui/desktop/mainwin/MainWindow.h>
 #include <ovito/gui/desktop/widgets/general/ViewportModeButton.h>
 #include <ovito/gui/desktop/properties/FloatParameterUI.h>
-#include <ovito/gui/desktop/properties/Vector3ParameterUI.h>
+#include <ovito/gui/desktop/properties/VectorParameterUI.h>
 #include <ovito/gui/desktop/properties/BooleanParameterUI.h>
 #include <ovito/gui/desktop/properties/ModifierDelegateFixedListParameterUI.h>
 #include <ovito/gui/desktop/properties/BooleanRadioButtonParameterUI.h>
@@ -47,7 +47,7 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(SliceModifierEditor);
+IMPLEMENT_CREATABLE_OVITO_CLASS(SliceModifierEditor);
 SET_OVITO_OBJECT_EDITOR(SliceModifier, SliceModifierEditor);
 
 /******************************************************************************
@@ -66,7 +66,7 @@ void SliceModifierEditor::createUI(const RolloutInsertionParameters& rolloutPara
     QHBoxLayout* sublayout = new QHBoxLayout();
     sublayout->setContentsMargins(0,0,0,0);
 
-    _reducedCoordinatesPUI = new BooleanRadioButtonParameterUI(this, PROPERTY_FIELD(SliceModifier::reducedCoordinates));
+    _reducedCoordinatesPUI = createParamUI<BooleanRadioButtonParameterUI>(PROPERTY_FIELD(SliceModifier::reducedCoordinates));
     _reducedCoordinatesPUI->buttonFalse()->setText(tr("Cartesian coordinates"));
     _reducedCoordinatesPUI->buttonTrue()->setText(tr("Miller indices"));
     sublayout->addWidget(_reducedCoordinatesPUI->buttonFalse(), 1);
@@ -84,13 +84,13 @@ void SliceModifierEditor::createUI(const RolloutInsertionParameters& rolloutPara
     gridlayout->setColumnStretch(1, 1);
 
     // Distance parameter.
-    _distancePUI = new FloatParameterUI(this, PROPERTY_FIELD(SliceModifier::distanceController));
+    _distancePUI = createParamUI<FloatParameterUI>(PROPERTY_FIELD(SliceModifier::distanceController));
     gridlayout->addWidget(_distancePUI->label(), 2, 0);
     gridlayout->addLayout(_distancePUI->createFieldLayout(), 2, 1);
 
     // Normal parameter.
     for(int i = 0; i < 3; i++) {
-        _normalPUI[i] = new Vector3ParameterUI(this, PROPERTY_FIELD(SliceModifier::normalController), i);
+        _normalPUI[i] = createParamUI<VectorParameterUI>(PROPERTY_FIELD(SliceModifier::normalController), i, 3);
         _normalPUI[i]->label()->setTextFormat(Qt::RichText);
         _normalPUI[i]->label()->setTextInteractionFlags(Qt::LinksAccessibleByMouse);
         connect(_normalPUI[i]->label(), &QLabel::linkActivated, this, &SliceModifierEditor::onAlignNormalWithAxis);
@@ -98,10 +98,9 @@ void SliceModifierEditor::createUI(const RolloutInsertionParameters& rolloutPara
         gridlayout->addLayout(_normalPUI[i]->createFieldLayout(), i + 3, 1);
     }
     connect(_reducedCoordinatesPUI->buttonFalse(), &QAbstractButton::toggled, this, &SliceModifierEditor::updateCoordinateLabels);
-    updateCoordinateLabels();
 
     // Slice width parameter.
-    FloatParameterUI* widthPUI = new FloatParameterUI(this, PROPERTY_FIELD(SliceModifier::widthController));
+    FloatParameterUI* widthPUI = createParamUI<FloatParameterUI>(PROPERTY_FIELD(SliceModifier::widthController));
     gridlayout->addWidget(widthPUI->label(), 6, 0);
     gridlayout->addLayout(widthPUI->createFieldLayout(), 6, 1);
 
@@ -109,19 +108,19 @@ void SliceModifierEditor::createUI(const RolloutInsertionParameters& rolloutPara
     layout->addSpacing(8);
 
     // Invert parameter.
-    BooleanParameterUI* invertPUI = new BooleanParameterUI(this, PROPERTY_FIELD(SliceModifier::inverse));
+    BooleanParameterUI* invertPUI = createParamUI<BooleanParameterUI>(PROPERTY_FIELD(SliceModifier::inverse));
     layout->addWidget(invertPUI->checkBox());
 
     // Create selection parameter.
-    BooleanParameterUI* createSelectionPUI = new BooleanParameterUI(this, PROPERTY_FIELD(SliceModifier::createSelection));
+    BooleanParameterUI* createSelectionPUI = createParamUI<BooleanParameterUI>(PROPERTY_FIELD(SliceModifier::createSelection));
     layout->addWidget(createSelectionPUI->checkBox());
 
     // Apply to selection only parameter.
-    BooleanParameterUI* applyToSelectionPUI = new BooleanParameterUI(this, PROPERTY_FIELD(SliceModifier::applyToSelection));
+    BooleanParameterUI* applyToSelectionPUI = createParamUI<BooleanParameterUI>(PROPERTY_FIELD(SliceModifier::applyToSelection));
     layout->addWidget(applyToSelectionPUI->checkBox());
 
     // Visualize plane.
-    BooleanParameterUI* visualizePlanePUI = new BooleanParameterUI(this, PROPERTY_FIELD(SliceModifier::enablePlaneVisualization));
+    BooleanParameterUI* visualizePlanePUI = createParamUI<BooleanParameterUI>(PROPERTY_FIELD(SliceModifier::enablePlaneVisualization));
     layout->addWidget(visualizePlanePUI->checkBox());
 
     layout->addSpacing(8);
@@ -137,9 +136,9 @@ void SliceModifierEditor::createUI(const RolloutInsertionParameters& rolloutPara
     connect(alignPlaneToViewBtn, &QPushButton::clicked, this, &SliceModifierEditor::onAlignPlaneToView);
     layout->addWidget(alignPlaneToViewBtn);
 
-    _pickPlanePointsInputMode = new PickPlanePointsInputMode(this);
-    connect(this, &QObject::destroyed, _pickPlanePointsInputMode, &ViewportInputMode::removeMode);
-    _pickPlanePointsInputModeAction = new ViewportModeAction(mainWindow(), tr("Pick three points"), this, _pickPlanePointsInputMode);
+    OORef<PickPlanePointsInputMode> pickPlanePointsInputMode = OORef<PickPlanePointsInputMode>::create(this);
+    connect(this, &QObject::destroyed, pickPlanePointsInputMode, &ViewportInputMode::removeMode);
+    _pickPlanePointsInputModeAction = new ViewportModeAction(mainWindow(), tr("Pick three points"), this, std::move(pickPlanePointsInputMode));
     layout->addWidget(new ViewportModeButton(_pickPlanePointsInputModeAction));
 
     // Deactivate input mode when editor is reset.
@@ -147,7 +146,7 @@ void SliceModifierEditor::createUI(const RolloutInsertionParameters& rolloutPara
 
     // Status label.
     layout->addSpacing(12);
-    layout->addWidget((new ObjectStatusDisplay(this))->statusWidget());
+    layout->addWidget(createParamUI<ObjectStatusDisplay>()->statusWidget());
 
     // Create a second rollout.
     rollout = createRollout(tr("Operate on"), rolloutParams.after(rollout), "manual:particles.modifiers.slice");
@@ -157,8 +156,54 @@ void SliceModifierEditor::createUI(const RolloutInsertionParameters& rolloutPara
     layout->setContentsMargins(4,4,4,4);
     layout->setSpacing(4);
 
-    ModifierDelegateFixedListParameterUI* delegatesPUI = new ModifierDelegateFixedListParameterUI(this, rolloutParams.after(rollout));
+    ModifierDelegateFixedListParameterUI* delegatesPUI = createParamUI<ModifierDelegateFixedListParameterUI>(rolloutParams.after(rollout));
     layout->addWidget(delegatesPUI->listWidget());
+
+    // Override automatic step size for the distance and slice width parameters.
+    _distanceUnit.emplace(mainWindow().unitsManager().getUnit(_distancePUI->parameterUnitType()));
+    _distancePUI->spinner()->setUnit(&_distanceUnit.value());
+    _slabWidthUnit.emplace(mainWindow().unitsManager().getUnit(widthPUI->parameterUnitType()));
+    widthPUI->spinner()->setUnit(&_slabWidthUnit.value());
+    _normalVectorUnit.emplace(mainWindow().unitsManager().getUnit(_normalPUI[0]->parameterUnitType()));
+    _normalPUI[0]->spinner()->setUnit(&_normalVectorUnit.value());
+    _normalPUI[1]->spinner()->setUnit(&_normalVectorUnit.value());
+    _normalPUI[2]->spinner()->setUnit(&_normalVectorUnit.value());
+
+    // Whenever the pipeline input of the modifier changes, update the increment scale of the distance and slab width parameters.
+    connect(this, &PropertiesEditor::pipelineInputChanged, this, &SliceModifierEditor::updateParameterUnitScales);
+
+    updateCoordinateLabels();
+}
+
+/******************************************************************************
+* Auto-adjusts the increment steps of the numeric parameter spinner widgets.
+******************************************************************************/
+void SliceModifierEditor::updateParameterUnitScales()
+{
+    OVITO_ASSERT(_distanceUnit.has_value());
+
+    bool usingReducedCoords = !_reducedCoordinatesPUI->buttonFalse()->isChecked();
+    FloatType distanceScale = 0;
+    FloatType slabWidthScale = 0;
+
+    if(usingReducedCoords) {
+        // When using reduced coordinates, use fixed step size.
+        distanceScale = 0.3;
+    }
+    if(DataOORef<const SimulationCell> cell = getPipelineInput().getObject<SimulationCell>()) {
+        // If a simulation cell is available, set the step size proportional to the cell's diameter.
+        FloatType cellDiameter = (
+                cell->cellMatrix().column(0) +
+                cell->cellMatrix().column(1) +
+                cell->cellMatrix().column(2) * !cell->is2D()).length() / 3;
+        if(!usingReducedCoords)
+            distanceScale = cellDiameter;
+        slabWidthScale = 0.1 * cellDiameter;
+    }
+
+    _distanceUnit->setScaleReference(distanceScale);
+    _slabWidthUnit->setScaleReference(slabWidthScale);
+    _normalVectorUnit->setScaleReference(1);
 }
 
 /******************************************************************************
@@ -180,6 +225,9 @@ void SliceModifierEditor::updateCoordinateLabels()
     _distancePUI->label()->setText(_reducedCoordinatesPUI->buttonFalse()->isChecked()
         ? tr("Distance:")
         : tr("<html>Distance [d<sub>hkl</sub>]:</html>"));
+
+    // Adjust the increment step size of the distance parameter.
+    updateParameterUnitScales();
 }
 
 /******************************************************************************
@@ -241,16 +289,14 @@ void SliceModifierEditor::onAlignNormalWithAxis(const QString& link)
 ******************************************************************************/
 void SliceModifierEditor::onAlignPlaneToView()
 {
-    TimeInterval interval;
-
     Viewport* vp = activeViewport();
     if(!vp) return;
 
     // Get the object to world transformation for the currently selected object.
-    Pipeline* pipeline = selectedPipeline();
-    if(!pipeline) return;
+    SceneNode* sceneNode = selectedSceneNode();
+    if(!sceneNode) return;
     AnimationTime time = currentAnimationTime();
-    const AffineTransformation& nodeTM = pipeline->getWorldTransform(time, interval);
+    const AffineTransformation& nodeTM = sceneNode->getWorldTransform(time);
 
     performTransaction(tr("Align plane to view"), [&]() {
 
@@ -260,6 +306,7 @@ void SliceModifierEditor::onAlignPlaneToView()
 
         const PipelineFlowState& input = getPipelineInput();
 
+        TimeInterval interval;
         Plane3 oldPlaneLocal = std::get<Plane3>(mod->slicingPlane(time, interval, input));
         Point3 basePoint = Point3::Origin() + oldPlaneLocal.normal * oldPlaneLocal.dist;
 
@@ -296,10 +343,10 @@ void SliceModifierEditor::onAlignViewToPlane()
         if(!vp) return;
 
         // Get the object to world transformation for the currently selected object
-        Pipeline* pipeline = selectedPipeline();
-        if(!pipeline) return;
+        SceneNode* sceneNode = selectedSceneNode();
+        if(!sceneNode) return;
         AnimationTime time = currentAnimationTime();
-        const AffineTransformation& nodeTM = pipeline->getWorldTransform(time, interval);
+        const AffineTransformation& nodeTM = sceneNode->getWorldTransform(time, interval);
 
         // Transform the current slicing plane to the world coordinate system.
         SliceModifier* mod = static_object_cast<SliceModifier>(editObject());
@@ -327,7 +374,7 @@ void SliceModifierEditor::onAlignViewToPlane()
             vp->setCameraDirection(-planeWorld.normal);
         }
 
-        vp->zoomToSelectionExtents();
+        vp->notifyDependents(Viewport::ZoomToSelectionExtentsRequested);
     });
 }
 
@@ -372,14 +419,14 @@ void PickPlanePointsInputMode::deactivated(bool temporary)
 /******************************************************************************
 * Handles the mouse events for a Viewport.
 ******************************************************************************/
-void PickPlanePointsInputMode::mouseMoveEvent(ViewportWindowInterface* vpwin, QMouseEvent* event)
+void PickPlanePointsInputMode::mouseMoveEvent(ViewportWindow* vpwin, QMouseEvent* event)
 {
     ViewportInputMode::mouseMoveEvent(vpwin, event);
 
-    ViewportPickResult pickResult = vpwin->pick(getMousePosition(event));
-    setCursor(pickResult.isValid() ? SelectionMode::selectionCursor() : QCursor());
-    if(pickResult.isValid() && _numPickedPoints < 3) {
-        _pickedPoints[_numPickedPoints] = pickResult.hitLocation();
+    std::optional<ViewportWindow::PickResult> pickResult = vpwin->pick(getMousePosition(event));
+    setCursor(pickResult ? SelectionMode::selectionCursor() : QCursor{});
+    if(pickResult && _numPickedPoints < 3) {
+        _pickedPoints[_numPickedPoints] = pickResult->hitLocation();
         _hasPreliminaryPoint = true;
         requestViewportUpdate();
     }
@@ -393,7 +440,7 @@ void PickPlanePointsInputMode::mouseMoveEvent(ViewportWindowInterface* vpwin, QM
 /******************************************************************************
 * Handles the mouse events for a Viewport.
 ******************************************************************************/
-void PickPlanePointsInputMode::mouseReleaseEvent(ViewportWindowInterface* vpwin, QMouseEvent* event)
+void PickPlanePointsInputMode::mouseReleaseEvent(ViewportWindow* vpwin, QMouseEvent* event)
 {
     if(event->button() == Qt::LeftButton) {
 
@@ -402,16 +449,15 @@ void PickPlanePointsInputMode::mouseReleaseEvent(ViewportWindowInterface* vpwin,
             requestViewportUpdate();
         }
 
-        ViewportPickResult pickResult = vpwin->pick(getMousePosition(event));
-        if(pickResult.isValid()) {
+        if(std::optional<ViewportWindow::PickResult> pickResult = vpwin->pick(getMousePosition(event))) {
 
             // Do not select the same point twice.
             bool ignore = false;
-            if(_numPickedPoints >= 1 && _pickedPoints[0].equals(pickResult.hitLocation(), FLOATTYPE_EPSILON)) ignore = true;
-            if(_numPickedPoints >= 2 && _pickedPoints[1].equals(pickResult.hitLocation(), FLOATTYPE_EPSILON)) ignore = true;
+            if(_numPickedPoints >= 1 && _pickedPoints[0].equals(pickResult->hitLocation(), FLOATTYPE_EPSILON)) ignore = true;
+            if(_numPickedPoints >= 2 && _pickedPoints[1].equals(pickResult->hitLocation(), FLOATTYPE_EPSILON)) ignore = true;
 
             if(!ignore) {
-                _pickedPoints[_numPickedPoints] = pickResult.hitLocation();
+                _pickedPoints[_numPickedPoints] = pickResult->hitLocation();
                 _numPickedPoints++;
                 _hasPreliminaryPoint = false;
                 requestViewportUpdate();
@@ -446,10 +492,10 @@ void PickPlanePointsInputMode::alignPlane(SliceModifier* mod)
         // Get the object-to-world transformation for the currently selected pipeline.
         ModificationNode* modNode = _editor->modificationNode();
         if(!modNode) return;
-        Pipeline* pipeline = _editor->selectedPipeline();
-        if(!pipeline) return;
+        SceneNode* sceneNode = _editor->selectedSceneNode();
+        if(!sceneNode) return;
         TimeInterval interval;
-        const AffineTransformation& nodeTM = pipeline->getWorldTransform(_editor->currentAnimationTime(), interval);
+        const AffineTransformation& nodeTM = sceneNode->getWorldTransform(_editor->currentAnimationTime(), interval);
 
         // Transform new plane from world to object space.
         Plane3 localPlane = nodeTM.inverse() * worldPlane;
@@ -479,51 +525,46 @@ void PickPlanePointsInputMode::alignPlane(SliceModifier* mod)
 /******************************************************************************
 * Lets the input mode render its overlay content in a viewport.
 ******************************************************************************/
-void PickPlanePointsInputMode::renderOverlay3D(Viewport* vp, SceneRenderer* renderer)
+void PickPlanePointsInputMode::renderOverlay(Viewport* vp, ViewportWindow* vpWin, FrameGraph& frameGraph, DataSet* dataset)
 {
-    if(!renderer->isImagePass())
-        return;
-
     int npoints = _numPickedPoints;
     if(_hasPreliminaryPoint && npoints < 3) npoints++;
     if(_numPickedPoints == 0)
         return;
 
-    renderer->setWorldTransform(AffineTransformation::Identity());
-    if(!renderer->isBoundingBoxPass()) {
-        MarkerPrimitive markers(MarkerPrimitive::BoxShape);
-        markers.makePositions(_pickedPoints, _pickedPoints + npoints);
-        markers.setColor(ColorA(1, 1, 1));
-        renderer->renderMarkers(markers);
+    Box3 boundingBox;
+    for(int i = 0; i < npoints; i++)
+        boundingBox.addPoint(_pickedPoints[i]);
 
-        if(npoints == 2) {
-            LinePrimitive lines;
-            lines.makePositions(_pickedPoints, _pickedPoints + 2);
-            lines.setUniformColor(ColorA(1, 1, 1));
-            renderer->renderLines(lines);
-        }
-        else if(npoints == 3) {
-            DataOORef<TriangleMesh> tri = DataOORef<TriangleMesh>::create(ObjectInitializationFlag::DontCreateVisElement);
-            tri->setVertexCount(3);
-            tri->setVertex(0, _pickedPoints[0]);
-            tri->setVertex(1, _pickedPoints[1]);
-            tri->setVertex(2, _pickedPoints[2]);
-            tri->addFace().setVertices(0, 1, 2);
-            MeshPrimitive meshPrimitive;
-            meshPrimitive.setMesh(std::move(tri), MeshPrimitive::ConvexShapeMode);
-            meshPrimitive.setUniformColor(ColorA(0.7, 0.7, 1.0, 0.5));
-            renderer->renderMesh(meshPrimitive);
+    std::unique_ptr<MarkerPrimitive> markers = std::make_unique<MarkerPrimitive>(MarkerPrimitive::BoxShape);
+    markers->makePositions(_pickedPoints, _pickedPoints + npoints);
+    markers->setColor(ColorA(1, 1, 1));
+    FrameGraph::RenderingCommandGroup& commandGroup = frameGraph.addCommandGroup(FrameGraph::SceneLayer);
+    commandGroup.addPrimitiveNonpickable(std::move(markers), AffineTransformation::Identity(), boundingBox);
 
-            LinePrimitive lines;
-            const Point3 vertices[6] = { _pickedPoints[0], _pickedPoints[1], _pickedPoints[1], _pickedPoints[2], _pickedPoints[2], _pickedPoints[0] };
-            lines.makePositions(vertices);
-            lines.setUniformColor(ColorA(1, 1, 1));
-            renderer->renderLines(lines);
-        }
+    if(npoints == 2) {
+        std::unique_ptr<LinePrimitive> lines = std::make_unique<LinePrimitive>();
+        lines->makePositions(_pickedPoints, _pickedPoints + 2);
+        lines->setUniformColor(ColorA(1, 1, 1));
+        commandGroup.addPrimitiveNonpickable(std::move(lines), AffineTransformation::Identity(), boundingBox);
     }
-    else {
-        for(int i = 0; i < npoints; i++)
-            renderer->addToLocalBoundingBox(_pickedPoints[i]);
+    else if(npoints == 3) {
+        DataOORef<TriangleMesh> tri = DataOORef<TriangleMesh>::create(ObjectInitializationFlag::DontCreateVisElement);
+        tri->setVertexCount(3);
+        tri->setVertex(0, _pickedPoints[0]);
+        tri->setVertex(1, _pickedPoints[1]);
+        tri->setVertex(2, _pickedPoints[2]);
+        tri->addFace().setVertices(0, 1, 2);
+        std::unique_ptr<MeshPrimitive> meshPrimitive = std::make_unique<MeshPrimitive>();
+        meshPrimitive->setMesh(std::move(tri), MeshPrimitive::ConvexShapeMode);
+        meshPrimitive->setUniformColor(ColorA(0.7, 0.7, 1.0, 0.5));
+        commandGroup.addPrimitiveNonpickable(std::move(meshPrimitive), AffineTransformation::Identity(), boundingBox);
+
+        std::unique_ptr<LinePrimitive> lines = std::make_unique<LinePrimitive>();
+        const Point3 vertices[6] = { _pickedPoints[0], _pickedPoints[1], _pickedPoints[1], _pickedPoints[2], _pickedPoints[2], _pickedPoints[0] };
+        lines->makePositions(vertices);
+        lines->setUniformColor(ColorA(1, 1, 1));
+        commandGroup.addPrimitiveNonpickable(std::move(lines), AffineTransformation::Identity(), boundingBox);
     }
 }
 
