@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -21,43 +21,48 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 
 #include <ovito/stdmod/StdMod.h>
-#include <ovito/core/dataset/DataSet.h>
 #include <ovito/stdobj/lines/Lines.h>
+#include <ovito/stdobj/vectors/Vectors.h>
 #include <ovito/stdobj/simcell/SimulationCell.h>
 #include <ovito/stdobj/simcell/PeriodicDomainObject.h>
 #include <ovito/stdobj/properties/Property.h>
+#include <ovito/core/dataset/DataSet.h>
 #include <ovito/core/dataset/pipeline/ModificationNode.h>
 #include <ovito/core/dataset/animation/AnimationSettings.h>
 #include "AffineTransformationModifier.h"
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(AffineTransformationModifier);
+IMPLEMENT_CREATABLE_OVITO_CLASS(AffineTransformationModifier);
+OVITO_CLASSINFO(AffineTransformationModifier, "DisplayName", "Affine transformation");
+OVITO_CLASSINFO(AffineTransformationModifier, "Description", "Apply an affine transformation to the dataset.");
+OVITO_CLASSINFO(AffineTransformationModifier, "ModifierCategory", "Modification");
 DEFINE_PROPERTY_FIELD(AffineTransformationModifier, transformationTM);
 DEFINE_PROPERTY_FIELD(AffineTransformationModifier, selectionOnly);
 DEFINE_PROPERTY_FIELD(AffineTransformationModifier, targetCell);
 DEFINE_PROPERTY_FIELD(AffineTransformationModifier, relativeMode);
 DEFINE_PROPERTY_FIELD(AffineTransformationModifier, translationReducedCoordinates);
 SET_PROPERTY_FIELD_LABEL(AffineTransformationModifier, transformationTM, "Transformation");
-SET_PROPERTY_FIELD_LABEL(AffineTransformationModifier, selectionOnly, "Transform selected elements only");
+SET_PROPERTY_FIELD_LABEL(AffineTransformationModifier, selectionOnly, "Transform only selected particles/vertices");
 SET_PROPERTY_FIELD_LABEL(AffineTransformationModifier, targetCell, "Target cell shape");
 SET_PROPERTY_FIELD_LABEL(AffineTransformationModifier, relativeMode, "Relative transformation");
 SET_PROPERTY_FIELD_LABEL(AffineTransformationModifier, translationReducedCoordinates, "Relative transformation");
 
-IMPLEMENT_OVITO_CLASS(AffineTransformationModifierDelegate);
-IMPLEMENT_OVITO_CLASS(LinesAffineTransformationModifierDelegate);
-IMPLEMENT_OVITO_CLASS(SimulationCellAffineTransformationModifierDelegate);
+IMPLEMENT_ABSTRACT_OVITO_CLASS(AffineTransformationModifierDelegate);
+IMPLEMENT_CREATABLE_OVITO_CLASS(LinesAffineTransformationModifierDelegate);
+OVITO_CLASSINFO(LinesAffineTransformationModifierDelegate, "DisplayName", "Lines");
+IMPLEMENT_CREATABLE_OVITO_CLASS(VectorsAffineTransformationModifierDelegate);
+OVITO_CLASSINFO(VectorsAffineTransformationModifierDelegate, "DisplayName", "Vectors");
+IMPLEMENT_CREATABLE_OVITO_CLASS(SimulationCellAffineTransformationModifierDelegate);
+OVITO_CLASSINFO(SimulationCellAffineTransformationModifierDelegate, "DisplayName", "Simulation cell");
 
 /******************************************************************************
-* Constructs the modifier object.
+* Constructor.
 ******************************************************************************/
-AffineTransformationModifier::AffineTransformationModifier(ObjectInitializationFlags flags) : MultiDelegatingModifier(flags),
-    _selectionOnly(false),
-    _transformationTM(AffineTransformation::Identity()),
-    _targetCell(AffineTransformation::Zero()),
-    _relativeMode(true),
-    _translationReducedCoordinates(false)
+void AffineTransformationModifier::initializeObject(ObjectInitializationFlags flags)
 {
+    MultiDelegatingModifier::initializeObject(flags);
+
     if(!flags.testFlag(ObjectInitializationFlag::DontInitializeObject)) {
         // Generate the list of delegate objects.
         createModifierDelegates(AffineTransformationModifierDelegate::OOClass());
@@ -74,7 +79,7 @@ void AffineTransformationModifier::initializeModifier(const ModifierInitializati
 
     // Take the simulation cell from the input object as the default destination cell geometry for absolute scaling.
     if(targetCell() == AffineTransformation::Zero()) {
-        const PipelineFlowState& input = request.modificationNode()->evaluateInputSynchronous(request);
+        const PipelineFlowState& input = request.modificationNode()->evaluateInput(request).blockForResult();
         if(const SimulationCell* cell = input.getObject<SimulationCell>())
             setTargetCell(cell->cellMatrix());
     }
@@ -108,9 +113,9 @@ AffineTransformation AffineTransformationModifier::effectiveAffineTransformation
 
 /******************************************************************************
 * Copies positions from one buffer to another while transforming them.
-* If enabled, the transformation is only applied to selected elements.
+* The transformation may be applied only to selected elements.
 ******************************************************************************/
-void AffineTransformationModifier::transformCoordinates(const PipelineFlowState& inputState, const Property* input, Property* output, const Property* selection)
+void AffineTransformationModifier::transformCoordinates(const AffineTransformation tm, bool selectionOnly, const Property* input, Property* output, const Property* selection)
 {
     OVITO_ASSERT(output != input);
     OVITO_ASSERT(output->size() == input->size());
@@ -118,17 +123,14 @@ void AffineTransformationModifier::transformCoordinates(const PipelineFlowState&
     if(input->size() == 0)
         return;
 
-    // Determine transformation matrix.
-    const AffineTransformation tm = effectiveAffineTransformation(inputState);
-
-    if(selectionOnly()) {
+    if(selectionOnly) {
         if(selection) {
 #ifdef OVITO_USE_SYCL
-            ExecutionContext::current().ui().taskManager().syclQueue().submit([&](SYCL_NS::handler& cgh) {
+            this_task::ui()->taskManager().syclQueue().submit([&](sycl::handler& cgh) {
                 SyclBufferAccess<const Point3, access_mode::read> posIn(input, cgh);
                 SyclBufferAccess<Point3, access_mode::discard_write> posOut(output, cgh);
                 SyclBufferAccess<const SelectionIntType, access_mode::read> selectionIn(selection, cgh);
-                cgh.parallel_for<class affine_coord_transformation_selection>(SYCL_NS::range(input->size()), [=](size_t i) {
+                OVITO_SYCL_PARALLEL_FOR(cgh, affine_coord_transformation_selection)(sycl::range(input->size()), [=](size_t i) {
                     posOut[i] = selectionIn[i] ? (tm * posIn[i]) : posIn[i];
                 });
             });
@@ -154,10 +156,10 @@ void AffineTransformationModifier::transformCoordinates(const PipelineFlowState&
         if(tm.isTranslationMatrix()) {
             const Vector3 translation = tm.translation();
 #ifdef OVITO_USE_SYCL
-            ExecutionContext::current().ui().taskManager().syclQueue().submit([&](SYCL_NS::handler& cgh) {
+            this_task::ui()->taskManager().syclQueue().submit([&](sycl::handler& cgh) {
                 SyclBufferAccess<const Point3, access_mode::read> posIn(input, cgh);
                 SyclBufferAccess<Point3, access_mode::discard_write> posOut(output, cgh);
-                cgh.parallel_for<class affine_coord_transformation_simple_translation>(SYCL_NS::range(input->size()), [=](size_t i) {
+                OVITO_SYCL_PARALLEL_FOR(cgh, affine_coord_transformation_simple_translation)(sycl::range(input->size()), [=](size_t i) {
                     posOut[i] = posIn[i] + translation;
                 });
             });
@@ -170,10 +172,10 @@ void AffineTransformationModifier::transformCoordinates(const PipelineFlowState&
         }
         else {
 #ifdef OVITO_USE_SYCL
-            ExecutionContext::current().ui().taskManager().syclQueue().submit([&](SYCL_NS::handler& cgh) {
+            this_task::ui()->taskManager().syclQueue().submit([&](sycl::handler& cgh) {
                 SyclBufferAccess<const Point3, access_mode::read> posIn(input, cgh);
                 SyclBufferAccess<Point3, access_mode::discard_write> posOut(output, cgh);
-                cgh.parallel_for<class affine_coord_transformation_full_xform>(SYCL_NS::range(input->size()), [=](size_t i) {
+                OVITO_SYCL_PARALLEL_FOR(cgh, affine_coord_transformation_full_xform)(sycl::range(input->size()), [=](size_t i) {
                     posOut[i] = tm * posIn[i];
                 });
             });
@@ -191,7 +193,7 @@ void AffineTransformationModifier::transformCoordinates(const PipelineFlowState&
 * Copies vectors from one buffer to another while transforming them.
 * If enabled, the transformation is only applied to selected elements.
 ******************************************************************************/
-void AffineTransformationModifier::transformVectors(const PipelineFlowState& inputState, const Property* input, Property* output, const Property* selection)
+void AffineTransformationModifier::transformVectors(const AffineTransformation tm, bool selectionOnly, const Property* input, Property* output, const Property* selection)
 {
     OVITO_ASSERT(output != input);
     OVITO_ASSERT(output->size() == input->size());
@@ -203,18 +205,18 @@ void AffineTransformationModifier::transformVectors(const PipelineFlowState& inp
         using T = decltype(_);
         using Vec3 = Vector_3<T>;
 
-        // Determine transformation matrix.
-        const AffineTransformationT<T> tm = effectiveAffineTransformation(inputState).toDataType<T>();
+        // Transformation matrix.
+        const AffineTransformationT<T> tm_typed = tm.toDataType<T>();
 
-        if(selectionOnly()) {
+        if(selectionOnly) {
             if(selection) {
 #ifdef OVITO_USE_SYCL
-                ExecutionContext::current().ui().taskManager().syclQueue().submit([&](SYCL_NS::handler& cgh) {
+                this_task::ui()->taskManager().syclQueue().submit([&](sycl::handler& cgh) {
                     SyclBufferAccess<const Vec3, access_mode::read> vecIn(input, cgh);
                     SyclBufferAccess<Vec3, access_mode::discard_write> vecOut(output, cgh);
                     SyclBufferAccess<const SelectionIntType, access_mode::read> selectionIn(selection, cgh);
-                    cgh.parallel_for<class affine_vec_transformation_selection>(SYCL_NS::range(input->size()), [=](size_t i) {
-                        vecOut[i] = selectionIn[i] ? (tm * vecIn[i]) : vecIn[i];
+                    OVITO_SYCL_PARALLEL_FOR(cgh, affine_vec_transformation_selection)(sycl::range(input->size()), [=](size_t i) {
+                        vecOut[i] = selectionIn[i] ? (tm_typed * vecIn[i]) : vecIn[i];
                     });
                 });
 #else
@@ -223,7 +225,7 @@ void AffineTransformationModifier::transformVectors(const PipelineFlowState& inp
                 BufferReadAccess<const SelectionIntType> selAccess(selection);
                 const auto* s = selAccess.cbegin();
                 for(Vec3& vout : BufferWriteAccess<Vec3, access_mode::discard_write>(output)) {
-                    vout = (*s++) ? (tm * (*vin)) : (*vin);
+                    vout = (*s++) ? (tm_typed * (*vin)) : (*vin);
                     ++vin;
                 }
 #endif
@@ -235,29 +237,94 @@ void AffineTransformationModifier::transformVectors(const PipelineFlowState& inp
         }
         else {
             // Check if the matrix describes a pure translation. If so, effectively, no vector transformation takes place.
-            if(tm.isTranslationMatrix()) {
+            if(tm_typed.isTranslationMatrix()) {
                 output->copyFrom(*input);
             }
             else {
 #ifdef OVITO_USE_SYCL
-                ExecutionContext::current().ui().taskManager().syclQueue().submit([&](SYCL_NS::handler& cgh) {
+                this_task::ui()->taskManager().syclQueue().submit([&](sycl::handler& cgh) {
                     SyclBufferAccess<const Vec3, access_mode::read> vecIn(input, cgh);
                     SyclBufferAccess<Vec3, access_mode::discard_write> vecOut(output, cgh);
-                    cgh.parallel_for<class affine_vec_transformation_full_xform>(SYCL_NS::range(input->size()), [=](size_t i) {
-                        vecOut[i] = tm * vecIn[i];
+                    OVITO_SYCL_PARALLEL_FOR(cgh, affine_vec_transformation_full_xform)(sycl::range(input->size()), [=](size_t i) {
+                        vecOut[i] = tm_typed * vecIn[i];
                     });
                 });
 #else
                 BufferReadAccess<const Vec3> vecIn(input);
                 const auto* vin = vecIn.cbegin();
                 for(Vec3& vout : BufferWriteAccess<Vec3, access_mode::discard_write>(output))
-                    vout = tm * (*vin++);
+                    vout = tm_typed * (*vin++);
 #endif
             }
         }
     });
 }
 
+/******************************************************************************
+* Copies quaternions from one buffer to another while rotating them.
+* If enabled, the rotation is only applied to selected elements.
+******************************************************************************/
+void AffineTransformationModifier::transformOrientations(const Quaternion q, bool selectionOnly, const Property* input, Property* output, const Property* selection)
+{
+    OVITO_ASSERT(output != input);
+    OVITO_ASSERT(output->size() == input->size());
+
+    if(input->size() == 0)
+        return;
+
+    input->forTypes<DataBuffer::Float32, DataBuffer::Float64>([&](auto _) {
+        using T = decltype(_);
+        using Quat = QuaternionT<T>;
+
+        // Rotation.
+        const Quat q_typed = q.toDataType<T>();
+
+        if(selectionOnly) {
+            if(selection) {
+#ifdef OVITO_USE_SYCL
+                ExecutionContext::current().ui().taskManager().syclQueue().submit([&](sycl::handler& cgh) {
+                    SyclBufferAccess<const Quat, access_mode::read> qIn(input, cgh);
+                    SyclBufferAccess<Quat, access_mode::discard_write> qOut(output, cgh);
+                    SyclBufferAccess<const SelectionIntType, access_mode::read> selectionIn(selection, cgh);
+                    OVITO_SYCL_PARALLEL_FOR(cgh, affine_quat_transformation_selection)(sycl::range(input->size()), [=](size_t i) {
+                        qOut[i] = selectionIn[i] ? (q_typed * qIn[i]) : qIn[i];
+                    });
+                });
+#else
+                BufferReadAccess<const Quat> qIn(input);
+                const auto* qin = qIn.cbegin();
+                BufferReadAccess<const SelectionIntType> selAccess(selection);
+                const auto* s = selAccess.cbegin();
+                for(Quat& qout : BufferWriteAccess<Quat, access_mode::discard_write>(output)) {
+                    qout = (*s++) ? (q_typed * (*qin)) : (*qin);
+                    ++qin;
+                }
+#endif
+            }
+            else {
+                // Without any selection property present, none of the elements get transformed.
+                output->copyFrom(*input);
+            }
+        }
+        else {
+#ifdef OVITO_USE_SYCL
+            ExecutionContext::current().ui().taskManager().syclQueue().submit([&](sycl::handler& cgh) {
+                SyclBufferAccess<const Quat, access_mode::read> qIn(input, cgh);
+                SyclBufferAccess<Quat, access_mode::discard_write> qOut(output, cgh);
+                OVITO_SYCL_PARALLEL_FOR(cgh, affine_quat_transformation_full)(sycl::range(input->size()), [=](size_t i) {
+                    qOut[i] = q_typed * qIn[i];
+                });
+            });
+#else
+            BufferReadAccess<const Quat> qIn(input);
+            const auto* qin = qIn.cbegin();
+            for(Quat& qout : BufferWriteAccess<Quat, access_mode::discard_write>(output))
+                qout = q_typed * (*qin++);
+#endif
+        }
+    });
+}
+
 /******************************************************************************
 * Asks the metaclass which data objects in the given input data collection the
 * modifier delegate can operate on.
@@ -274,30 +341,29 @@ QVector<DataObjectReference> SimulationCellAffineTransformationModifierDelegate:
 /******************************************************************************
 * Applies the modifier operation to the data in a pipeline flow state.
 ******************************************************************************/
-PipelineStatus SimulationCellAffineTransformationModifierDelegate::apply(const ModifierEvaluationRequest& request, PipelineFlowState& state, const PipelineFlowState& inputState, const std::vector<std::reference_wrapper<const PipelineFlowState>>& additionalInputs)
+Future<PipelineFlowState> SimulationCellAffineTransformationModifierDelegate::apply(const ModifierEvaluationRequest& request, PipelineFlowState&& state, const PipelineFlowState& originalState, const std::vector<std::reference_wrapper<const PipelineFlowState>>& additionalInputs)
 {
-    const AffineTransformationModifier* mod = static_object_cast<AffineTransformationModifier>(request.modifier());
+    AffineTransformationModifier* modifier = static_object_cast<AffineTransformationModifier>(request.modifier());
 
-    // Transform the SimulationCell.
+    // Transform the simulation box.
     if(const SimulationCell* inputCell = state.getObject<SimulationCell>()) {
         SimulationCell* outputCell = state.makeMutable(inputCell);
-        outputCell->setCellMatrix(mod->relativeMode() ? (mod->effectiveAffineTransformation(inputState) * inputCell->cellMatrix()) : mod->targetCell());
+        outputCell->setCellMatrix(modifier->relativeMode() ? (modifier->effectiveAffineTransformation(originalState) * inputCell->cellMatrix()) : modifier->targetCell());
     }
 
-    if(mod->selectionOnly())
-        return PipelineStatus::Success;
-
-    // Transform the domains of PeriodicDomainDataObjects.
-    for(const DataObject* obj : state.data()->objects()) {
-        if(const PeriodicDomainObject* existingObject = dynamic_object_cast<PeriodicDomainObject>(obj)) {
-            if(existingObject->domain()) {
-                PeriodicDomainObject* newObject = state.makeMutable(existingObject);
-                newObject->mutableDomain()->setCellMatrix(mod->effectiveAffineTransformation(inputState) * existingObject->domain()->cellMatrix());
+    if(!modifier->selectionOnly()) {
+        // Transform the domains of PeriodicDomainDataObjects.
+        for(qsizetype i = 0; i < state.data()->objects().size(); i++) {
+            if(const PeriodicDomainObject* existingObject = dynamic_object_cast<PeriodicDomainObject>(state.data()->objects()[i])) {
+                if(existingObject->domain()) {
+                    PeriodicDomainObject* newObject = state.makeMutable(existingObject);
+                    newObject->mutableDomain()->setCellMatrix(modifier->effectiveAffineTransformation(originalState) * existingObject->domain()->cellMatrix());
+                }
             }
         }
     }
 
-    return PipelineStatus::Success;
+    return std::move(state);
 }
 
 /******************************************************************************
@@ -317,32 +383,102 @@ QVector<DataObjectReference> LinesAffineTransformationModifierDelegate::OOMetaCl
 /******************************************************************************
  * Applies the modifier operation to the data in a pipeline flow state.
  ******************************************************************************/
-PipelineStatus LinesAffineTransformationModifierDelegate::apply(
-    const ModifierEvaluationRequest& request, PipelineFlowState& state, const PipelineFlowState& inputState,
+Future<PipelineFlowState> LinesAffineTransformationModifierDelegate::apply(const ModifierEvaluationRequest& request, PipelineFlowState&& state, const PipelineFlowState& originalState, const std::vector<std::reference_wrapper<const PipelineFlowState>>& additionalInputs)
+{
+    AffineTransformationModifier* modifier = static_object_cast<AffineTransformationModifier>(request.modifier());
+
+    // The actual work can be performed in a separate thread.
+    return asyncLaunch([
+            state = std::move(state),
+            tm = modifier->effectiveAffineTransformation(originalState),
+            selectionOnly = modifier->selectionOnly()]() mutable {
+
+        // Loop over all lines objects in the data collection
+        for(qsizetype i = 0; i < state.data()->objects().size(); i++) {
+            // Transform the line vertices.
+            if(const Lines* inputLines = dynamic_object_cast<Lines>(state.data()->objects()[i])) {
+                // Get the input line coordinates (as strong reference to force creation of a mutable clone below).
+                ConstPropertyPtr inputPositionProperty = inputLines->expectProperty(Lines::PositionProperty);
+
+                // Make sure we can safely modify the lines object.
+                Lines* outputLines = state.makeMutable(inputLines);
+
+                // Create an uninitialized copy of the position property.
+                Property* outputPositionProperty = outputLines->makePropertyMutable(inputPositionProperty, DataBuffer::Uninitialized);
+
+                // Let the modifier class do the actual coordinate transformation work.
+                // Note: "Selection" property is currently not supported by Lines objects.
+                OVITO_ASSERT(Lines::OOClass().isValidStandardPropertyId(Property::GenericSelectionProperty) == false);
+                AffineTransformationModifier::transformCoordinates(tm, selectionOnly, inputPositionProperty, outputPositionProperty, nullptr);
+            }
+        }
+
+        return std::move(state);
+    });
+}
+
+/******************************************************************************
+ * Asks the metaclass which data objects in the given input data collection the
+ * modifier delegate can operate on.
+ ******************************************************************************/
+QVector<DataObjectReference> VectorsAffineTransformationModifierDelegate::OOMetaClass::getApplicableObjects(
+    const DataCollection& input) const
+{
+    // Gather list of all vectors objects in the input data collection.
+    QVector<DataObjectReference> objects;
+    for(const ConstDataObjectPath& path : input.getObjectsRecursive(Vectors::OOClass())) {
+        objects.push_back(path);
+    }
+
+    return objects;
+}
+
+/******************************************************************************
+ * Applies the modifier operation to the data in a pipeline flow state.
+ ******************************************************************************/
+Future<PipelineFlowState> VectorsAffineTransformationModifierDelegate::apply(
+    const ModifierEvaluationRequest& request, PipelineFlowState&& state, const PipelineFlowState& originalState,
     const std::vector<std::reference_wrapper<const PipelineFlowState>>& additionalInputs)
 {
-    const AffineTransformationModifier* mod = static_object_cast<AffineTransformationModifier>(request.modifier());
+    AffineTransformationModifier* modifier = static_object_cast<AffineTransformationModifier>(request.modifier());
 
-    // Loop over all lines objects in data collection
-    for(const DataObject* obj : state.data()->objects()) {
-        // Transform the Lines.
-        if(const Lines* inputLines = dynamic_object_cast<Lines>(obj)) {
-            // Get the input line coordinates (as strong reference to force creation of a mutable clone below).
-            ConstPropertyPtr inputPositionProperty = inputLines->expectProperty(Lines::PositionProperty);
+    // The actual work can be performed in a separate thread.
+    return asyncLaunch([state = std::move(state), tm = modifier->effectiveAffineTransformation(originalState),
+                        selectionOnly = modifier->selectionOnly()]() mutable {
+        // Loop over all vectors objects in the data collection
+        for(qsizetype i = 0; i < state.data()->objects().size(); i++) {
+            // Transform the vectors.
+            if(const Vectors* inputVectors = dynamic_object_cast<Vectors>(state.data()->objects()[i])) {
+                // Make sure we can safely modify the vectors object.
+                Vectors* outputVectors = state.makeMutable(inputVectors);
 
-            // Make sure we can safely modify the lines object.
-            Lines* outputLines = state.makeMutable(inputLines);
+                // Transform the basis points
 
-            // Create an uninitialized copy of the position property.
-            Property* outputPositionProperty = outputLines->makePropertyMutable(inputPositionProperty, DataBuffer::Uninitialized);
+                // Get the input basis points (as strong reference to force creation of a mutable clone below).
+                ConstPropertyPtr inputPositionProperty = outputVectors->expectProperty(Vectors::PositionProperty);
 
-            // Let the modifier class do the actual coordinate transformation work.
-            AffineTransformationModifier* mod = static_object_cast<AffineTransformationModifier>(request.modifier());
-            // Note: passing nullptr since "Selection" property is currently not supported by Lines objects.
-            mod->transformCoordinates(inputState, inputPositionProperty, outputPositionProperty, nullptr);
+                // Create an uninitialized copy of the position property.
+                Property* outputPositionProperty = outputVectors->makePropertyMutable(inputPositionProperty, DataBuffer::Uninitialized);
+
+                // Let the modifier class do the actual coordinate transformation work.
+                AffineTransformationModifier::transformCoordinates(tm, selectionOnly, inputPositionProperty, outputPositionProperty,
+                                                                   nullptr);
+
+                // Transform the directions
+
+                // Get the input line coordinates (as strong reference to force creation of a mutable clone below).
+                ConstPropertyPtr inputDirectionProperty = outputVectors->expectProperty(Vectors::DirectionProperty);
+
+                // Create an uninitialized copy of the position property.
+                Property* outputDirectionProperty = outputVectors->makePropertyMutable(inputDirectionProperty, DataBuffer::Uninitialized);
+
+                // Let the modifier class do the actual coordinate transformation work.
+                AffineTransformationModifier::transformVectors(tm, selectionOnly, inputDirectionProperty, outputDirectionProperty, nullptr);
+            }
         }
-    }
-    return PipelineStatus::Success;
+
+        return std::move(state);
+    });
 }
 
 }   // End of namespace
