@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -25,7 +25,10 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(DeleteSelectedModifierDelegate);
-IMPLEMENT_OVITO_CLASS(DeleteSelectedModifier);
+IMPLEMENT_ABSTRACT_OVITO_CLASS(DeleteSelectedModifierDelegate);
+IMPLEMENT_CREATABLE_OVITO_CLASS(DeleteSelectedModifier);
+OVITO_CLASSINFO(DeleteSelectedModifier, "DisplayName", "Delete selected");
+OVITO_CLASSINFO(DeleteSelectedModifier, "Description", "Remove all currently selected elements.");
+OVITO_CLASSINFO(DeleteSelectedModifier, "ModifierCategory", "Modification");
 
 }   // End of namespace
