@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -32,26 +32,35 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(FreezePropertyModifier);
+IMPLEMENT_CREATABLE_OVITO_CLASS(FreezePropertyModifier);
+OVITO_CLASSINFO(FreezePropertyModifier, "DisplayName", "Freeze property");
+OVITO_CLASSINFO(FreezePropertyModifier, "Description", "Copy the values of a varying property from one trajectory frame to all others.");
+OVITO_CLASSINFO(FreezePropertyModifier, "ModifierCategory", "Modification");
 DEFINE_PROPERTY_FIELD(FreezePropertyModifier, sourceProperty);
 DEFINE_PROPERTY_FIELD(FreezePropertyModifier, destinationProperty);
 DEFINE_PROPERTY_FIELD(FreezePropertyModifier, freezeTime);
+DEFINE_PROPERTY_FIELD(FreezePropertyModifier, tolerateNewElements);
+DEFINE_PROPERTY_FIELD(FreezePropertyModifier, selectNewElements);
 SET_PROPERTY_FIELD_LABEL(FreezePropertyModifier, sourceProperty, "Property");
 SET_PROPERTY_FIELD_LABEL(FreezePropertyModifier, destinationProperty, "Destination property");
 SET_PROPERTY_FIELD_LABEL(FreezePropertyModifier, freezeTime, "Freeze at frame");
+SET_PROPERTY_FIELD_LABEL(FreezePropertyModifier, tolerateNewElements, "Tolerate newly appearing elements");
+SET_PROPERTY_FIELD_LABEL(FreezePropertyModifier, selectNewElements, "Select newly appearing elements");
 
-IMPLEMENT_OVITO_CLASS(FreezePropertyModificationNode);
+IMPLEMENT_CREATABLE_OVITO_CLASS(FreezePropertyModificationNode);
+OVITO_CLASSINFO(FreezePropertyModificationNode, "ClassNameAlias", "FreezePropertyModifierApplication");  // For backward compatibility with OVITO 3.9.2
 DEFINE_REFERENCE_FIELD(FreezePropertyModificationNode, property);
 DEFINE_REFERENCE_FIELD(FreezePropertyModificationNode, identifiers);
 DEFINE_VECTOR_REFERENCE_FIELD(FreezePropertyModificationNode, cachedVisElements);
 SET_MODIFICATION_NODE_TYPE(FreezePropertyModifier, FreezePropertyModificationNode);
 
 /******************************************************************************
-* Constructs the modifier object.
+* Constructor.
 ******************************************************************************/
-FreezePropertyModifier::FreezePropertyModifier(ObjectInitializationFlags flags) : GenericPropertyModifier(flags),
-    _freezeTime(0)
+void FreezePropertyModifier::initializeObject(ObjectInitializationFlags flags)
 {
+    GenericPropertyModifier::initializeObject(flags);
+
     // Operate on particles by default.
     setDefaultSubject(QStringLiteral("Particles"), QStringLiteral("Particles"));
 }
@@ -65,11 +74,11 @@ void FreezePropertyModifier::initializeModifier(const ModifierInitializationRequ
     GenericPropertyModifier::initializeModifier(request);
 
     // Use the first available particle property from the input state as data source when the modifier is newly created.
-    if(sourceProperty().isNull() && subject() && ExecutionContext::isInteractive()) {
-        const PipelineFlowState& input = request.modificationNode()->evaluateInputSynchronous(request);
+    if(!sourceProperty() && subject() && this_task::isInteractive()) {
+        const PipelineFlowState& input = request.modificationNode()->evaluateInput(request).blockForResult();
         if(const PropertyContainer* container = input.getLeafObject(subject())) {
             for(const Property* property : container->properties()) {
-                setSourceProperty(PropertyReference(subject().dataClass(), property));
+                setSourceProperty(property);
                 setDestinationProperty(sourceProperty());
                 break;
             }
@@ -82,12 +91,7 @@ void FreezePropertyModifier::initializeModifier(const ModifierInitializationRequ
 ******************************************************************************/
 void FreezePropertyModifier::propertyChanged(const PropertyFieldDescriptor* field)
 {
-    // Whenever the selected property class of this modifier changes, update the property references accordingly.
-    if(field == PROPERTY_FIELD(GenericPropertyModifier::subject) && !isBeingLoaded() && !isAboutToBeDeleted() && !isUndoingOrRedoing()) {
-        setSourceProperty(sourceProperty().convertToContainerClass(subject().dataClass()));
-        setDestinationProperty(destinationProperty().convertToContainerClass(subject().dataClass()));
-    }
-    else if(field == PROPERTY_FIELD(FreezePropertyModifier::sourceProperty) && !isBeingLoaded()) {
+    if(field == PROPERTY_FIELD(FreezePropertyModifier::sourceProperty) && !isBeingLoaded()) {
         // Changes of some the modifier's parameters affect the result of FreezePropertyModifier::getPipelineEditorShortInfo().
         notifyDependents(ReferenceEvent::ObjectStatusChanged);
     }
@@ -98,49 +102,50 @@ void FreezePropertyModifier::propertyChanged(const PropertyFieldDescriptor* fiel
 /******************************************************************************
 * Modifies the input data.
 ******************************************************************************/
-Future<PipelineFlowState> FreezePropertyModifier::evaluate(const ModifierEvaluationRequest& request, const PipelineFlowState& input)
+Future<PipelineFlowState> FreezePropertyModifier::evaluateModifier(const ModifierEvaluationRequest& request, PipelineFlowState&& state)
 {
     // Check if we already have the frozen property available.
-    if(FreezePropertyModificationNode* node = dynamic_object_cast<FreezePropertyModificationNode>(request.modificationNode())) {
-        if(node->hasFrozenState(AnimationTime::fromFrame(freezeTime()))) {
+    if(FreezePropertyModificationNode* modNode = dynamic_object_cast<FreezePropertyModificationNode>(request.modificationNode())) {
+        if(modNode->hasFrozenState(AnimationTime::fromFrame(freezeTime()))) {
             // Perform replacement of the property in the input pipeline state.
-            PipelineFlowState output = input;
-            evaluateSynchronous(request, output);
-            return std::move(output);
+            return transferFrozenProperty(modNode, std::move(state));
         }
     }
 
-    // Set up the upstream pipeline request.
+    // Cannot make upstream requests in interactive mode - this would take too long.
+    if(request.interactiveMode())
+        throw Exception(tr("No stored property values available yet."));
+
+    // Set up the upstream pipeline request for the freeze time.
     PipelineEvaluationRequest upstreamRequest = request;
     upstreamRequest.setTime(AnimationTime::fromFrame(freezeTime()));
 
     // Request the frozen state from the upstream pipeline.
     return request.modificationNode()->evaluateInput(upstreamRequest)
-        .then(*this, [this, request, node = OORef<ModificationNode>(request.modificationNode()), state = input](const PipelineFlowState& frozenState) mutable {
+        .then(ObjectExecutor(this), [this, request, state = std::move(state)](const PipelineFlowState& frozenState) mutable {
 
-            // Extract the input property.
-            if(FreezePropertyModificationNode* myModNode = dynamic_object_cast<FreezePropertyModificationNode>(request.modificationNode())) {
-                if(myModNode->modifier() == this && !sourceProperty().isNull() && subject()) {
+            // Extract the property to freeze.
+            if(FreezePropertyModificationNode* modNode = dynamic_object_cast<FreezePropertyModificationNode>(request.modificationNode())) {
+                if(modNode->modifier() == this && sourceProperty() && subject()) {
 
                     const PropertyContainer* container = frozenState.expectLeafObject(subject());
                     if(const Property* property = sourceProperty().findInContainer(container)) {
 
                         // Cache the property to be frozen in the ModificationNode.
-                        myModNode->updateStoredData(property,
+                        modNode->updateStoredData(property,
                             container->getOOMetaClass().isValidStandardPropertyId(Property::GenericIdentifierProperty)
                                 ? container->getProperty(Property::GenericIdentifierProperty)
                                 : nullptr,
                             frozenState.stateValidity());
 
                         // Perform the actual replacement of the property in the input pipeline state.
-                        evaluateSynchronous(request, state);
-                        return std::move(state);
+                        return transferFrozenProperty(modNode, std::move(state));
                     }
                     else {
                         throw Exception(tr("The property '%1' is not present in the input state.").arg(sourceProperty().name()));
                     }
                 }
-                myModNode->invalidateFrozenState();
+                modNode->invalidateFrozenState();
             }
 
             return std::move(state);
@@ -148,23 +153,22 @@ Future<PipelineFlowState> FreezePropertyModifier::evaluate(const ModifierEvaluat
 }
 
 /******************************************************************************
-* Modifies the input data synchronously.
+* Copies the stored property to the current pipeline state.
 ******************************************************************************/
-void FreezePropertyModifier::evaluateSynchronous(const ModifierEvaluationRequest& request, PipelineFlowState& state)
+PipelineFlowState FreezePropertyModifier::transferFrozenProperty(FreezePropertyModificationNode* modNode, PipelineFlowState state) const
 {
     if(!subject())
         throw Exception(tr("No property type selected."));
 
-    if(sourceProperty().isNull()) {
+    if(!sourceProperty()) {
         state.setStatus(PipelineStatus(PipelineStatus::Warning, tr("No source property selected.")));
-        return;
+        return state;
     }
-    if(destinationProperty().isNull())
+    if(!destinationProperty())
         throw Exception(tr("No output property selected."));
 
     // Retrieve the property values stored in the ModificationNode.
-    FreezePropertyModificationNode* myModApp = dynamic_object_cast<FreezePropertyModificationNode>(request.modificationNode());
-    if(!myModApp || !myModApp->property())
+    if(!modNode || !modNode->property())
         throw Exception(tr("No stored property values available."));
 
     // Look up the property container object.
@@ -173,62 +177,101 @@ void FreezePropertyModifier::evaluateSynchronous(const ModifierEvaluationRequest
 
     // Get the property that will be overwritten by the stored one.
     Property* outputProperty;
-    if(destinationProperty().type() != Property::GenericUserProperty) {
-        outputProperty = container->createProperty(DataBuffer::Initialized, destinationProperty().type());
-        if(outputProperty->dataType() != myModApp->property()->dataType()
-            || outputProperty->componentCount() != myModApp->property()->componentCount()
-            || outputProperty->stride() != myModApp->property()->stride())
-            throw Exception(tr("Types of source property and output property are not compatible. Cannot restore saved property values."));
+    int destTypeId = destinationProperty().standardTypeId(&container->getOOMetaClass());
+    if(destTypeId != 0) {
+        outputProperty = container->createProperty(DataBuffer::Initialized, destTypeId);
     }
     else {
         outputProperty = container->createProperty(DataBuffer::Initialized, destinationProperty().name(),
-            myModApp->property()->dataType(), myModApp->property()->componentCount());
-        outputProperty->setComponentNames(myModApp->property()->componentNames());
+            modNode->property()->dataType(), modNode->property()->componentCount());
+        outputProperty->setComponentNames(modNode->property()->componentNames());
     }
-    OVITO_ASSERT(outputProperty->stride() == myModApp->property()->stride());
+    OVITO_ASSERT(outputProperty->stride() == modNode->property()->stride());
+
+    // Data types of the source property and the output property must be compatible for the copy operation.
+    if(outputProperty->dataType() != modNode->property()->dataType()
+            || outputProperty->componentCount() != modNode->property()->componentCount()
+            || outputProperty->stride() != modNode->property()->stride())
+        throw Exception(tr("Data types of source property and output property are not compatible. Cannot transfer frozen property values."));
+
+    // Create the output selection property if requested.
+    BufferWriteAccessAndRef<SelectionIntType, access_mode::write> selection;
+    if(selectNewElements() && container->getOOMetaClass().isValidStandardPropertyId(Property::GenericSelectionProperty))
+        selection = container->getOOMetaClass().createStandardProperty(DataBuffer::Initialized, container->elementCount(), Property::GenericSelectionProperty);
 
     // Check if particle IDs are present and if the order of particles has changed
     // since we took the snapshot of the property values.
     BufferReadAccess<IdentifierIntType> idProperty = container->getOOMetaClass().isValidStandardPropertyId(Property::GenericIdentifierProperty)
         ? container->getProperty(Property::GenericIdentifierProperty)
         : nullptr;
-    BufferReadAccess<IdentifierIntType> storedIds = myModApp->identifiers();
+    BufferReadAccess<IdentifierIntType> storedIds = modNode->identifiers();
     if(storedIds && idProperty && (idProperty.size() != storedIds.size() || !boost::equal(idProperty, storedIds))) {
 
-        // Build ID-to-index map.
-        std::unordered_map<IdentifierIntType,size_t> idmap;
-        size_t index = 0;
-        for(auto id : storedIds) {
-            if(!idmap.insert(std::make_pair(id, index)).second)
-                throw Exception(tr("Detected duplicate element ID %1 in saved snapshot. Cannot apply saved property values.").arg(id));
-            index++;
+        // Obtain ID-to-index map for frozen state.
+        // The map needs to be built only once and can be cached in the modification node.
+        if(modNode->idmap().empty()) {
+            std::unordered_map<IdentifierIntType, size_t> idmap;
+            size_t index = 0;
+            for(auto id : storedIds) {
+                if(!idmap.insert(std::make_pair(id, index)).second)
+                    throw Exception(tr("Detected non-unique element ID %1 in frozen snapshot. Cannot map property values from frozen state to current state.").arg(id));
+                index++;
+            }
+            // Store the ID-to-index map in the modification node.
+            modNode->idmap().swap(idmap);
         }
         storedIds.reset();
 
-        // Build index-to-index map.
+        // Build index-to-index map for the current state.
+        const std::unordered_map<IdentifierIntType, size_t>& idmap = modNode->idmap();
         std::vector<size_t> mapping(outputProperty->size());
         auto id = idProperty.cbegin();
         for(size_t& mappedIndex : mapping) {
             auto mapEntry = idmap.find(*id++);
-            if(mapEntry == idmap.end())
-                throw Exception(tr("Detected new element ID %1, which didn't exist when the snapshot was created. Cannot restore saved property values.").arg(*id));
-            mappedIndex = mapEntry->second;
+            if(mapEntry == idmap.end()) {
+                if(!tolerateNewElements())
+                    throw Exception(tr("Detected newly added element ID %1 at current timestep, which didn't exist at frozen timestep. Property value transfer is not possible in this case.").arg(*id));
+                else {
+                    mappedIndex = std::numeric_limits<size_t>::max();
+                    if(selection)
+                        selection[&mappedIndex - mapping.data()] = 1;
+                }
+            }
+            else {
+                mappedIndex = mapEntry->second;
+            }
         }
         idProperty.reset();
 
         // Copy and reorder property data.
-        myModApp->property()->mappedCopyTo(*outputProperty, mapping);
+        // Accept invalid indices if the user explicitly allowed new elements to be added.
+        modNode->property()->mappedCopyTo(*outputProperty, mapping, tolerateNewElements());
     }
     else {
         storedIds.reset();
         idProperty.reset();
 
         // Make sure the number of elements didn't change when no IDs are defined.
-        if(myModApp->property()->size() != outputProperty->size())
-            throw Exception(tr("Number of input elements has changed. Cannot restore saved property values. There were %1 elements when the snapshot was created. Now there are %2.").arg(myModApp->property()->size()).arg(outputProperty->size()));
-
-        if(outputProperty->dataType() == myModApp->property()->dataType() && outputProperty->stride() == myModApp->property()->stride())
-            outputProperty->copyFrom(*myModApp->property());
+        if(modNode->property()->size() != outputProperty->size()) {
+            if(!tolerateNewElements()) {
+                throw Exception(tr("Cannot transfer frozen property values because the number of %1 has changed. There were %2 %1 at the frozen timestep; now there are %3.")
+                    .arg(container->getOOMetaClass().elementDescriptionName())
+                    .arg(modNode->property()->size())
+                    .arg(outputProperty->size()));
+            }
+            else {
+                // Transfer the property data only for a sub-range of the elements if the number of elements has changed (in either direction).
+                outputProperty->copyRangeFrom(*modNode->property(), 0, 0, std::min(modNode->property()->size(), outputProperty->size()));
+                // Select newly added elements if requested.
+                if(selection && modNode->property()->size() < outputProperty->size()) {
+                    std::fill(selection.begin() + modNode->property()->size(), selection.end(), 1);
+                }
+            }
+        }
+        else {
+            // Perform a 1-to-1 copy of the property data.
+            outputProperty->copyFrom(*modNode->property());
+        }
     }
 
     // Replace vis elements of output property with cached ones and cache any new elements.
@@ -236,13 +279,19 @@ void FreezePropertyModifier::evaluateSynchronous(const ModifierEvaluationRequest
     // each time the modifier is re-evaluated or when serializing the modifier application.
     OORefVector<DataVis> currentVisElements = outputProperty->visElements();
     // Replace with cached vis elements if they are of the same class type.
-    for(int i = 0; i < currentVisElements.size() && i < myModApp->cachedVisElements().size(); i++) {
-        if(currentVisElements[i]->getOOClass() == myModApp->cachedVisElements()[i]->getOOClass()) {
-            currentVisElements[i] = myModApp->cachedVisElements()[i];
+    for(int i = 0; i < currentVisElements.size() && i < modNode->cachedVisElements().size(); i++) {
+        if(currentVisElements[i]->getOOClass() == modNode->cachedVisElements()[i]->getOOClass()) {
+            currentVisElements[i] = modNode->cachedVisElements()[i];
         }
     }
     outputProperty->setVisElements(currentVisElements);
-    myModApp->setCachedVisElements(std::move(currentVisElements));
+    modNode->setCachedVisElements(std::move(currentVisElements));
+
+    // Add selection property to the output container.
+    if(selection)
+        container->addProperty(static_object_cast<Property>(selection.take()));
+
+    return state;
 }
 
 /******************************************************************************
@@ -256,10 +305,11 @@ void FreezePropertyModificationNode::updateStoredData(const Property* property,
     setProperty(cloneHelper.cloneObject(property, false));
     setIdentifiers(cloneHelper.cloneObject(identifiers, false));
     _validityInterval = validityInterval;
+    _idmap.clear();
 }
 
 /******************************************************************************
-* Is called when a RefTarget referenced by this object has generated an event.
+* Is called when a RefTarget referenced by this object generated an event.
 ******************************************************************************/
 bool FreezePropertyModificationNode::referenceEvent(RefTarget* source, const ReferenceEvent& event)
 {
@@ -295,10 +345,12 @@ void FreezePropertyModifier::loadFromStreamComplete(ObjectLoadStream& stream)
         if(ModificationNode* node = someNode()) {
             QSet<Pipeline*> pipelines = node->pipelines(true);
             if(!pipelines.empty()) {
-                if(Scene* scene = (*pipelines.begin())->scene()) {
-                    if(scene->animationSettings()) {
-                        int ticksPerFrame = (int)std::round(4800.0f / scene->animationSettings()->framesPerSecond());
-                        setFreezeTime(freezeTime() / ticksPerFrame);
+                if(SceneNode* sceneNode = (*pipelines.begin())->someSceneNode()) {
+                    if(Scene* scene = sceneNode->scene()) {
+                        if(scene->animationSettings()) {
+                            int ticksPerFrame = (int)std::round(4800.0f / scene->animationSettings()->framesPerSecond());
+                            setFreezeTime(freezeTime() / ticksPerFrame);
+                        }
                     }
                 }
             }
