@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -24,13 +24,11 @@
 #include <ovito/gui/desktop/mainwin/MainWindow.h>
 #include <ovito/gui/desktop/dialogs/LoadImageFileDialog.h>
 #include <ovito/gui/desktop/properties/FloatParameterUI.h>
-#include <ovito/gui/desktop/properties/Vector3ParameterUI.h>
+#include <ovito/gui/desktop/properties/VectorParameterUI.h>
 #include <ovito/gui/desktop/properties/ColorParameterUI.h>
 #include <ovito/gui/desktop/properties/BooleanParameterUI.h>
-#include <ovito/gui/desktop/properties/CustomParameterUI.h>
 #include <ovito/gui/desktop/properties/ModifierDelegateParameterUI.h>
 #include <ovito/gui/desktop/dialogs/SaveImageFileDialog.h>
-#include <ovito/gui/desktop/utilities/concurrent/ProgressDialog.h>
 #include <ovito/gui/base/viewport/ViewportInputMode.h>
 #include <ovito/core/app/PluginManager.h>
 #include <ovito/core/oo/OvitoClass.h>
@@ -38,7 +36,7 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(PropertyColorMappingEditor);
+IMPLEMENT_CREATABLE_OVITO_CLASS(PropertyColorMappingEditor);
 SET_OVITO_OBJECT_EDITOR(PropertyColorMapping, PropertyColorMappingEditor);
 
 /******************************************************************************
@@ -54,7 +52,7 @@ void PropertyColorMappingEditor::createUI(const RolloutInsertionParameters& roll
     layout1->setContentsMargins(4,4,4,4);
     layout1->setSpacing(2);
 
-    _sourcePropertyUI = new PropertyReferenceParameterUI(this, PROPERTY_FIELD(PropertyColorMapping::sourceProperty));
+    _sourcePropertyUI = createParamUI<PropertyReferenceParameterUI>(PROPERTY_FIELD(PropertyColorMapping::sourceProperty));
     layout1->addWidget(new QLabel(tr("Source property:")));
     layout1->addWidget(_sourcePropertyUI->comboBox());
 
@@ -63,9 +61,8 @@ void PropertyColorMappingEditor::createUI(const RolloutInsertionParameters& roll
     layout1->addWidget(_colorGradientList);
     _colorGradientList->setIconSize(QSize(48,16));
     connect(_colorGradientList, qOverload<int>(&QComboBox::activated), this, &PropertyColorMappingEditor::onColorGradientSelected);
-    QVector<OvitoClassPtr> sortedColormapClassList = PluginManager::instance().listClasses(ColorCodingGradient::OOClass());
-    std::sort(sortedColormapClassList.begin(), sortedColormapClassList.end(),
-        [](OvitoClassPtr a, OvitoClassPtr b) { return QString::localeAwareCompare(a->displayName(), b->displayName()) < 0; });
+    std::vector<OvitoClassPtr> sortedColormapClassList = PluginManager::instance().listClasses(ColorCodingGradient::OOClass());
+    boost::sort(sortedColormapClassList, [](OvitoClassPtr a, OvitoClassPtr b) { return QString::localeAwareCompare(a->displayName(), b->displayName()) < 0; });
     for(OvitoClassPtr clazz : sortedColormapClassList) {
         if(clazz == &ColorCodingImageGradient::OOClass() || clazz == &ColorCodingTableGradient::OOClass())
             continue;
@@ -84,7 +81,7 @@ void PropertyColorMappingEditor::createUI(const RolloutInsertionParameters& roll
     layout1->addLayout(layout2);
 
     // End value parameter.
-    _endValueUI = new FloatParameterUI(this, PROPERTY_FIELD(PropertyColorMapping::endValue));
+    _endValueUI = createParamUI<FloatParameterUI>(PROPERTY_FIELD(PropertyColorMapping::endValue));
     layout2->addWidget(_endValueUI->label(), 0, 0);
     layout2->addLayout(_endValueUI->createFieldLayout(), 0, 1);
 
@@ -114,7 +111,7 @@ void PropertyColorMappingEditor::createUI(const RolloutInsertionParameters& roll
     layout2->addWidget(_colorLegendLabel, 1, 1);
 
     // Start value parameter.
-    _startValueUI = new FloatParameterUI(this, PROPERTY_FIELD(PropertyColorMapping::startValue));
+    _startValueUI = createParamUI<FloatParameterUI>(PROPERTY_FIELD(PropertyColorMapping::startValue));
     layout2->addWidget(_startValueUI->label(), 2, 0);
     layout2->addLayout(_startValueUI->createFieldLayout(), 2, 1);
 
@@ -127,6 +124,11 @@ void PropertyColorMappingEditor::createUI(const RolloutInsertionParameters& roll
     connect(exportBtn, &QPushButton::clicked, this, &PropertyColorMappingEditor::onExportColorScale);
     layout2->addWidget(exportBtn, 1, 0, Qt::AlignCenter);
 
+    // Symmetric range
+    BooleanParameterUI* symmetricRangePUI = createParamUI<BooleanParameterUI>(PROPERTY_FIELD(PropertyColorMapping::symmetricRange));
+    layout2->addWidget(symmetricRangePUI->checkBox(), 3, 1);
+    connect(symmetricRangePUI->checkBox(), &QCheckBox::toggled, _startValueUI, &FloatParameterUI::setDisabled);
+
     layout1->addSpacing(8);
 
     _adjustRangeBtn = new QPushButton(tr("Adjust range"), rollout);
@@ -209,18 +211,19 @@ std::optional<std::pair<FloatType, FloatType>> PropertyColorMappingEditor::deter
 {
     // Get the color mapping object.
     PropertyColorMapping* mapping = static_object_cast<PropertyColorMapping>(editObject());
-    if(!mapping) return {};
+    if(!mapping)
+        return {};
 
     // Get the property container.
     const PropertyContainer* container = _sourcePropertyUI->container();
-    if(!container) return {};
+    if(!container)
+        return {};
 
     // Look up the selected property.
-    const Property* pseudoColorProperty = mapping->sourceProperty().findInContainer(container);
-    if(!pseudoColorProperty) return {};
-
-    if(mapping->sourceProperty().vectorComponent() >= (int)pseudoColorProperty->componentCount()) return {};
-    int pseudoColorPropertyComponent = std::max(0, mapping->sourceProperty().vectorComponent());
+    QString errorDescr;
+    auto [pseudoColorProperty, pseudoColorPropertyComponent] = mapping->sourceProperty().findInContainerWithComponent(container, errorDescr);
+    if(!pseudoColorProperty)
+        return {};
 
     // Determine min/max value range.
     return mapping->determineValueRange(pseudoColorProperty, pseudoColorPropertyComponent);
@@ -281,7 +284,9 @@ void PropertyColorMappingEditor::onAdjustRange()
     performTransaction(tr("Adjust range"), [&]() {
         if(PropertyColorMapping* mapping = static_object_cast<PropertyColorMapping>(editObject())) {
             if(std::optional<std::pair<FloatType, FloatType>> range = determineValueRange()) {
-                mapping->setStartValue(range->first);
+                if(!mapping->symmetricRange()) {
+                    mapping->setStartValue(range->first);
+                }
                 mapping->setEndValue(range->second);
             }
         }
