@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -32,20 +32,21 @@ namespace Ovito {
  * Maps a file column to a standard property unless there is already another
  * column mapped to the same property.
  *****************************************************************************/
-bool InputColumnMapping::mapStandardColumn(int column, int typeId, int vectorComponent)
+bool InputColumnMapping::mapColumnToStandardProperty(int column, int typeId, int vectorComponent)
 {
     OVITO_ASSERT(column >= 0 && column < this->size());
     OVITO_ASSERT(typeId != Property::GenericUserProperty);
     OVITO_ASSERT(containerClass());
+    PropertyReference ref(containerClass(), typeId, vectorComponent);
 
     // Check if there is another file column already mapped to the same target property.
     for(const InputColumnInfo& columnInfo : *this) {
-        if(columnInfo.property.type() == typeId && columnInfo.property.vectorComponent() == vectorComponent)
+        if(columnInfo.property == ref)
             return false;
     }
 
     // If not, record the mapping.
-    (*this)[column].mapStandardColumn(containerClass(), typeId, vectorComponent);
+    (*this)[column].mapToStandardProperty(containerClass(), typeId, vectorComponent);
     return true;
 }
 
@@ -53,22 +54,48 @@ bool InputColumnMapping::mapStandardColumn(int column, int typeId, int vectorCom
  * Maps this column to a user-defined property unless there is already another
  * column mapped to the same property.
  *****************************************************************************/
-bool InputColumnMapping::mapCustomColumn(int column, const QString& propertyName, int dataType, int vectorComponent)
+bool InputColumnMapping::mapColumnToUserProperty(int column, const QString& propertyName, int dataType, int vectorComponent)
 {
     OVITO_ASSERT(column >= 0 && column < this->size());
-    OVITO_ASSERT(containerClass());
+    PropertyReference ref(vectorComponent < 0 ? propertyName : QStringLiteral("%1.%2").arg(propertyName).arg(vectorComponent + 1));
 
     // Check if there is another file column already mapped to the same target property.
     for(const InputColumnInfo& columnInfo : *this) {
-        if(columnInfo.property.type() == Property::GenericUserProperty && columnInfo.property.name() == propertyName && columnInfo.property.vectorComponent() == vectorComponent)
+        if(columnInfo.property == ref)
             return false;
     }
 
     // If not, record the mapping.
-    (*this)[column].mapCustomColumn(containerClass(), propertyName, dataType, vectorComponent);
+    (*this)[column].mapToUserProperty(propertyName, dataType, vectorComponent);
     return true;
 }
 
+/******************************************************************************
+ * Converts the mapping to a different target container class.
+ *****************************************************************************/
+void InputColumnMapping::convertToContainerClass(PropertyContainerClassPtr newClass)
+{
+    OVITO_ASSERT(newClass);
+    if(containerClass() == newClass)
+        return;
+
+    for(InputColumnInfo& col : *this) {
+        if(col.property) {
+            int typeId = col.property.standardTypeId(newClass);
+            if(typeId != 0) {
+                col.dataType = newClass->standardPropertyDataType(typeId);
+            }
+            else {
+                int componentIndex = col.property.componentIndex(containerClass());
+                auto name = col.property.name();
+                col.mapToUserProperty(name.toString(), DataBuffer::FloatDefault, componentIndex);
+            }
+        }
+    }
+
+    _containerClass = newClass;
+}
+
 /******************************************************************************
  * Saves the mapping to the given stream.
  *****************************************************************************/
@@ -109,9 +136,9 @@ LoadStream& operator>>(LoadStream& stream, InputColumnMapping& m)
             stream >> vectorComponent;
             if(col.dataType != QMetaType::Void) {
                 if(propertyType == Property::GenericUserProperty)
-                    col.property = PropertyReference(m.containerClass(), propertyName, vectorComponent);
+                    col.mapToUserProperty(propertyName, col.dataType, vectorComponent);
                 else
-                    col.property = PropertyReference(m.containerClass(), propertyType, vectorComponent);
+                    col.mapToStandardProperty(m.containerClass(), propertyType, vectorComponent);
             }
         }
     }
@@ -160,12 +187,15 @@ void InputColumnMapping::fromByteArray(const QByteArray& array)
  *****************************************************************************/
 void InputColumnMapping::validate(const QString& fileFormatName) const
 {
-    OVITO_ASSERT(containerClass());
+    // Make sure a target property container type has been specified.
+    if(!containerClass())
+        throw Exception(InputColumnReader::tr("File column mapping%1 is invalid. No target property container type has been specified.")
+            .arg(fileFormatName.isEmpty() ? QString() : InputColumnReader::tr(" for parsing \"%1\"").arg(fileFormatName)));
 
     // Let the property container class perform specific checks.
     containerClass()->validateInputColumnMapping(*this);
 
-    // Check for conflicting mappings, i.e. several file columns being mapped to the same particle property.
+    // Check for conflicting mappings, i.e. several file columns being mapped to the same property.
     int numMapped = 0;
     for(auto m1 = begin(); m1 != end(); ++m1) {
         // Skip columns to be ignored.
@@ -173,11 +203,9 @@ void InputColumnMapping::validate(const QString& fileFormatName) const
             continue;
 
         // Validate property names.
-        if(m1->property.type() == Property::GenericUserProperty)
-            Property::throwIfInvalidPropertyName(m1->property.name());
+        Property::throwIfInvalidPropertyName(m1->property.name());
 
         numMapped++;
-        OVITO_ASSERT(m1->property.containerClass() == containerClass());
         for(auto m2 = std::next(m1); m2 != end(); ++m2) {
             if(m1->property == m2->property)
                 throw Exception(InputColumnReader::tr("Invalid file column mapping%4: File columns %1 and %2 cannot both be mapped to the same property '%3'.")
@@ -200,15 +228,17 @@ void InputColumnMapping::validate(const QString& fileFormatName) const
 InputColumnReader::InputColumnReader(StandardFrameLoader& frameLoader, const InputColumnMapping& mapping, PropertyContainer* container, bool removeExistingProperties, bool validateMapping)
     : _frameLoader(frameLoader), _mapping(mapping), _container(container)
 {
-    if(validateMapping)
-        mapping.validate(frameLoader.pipelineNode() ? frameLoader.pipelineNode()->objectTitle() : QString());
+    if(validateMapping) {
+        OORef<const PipelineNode> pipelineNode = frameLoader.pipelineNode().lock();
+        mapping.validate(pipelineNode ? pipelineNode->objectTitle() : QString{});
+    }
 
     // Create target properties as defined by the mapping.
     for(int i = 0; i < (int)mapping.size(); i++) {
 
         const PropertyReference& pref = mapping[i].property;
-
-        int vectorComponent = std::max(0, pref.vectorComponent());
+        auto name = pref.name();
+        int vectorComponent = std::max(0, pref.componentIndex(mapping.containerClass()));
         int dataType = mapping[i].dataType;
 
         TargetPropertyRecord rec;
@@ -218,31 +248,32 @@ InputColumnReader::InputColumnReader(StandardFrameLoader& frameLoader, const Inp
             if(dataType != Property::Int8 && dataType != Property::Int32 && dataType != Property::Int64 && dataType != Property::Float32 && dataType != Property::Float64)
                 throw Exception(tr("Invalid user-defined target property (data type %1) for input file column %2").arg(dataType).arg(i+1));
 
+            int typeId = mapping.containerClass()->standardPropertyTypeId(name);
             Property* property;
-            if(pref.type() != Property::GenericUserProperty) {
+            if(typeId != Property::GenericUserProperty) {
                 // Create standard property.
-                property = container->createProperty(DataBuffer::Initialized, pref.type());
+                property = container->createProperty(DataBuffer::Initialized, typeId);
                 // File reader may want to override the property's name.
-                property->setName(pref.name());
+                property->setName(name.toString());
                 // If this is a typed property, determine the kind of ElementType objects to create for it.
-                rec.elementTypeClass = container->getOOMetaClass().typedPropertyElementClass(pref.type());
+                rec.elementTypeClass = container->getOOMetaClass().typedPropertyElementClass(typeId);
             }
             else {
                 // Determine the number of vector components we need for this user-defined property.
                 int componentCount = vectorComponent + 1;
                 for(const InputColumnInfo& col : mapping)
-                    if(col.property.type() == Property::GenericUserProperty && col.property.name() == pref.name())
-                        componentCount = std::max(componentCount, col.property.vectorComponent() + 1);
+                    if(col.property.name() == name)
+                        componentCount = std::max(componentCount, col.property.componentIndex(mapping.containerClass()) + 1);
 
                 // Look for existing user-defined property with the same name.
                 if(const Property* existingProperty = container->getProperty(pref.name())) {
                     // If the existing property is incompatible, remove it from the container and create a new one.
-                    if(existingProperty->type() != pref.type() || existingProperty->componentCount() != componentCount || existingProperty->dataType() != dataType)
+                    if(existingProperty->typeId() != typeId || existingProperty->componentCount() != componentCount || existingProperty->dataType() != dataType)
                         container->removeProperty(existingProperty);
                 }
 
                 // Create a new user-defined property for the column.
-                property = container->createProperty(DataBuffer::Initialized, pref.name(), dataType, componentCount);
+                property = container->createProperty(DataBuffer::Initialized, name, dataType, componentCount);
             }
 
             OVITO_ASSERT(vectorComponent < (int)property->componentCount());
@@ -384,7 +415,7 @@ void InputColumnReader::assignTypeNamesFromSeparateColumns()
                     ElementType* elementType = record.property->makeMutable(type);
                     elementType->setName(name);
                     // Load the color and radius presets for named particle types:
-                    elementType->initializeType(PropertyReference(&_container->getOOMetaClass(), record.property));
+                    elementType->initializeType(OwnerPropertyRef(&_container->getOOMetaClass(), record.property));
 
                     // Log in type name assigned by the file reader as default value for the element type.
                     // This is needed for the Python code generator to detect manual changes subsequently made by the user.
@@ -468,7 +499,7 @@ void InputColumnReader::readElement(size_t elementIndex, const double* values, i
     auto prec = _properties.cbegin();
     const double* token = values;
 
-    for(int columnIndex = 0; prec != _properties.cend(); ++columnIndex, ++token, ++prec) {
+    for(; prec != _properties.cend(); ++token, ++prec) {
         if(!prec->property) continue;
 
         if(elementIndex >= prec->count)
