@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -27,7 +27,7 @@
 
 namespace Ovito {
 
-IMPLEMENT_OVITO_CLASS(PropertyContainer);
+IMPLEMENT_ABSTRACT_OVITO_CLASS(PropertyContainer);
 DEFINE_VECTOR_REFERENCE_FIELD(PropertyContainer, properties);
 DEFINE_PROPERTY_FIELD(PropertyContainer, elementCount);
 DEFINE_PROPERTY_FIELD(PropertyContainer, title);
@@ -40,12 +40,14 @@ SET_PROPERTY_FIELD_CHANGE_EVENT(PropertyContainer, title, ReferenceEvent::TitleC
 /******************************************************************************
 * Constructor.
 ******************************************************************************/
-PropertyContainer::PropertyContainer(ObjectInitializationFlags flags, const QString& title) : DataObject(flags),
-    _elementCount(0),
-    _title(title)
+void PropertyContainer::initializeObject(ObjectInitializationFlags flags, const QString& title)
 {
-    if(!title.isEmpty())
+    DataObject::initializeObject(flags);
+
+    if(!title.isEmpty()) {
+        setTitle(title);
         freezeInitialParameterValues({SHADOW_PROPERTY_FIELD(PropertyContainer::title)});
+    }
 }
 
 /******************************************************************************
@@ -81,7 +83,7 @@ const Property* PropertyContainer::expectProperty(int typeId) const
 /******************************************************************************
 * Returns the property with the given name and data layout.
 ******************************************************************************/
-const Property* PropertyContainer::expectProperty(const QString& propertyName, int dataType, size_t componentCount) const
+const Property* PropertyContainer::expectProperty(const QStringView propertyName, int dataType, size_t componentCount) const
 {
     const Property* property = getProperty(propertyName);
     if(!property)
@@ -95,6 +97,88 @@ const Property* PropertyContainer::expectProperty(const QString& propertyName, i
     return property;
 }
 
+/******************************************************************************
+* Looks up the named property in the container and resolves the specified
+* vector component (if any). If lookup fails, an error message is returned
+* in the errorDescription parameter.
+******************************************************************************/
+std::pair<const Property*, int> PropertyContainer::findPropertyWithComponent(const QStringView nameWithComponent, QString& errorDescription, bool requireComponent) const
+{
+    // Split property name into base name and vector component.
+    QList<QStringView> parts = nameWithComponent.split(QChar('.'));
+    if(parts.length() > 2) {
+        errorDescription = tr("The property name '%1' contains too many dots.").arg(nameWithComponent);
+        return { nullptr, -1 };
+    }
+    else if(parts.length() == 0 || parts[0].isEmpty()) {
+        errorDescription = tr("Property name is empty.");
+        return { nullptr, -1 };
+    }
+
+    // Look up the property by name.
+    const QStringView& name = parts[0];
+    const Property* property = getProperty(name);
+    if(!property) {
+
+        // Property not found.
+        // As a fallback, look for a property that includes the vector component suffix in its name.
+        // Such technically illegal property names may be produced by the SpatialBinningModifier for example.
+        property = getProperty(nameWithComponent);
+        if(property && property->componentCount() == 1) {
+            return { property, requireComponent ? 0 : -1 };
+        }
+
+        errorDescription = tr("The %1 property with the name '%2' does not exist or has not been computed by the pipeline.").arg(getOOMetaClass().elementDescriptionName()).arg(name);
+        return { nullptr, -1 };
+    }
+
+    int resolvedVectorComponent = -1;
+    if(parts.size() == 2 && !parts[1].isEmpty()) {
+        const QStringView& vectorComponentName = parts[1];
+        if(!property->componentNames().empty()) {
+            resolvedVectorComponent = property->componentNames().indexOf(vectorComponentName);
+            if(resolvedVectorComponent < 0) {
+                errorDescription = tr("The selected vector property component '%1' is invalid. Property '%2' has the following named components: %3")
+                    .arg(vectorComponentName)
+                    .arg(property->name())
+                    .arg(property->componentNames().join(QStringLiteral(", ")));
+                return { nullptr, -1 };
+            }
+        }
+        else {
+            bool ok;
+            resolvedVectorComponent = vectorComponentName.toInt(&ok) - 1;
+            if(ok) {
+                if(resolvedVectorComponent < 0 || resolvedVectorComponent >= property->componentCount()) {
+                    errorDescription = tr("The selected vector property component '%1' is out of range. Property '%2' has %3 component(s).")
+                        .arg(vectorComponentName)
+                        .arg(property->name())
+                        .arg(property->componentCount());
+                    return { nullptr, -1 };
+                }
+            }
+            else {
+                errorDescription = Property::tr("The selected vector property component '%1' cannot be resolved, because property '%2' does not have named components.")
+                    .arg(vectorComponentName)
+                    .arg(property->name());
+                return { nullptr, -1 };
+            }
+        }
+    }
+
+    if(resolvedVectorComponent >= (int)property->componentCount()) {
+        errorDescription = tr("The selected vector property component is out of range. The property '%1' has only %2 values per data element.")
+                                        .arg(property->name())
+                                        .arg(resolvedVectorComponent);
+        return { nullptr, -1 };
+    }
+
+    if(requireComponent && resolvedVectorComponent < 0)
+        resolvedVectorComponent = 0;
+
+    return { property, resolvedVectorComponent };
+}
+
 /******************************************************************************
 * Duplicates and replaces the given property with its copy if it not exclusively
 * owned by this container or is being accessed from Python. If a copy is being
@@ -109,24 +193,14 @@ Property* PropertyContainer::makePropertyMutable(const Property* property, DataB
 
     // Always clone property if its memory is currently being accessed from Python code.
     // That's required, because Python value holders are not strong DataOORefs protecting the property object from changes.
-    if(property && ((property->isBeingAccessedFromPython() && !ignorePythonAccess) || !isSafeToModifySubObject(property))) {
-        DataOORef<Property> clone;
+    if(property && ((property->isBeingAccessedExternally() && !ignorePythonAccess) || !isSafeToModifySubObject(property))) {
+        PropertyPtr clone;
         if(cloneMode == DataBuffer::Initialized) {
             clone = CloneHelper::cloneSingleObject(property, false);
         }
         else {
             // Custom clone implementation, which copies only the metadata but not the contents of the original property.
-            clone = DataOORef<Property>::create(
-                ObjectInitializationFlag::DontInitializeObject, DataBuffer::Uninitialized, property->size(), property->dataType(),
-                property->componentCount(), property->name(), property->type(), property->componentNames());
-            {
-                UndoSuspender noUndo;
-                clone->setVisElements(property->visElements());
-                clone->setElementTypes(property->elementTypes());
-                clone->setTitle(property->title());
-                clone->setCreatedByNode(property->createdByNode());
-                clone->setEditableProxy(property->editableProxy());
-            }
+            clone = property->cloneWithoutData(property->size());
         }
         replaceReferencesTo(property, clone);
         OVITO_ASSERT(hasReferenceTo(clone));
@@ -151,19 +225,9 @@ Property* PropertyContainer::makePropertyMutableUnallocated(const Property* prop
 
     // Always clone property if it is currently being accessed from Python code.
     // That's required, because Python code never holds a strong DataOORef to the property object.
-    if(property->isBeingAccessedFromPython() || !isSafeToModifySubObject(property)) {
+    if(property->isBeingAccessedExternally() || !isSafeToModifySubObject(property)) {
         // Custom clone implementation, which copies only the metadata but not the contents of the original property.
-        DataOORef<Property> clone = DataOORef<Property>::create(
-                ObjectInitializationFlag::DontInitializeObject, DataBuffer::Uninitialized, 0, property->dataType(),
-                property->componentCount(), property->name(), property->type(), property->componentNames());
-        {
-            UndoSuspender noUndo;
-            clone->setVisElements(property->visElements());
-            clone->setElementTypes(property->elementTypes());
-            clone->setTitle(property->title());
-            clone->setCreatedByNode(property->createdByNode());
-            clone->setEditableProxy(property->editableProxy());
-        }
+        PropertyPtr clone = property->cloneWithoutData(0);
         replaceReferencesTo(property, clone);
         OVITO_ASSERT(hasReferenceTo(clone));
         OVITO_ASSERT(!hasReferenceTo(property));
@@ -253,14 +317,12 @@ size_t PropertyContainer::deleteElements(ConstDataBufferPtr selection, size_t se
     OVITO_CHECK_OBJECT_POINTER(this);
     OVITO_ASSERT(isSafeToModify());
 
+    if(elementCount() == 0)
+        return 0;
+
     // Determine number of selected elements in the selection array if it wasn't provided by the caller.
-    if(selectionCount == std::numeric_limits<size_t>::max()) {
-        selectionCount = 0;
-        for(auto s : BufferReadAccess<SelectionIntType>(selection)) {
-            if(s)
-                selectionCount++;
-        }
-    }
+    if(selectionCount == std::numeric_limits<size_t>::max())
+        selectionCount = selection->nonzeroCount();
     if(selectionCount == 0)
         return 0;   // Nothing to delete.
 
@@ -316,7 +378,7 @@ Property* PropertyContainer::createProperty(DataBuffer::BufferInitialization ini
 * Creates a user-defined property and adds it to the container.
 * In case the property already exists, it is made sure that it's safe to modify it.
 ******************************************************************************/
-Property* PropertyContainer::createProperty(DataBuffer::BufferInitialization init, const QString& name, int dataType, size_t componentCount, QStringList componentNames)
+Property* PropertyContainer::createProperty(DataBuffer::BufferInitialization init, const QStringView name, int dataType, size_t componentCount, QStringList componentNames)
 {
     OVITO_ASSERT(isSafeToModify());
 
@@ -348,7 +410,7 @@ const Property* PropertyContainer::createProperty(const Property* property)
 {
     OVITO_CHECK_POINTER(property);
     OVITO_ASSERT(isSafeToModify());
-    OVITO_ASSERT(property->type() == 0 || getOOMetaClass().isValidStandardPropertyId(property->type()));
+    OVITO_ASSERT(!property->isStandardProperty() || getOOMetaClass().isValidStandardPropertyId(property->typeId()));
 
     // Length of first property array determines number of data elements in the container.
     if(properties().empty() && elementCount() == 0)
@@ -364,13 +426,13 @@ const Property* PropertyContainer::createProperty(const Property* property)
 
     // Check if the same property already exists in the container.
     const Property* existingProperty;
-    if(property->type() != 0) {
-        existingProperty = getProperty(property->type());
+    if(property->isStandardProperty()) {
+        existingProperty = getProperty(property->typeId());
     }
     else {
         existingProperty = nullptr;
         for(const Property* p : properties()) {
-            if(p->type() == 0 && p->name() == property->name()) {
+            if(!p->isStandardProperty() && p->name() == property->name()) {
                 existingProperty = p;
                 break;
             }
@@ -414,7 +476,7 @@ void PropertyContainer::setContent(size_t newElementCount, const DataRefVector<P
 }
 
 /******************************************************************************
-* Duplicates all data elements by extensing the property arrays and
+* Duplicates all data elements by extending the property arrays and
 * replicating the existing data N times.
 ******************************************************************************/
 void PropertyContainer::replicate(size_t n)
@@ -432,6 +494,26 @@ void PropertyContainer::replicate(size_t n)
     }
 }
 
+/******************************************************************************
+* Performs a numeric data type conversion of a property (unless the property already has the requested type).
+******************************************************************************/
+const Property* PropertyContainer::convertPropertyToDataType(const ConstPropertyPtr& property, int dataType)
+{
+    OVITO_ASSERT(property);
+    OVITO_ASSERT(properties().contains(property.get()));
+    OVITO_ASSERT(!property->isStandardProperty() || dataType == getOOMetaClass().standardPropertyDataType(property->typeId()));
+
+    if(property->dataType() != dataType) {
+        PropertyPtr convertedProperty = property->cloneWithoutData(property->size(), dataType);
+        convertedProperty->copyFromAndConvert(*property);
+        replaceReferencesTo(property, convertedProperty);
+        return convertedProperty.get();
+    }
+    else {
+        return property.get();
+    }
+}
+
 /******************************************************************************
 * Sorts the data elements with respect to their unique IDs.
 * Does nothing if data elements do not have IDs.
@@ -449,13 +531,14 @@ std::vector<size_t> PropertyContainer::sortById()
 
     // Determine new permutation of data elements which sorts them by ascending ID.
     std::vector<size_t> permutation(ids.size());
-    std::iota(permutation.begin(), permutation.end(), (size_t)0);
-    std::sort(permutation.begin(), permutation.end(), [&](size_t a, size_t b) { return ids[a] < ids[b]; });
+    boost::algorithm::iota(permutation, (size_t)0);
+    boost::sort(permutation, [&](size_t a, size_t b) { return ids[a] < ids[b]; });
     std::vector<size_t> invertedPermutation(ids.size());
     bool isAlreadySorted = true;
     for(size_t i = 0; i < permutation.size(); i++) {
         invertedPermutation[permutation[i]] = i;
-        if(permutation[i] != i) isAlreadySorted = false;
+        if(permutation[i] != i)
+            isAlreadySorted = false;
     }
     ids.reset();
     if(isAlreadySorted)
@@ -472,6 +555,84 @@ std::vector<size_t> PropertyContainer::sortById()
     return invertedPermutation;
 }
 
+/******************************************************************************
+* Copies one or more property arrays from another container to this container if possible.
+* Takes care of remapping the property values to the new container's element order.
+* Non-existing elements in the source container are filled with default values.
+* Copying may not be possible if the property containers have different element counts without unique identifiers.
+******************************************************************************/
+void PropertyContainer::tryToAdoptProperties(const PropertyContainer* sourceContainer, const std::vector<const Property*>& sourceProperties, const ConstDataObjectPath& containerPath)
+{
+    OVITO_ASSERT(this != sourceContainer);
+    OVITO_ASSERT(this->getOOClass() == sourceContainer->getOOClass());
+    OVITO_ASSERT(containerPath.empty() || containerPath.back() == this);
+
+    // Are there any properties to adopt? If not, bail out early.
+    if(std::all_of(sourceProperties.begin(), sourceProperties.end(), [](const Property* p) { return p == nullptr; }))
+        return;
+
+    if(this->getOOMetaClass().isValidStandardPropertyId(Property::GenericIdentifierProperty)) {
+        const Property* sourceIdentifiers = sourceContainer->getProperty(Property::GenericIdentifierProperty);
+        const Property* destIdentifiers = this->getProperty(Property::GenericIdentifierProperty);
+        if(sourceIdentifiers && destIdentifiers) {
+
+            // First, check if the mapping is non-trivial, i.e., the IDs are not the same in both containers.
+            if(sourceIdentifiers != destIdentifiers || sourceIdentifiers->size() != destIdentifiers->size() || sourceIdentifiers->checksum() != destIdentifiers->checksum()) {
+
+                // Build id-to-index map for source container.
+                std::unordered_map<IdentifierIntType, size_t> idMap;
+                idMap.reserve(sourceIdentifiers->size());
+                size_t index = 0;
+                for(auto id : BufferReadAccess<IdentifierIntType>{sourceIdentifiers})
+                    idMap.insert(std::make_pair(id, index++));
+
+                for(const Property* sourceProperty : sourceProperties) {
+                    if(!sourceProperty)
+                        continue;
+                    OVITO_ASSERT(sourceIdentifiers->size() == sourceProperty->size());
+
+                    // Create destination property array. Initialize it with default values.
+                    Property* destProperty;
+                    if(sourceProperty->isStandardProperty())
+                        destProperty = this->createProperty(DataBuffer::Initialized, sourceProperty->typeId(), containerPath);
+                    else
+                        destProperty = this->createProperty(DataBuffer::Initialized, sourceProperty->name(), sourceProperty->dataType(), sourceProperty->componentCount(), sourceProperty->componentNames());
+                    destProperty->setVisElements(sourceProperty->visElements());
+
+                    // Perform mapped copy of property values to destination container.
+                    RawBufferAccess<access_mode::write> destAcc(destProperty);
+                    RawBufferReadAccess sourceAcc(sourceProperty);
+                    const auto stride = destProperty->stride();
+                    auto* __restrict dst = destAcc.data();
+                    for(auto id : BufferReadAccess<IdentifierIntType>{destIdentifiers}) {
+                        if(auto it = idMap.find(id); it != idMap.end()) {
+                            OVITO_ASSERT(it->second < sourceProperty->size());
+                            const auto* __restrict src = sourceAcc.cdata() + it->second * stride;
+                            std::memcpy(dst, src, stride);
+                        }
+                        dst += stride;
+                    }
+                }
+
+                return;
+            }
+        }
+        else if(sourceIdentifiers || destIdentifiers) {
+            return;
+        }
+    }
+
+    // In case of no unique identifiers, we can only copy properties if the element counts match exactly.
+    if(this->elementCount() == sourceContainer->elementCount()) {
+        for(const Property* sourceProperty : sourceProperties) {
+            if(sourceProperty) {
+                // Copying is trivial in this case: simply adopt the existing Property object into the new container.
+                this->createProperty(sourceProperty);
+            }
+        }
+    };
+}
+
 /******************************************************************************
 * Makes sure that all property arrays in this container have a consistent length.
 * If this is not the case, the method throws an exception.
@@ -480,7 +641,6 @@ void PropertyContainer::verifyIntegrity() const
 {
     size_t c = elementCount();
     for(const Property* property : properties()) {
-//      OVITO_ASSERT_MSG(property->size() == c, "PropertyContainer::verifyIntegrity()", qPrintable(QString("Property array '%1' has wrong length. It does not match the number of elements in the parent %2 container.").arg(property->name()).arg(getOOMetaClass().propertyClassDisplayName())));
         if(property->size() != c) {
             throw Exception(tr("Property array '%1' has wrong length. It does not match the number of elements in the parent %2 container.").arg(property->name()).arg(getOOMetaClass().propertyClassDisplayName()));
         }
@@ -543,25 +703,24 @@ void PropertyContainer::loadFromStreamComplete(ObjectLoadStream& stream)
     if(stream.formatVersion() < 30007) {
         for(const Property* property : properties()) {
             for(const ElementType* type : property->elementTypes()) {
-                if(type->ownerProperty().isNull()) {
-                    const_cast<ElementType*>(type)->_ownerProperty.set(const_cast<ElementType*>(type), PROPERTY_FIELD(ElementType::ownerProperty), PropertyReference(&OOClass(), property));
+                if(!type->ownerProperty()) {
+                    const_cast<ElementType*>(type)->_ownerProperty.set(const_cast<ElementType*>(type), PROPERTY_FIELD(ElementType::ownerProperty), OwnerPropertyRef(&OOClass(), property));
                 }
                 if(ElementType* proxyType = dynamic_object_cast<ElementType>(type->editableProxy())) {
-                    if(proxyType->ownerProperty().isNull())
+                    if(!proxyType->ownerProperty())
                         proxyType->_ownerProperty.set(proxyType, PROPERTY_FIELD(ElementType::ownerProperty), type->ownerProperty());
                 }
             }
         }
     }
 
-    // For backward compatibility with older OVITO versions that only knew FloatType.
-    // Perform data type conversion if necessary.
+    // For backward compatibility with older OVITO versions that only knew FloatType, i.e., not both single and double precision floating-point.
+    // Perform data type conversion for standard properties if necessary.
     if(stream.formatVersion() < 30010) {
         for(const Property* property : properties()) {
-            if(property->type() != 0) {
-                int expectedDataType = getOOMetaClass().standardPropertyDataType(property->type());
-                if(property->dataType() != expectedDataType)
-                    makeMutable(property)->convertToDataType(expectedDataType);
+            if(property->isStandardProperty()) {
+                int expectedDataType = getOOMetaClass().standardPropertyDataType(property->typeId());
+                convertPropertyToDataType(property, expectedDataType);
             }
         }
     }
@@ -575,49 +734,28 @@ QString PropertyContainer::elementInfoString(size_t elementIndex, const ConstDat
     QString str;
     for(const Property* property : properties()) {
         if(property->size() <= elementIndex) continue;
-        if(property->type() == Property::GenericSelectionProperty) continue;
-        if(property->type() == Property::GenericColorProperty) continue;
+        if(property->typeId() == Property::GenericSelectionProperty) continue;
+        if(property->typeId() == Property::GenericColorProperty) continue;
         if(!str.isEmpty()) str += QStringLiteral("<sep>");
         str += QStringLiteral("<key>");
         str += property->name().toHtmlEscaped();
         str += QStringLiteral(":</key> <val>");
-        if(property->dataType() == Property::Int32) {
-            BufferReadAccess<int*> data(property);
-            for(size_t component = 0; component < data.componentCount(); component++) {
+        property->forAnyType([&](auto _) {
+            using T = decltype(_);
+            BufferReadAccess<T*> data(property);
+            for(size_t component = 0; component < property->componentCount(); component++) {
                 if(component != 0) str += QStringLiteral(", ");
                 str += QString::number(data.get(elementIndex, component));
-                if(property->elementTypes().empty() == false) {
-                    if(const ElementType* ptype = property->elementType(data.get(elementIndex, component))) {
-                        if(!ptype->name().isEmpty())
-                            str += QString(" (%1)").arg(ptype->name().toHtmlEscaped());
+                if constexpr(std::is_same_v<T, int32_t>) {
+                    if(property->elementTypes().empty() == false) {
+                        if(const ElementType* ptype = property->elementType(data.get(elementIndex, component))) {
+                            if(!ptype->name().isEmpty())
+                                str += QStringLiteral(" (%1)").arg(ptype->name().toHtmlEscaped());
+                        }
                     }
                 }
             }
-        }
-        else if(property->dataType() == Property::Int64) {
-            BufferReadAccess<int64_t*> data(property);
-            for(size_t component = 0; component < property->componentCount(); component++) {
-                if(component != 0) str += QStringLiteral(", ");
-                str += QString::number(data.get(elementIndex, component));
-            }
-        }
-        else if(property->dataType() == Property::Float32) {
-            BufferReadAccess<float*> data(property);
-            for(size_t component = 0; component < property->componentCount(); component++) {
-                if(component != 0) str += QStringLiteral(", ");
-                str += QString::number(data.get(elementIndex, component));
-            }
-        }
-        else if(property->dataType() == Property::Float64) {
-            BufferReadAccess<double*> data(property);
-            for(size_t component = 0; component < property->componentCount(); component++) {
-                if(component != 0) str += QStringLiteral(", ");
-                str += QString::number(data.get(elementIndex, component));
-            }
-        }
-        else {
-            str += QStringLiteral("<%1>").arg(property->dataTypeName());
-        }
+        });
         str += QStringLiteral("</val>");
     }
     return str;
