@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -29,7 +29,7 @@
 #include <ovito/core/dataset/DataSet.h>
 #include "Property.h"
 #include "PropertyContainerClass.h"
-#include "PropertyReference.h"
+#include <ovito/stdobj/vectors/VectorVis.h>
 
 namespace Ovito {
 
@@ -43,7 +43,7 @@ class OVITO_STDOBJ_EXPORT PropertyContainer : public DataObject
 public:
 
     /// Constructor.
-    explicit PropertyContainer(ObjectInitializationFlags flags, const QString& title = {});
+    void initializeObject(ObjectInitializationFlags flags, const QString& title = {});
 
     /// Returns the display title of this object.
     virtual QString objectTitle() const override;
@@ -84,14 +84,14 @@ public:
         OVITO_ASSERT(typeId != 0);
         OVITO_ASSERT(getOOMetaClass().isValidStandardPropertyId(typeId));
         for(const Property* property : properties()) {
-            if(property->type() == typeId)
+            if(property->typeId() == typeId)
                 return property;
         }
         return nullptr;
     }
 
     /// Looks up the user-defined property with the given name.
-    const Property* getProperty(const QString& name) const {
+    const Property* getProperty(const QStringView name) const {
         OVITO_ASSERT(!name.isEmpty());
         for(const Property* property : properties()) {
             // Note: Prior to OVITO 3.7, we required the type id of candidate properties to be 0 here,
@@ -103,6 +103,18 @@ public:
         return nullptr;
     }
 
+    /// Looks up a property with the same type identifier, name, data type and component count as the given property.
+    const Property* getPropertyLike(const Property* property) const {
+        OVITO_ASSERT(property);
+        if(property->isStandardProperty())
+            return getProperty(property->typeId());
+        for(const Property* p : properties()) {
+            if(!p->isStandardProperty() && p->name() == property->name() && p->dataType() == property->dataType() && p->componentCount() == property->componentCount())
+                return p;
+        }
+        return nullptr;
+    }
+
     /// Looks up the standard property with the given ID, removes it from this container and returns it to the caller.
     ConstPropertyPtr takeProperty(int typeId) {
         OVITO_ASSERT(typeId != 0);
@@ -110,7 +122,7 @@ public:
         OVITO_ASSERT(isSafeToModify());
         for(int index = 0; index < properties().size(); index++) {
             const Property* property = properties()[index];
-            if(property->type() == typeId) {
+            if(property->typeId() == typeId) {
                 return _properties.remove(this, PROPERTY_FIELD(properties), index);
             }
         }
@@ -126,7 +138,7 @@ public:
     }
 
     /// Looks up a user-defined property with the given name and makes it mutable if necessary.
-    Property* getMutableProperty(const QString& name, DataBuffer::BufferInitialization cloneMode = DataBuffer::Initialized) {
+    Property* getMutableProperty(const QStringView name, DataBuffer::BufferInitialization cloneMode = DataBuffer::Initialized) {
         if(const Property* p = getProperty(name))
             return makePropertyMutable(p, cloneMode);
         else
@@ -139,7 +151,12 @@ public:
 
     /// Returns the property with the given name and data layout.
     /// If the container does not contain a property with the given name and data type, then an exception is thrown.
-    const Property* expectProperty(const QString& propertyName, int dataType, size_t componentCount = 1) const;
+    const Property* expectProperty(const QStringView propertyName, int dataType, size_t componentCount = 1) const;
+
+    /// Looks up the named property in the container and resolves the specified
+    /// vector component (if any). If lookup fails, an error message is returned
+    /// in the errorDescription parameter.
+    std::pair<const Property*, int> findPropertyWithComponent(const QStringView nameWithComponent, QString& errorDescription, bool requireComponent = true) const;
 
     /// Returns the given standard property after making sure it can be safely modified.
     /// If it does not exist, an exception is thrown.
@@ -158,7 +175,11 @@ public:
     /// After this method returns, all property objects are exclusively owned by the container and
     /// can be safely modified without unwanted side effects.
     auto makePropertiesMutable() {
+
+        // Replace any shared properties with mutable copies.
         makePropertiesMutableInternal();
+
+        // Return an interator range allowing non-const access to the properties, e.g. for modifying them in a ranged-based for-loop.
         auto const_cast_op = [](const DataOORef<const Property>& p) noexcept { return const_cast<Property*>(p.get()); };
         using const_cast_iter_type = boost::transform_iterator<decltype(const_cast_op), typename std::decay_t<decltype(std::declval<PropertyContainer>().properties())>::const_iterator>;
         return boost::make_iterator_range(
@@ -167,6 +188,9 @@ public:
         );
     }
 
+    /// Performs a numeric data type conversion of a property (unless the property already has the requested type).
+    const Property* convertPropertyToDataType(const ConstPropertyPtr& property, int dataType);
+
     /// Creates a standard property and adds it to the container.
     /// In case the property already exists, it is made sure that it's safe to modify it.
     Property* createProperty(DataBuffer::BufferInitialization init, int typeId, const ConstDataObjectPath& containerPath = {});
@@ -179,11 +203,11 @@ public:
 
     /// Creates a user-defined property and adds it to the container.
     /// In case the property already exists, it is made sure that it's safe to modify it.
-    Property* createProperty(DataBuffer::BufferInitialization init, const QString& name, int dataType, size_t componentCount = 1, QStringList componentNames = {});
+    Property* createProperty(DataBuffer::BufferInitialization init, const QStringView name, int dataType, size_t componentCount = 1, QStringList componentNames = {});
 
     /// Creates a user-defined property and adds it to the container.
     /// In case the property already exists, it is made sure that it's safe to modify it.
-    Property* createProperty(const QString& name, int dataType, size_t componentCount = 1, QStringList componentNames = {}) {
+    Property* createProperty(const QStringView name, int dataType, size_t componentCount = 1, QStringList componentNames = {}) {
         return createProperty(DataBuffer::BufferInitialization::Uninitialized, name, dataType, componentCount, std::move(componentNames));
     }
 
@@ -213,12 +237,25 @@ public:
     /// Does nothing if data elements do not have the ID property.
     virtual std::vector<size_t> sortById();
 
+    /// Copies one or more property arrays from another container to this container if possible.
+    /// Takes care of remapping the property values to the new container's element order.
+    /// Non-existing elements in the source container are filled with default values.
+    /// Copying may not be possible if the property containers have different element counts without unique identifiers.
+    void tryToAdoptProperties(const PropertyContainer* sourceContainer, const std::vector<const Property*>& sourceProperties, const ConstDataObjectPath& containerPath = {});
+
     /// Makes sure that all property arrays in this container have a consistent length.
     /// If this is not the case, the method throws an exception.
     void verifyIntegrity() const;
 
-    /// Returns the base point and vector information for visualizing a vector property from this container using a VectorVis element.
-    virtual std::tuple<ConstDataBufferPtr, ConstDataBufferPtr> getVectorVisData(const ConstDataObjectPath& path, const PipelineFlowState& state, MixedKeyCache& visCache) const { return {}; }
+    /// Returns the data for visualizing a vector property from this container using a VectorVis element.
+    virtual VectorVis::VectorData getVectorVisData(const ConstDataObjectPath& path, const PipelineFlowState& state,
+                                                   const RendererResourceCache::ResourceFrame& visCache) const
+    {
+        return {};
+    }
+
+    /// Tells a VectorVis element whether per-vector color and transparency properties are available.
+    virtual std::array<bool, 2> hasVectorVisColorsAndTransparencies() const { return {false, false}; }
 
     /// Generates the info string to be displayed in the OVITO status bar for an element from this container.
     virtual QString elementInfoString(size_t elementIndex, const ConstDataObjectRefPath& path = {}) const;
@@ -262,8 +299,8 @@ public:
             // Grow property arrays.
             for(const Property* prop : _container->properties()) {
                 OVITO_ASSERT(prop->size() == _elementCount);
-                bool b = const_cast<Property*>(prop)->grow(numAdditionalElements, prop->type() == alreadyLockedPropertyType);
-                if(b && prop->type() == alreadyLockedPropertyType)
+                bool b = const_cast<Property*>(prop)->grow(numAdditionalElements, prop->typeId() == alreadyLockedPropertyType);
+                if(b && prop->typeId() == alreadyLockedPropertyType)
                     wasReallocated = true;
             }
             // Update only our internal element count. Container will be updated by destructor.
@@ -278,7 +315,7 @@ public:
             // Truncate each property array.
             for(const Property* prop : _container->properties()) {
                 OVITO_ASSERT(prop->size() == _elementCount);
-                const_cast<Property*>(prop)->truncate(numElementsToTruncate, prop->type() == alreadyLockedPropertyType);
+                const_cast<Property*>(prop)->truncate(numElementsToTruncate, prop->typeId() == alreadyLockedPropertyType);
             }
 
             // Update only our internal element count. Container will be updated by destructor.
@@ -291,15 +328,15 @@ public:
             OVITO_ASSERT(toIndex < _elementCount);
             for(const Property* prop : _container->properties()) {
                 OVITO_ASSERT(prop->size() == _elementCount);
-                const_cast<Property*>(prop)->moveElement(fromIndex, toIndex, prop->type() == alreadyLockedPropertyType);
+                const_cast<Property*>(prop)->moveElement(fromIndex, toIndex, prop->typeId() == alreadyLockedPropertyType);
             }
         }
 
-        Property* mutableProperty(int type) const {
+        Property* mutableProperty(int typeId) const {
             OVITO_ASSERT(_container->isSafeToModify());
             for(const Property* prop : _container->properties()) {
                 OVITO_ASSERT(_container->isSafeToModifySubObject(prop));
-                if(prop->type() == type) {
+                if(prop->typeId() == typeId) {
                     return const_cast<Property*>(prop);
                 }
             }
@@ -338,10 +375,10 @@ private:
     DECLARE_MODIFIABLE_VECTOR_REFERENCE_FIELD(DataOORef<const Property>, properties, setProperties);
 
     /// Keeps track of the number of data elements this property container contains.
-    DECLARE_PROPERTY_FIELD(size_t, elementCount);
+    DECLARE_PROPERTY_FIELD(size_t{0}, elementCount);
 
     /// The assigned title of the data object, which is displayed in the user interface.
-    DECLARE_MODIFIABLE_PROPERTY_FIELD(QString, title, setTitle);
+    DECLARE_MODIFIABLE_PROPERTY_FIELD(QString{}, title, setTitle);
     DECLARE_SHADOW_PROPERTY_FIELD(title);
 };
 
