@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////////////
 //
-//  Copyright 2023 OVITO GmbH, Germany
+//  Copyright 2025 OVITO GmbH, Germany
 //
 //  This file is part of OVITO (Open Visualization Tool).
 //
@@ -31,6 +31,7 @@
 namespace Ovito {
 
 /// List of characters allowed in variable names.
+/// Note: Keep this list in sync with the regex used in PropertyExpressionRewriter::tokenizeExpression().
 mu::string_type PropertyExpressionEvaluator::_validVariableNameChars(_T("0123456789_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ.@"));
 
 /******************************************************************************
@@ -38,13 +39,48 @@ mu::string_type PropertyExpressionEvaluator::_validVariableNameChars(_T("0123456
 * list of input variables.
 ******************************************************************************/
 void PropertyExpressionEvaluator::initialize(const QStringList& expressions, const PipelineFlowState& state, const ConstDataObjectPath& containerPath, int animationFrame)
+{
+    // Create the input variables.
+    initializeInputs(state, containerPath, animationFrame);
+
+    // Expression parser and rewriter
+    std::optional<PropertyExpressionRewriter::Parser> parser;
+    std::optional<PropertyExpressionRewriter::ASTWriter> rewriter;
+
+    // Process expressions being passed in.
+    _expressions.clear();
+    _expressions.reserve(expressions.size());
+    for(const auto& expr : expressions) {
+        // Check if expression contains a reference to a named type.
+        if(PropertyExpressionRewriter::expressionNeedsRewrite(expr)) {
+            // Rewrite expressions if required.
+            if(!parser) {
+                parser.emplace(_typeMapping);
+                rewriter.emplace(_typeMapping);
+            }
+            const QStringList& tokens = PropertyExpressionRewriter::tokenizeExpression(expr);
+            std::unique_ptr<PropertyExpressionRewriter::ASTNode> ast = parser->parse(&expr, &tokens);
+            // qDebug() << rewriter->write(ast.get());
+            _expressions.push_back(convertQString(rewriter->write(ast.get())));
+        }
+        else {
+            // Use original expression as is.
+            _expressions.push_back(convertQString(expr));
+        }
+    }
+}
+
+/******************************************************************************
+ * Creates the input variables.
+ ******************************************************************************/
+void PropertyExpressionEvaluator::initializeInputs(const PipelineFlowState& state, const ConstDataObjectPath& containerPath,
+                                                   int animationFrame)
 {
     const PropertyContainer* container = static_object_cast<PropertyContainer>(containerPath.back());
 
     // Build list of properties that will be made available as expression variables.
     std::vector<ConstPropertyPtr> inputProperties;
-    for(const Property* property : container->properties())
-        inputProperties.push_back(property);
+    for(const Property* property : container->properties()) inputProperties.push_back(property);
     _elementDescriptionName = container->getOOMetaClass().elementDescriptionName();
 
     // Get simulation cell information.
@@ -57,10 +93,6 @@ void PropertyExpressionEvaluator::initialize(const QStringList& expressions, con
 
     // Create list of input variables.
     createInputVariables(inputProperties, simCell, state.buildAttributesMap(), animationFrame);
-
-    // Copy expression strings into internal array.
-    _expressions.resize(expressions.size());
-    std::transform(expressions.begin(), expressions.end(), _expressions.begin(), &convertQString);
 }
 
 /******************************************************************************
@@ -97,12 +129,12 @@ void PropertyExpressionEvaluator::createInputVariables(const std::vector<ConstPr
         _simCell = simCell;
 
         // Cell volume
-        registerGlobalParameter("CellVolume", simCell->volume3D(), tr("simulation cell volume"));
+        registerGlobalParameter("CellVolume", _simCell.is2D() ? _simCell.volume2D() : _simCell.volume3D(), tr("simulation cell volume"));
 
         // Cell size
-        registerGlobalParameter("CellSize.X", std::abs(simCell->matrix().column(0).x()), tr("size along X"));
-        registerGlobalParameter("CellSize.Y", std::abs(simCell->matrix().column(1).y()), tr("size along Y"));
-        registerGlobalParameter("CellSize.Z", std::abs(simCell->matrix().column(2).z()), tr("size along Z"));
+        registerGlobalParameter("CellSize.X", std::abs(_simCell.cellMatrix().column(0).x()), tr("size along X"));
+        registerGlobalParameter("CellSize.Y", std::abs(_simCell.cellMatrix().column(1).y()), tr("size along Y"));
+        registerGlobalParameter("CellSize.Z", std::abs(_simCell.cellMatrix().column(2).z()), tr("size along Z"));
     }
 
     // Constant pi
@@ -144,7 +176,7 @@ void PropertyExpressionEvaluator::registerPropertyVariables(const std::vector<Co
         QString propertyName = property->name();
         // If the name is empty, generate one.
         if(propertyName.isEmpty())
-            propertyName = QString("Property%1").arg(propertyIndex);
+            propertyName = QStringLiteral("Property%1").arg(propertyIndex);
         // If the name starts with a number, prepend an underscore.
         else if(propertyName[0].isDigit())
             propertyName.prepend(QChar('_'));
@@ -153,7 +185,7 @@ void PropertyExpressionEvaluator::registerPropertyVariables(const std::vector<Co
 
             QString fullPropertyName = propertyName;
             if(property->componentNames().size() == property->componentCount())
-                fullPropertyName += "." + property->componentNames()[k];
+                fullPropertyName += QStringLiteral(".") + property->componentNames()[k];
             if(!namePrefix)
                 v.name = convertQString(fullPropertyName);
             else
@@ -165,6 +197,11 @@ void PropertyExpressionEvaluator::registerPropertyVariables(const std::vector<Co
 
             // Register variable.
             addVariable(v);
+
+            // Register typed property to map
+            if(property->isTypedProperty()) {
+                addTypedPropertyToMap(convertMuString(_variables.back().mangledName), property);
+            }
         }
 
         propertyIndex++;
@@ -199,6 +236,30 @@ size_t PropertyExpressionEvaluator::addVariable(ExpressionVariable v)
     return _variables.size() - 1;
 }
 
+/******************************************************************************
+ * Registers a typed property with its mangled name and adds it to _typeMapping
+ ******************************************************************************/
+void PropertyExpressionEvaluator::addTypedPropertyToMap(const QString& mangledPropertyName, const ConstPropertyPtr& property)
+{
+    // Access or insert mangledName in _typeMapping (outer map)
+    auto& omap = _typeMapping[mangledPropertyName];
+
+    // Register names and numeric ids of all types defined for the property.
+    // Type names get placed in double quotes to make them compatible with the format used in input expressions.
+    // Numeric ids are converted to strings to make them compatible with the format used in output expressions.
+    for(const ElementType* type : property->elementTypes()) {
+        QString typeIdStr = QString::number(type->numericId());
+        QString typeName = QStringLiteral("\"%1\"").arg(type->nameOrNumericId());
+
+        // Access or insert typeName in omap.
+        auto& imap = omap[typeName];
+
+        // Append to QStringList
+        if(!imap.contains(typeIdStr))
+            imap << typeIdStr;
+    }
+}
+
 /******************************************************************************
 * Returns the list of available input variables.
 ******************************************************************************/
@@ -209,6 +270,11 @@ QStringList PropertyExpressionEvaluator::inputVariableNames() const
         if(v.isRegistered)
             vlist << convertMuString(v.mangledName);
     }
+    for(const auto& [_, ovalue] : _typeMapping) {
+        for(const auto& [ikey, _] : ovalue) {
+            if(!vlist.contains(ikey)) vlist << ikey;
+        }
+    }
     return vlist;
 }
 
@@ -229,52 +295,6 @@ bool PropertyExpressionEvaluator::isVariableUsed(const mu::char_type* varName)
     return false;
 }
 
-/******************************************************************************
-* Initializes the parser object and evaluates the expressions for every data element
-******************************************************************************/
-void PropertyExpressionEvaluator::evaluate(const std::function<void(size_t,size_t,double)>& callback, const std::function<bool(size_t)>& filter)
-{
-    // Make sure initialize() has been called.
-    OVITO_ASSERT(!_variables.empty());
-
-    // Determine the number of parallel threads to use.
-    size_t nthreads = Application::instance()->idealThreadCount();
-    if(maxThreadCount() != 0 && nthreads > maxThreadCount()) nthreads = maxThreadCount();
-    if(elementCount() == 0)
-        return;
-    else if(elementCount() < 100) // Not worth spawning multiple threads for so few elements.
-        nthreads = 1;
-    else if(nthreads > elementCount())
-        nthreads = elementCount();
-
-    if(nthreads == 1) {
-        Worker worker(*this);
-        _variables = worker._variables;
-        _referencedVariablesKnown = true;
-        worker.run(0, elementCount(), callback, filter);
-        if(!worker._errorMsg.isEmpty())
-            throw Exception(worker._errorMsg);
-    }
-    else if(nthreads > 1) {
-        Worker worker0(*this);
-        _variables = worker0._variables;
-        _referencedVariablesKnown = true;
-        parallelForChunks(elementCount(), [this, &callback, &filter, &worker0](size_t startIndex, size_t chunkSize) {
-            if(startIndex == 0) {
-                worker0.run(startIndex, startIndex + chunkSize, callback, filter);
-                if(!worker0._errorMsg.isEmpty())
-                    throw Exception(worker0._errorMsg);
-            }
-            else {
-                Worker worker(*this);
-                worker.run(startIndex, startIndex + chunkSize, callback, filter);
-                if(!worker._errorMsg.isEmpty())
-                    throw Exception(worker._errorMsg);
-            }
-        });
-    }
-}
-
 /******************************************************************************
 * Initializes the parser objects of this thread.
 ******************************************************************************/
@@ -423,7 +443,7 @@ void PropertyExpressionEvaluator::ExpressionVariable::updateValue(size_t element
 ******************************************************************************/
 QString PropertyExpressionEvaluator::inputVariableTable() const
 {
-    QString str(tr("<p>Available input variables:</p><p><b>Properties:</b><ul>"));
+    QString str(tr("<p><b>Properties:</b><ul>"));
     for(const ExpressionVariable& v : _variables) {
         if((v.type == FLOAT32_PROPERTY || v.type == FLOAT64_PROPERTY || v.type == INT8_PROPERTY || v.type == INT32_PROPERTY || v.type == INT64_PROPERTY || v.type == ELEMENT_INDEX || v.type == DERIVED_PROPERTY) && v.isRegistered && v.variableClass == 0) {
             if(v.description.isEmpty())
@@ -454,4 +474,4 @@ QString PropertyExpressionEvaluator::inputVariableTable() const
     return str;
 }
 
-}   // End of namespace
+}  // namespace Ovito
